/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Register Information Header Fragment                                       *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#include "llvm/CodeGen/TargetRegisterInfo.h"

namespace llvm {

class AArch64FrameLowering;

struct AArch64GenRegisterInfo : public TargetRegisterInfo {
  explicit AArch64GenRegisterInfo(unsigned RA, unsigned D = 0, unsigned E = 0,
      unsigned PC = 0, unsigned HwMode = 0);
  unsigned composeSubRegIndicesImpl(unsigned, unsigned) const override;
  unsigned reverseComposeSubRegIndicesImpl(unsigned, unsigned) const override;
  LaneBitmask composeSubRegIndexLaneMaskImpl(unsigned, LaneBitmask) const override;
  LaneBitmask reverseComposeSubRegIndexLaneMaskImpl(unsigned, LaneBitmask) const override;
  const TargetRegisterClass *getSubClassWithSubReg(const TargetRegisterClass *, unsigned) const override;
  const TargetRegisterClass *getSubRegisterClass(const TargetRegisterClass *, unsigned) const override;
  const RegClassWeight &getRegClassWeight(const TargetRegisterClass *RC) const override;
  unsigned getRegUnitWeight(MCRegUnit RegUnit) const override;
  unsigned getNumRegPressureSets() const override;
  const char *getRegPressureSetName(unsigned Idx) const override;
  unsigned getRegPressureSetLimit(const MachineFunction &MF, unsigned Idx) const override;
  const int *getRegClassPressureSets(const TargetRegisterClass *RC) const override;
  const int *getRegUnitPressureSets(MCRegUnit RegUnit) const override;
  ArrayRef<const char *> getRegMaskNames() const override;
  ArrayRef<const uint32_t *> getRegMasks() const override;
  bool isGeneralPurposeRegister(const MachineFunction &, MCRegister) const override;
  bool isGeneralPurposeRegisterClass(const TargetRegisterClass *RC) const override;
  bool isFixedRegister(const MachineFunction &, MCRegister) const override;
  bool isArgumentRegister(const MachineFunction &, MCRegister) const override;
  bool isConstantPhysReg(MCRegister PhysReg) const override final;
  /// Devirtualized TargetFrameLowering.
  static const AArch64FrameLowering *getFrameLowering(
      const MachineFunction &MF);
};

namespace AArch64 { // Register classes
  extern const TargetRegisterClass W_HI_DummyRCRegClass;
  extern const TargetRegisterClass B_HI_DummyRCRegClass;
  extern const TargetRegisterClass D_HI_DummyRCRegClass;
  extern const TargetRegisterClass H_HI_DummyRCRegClass;
  extern const TargetRegisterClass Q_HI_DummyRCRegClass;
  extern const TargetRegisterClass S_HI_DummyRCRegClass;
  extern const TargetRegisterClass FPR8RegClass;
  extern const TargetRegisterClass FPR16RegClass;
  extern const TargetRegisterClass PPRorPNRRegClass;
  extern const TargetRegisterClass FPR16_loRegClass;
  extern const TargetRegisterClass PNRRegClass;
  extern const TargetRegisterClass PPRRegClass;
  extern const TargetRegisterClass PNR_3bRegClass;
  extern const TargetRegisterClass PNR_p8to15RegClass;
  extern const TargetRegisterClass PPRMul2RegClass;
  extern const TargetRegisterClass PPR_3bRegClass;
  extern const TargetRegisterClass PPR_p8to15RegClass;
  extern const TargetRegisterClass PPRMul2_and_PPR_3bRegClass;
  extern const TargetRegisterClass PPRMul2_and_PPR_p8to15RegClass;
  extern const TargetRegisterClass PPR2RegClass;
  extern const TargetRegisterClass PPR2Mul2RegClass;
  extern const TargetRegisterClass PPR2_with_psub1_in_PPRMul2RegClass;
  extern const TargetRegisterClass PPR2_with_psub1_in_PPR_3bRegClass;
  extern const TargetRegisterClass PPR2_with_psub1_in_PPR_p8to15RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_3bRegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_p8to15RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_3b_and_PPR2_with_psub1_in_PPR_3bRegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_p8to15_and_PPR2_with_psub1_in_PPR_p8to15RegClass;
  extern const TargetRegisterClass PPR2Mul2_and_PPR2_with_psub_in_PNR_3bRegClass;
  extern const TargetRegisterClass PPR2Mul2_and_PPR2_with_psub_in_PNR_p8to15RegClass;
  extern const TargetRegisterClass PPR2_with_psub1_in_PPRMul2_and_PPR_3bRegClass;
  extern const TargetRegisterClass PPR2_with_psub1_in_PPRMul2_and_PPR_p8to15RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_3b_and_PPR2_with_psub1_in_PPRMul2RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_p8to15_and_PPR2_with_psub1_in_PPRMul2RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_3b_and_PPR2_with_psub1_in_PPRMul2_and_PPR_3bRegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_p8to15_and_PPR2_with_psub1_in_PPRMul2_and_PPR_p8to15RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_3b_and_PPR2_with_psub1_in_PPR_p8to15RegClass;
  extern const TargetRegisterClass PPR2_with_psub_in_PNR_p8to15_and_PPR2_with_psub1_in_PPR_3bRegClass;
  extern const TargetRegisterClass GPR32allRegClass;
  extern const TargetRegisterClass FPR32RegClass;
  extern const TargetRegisterClass GPR32RegClass;
  extern const TargetRegisterClass GPR32spRegClass;
  extern const TargetRegisterClass GPR32commonRegClass;
  extern const TargetRegisterClass FPR32_with_hsub_in_FPR16_loRegClass;
  extern const TargetRegisterClass GPR32argRegClass;
  extern const TargetRegisterClass MatrixIndexGPR32_12_15RegClass;
  extern const TargetRegisterClass MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass CCRRegClass;
  extern const TargetRegisterClass GPR32sponlyRegClass;
  extern const TargetRegisterClass WSeqPairsClassRegClass;
  extern const TargetRegisterClass WSeqPairsClass_with_subo32_in_GPR32commonRegClass;
  extern const TargetRegisterClass WSeqPairsClass_with_sube32_in_GPR32argRegClass;
  extern const TargetRegisterClass WSeqPairsClass_with_sube32_in_MatrixIndexGPR32_12_15RegClass;
  extern const TargetRegisterClass WSeqPairsClass_with_sube32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64allRegClass;
  extern const TargetRegisterClass FPR64RegClass;
  extern const TargetRegisterClass GPR64RegClass;
  extern const TargetRegisterClass GPR64spRegClass;
  extern const TargetRegisterClass GPR64commonRegClass;
  extern const TargetRegisterClass GPR64noipRegClass;
  extern const TargetRegisterClass GPR64common_and_GPR64noipRegClass;
  extern const TargetRegisterClass tcGPR64RegClass;
  extern const TargetRegisterClass tcGPRnotx16RegClass;
  extern const TargetRegisterClass tcGPRnotx16x17RegClass;
  extern const TargetRegisterClass FPR64_loRegClass;
  extern const TargetRegisterClass GPR64argRegClass;
  extern const TargetRegisterClass FIXED_REGSRegClass;
  extern const TargetRegisterClass GPR64_with_sub_32_in_MatrixIndexGPR32_12_15RegClass;
  extern const TargetRegisterClass GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass FIXED_REGS_with_sub_32RegClass;
  extern const TargetRegisterClass tcGPRx16x17RegClass;
  extern const TargetRegisterClass FIXED_REGS_and_GPR64RegClass;
  extern const TargetRegisterClass GPR64sponlyRegClass;
  extern const TargetRegisterClass tcGPRx17RegClass;
  extern const TargetRegisterClass DDRegClass;
  extern const TargetRegisterClass DD_with_dsub0_in_FPR64_loRegClass;
  extern const TargetRegisterClass DD_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass XSeqPairsClassRegClass;
  extern const TargetRegisterClass DD_with_dsub0_in_FPR64_lo_and_DD_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_subo64_in_GPR64commonRegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_subo64_in_GPR64noipRegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_GPR64noipRegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_tcGPR64RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_tcGPRnotx16RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_subo64_in_tcGPR64RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_subo64_in_tcGPRnotx16x17RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_GPR64argRegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_GPR64_with_sub_32_in_MatrixIndexGPR32_12_15RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_sube64_in_tcGPRx16x17RegClass;
  extern const TargetRegisterClass XSeqPairsClass_with_subo64_in_FIXED_REGSRegClass;
  extern const TargetRegisterClass FPR128RegClass;
  extern const TargetRegisterClass ZPRRegClass;
  extern const TargetRegisterClass FPR128_loRegClass;
  extern const TargetRegisterClass MPR128RegClass;
  extern const TargetRegisterClass ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR_4bRegClass;
  extern const TargetRegisterClass FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR_KRegClass;
  extern const TargetRegisterClass ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass DDDRegClass;
  extern const TargetRegisterClass DDD_with_dsub0_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDD_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDD_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDD_with_dsub0_in_FPR64_lo_and_DDD_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDD_with_dsub1_in_FPR64_lo_and_DDD_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDD_with_dsub0_in_FPR64_lo_and_DDD_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDDRegClass;
  extern const TargetRegisterClass DDDD_with_dsub0_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub0_in_FPR64_lo_and_DDDD_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub1_in_FPR64_lo_and_DDDD_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub2_in_FPR64_lo_and_DDDD_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub0_in_FPR64_lo_and_DDDD_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub1_in_FPR64_lo_and_DDDD_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass DDDD_with_dsub0_in_FPR64_lo_and_DDDD_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQRegClass;
  extern const TargetRegisterClass ZPR2RegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguousRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2RegClass;
  extern const TargetRegisterClass QQ_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQ_with_qsub0_in_FPR128_loRegClass;
  extern const TargetRegisterClass ZPR2Mul2RegClass;
  extern const TargetRegisterClass ZPR2StridedRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_dsub_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR2_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_loRegClass;
  extern const TargetRegisterClass QQ_with_qsub0_in_FPR128_lo_and_QQ_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_lo_and_ZPR2_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQ_with_qsub0_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQ_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR2Mul2_HiRegClass;
  extern const TargetRegisterClass ZPR2Mul2_LoRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2Strided_with_dsub_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR2_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_lo_and_ZPR2_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass QQ_with_qsub0_in_FPR128_0to7_and_QQ_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_0to7_and_ZPR2_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_lo_and_ZPR2_with_zsub1_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_0to7_and_ZPR2_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_lo_and_ZPR2_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2StridedOrContiguous_with_zsub0_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_0to7_and_ZPR2_with_zsub1_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_lo_and_ZPR2_with_zsub1_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub1_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_0to7_and_ZPR2_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2Strided_with_zsub0_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_dsub1_in_FPR64_lo_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_0to7_and_ZPR2_with_zsub1_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR2_with_zsub_in_FPR128_lo_and_ZPR2_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass MPR64RegClass;
  extern const TargetRegisterClass QQQRegClass;
  extern const TargetRegisterClass ZPR3RegClass;
  extern const TargetRegisterClass QQQ_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQ_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQ_with_qsub0_in_FPR128_loRegClass;
  extern const TargetRegisterClass ZPR3_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR3_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_loRegClass;
  extern const TargetRegisterClass QQQ_with_dsub1_in_FPR64_lo_and_QQQ_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQ_with_qsub0_in_FPR128_lo_and_QQQ_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR3_with_dsub1_in_FPR64_lo_and_ZPR3_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQ_with_qsub0_in_FPR128_lo_and_QQQ_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQ_with_qsub0_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQ_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQ_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_LoRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass QQQ_with_dsub1_in_FPR64_lo_and_QQQ_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQ_with_qsub0_in_FPR128_0to7_and_QQQ_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_dsub1_in_FPR64_lo_and_ZPR3_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub1_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub2_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass QQQ_with_qsub0_in_FPR128_0to7_and_QQQ_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPR_K_and_ZPR3_with_zsub2_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub2_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub2_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub2_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_dsub2_in_FPR64_lo_and_ZPR3_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub1_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub2_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_dsub1_in_FPR64_lo_and_ZPR3_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR3_with_dsub2_in_FPR64_lo_and_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_in_ZPR_K_and_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub1_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_0to7_and_ZPR3_with_zsub2_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR3_with_zsub_in_FPR128_lo_and_ZPR3_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass QQQQRegClass;
  extern const TargetRegisterClass ZPR4RegClass;
  extern const TargetRegisterClass QQQQ_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_loRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguousRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_loRegClass;
  extern const TargetRegisterClass QQQQ_with_dsub1_in_FPR64_lo_and_QQQQ_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_dsub2_in_FPR64_lo_and_QQQQ_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_lo_and_QQQQ_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub2_in_FPR64_lo_and_ZPR4_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_dsub1_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_dsub1_in_FPR64_lo_and_QQQQ_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_lo_and_QQQQ_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_dsub2_in_FPR64_loRegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_lo_and_QQQQ_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_dsub3_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQQ_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQQ_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQQ_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4Mul4RegClass;
  extern const TargetRegisterClass ZPR4StridedRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_dsub_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_LoRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass QQQQ_with_dsub1_in_FPR64_lo_and_QQQQ_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQQ_with_dsub2_in_FPR64_lo_and_QQQQ_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_0to7_and_QQQQ_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_zsub3_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub2_in_FPR64_lo_and_ZPR4_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_qsub1_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub1_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub2_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass QQQQ_with_dsub1_in_FPR64_lo_and_QQQQ_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_0to7_and_QQQQ_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPR_K_and_ZPR4_with_zsub2_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPR_K_and_ZPR4_with_zsub3_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_qsub2_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub3_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass QQQQ_with_qsub0_in_FPR128_0to7_and_QQQQ_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_qsub3_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2_and_ZPR4StridedOrContiguous_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4Strided_with_dsub_in_FPR64_loRegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub0_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub2_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPR_K_and_ZPR4_with_zsub3_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul2_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub3_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul4_and_ZPR4StridedOrContiguous_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4StridedOrContiguous_with_zsub0_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_zsub3_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub3_in_FPR64_lo_and_ZPR4_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_Hi_and_ZPR3_with_zsub2_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul2_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub1_zsub2_zsub3_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Lo_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub3_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4Mul4_and_ZPR4_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4Mul4_and_ZPR4_with_zsub_in_FPR128_0to7RegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub0_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub0_in_ZPRMul2_LoRegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_dsub2_in_FPR64_lo_and_ZPR4_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub3_in_FPR64_lo_and_ZPR4_with_zsub1_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub3_in_FPR64_lo_with_zsub0_zsub1_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_and_ZPR2_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub3_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub1_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub2_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub3_in_ZPRMul4_and_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub1_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub2_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub3_in_ZPRMul2_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub3_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub3_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub0_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4Strided_with_zsub0_in_ZPRMul2_Lo_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_dsub1_in_FPR64_lo_and_ZPR4_with_zsub0_in_ZPR_KRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub2_in_FPR64_lo_and_ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_dsub3_in_FPR64_lo_and_ZPR4_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub1_zsub2_in_ZPR2Mul2_Hi_and_ZPR2_with_zsub0_in_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_in_ZPR_K_and_ZPR4_with_zsub3_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub0_zsub1_zsub2_in_ZPR3_with_zsub0_zsub1_in_ZPR2Mul2_with_zsub1_zsub2_in_ZPR2_with_zsub0_in_ZPR_K_and_ZPR2_with_zsub1_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub1_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub2_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_0to7_and_ZPR4_with_zsub3_in_ZPRMul4_and_ZPR_3bRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub1_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub2_in_ZPRMul2_HiRegClass;
  extern const TargetRegisterClass ZPR4_with_zsub_in_FPR128_lo_and_ZPR4_with_zsub3_in_ZPRMul2_Hi_and_ZPRMul4RegClass;
  extern const TargetRegisterClass GPR64x8ClassRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_2_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPRnotx16RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_1_in_tcGPR64RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_1_in_tcGPRnotx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_tcGPRnotx16RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_4_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_3_in_tcGPRnotx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_tcGPRnotx16RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_0_in_tcGPR64_and_GPR64x8Class_with_x8sub_0_in_GPR64noip_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64noip_and_GPR64x8Class_with_x8sub_2_in_GPR64noip_and_GPR64x8Class_with_x8sub_1_in_tcGPR64_and_GPR64x8Class_with_x8sub_6_in_GPR64noipRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_5_in_tcGPRnotx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_6_in_tcGPRnotx16RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_7_in_tcGPRnotx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_sub_32_in_GPR32argRegClass;
  extern const TargetRegisterClass MPR32RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64argRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_sub_32_in_MatrixIndexGPR32_12_15RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64_with_sub_32_in_MatrixIndexGPR32_12_15RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64argRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_6_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_sub_32_in_GPR32arg_and_GPR64x8Class_with_x8sub_2_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_sub_32_in_MatrixIndexGPR32_8_11_and_GPR64x8Class_with_x8sub_2_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_0_in_tcGPRx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_GPR64arg_and_GPR64x8Class_with_x8sub_4_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_2_in_tcGPRx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_GPR64arg_and_GPR64x8Class_with_x8sub_6_in_GPR64_with_sub_32_in_MatrixIndexGPR32_8_11RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_4_in_tcGPRx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_6_in_GPR64argRegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_6_in_tcGPRx16x17RegClass;
  extern const TargetRegisterClass GPR64x8Class_with_x8sub_7_in_FIXED_REGSRegClass;
  extern const TargetRegisterClass ZTRRegClass;
  extern const TargetRegisterClass MPR16RegClass;
  extern const TargetRegisterClass MPRRegClass;
  extern const TargetRegisterClass MPR8RegClass;
} // end namespace AArch64

} // end namespace llvm

