/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|*  * Hexagon Disassembler                                                    *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#include "llvm/MC/MCInst.h"
#include "llvm/MC/MCSubtargetInfo.h"
#include "llvm/Support/DataTypes.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/LEB128.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/TargetParser/SubtargetFeature.h"
#include <assert.h>

namespace {

// InsnBitWidth is essentially a type trait used by the decoder emitter to query
// the supported bitwidth for a given type. But default, the value is 0, making
// it an invalid type for use as `InsnType` when instantiating the decoder.
// Individual targets are expected to provide specializations for these based
// on their usage.
template <typename T> constexpr uint32_t InsnBitWidth = 0;

static const uint8_t DecoderTable32[19322] = {
  OPC_SwitchField, 28, 4,                    //     0: switch Inst[31:28] {
  0, 4,                                      //     3: case 0x0: {
  OPC_Decode, 235, 8, 0,                     //     5:  decode to A4_ext using decoder 0
                                             //     5: }
  1, 217, 6,                                 //     9: case 0x1: {
  OPC_SwitchField, 22, 6,                    //    12:  switch Inst[27:22] {
  0, 23,                                     //    15:  case 0x0: {
  OPC_SwitchField, 13, 1,                    //    17:   switch Inst[13] {
  0, 8,                                      //    20:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //    22:    check Inst[0] == 0x0
  OPC_Decode, 136, 11, 1,                    //    26:    decode to J4_cmpeqi_tp0_jump_nt using decoder 1
                                             //    26:   }
  1, 0,                                      //    30:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //    32:    check Inst[0] == 0x0
  OPC_Decode, 137, 11, 1,                    //    36:    decode to J4_cmpeqi_tp0_jump_t using decoder 1
                                             //    36:   }
                                             //    36:   } // switch Inst[13]
                                             //    36:  }
  1, 23,                                     //    40:  case 0x1: {
  OPC_SwitchField, 13, 1,                    //    42:   switch Inst[13] {
  0, 8,                                      //    45:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //    47:    check Inst[0] == 0x0
  OPC_Decode, 130, 11, 1,                    //    51:    decode to J4_cmpeqi_fp0_jump_nt using decoder 1
                                             //    51:   }
  1, 0,                                      //    55:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //    57:    check Inst[0] == 0x0
  OPC_Decode, 131, 11, 1,                    //    61:    decode to J4_cmpeqi_fp0_jump_t using decoder 1
                                             //    61:   }
                                             //    61:   } // switch Inst[13]
                                             //    61:  }
  2, 23,                                     //    65:  case 0x2: {
  OPC_SwitchField, 13, 1,                    //    67:   switch Inst[13] {
  0, 8,                                      //    70:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //    72:    check Inst[0] == 0x0
  OPC_Decode, 172, 11, 1,                    //    76:    decode to J4_cmpgti_tp0_jump_nt using decoder 1
                                             //    76:   }
  1, 0,                                      //    80:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //    82:    check Inst[0] == 0x0
  OPC_Decode, 173, 11, 1,                    //    86:    decode to J4_cmpgti_tp0_jump_t using decoder 1
                                             //    86:   }
                                             //    86:   } // switch Inst[13]
                                             //    86:  }
  3, 23,                                     //    90:  case 0x3: {
  OPC_SwitchField, 13, 1,                    //    92:   switch Inst[13] {
  0, 8,                                      //    95:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //    97:    check Inst[0] == 0x0
  OPC_Decode, 166, 11, 1,                    //   101:    decode to J4_cmpgti_fp0_jump_nt using decoder 1
                                             //   101:   }
  1, 0,                                      //   105:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   107:    check Inst[0] == 0x0
  OPC_Decode, 167, 11, 1,                    //   111:    decode to J4_cmpgti_fp0_jump_t using decoder 1
                                             //   111:   }
                                             //   111:   } // switch Inst[13]
                                             //   111:  }
  4, 23,                                     //   115:  case 0x4: {
  OPC_SwitchField, 13, 1,                    //   117:   switch Inst[13] {
  0, 8,                                      //   120:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   122:    check Inst[0] == 0x0
  OPC_Decode, 208, 11, 1,                    //   126:    decode to J4_cmpgtui_tp0_jump_nt using decoder 1
                                             //   126:   }
  1, 0,                                      //   130:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   132:    check Inst[0] == 0x0
  OPC_Decode, 209, 11, 1,                    //   136:    decode to J4_cmpgtui_tp0_jump_t using decoder 1
                                             //   136:   }
                                             //   136:   } // switch Inst[13]
                                             //   136:  }
  5, 23,                                     //   140:  case 0x5: {
  OPC_SwitchField, 13, 1,                    //   142:   switch Inst[13] {
  0, 8,                                      //   145:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   147:    check Inst[0] == 0x0
  OPC_Decode, 202, 11, 1,                    //   151:    decode to J4_cmpgtui_fp0_jump_nt using decoder 1
                                             //   151:   }
  1, 0,                                      //   155:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   157:    check Inst[0] == 0x0
  OPC_Decode, 203, 11, 1,                    //   161:    decode to J4_cmpgtui_fp0_jump_t using decoder 1
                                             //   161:   }
                                             //   161:   } // switch Inst[13]
                                             //   161:  }
  6, 63,                                     //   165:  case 0x6: {
  OPC_SwitchField, 8, 6,                     //   167:   switch Inst[13:8] {
  0, 8,                                      //   170:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   172:    check Inst[0] == 0x0
  OPC_Decode, 148, 11, 2,                    //   176:    decode to J4_cmpeqn1_tp0_jump_nt using decoder 2
                                             //   176:   }
  1, 8,                                      //   180:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   182:    check Inst[0] == 0x0
  OPC_Decode, 184, 11, 2,                    //   186:    decode to J4_cmpgtn1_tp0_jump_nt using decoder 2
                                             //   186:   }
  3, 8,                                      //   190:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   192:    check Inst[0] == 0x0
  OPC_Decode, 231, 11, 3,                    //   196:    decode to J4_tstbit0_tp0_jump_nt using decoder 3
                                             //   196:   }
  32, 8,                                     //   200:   case 0x20: {
  OPC_CheckField, 0, 1, 0,                   //   202:    check Inst[0] == 0x0
  OPC_Decode, 149, 11, 2,                    //   206:    decode to J4_cmpeqn1_tp0_jump_t using decoder 2
                                             //   206:   }
  33, 8,                                     //   210:   case 0x21: {
  OPC_CheckField, 0, 1, 0,                   //   212:    check Inst[0] == 0x0
  OPC_Decode, 185, 11, 2,                    //   216:    decode to J4_cmpgtn1_tp0_jump_t using decoder 2
                                             //   216:   }
  35, 0,                                     //   220:   case 0x23: {
  OPC_CheckField, 0, 1, 0,                   //   222:    check Inst[0] == 0x0
  OPC_Decode, 232, 11, 3,                    //   226:    decode to J4_tstbit0_tp0_jump_t using decoder 3
                                             //   226:   }
                                             //   226:   } // switch Inst[13:8]
                                             //   226:  }
  7, 63,                                     //   230:  case 0x7: {
  OPC_SwitchField, 8, 6,                     //   232:   switch Inst[13:8] {
  0, 8,                                      //   235:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   237:    check Inst[0] == 0x0
  OPC_Decode, 142, 11, 2,                    //   241:    decode to J4_cmpeqn1_fp0_jump_nt using decoder 2
                                             //   241:   }
  1, 8,                                      //   245:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   247:    check Inst[0] == 0x0
  OPC_Decode, 178, 11, 2,                    //   251:    decode to J4_cmpgtn1_fp0_jump_nt using decoder 2
                                             //   251:   }
  3, 8,                                      //   255:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   257:    check Inst[0] == 0x0
  OPC_Decode, 225, 11, 3,                    //   261:    decode to J4_tstbit0_fp0_jump_nt using decoder 3
                                             //   261:   }
  32, 8,                                     //   265:   case 0x20: {
  OPC_CheckField, 0, 1, 0,                   //   267:    check Inst[0] == 0x0
  OPC_Decode, 143, 11, 2,                    //   271:    decode to J4_cmpeqn1_fp0_jump_t using decoder 2
                                             //   271:   }
  33, 8,                                     //   275:   case 0x21: {
  OPC_CheckField, 0, 1, 0,                   //   277:    check Inst[0] == 0x0
  OPC_Decode, 179, 11, 2,                    //   281:    decode to J4_cmpgtn1_fp0_jump_t using decoder 2
                                             //   281:   }
  35, 0,                                     //   285:   case 0x23: {
  OPC_CheckField, 0, 1, 0,                   //   287:    check Inst[0] == 0x0
  OPC_Decode, 226, 11, 3,                    //   291:    decode to J4_tstbit0_fp0_jump_t using decoder 3
                                             //   291:   }
                                             //   291:   } // switch Inst[13:8]
                                             //   291:  }
  8, 23,                                     //   295:  case 0x8: {
  OPC_SwitchField, 13, 1,                    //   297:   switch Inst[13] {
  0, 8,                                      //   300:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   302:    check Inst[0] == 0x0
  OPC_Decode, 138, 11, 1,                    //   306:    decode to J4_cmpeqi_tp1_jump_nt using decoder 1
                                             //   306:   }
  1, 0,                                      //   310:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   312:    check Inst[0] == 0x0
  OPC_Decode, 139, 11, 1,                    //   316:    decode to J4_cmpeqi_tp1_jump_t using decoder 1
                                             //   316:   }
                                             //   316:   } // switch Inst[13]
                                             //   316:  }
  9, 23,                                     //   320:  case 0x9: {
  OPC_SwitchField, 13, 1,                    //   322:   switch Inst[13] {
  0, 8,                                      //   325:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   327:    check Inst[0] == 0x0
  OPC_Decode, 132, 11, 1,                    //   331:    decode to J4_cmpeqi_fp1_jump_nt using decoder 1
                                             //   331:   }
  1, 0,                                      //   335:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   337:    check Inst[0] == 0x0
  OPC_Decode, 133, 11, 1,                    //   341:    decode to J4_cmpeqi_fp1_jump_t using decoder 1
                                             //   341:   }
                                             //   341:   } // switch Inst[13]
                                             //   341:  }
  10, 23,                                    //   345:  case 0xa: {
  OPC_SwitchField, 13, 1,                    //   347:   switch Inst[13] {
  0, 8,                                      //   350:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   352:    check Inst[0] == 0x0
  OPC_Decode, 174, 11, 1,                    //   356:    decode to J4_cmpgti_tp1_jump_nt using decoder 1
                                             //   356:   }
  1, 0,                                      //   360:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   362:    check Inst[0] == 0x0
  OPC_Decode, 175, 11, 1,                    //   366:    decode to J4_cmpgti_tp1_jump_t using decoder 1
                                             //   366:   }
                                             //   366:   } // switch Inst[13]
                                             //   366:  }
  11, 23,                                    //   370:  case 0xb: {
  OPC_SwitchField, 13, 1,                    //   372:   switch Inst[13] {
  0, 8,                                      //   375:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   377:    check Inst[0] == 0x0
  OPC_Decode, 168, 11, 1,                    //   381:    decode to J4_cmpgti_fp1_jump_nt using decoder 1
                                             //   381:   }
  1, 0,                                      //   385:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   387:    check Inst[0] == 0x0
  OPC_Decode, 169, 11, 1,                    //   391:    decode to J4_cmpgti_fp1_jump_t using decoder 1
                                             //   391:   }
                                             //   391:   } // switch Inst[13]
                                             //   391:  }
  12, 23,                                    //   395:  case 0xc: {
  OPC_SwitchField, 13, 1,                    //   397:   switch Inst[13] {
  0, 8,                                      //   400:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   402:    check Inst[0] == 0x0
  OPC_Decode, 210, 11, 1,                    //   406:    decode to J4_cmpgtui_tp1_jump_nt using decoder 1
                                             //   406:   }
  1, 0,                                      //   410:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   412:    check Inst[0] == 0x0
  OPC_Decode, 211, 11, 1,                    //   416:    decode to J4_cmpgtui_tp1_jump_t using decoder 1
                                             //   416:   }
                                             //   416:   } // switch Inst[13]
                                             //   416:  }
  13, 23,                                    //   420:  case 0xd: {
  OPC_SwitchField, 13, 1,                    //   422:   switch Inst[13] {
  0, 8,                                      //   425:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   427:    check Inst[0] == 0x0
  OPC_Decode, 204, 11, 1,                    //   431:    decode to J4_cmpgtui_fp1_jump_nt using decoder 1
                                             //   431:   }
  1, 0,                                      //   435:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   437:    check Inst[0] == 0x0
  OPC_Decode, 205, 11, 1,                    //   441:    decode to J4_cmpgtui_fp1_jump_t using decoder 1
                                             //   441:   }
                                             //   441:   } // switch Inst[13]
                                             //   441:  }
  14, 63,                                    //   445:  case 0xe: {
  OPC_SwitchField, 8, 6,                     //   447:   switch Inst[13:8] {
  0, 8,                                      //   450:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   452:    check Inst[0] == 0x0
  OPC_Decode, 150, 11, 2,                    //   456:    decode to J4_cmpeqn1_tp1_jump_nt using decoder 2
                                             //   456:   }
  1, 8,                                      //   460:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   462:    check Inst[0] == 0x0
  OPC_Decode, 186, 11, 2,                    //   466:    decode to J4_cmpgtn1_tp1_jump_nt using decoder 2
                                             //   466:   }
  3, 8,                                      //   470:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   472:    check Inst[0] == 0x0
  OPC_Decode, 233, 11, 3,                    //   476:    decode to J4_tstbit0_tp1_jump_nt using decoder 3
                                             //   476:   }
  32, 8,                                     //   480:   case 0x20: {
  OPC_CheckField, 0, 1, 0,                   //   482:    check Inst[0] == 0x0
  OPC_Decode, 151, 11, 2,                    //   486:    decode to J4_cmpeqn1_tp1_jump_t using decoder 2
                                             //   486:   }
  33, 8,                                     //   490:   case 0x21: {
  OPC_CheckField, 0, 1, 0,                   //   492:    check Inst[0] == 0x0
  OPC_Decode, 187, 11, 2,                    //   496:    decode to J4_cmpgtn1_tp1_jump_t using decoder 2
                                             //   496:   }
  35, 0,                                     //   500:   case 0x23: {
  OPC_CheckField, 0, 1, 0,                   //   502:    check Inst[0] == 0x0
  OPC_Decode, 234, 11, 3,                    //   506:    decode to J4_tstbit0_tp1_jump_t using decoder 3
                                             //   506:   }
                                             //   506:   } // switch Inst[13:8]
                                             //   506:  }
  15, 63,                                    //   510:  case 0xf: {
  OPC_SwitchField, 8, 6,                     //   512:   switch Inst[13:8] {
  0, 8,                                      //   515:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   517:    check Inst[0] == 0x0
  OPC_Decode, 144, 11, 2,                    //   521:    decode to J4_cmpeqn1_fp1_jump_nt using decoder 2
                                             //   521:   }
  1, 8,                                      //   525:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   527:    check Inst[0] == 0x0
  OPC_Decode, 180, 11, 2,                    //   531:    decode to J4_cmpgtn1_fp1_jump_nt using decoder 2
                                             //   531:   }
  3, 8,                                      //   535:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   537:    check Inst[0] == 0x0
  OPC_Decode, 227, 11, 3,                    //   541:    decode to J4_tstbit0_fp1_jump_nt using decoder 3
                                             //   541:   }
  32, 8,                                     //   545:   case 0x20: {
  OPC_CheckField, 0, 1, 0,                   //   547:    check Inst[0] == 0x0
  OPC_Decode, 145, 11, 2,                    //   551:    decode to J4_cmpeqn1_fp1_jump_t using decoder 2
                                             //   551:   }
  33, 8,                                     //   555:   case 0x21: {
  OPC_CheckField, 0, 1, 0,                   //   557:    check Inst[0] == 0x0
  OPC_Decode, 181, 11, 2,                    //   561:    decode to J4_cmpgtn1_fp1_jump_t using decoder 2
                                             //   561:   }
  35, 0,                                     //   565:   case 0x23: {
  OPC_CheckField, 0, 1, 0,                   //   567:    check Inst[0] == 0x0
  OPC_Decode, 228, 11, 3,                    //   571:    decode to J4_tstbit0_fp1_jump_t using decoder 3
                                             //   571:   }
                                             //   571:   } // switch Inst[13:8]
                                             //   571:  }
  16, 43,                                    //   575:  case 0x10: {
  OPC_SwitchField, 12, 2,                    //   577:   switch Inst[13:12] {
  0, 8,                                      //   580:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   582:    check Inst[0] == 0x0
  OPC_Decode, 252, 10, 4,                    //   586:    decode to J4_cmpeq_tp0_jump_nt using decoder 4
                                             //   586:   }
  1, 8,                                      //   590:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   592:    check Inst[0] == 0x0
  OPC_Decode, 254, 10, 4,                    //   596:    decode to J4_cmpeq_tp1_jump_nt using decoder 4
                                             //   596:   }
  2, 8,                                      //   600:   case 0x2: {
  OPC_CheckField, 0, 1, 0,                   //   602:    check Inst[0] == 0x0
  OPC_Decode, 253, 10, 4,                    //   606:    decode to J4_cmpeq_tp0_jump_t using decoder 4
                                             //   606:   }
  3, 0,                                      //   610:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   612:    check Inst[0] == 0x0
  OPC_Decode, 255, 10, 4,                    //   616:    decode to J4_cmpeq_tp1_jump_t using decoder 4
                                             //   616:   }
                                             //   616:   } // switch Inst[13:12]
                                             //   616:  }
  17, 43,                                    //   620:  case 0x11: {
  OPC_SwitchField, 12, 2,                    //   622:   switch Inst[13:12] {
  0, 8,                                      //   625:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   627:    check Inst[0] == 0x0
  OPC_Decode, 246, 10, 4,                    //   631:    decode to J4_cmpeq_fp0_jump_nt using decoder 4
                                             //   631:   }
  1, 8,                                      //   635:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   637:    check Inst[0] == 0x0
  OPC_Decode, 248, 10, 4,                    //   641:    decode to J4_cmpeq_fp1_jump_nt using decoder 4
                                             //   641:   }
  2, 8,                                      //   645:   case 0x2: {
  OPC_CheckField, 0, 1, 0,                   //   647:    check Inst[0] == 0x0
  OPC_Decode, 247, 10, 4,                    //   651:    decode to J4_cmpeq_fp0_jump_t using decoder 4
                                             //   651:   }
  3, 0,                                      //   655:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   657:    check Inst[0] == 0x0
  OPC_Decode, 249, 10, 4,                    //   661:    decode to J4_cmpeq_fp1_jump_t using decoder 4
                                             //   661:   }
                                             //   661:   } // switch Inst[13:12]
                                             //   661:  }
  18, 43,                                    //   665:  case 0x12: {
  OPC_SwitchField, 12, 2,                    //   667:   switch Inst[13:12] {
  0, 8,                                      //   670:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   672:    check Inst[0] == 0x0
  OPC_Decode, 160, 11, 4,                    //   676:    decode to J4_cmpgt_tp0_jump_nt using decoder 4
                                             //   676:   }
  1, 8,                                      //   680:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   682:    check Inst[0] == 0x0
  OPC_Decode, 162, 11, 4,                    //   686:    decode to J4_cmpgt_tp1_jump_nt using decoder 4
                                             //   686:   }
  2, 8,                                      //   690:   case 0x2: {
  OPC_CheckField, 0, 1, 0,                   //   692:    check Inst[0] == 0x0
  OPC_Decode, 161, 11, 4,                    //   696:    decode to J4_cmpgt_tp0_jump_t using decoder 4
                                             //   696:   }
  3, 0,                                      //   700:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   702:    check Inst[0] == 0x0
  OPC_Decode, 163, 11, 4,                    //   706:    decode to J4_cmpgt_tp1_jump_t using decoder 4
                                             //   706:   }
                                             //   706:   } // switch Inst[13:12]
                                             //   706:  }
  19, 43,                                    //   710:  case 0x13: {
  OPC_SwitchField, 12, 2,                    //   712:   switch Inst[13:12] {
  0, 8,                                      //   715:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   717:    check Inst[0] == 0x0
  OPC_Decode, 154, 11, 4,                    //   721:    decode to J4_cmpgt_fp0_jump_nt using decoder 4
                                             //   721:   }
  1, 8,                                      //   725:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   727:    check Inst[0] == 0x0
  OPC_Decode, 156, 11, 4,                    //   731:    decode to J4_cmpgt_fp1_jump_nt using decoder 4
                                             //   731:   }
  2, 8,                                      //   735:   case 0x2: {
  OPC_CheckField, 0, 1, 0,                   //   737:    check Inst[0] == 0x0
  OPC_Decode, 155, 11, 4,                    //   741:    decode to J4_cmpgt_fp0_jump_t using decoder 4
                                             //   741:   }
  3, 0,                                      //   745:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   747:    check Inst[0] == 0x0
  OPC_Decode, 157, 11, 4,                    //   751:    decode to J4_cmpgt_fp1_jump_t using decoder 4
                                             //   751:   }
                                             //   751:   } // switch Inst[13:12]
                                             //   751:  }
  20, 43,                                    //   755:  case 0x14: {
  OPC_SwitchField, 12, 2,                    //   757:   switch Inst[13:12] {
  0, 8,                                      //   760:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   762:    check Inst[0] == 0x0
  OPC_Decode, 196, 11, 4,                    //   766:    decode to J4_cmpgtu_tp0_jump_nt using decoder 4
                                             //   766:   }
  1, 8,                                      //   770:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   772:    check Inst[0] == 0x0
  OPC_Decode, 198, 11, 4,                    //   776:    decode to J4_cmpgtu_tp1_jump_nt using decoder 4
                                             //   776:   }
  2, 8,                                      //   780:   case 0x2: {
  OPC_CheckField, 0, 1, 0,                   //   782:    check Inst[0] == 0x0
  OPC_Decode, 197, 11, 4,                    //   786:    decode to J4_cmpgtu_tp0_jump_t using decoder 4
                                             //   786:   }
  3, 0,                                      //   790:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   792:    check Inst[0] == 0x0
  OPC_Decode, 199, 11, 4,                    //   796:    decode to J4_cmpgtu_tp1_jump_t using decoder 4
                                             //   796:   }
                                             //   796:   } // switch Inst[13:12]
                                             //   796:  }
  21, 43,                                    //   800:  case 0x15: {
  OPC_SwitchField, 12, 2,                    //   802:   switch Inst[13:12] {
  0, 8,                                      //   805:   case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //   807:    check Inst[0] == 0x0
  OPC_Decode, 190, 11, 4,                    //   811:    decode to J4_cmpgtu_fp0_jump_nt using decoder 4
                                             //   811:   }
  1, 8,                                      //   815:   case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //   817:    check Inst[0] == 0x0
  OPC_Decode, 192, 11, 4,                    //   821:    decode to J4_cmpgtu_fp1_jump_nt using decoder 4
                                             //   821:   }
  2, 8,                                      //   825:   case 0x2: {
  OPC_CheckField, 0, 1, 0,                   //   827:    check Inst[0] == 0x0
  OPC_Decode, 191, 11, 4,                    //   831:    decode to J4_cmpgtu_fp0_jump_t using decoder 4
                                             //   831:   }
  3, 0,                                      //   835:   case 0x3: {
  OPC_CheckField, 0, 1, 0,                   //   837:    check Inst[0] == 0x0
  OPC_Decode, 193, 11, 4,                    //   841:    decode to J4_cmpgtu_fp1_jump_t using decoder 4
                                             //   841:   }
                                             //   841:   } // switch Inst[13:12]
                                             //   841:  }
  24, 8,                                     //   845:  case 0x18: {
  OPC_CheckField, 0, 1, 0,                   //   847:   check Inst[0] == 0x0
  OPC_Decode, 221, 11, 5,                    //   851:   decode to J4_jumpseti using decoder 5
                                             //   851:  }
  28, 0,                                     //   855:  case 0x1c: {
  OPC_CheckField, 12, 2, 0,                  //   857:   check Inst[13:12] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   861:   check Inst[0] == 0x0
  OPC_Decode, 222, 11, 6,                    //   865:   decode to J4_jumpsetr using decoder 6
                                             //   865:  }
                                             //   865:  } // switch Inst[27:22]
                                             //   865: }
  2, 217, 5,                                 //   869: case 0x2: {
  OPC_SwitchField, 22, 6,                    //   872:  switch Inst[27:22] {
  0, 31,                                     //   875:  case 0x0: {
  OPC_SwitchField, 13, 1,                    //   877:   switch Inst[13] {
  0, 12,                                     //   880:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //   882:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   886:    check Inst[0] == 0x0
  OPC_Decode, 250, 10, 7,                    //   890:    decode to J4_cmpeq_t_jumpnv_nt using decoder 7
                                             //   890:   }
  1, 0,                                      //   894:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //   896:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   900:    check Inst[0] == 0x0
  OPC_Decode, 251, 10, 7,                    //   904:    decode to J4_cmpeq_t_jumpnv_t using decoder 7
                                             //   904:   }
                                             //   904:   } // switch Inst[13]
                                             //   904:  }
  1, 31,                                     //   908:  case 0x1: {
  OPC_SwitchField, 13, 1,                    //   910:   switch Inst[13] {
  0, 12,                                     //   913:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //   915:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   919:    check Inst[0] == 0x0
  OPC_Decode, 244, 10, 7,                    //   923:    decode to J4_cmpeq_f_jumpnv_nt using decoder 7
                                             //   923:   }
  1, 0,                                      //   927:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //   929:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   933:    check Inst[0] == 0x0
  OPC_Decode, 245, 10, 7,                    //   937:    decode to J4_cmpeq_f_jumpnv_t using decoder 7
                                             //   937:   }
                                             //   937:   } // switch Inst[13]
                                             //   937:  }
  2, 31,                                     //   941:  case 0x2: {
  OPC_SwitchField, 13, 1,                    //   943:   switch Inst[13] {
  0, 12,                                     //   946:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //   948:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   952:    check Inst[0] == 0x0
  OPC_Decode, 158, 11, 7,                    //   956:    decode to J4_cmpgt_t_jumpnv_nt using decoder 7
                                             //   956:   }
  1, 0,                                      //   960:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //   962:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   966:    check Inst[0] == 0x0
  OPC_Decode, 159, 11, 7,                    //   970:    decode to J4_cmpgt_t_jumpnv_t using decoder 7
                                             //   970:   }
                                             //   970:   } // switch Inst[13]
                                             //   970:  }
  3, 31,                                     //   974:  case 0x3: {
  OPC_SwitchField, 13, 1,                    //   976:   switch Inst[13] {
  0, 12,                                     //   979:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //   981:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   985:    check Inst[0] == 0x0
  OPC_Decode, 152, 11, 7,                    //   989:    decode to J4_cmpgt_f_jumpnv_nt using decoder 7
                                             //   989:   }
  1, 0,                                      //   993:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //   995:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //   999:    check Inst[0] == 0x0
  OPC_Decode, 153, 11, 7,                    //  1003:    decode to J4_cmpgt_f_jumpnv_t using decoder 7
                                             //  1003:   }
                                             //  1003:   } // switch Inst[13]
                                             //  1003:  }
  4, 31,                                     //  1007:  case 0x4: {
  OPC_SwitchField, 13, 1,                    //  1009:   switch Inst[13] {
  0, 12,                                     //  1012:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1014:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1018:    check Inst[0] == 0x0
  OPC_Decode, 194, 11, 7,                    //  1022:    decode to J4_cmpgtu_t_jumpnv_nt using decoder 7
                                             //  1022:   }
  1, 0,                                      //  1026:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1028:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1032:    check Inst[0] == 0x0
  OPC_Decode, 195, 11, 7,                    //  1036:    decode to J4_cmpgtu_t_jumpnv_t using decoder 7
                                             //  1036:   }
                                             //  1036:   } // switch Inst[13]
                                             //  1036:  }
  5, 31,                                     //  1040:  case 0x5: {
  OPC_SwitchField, 13, 1,                    //  1042:   switch Inst[13] {
  0, 12,                                     //  1045:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1047:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1051:    check Inst[0] == 0x0
  OPC_Decode, 188, 11, 7,                    //  1055:    decode to J4_cmpgtu_f_jumpnv_nt using decoder 7
                                             //  1055:   }
  1, 0,                                      //  1059:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1061:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1065:    check Inst[0] == 0x0
  OPC_Decode, 189, 11, 7,                    //  1069:    decode to J4_cmpgtu_f_jumpnv_t using decoder 7
                                             //  1069:   }
                                             //  1069:   } // switch Inst[13]
                                             //  1069:  }
  6, 31,                                     //  1073:  case 0x6: {
  OPC_SwitchField, 13, 1,                    //  1075:   switch Inst[13] {
  0, 12,                                     //  1078:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1080:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1084:    check Inst[0] == 0x0
  OPC_Decode, 214, 11, 8,                    //  1088:    decode to J4_cmplt_t_jumpnv_nt using decoder 8
                                             //  1088:   }
  1, 0,                                      //  1092:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1094:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1098:    check Inst[0] == 0x0
  OPC_Decode, 215, 11, 8,                    //  1102:    decode to J4_cmplt_t_jumpnv_t using decoder 8
                                             //  1102:   }
                                             //  1102:   } // switch Inst[13]
                                             //  1102:  }
  7, 31,                                     //  1106:  case 0x7: {
  OPC_SwitchField, 13, 1,                    //  1108:   switch Inst[13] {
  0, 12,                                     //  1111:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1113:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1117:    check Inst[0] == 0x0
  OPC_Decode, 212, 11, 8,                    //  1121:    decode to J4_cmplt_f_jumpnv_nt using decoder 8
                                             //  1121:   }
  1, 0,                                      //  1125:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1127:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1131:    check Inst[0] == 0x0
  OPC_Decode, 213, 11, 8,                    //  1135:    decode to J4_cmplt_f_jumpnv_t using decoder 8
                                             //  1135:   }
                                             //  1135:   } // switch Inst[13]
                                             //  1135:  }
  8, 31,                                     //  1139:  case 0x8: {
  OPC_SwitchField, 13, 1,                    //  1141:   switch Inst[13] {
  0, 12,                                     //  1144:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1146:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1150:    check Inst[0] == 0x0
  OPC_Decode, 218, 11, 8,                    //  1154:    decode to J4_cmpltu_t_jumpnv_nt using decoder 8
                                             //  1154:   }
  1, 0,                                      //  1158:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1160:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1164:    check Inst[0] == 0x0
  OPC_Decode, 219, 11, 8,                    //  1168:    decode to J4_cmpltu_t_jumpnv_t using decoder 8
                                             //  1168:   }
                                             //  1168:   } // switch Inst[13]
                                             //  1168:  }
  9, 31,                                     //  1172:  case 0x9: {
  OPC_SwitchField, 13, 1,                    //  1174:   switch Inst[13] {
  0, 12,                                     //  1177:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1179:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1183:    check Inst[0] == 0x0
  OPC_Decode, 216, 11, 8,                    //  1187:    decode to J4_cmpltu_f_jumpnv_nt using decoder 8
                                             //  1187:   }
  1, 0,                                      //  1191:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1193:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1197:    check Inst[0] == 0x0
  OPC_Decode, 217, 11, 8,                    //  1201:    decode to J4_cmpltu_f_jumpnv_t using decoder 8
                                             //  1201:   }
                                             //  1201:   } // switch Inst[13]
                                             //  1201:  }
  16, 31,                                    //  1205:  case 0x10: {
  OPC_SwitchField, 13, 1,                    //  1207:   switch Inst[13] {
  0, 12,                                     //  1210:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1212:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1216:    check Inst[0] == 0x0
  OPC_Decode, 134, 11, 9,                    //  1220:    decode to J4_cmpeqi_t_jumpnv_nt using decoder 9
                                             //  1220:   }
  1, 0,                                      //  1224:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1226:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1230:    check Inst[0] == 0x0
  OPC_Decode, 135, 11, 9,                    //  1234:    decode to J4_cmpeqi_t_jumpnv_t using decoder 9
                                             //  1234:   }
                                             //  1234:   } // switch Inst[13]
                                             //  1234:  }
  17, 31,                                    //  1238:  case 0x11: {
  OPC_SwitchField, 13, 1,                    //  1240:   switch Inst[13] {
  0, 12,                                     //  1243:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1245:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1249:    check Inst[0] == 0x0
  OPC_Decode, 128, 11, 9,                    //  1253:    decode to J4_cmpeqi_f_jumpnv_nt using decoder 9
                                             //  1253:   }
  1, 0,                                      //  1257:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1259:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1263:    check Inst[0] == 0x0
  OPC_Decode, 129, 11, 9,                    //  1267:    decode to J4_cmpeqi_f_jumpnv_t using decoder 9
                                             //  1267:   }
                                             //  1267:   } // switch Inst[13]
                                             //  1267:  }
  18, 31,                                    //  1271:  case 0x12: {
  OPC_SwitchField, 13, 1,                    //  1273:   switch Inst[13] {
  0, 12,                                     //  1276:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1278:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1282:    check Inst[0] == 0x0
  OPC_Decode, 170, 11, 9,                    //  1286:    decode to J4_cmpgti_t_jumpnv_nt using decoder 9
                                             //  1286:   }
  1, 0,                                      //  1290:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1292:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1296:    check Inst[0] == 0x0
  OPC_Decode, 171, 11, 9,                    //  1300:    decode to J4_cmpgti_t_jumpnv_t using decoder 9
                                             //  1300:   }
                                             //  1300:   } // switch Inst[13]
                                             //  1300:  }
  19, 31,                                    //  1304:  case 0x13: {
  OPC_SwitchField, 13, 1,                    //  1306:   switch Inst[13] {
  0, 12,                                     //  1309:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1311:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1315:    check Inst[0] == 0x0
  OPC_Decode, 164, 11, 9,                    //  1319:    decode to J4_cmpgti_f_jumpnv_nt using decoder 9
                                             //  1319:   }
  1, 0,                                      //  1323:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1325:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1329:    check Inst[0] == 0x0
  OPC_Decode, 165, 11, 9,                    //  1333:    decode to J4_cmpgti_f_jumpnv_t using decoder 9
                                             //  1333:   }
                                             //  1333:   } // switch Inst[13]
                                             //  1333:  }
  20, 31,                                    //  1337:  case 0x14: {
  OPC_SwitchField, 13, 1,                    //  1339:   switch Inst[13] {
  0, 12,                                     //  1342:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1344:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1348:    check Inst[0] == 0x0
  OPC_Decode, 206, 11, 9,                    //  1352:    decode to J4_cmpgtui_t_jumpnv_nt using decoder 9
                                             //  1352:   }
  1, 0,                                      //  1356:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1358:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1362:    check Inst[0] == 0x0
  OPC_Decode, 207, 11, 9,                    //  1366:    decode to J4_cmpgtui_t_jumpnv_t using decoder 9
                                             //  1366:   }
                                             //  1366:   } // switch Inst[13]
                                             //  1366:  }
  21, 31,                                    //  1370:  case 0x15: {
  OPC_SwitchField, 13, 1,                    //  1372:   switch Inst[13] {
  0, 12,                                     //  1375:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1377:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1381:    check Inst[0] == 0x0
  OPC_Decode, 200, 11, 9,                    //  1385:    decode to J4_cmpgtui_f_jumpnv_nt using decoder 9
                                             //  1385:   }
  1, 0,                                      //  1389:   case 0x1: {
  OPC_CheckField, 19, 1, 0,                  //  1391:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1395:    check Inst[0] == 0x0
  OPC_Decode, 201, 11, 9,                    //  1399:    decode to J4_cmpgtui_f_jumpnv_t using decoder 9
                                             //  1399:   }
                                             //  1399:   } // switch Inst[13]
                                             //  1399:  }
  22, 31,                                    //  1403:  case 0x16: {
  OPC_SwitchField, 8, 6,                     //  1405:   switch Inst[13:8] {
  0, 12,                                     //  1408:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1410:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1414:    check Inst[0] == 0x0
  OPC_Decode, 229, 11, 10,                   //  1418:    decode to J4_tstbit0_t_jumpnv_nt using decoder 10
                                             //  1418:   }
  32, 0,                                     //  1422:   case 0x20: {
  OPC_CheckField, 19, 1, 0,                  //  1424:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1428:    check Inst[0] == 0x0
  OPC_Decode, 230, 11, 10,                   //  1432:    decode to J4_tstbit0_t_jumpnv_t using decoder 10
                                             //  1432:   }
                                             //  1432:   } // switch Inst[13:8]
                                             //  1432:  }
  23, 31,                                    //  1436:  case 0x17: {
  OPC_SwitchField, 8, 6,                     //  1438:   switch Inst[13:8] {
  0, 12,                                     //  1441:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1443:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1447:    check Inst[0] == 0x0
  OPC_Decode, 223, 11, 10,                   //  1451:    decode to J4_tstbit0_f_jumpnv_nt using decoder 10
                                             //  1451:   }
  32, 0,                                     //  1455:   case 0x20: {
  OPC_CheckField, 19, 1, 0,                  //  1457:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1461:    check Inst[0] == 0x0
  OPC_Decode, 224, 11, 10,                   //  1465:    decode to J4_tstbit0_f_jumpnv_t using decoder 10
                                             //  1465:   }
                                             //  1465:   } // switch Inst[13:8]
                                             //  1465:  }
  24, 31,                                    //  1469:  case 0x18: {
  OPC_SwitchField, 8, 6,                     //  1471:   switch Inst[13:8] {
  0, 12,                                     //  1474:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1476:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1480:    check Inst[0] == 0x0
  OPC_Decode, 146, 11, 11,                   //  1484:    decode to J4_cmpeqn1_t_jumpnv_nt using decoder 11
                                             //  1484:   }
  32, 0,                                     //  1488:   case 0x20: {
  OPC_CheckField, 19, 1, 0,                  //  1490:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1494:    check Inst[0] == 0x0
  OPC_Decode, 147, 11, 11,                   //  1498:    decode to J4_cmpeqn1_t_jumpnv_t using decoder 11
                                             //  1498:   }
                                             //  1498:   } // switch Inst[13:8]
                                             //  1498:  }
  25, 31,                                    //  1502:  case 0x19: {
  OPC_SwitchField, 8, 6,                     //  1504:   switch Inst[13:8] {
  0, 12,                                     //  1507:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1509:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1513:    check Inst[0] == 0x0
  OPC_Decode, 140, 11, 11,                   //  1517:    decode to J4_cmpeqn1_f_jumpnv_nt using decoder 11
                                             //  1517:   }
  32, 0,                                     //  1521:   case 0x20: {
  OPC_CheckField, 19, 1, 0,                  //  1523:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1527:    check Inst[0] == 0x0
  OPC_Decode, 141, 11, 11,                   //  1531:    decode to J4_cmpeqn1_f_jumpnv_t using decoder 11
                                             //  1531:   }
                                             //  1531:   } // switch Inst[13:8]
                                             //  1531:  }
  26, 31,                                    //  1535:  case 0x1a: {
  OPC_SwitchField, 8, 6,                     //  1537:   switch Inst[13:8] {
  0, 12,                                     //  1540:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1542:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1546:    check Inst[0] == 0x0
  OPC_Decode, 182, 11, 11,                   //  1550:    decode to J4_cmpgtn1_t_jumpnv_nt using decoder 11
                                             //  1550:   }
  32, 0,                                     //  1554:   case 0x20: {
  OPC_CheckField, 19, 1, 0,                  //  1556:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1560:    check Inst[0] == 0x0
  OPC_Decode, 183, 11, 11,                   //  1564:    decode to J4_cmpgtn1_t_jumpnv_t using decoder 11
                                             //  1564:   }
                                             //  1564:   } // switch Inst[13:8]
                                             //  1564:  }
  27, 0,                                     //  1568:  case 0x1b: {
  OPC_SwitchField, 8, 6,                     //  1570:   switch Inst[13:8] {
  0, 12,                                     //  1573:   case 0x0: {
  OPC_CheckField, 19, 1, 0,                  //  1575:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1579:    check Inst[0] == 0x0
  OPC_Decode, 176, 11, 11,                   //  1583:    decode to J4_cmpgtn1_f_jumpnv_nt using decoder 11
                                             //  1583:   }
  32, 0,                                     //  1587:   case 0x20: {
  OPC_CheckField, 19, 1, 0,                  //  1589:    check Inst[19] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  1593:    check Inst[0] == 0x0
  OPC_Decode, 177, 11, 11,                   //  1597:    decode to J4_cmpgtn1_f_jumpnv_t using decoder 11
                                             //  1597:   }
                                             //  1597:   } // switch Inst[13:8]
                                             //  1597:  }
                                             //  1597:  } // switch Inst[27:22]
                                             //  1597: }
  3, 212, 6,                                 //  1601: case 0x3: {
  OPC_SwitchField, 21, 7,                    //  1604:  switch Inst[27:21] {
  0, 4,                                      //  1607:  case 0x0: {
  OPC_Decode, 167, 13, 12,                   //  1609:   decode to L4_ploadrbt_rr using decoder 12
                                             //  1609:  }
  1, 4,                                      //  1613:  case 0x1: {
  OPC_Decode, 199, 13, 12,                   //  1615:   decode to L4_ploadrubt_rr using decoder 12
                                             //  1615:  }
  2, 4,                                      //  1619:  case 0x2: {
  OPC_Decode, 183, 13, 12,                   //  1621:   decode to L4_ploadrht_rr using decoder 12
                                             //  1621:  }
  3, 4,                                      //  1625:  case 0x3: {
  OPC_Decode, 207, 13, 12,                   //  1627:   decode to L4_ploadruht_rr using decoder 12
                                             //  1627:  }
  4, 4,                                      //  1631:  case 0x4: {
  OPC_Decode, 191, 13, 12,                   //  1633:   decode to L4_ploadrit_rr using decoder 12
                                             //  1633:  }
  6, 4,                                      //  1637:  case 0x6: {
  OPC_Decode, 175, 13, 13,                   //  1639:   decode to L4_ploadrdt_rr using decoder 13
                                             //  1639:  }
  8, 4,                                      //  1643:  case 0x8: {
  OPC_Decode, 163, 13, 12,                   //  1645:   decode to L4_ploadrbf_rr using decoder 12
                                             //  1645:  }
  9, 4,                                      //  1649:  case 0x9: {
  OPC_Decode, 195, 13, 12,                   //  1651:   decode to L4_ploadrubf_rr using decoder 12
                                             //  1651:  }
  10, 4,                                     //  1655:  case 0xa: {
  OPC_Decode, 179, 13, 12,                   //  1657:   decode to L4_ploadrhf_rr using decoder 12
                                             //  1657:  }
  11, 4,                                     //  1661:  case 0xb: {
  OPC_Decode, 203, 13, 12,                   //  1663:   decode to L4_ploadruhf_rr using decoder 12
                                             //  1663:  }
  12, 4,                                     //  1667:  case 0xc: {
  OPC_Decode, 187, 13, 12,                   //  1669:   decode to L4_ploadrif_rr using decoder 12
                                             //  1669:  }
  14, 4,                                     //  1673:  case 0xe: {
  OPC_Decode, 171, 13, 13,                   //  1675:   decode to L4_ploadrdf_rr using decoder 13
                                             //  1675:  }
  16, 4,                                     //  1679:  case 0x10: {
  OPC_Decode, 169, 13, 12,                   //  1681:   decode to L4_ploadrbtnew_rr using decoder 12
                                             //  1681:  }
  17, 4,                                     //  1685:  case 0x11: {
  OPC_Decode, 201, 13, 12,                   //  1687:   decode to L4_ploadrubtnew_rr using decoder 12
                                             //  1687:  }
  18, 4,                                     //  1691:  case 0x12: {
  OPC_Decode, 185, 13, 12,                   //  1693:   decode to L4_ploadrhtnew_rr using decoder 12
                                             //  1693:  }
  19, 4,                                     //  1697:  case 0x13: {
  OPC_Decode, 209, 13, 12,                   //  1699:   decode to L4_ploadruhtnew_rr using decoder 12
                                             //  1699:  }
  20, 4,                                     //  1703:  case 0x14: {
  OPC_Decode, 193, 13, 12,                   //  1705:   decode to L4_ploadritnew_rr using decoder 12
                                             //  1705:  }
  22, 4,                                     //  1709:  case 0x16: {
  OPC_Decode, 177, 13, 13,                   //  1711:   decode to L4_ploadrdtnew_rr using decoder 13
                                             //  1711:  }
  24, 4,                                     //  1715:  case 0x18: {
  OPC_Decode, 165, 13, 12,                   //  1717:   decode to L4_ploadrbfnew_rr using decoder 12
                                             //  1717:  }
  25, 4,                                     //  1721:  case 0x19: {
  OPC_Decode, 197, 13, 12,                   //  1723:   decode to L4_ploadrubfnew_rr using decoder 12
                                             //  1723:  }
  26, 4,                                     //  1727:  case 0x1a: {
  OPC_Decode, 181, 13, 12,                   //  1729:   decode to L4_ploadrhfnew_rr using decoder 12
                                             //  1729:  }
  27, 4,                                     //  1733:  case 0x1b: {
  OPC_Decode, 205, 13, 12,                   //  1735:   decode to L4_ploadruhfnew_rr using decoder 12
                                             //  1735:  }
  28, 4,                                     //  1739:  case 0x1c: {
  OPC_Decode, 189, 13, 12,                   //  1741:   decode to L4_ploadrifnew_rr using decoder 12
                                             //  1741:  }
  30, 4,                                     //  1745:  case 0x1e: {
  OPC_Decode, 173, 13, 13,                   //  1747:   decode to L4_ploadrdfnew_rr using decoder 13
                                             //  1747:  }
  32, 4,                                     //  1751:  case 0x20: {
  OPC_Decode, 142, 19, 14,                   //  1753:   decode to S4_pstorerbt_rr using decoder 14
                                             //  1753:  }
  34, 4,                                     //  1757:  case 0x22: {
  OPC_Decode, 182, 19, 14,                   //  1759:   decode to S4_pstorerht_rr using decoder 14
                                             //  1759:  }
  35, 4,                                     //  1763:  case 0x23: {
  OPC_Decode, 162, 19, 14,                   //  1765:   decode to S4_pstorerft_rr using decoder 14
                                             //  1765:  }
  36, 4,                                     //  1769:  case 0x24: {
  OPC_Decode, 202, 19, 14,                   //  1771:   decode to S4_pstorerit_rr using decoder 14
                                             //  1771:  }
  37, 21,                                    //  1775:  case 0x25: {
  OPC_SwitchField, 3, 2,                     //  1777:   switch Inst[4:3] {
  0, 4,                                      //  1780:   case 0x0: {
  OPC_Decode, 137, 19, 15,                   //  1782:    decode to S4_pstorerbnewt_rr using decoder 15
                                             //  1782:   }
  1, 4,                                      //  1786:   case 0x1: {
  OPC_Decode, 177, 19, 15,                   //  1788:    decode to S4_pstorerhnewt_rr using decoder 15
                                             //  1788:   }
  2, 0,                                      //  1792:   case 0x2: {
  OPC_Decode, 197, 19, 15,                   //  1794:    decode to S4_pstorerinewt_rr using decoder 15
                                             //  1794:   }
                                             //  1794:   } // switch Inst[4:3]
                                             //  1794:  }
  38, 4,                                     //  1798:  case 0x26: {
  OPC_Decode, 152, 19, 16,                   //  1800:   decode to S4_pstorerdt_rr using decoder 16
                                             //  1800:  }
  40, 4,                                     //  1804:  case 0x28: {
  OPC_Decode, 255, 18, 14,                   //  1806:   decode to S4_pstorerbf_rr using decoder 14
                                             //  1806:  }
  42, 4,                                     //  1810:  case 0x2a: {
  OPC_Decode, 167, 19, 14,                   //  1812:   decode to S4_pstorerhf_rr using decoder 14
                                             //  1812:  }
  43, 4,                                     //  1816:  case 0x2b: {
  OPC_Decode, 157, 19, 14,                   //  1818:   decode to S4_pstorerff_rr using decoder 14
                                             //  1818:  }
  44, 4,                                     //  1822:  case 0x2c: {
  OPC_Decode, 187, 19, 14,                   //  1824:   decode to S4_pstorerif_rr using decoder 14
                                             //  1824:  }
  45, 21,                                    //  1828:  case 0x2d: {
  OPC_SwitchField, 3, 2,                     //  1830:   switch Inst[4:3] {
  0, 4,                                      //  1833:   case 0x0: {
  OPC_Decode, 132, 19, 15,                   //  1835:    decode to S4_pstorerbnewf_rr using decoder 15
                                             //  1835:   }
  1, 4,                                      //  1839:   case 0x1: {
  OPC_Decode, 172, 19, 15,                   //  1841:    decode to S4_pstorerhnewf_rr using decoder 15
                                             //  1841:   }
  2, 0,                                      //  1845:   case 0x2: {
  OPC_Decode, 192, 19, 15,                   //  1847:    decode to S4_pstorerinewf_rr using decoder 15
                                             //  1847:   }
                                             //  1847:   } // switch Inst[4:3]
                                             //  1847:  }
  46, 4,                                     //  1851:  case 0x2e: {
  OPC_Decode, 147, 19, 16,                   //  1853:   decode to S4_pstorerdf_rr using decoder 16
                                             //  1853:  }
  48, 4,                                     //  1857:  case 0x30: {
  OPC_Decode, 145, 19, 14,                   //  1859:   decode to S4_pstorerbtnew_rr using decoder 14
                                             //  1859:  }
  50, 4,                                     //  1863:  case 0x32: {
  OPC_Decode, 185, 19, 14,                   //  1865:   decode to S4_pstorerhtnew_rr using decoder 14
                                             //  1865:  }
  51, 4,                                     //  1869:  case 0x33: {
  OPC_Decode, 165, 19, 14,                   //  1871:   decode to S4_pstorerftnew_rr using decoder 14
                                             //  1871:  }
  52, 4,                                     //  1875:  case 0x34: {
  OPC_Decode, 205, 19, 14,                   //  1877:   decode to S4_pstoreritnew_rr using decoder 14
                                             //  1877:  }
  53, 21,                                    //  1881:  case 0x35: {
  OPC_SwitchField, 3, 2,                     //  1883:   switch Inst[4:3] {
  0, 4,                                      //  1886:   case 0x0: {
  OPC_Decode, 140, 19, 15,                   //  1888:    decode to S4_pstorerbnewtnew_rr using decoder 15
                                             //  1888:   }
  1, 4,                                      //  1892:   case 0x1: {
  OPC_Decode, 180, 19, 15,                   //  1894:    decode to S4_pstorerhnewtnew_rr using decoder 15
                                             //  1894:   }
  2, 0,                                      //  1898:   case 0x2: {
  OPC_Decode, 200, 19, 15,                   //  1900:    decode to S4_pstorerinewtnew_rr using decoder 15
                                             //  1900:   }
                                             //  1900:   } // switch Inst[4:3]
                                             //  1900:  }
  54, 4,                                     //  1904:  case 0x36: {
  OPC_Decode, 155, 19, 16,                   //  1906:   decode to S4_pstorerdtnew_rr using decoder 16
                                             //  1906:  }
  56, 4,                                     //  1910:  case 0x38: {
  OPC_Decode, 130, 19, 14,                   //  1912:   decode to S4_pstorerbfnew_rr using decoder 14
                                             //  1912:  }
  58, 4,                                     //  1916:  case 0x3a: {
  OPC_Decode, 170, 19, 14,                   //  1918:   decode to S4_pstorerhfnew_rr using decoder 14
                                             //  1918:  }
  59, 4,                                     //  1922:  case 0x3b: {
  OPC_Decode, 160, 19, 14,                   //  1924:   decode to S4_pstorerffnew_rr using decoder 14
                                             //  1924:  }
  60, 4,                                     //  1928:  case 0x3c: {
  OPC_Decode, 190, 19, 14,                   //  1930:   decode to S4_pstorerifnew_rr using decoder 14
                                             //  1930:  }
  61, 21,                                    //  1934:  case 0x3d: {
  OPC_SwitchField, 3, 2,                     //  1936:   switch Inst[4:3] {
  0, 4,                                      //  1939:   case 0x0: {
  OPC_Decode, 135, 19, 15,                   //  1941:    decode to S4_pstorerbnewfnew_rr using decoder 15
                                             //  1941:   }
  1, 4,                                      //  1945:   case 0x1: {
  OPC_Decode, 175, 19, 15,                   //  1947:    decode to S4_pstorerhnewfnew_rr using decoder 15
                                             //  1947:   }
  2, 0,                                      //  1951:   case 0x2: {
  OPC_Decode, 195, 19, 15,                   //  1953:    decode to S4_pstorerinewfnew_rr using decoder 15
                                             //  1953:   }
                                             //  1953:   } // switch Inst[4:3]
                                             //  1953:  }
  62, 4,                                     //  1957:  case 0x3e: {
  OPC_Decode, 150, 19, 16,                   //  1959:   decode to S4_pstorerdfnew_rr using decoder 16
                                             //  1959:  }
  64, 4,                                     //  1963:  case 0x40: {
  OPC_Decode, 212, 19, 17,                   //  1965:   decode to S4_storeirbt_io using decoder 17
                                             //  1965:  }
  65, 4,                                     //  1969:  case 0x41: {
  OPC_Decode, 217, 19, 18,                   //  1971:   decode to S4_storeirht_io using decoder 18
                                             //  1971:  }
  66, 4,                                     //  1975:  case 0x42: {
  OPC_Decode, 222, 19, 19,                   //  1977:   decode to S4_storeirit_io using decoder 19
                                             //  1977:  }
  68, 4,                                     //  1981:  case 0x44: {
  OPC_Decode, 210, 19, 17,                   //  1983:   decode to S4_storeirbf_io using decoder 17
                                             //  1983:  }
  69, 4,                                     //  1987:  case 0x45: {
  OPC_Decode, 215, 19, 18,                   //  1989:   decode to S4_storeirhf_io using decoder 18
                                             //  1989:  }
  70, 4,                                     //  1993:  case 0x46: {
  OPC_Decode, 220, 19, 19,                   //  1995:   decode to S4_storeirif_io using decoder 19
                                             //  1995:  }
  72, 4,                                     //  1999:  case 0x48: {
  OPC_Decode, 213, 19, 17,                   //  2001:   decode to S4_storeirbtnew_io using decoder 17
                                             //  2001:  }
  73, 4,                                     //  2005:  case 0x49: {
  OPC_Decode, 218, 19, 18,                   //  2007:   decode to S4_storeirhtnew_io using decoder 18
                                             //  2007:  }
  74, 4,                                     //  2011:  case 0x4a: {
  OPC_Decode, 223, 19, 19,                   //  2013:   decode to S4_storeiritnew_io using decoder 19
                                             //  2013:  }
  76, 4,                                     //  2017:  case 0x4c: {
  OPC_Decode, 211, 19, 17,                   //  2019:   decode to S4_storeirbfnew_io using decoder 17
                                             //  2019:  }
  77, 4,                                     //  2023:  case 0x4d: {
  OPC_Decode, 216, 19, 18,                   //  2025:   decode to S4_storeirhfnew_io using decoder 18
                                             //  2025:  }
  78, 4,                                     //  2029:  case 0x4e: {
  OPC_Decode, 221, 19, 19,                   //  2031:   decode to S4_storeirifnew_io using decoder 19
                                             //  2031:  }
  80, 8,                                     //  2035:  case 0x50: {
  OPC_CheckField, 5, 2, 0,                   //  2037:   check Inst[6:5] == 0x0
  OPC_Decode, 141, 13, 20,                   //  2041:   decode to L4_loadrb_rr using decoder 20
                                             //  2041:  }
  81, 8,                                     //  2045:  case 0x51: {
  OPC_CheckField, 5, 2, 0,                   //  2047:   check Inst[6:5] == 0x0
  OPC_Decode, 153, 13, 20,                   //  2051:   decode to L4_loadrub_rr using decoder 20
                                             //  2051:  }
  82, 8,                                     //  2055:  case 0x52: {
  OPC_CheckField, 5, 2, 0,                   //  2057:   check Inst[6:5] == 0x0
  OPC_Decode, 147, 13, 20,                   //  2061:   decode to L4_loadrh_rr using decoder 20
                                             //  2061:  }
  83, 8,                                     //  2065:  case 0x53: {
  OPC_CheckField, 5, 2, 0,                   //  2067:   check Inst[6:5] == 0x0
  OPC_Decode, 156, 13, 20,                   //  2071:   decode to L4_loadruh_rr using decoder 20
                                             //  2071:  }
  84, 8,                                     //  2075:  case 0x54: {
  OPC_CheckField, 5, 2, 0,                   //  2077:   check Inst[6:5] == 0x0
  OPC_Decode, 150, 13, 20,                   //  2081:   decode to L4_loadri_rr using decoder 20
                                             //  2081:  }
  86, 8,                                     //  2085:  case 0x56: {
  OPC_CheckField, 5, 2, 0,                   //  2087:   check Inst[6:5] == 0x0
  OPC_Decode, 144, 13, 21,                   //  2091:   decode to L4_loadrd_rr using decoder 21
                                             //  2091:  }
  88, 8,                                     //  2095:  case 0x58: {
  OPC_CheckField, 5, 2, 0,                   //  2097:   check Inst[6:5] == 0x0
  OPC_Decode, 225, 19, 22,                   //  2101:   decode to S4_storerb_rr using decoder 22
                                             //  2101:  }
  90, 8,                                     //  2105:  case 0x5a: {
  OPC_CheckField, 5, 2, 0,                   //  2107:   check Inst[6:5] == 0x0
  OPC_Decode, 237, 19, 22,                   //  2111:   decode to S4_storerh_rr using decoder 22
                                             //  2111:  }
  91, 8,                                     //  2115:  case 0x5b: {
  OPC_CheckField, 5, 2, 0,                   //  2117:   check Inst[6:5] == 0x0
  OPC_Decode, 234, 19, 22,                   //  2121:   decode to S4_storerf_rr using decoder 22
                                             //  2121:  }
  92, 8,                                     //  2125:  case 0x5c: {
  OPC_CheckField, 5, 2, 0,                   //  2127:   check Inst[6:5] == 0x0
  OPC_Decode, 243, 19, 22,                   //  2131:   decode to S4_storeri_rr using decoder 22
                                             //  2131:  }
  93, 21,                                    //  2135:  case 0x5d: {
  OPC_SwitchField, 3, 4,                     //  2137:   switch Inst[6:3] {
  0, 4,                                      //  2140:   case 0x0: {
  OPC_Decode, 228, 19, 23,                   //  2142:    decode to S4_storerbnew_rr using decoder 23
                                             //  2142:   }
  1, 4,                                      //  2146:   case 0x1: {
  OPC_Decode, 240, 19, 23,                   //  2148:    decode to S4_storerhnew_rr using decoder 23
                                             //  2148:   }
  2, 0,                                      //  2152:   case 0x2: {
  OPC_Decode, 246, 19, 23,                   //  2154:    decode to S4_storerinew_rr using decoder 23
                                             //  2154:   }
                                             //  2154:   } // switch Inst[6:3]
                                             //  2154:  }
  94, 8,                                     //  2158:  case 0x5e: {
  OPC_CheckField, 5, 2, 0,                   //  2160:   check Inst[6:5] == 0x0
  OPC_Decode, 231, 19, 24,                   //  2164:   decode to S4_storerd_rr using decoder 24
                                             //  2164:  }
  96, 4,                                     //  2168:  case 0x60: {
  OPC_Decode, 209, 19, 25,                   //  2170:   decode to S4_storeirb_io using decoder 25
                                             //  2170:  }
  97, 4,                                     //  2174:  case 0x61: {
  OPC_Decode, 214, 19, 26,                   //  2176:   decode to S4_storeirh_io using decoder 26
                                             //  2176:  }
  98, 4,                                     //  2180:  case 0x62: {
  OPC_Decode, 219, 19, 27,                   //  2182:   decode to S4_storeiri_io using decoder 27
                                             //  2182:  }
  112, 43,                                   //  2186:  case 0x70: {
  OPC_SwitchField, 5, 2,                     //  2188:   switch Inst[6:5] {
  0, 8,                                      //  2191:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2193:    check Inst[13] == 0x0
  OPC_Decode, 236, 12, 28,                   //  2197:    decode to L4_add_memopb_io using decoder 28
                                             //  2197:   }
  1, 8,                                      //  2201:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2203:    check Inst[13] == 0x0
  OPC_Decode, 217, 13, 28,                   //  2207:    decode to L4_sub_memopb_io using decoder 28
                                             //  2207:   }
  2, 8,                                      //  2211:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2213:    check Inst[13] == 0x0
  OPC_Decode, 239, 12, 28,                   //  2217:    decode to L4_and_memopb_io using decoder 28
                                             //  2217:   }
  3, 0,                                      //  2221:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2223:    check Inst[13] == 0x0
  OPC_Decode, 159, 13, 28,                   //  2227:    decode to L4_or_memopb_io using decoder 28
                                             //  2227:   }
                                             //  2227:   } // switch Inst[6:5]
                                             //  2227:  }
  113, 43,                                   //  2231:  case 0x71: {
  OPC_SwitchField, 5, 2,                     //  2233:   switch Inst[6:5] {
  0, 8,                                      //  2236:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2238:    check Inst[13] == 0x0
  OPC_Decode, 237, 12, 29,                   //  2242:    decode to L4_add_memoph_io using decoder 29
                                             //  2242:   }
  1, 8,                                      //  2246:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2248:    check Inst[13] == 0x0
  OPC_Decode, 218, 13, 29,                   //  2252:    decode to L4_sub_memoph_io using decoder 29
                                             //  2252:   }
  2, 8,                                      //  2256:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2258:    check Inst[13] == 0x0
  OPC_Decode, 240, 12, 29,                   //  2262:    decode to L4_and_memoph_io using decoder 29
                                             //  2262:   }
  3, 0,                                      //  2266:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2268:    check Inst[13] == 0x0
  OPC_Decode, 160, 13, 29,                   //  2272:    decode to L4_or_memoph_io using decoder 29
                                             //  2272:   }
                                             //  2272:   } // switch Inst[6:5]
                                             //  2272:  }
  114, 43,                                   //  2276:  case 0x72: {
  OPC_SwitchField, 5, 2,                     //  2278:   switch Inst[6:5] {
  0, 8,                                      //  2281:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2283:    check Inst[13] == 0x0
  OPC_Decode, 238, 12, 30,                   //  2287:    decode to L4_add_memopw_io using decoder 30
                                             //  2287:   }
  1, 8,                                      //  2291:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2293:    check Inst[13] == 0x0
  OPC_Decode, 219, 13, 30,                   //  2297:    decode to L4_sub_memopw_io using decoder 30
                                             //  2297:   }
  2, 8,                                      //  2301:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2303:    check Inst[13] == 0x0
  OPC_Decode, 241, 12, 30,                   //  2307:    decode to L4_and_memopw_io using decoder 30
                                             //  2307:   }
  3, 0,                                      //  2311:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2313:    check Inst[13] == 0x0
  OPC_Decode, 161, 13, 30,                   //  2317:    decode to L4_or_memopw_io using decoder 30
                                             //  2317:   }
                                             //  2317:   } // switch Inst[6:5]
                                             //  2317:  }
  120, 43,                                   //  2321:  case 0x78: {
  OPC_SwitchField, 5, 2,                     //  2323:   switch Inst[6:5] {
  0, 8,                                      //  2326:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2328:    check Inst[13] == 0x0
  OPC_Decode, 242, 12, 31,                   //  2332:    decode to L4_iadd_memopb_io using decoder 31
                                             //  2332:   }
  1, 8,                                      //  2336:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2338:    check Inst[13] == 0x0
  OPC_Decode, 251, 12, 31,                   //  2342:    decode to L4_isub_memopb_io using decoder 31
                                             //  2342:   }
  2, 8,                                      //  2346:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2348:    check Inst[13] == 0x0
  OPC_Decode, 245, 12, 31,                   //  2352:    decode to L4_iand_memopb_io using decoder 31
                                             //  2352:   }
  3, 0,                                      //  2356:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2358:    check Inst[13] == 0x0
  OPC_Decode, 248, 12, 31,                   //  2362:    decode to L4_ior_memopb_io using decoder 31
                                             //  2362:   }
                                             //  2362:   } // switch Inst[6:5]
                                             //  2362:  }
  121, 43,                                   //  2366:  case 0x79: {
  OPC_SwitchField, 5, 2,                     //  2368:   switch Inst[6:5] {
  0, 8,                                      //  2371:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2373:    check Inst[13] == 0x0
  OPC_Decode, 243, 12, 32,                   //  2377:    decode to L4_iadd_memoph_io using decoder 32
                                             //  2377:   }
  1, 8,                                      //  2381:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2383:    check Inst[13] == 0x0
  OPC_Decode, 252, 12, 32,                   //  2387:    decode to L4_isub_memoph_io using decoder 32
                                             //  2387:   }
  2, 8,                                      //  2391:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2393:    check Inst[13] == 0x0
  OPC_Decode, 246, 12, 32,                   //  2397:    decode to L4_iand_memoph_io using decoder 32
                                             //  2397:   }
  3, 0,                                      //  2401:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2403:    check Inst[13] == 0x0
  OPC_Decode, 249, 12, 32,                   //  2407:    decode to L4_ior_memoph_io using decoder 32
                                             //  2407:   }
                                             //  2407:   } // switch Inst[6:5]
                                             //  2407:  }
  122, 0,                                    //  2411:  case 0x7a: {
  OPC_SwitchField, 5, 2,                     //  2413:   switch Inst[6:5] {
  0, 8,                                      //  2416:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2418:    check Inst[13] == 0x0
  OPC_Decode, 244, 12, 33,                   //  2422:    decode to L4_iadd_memopw_io using decoder 33
                                             //  2422:   }
  1, 8,                                      //  2426:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2428:    check Inst[13] == 0x0
  OPC_Decode, 253, 12, 33,                   //  2432:    decode to L4_isub_memopw_io using decoder 33
                                             //  2432:   }
  2, 8,                                      //  2436:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2438:    check Inst[13] == 0x0
  OPC_Decode, 247, 12, 33,                   //  2442:    decode to L4_iand_memopw_io using decoder 33
                                             //  2442:   }
  3, 0,                                      //  2446:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2448:    check Inst[13] == 0x0
  OPC_Decode, 250, 12, 33,                   //  2452:    decode to L4_ior_memopw_io using decoder 33
                                             //  2452:   }
                                             //  2452:   } // switch Inst[6:5]
                                             //  2452:  }
                                             //  2452:  } // switch Inst[27:21]
                                             //  2452: }
  4, 153, 6,                                 //  2456: case 0x4: {
  OPC_SwitchField, 21, 4,                    //  2459:  switch Inst[24:21] {
  0, 54,                                     //  2462:  case 0x0: {
  OPC_SwitchField, 27, 1,                    //  2464:   switch Inst[27] {
  0, 43,                                     //  2467:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2469:    switch Inst[26:25] {
  0, 8,                                      //  2472:    case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2474:     check Inst[2] == 0x0
  OPC_Decode, 220, 17, 34,                   //  2478:     decode to S2_pstorerbt_io using decoder 34
                                             //  2478:    }
  1, 8,                                      //  2482:    case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2484:     check Inst[2] == 0x0
  OPC_Decode, 144, 19, 34,                   //  2488:     decode to S4_pstorerbtnew_io using decoder 34
                                             //  2488:    }
  2, 8,                                      //  2492:    case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2494:     check Inst[2] == 0x0
  OPC_Decode, 211, 17, 34,                   //  2498:     decode to S2_pstorerbf_io using decoder 34
                                             //  2498:    }
  3, 0,                                      //  2502:    case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2504:     check Inst[2] == 0x0
  OPC_Decode, 129, 19, 34,                   //  2508:     decode to S4_pstorerbfnew_io using decoder 34
                                             //  2508:    }
                                             //  2508:    } // switch Inst[26:25]
                                             //  2508:   }
  1, 0,                                      //  2512:   case 0x1: {
  OPC_Decode, 143, 18, 35,                   //  2514:    decode to S2_storerbgp using decoder 35
                                             //  2514:   }
                                             //  2514:   } // switch Inst[27]
                                             //  2514:  }
  2, 54,                                     //  2518:  case 0x2: {
  OPC_SwitchField, 27, 1,                    //  2520:   switch Inst[27] {
  0, 43,                                     //  2523:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2525:    switch Inst[26:25] {
  0, 8,                                      //  2528:    case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2530:     check Inst[2] == 0x0
  OPC_Decode, 244, 17, 36,                   //  2534:     decode to S2_pstorerht_io using decoder 36
                                             //  2534:    }
  1, 8,                                      //  2538:    case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2540:     check Inst[2] == 0x0
  OPC_Decode, 184, 19, 36,                   //  2544:     decode to S4_pstorerhtnew_io using decoder 36
                                             //  2544:    }
  2, 8,                                      //  2548:    case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2550:     check Inst[2] == 0x0
  OPC_Decode, 235, 17, 36,                   //  2554:     decode to S2_pstorerhf_io using decoder 36
                                             //  2554:    }
  3, 0,                                      //  2558:    case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2560:     check Inst[2] == 0x0
  OPC_Decode, 169, 19, 36,                   //  2564:     decode to S4_pstorerhfnew_io using decoder 36
                                             //  2564:    }
                                             //  2564:    } // switch Inst[26:25]
                                             //  2564:   }
  1, 0,                                      //  2568:   case 0x1: {
  OPC_Decode, 171, 18, 37,                   //  2570:    decode to S2_storerhgp using decoder 37
                                             //  2570:   }
                                             //  2570:   } // switch Inst[27]
                                             //  2570:  }
  3, 54,                                     //  2574:  case 0x3: {
  OPC_SwitchField, 27, 1,                    //  2576:   switch Inst[27] {
  0, 43,                                     //  2579:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2581:    switch Inst[26:25] {
  0, 8,                                      //  2584:    case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2586:     check Inst[2] == 0x0
  OPC_Decode, 232, 17, 36,                   //  2590:     decode to S2_pstorerft_io using decoder 36
                                             //  2590:    }
  1, 8,                                      //  2594:    case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2596:     check Inst[2] == 0x0
  OPC_Decode, 164, 19, 36,                   //  2600:     decode to S4_pstorerftnew_io using decoder 36
                                             //  2600:    }
  2, 8,                                      //  2604:    case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2606:     check Inst[2] == 0x0
  OPC_Decode, 229, 17, 36,                   //  2610:     decode to S2_pstorerff_io using decoder 36
                                             //  2610:    }
  3, 0,                                      //  2614:    case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2616:     check Inst[2] == 0x0
  OPC_Decode, 159, 19, 36,                   //  2620:     decode to S4_pstorerffnew_io using decoder 36
                                             //  2620:    }
                                             //  2620:    } // switch Inst[26:25]
                                             //  2620:   }
  1, 0,                                      //  2624:   case 0x1: {
  OPC_Decode, 164, 18, 37,                   //  2626:    decode to S2_storerfgp using decoder 37
                                             //  2626:   }
                                             //  2626:   } // switch Inst[27]
                                             //  2626:  }
  4, 54,                                     //  2630:  case 0x4: {
  OPC_SwitchField, 27, 1,                    //  2632:   switch Inst[27] {
  0, 43,                                     //  2635:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2637:    switch Inst[26:25] {
  0, 8,                                      //  2640:    case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2642:     check Inst[2] == 0x0
  OPC_Decode, 128, 18, 38,                   //  2646:     decode to S2_pstorerit_io using decoder 38
                                             //  2646:    }
  1, 8,                                      //  2650:    case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2652:     check Inst[2] == 0x0
  OPC_Decode, 204, 19, 38,                   //  2656:     decode to S4_pstoreritnew_io using decoder 38
                                             //  2656:    }
  2, 8,                                      //  2660:    case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2662:     check Inst[2] == 0x0
  OPC_Decode, 247, 17, 38,                   //  2666:     decode to S2_pstorerif_io using decoder 38
                                             //  2666:    }
  3, 0,                                      //  2670:    case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2672:     check Inst[2] == 0x0
  OPC_Decode, 189, 19, 38,                   //  2676:     decode to S4_pstorerifnew_io using decoder 38
                                             //  2676:    }
                                             //  2676:    } // switch Inst[26:25]
                                             //  2676:   }
  1, 0,                                      //  2680:   case 0x1: {
  OPC_Decode, 185, 18, 39,                   //  2682:    decode to S2_storerigp using decoder 39
                                             //  2682:   }
                                             //  2682:   } // switch Inst[27]
                                             //  2682:  }
  5, 171, 1,                                 //  2686:  case 0x5: {
  OPC_SwitchField, 11, 2,                    //  2689:   switch Inst[12:11] {
  0, 54,                                     //  2692:   case 0x0: {
  OPC_SwitchField, 27, 1,                    //  2694:    switch Inst[27] {
  0, 43,                                     //  2697:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2699:     switch Inst[26:25] {
  0, 8,                                      //  2702:     case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2704:      check Inst[2] == 0x0
  OPC_Decode, 217, 17, 40,                   //  2708:      decode to S2_pstorerbnewt_io using decoder 40
                                             //  2708:     }
  1, 8,                                      //  2712:     case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2714:      check Inst[2] == 0x0
  OPC_Decode, 139, 19, 40,                   //  2718:      decode to S4_pstorerbnewtnew_io using decoder 40
                                             //  2718:     }
  2, 8,                                      //  2722:     case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2724:      check Inst[2] == 0x0
  OPC_Decode, 214, 17, 40,                   //  2728:      decode to S2_pstorerbnewf_io using decoder 40
                                             //  2728:     }
  3, 0,                                      //  2732:     case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2734:      check Inst[2] == 0x0
  OPC_Decode, 134, 19, 40,                   //  2738:      decode to S4_pstorerbnewfnew_io using decoder 40
                                             //  2738:     }
                                             //  2738:     } // switch Inst[26:25]
                                             //  2738:    }
  1, 0,                                      //  2742:    case 0x1: {
  OPC_Decode, 150, 18, 41,                   //  2744:     decode to S2_storerbnewgp using decoder 41
                                             //  2744:    }
                                             //  2744:    } // switch Inst[27]
                                             //  2744:   }
  1, 54,                                     //  2748:   case 0x1: {
  OPC_SwitchField, 27, 1,                    //  2750:    switch Inst[27] {
  0, 43,                                     //  2753:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2755:     switch Inst[26:25] {
  0, 8,                                      //  2758:     case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2760:      check Inst[2] == 0x0
  OPC_Decode, 241, 17, 42,                   //  2764:      decode to S2_pstorerhnewt_io using decoder 42
                                             //  2764:     }
  1, 8,                                      //  2768:     case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2770:      check Inst[2] == 0x0
  OPC_Decode, 179, 19, 42,                   //  2774:      decode to S4_pstorerhnewtnew_io using decoder 42
                                             //  2774:     }
  2, 8,                                      //  2778:     case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2780:      check Inst[2] == 0x0
  OPC_Decode, 238, 17, 42,                   //  2784:      decode to S2_pstorerhnewf_io using decoder 42
                                             //  2784:     }
  3, 0,                                      //  2788:     case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2790:      check Inst[2] == 0x0
  OPC_Decode, 174, 19, 42,                   //  2794:      decode to S4_pstorerhnewfnew_io using decoder 42
                                             //  2794:     }
                                             //  2794:     } // switch Inst[26:25]
                                             //  2794:    }
  1, 0,                                      //  2798:    case 0x1: {
  OPC_Decode, 178, 18, 43,                   //  2800:     decode to S2_storerhnewgp using decoder 43
                                             //  2800:    }
                                             //  2800:    } // switch Inst[27]
                                             //  2800:   }
  2, 0,                                      //  2804:   case 0x2: {
  OPC_SwitchField, 27, 1,                    //  2806:    switch Inst[27] {
  0, 43,                                     //  2809:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2811:     switch Inst[26:25] {
  0, 8,                                      //  2814:     case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2816:      check Inst[2] == 0x0
  OPC_Decode, 253, 17, 44,                   //  2820:      decode to S2_pstorerinewt_io using decoder 44
                                             //  2820:     }
  1, 8,                                      //  2824:     case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2826:      check Inst[2] == 0x0
  OPC_Decode, 199, 19, 44,                   //  2830:      decode to S4_pstorerinewtnew_io using decoder 44
                                             //  2830:     }
  2, 8,                                      //  2834:     case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2836:      check Inst[2] == 0x0
  OPC_Decode, 250, 17, 44,                   //  2840:      decode to S2_pstorerinewf_io using decoder 44
                                             //  2840:     }
  3, 0,                                      //  2844:     case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2846:      check Inst[2] == 0x0
  OPC_Decode, 194, 19, 44,                   //  2850:      decode to S4_pstorerinewfnew_io using decoder 44
                                             //  2850:     }
                                             //  2850:     } // switch Inst[26:25]
                                             //  2850:    }
  1, 0,                                      //  2854:    case 0x1: {
  OPC_Decode, 192, 18, 45,                   //  2856:     decode to S2_storerinewgp using decoder 45
                                             //  2856:    }
                                             //  2856:    } // switch Inst[27]
                                             //  2856:   }
                                             //  2856:   } // switch Inst[12:11]
                                             //  2856:  }
  6, 54,                                     //  2860:  case 0x6: {
  OPC_SwitchField, 27, 1,                    //  2862:   switch Inst[27] {
  0, 43,                                     //  2865:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2867:    switch Inst[26:25] {
  0, 8,                                      //  2870:    case 0x0: {
  OPC_CheckField, 2, 1, 0,                   //  2872:     check Inst[2] == 0x0
  OPC_Decode, 226, 17, 46,                   //  2876:     decode to S2_pstorerdt_io using decoder 46
                                             //  2876:    }
  1, 8,                                      //  2880:    case 0x1: {
  OPC_CheckField, 2, 1, 0,                   //  2882:     check Inst[2] == 0x0
  OPC_Decode, 154, 19, 46,                   //  2886:     decode to S4_pstorerdtnew_io using decoder 46
                                             //  2886:    }
  2, 8,                                      //  2890:    case 0x2: {
  OPC_CheckField, 2, 1, 0,                   //  2892:     check Inst[2] == 0x0
  OPC_Decode, 223, 17, 46,                   //  2896:     decode to S2_pstorerdf_io using decoder 46
                                             //  2896:    }
  3, 0,                                      //  2900:    case 0x3: {
  OPC_CheckField, 2, 1, 0,                   //  2902:     check Inst[2] == 0x0
  OPC_Decode, 149, 19, 46,                   //  2906:     decode to S4_pstorerdfnew_io using decoder 46
                                             //  2906:    }
                                             //  2906:    } // switch Inst[26:25]
                                             //  2906:   }
  1, 0,                                      //  2910:   case 0x1: {
  OPC_Decode, 157, 18, 47,                   //  2912:    decode to S2_storerdgp using decoder 47
                                             //  2912:   }
                                             //  2912:   } // switch Inst[27]
                                             //  2912:  }
  8, 54,                                     //  2916:  case 0x8: {
  OPC_SwitchField, 27, 1,                    //  2918:   switch Inst[27] {
  0, 43,                                     //  2921:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2923:    switch Inst[26:25] {
  0, 8,                                      //  2926:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2928:     check Inst[13] == 0x0
  OPC_Decode, 192, 12, 48,                   //  2932:     decode to L2_ploadrbt_io using decoder 48
                                             //  2932:    }
  1, 8,                                      //  2936:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2938:     check Inst[13] == 0x0
  OPC_Decode, 194, 12, 48,                   //  2942:     decode to L2_ploadrbtnew_io using decoder 48
                                             //  2942:    }
  2, 8,                                      //  2946:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  2948:     check Inst[13] == 0x0
  OPC_Decode, 188, 12, 48,                   //  2952:     decode to L2_ploadrbf_io using decoder 48
                                             //  2952:    }
  3, 0,                                      //  2956:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  2958:     check Inst[13] == 0x0
  OPC_Decode, 190, 12, 48,                   //  2962:     decode to L2_ploadrbfnew_io using decoder 48
                                             //  2962:    }
                                             //  2962:    } // switch Inst[26:25]
                                             //  2962:   }
  1, 0,                                      //  2966:   case 0x1: {
  OPC_Decode, 150, 12, 49,                   //  2968:    decode to L2_loadrbgp using decoder 49
                                             //  2968:   }
                                             //  2968:   } // switch Inst[27]
                                             //  2968:  }
  9, 54,                                     //  2972:  case 0x9: {
  OPC_SwitchField, 27, 1,                    //  2974:   switch Inst[27] {
  0, 43,                                     //  2977:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  2979:    switch Inst[26:25] {
  0, 8,                                      //  2982:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  2984:     check Inst[13] == 0x0
  OPC_Decode, 224, 12, 48,                   //  2988:     decode to L2_ploadrubt_io using decoder 48
                                             //  2988:    }
  1, 8,                                      //  2992:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  2994:     check Inst[13] == 0x0
  OPC_Decode, 226, 12, 48,                   //  2998:     decode to L2_ploadrubtnew_io using decoder 48
                                             //  2998:    }
  2, 8,                                      //  3002:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  3004:     check Inst[13] == 0x0
  OPC_Decode, 220, 12, 48,                   //  3008:     decode to L2_ploadrubf_io using decoder 48
                                             //  3008:    }
  3, 0,                                      //  3012:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  3014:     check Inst[13] == 0x0
  OPC_Decode, 222, 12, 48,                   //  3018:     decode to L2_ploadrubfnew_io using decoder 48
                                             //  3018:    }
                                             //  3018:    } // switch Inst[26:25]
                                             //  3018:   }
  1, 0,                                      //  3022:   case 0x1: {
  OPC_Decode, 178, 12, 49,                   //  3024:    decode to L2_loadrubgp using decoder 49
                                             //  3024:   }
                                             //  3024:   } // switch Inst[27]
                                             //  3024:  }
  10, 54,                                    //  3028:  case 0xa: {
  OPC_SwitchField, 27, 1,                    //  3030:   switch Inst[27] {
  0, 43,                                     //  3033:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  3035:    switch Inst[26:25] {
  0, 8,                                      //  3038:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  3040:     check Inst[13] == 0x0
  OPC_Decode, 208, 12, 50,                   //  3044:     decode to L2_ploadrht_io using decoder 50
                                             //  3044:    }
  1, 8,                                      //  3048:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  3050:     check Inst[13] == 0x0
  OPC_Decode, 210, 12, 50,                   //  3054:     decode to L2_ploadrhtnew_io using decoder 50
                                             //  3054:    }
  2, 8,                                      //  3058:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  3060:     check Inst[13] == 0x0
  OPC_Decode, 204, 12, 50,                   //  3064:     decode to L2_ploadrhf_io using decoder 50
                                             //  3064:    }
  3, 0,                                      //  3068:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  3070:     check Inst[13] == 0x0
  OPC_Decode, 206, 12, 50,                   //  3074:     decode to L2_ploadrhfnew_io using decoder 50
                                             //  3074:    }
                                             //  3074:    } // switch Inst[26:25]
                                             //  3074:   }
  1, 0,                                      //  3078:   case 0x1: {
  OPC_Decode, 164, 12, 51,                   //  3080:    decode to L2_loadrhgp using decoder 51
                                             //  3080:   }
                                             //  3080:   } // switch Inst[27]
                                             //  3080:  }
  11, 54,                                    //  3084:  case 0xb: {
  OPC_SwitchField, 27, 1,                    //  3086:   switch Inst[27] {
  0, 43,                                     //  3089:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  3091:    switch Inst[26:25] {
  0, 8,                                      //  3094:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  3096:     check Inst[13] == 0x0
  OPC_Decode, 232, 12, 50,                   //  3100:     decode to L2_ploadruht_io using decoder 50
                                             //  3100:    }
  1, 8,                                      //  3104:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  3106:     check Inst[13] == 0x0
  OPC_Decode, 234, 12, 50,                   //  3110:     decode to L2_ploadruhtnew_io using decoder 50
                                             //  3110:    }
  2, 8,                                      //  3114:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  3116:     check Inst[13] == 0x0
  OPC_Decode, 228, 12, 50,                   //  3120:     decode to L2_ploadruhf_io using decoder 50
                                             //  3120:    }
  3, 0,                                      //  3124:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  3126:     check Inst[13] == 0x0
  OPC_Decode, 230, 12, 50,                   //  3130:     decode to L2_ploadruhfnew_io using decoder 50
                                             //  3130:    }
                                             //  3130:    } // switch Inst[26:25]
                                             //  3130:   }
  1, 0,                                      //  3134:   case 0x1: {
  OPC_Decode, 185, 12, 51,                   //  3136:    decode to L2_loadruhgp using decoder 51
                                             //  3136:   }
                                             //  3136:   } // switch Inst[27]
                                             //  3136:  }
  12, 54,                                    //  3140:  case 0xc: {
  OPC_SwitchField, 27, 1,                    //  3142:   switch Inst[27] {
  0, 43,                                     //  3145:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  3147:    switch Inst[26:25] {
  0, 8,                                      //  3150:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  3152:     check Inst[13] == 0x0
  OPC_Decode, 216, 12, 52,                   //  3156:     decode to L2_ploadrit_io using decoder 52
                                             //  3156:    }
  1, 8,                                      //  3160:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  3162:     check Inst[13] == 0x0
  OPC_Decode, 218, 12, 52,                   //  3166:     decode to L2_ploadritnew_io using decoder 52
                                             //  3166:    }
  2, 8,                                      //  3170:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  3172:     check Inst[13] == 0x0
  OPC_Decode, 212, 12, 52,                   //  3176:     decode to L2_ploadrif_io using decoder 52
                                             //  3176:    }
  3, 0,                                      //  3180:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  3182:     check Inst[13] == 0x0
  OPC_Decode, 214, 12, 52,                   //  3186:     decode to L2_ploadrifnew_io using decoder 52
                                             //  3186:    }
                                             //  3186:    } // switch Inst[26:25]
                                             //  3186:   }
  1, 0,                                      //  3190:   case 0x1: {
  OPC_Decode, 171, 12, 53,                   //  3192:    decode to L2_loadrigp using decoder 53
                                             //  3192:   }
                                             //  3192:   } // switch Inst[27]
                                             //  3192:  }
  14, 0,                                     //  3196:  case 0xe: {
  OPC_SwitchField, 27, 1,                    //  3198:   switch Inst[27] {
  0, 43,                                     //  3201:   case 0x0: {
  OPC_SwitchField, 25, 2,                    //  3203:    switch Inst[26:25] {
  0, 8,                                      //  3206:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  3208:     check Inst[13] == 0x0
  OPC_Decode, 200, 12, 54,                   //  3212:     decode to L2_ploadrdt_io using decoder 54
                                             //  3212:    }
  1, 8,                                      //  3216:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  3218:     check Inst[13] == 0x0
  OPC_Decode, 202, 12, 54,                   //  3222:     decode to L2_ploadrdtnew_io using decoder 54
                                             //  3222:    }
  2, 8,                                      //  3226:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  3228:     check Inst[13] == 0x0
  OPC_Decode, 196, 12, 54,                   //  3232:     decode to L2_ploadrdf_io using decoder 54
                                             //  3232:    }
  3, 0,                                      //  3236:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  3238:     check Inst[13] == 0x0
  OPC_Decode, 198, 12, 54,                   //  3242:     decode to L2_ploadrdfnew_io using decoder 54
                                             //  3242:    }
                                             //  3242:    } // switch Inst[26:25]
                                             //  3242:   }
  1, 0,                                      //  3246:   case 0x1: {
  OPC_Decode, 157, 12, 55,                   //  3248:    decode to L2_loadrdgp using decoder 55
                                             //  3248:   }
                                             //  3248:   } // switch Inst[27]
                                             //  3248:  }
                                             //  3248:  } // switch Inst[24:21]
                                             //  3248: }
  5, 187, 4,                                 //  3252: case 0x5: {
  OPC_SwitchField, 25, 3,                    //  3255:  switch Inst[27:25] {
  0, 53,                                     //  3258:  case 0x0: {
  OPC_SwitchField, 21, 4,                    //  3260:   switch Inst[24:21] {
  5, 8,                                      //  3263:   case 0x5: {
  OPC_CheckField, 0, 14, 0,                  //  3265:    check Inst[13:0] == 0x0
  OPC_Decode, 193, 10, 56,                   //  3269:    decode to J2_callr using decoder 56
                                             //  3269:   }
  6, 10,                                     //  3273:   case 0x6: {
  OPC_CheckPredicate, 0,                     //  3275:    check predicate 0
  OPC_CheckField, 0, 14, 0,                  //  3277:    check Inst[13:0] == 0x0
  OPC_Decode, 195, 10, 56,                   //  3281:    decode to J2_callrh using decoder 56
                                             //  3281:   }
  8, 12,                                     //  3285:   case 0x8: {
  OPC_CheckField, 10, 4, 0,                  //  3287:    check Inst[13:10] == 0x0
  OPC_CheckField, 0, 8, 0,                   //  3291:    check Inst[7:0] == 0x0
  OPC_Decode, 196, 10, 57,                   //  3295:    decode to J2_callrt using decoder 57
                                             //  3295:   }
  9, 0,                                      //  3299:   case 0x9: {
  OPC_CheckField, 10, 4, 0,                  //  3301:    check Inst[13:10] == 0x0
  OPC_CheckField, 0, 8, 0,                   //  3305:    check Inst[7:0] == 0x0
  OPC_Decode, 194, 10, 57,                   //  3309:    decode to J2_callrf using decoder 57
                                             //  3309:   }
                                             //  3309:   } // switch Inst[24:21]
                                             //  3309:  }
  1, 129, 1,                                 //  3313:  case 0x1: {
  OPC_SwitchField, 21, 4,                    //  3316:   switch Inst[24:21] {
  4, 8,                                      //  3319:   case 0x4: {
  OPC_CheckField, 0, 14, 0,                  //  3321:    check Inst[13:0] == 0x0
  OPC_Decode, 203, 10, 56,                   //  3325:    decode to J2_jumpr using decoder 56
                                             //  3325:   }
  5, 8,                                      //  3329:   case 0x5: {
  OPC_CheckField, 0, 14, 0,                  //  3331:    check Inst[13:0] == 0x0
  OPC_Decode, 220, 11, 56,                   //  3335:    decode to J4_hintjumpr using decoder 56
                                             //  3335:   }
  6, 10,                                     //  3339:   case 0x6: {
  OPC_CheckPredicate, 0,                     //  3341:    check predicate 0
  OPC_CheckField, 0, 14, 0,                  //  3343:    check Inst[13:0] == 0x0
  OPC_Decode, 210, 10, 56,                   //  3347:    decode to J2_jumprh using decoder 56
                                             //  3347:   }
  10, 45,                                    //  3351:   case 0xa: {
  OPC_SwitchField, 10, 4,                    //  3353:    switch Inst[13:10] {
  0, 8,                                      //  3356:    case 0x0: {
  OPC_CheckField, 0, 8, 0,                   //  3358:     check Inst[7:0] == 0x0
  OPC_Decode, 215, 10, 57,                   //  3362:     decode to J2_jumprt using decoder 57
                                             //  3362:    }
  2, 8,                                      //  3366:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   //  3368:     check Inst[7:0] == 0x0
  OPC_Decode, 216, 10, 57,                   //  3372:     decode to J2_jumprtnew using decoder 57
                                             //  3372:    }
  4, 10,                                     //  3376:    case 0x4: {
  OPC_CheckPredicate, 1,                     //  3378:     check predicate 1
  OPC_CheckField, 0, 8, 0,                   //  3380:     check Inst[7:0] == 0x0
  OPC_Decode, 218, 10, 57,                   //  3384:     decode to J2_jumprtpt using decoder 57
                                             //  3384:    }
  6, 0,                                      //  3388:    case 0x6: {
  OPC_CheckField, 0, 8, 0,                   //  3390:     check Inst[7:0] == 0x0
  OPC_Decode, 217, 10, 57,                   //  3394:     decode to J2_jumprtnewpt using decoder 57
                                             //  3394:    }
                                             //  3394:    } // switch Inst[13:10]
                                             //  3394:   }
  11, 0,                                     //  3398:   case 0xb: {
  OPC_SwitchField, 10, 4,                    //  3400:    switch Inst[13:10] {
  0, 8,                                      //  3403:    case 0x0: {
  OPC_CheckField, 0, 8, 0,                   //  3405:     check Inst[7:0] == 0x0
  OPC_Decode, 204, 10, 57,                   //  3409:     decode to J2_jumprf using decoder 57
                                             //  3409:    }
  2, 8,                                      //  3413:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   //  3415:     check Inst[7:0] == 0x0
  OPC_Decode, 205, 10, 57,                   //  3419:     decode to J2_jumprfnew using decoder 57
                                             //  3419:    }
  4, 10,                                     //  3423:    case 0x4: {
  OPC_CheckPredicate, 1,                     //  3425:     check predicate 1
  OPC_CheckField, 0, 8, 0,                   //  3427:     check Inst[7:0] == 0x0
  OPC_Decode, 207, 10, 57,                   //  3431:     decode to J2_jumprfpt using decoder 57
                                             //  3431:    }
  6, 0,                                      //  3435:    case 0x6: {
  OPC_CheckField, 0, 8, 0,                   //  3437:     check Inst[7:0] == 0x0
  OPC_Decode, 206, 10, 57,                   //  3441:     decode to J2_jumprfnewpt using decoder 57
                                             //  3441:    }
                                             //  3441:    } // switch Inst[13:10]
                                             //  3441:   }
                                             //  3441:   } // switch Inst[24:21]
                                             //  3441:  }
  2, 126,                                    //  3445:  case 0x2: {
  OPC_SwitchField, 21, 4,                    //  3447:   switch Inst[24:21] {
  0, 20,                                     //  3450:   case 0x0: {
  OPC_CheckField, 16, 5, 0,                  //  3452:    check Inst[20:16] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  3456:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3460:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  3464:    check Inst[1:0] == 0x0
  OPC_Decode, 241, 10, 58,                   //  3468:    decode to J2_trap0 using decoder 58
                                             //  3468:   }
  2, 20,                                     //  3472:   case 0x2: {
  OPC_CheckField, 18, 3, 0,                  //  3474:    check Inst[20:18] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  3478:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3482:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  3486:    check Inst[1:0] == 0x0
  OPC_Decode, 233, 10, 59,                   //  3490:    decode to J2_pause using decoder 59
                                             //  3490:   }
  4, 30,                                     //  3494:   case 0x4: {
  OPC_CheckField, 0, 2, 0,                   //  3496:    check Inst[1:0] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3500:    check Inst[7:5] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  3504:    check Inst[13] == 0x0
  OPC_Scope, 10,                             //  3508:    try {
  OPC_CheckField, 16, 5, 0,                  //  3510:     check Inst[20:16] == 0x0
  OPC_CheckPredicate, 2,                     //  3514:     check predicate 2
  OPC_Decode, 197, 16, 58,                   //  3516:     decode to PS_trap1 using decoder 58
                                             //  3516:    } else try {
  OPC_CheckPredicate, 3,                     //  3520:     check predicate 3
  OPC_Decode, 242, 10, 60,                   //  3522:     decode to J2_trap1 using decoder 60
                                             //  3522:    }
                                             //  3522:   }
  13, 8,                                     //  3526:   case 0xd: {
  OPC_CheckField, 5, 9, 0,                   //  3528:    check Inst[13:5] == 0x0
  OPC_Decode, 158, 26, 61,                   //  3532:    decode to Y2_icdatar using decoder 61
                                             //  3532:   }
  14, 25,                                    //  3536:   case 0xe: {
  OPC_SwitchField, 13, 1,                    //  3538:    switch Inst[13] {
  0, 8,                                      //  3541:    case 0x0: {
  OPC_CheckField, 0, 8, 0,                   //  3543:     check Inst[7:0] == 0x0
  OPC_Decode, 164, 26, 62,                   //  3547:     decode to Y2_ictagw using decoder 62
                                             //  3547:    }
  1, 0,                                      //  3551:    case 0x1: {
  OPC_CheckPredicate, 4,                     //  3553:     check predicate 4
  OPC_CheckField, 0, 8, 0,                   //  3555:     check Inst[7:0] == 0x0
  OPC_Decode, 159, 26, 62,                   //  3559:     decode to Y2_icdataw using decoder 62
                                             //  3559:    }
                                             //  3559:    } // switch Inst[13]
                                             //  3559:   }
  15, 0,                                     //  3563:   case 0xf: {
  OPC_CheckField, 5, 9, 0,                   //  3565:    check Inst[13:5] == 0x0
  OPC_Decode, 163, 26, 61,                   //  3569:    decode to Y2_ictagr using decoder 61
                                             //  3569:   }
                                             //  3569:   } // switch Inst[24:21]
                                             //  3569:  }
  3, 68,                                     //  3573:  case 0x3: {
  OPC_SwitchField, 0, 14,                    //  3575:   switch Inst[13:0] {
  0, 19,                                     //  3578:   case 0x0: {
  OPC_SwitchField, 21, 4,                    //  3580:    switch Inst[24:21] {
  6, 4,                                      //  3583:    case 0x6: {
  OPC_Decode, 160, 26, 56,                   //  3585:     decode to Y2_icinva using decoder 56
                                             //  3585:    }
  15, 0,                                     //  3589:    case 0xf: {
  OPC_CheckField, 16, 5, 0,                  //  3591:     check Inst[20:16] == 0x0
  OPC_Decode, 240, 10, 63,                   //  3595:     decode to J2_rte using decoder 63
                                             //  3595:    }
                                             //  3595:    } // switch Inst[24:21]
                                             //  3595:   }
  2, 9,                                      //  3599:   case 0x2: {
  OPC_CheckField, 16, 9, 192, 3,             //  3601:    check Inst[24:16] == 0x1c0
  OPC_Decode, 165, 26, 63,                   //  3606:    decode to Y2_isync using decoder 63
                                             //  3606:   }
  128, 16, 8,                                //  3610:   case 0x800: {
  OPC_CheckField, 21, 4, 6,                  //  3613:    check Inst[24:21] == 0x6
  OPC_Decode, 161, 26, 56,                   //  3617:    decode to Y2_icinvidx using decoder 56
                                             //  3617:   }
  128, 32, 0,                                //  3621:   case 0x1000: {
  OPC_SwitchField, 16, 9,                    //  3624:    switch Inst[24:16] {
  192, 1, 4,                                 //  3627:    case 0xc0: {
  OPC_Decode, 162, 26, 63,                   //  3630:     decode to Y2_ickill using decoder 63
                                             //  3630:    }
  224, 3, 0,                                 //  3634:    case 0x1e0: {
  OPC_CheckPredicate, 0,                     //  3637:     check predicate 0
  OPC_Decode, 243, 10, 63,                   //  3639:     decode to J2_unpause using decoder 63
                                             //  3639:    }
                                             //  3639:    } // switch Inst[24:16]
                                             //  3639:   }
                                             //  3639:   } // switch Inst[13:0]
                                             //  3639:  }
  4, 8,                                      //  3643:  case 0x4: {
  OPC_CheckField, 0, 1, 0,                   //  3645:   check Inst[0] == 0x0
  OPC_Decode, 198, 10, 64,                   //  3649:   decode to J2_jump using decoder 64
                                             //  3649:  }
  5, 8,                                      //  3653:  case 0x5: {
  OPC_CheckField, 0, 1, 0,                   //  3655:   check Inst[0] == 0x0
  OPC_Decode, 191, 10, 64,                   //  3659:   decode to J2_call using decoder 64
                                             //  3659:  }
  6, 0,                                      //  3663:  case 0x6: {
  OPC_SwitchField, 10, 3,                    //  3665:   switch Inst[12:10] {
  0, 53,                                     //  3668:   case 0x0: {
  OPC_SwitchField, 21, 1,                    //  3670:    switch Inst[21] {
  0, 23,                                     //  3673:    case 0x0: {
  OPC_SwitchField, 24, 1,                    //  3675:     switch Inst[24] {
  0, 8,                                      //  3678:     case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //  3680:      check Inst[0] == 0x0
  OPC_Decode, 221, 10, 65,                   //  3684:      decode to J2_jumpt using decoder 65
                                             //  3684:     }
  1, 0,                                      //  3688:     case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //  3690:      check Inst[0] == 0x0
  OPC_Decode, 197, 10, 65,                   //  3694:      decode to J2_callt using decoder 65
                                             //  3694:     }
                                             //  3694:     } // switch Inst[24]
                                             //  3694:    }
  1, 0,                                      //  3698:    case 0x1: {
  OPC_SwitchField, 24, 1,                    //  3700:     switch Inst[24] {
  0, 8,                                      //  3703:     case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //  3705:      check Inst[0] == 0x0
  OPC_Decode, 199, 10, 65,                   //  3709:      decode to J2_jumpf using decoder 65
                                             //  3709:     }
  1, 0,                                      //  3713:     case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //  3715:      check Inst[0] == 0x0
  OPC_Decode, 192, 10, 65,                   //  3719:      decode to J2_callf using decoder 65
                                             //  3719:     }
                                             //  3719:     } // switch Inst[24]
                                             //  3719:    }
                                             //  3719:    } // switch Inst[21]
                                             //  3719:   }
  2, 31,                                     //  3723:   case 0x2: {
  OPC_SwitchField, 21, 1,                    //  3725:    switch Inst[21] {
  0, 12,                                     //  3728:    case 0x0: {
  OPC_CheckField, 24, 1, 0,                  //  3730:     check Inst[24] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  3734:     check Inst[0] == 0x0
  OPC_Decode, 222, 10, 65,                   //  3738:     decode to J2_jumptnew using decoder 65
                                             //  3738:    }
  1, 0,                                      //  3742:    case 0x1: {
  OPC_CheckField, 24, 1, 0,                  //  3744:     check Inst[24] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  3748:     check Inst[0] == 0x0
  OPC_Decode, 200, 10, 65,                   //  3752:     decode to J2_jumpfnew using decoder 65
                                             //  3752:    }
                                             //  3752:    } // switch Inst[21]
                                             //  3752:   }
  4, 35,                                     //  3756:   case 0x4: {
  OPC_SwitchField, 21, 1,                    //  3758:    switch Inst[21] {
  0, 14,                                     //  3761:    case 0x0: {
  OPC_CheckPredicate, 1,                     //  3763:     check predicate 1
  OPC_CheckField, 24, 1, 0,                  //  3765:     check Inst[24] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  3769:     check Inst[0] == 0x0
  OPC_Decode, 224, 10, 65,                   //  3773:     decode to J2_jumptpt using decoder 65
                                             //  3773:    }
  1, 0,                                      //  3777:    case 0x1: {
  OPC_CheckPredicate, 1,                     //  3779:     check predicate 1
  OPC_CheckField, 24, 1, 0,                  //  3781:     check Inst[24] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  3785:     check Inst[0] == 0x0
  OPC_Decode, 202, 10, 65,                   //  3789:     decode to J2_jumpfpt using decoder 65
                                             //  3789:    }
                                             //  3789:    } // switch Inst[21]
                                             //  3789:   }
  6, 0,                                      //  3793:   case 0x6: {
  OPC_SwitchField, 21, 1,                    //  3795:    switch Inst[21] {
  0, 12,                                     //  3798:    case 0x0: {
  OPC_CheckField, 24, 1, 0,                  //  3800:     check Inst[24] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  3804:     check Inst[0] == 0x0
  OPC_Decode, 223, 10, 65,                   //  3808:     decode to J2_jumptnewpt using decoder 65
                                             //  3808:    }
  1, 0,                                      //  3812:    case 0x1: {
  OPC_CheckField, 24, 1, 0,                  //  3814:     check Inst[24] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  3818:     check Inst[0] == 0x0
  OPC_Decode, 201, 10, 65,                   //  3822:     decode to J2_jumpfnewpt using decoder 65
                                             //  3822:    }
                                             //  3822:    } // switch Inst[21]
                                             //  3822:   }
                                             //  3822:   } // switch Inst[12:10]
                                             //  3822:  }
                                             //  3822:  } // switch Inst[27:25]
                                             //  3822: }
  6, 142, 9,                                 //  3826: case 0x6: {
  OPC_SwitchField, 23, 5,                    //  3829:  switch Inst[27:23] {
  0, 39,                                     //  3832:  case 0x0: {
  OPC_SwitchField, 21, 2,                    //  3834:   switch Inst[22:21] {
  0, 16,                                     //  3837:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  3839:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3843:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 3, 0,                   //  3847:    check Inst[2:0] == 0x0
  OPC_Decode, 227, 10, 66,                   //  3851:    decode to J2_loop0r using decoder 66
                                             //  3851:   }
  1, 0,                                      //  3855:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  3857:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3861:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 3, 0,                   //  3865:    check Inst[2:0] == 0x0
  OPC_Decode, 231, 10, 66,                   //  3869:    decode to J2_loop1r using decoder 66
                                             //  3869:   }
                                             //  3869:   } // switch Inst[22:21]
                                             //  3869:  }
  1, 57,                                     //  3873:  case 0x1: {
  OPC_SwitchField, 21, 2,                    //  3875:   switch Inst[22:21] {
  1, 16,                                     //  3878:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  3880:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3884:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 3, 0,                   //  3888:    check Inst[2:0] == 0x0
  OPC_Decode, 235, 10, 66,                   //  3892:    decode to J2_ploop1sr using decoder 66
                                             //  3892:   }
  2, 16,                                     //  3896:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  3898:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3902:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 3, 0,                   //  3906:    check Inst[2:0] == 0x0
  OPC_Decode, 237, 10, 66,                   //  3910:    decode to J2_ploop2sr using decoder 66
                                             //  3910:   }
  3, 0,                                      //  3914:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  3916:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  3920:    check Inst[7:5] == 0x0
  OPC_CheckField, 0, 3, 0,                   //  3924:    check Inst[2:0] == 0x0
  OPC_Decode, 239, 10, 66,                   //  3928:    decode to J2_ploop3sr using decoder 66
                                             //  3928:   }
                                             //  3928:   } // switch Inst[22:21]
                                             //  3928:  }
  2, 53,                                     //  3932:  case 0x2: {
  OPC_SwitchField, 12, 1,                    //  3934:   switch Inst[12] {
  0, 23,                                     //  3937:   case 0x0: {
  OPC_SwitchField, 22, 1,                    //  3939:    switch Inst[22] {
  0, 8,                                      //  3942:    case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //  3944:     check Inst[0] == 0x0
  OPC_Decode, 219, 10, 67,                   //  3948:     decode to J2_jumprz using decoder 67
                                             //  3948:    }
  1, 0,                                      //  3952:    case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //  3954:     check Inst[0] == 0x0
  OPC_Decode, 208, 10, 67,                   //  3958:     decode to J2_jumprgtez using decoder 67
                                             //  3958:    }
                                             //  3958:    } // switch Inst[22]
                                             //  3958:   }
  1, 0,                                      //  3962:   case 0x1: {
  OPC_SwitchField, 22, 1,                    //  3964:    switch Inst[22] {
  0, 8,                                      //  3967:    case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //  3969:     check Inst[0] == 0x0
  OPC_Decode, 220, 10, 67,                   //  3973:     decode to J2_jumprzpt using decoder 67
                                             //  3973:    }
  1, 0,                                      //  3977:    case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //  3979:     check Inst[0] == 0x0
  OPC_Decode, 209, 10, 67,                   //  3983:     decode to J2_jumprgtezpt using decoder 67
                                             //  3983:    }
                                             //  3983:    } // switch Inst[22]
                                             //  3983:   }
                                             //  3983:   } // switch Inst[12]
                                             //  3983:  }
  3, 53,                                     //  3987:  case 0x3: {
  OPC_SwitchField, 12, 1,                    //  3989:   switch Inst[12] {
  0, 23,                                     //  3992:   case 0x0: {
  OPC_SwitchField, 22, 1,                    //  3994:    switch Inst[22] {
  0, 8,                                      //  3997:    case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //  3999:     check Inst[0] == 0x0
  OPC_Decode, 213, 10, 67,                   //  4003:     decode to J2_jumprnz using decoder 67
                                             //  4003:    }
  1, 0,                                      //  4007:    case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //  4009:     check Inst[0] == 0x0
  OPC_Decode, 211, 10, 67,                   //  4013:     decode to J2_jumprltez using decoder 67
                                             //  4013:    }
                                             //  4013:    } // switch Inst[22]
                                             //  4013:   }
  1, 0,                                      //  4017:   case 0x1: {
  OPC_SwitchField, 22, 1,                    //  4019:    switch Inst[22] {
  0, 8,                                      //  4022:    case 0x0: {
  OPC_CheckField, 0, 1, 0,                   //  4024:     check Inst[0] == 0x0
  OPC_Decode, 214, 10, 67,                   //  4028:     decode to J2_jumprnzpt using decoder 67
                                             //  4028:    }
  1, 0,                                      //  4032:    case 0x1: {
  OPC_CheckField, 0, 1, 0,                   //  4034:     check Inst[0] == 0x0
  OPC_Decode, 212, 10, 67,                   //  4038:     decode to J2_jumprltezpt using decoder 67
                                             //  4038:    }
                                             //  4038:    } // switch Inst[22]
                                             //  4038:   }
                                             //  4038:   } // switch Inst[12]
                                             //  4038:  }
  4, 102,                                    //  4042:  case 0x4: {
  OPC_SwitchField, 5, 3,                     //  4044:   switch Inst[7:5] {
  0, 37,                                     //  4047:   case 0x0: {
  OPC_SwitchField, 21, 2,                    //  4049:    switch Inst[22:21] {
  0, 8,                                      //  4052:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  4054:     check Inst[13:8] == 0x0
  OPC_Decode, 189, 10, 68,                   //  4058:     decode to G4_tfrgrcr using decoder 68
                                             //  4058:    }
  1, 8,                                      //  4062:    case 0x1: {
  OPC_CheckField, 8, 6, 0,                   //  4064:     check Inst[13:8] == 0x0
  OPC_Decode, 145, 8, 69,                    //  4068:     decode to A2_tfrrcr using decoder 69
                                             //  4068:    }
  2, 0,                                      //  4072:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  4074:     check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4078:     check Inst[4:0] == 0x0
  OPC_Decode, 195, 26, 56,                   //  4082:     decode to Y4_trace using decoder 56
                                             //  4082:    }
                                             //  4082:    } // switch Inst[22:21]
                                             //  4082:   }
  1, 18,                                     //  4086:   case 0x1: {
  OPC_CheckPredicate, 5,                     //  4088:    check predicate 5
  OPC_CheckField, 21, 2, 2,                  //  4090:    check Inst[22:21] == 0x2
  OPC_CheckField, 8, 6, 0,                   //  4094:    check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4098:    check Inst[4:0] == 0x0
  OPC_Decode, 207, 26, 56,                   //  4102:    decode to Y6_diag using decoder 56
                                             //  4102:   }
  2, 18,                                     //  4106:   case 0x2: {
  OPC_CheckPredicate, 5,                     //  4108:    check predicate 5
  OPC_CheckField, 21, 2, 2,                  //  4110:    check Inst[22:21] == 0x2
  OPC_CheckField, 13, 1, 0,                  //  4114:    check Inst[13] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4118:    check Inst[4:0] == 0x0
  OPC_Decode, 208, 26, 70,                   //  4122:    decode to Y6_diag0 using decoder 70
                                             //  4122:   }
  3, 0,                                      //  4126:   case 0x3: {
  OPC_CheckPredicate, 5,                     //  4128:    check predicate 5
  OPC_CheckField, 21, 2, 2,                  //  4130:    check Inst[22:21] == 0x2
  OPC_CheckField, 13, 1, 0,                  //  4134:    check Inst[13] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4138:    check Inst[4:0] == 0x0
  OPC_Decode, 209, 26, 70,                   //  4142:    decode to Y6_diag1 using decoder 70
                                             //  4142:   }
                                             //  4142:   } // switch Inst[7:5]
                                             //  4142:  }
  6, 23,                                     //  4146:  case 0x6: {
  OPC_SwitchField, 21, 2,                    //  4148:   switch Inst[22:21] {
  0, 8,                                      //  4151:   case 0x0: {
  OPC_CheckField, 5, 9, 0,                   //  4153:    check Inst[13:5] == 0x0
  OPC_Decode, 188, 10, 71,                   //  4157:    decode to G4_tfrgpcp using decoder 71
                                             //  4157:   }
  1, 0,                                      //  4161:   case 0x1: {
  OPC_CheckField, 5, 9, 0,                   //  4163:    check Inst[13:5] == 0x0
  OPC_Decode, 145, 9, 72,                    //  4167:    decode to A4_tfrpcp using decoder 72
                                             //  4167:   }
                                             //  4167:   } // switch Inst[22:21]
                                             //  4167:  }
  8, 78,                                     //  4171:  case 0x8: {
  OPC_SwitchField, 0, 14,                    //  4173:   switch Inst[13:0] {
  0, 23,                                     //  4176:   case 0x0: {
  OPC_SwitchField, 21, 2,                    //  4178:    switch Inst[22:21] {
  0, 4,                                      //  4181:    case 0x0: {
  OPC_Decode, 175, 26, 56,                   //  4183:     decode to Y2_swi using decoder 56
                                             //  4183:    }
  2, 6,                                      //  4187:    case 0x2: {
  OPC_CheckPredicate, 3,                     //  4189:     check predicate 3
  OPC_Decode, 185, 26, 56,                   //  4191:     decode to Y2_wait using decoder 56
                                             //  4191:    }
  3, 0,                                      //  4195:    case 0x3: {
  OPC_Decode, 174, 26, 56,                   //  4197:     decode to Y2_stop using decoder 56
                                             //  4197:    }
                                             //  4197:    } // switch Inst[22:21]
                                             //  4197:   }
  32, 21,                                    //  4201:   case 0x20: {
  OPC_SwitchField, 21, 2,                    //  4203:    switch Inst[22:21] {
  0, 4,                                      //  4206:    case 0x0: {
  OPC_Decode, 143, 26, 56,                   //  4208:     decode to Y2_cswi using decoder 56
                                             //  4208:    }
  2, 4,                                      //  4212:    case 0x2: {
  OPC_Decode, 170, 26, 56,                   //  4214:     decode to Y2_resume using decoder 56
                                             //  4214:    }
  3, 0,                                      //  4218:    case 0x3: {
  OPC_Decode, 173, 26, 56,                   //  4220:     decode to Y2_start using decoder 56
                                             //  4220:    }
                                             //  4220:    } // switch Inst[22:21]
                                             //  4220:   }
  64, 15,                                    //  4224:   case 0x40: {
  OPC_SwitchField, 21, 2,                    //  4226:    switch Inst[22:21] {
  0, 4,                                      //  4229:    case 0x0: {
  OPC_Decode, 157, 26, 56,                   //  4231:     decode to Y2_iassignw using decoder 56
                                             //  4231:    }
  3, 0,                                      //  4235:    case 0x3: {
  OPC_Decode, 191, 26, 56,                   //  4237:     decode to Y4_nmi using decoder 56
                                             //  4237:    }
                                             //  4237:    } // switch Inst[22:21]
                                             //  4237:   }
  96, 0,                                     //  4241:   case 0x60: {
  OPC_CheckField, 21, 2, 0,                  //  4243:    check Inst[22:21] == 0x0
  OPC_Decode, 141, 26, 56,                   //  4247:    decode to Y2_ciad using decoder 56
                                             //  4247:   }
                                             //  4247:   } // switch Inst[13:0]
                                             //  4247:  }
  9, 47,                                     //  4251:  case 0x9: {
  OPC_SwitchField, 0, 8,                     //  4253:   switch Inst[7:0] {
  0, 12,                                     //  4256:   case 0x0: {
  OPC_CheckField, 21, 2, 0,                  //  4258:    check Inst[22:21] == 0x0
  OPC_CheckField, 10, 4, 0,                  //  4262:    check Inst[13:10] == 0x0
  OPC_Decode, 171, 26, 57,                   //  4266:    decode to Y2_setimask using decoder 57
                                             //  4266:   }
  32, 14,                                    //  4270:   case 0x20: {
  OPC_CheckPredicate, 4,                     //  4272:    check predicate 4
  OPC_CheckField, 21, 2, 0,                  //  4274:    check Inst[22:21] == 0x0
  OPC_CheckField, 10, 4, 0,                  //  4278:    check Inst[13:10] == 0x0
  OPC_Decode, 172, 26, 57,                   //  4282:    decode to Y2_setprio using decoder 57
                                             //  4282:   }
  96, 0,                                     //  4286:   case 0x60: {
  OPC_CheckField, 21, 2, 0,                  //  4288:    check Inst[22:21] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  4292:    check Inst[13:8] == 0x0
  OPC_Decode, 192, 26, 56,                   //  4296:    decode to Y4_siad using decoder 56
                                             //  4296:   }
                                             //  4296:   } // switch Inst[7:0]
                                             //  4296:  }
  10, 23,                                    //  4300:  case 0xa: {
  OPC_SwitchField, 21, 2,                    //  4302:   switch Inst[22:21] {
  0, 8,                                      //  4305:   case 0x0: {
  OPC_CheckField, 0, 14, 0,                  //  4307:    check Inst[13:0] == 0x0
  OPC_Decode, 142, 26, 73,                   //  4311:    decode to Y2_crswap0 using decoder 73
                                             //  4311:   }
  1, 0,                                      //  4315:   case 0x1: {
  OPC_CheckField, 0, 14, 0,                  //  4317:    check Inst[13:0] == 0x0
  OPC_Decode, 186, 26, 73,                   //  4321:    decode to Y4_crswap1 using decoder 73
                                             //  4321:   }
                                             //  4321:   } // switch Inst[22:21]
                                             //  4321:  }
  12, 23,                                    //  4325:  case 0xc: {
  OPC_SwitchField, 21, 2,                    //  4327:   switch Inst[22:21] {
  0, 8,                                      //  4330:   case 0x0: {
  OPC_CheckField, 5, 9, 0,                   //  4332:    check Inst[13:5] == 0x0
  OPC_Decode, 155, 26, 61,                   //  4336:    decode to Y2_getimask using decoder 61
                                             //  4336:   }
  3, 0,                                      //  4340:   case 0x3: {
  OPC_CheckField, 5, 9, 0,                   //  4342:    check Inst[13:5] == 0x0
  OPC_Decode, 156, 26, 61,                   //  4346:    decode to Y2_iassignr using decoder 61
                                             //  4346:   }
                                             //  4346:   } // switch Inst[22:21]
                                             //  4346:  }
  14, 12,                                    //  4350:  case 0xe: {
  OPC_CheckField, 21, 2, 0,                  //  4352:   check Inst[22:21] == 0x0
  OPC_CheckField, 7, 7, 0,                   //  4356:   check Inst[13:7] == 0x0
  OPC_Decode, 178, 26, 74,                   //  4360:   decode to Y2_tfrsrcr using decoder 74
                                             //  4360:  }
  16, 23,                                    //  4364:  case 0x10: {
  OPC_SwitchField, 21, 2,                    //  4366:   switch Inst[22:21] {
  0, 8,                                      //  4369:   case 0x0: {
  OPC_CheckField, 5, 9, 0,                   //  4371:    check Inst[13:5] == 0x0
  OPC_Decode, 144, 9, 75,                    //  4375:    decode to A4_tfrcpp using decoder 75
                                             //  4375:   }
  1, 0,                                      //  4379:   case 0x1: {
  OPC_CheckField, 5, 9, 0,                   //  4381:    check Inst[13:5] == 0x0
  OPC_Decode, 186, 10, 76,                   //  4385:    decode to G4_tfrgcpp using decoder 76
                                             //  4385:   }
                                             //  4385:   } // switch Inst[22:21]
                                             //  4385:  }
  18, 31,                                    //  4389:  case 0x12: {
  OPC_SwitchField, 21, 2,                    //  4391:   switch Inst[22:21] {
  0, 12,                                     //  4394:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  4396:    check Inst[13] == 0x0
  OPC_CheckField, 2, 1, 0,                   //  4400:    check Inst[2] == 0x0
  OPC_Decode, 225, 10, 77,                   //  4404:    decode to J2_loop0i using decoder 77
                                             //  4404:   }
  1, 0,                                      //  4408:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  4410:    check Inst[13] == 0x0
  OPC_CheckField, 2, 1, 0,                   //  4414:    check Inst[2] == 0x0
  OPC_Decode, 229, 10, 77,                   //  4418:    decode to J2_loop1i using decoder 77
                                             //  4418:   }
                                             //  4418:   } // switch Inst[22:21]
                                             //  4418:  }
  19, 45,                                    //  4422:  case 0x13: {
  OPC_SwitchField, 21, 2,                    //  4424:   switch Inst[22:21] {
  1, 12,                                     //  4427:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  4429:    check Inst[13] == 0x0
  OPC_CheckField, 2, 1, 0,                   //  4433:    check Inst[2] == 0x0
  OPC_Decode, 234, 10, 77,                   //  4437:    decode to J2_ploop1si using decoder 77
                                             //  4437:   }
  2, 12,                                     //  4441:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  4443:    check Inst[13] == 0x0
  OPC_CheckField, 2, 1, 0,                   //  4447:    check Inst[2] == 0x0
  OPC_Decode, 236, 10, 77,                   //  4451:    decode to J2_ploop2si using decoder 77
                                             //  4451:   }
  3, 0,                                      //  4455:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  4457:    check Inst[13] == 0x0
  OPC_CheckField, 2, 1, 0,                   //  4461:    check Inst[2] == 0x0
  OPC_Decode, 238, 10, 77,                   //  4465:    decode to J2_ploop3si using decoder 77
                                             //  4465:   }
                                             //  4465:   } // switch Inst[22:21]
                                             //  4465:  }
  20, 41,                                    //  4469:  case 0x14: {
  OPC_SwitchField, 21, 2,                    //  4471:   switch Inst[22:21] {
  0, 8,                                      //  4474:   case 0x0: {
  OPC_CheckField, 5, 9, 0,                   //  4476:    check Inst[13:5] == 0x0
  OPC_Decode, 142, 8, 78,                    //  4480:    decode to A2_tfrcrr using decoder 78
                                             //  4480:   }
  1, 8,                                      //  4484:   case 0x1: {
  OPC_CheckField, 5, 9, 0,                   //  4486:    check Inst[13:5] == 0x0
  OPC_Decode, 187, 10, 79,                   //  4490:    decode to G4_tfrgcrr using decoder 79
                                             //  4490:   }
  2, 0,                                      //  4494:   case 0x2: {
  OPC_CheckField, 16, 5, 9,                  //  4496:    check Inst[20:16] == 0x9
  OPC_CheckField, 13, 1, 0,                  //  4500:    check Inst[13] == 0x0
  OPC_CheckField, 5, 2, 0,                   //  4504:    check Inst[6:5] == 0x0
  OPC_Decode, 211, 9, 80,                    //  4508:    decode to C4_addipc using decoder 80
                                             //  4508:   }
                                             //  4508:   } // switch Inst[22:21]
                                             //  4508:  }
  22, 141, 1,                                //  4512:  case 0x16: {
  OPC_SwitchField, 18, 5,                    //  4515:   switch Inst[22:18] {
  0, 23,                                     //  4518:   case 0x0: {
  OPC_SwitchField, 2, 6,                     //  4520:    switch Inst[7:2] {
  0, 8,                                      //  4523:    case 0x0: {
  OPC_CheckField, 10, 4, 0,                  //  4525:     check Inst[13:10] == 0x0
  OPC_Decode, 175, 9, 81,                    //  4529:     decode to C2_and using decoder 81
                                             //  4529:    }
  36, 0,                                     //  4533:    case 0x24: {
  OPC_CheckField, 10, 4, 8,                  //  4535:     check Inst[13:10] == 0x8
  OPC_Decode, 222, 9, 82,                    //  4539:     decode to C4_fastcorner9 using decoder 82
                                             //  4539:    }
                                             //  4539:    } // switch Inst[7:2]
                                             //  4539:   }
  4, 27,                                     //  4543:   case 0x4: {
  OPC_SwitchField, 2, 4,                     //  4545:    switch Inst[5:2] {
  0, 8,                                      //  4548:    case 0x0: {
  OPC_CheckField, 10, 4, 0,                  //  4550:     check Inst[13:10] == 0x0
  OPC_Decode, 212, 9, 83,                    //  4554:     decode to C4_and_and using decoder 83
                                             //  4554:    }
  4, 0,                                      //  4558:    case 0x4: {
  OPC_CheckField, 10, 4, 8,                  //  4560:     check Inst[13:10] == 0x8
  OPC_CheckField, 6, 2, 2,                   //  4564:     check Inst[7:6] == 0x2
  OPC_Decode, 223, 9, 82,                    //  4568:     decode to C4_fastcorner9_not using decoder 82
                                             //  4568:    }
                                             //  4568:    } // switch Inst[5:2]
                                             //  4568:   }
  8, 12,                                     //  4572:   case 0x8: {
  OPC_CheckField, 10, 4, 0,                  //  4574:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  4578:    check Inst[7:2] == 0x0
  OPC_Decode, 204, 9, 81,                    //  4582:    decode to C2_or using decoder 81
                                             //  4582:   }
  12, 12,                                    //  4586:   case 0xc: {
  OPC_CheckField, 10, 4, 0,                  //  4588:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4592:    check Inst[5:2] == 0x0
  OPC_Decode, 214, 9, 83,                    //  4596:    decode to C4_and_or using decoder 83
                                             //  4596:   }
  16, 12,                                    //  4600:   case 0x10: {
  OPC_CheckField, 10, 4, 0,                  //  4602:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  4606:    check Inst[7:2] == 0x0
  OPC_Decode, 210, 9, 82,                    //  4610:    decode to C2_xor using decoder 82
                                             //  4610:   }
  20, 12,                                    //  4614:   case 0x14: {
  OPC_CheckField, 10, 4, 0,                  //  4616:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4620:    check Inst[5:2] == 0x0
  OPC_Decode, 227, 9, 83,                    //  4624:    decode to C4_or_and using decoder 83
                                             //  4624:   }
  24, 12,                                    //  4628:   case 0x18: {
  OPC_CheckField, 10, 4, 0,                  //  4630:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  4634:    check Inst[7:2] == 0x0
  OPC_Decode, 176, 9, 81,                    //  4638:    decode to C2_andn using decoder 81
                                             //  4638:   }
  28, 0,                                     //  4642:   case 0x1c: {
  OPC_CheckField, 10, 4, 0,                  //  4644:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4648:    check Inst[5:2] == 0x0
  OPC_Decode, 229, 9, 83,                    //  4652:    decode to C4_or_or using decoder 83
                                             //  4652:   }
                                             //  4652:   } // switch Inst[22:18]
                                             //  4652:  }
  23, 103,                                   //  4656:  case 0x17: {
  OPC_SwitchField, 18, 5,                    //  4658:   switch Inst[22:18] {
  0, 8,                                      //  4661:   case 0x0: {
  OPC_CheckField, 2, 12, 0,                  //  4663:    check Inst[13:2] == 0x0
  OPC_Decode, 177, 9, 84,                    //  4667:    decode to C2_any8 using decoder 84
                                             //  4667:   }
  4, 12,                                     //  4671:   case 0x4: {
  OPC_CheckField, 10, 4, 0,                  //  4673:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4677:    check Inst[5:2] == 0x0
  OPC_Decode, 213, 9, 83,                    //  4681:    decode to C4_and_andn using decoder 83
                                             //  4681:   }
  8, 8,                                      //  4685:   case 0x8: {
  OPC_CheckField, 2, 12, 0,                  //  4687:    check Inst[13:2] == 0x0
  OPC_Decode, 174, 9, 84,                    //  4691:    decode to C2_all8 using decoder 84
                                             //  4691:   }
  12, 12,                                    //  4695:   case 0xc: {
  OPC_CheckField, 10, 4, 0,                  //  4697:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4701:    check Inst[5:2] == 0x0
  OPC_Decode, 215, 9, 83,                    //  4705:    decode to C4_and_orn using decoder 83
                                             //  4705:   }
  16, 8,                                     //  4709:   case 0x10: {
  OPC_CheckField, 2, 12, 0,                  //  4711:    check Inst[13:2] == 0x0
  OPC_Decode, 203, 9, 84,                    //  4715:    decode to C2_not using decoder 84
                                             //  4715:   }
  20, 12,                                    //  4719:   case 0x14: {
  OPC_CheckField, 10, 4, 0,                  //  4721:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4725:    check Inst[5:2] == 0x0
  OPC_Decode, 228, 9, 83,                    //  4729:    decode to C4_or_andn using decoder 83
                                             //  4729:   }
  24, 12,                                    //  4733:   case 0x18: {
  OPC_CheckField, 10, 4, 0,                  //  4735:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  4739:    check Inst[7:2] == 0x0
  OPC_Decode, 205, 9, 81,                    //  4743:    decode to C2_orn using decoder 81
                                             //  4743:   }
  28, 0,                                     //  4747:   case 0x1c: {
  OPC_CheckField, 10, 4, 0,                  //  4749:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 4, 0,                   //  4753:    check Inst[5:2] == 0x0
  OPC_Decode, 230, 9, 83,                    //  4757:    decode to C4_or_orn using decoder 83
                                             //  4757:   }
                                             //  4757:   } // switch Inst[22:18]
                                             //  4757:  }
  24, 134, 1,                                //  4761:  case 0x18: {
  OPC_SwitchField, 5, 3,                     //  4764:   switch Inst[7:5] {
  0, 57,                                     //  4767:   case 0x0: {
  OPC_SwitchField, 21, 2,                    //  4769:    switch Inst[22:21] {
  0, 12,                                     //  4772:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  4774:     check Inst[13] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4778:     check Inst[4:0] == 0x0
  OPC_Decode, 184, 26, 85,                   //  4782:     decode to Y2_tlbw using decoder 85
                                             //  4782:    }
  1, 16,                                     //  4786:    case 0x1: {
  OPC_CheckField, 16, 5, 0,                  //  4788:     check Inst[20:16] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  4792:     check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4796:     check Inst[4:0] == 0x0
  OPC_Decode, 140, 26, 63,                   //  4800:     decode to Y2_break using decoder 63
                                             //  4800:    }
  2, 8,                                      //  4804:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  4806:     check Inst[13:8] == 0x0
  OPC_Decode, 182, 26, 86,                   //  4810:     decode to Y2_tlbr using decoder 86
                                             //  4810:    }
  3, 0,                                      //  4814:    case 0x3: {
  OPC_CheckPredicate, 6,                     //  4816:     check predicate 6
  OPC_CheckField, 8, 6, 0,                   //  4818:     check Inst[13:8] == 0x0
  OPC_Decode, 181, 26, 87,                   //  4822:     decode to Y2_tlbpp using decoder 87
                                             //  4822:    }
                                             //  4822:    } // switch Inst[22:21]
                                             //  4822:   }
  1, 16,                                     //  4826:   case 0x1: {
  OPC_CheckField, 16, 7, 32,                 //  4828:    check Inst[22:16] == 0x20
  OPC_CheckField, 8, 6, 0,                   //  4832:    check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4836:    check Inst[4:0] == 0x0
  OPC_Decode, 179, 26, 63,                   //  4840:    decode to Y2_tlblock using decoder 63
                                             //  4840:   }
  2, 16,                                     //  4844:   case 0x2: {
  OPC_CheckField, 16, 7, 32,                 //  4846:    check Inst[22:16] == 0x20
  OPC_CheckField, 8, 6, 0,                   //  4850:    check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4854:    check Inst[4:0] == 0x0
  OPC_Decode, 183, 26, 63,                   //  4858:    decode to Y2_tlbunlock using decoder 63
                                             //  4858:   }
  3, 16,                                     //  4862:   case 0x3: {
  OPC_CheckField, 16, 7, 32,                 //  4864:    check Inst[22:16] == 0x20
  OPC_CheckField, 8, 6, 0,                   //  4868:    check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4872:    check Inst[4:0] == 0x0
  OPC_Decode, 166, 26, 63,                   //  4876:    decode to Y2_k0lock using decoder 63
                                             //  4876:   }
  4, 0,                                      //  4880:   case 0x4: {
  OPC_CheckField, 16, 7, 32,                 //  4882:    check Inst[22:16] == 0x20
  OPC_CheckField, 8, 6, 0,                   //  4886:    check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  4890:    check Inst[4:0] == 0x0
  OPC_Decode, 167, 26, 63,                   //  4894:    decode to Y2_k0unlock using decoder 63
                                             //  4894:   }
                                             //  4894:   } // switch Inst[7:5]
                                             //  4894:  }
  25, 47,                                    //  4898:  case 0x19: {
  OPC_SwitchField, 21, 2,                    //  4900:   switch Inst[22:21] {
  0, 8,                                      //  4903:   case 0x0: {
  OPC_CheckField, 5, 9, 0,                   //  4905:    check Inst[13:5] == 0x0
  OPC_Decode, 180, 26, 61,                   //  4909:    decode to Y2_tlbp using decoder 61
                                             //  4909:   }
  1, 8,                                      //  4913:   case 0x1: {
  OPC_CheckField, 0, 14, 0,                  //  4915:    check Inst[13:0] == 0x0
  OPC_Decode, 205, 26, 56,                   //  4919:    decode to Y5_tlbasidi using decoder 56
                                             //  4919:   }
  2, 12,                                     //  4923:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  4925:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  4929:    check Inst[7:5] == 0x0
  OPC_Decode, 196, 26, 88,                   //  4933:    decode to Y5_ctlbw using decoder 88
                                             //  4933:   }
  3, 0,                                      //  4937:   case 0x3: {
  OPC_CheckField, 5, 9, 0,                   //  4939:    check Inst[13:5] == 0x0
  OPC_Decode, 206, 26, 87,                   //  4943:    decode to Y5_tlboc using decoder 87
                                             //  4943:   }
                                             //  4943:   } // switch Inst[22:21]
                                             //  4943:  }
  26, 12,                                    //  4947:  case 0x1a: {
  OPC_CheckField, 21, 2, 0,                  //  4949:   check Inst[22:21] == 0x0
  OPC_CheckField, 7, 7, 0,                   //  4953:   check Inst[13:7] == 0x0
  OPC_Decode, 194, 26, 89,                   //  4957:   decode to Y4_tfrspcp using decoder 89
                                             //  4957:  }
  27, 12,                                    //  4961:  case 0x1b: {
  OPC_CheckField, 21, 2, 0,                  //  4963:   check Inst[22:21] == 0x0
  OPC_CheckField, 0, 14, 0,                  //  4967:   check Inst[13:0] == 0x0
  OPC_Decode, 187, 26, 90,                   //  4971:   decode to Y4_crswap10 using decoder 90
                                             //  4971:  }
  29, 8,                                     //  4975:  case 0x1d: {
  OPC_CheckField, 5, 9, 0,                   //  4977:   check Inst[13:5] == 0x0
  OPC_Decode, 177, 26, 91,                   //  4981:   decode to Y2_tfrscrr using decoder 91
                                             //  4981:  }
  30, 0,                                     //  4985:  case 0x1e: {
  OPC_CheckField, 5, 9, 0,                   //  4987:   check Inst[13:5] == 0x0
  OPC_Decode, 193, 26, 92,                   //  4991:   decode to Y4_tfrscpp using decoder 92
                                             //  4991:  }
                                             //  4991:  } // switch Inst[27:23]
                                             //  4991: }
  7, 172, 5,                                 //  4995: case 0x7: {
  OPC_SwitchField, 25, 3,                    //  4998:  switch Inst[27:25] {
  0, 248, 2,                                 //  5001:  case 0x0: {
  OPC_SwitchField, 21, 1,                    //  5004:   switch Inst[21] {
  0, 168, 1,                                 //  5007:   case 0x0: {
  OPC_SwitchField, 10, 4,                    //  5010:    switch Inst[13:10] {
  0, 23,                                     //  5013:    case 0x0: {
  OPC_SwitchField, 22, 3,                    //  5015:     switch Inst[24:22] {
  0, 8,                                      //  5018:     case 0x0: {
  OPC_CheckField, 5, 5, 0,                   //  5020:      check Inst[9:5] == 0x0
  OPC_Decode, 187, 7, 61,                    //  5024:      decode to A2_aslh using decoder 61
                                             //  5024:     }
  3, 0,                                      //  5028:     case 0x3: {
  OPC_CheckField, 5, 5, 0,                   //  5030:      check Inst[9:5] == 0x0
  OPC_Decode, 210, 8, 61,                    //  5034:      decode to A2_zxth using decoder 61
                                             //  5034:     }
                                             //  5034:     } // switch Inst[24:22]
                                             //  5034:    }
  8, 33,                                     //  5038:    case 0x8: {
  OPC_SwitchField, 22, 3,                    //  5040:     switch Inst[24:22] {
  0, 8,                                      //  5043:     case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5045:      check Inst[7:5] == 0x0
  OPC_Decode, 241, 8, 93,                    //  5049:      decode to A4_paslht using decoder 93
                                             //  5049:     }
  2, 8,                                      //  5053:     case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5055:      check Inst[7:5] == 0x0
  OPC_Decode, 129, 9, 93,                    //  5059:      decode to A4_pzxtbt using decoder 93
                                             //  5059:     }
  3, 0,                                      //  5063:     case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5065:      check Inst[7:5] == 0x0
  OPC_Decode, 133, 9, 93,                    //  5069:      decode to A4_pzxtht using decoder 93
                                             //  5069:     }
                                             //  5069:     } // switch Inst[24:22]
                                             //  5069:    }
  9, 33,                                     //  5073:    case 0x9: {
  OPC_SwitchField, 22, 3,                    //  5075:     switch Inst[24:22] {
  0, 8,                                      //  5078:     case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5080:      check Inst[7:5] == 0x0
  OPC_Decode, 242, 8, 93,                    //  5084:      decode to A4_paslhtnew using decoder 93
                                             //  5084:     }
  2, 8,                                      //  5088:     case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5090:      check Inst[7:5] == 0x0
  OPC_Decode, 130, 9, 93,                    //  5094:      decode to A4_pzxtbtnew using decoder 93
                                             //  5094:     }
  3, 0,                                      //  5098:     case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5100:      check Inst[7:5] == 0x0
  OPC_Decode, 134, 9, 93,                    //  5104:      decode to A4_pzxthtnew using decoder 93
                                             //  5104:     }
                                             //  5104:     } // switch Inst[24:22]
                                             //  5104:    }
  10, 33,                                    //  5108:    case 0xa: {
  OPC_SwitchField, 22, 3,                    //  5110:     switch Inst[24:22] {
  0, 8,                                      //  5113:     case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5115:      check Inst[7:5] == 0x0
  OPC_Decode, 239, 8, 93,                    //  5119:      decode to A4_paslhf using decoder 93
                                             //  5119:     }
  2, 8,                                      //  5123:     case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5125:      check Inst[7:5] == 0x0
  OPC_Decode, 255, 8, 93,                    //  5129:      decode to A4_pzxtbf using decoder 93
                                             //  5129:     }
  3, 0,                                      //  5133:     case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5135:      check Inst[7:5] == 0x0
  OPC_Decode, 131, 9, 93,                    //  5139:      decode to A4_pzxthf using decoder 93
                                             //  5139:     }
                                             //  5139:     } // switch Inst[24:22]
                                             //  5139:    }
  11, 0,                                     //  5143:    case 0xb: {
  OPC_SwitchField, 22, 3,                    //  5145:     switch Inst[24:22] {
  0, 8,                                      //  5148:     case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5150:      check Inst[7:5] == 0x0
  OPC_Decode, 240, 8, 93,                    //  5154:      decode to A4_paslhfnew using decoder 93
                                             //  5154:     }
  2, 8,                                      //  5158:     case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5160:      check Inst[7:5] == 0x0
  OPC_Decode, 128, 9, 93,                    //  5164:      decode to A4_pzxtbfnew using decoder 93
                                             //  5164:     }
  3, 0,                                      //  5168:     case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5170:      check Inst[7:5] == 0x0
  OPC_Decode, 132, 9, 93,                    //  5174:      decode to A4_pzxthfnew using decoder 93
                                             //  5174:     }
                                             //  5174:     } // switch Inst[24:22]
                                             //  5174:    }
                                             //  5174:    } // switch Inst[13:10]
                                             //  5174:   }
  1, 0,                                      //  5178:   case 0x1: {
  OPC_SwitchField, 24, 1,                    //  5180:    switch Inst[24] {
  0, 188, 1,                                 //  5183:    case 0x0: {
  OPC_SwitchField, 10, 4,                    //  5186:     switch Inst[13:10] {
  0, 43,                                     //  5189:     case 0x0: {
  OPC_SwitchField, 22, 2,                    //  5191:      switch Inst[23:22] {
  0, 8,                                      //  5194:      case 0x0: {
  OPC_CheckField, 5, 5, 0,                   //  5196:       check Inst[9:5] == 0x0
  OPC_Decode, 188, 7, 61,                    //  5200:       decode to A2_asrh using decoder 61
                                             //  5200:      }
  1, 8,                                      //  5204:      case 0x1: {
  OPC_CheckField, 5, 5, 0,                   //  5206:       check Inst[9:5] == 0x0
  OPC_Decode, 141, 8, 61,                    //  5210:       decode to A2_tfr using decoder 61
                                             //  5210:      }
  2, 8,                                      //  5214:      case 0x2: {
  OPC_CheckField, 5, 5, 0,                   //  5216:       check Inst[9:5] == 0x0
  OPC_Decode, 138, 8, 61,                    //  5220:       decode to A2_sxtb using decoder 61
                                             //  5220:      }
  3, 0,                                      //  5224:      case 0x3: {
  OPC_CheckField, 5, 5, 0,                   //  5226:       check Inst[9:5] == 0x0
  OPC_Decode, 139, 8, 61,                    //  5230:       decode to A2_sxth using decoder 61
                                             //  5230:      }
                                             //  5230:      } // switch Inst[23:22]
                                             //  5230:     }
  8, 33,                                     //  5234:     case 0x8: {
  OPC_SwitchField, 22, 2,                    //  5236:      switch Inst[23:22] {
  0, 8,                                      //  5239:      case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5241:       check Inst[7:5] == 0x0
  OPC_Decode, 245, 8, 93,                    //  5245:       decode to A4_pasrht using decoder 93
                                             //  5245:      }
  2, 8,                                      //  5249:      case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5251:       check Inst[7:5] == 0x0
  OPC_Decode, 249, 8, 93,                    //  5255:       decode to A4_psxtbt using decoder 93
                                             //  5255:      }
  3, 0,                                      //  5259:      case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5261:       check Inst[7:5] == 0x0
  OPC_Decode, 253, 8, 93,                    //  5265:       decode to A4_psxtht using decoder 93
                                             //  5265:      }
                                             //  5265:      } // switch Inst[23:22]
                                             //  5265:     }
  9, 33,                                     //  5269:     case 0x9: {
  OPC_SwitchField, 22, 2,                    //  5271:      switch Inst[23:22] {
  0, 8,                                      //  5274:      case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5276:       check Inst[7:5] == 0x0
  OPC_Decode, 246, 8, 93,                    //  5280:       decode to A4_pasrhtnew using decoder 93
                                             //  5280:      }
  2, 8,                                      //  5284:      case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5286:       check Inst[7:5] == 0x0
  OPC_Decode, 250, 8, 93,                    //  5290:       decode to A4_psxtbtnew using decoder 93
                                             //  5290:      }
  3, 0,                                      //  5294:      case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5296:       check Inst[7:5] == 0x0
  OPC_Decode, 254, 8, 93,                    //  5300:       decode to A4_psxthtnew using decoder 93
                                             //  5300:      }
                                             //  5300:      } // switch Inst[23:22]
                                             //  5300:     }
  10, 33,                                    //  5304:     case 0xa: {
  OPC_SwitchField, 22, 2,                    //  5306:      switch Inst[23:22] {
  0, 8,                                      //  5309:      case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5311:       check Inst[7:5] == 0x0
  OPC_Decode, 243, 8, 93,                    //  5315:       decode to A4_pasrhf using decoder 93
                                             //  5315:      }
  2, 8,                                      //  5319:      case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5321:       check Inst[7:5] == 0x0
  OPC_Decode, 247, 8, 93,                    //  5325:       decode to A4_psxtbf using decoder 93
                                             //  5325:      }
  3, 0,                                      //  5329:      case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5331:       check Inst[7:5] == 0x0
  OPC_Decode, 251, 8, 93,                    //  5335:       decode to A4_psxthf using decoder 93
                                             //  5335:      }
                                             //  5335:      } // switch Inst[23:22]
                                             //  5335:     }
  11, 0,                                     //  5339:     case 0xb: {
  OPC_SwitchField, 22, 2,                    //  5341:      switch Inst[23:22] {
  0, 8,                                      //  5344:      case 0x0: {
  OPC_CheckField, 5, 3, 0,                   //  5346:       check Inst[7:5] == 0x0
  OPC_Decode, 244, 8, 93,                    //  5350:       decode to A4_pasrhfnew using decoder 93
                                             //  5350:      }
  2, 8,                                      //  5354:      case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  5356:       check Inst[7:5] == 0x0
  OPC_Decode, 248, 8, 93,                    //  5360:       decode to A4_psxtbfnew using decoder 93
                                             //  5360:      }
  3, 0,                                      //  5364:      case 0x3: {
  OPC_CheckField, 5, 3, 0,                   //  5366:       check Inst[7:5] == 0x0
  OPC_Decode, 252, 8, 93,                    //  5370:       decode to A4_psxthfnew using decoder 93
                                             //  5370:      }
                                             //  5370:      } // switch Inst[23:22]
                                             //  5370:     }
                                             //  5370:     } // switch Inst[13:10]
                                             //  5370:    }
  1, 0,                                      //  5374:    case 0x1: {
  OPC_Decode, 144, 8, 94,                    //  5376:     decode to A2_tfril using decoder 94
                                             //  5376:    }
                                             //  5376:    } // switch Inst[24]
                                             //  5376:   }
                                             //  5376:   } // switch Inst[21]
                                             //  5376:  }
  1, 64,                                     //  5380:  case 0x1: {
  OPC_SwitchField, 24, 1,                    //  5382:   switch Inst[24] {
  0, 8,                                      //  5385:   case 0x0: {
  OPC_CheckField, 21, 1, 1,                  //  5387:    check Inst[21] == 0x1
  OPC_Decode, 143, 8, 94,                    //  5391:    decode to A2_tfrih using decoder 94
                                             //  5391:   }
  1, 0,                                      //  5395:   case 0x1: {
  OPC_SwitchField, 13, 1,                    //  5397:    switch Inst[13] {
  0, 15,                                     //  5400:    case 0x0: {
  OPC_SwitchField, 23, 1,                    //  5402:     switch Inst[23] {
  0, 4,                                      //  5405:     case 0x0: {
  OPC_Decode, 201, 9, 95,                    //  5407:      decode to C2_muxir using decoder 95
                                             //  5407:     }
  1, 0,                                      //  5411:     case 0x1: {
  OPC_Decode, 202, 9, 96,                    //  5413:      decode to C2_muxri using decoder 96
                                             //  5413:     }
                                             //  5413:     } // switch Inst[23]
                                             //  5413:    }
  1, 0,                                      //  5417:    case 0x1: {
  OPC_SwitchField, 21, 3,                    //  5419:     switch Inst[23:21] {
  0, 4,                                      //  5422:     case 0x0: {
  OPC_Decode, 232, 8, 97,                    //  5424:      decode to A4_combineri using decoder 97
                                             //  5424:     }
  1, 4,                                      //  5428:     case 0x1: {
  OPC_Decode, 231, 8, 98,                    //  5430:      decode to A4_combineir using decoder 98
                                             //  5430:     }
  2, 4,                                      //  5434:     case 0x2: {
  OPC_Decode, 136, 9, 99,                    //  5436:      decode to A4_rcmpeqi using decoder 99
                                             //  5436:     }
  3, 0,                                      //  5440:     case 0x3: {
  OPC_Decode, 138, 9, 99,                    //  5442:      decode to A4_rcmpneqi using decoder 99
                                             //  5442:     }
                                             //  5442:     } // switch Inst[23:21]
                                             //  5442:    }
                                             //  5442:    } // switch Inst[13]
                                             //  5442:   }
                                             //  5442:   } // switch Inst[24]
                                             //  5442:  }
  2, 101,                                    //  5446:  case 0x2: {
  OPC_SwitchField, 23, 2,                    //  5448:   switch Inst[24:23] {
  0, 15,                                     //  5451:   case 0x0: {
  OPC_SwitchField, 13, 1,                    //  5453:    switch Inst[13] {
  0, 4,                                      //  5456:    case 0x0: {
  OPC_Decode, 214, 7, 95,                    //  5458:     decode to A2_paddit using decoder 95
                                             //  5458:    }
  1, 0,                                      //  5462:    case 0x1: {
  OPC_Decode, 215, 7, 95,                    //  5464:     decode to A2_padditnew using decoder 95
                                             //  5464:    }
                                             //  5464:    } // switch Inst[13]
                                             //  5464:   }
  1, 15,                                     //  5468:   case 0x1: {
  OPC_SwitchField, 13, 1,                    //  5470:    switch Inst[13] {
  0, 4,                                      //  5473:    case 0x0: {
  OPC_Decode, 212, 7, 95,                    //  5475:     decode to A2_paddif using decoder 95
                                             //  5475:    }
  1, 0,                                      //  5479:    case 0x1: {
  OPC_Decode, 213, 7, 95,                    //  5481:     decode to A2_paddifnew using decoder 95
                                             //  5481:    }
                                             //  5481:    } // switch Inst[13]
                                             //  5481:   }
  2, 37,                                     //  5485:   case 0x2: {
  OPC_SwitchField, 2, 3,                     //  5487:    switch Inst[4:2] {
  0, 15,                                     //  5490:    case 0x0: {
  OPC_SwitchField, 22, 1,                    //  5492:     switch Inst[22] {
  0, 4,                                      //  5495:     case 0x0: {
  OPC_Decode, 190, 9, 100,                   //  5497:      decode to C2_cmpeqi using decoder 100
                                             //  5497:     }
  1, 0,                                      //  5501:     case 0x1: {
  OPC_Decode, 193, 9, 100,                   //  5503:      decode to C2_cmpgti using decoder 100
                                             //  5503:     }
                                             //  5503:     } // switch Inst[22]
                                             //  5503:    }
  4, 0,                                      //  5507:    case 0x4: {
  OPC_SwitchField, 22, 1,                    //  5509:     switch Inst[22] {
  0, 4,                                      //  5512:     case 0x0: {
  OPC_Decode, 221, 9, 100,                   //  5514:      decode to C4_cmpneqi using decoder 100
                                             //  5514:     }
  1, 0,                                      //  5518:     case 0x1: {
  OPC_Decode, 217, 9, 100,                   //  5520:      decode to C4_cmpltei using decoder 100
                                             //  5520:     }
                                             //  5520:     } // switch Inst[22]
                                             //  5520:    }
                                             //  5520:    } // switch Inst[4:2]
                                             //  5520:   }
  3, 0,                                      //  5524:   case 0x3: {
  OPC_SwitchField, 2, 3,                     //  5526:    switch Inst[4:2] {
  0, 8,                                      //  5529:    case 0x0: {
  OPC_CheckField, 21, 2, 0,                  //  5531:     check Inst[22:21] == 0x0
  OPC_Decode, 196, 9, 101,                   //  5535:     decode to C2_cmpgtui using decoder 101
                                             //  5535:    }
  4, 0,                                      //  5539:    case 0x4: {
  OPC_CheckField, 21, 2, 0,                  //  5541:     check Inst[22:21] == 0x0
  OPC_Decode, 219, 9, 101,                   //  5545:     decode to C4_cmplteui using decoder 101
                                             //  5545:    }
                                             //  5545:    } // switch Inst[4:2]
                                             //  5545:   }
                                             //  5545:   } // switch Inst[24:23]
                                             //  5545:  }
  3, 21,                                     //  5549:  case 0x3: {
  OPC_SwitchField, 22, 3,                    //  5551:   switch Inst[24:22] {
  0, 4,                                      //  5554:   case 0x0: {
  OPC_Decode, 185, 7, 102,                   //  5556:    decode to A2_andir using decoder 102
                                             //  5556:   }
  1, 4,                                      //  5560:   case 0x1: {
  OPC_Decode, 254, 7, 103,                   //  5562:    decode to A2_subri using decoder 103
                                             //  5562:   }
  2, 0,                                      //  5566:   case 0x2: {
  OPC_Decode, 208, 7, 102,                   //  5568:    decode to A2_orir using decoder 102
                                             //  5568:   }
                                             //  5568:   } // switch Inst[24:22]
                                             //  5568:  }
  4, 12,                                     //  5572:  case 0x4: {
  OPC_CheckField, 24, 1, 0,                  //  5574:   check Inst[24] == 0x0
  OPC_CheckField, 21, 1, 0,                  //  5578:   check Inst[21] == 0x0
  OPC_Decode, 146, 8, 104,                   //  5582:   decode to A2_tfrsi using decoder 104
                                             //  5582:  }
  5, 4,                                      //  5586:  case 0x5: {
  OPC_Decode, 200, 9, 105,                   //  5588:   decode to C2_muxii using decoder 105
                                             //  5588:  }
  6, 19,                                     //  5592:  case 0x6: {
  OPC_SwitchField, 23, 2,                    //  5594:   switch Inst[24:23] {
  0, 4,                                      //  5597:   case 0x0: {
  OPC_Decode, 193, 7, 106,                   //  5599:    decode to A2_combineii using decoder 106
                                             //  5599:   }
  1, 0,                                      //  5603:   case 0x1: {
  OPC_CheckField, 21, 2, 0,                  //  5605:    check Inst[22:21] == 0x0
  OPC_Decode, 230, 8, 107,                   //  5609:    decode to A4_combineii using decoder 107
                                             //  5609:   }
                                             //  5609:   } // switch Inst[24:23]
                                             //  5609:  }
  7, 0,                                      //  5613:  case 0x7: {
  OPC_SwitchField, 23, 2,                    //  5615:   switch Inst[24:23] {
  0, 23,                                     //  5618:   case 0x0: {
  OPC_SwitchField, 13, 1,                    //  5620:    switch Inst[13] {
  0, 8,                                      //  5623:    case 0x0: {
  OPC_CheckField, 20, 1, 0,                  //  5625:     check Inst[20] == 0x0
  OPC_Decode, 186, 9, 108,                   //  5629:     decode to C2_cmoveit using decoder 108
                                             //  5629:    }
  1, 0,                                      //  5633:    case 0x1: {
  OPC_CheckField, 20, 1, 0,                  //  5635:     check Inst[20] == 0x0
  OPC_Decode, 188, 9, 108,                   //  5639:     decode to C2_cmovenewit using decoder 108
                                             //  5639:    }
                                             //  5639:    } // switch Inst[13]
                                             //  5639:   }
  1, 23,                                     //  5643:   case 0x1: {
  OPC_SwitchField, 13, 1,                    //  5645:    switch Inst[13] {
  0, 8,                                      //  5648:    case 0x0: {
  OPC_CheckField, 20, 1, 0,                  //  5650:     check Inst[20] == 0x0
  OPC_Decode, 185, 9, 108,                   //  5654:     decode to C2_cmoveif using decoder 108
                                             //  5654:    }
  1, 0,                                      //  5658:    case 0x1: {
  OPC_CheckField, 20, 1, 0,                  //  5660:     check Inst[20] == 0x0
  OPC_Decode, 187, 9, 108,                   //  5664:     decode to C2_cmovenewif using decoder 108
                                             //  5664:    }
                                             //  5664:    } // switch Inst[13]
                                             //  5664:   }
  2, 0,                                      //  5668:   case 0x2: {
  OPC_CheckField, 16, 7, 0,                  //  5670:    check Inst[22:16] == 0x0
  OPC_CheckField, 0, 14, 0,                  //  5674:    check Inst[13:0] == 0x0
  OPC_Decode, 205, 7, 63,                    //  5678:    decode to A2_nop using decoder 63
                                             //  5678:   }
                                             //  5678:   } // switch Inst[24:23]
                                             //  5678:  }
                                             //  5678:  } // switch Inst[27:25]
                                             //  5678: }
  8, 202, 14,                                //  5682: case 0x8: {
  OPC_SwitchField, 24, 4,                    //  5685:  switch Inst[27:24] {
  0, 227, 2,                                 //  5688:  case 0x0: {
  OPC_SwitchField, 5, 3,                     //  5691:   switch Inst[7:5] {
  0, 49,                                     //  5694:   case 0x0: {
  OPC_SwitchField, 21, 3,                    //  5696:    switch Inst[23:21] {
  0, 4,                                      //  5699:    case 0x0: {
  OPC_Decode, 239, 16, 109,                  //  5701:     decode to S2_asr_i_p using decoder 109
                                             //  5701:    }
  1, 8,                                      //  5705:    case 0x1: {
  OPC_CheckField, 12, 2, 0,                  //  5707:     check Inst[13:12] == 0x0
  OPC_Decode, 134, 20, 110,                  //  5711:     decode to S5_vasrhrnd using decoder 110
                                             //  5711:    }
  2, 8,                                      //  5715:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  5717:     check Inst[13] == 0x0
  OPC_Decode, 253, 16, 111,                  //  5721:     decode to S2_asr_i_vw using decoder 111
                                             //  5721:    }
  4, 8,                                      //  5725:    case 0x4: {
  OPC_CheckField, 12, 2, 0,                  //  5727:     check Inst[13:12] == 0x0
  OPC_Decode, 252, 16, 110,                  //  5731:     decode to S2_asr_i_vh using decoder 110
                                             //  5731:    }
  7, 0,                                      //  5735:    case 0x7: {
  OPC_CheckField, 8, 6, 0,                   //  5737:     check Inst[13:8] == 0x0
  OPC_Decode, 253, 9, 112,                   //  5741:     decode to F2_conv_df2d using decoder 112
                                             //  5741:    }
                                             //  5741:    } // switch Inst[23:21]
                                             //  5741:   }
  1, 39,                                     //  5745:   case 0x1: {
  OPC_SwitchField, 21, 3,                    //  5747:    switch Inst[23:21] {
  0, 4,                                      //  5750:    case 0x0: {
  OPC_Decode, 181, 17, 109,                  //  5752:     decode to S2_lsr_i_p using decoder 109
                                             //  5752:    }
  2, 8,                                      //  5756:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  5758:     check Inst[13] == 0x0
  OPC_Decode, 194, 17, 111,                  //  5762:     decode to S2_lsr_i_vw using decoder 111
                                             //  5762:    }
  4, 8,                                      //  5766:    case 0x4: {
  OPC_CheckField, 12, 2, 0,                  //  5768:     check Inst[13:12] == 0x0
  OPC_Decode, 193, 17, 110,                  //  5772:     decode to S2_lsr_i_vh using decoder 110
                                             //  5772:    }
  7, 0,                                      //  5776:    case 0x7: {
  OPC_CheckField, 8, 6, 0,                   //  5778:     check Inst[13:8] == 0x0
  OPC_Decode, 128, 10, 112,                  //  5782:     decode to F2_conv_df2ud using decoder 112
                                             //  5782:    }
                                             //  5782:    } // switch Inst[23:21]
                                             //  5782:   }
  2, 39,                                     //  5786:   case 0x2: {
  OPC_SwitchField, 21, 3,                    //  5788:    switch Inst[23:21] {
  0, 4,                                      //  5791:    case 0x0: {
  OPC_Decode, 210, 16, 109,                  //  5793:     decode to S2_asl_i_p using decoder 109
                                             //  5793:    }
  2, 8,                                      //  5797:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  5799:     check Inst[13] == 0x0
  OPC_Decode, 224, 16, 111,                  //  5803:     decode to S2_asl_i_vw using decoder 111
                                             //  5803:    }
  4, 8,                                      //  5807:    case 0x4: {
  OPC_CheckField, 12, 2, 0,                  //  5809:     check Inst[13:12] == 0x0
  OPC_Decode, 223, 16, 110,                  //  5813:     decode to S2_asl_i_vh using decoder 110
                                             //  5813:    }
  7, 0,                                      //  5817:    case 0x7: {
  OPC_CheckField, 8, 6, 0,                   //  5819:     check Inst[13:8] == 0x0
  OPC_Decode, 143, 10, 112,                  //  5823:     decode to F2_conv_ud2df using decoder 112
                                             //  5823:    }
                                             //  5823:    } // switch Inst[23:21]
                                             //  5823:   }
  3, 21,                                     //  5827:   case 0x3: {
  OPC_SwitchField, 21, 3,                    //  5829:    switch Inst[23:21] {
  0, 6,                                      //  5832:    case 0x0: {
  OPC_CheckPredicate, 1,                     //  5834:     check predicate 1
  OPC_Decode, 135, 20, 109,                  //  5836:     decode to S6_rol_i_p using decoder 109
                                             //  5836:    }
  7, 0,                                      //  5840:    case 0x7: {
  OPC_CheckField, 8, 6, 0,                   //  5842:     check Inst[13:8] == 0x0
  OPC_Decode, 251, 9, 112,                   //  5846:     decode to F2_conv_d2df using decoder 112
                                             //  5846:    }
                                             //  5846:    } // switch Inst[23:21]
                                             //  5846:   }
  4, 43,                                     //  5850:   case 0x4: {
  OPC_SwitchField, 21, 3,                    //  5852:    switch Inst[23:21] {
  0, 8,                                      //  5855:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  5857:     check Inst[13:8] == 0x0
  OPC_Decode, 216, 18, 112,                  //  5861:     decode to S2_vsathub_nopack using decoder 112
                                             //  5861:    }
  2, 8,                                      //  5865:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  5867:     check Inst[13:8] == 0x0
  OPC_Decode, 147, 8, 112,                   //  5871:     decode to A2_vabsh using decoder 112
                                             //  5871:    }
  4, 8,                                      //  5875:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  5877:     check Inst[13:8] == 0x0
  OPC_Decode, 206, 7, 112,                   //  5881:     decode to A2_notp using decoder 112
                                             //  5881:    }
  6, 0,                                      //  5885:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  5887:     check Inst[13:8] == 0x0
  OPC_Decode, 157, 17, 112,                  //  5891:     decode to S2_deinterleave using decoder 112
                                             //  5891:    }
                                             //  5891:    } // switch Inst[23:21]
                                             //  5891:   }
  5, 43,                                     //  5895:   case 0x5: {
  OPC_SwitchField, 21, 3,                    //  5897:    switch Inst[23:21] {
  0, 8,                                      //  5900:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  5902:     check Inst[13:8] == 0x0
  OPC_Decode, 220, 18, 112,                  //  5906:     decode to S2_vsatwuh_nopack using decoder 112
                                             //  5906:    }
  2, 8,                                      //  5910:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  5912:     check Inst[13:8] == 0x0
  OPC_Decode, 148, 8, 112,                   //  5916:     decode to A2_vabshsat using decoder 112
                                             //  5916:    }
  4, 8,                                      //  5920:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  5922:     check Inst[13:8] == 0x0
  OPC_Decode, 203, 7, 112,                   //  5926:     decode to A2_negp using decoder 112
                                             //  5926:    }
  6, 0,                                      //  5930:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  5932:     check Inst[13:8] == 0x0
  OPC_Decode, 166, 17, 112,                  //  5936:     decode to S2_interleave using decoder 112
                                             //  5936:    }
                                             //  5936:    } // switch Inst[23:21]
                                             //  5936:   }
  6, 53,                                     //  5940:   case 0x6: {
  OPC_SwitchField, 21, 3,                    //  5942:    switch Inst[23:21] {
  0, 8,                                      //  5945:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  5947:     check Inst[13:8] == 0x0
  OPC_Decode, 218, 18, 112,                  //  5951:     decode to S2_vsatwh_nopack using decoder 112
                                             //  5951:    }
  2, 8,                                      //  5955:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  5957:     check Inst[13:8] == 0x0
  OPC_Decode, 149, 8, 112,                   //  5961:     decode to A2_vabsw using decoder 112
                                             //  5961:    }
  4, 8,                                      //  5965:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  5967:     check Inst[13:8] == 0x0
  OPC_Decode, 163, 7, 112,                   //  5971:     decode to A2_absp using decoder 112
                                             //  5971:    }
  6, 8,                                      //  5975:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  5977:     check Inst[13:8] == 0x0
  OPC_Decode, 142, 17, 112,                  //  5981:     decode to S2_brevp using decoder 112
                                             //  5981:    }
  7, 0,                                      //  5985:    case 0x7: {
  OPC_CheckField, 8, 6, 0,                   //  5987:     check Inst[13:8] == 0x0
  OPC_Decode, 254, 9, 112,                   //  5991:     decode to F2_conv_df2d_chop using decoder 112
                                             //  5991:    }
                                             //  5991:    } // switch Inst[23:21]
                                             //  5991:   }
  7, 0,                                      //  5995:   case 0x7: {
  OPC_SwitchField, 21, 3,                    //  5997:    switch Inst[23:21] {
  0, 8,                                      //  6000:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  6002:     check Inst[13:8] == 0x0
  OPC_Decode, 214, 18, 112,                  //  6006:     decode to S2_vsathb_nopack using decoder 112
                                             //  6006:    }
  2, 8,                                      //  6010:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  6012:     check Inst[13:8] == 0x0
  OPC_Decode, 150, 8, 112,                   //  6016:     decode to A2_vabswsat using decoder 112
                                             //  6016:    }
  4, 8,                                      //  6020:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  6022:     check Inst[13:8] == 0x0
  OPC_Decode, 178, 8, 112,                   //  6026:     decode to A2_vconj using decoder 112
                                             //  6026:    }
  6, 4,                                      //  6030:    case 0x6: {
  OPC_Decode, 244, 16, 109,                  //  6032:     decode to S2_asr_i_p_rnd using decoder 109
                                             //  6032:    }
  7, 0,                                      //  6036:    case 0x7: {
  OPC_CheckField, 8, 6, 0,                   //  6038:     check Inst[13:8] == 0x0
  OPC_Decode, 129, 10, 112,                  //  6042:     decode to F2_conv_df2ud_chop using decoder 112
                                             //  6042:    }
                                             //  6042:    } // switch Inst[23:21]
                                             //  6042:   }
                                             //  6042:   } // switch Inst[7:5]
                                             //  6042:  }
  1, 4,                                      //  6046:  case 0x1: {
  OPC_Decode, 160, 17, 113,                  //  6048:   decode to S2_extractup using decoder 113
                                             //  6048:  }
  2, 167, 1,                                 //  6052:  case 0x2: {
  OPC_SwitchField, 5, 3,                     //  6055:   switch Inst[7:5] {
  0, 15,                                     //  6058:   case 0x0: {
  OPC_SwitchField, 21, 3,                    //  6060:    switch Inst[23:21] {
  0, 4,                                      //  6063:    case 0x0: {
  OPC_Decode, 242, 16, 114,                  //  6065:     decode to S2_asr_i_p_nac using decoder 114
                                             //  6065:    }
  2, 0,                                      //  6069:    case 0x2: {
  OPC_Decode, 241, 16, 114,                  //  6071:     decode to S2_asr_i_p_and using decoder 114
                                             //  6071:    }
                                             //  6071:    } // switch Inst[23:21]
                                             //  6071:   }
  1, 21,                                     //  6075:   case 0x1: {
  OPC_SwitchField, 21, 3,                    //  6077:    switch Inst[23:21] {
  0, 4,                                      //  6080:    case 0x0: {
  OPC_Decode, 184, 17, 114,                  //  6082:     decode to S2_lsr_i_p_nac using decoder 114
                                             //  6082:    }
  2, 4,                                      //  6086:    case 0x2: {
  OPC_Decode, 183, 17, 114,                  //  6088:     decode to S2_lsr_i_p_and using decoder 114
                                             //  6088:    }
  4, 0,                                      //  6092:    case 0x4: {
  OPC_Decode, 186, 17, 114,                  //  6094:     decode to S2_lsr_i_p_xacc using decoder 114
                                             //  6094:    }
                                             //  6094:    } // switch Inst[23:21]
                                             //  6094:   }
  2, 21,                                     //  6098:   case 0x2: {
  OPC_SwitchField, 21, 3,                    //  6100:    switch Inst[23:21] {
  0, 4,                                      //  6103:    case 0x0: {
  OPC_Decode, 213, 16, 114,                  //  6105:     decode to S2_asl_i_p_nac using decoder 114
                                             //  6105:    }
  2, 4,                                      //  6109:    case 0x2: {
  OPC_Decode, 212, 16, 114,                  //  6111:     decode to S2_asl_i_p_and using decoder 114
                                             //  6111:    }
  4, 0,                                      //  6115:    case 0x4: {
  OPC_Decode, 215, 16, 114,                  //  6117:     decode to S2_asl_i_p_xacc using decoder 114
                                             //  6117:    }
                                             //  6117:    } // switch Inst[23:21]
                                             //  6117:   }
  3, 27,                                     //  6121:   case 0x3: {
  OPC_SwitchField, 21, 3,                    //  6123:    switch Inst[23:21] {
  0, 6,                                      //  6126:    case 0x0: {
  OPC_CheckPredicate, 1,                     //  6128:     check predicate 1
  OPC_Decode, 138, 20, 114,                  //  6130:     decode to S6_rol_i_p_nac using decoder 114
                                             //  6130:    }
  2, 6,                                      //  6134:    case 0x2: {
  OPC_CheckPredicate, 1,                     //  6136:     check predicate 1
  OPC_Decode, 137, 20, 114,                  //  6138:     decode to S6_rol_i_p_and using decoder 114
                                             //  6138:    }
  4, 0,                                      //  6142:    case 0x4: {
  OPC_CheckPredicate, 1,                     //  6144:     check predicate 1
  OPC_Decode, 140, 20, 114,                  //  6146:     decode to S6_rol_i_p_xacc using decoder 114
                                             //  6146:    }
                                             //  6146:    } // switch Inst[23:21]
                                             //  6146:   }
  4, 15,                                     //  6150:   case 0x4: {
  OPC_SwitchField, 21, 3,                    //  6152:    switch Inst[23:21] {
  0, 4,                                      //  6155:    case 0x0: {
  OPC_Decode, 240, 16, 114,                  //  6157:     decode to S2_asr_i_p_acc using decoder 114
                                             //  6157:    }
  2, 0,                                      //  6161:    case 0x2: {
  OPC_Decode, 243, 16, 114,                  //  6163:     decode to S2_asr_i_p_or using decoder 114
                                             //  6163:    }
                                             //  6163:    } // switch Inst[23:21]
                                             //  6163:   }
  5, 15,                                     //  6167:   case 0x5: {
  OPC_SwitchField, 21, 3,                    //  6169:    switch Inst[23:21] {
  0, 4,                                      //  6172:    case 0x0: {
  OPC_Decode, 182, 17, 114,                  //  6174:     decode to S2_lsr_i_p_acc using decoder 114
                                             //  6174:    }
  2, 0,                                      //  6178:    case 0x2: {
  OPC_Decode, 185, 17, 114,                  //  6180:     decode to S2_lsr_i_p_or using decoder 114
                                             //  6180:    }
                                             //  6180:    } // switch Inst[23:21]
                                             //  6180:   }
  6, 15,                                     //  6184:   case 0x6: {
  OPC_SwitchField, 21, 3,                    //  6186:    switch Inst[23:21] {
  0, 4,                                      //  6189:    case 0x0: {
  OPC_Decode, 211, 16, 114,                  //  6191:     decode to S2_asl_i_p_acc using decoder 114
                                             //  6191:    }
  2, 0,                                      //  6195:    case 0x2: {
  OPC_Decode, 214, 16, 114,                  //  6197:     decode to S2_asl_i_p_or using decoder 114
                                             //  6197:    }
                                             //  6197:    } // switch Inst[23:21]
                                             //  6197:   }
  7, 0,                                      //  6201:   case 0x7: {
  OPC_SwitchField, 21, 3,                    //  6203:    switch Inst[23:21] {
  0, 6,                                      //  6206:    case 0x0: {
  OPC_CheckPredicate, 1,                     //  6208:     check predicate 1
  OPC_Decode, 136, 20, 114,                  //  6210:     decode to S6_rol_i_p_acc using decoder 114
                                             //  6210:    }
  2, 0,                                      //  6214:    case 0x2: {
  OPC_CheckPredicate, 1,                     //  6216:     check predicate 1
  OPC_Decode, 139, 20, 114,                  //  6218:     decode to S6_rol_i_p_or using decoder 114
                                             //  6218:    }
                                             //  6218:    } // switch Inst[23:21]
                                             //  6218:   }
                                             //  6218:   } // switch Inst[7:5]
                                             //  6218:  }
  3, 4,                                      //  6222:  case 0x3: {
  OPC_Decode, 164, 17, 115,                  //  6224:   decode to S2_insertp using decoder 115
                                             //  6224:  }
  4, 121,                                    //  6228:  case 0x4: {
  OPC_SwitchField, 5, 9,                     //  6230:   switch Inst[13:5] {
  0, 21,                                     //  6233:   case 0x0: {
  OPC_SwitchField, 21, 3,                    //  6235:    switch Inst[23:21] {
  0, 4,                                      //  6238:    case 0x0: {
  OPC_Decode, 225, 18, 86,                   //  6240:     decode to S2_vsxtbh using decoder 86
                                             //  6240:    }
  2, 4,                                      //  6244:    case 0x2: {
  OPC_Decode, 140, 8, 86,                    //  6246:     decode to A2_sxtw using decoder 86
                                             //  6246:    }
  4, 0,                                      //  6250:    case 0x4: {
  OPC_Decode, 136, 10, 86,                   //  6252:     decode to F2_conv_sf2df using decoder 86
                                             //  6252:    }
                                             //  6252:    } // switch Inst[23:21]
                                             //  6252:   }
  1, 8,                                      //  6256:   case 0x1: {
  OPC_CheckField, 21, 3, 4,                  //  6258:    check Inst[23:21] == 0x4
  OPC_Decode, 145, 10, 86,                   //  6262:    decode to F2_conv_uw2df using decoder 86
                                             //  6262:   }
  2, 21,                                     //  6266:   case 0x2: {
  OPC_SwitchField, 21, 3,                    //  6268:    switch Inst[23:21] {
  0, 4,                                      //  6271:    case 0x0: {
  OPC_Decode, 231, 18, 86,                   //  6273:     decode to S2_vzxtbh using decoder 86
                                             //  6273:    }
  2, 4,                                      //  6277:    case 0x2: {
  OPC_Decode, 222, 18, 86,                   //  6279:     decode to S2_vsplatrh using decoder 86
                                             //  6279:    }
  4, 0,                                      //  6283:    case 0x4: {
  OPC_Decode, 147, 10, 86,                   //  6285:     decode to F2_conv_w2df using decoder 86
                                             //  6285:    }
                                             //  6285:    } // switch Inst[23:21]
                                             //  6285:   }
  3, 8,                                      //  6289:   case 0x3: {
  OPC_CheckField, 21, 3, 4,                  //  6291:    check Inst[23:21] == 0x4
  OPC_Decode, 137, 10, 86,                   //  6295:    decode to F2_conv_sf2ud using decoder 86
                                             //  6295:   }
  4, 23,                                     //  6299:   case 0x4: {
  OPC_SwitchField, 21, 3,                    //  6301:    switch Inst[23:21] {
  0, 4,                                      //  6304:    case 0x0: {
  OPC_Decode, 226, 18, 86,                   //  6306:     decode to S2_vsxthw using decoder 86
                                             //  6306:    }
  2, 6,                                      //  6310:    case 0x2: {
  OPC_CheckPredicate, 7,                     //  6312:     check predicate 7
  OPC_Decode, 147, 20, 86,                   //  6314:     decode to S6_vsplatrbp using decoder 86
                                             //  6314:    }
  4, 0,                                      //  6318:    case 0x4: {
  OPC_Decode, 134, 10, 86,                   //  6320:     decode to F2_conv_sf2d using decoder 86
                                             //  6320:    }
                                             //  6320:    } // switch Inst[23:21]
                                             //  6320:   }
  5, 8,                                      //  6324:   case 0x5: {
  OPC_CheckField, 21, 3, 4,                  //  6326:    check Inst[23:21] == 0x4
  OPC_Decode, 138, 10, 86,                   //  6330:    decode to F2_conv_sf2ud_chop using decoder 86
                                             //  6330:   }
  6, 0,                                      //  6334:   case 0x6: {
  OPC_SwitchField, 21, 3,                    //  6336:    switch Inst[23:21] {
  0, 4,                                      //  6339:    case 0x0: {
  OPC_Decode, 232, 18, 86,                   //  6341:     decode to S2_vzxthw using decoder 86
                                             //  6341:    }
  4, 0,                                      //  6345:    case 0x4: {
  OPC_Decode, 135, 10, 86,                   //  6347:     decode to F2_conv_sf2d_chop using decoder 86
                                             //  6347:    }
                                             //  6347:    } // switch Inst[23:21]
                                             //  6347:   }
                                             //  6347:   } // switch Inst[13:5]
                                             //  6347:  }
  5, 75,                                     //  6351:  case 0x5: {
  OPC_SwitchField, 21, 3,                    //  6353:   switch Inst[23:21] {
  0, 12,                                     //  6356:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  6358:    check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  6362:    check Inst[7:2] == 0x0
  OPC_Decode, 204, 18, 116,                  //  6366:    decode to S2_tstbit_i using decoder 116
                                             //  6366:   }
  1, 12,                                     //  6370:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  6372:    check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  6376:    check Inst[7:2] == 0x0
  OPC_Decode, 246, 18, 116,                  //  6380:    decode to S4_ntstbit_i using decoder 116
                                             //  6380:   }
  2, 8,                                      //  6384:   case 0x2: {
  OPC_CheckField, 2, 12, 0,                  //  6386:    check Inst[13:2] == 0x0
  OPC_Decode, 207, 9, 117,                   //  6390:    decode to C2_tfrrp using decoder 117
                                             //  6390:   }
  4, 8,                                      //  6394:   case 0x4: {
  OPC_CheckField, 2, 6, 0,                   //  6396:    check Inst[7:2] == 0x0
  OPC_Decode, 179, 9, 118,                   //  6400:    decode to C2_bitsclri using decoder 118
                                             //  6400:   }
  5, 8,                                      //  6404:   case 0x5: {
  OPC_CheckField, 2, 6, 0,                   //  6406:    check Inst[7:2] == 0x0
  OPC_Decode, 225, 9, 118,                   //  6410:    decode to C4_nbitsclri using decoder 118
                                             //  6410:   }
  7, 0,                                      //  6414:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  //  6416:    check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   //  6420:    check Inst[7:2] == 0x0
  OPC_Decode, 165, 10, 116,                  //  6424:    decode to F2_sfclass using decoder 116
                                             //  6424:   }
                                             //  6424:   } // switch Inst[23:21]
                                             //  6424:  }
  6, 16,                                     //  6428:  case 0x6: {
  OPC_CheckField, 16, 8, 0,                  //  6430:   check Inst[23:16] == 0x0
  OPC_CheckField, 10, 4, 0,                  //  6434:   check Inst[13:10] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  6438:   check Inst[7:5] == 0x0
  OPC_Decode, 198, 9, 119,                   //  6442:   decode to C2_mask using decoder 119
                                             //  6442:  }
  7, 27,                                     //  6446:  case 0x7: {
  OPC_SwitchField, 22, 2,                    //  6448:   switch Inst[23:22] {
  0, 4,                                      //  6451:   case 0x0: {
  OPC_Decode, 198, 18, 120,                  //  6453:    decode to S2_tableidxb using decoder 120
                                             //  6453:   }
  1, 4,                                      //  6457:   case 0x1: {
  OPC_Decode, 200, 18, 120,                  //  6459:    decode to S2_tableidxh using decoder 120
                                             //  6459:   }
  2, 4,                                      //  6463:   case 0x2: {
  OPC_Decode, 201, 18, 120,                  //  6465:    decode to S2_tableidxw using decoder 120
                                             //  6465:   }
  3, 0,                                      //  6469:   case 0x3: {
  OPC_Decode, 199, 18, 120,                  //  6471:    decode to S2_tableidxd using decoder 120
                                             //  6471:   }
                                             //  6471:   } // switch Inst[23:22]
                                             //  6471:  }
  8, 147, 2,                                 //  6475:  case 0x8: {
  OPC_SwitchField, 21, 3,                    //  6478:   switch Inst[23:21] {
  0, 33,                                     //  6481:   case 0x0: {
  OPC_SwitchField, 5, 9,                     //  6483:    switch Inst[13:5] {
  0, 4,                                      //  6486:    case 0x0: {
  OPC_Decode, 215, 18, 87,                   //  6488:     decode to S2_vsathub using decoder 87
                                             //  6488:    }
  1, 4,                                      //  6492:    case 0x1: {
  OPC_Decode, 255, 9, 87,                    //  6494:     decode to F2_conv_df2sf using decoder 87
                                             //  6494:    }
  2, 4,                                      //  6498:    case 0x2: {
  OPC_Decode, 217, 18, 87,                   //  6500:     decode to S2_vsatwh using decoder 87
                                             //  6500:    }
  4, 4,                                      //  6504:    case 0x4: {
  OPC_Decode, 219, 18, 87,                   //  6506:     decode to S2_vsatwuh using decoder 87
                                             //  6506:    }
  6, 0,                                      //  6510:    case 0x6: {
  OPC_Decode, 213, 18, 87,                   //  6512:     decode to S2_vsathb using decoder 87
                                             //  6512:    }
                                             //  6512:    } // switch Inst[13:5]
                                             //  6512:   }
  1, 8,                                      //  6516:   case 0x1: {
  OPC_CheckField, 5, 9, 1,                   //  6518:    check Inst[13:5] == 0x1
  OPC_Decode, 144, 10, 87,                   //  6522:    decode to F2_conv_ud2sf using decoder 87
                                             //  6522:   }
  2, 27,                                     //  6526:   case 0x2: {
  OPC_SwitchField, 5, 9,                     //  6528:    switch Inst[13:5] {
  0, 4,                                      //  6531:    case 0x0: {
  OPC_Decode, 150, 17, 87,                   //  6533:     decode to S2_clbp using decoder 87
                                             //  6533:    }
  1, 4,                                      //  6537:    case 0x1: {
  OPC_Decode, 252, 9, 87,                    //  6539:     decode to F2_conv_d2sf using decoder 87
                                             //  6539:    }
  2, 4,                                      //  6543:    case 0x2: {
  OPC_Decode, 145, 17, 87,                   //  6545:     decode to S2_cl0p using decoder 87
                                             //  6545:    }
  4, 0,                                      //  6549:    case 0x4: {
  OPC_Decode, 147, 17, 87,                   //  6551:     decode to S2_cl1p using decoder 87
                                             //  6551:    }
                                             //  6551:    } // switch Inst[13:5]
                                             //  6551:   }
  3, 59,                                     //  6555:   case 0x3: {
  OPC_SwitchField, 5, 3,                     //  6557:    switch Inst[7:5] {
  0, 8,                                      //  6560:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  6562:     check Inst[13:8] == 0x0
  OPC_Decode, 240, 18, 87,                   //  6566:     decode to S4_clbpnorm using decoder 87
                                             //  6566:    }
  1, 8,                                      //  6570:    case 0x1: {
  OPC_CheckField, 8, 6, 0,                   //  6572:     check Inst[13:8] == 0x0
  OPC_Decode, 130, 10, 87,                   //  6576:     decode to F2_conv_df2uw using decoder 87
                                             //  6576:    }
  2, 4,                                      //  6580:    case 0x2: {
  OPC_Decode, 239, 18, 121,                  //  6582:     decode to S4_clbpaddi using decoder 121
                                             //  6582:    }
  3, 8,                                      //  6586:    case 0x3: {
  OPC_CheckField, 8, 6, 0,                   //  6588:     check Inst[13:8] == 0x0
  OPC_Decode, 133, 20, 87,                   //  6592:     decode to S5_popcountp using decoder 87
                                             //  6592:    }
  4, 8,                                      //  6596:    case 0x4: {
  OPC_CheckField, 12, 2, 0,                  //  6598:     check Inst[13:12] == 0x0
  OPC_Decode, 131, 20, 122,                  //  6602:     decode to S5_asrhub_rnd_sat using decoder 122
                                             //  6602:    }
  5, 0,                                      //  6606:    case 0x5: {
  OPC_CheckField, 12, 2, 0,                  //  6608:     check Inst[13:12] == 0x0
  OPC_Decode, 132, 20, 122,                  //  6612:     decode to S5_asrhub_sat using decoder 122
                                             //  6612:    }
                                             //  6612:    } // switch Inst[7:5]
                                             //  6612:   }
  4, 33,                                     //  6616:   case 0x4: {
  OPC_SwitchField, 5, 9,                     //  6618:    switch Inst[13:5] {
  0, 4,                                      //  6621:    case 0x0: {
  OPC_Decode, 229, 18, 87,                   //  6623:     decode to S2_vtrunohb using decoder 87
                                             //  6623:    }
  1, 4,                                      //  6627:    case 0x1: {
  OPC_Decode, 132, 10, 87,                   //  6629:     decode to F2_conv_df2w using decoder 87
                                             //  6629:    }
  2, 4,                                      //  6633:    case 0x2: {
  OPC_Decode, 227, 18, 87,                   //  6635:     decode to S2_vtrunehb using decoder 87
                                             //  6635:    }
  4, 4,                                      //  6639:    case 0x4: {
  OPC_Decode, 211, 18, 87,                   //  6641:     decode to S2_vrndpackwh using decoder 87
                                             //  6641:    }
  6, 0,                                      //  6645:    case 0x6: {
  OPC_Decode, 212, 18, 87,                   //  6647:     decode to S2_vrndpackwhs using decoder 87
                                             //  6647:    }
                                             //  6647:    } // switch Inst[13:5]
                                             //  6647:   }
  5, 8,                                      //  6651:   case 0x5: {
  OPC_CheckField, 5, 9, 1,                   //  6653:    check Inst[13:5] == 0x1
  OPC_Decode, 131, 10, 87,                   //  6657:    decode to F2_conv_df2uw_chop using decoder 87
                                             //  6657:   }
  6, 67,                                     //  6661:   case 0x6: {
  OPC_SwitchField, 5, 3,                     //  6663:    switch Inst[7:5] {
  0, 8,                                      //  6666:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  6668:     check Inst[13:8] == 0x0
  OPC_Decode, 235, 7, 87,                    //  6672:     decode to A2_sat using decoder 87
                                             //  6672:    }
  1, 8,                                      //  6676:    case 0x1: {
  OPC_CheckField, 8, 6, 0,                   //  6678:     check Inst[13:8] == 0x0
  OPC_Decode, 234, 7, 87,                    //  6682:     decode to A2_roundsat using decoder 87
                                             //  6682:    }
  2, 8,                                      //  6686:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  6688:     check Inst[13] == 0x0
  OPC_Decode, 251, 16, 123,                  //  6692:     decode to S2_asr_i_svw_trun using decoder 123
                                             //  6692:    }
  4, 8,                                      //  6696:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  //  6698:     check Inst[13] == 0x0
  OPC_Decode, 215, 8, 124,                   //  6702:     decode to A4_bitspliti using decoder 124
                                             //  6702:    }
  5, 10,                                     //  6706:    case 0x5: {
  OPC_CheckPredicate, 8,                     //  6708:     check predicate 8
  OPC_CheckField, 13, 1, 0,                  //  6710:     check Inst[13] == 0x0
  OPC_Decode, 170, 9, 125,                   //  6714:     decode to A7_clip using decoder 125
                                             //  6714:    }
  6, 0,                                      //  6718:    case 0x6: {
  OPC_CheckPredicate, 8,                     //  6720:     check predicate 8
  OPC_CheckField, 13, 1, 0,                  //  6722:     check Inst[13] == 0x0
  OPC_Decode, 173, 9, 111,                   //  6726:     decode to A7_vclip using decoder 111
                                             //  6726:    }
                                             //  6726:    } // switch Inst[7:5]
                                             //  6726:   }
  7, 0,                                      //  6730:   case 0x7: {
  OPC_SwitchField, 5, 9,                     //  6732:    switch Inst[13:5] {
  1, 4,                                      //  6735:    case 0x1: {
  OPC_Decode, 133, 10, 87,                   //  6737:     decode to F2_conv_df2w_chop using decoder 87
                                             //  6737:    }
  2, 4,                                      //  6741:    case 0x2: {
  OPC_Decode, 154, 17, 87,                   //  6743:     decode to S2_ct0p using decoder 87
                                             //  6743:    }
  4, 0,                                      //  6747:    case 0x4: {
  OPC_Decode, 156, 17, 87,                   //  6749:     decode to S2_ct1p using decoder 87
                                             //  6749:    }
                                             //  6749:    } // switch Inst[13:5]
                                             //  6749:   }
                                             //  6749:   } // switch Inst[23:21]
                                             //  6749:  }
  9, 27,                                     //  6753:  case 0x9: {
  OPC_SwitchField, 18, 6,                    //  6755:   switch Inst[23:18] {
  0, 12,                                     //  6758:   case 0x0: {
  OPC_CheckField, 10, 4, 0,                  //  6760:    check Inst[13:10] == 0x0
  OPC_CheckField, 5, 3, 0,                   //  6764:    check Inst[7:5] == 0x0
  OPC_Decode, 208, 9, 126,                   //  6768:    decode to C2_vitpack using decoder 126
                                             //  6768:   }
  16, 0,                                     //  6772:   case 0x10: {
  OPC_CheckField, 5, 9, 0,                   //  6774:    check Inst[13:5] == 0x0
  OPC_Decode, 206, 9, 127,                   //  6778:    decode to C2_tfrpr using decoder 127
                                             //  6778:   }
                                             //  6778:   } // switch Inst[23:18]
                                             //  6778:  }
  10, 4,                                     //  6782:  case 0xa: {
  OPC_Decode, 243, 18, 113,                  //  6784:   decode to S4_extractp using decoder 113
                                             //  6784:  }
  11, 78,                                    //  6788:  case 0xb: {
  OPC_SwitchField, 21, 3,                    //  6790:   switch Inst[23:21] {
  1, 8,                                      //  6793:   case 0x1: {
  OPC_CheckField, 5, 9, 0,                   //  6795:    check Inst[13:5] == 0x0
  OPC_Decode, 146, 10, 61,                   //  6799:    decode to F2_conv_uw2sf using decoder 61
                                             //  6799:   }
  2, 8,                                      //  6803:   case 0x2: {
  OPC_CheckField, 5, 9, 0,                   //  6805:    check Inst[13:5] == 0x0
  OPC_Decode, 148, 10, 61,                   //  6809:    decode to F2_conv_w2sf using decoder 61
                                             //  6809:   }
  3, 15,                                     //  6813:   case 0x3: {
  OPC_SwitchField, 5, 9,                     //  6815:    switch Inst[13:5] {
  0, 4,                                      //  6818:    case 0x0: {
  OPC_Decode, 139, 10, 61,                   //  6820:     decode to F2_conv_sf2uw using decoder 61
                                             //  6820:    }
  1, 0,                                      //  6824:    case 0x1: {
  OPC_Decode, 140, 10, 61,                   //  6826:     decode to F2_conv_sf2uw_chop using decoder 61
                                             //  6826:    }
                                             //  6826:    } // switch Inst[13:5]
                                             //  6826:   }
  4, 15,                                     //  6830:   case 0x4: {
  OPC_SwitchField, 5, 9,                     //  6832:    switch Inst[13:5] {
  0, 4,                                      //  6835:    case 0x0: {
  OPC_Decode, 141, 10, 61,                   //  6837:     decode to F2_conv_sf2w using decoder 61
                                             //  6837:    }
  1, 0,                                      //  6841:    case 0x1: {
  OPC_Decode, 142, 10, 61,                   //  6843:     decode to F2_conv_sf2w_chop using decoder 61
                                             //  6843:    }
                                             //  6843:    } // switch Inst[13:5]
                                             //  6843:   }
  5, 8,                                      //  6847:   case 0x5: {
  OPC_CheckField, 5, 9, 0,                   //  6849:    check Inst[13:5] == 0x0
  OPC_Decode, 172, 10, 61,                   //  6853:    decode to F2_sffixupr using decoder 61
                                             //  6853:   }
  7, 0,                                      //  6857:   case 0x7: {
  OPC_CheckField, 7, 7, 0,                   //  6859:    check Inst[13:7] == 0x0
  OPC_Decode, 180, 10, 128, 1,               //  6863:    decode to F2_sfinvsqrta using decoder 128
                                             //  6863:   }
                                             //  6863:   } // switch Inst[23:21]
                                             //  6863:  }
  12, 231, 2,                                //  6868:  case 0xc: {
  OPC_SwitchField, 5, 3,                     //  6871:   switch Inst[7:5] {
  0, 60,                                     //  6874:   case 0x0: {
  OPC_SwitchField, 21, 3,                    //  6876:    switch Inst[23:21] {
  0, 8,                                      //  6879:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  6881:     check Inst[13] == 0x0
  OPC_Decode, 245, 16, 125,                  //  6885:     decode to S2_asr_i_r using decoder 125
                                             //  6885:    }
  1, 5,                                      //  6889:    case 0x1: {
  OPC_Decode, 238, 18, 129, 1,               //  6891:     decode to S4_clbaddi using decoder 129
                                             //  6891:    }
  2, 8,                                      //  6896:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  6898:     check Inst[13] == 0x0
  OPC_Decode, 250, 16, 125,                  //  6902:     decode to S2_asr_i_r_rnd using decoder 125
                                             //  6902:    }
  4, 8,                                      //  6906:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  6908:     check Inst[13:8] == 0x0
  OPC_Decode, 196, 18, 61,                   //  6912:     decode to S2_svsathb using decoder 61
                                             //  6912:    }
  6, 8,                                      //  6916:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  //  6918:     check Inst[13] == 0x0
  OPC_Decode, 131, 18, 125,                  //  6922:     decode to S2_setbit_i using decoder 125
                                             //  6922:    }
  7, 0,                                      //  6926:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  //  6928:     check Inst[13] == 0x0
  OPC_Decode, 233, 8, 125,                   //  6932:     decode to A4_cround_ri using decoder 125
                                             //  6932:    }
                                             //  6932:    } // switch Inst[23:21]
                                             //  6932:   }
  1, 23,                                     //  6936:   case 0x1: {
  OPC_SwitchField, 21, 3,                    //  6938:    switch Inst[23:21] {
  0, 8,                                      //  6941:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  6943:     check Inst[13] == 0x0
  OPC_Decode, 187, 17, 125,                  //  6947:     decode to S2_lsr_i_r using decoder 125
                                             //  6947:    }
  6, 0,                                      //  6951:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  //  6953:     check Inst[13] == 0x0
  OPC_Decode, 151, 17, 125,                  //  6957:     decode to S2_clrbit_i using decoder 125
                                             //  6957:    }
                                             //  6957:    } // switch Inst[23:21]
                                             //  6957:   }
  2, 51,                                     //  6961:   case 0x2: {
  OPC_SwitchField, 21, 3,                    //  6963:    switch Inst[23:21] {
  0, 8,                                      //  6966:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  6968:     check Inst[13] == 0x0
  OPC_Decode, 216, 16, 125,                  //  6972:     decode to S2_asl_i_r using decoder 125
                                             //  6972:    }
  2, 8,                                      //  6976:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  6978:     check Inst[13] == 0x0
  OPC_Decode, 221, 16, 125,                  //  6982:     decode to S2_asl_i_r_sat using decoder 125
                                             //  6982:    }
  4, 8,                                      //  6986:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  6988:     check Inst[13:8] == 0x0
  OPC_Decode, 197, 18, 61,                   //  6992:     decode to S2_svsathub using decoder 61
                                             //  6992:    }
  6, 8,                                      //  6996:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  //  6998:     check Inst[13] == 0x0
  OPC_Decode, 202, 18, 125,                  //  7002:     decode to S2_togglebit_i using decoder 125
                                             //  7002:    }
  7, 0,                                      //  7006:    case 0x7: {
  OPC_CheckPredicate, 8,                     //  7008:     check predicate 8
  OPC_Decode, 171, 9, 109,                   //  7010:     decode to A7_croundd_ri using decoder 109
                                             //  7010:    }
                                             //  7010:    } // switch Inst[23:21]
                                             //  7010:   }
  3, 14,                                     //  7014:   case 0x3: {
  OPC_CheckPredicate, 1,                     //  7016:    check predicate 1
  OPC_CheckField, 21, 3, 0,                  //  7018:    check Inst[23:21] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  7022:    check Inst[13] == 0x0
  OPC_Decode, 141, 20, 125,                  //  7026:    decode to S6_rol_i_r using decoder 125
                                             //  7026:   }
  4, 53,                                     //  7030:   case 0x4: {
  OPC_SwitchField, 21, 3,                    //  7032:    switch Inst[23:21] {
  0, 8,                                      //  7035:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  7037:     check Inst[13:8] == 0x0
  OPC_Decode, 148, 17, 61,                   //  7041:     decode to S2_clb using decoder 61
                                             //  7041:    }
  2, 8,                                      //  7045:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  7047:     check Inst[13:8] == 0x0
  OPC_Decode, 153, 17, 61,                   //  7051:     decode to S2_ct0 using decoder 61
                                             //  7051:    }
  4, 8,                                      //  7055:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  7057:     check Inst[13:8] == 0x0
  OPC_Decode, 162, 7, 61,                    //  7061:     decode to A2_abs using decoder 61
                                             //  7061:    }
  6, 8,                                      //  7065:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  7067:     check Inst[13:8] == 0x0
  OPC_Decode, 237, 7, 61,                    //  7071:     decode to A2_sath using decoder 61
                                             //  7071:    }
  7, 0,                                      //  7075:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  //  7077:     check Inst[13] == 0x0
  OPC_Decode, 139, 9, 125,                   //  7081:     decode to A4_round_ri using decoder 125
                                             //  7081:    }
                                             //  7081:    } // switch Inst[23:21]
                                             //  7081:   }
  5, 43,                                     //  7085:   case 0x5: {
  OPC_SwitchField, 21, 3,                    //  7087:    switch Inst[23:21] {
  0, 8,                                      //  7090:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  7092:     check Inst[13:8] == 0x0
  OPC_Decode, 144, 17, 61,                   //  7096:     decode to S2_cl0 using decoder 61
                                             //  7096:    }
  2, 8,                                      //  7100:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  7102:     check Inst[13:8] == 0x0
  OPC_Decode, 155, 17, 61,                   //  7106:     decode to S2_ct1 using decoder 61
                                             //  7106:    }
  4, 8,                                      //  7110:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  7112:     check Inst[13:8] == 0x0
  OPC_Decode, 164, 7, 61,                    //  7116:     decode to A2_abssat using decoder 61
                                             //  7116:    }
  6, 0,                                      //  7120:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  7122:     check Inst[13:8] == 0x0
  OPC_Decode, 239, 7, 61,                    //  7126:     decode to A2_satuh using decoder 61
                                             //  7126:    }
                                             //  7126:    } // switch Inst[23:21]
                                             //  7126:   }
  6, 53,                                     //  7130:   case 0x6: {
  OPC_SwitchField, 21, 3,                    //  7132:    switch Inst[23:21] {
  0, 8,                                      //  7135:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  7137:     check Inst[13:8] == 0x0
  OPC_Decode, 146, 17, 61,                   //  7141:     decode to S2_cl1 using decoder 61
                                             //  7141:    }
  2, 8,                                      //  7145:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  7147:     check Inst[13:8] == 0x0
  OPC_Decode, 141, 17, 61,                   //  7151:     decode to S2_brev using decoder 61
                                             //  7151:    }
  4, 8,                                      //  7155:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  7157:     check Inst[13:8] == 0x0
  OPC_Decode, 204, 7, 61,                    //  7161:     decode to A2_negsat using decoder 61
                                             //  7161:    }
  6, 8,                                      //  7165:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  7167:     check Inst[13:8] == 0x0
  OPC_Decode, 238, 7, 61,                    //  7171:     decode to A2_satub using decoder 61
                                             //  7171:    }
  7, 0,                                      //  7175:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  //  7177:     check Inst[13] == 0x0
  OPC_Decode, 140, 9, 125,                   //  7181:     decode to A4_round_ri_sat using decoder 125
                                             //  7181:    }
                                             //  7181:    } // switch Inst[23:21]
                                             //  7181:   }
  7, 0,                                      //  7185:   case 0x7: {
  OPC_SwitchField, 21, 3,                    //  7187:    switch Inst[23:21] {
  0, 8,                                      //  7190:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  7192:     check Inst[13:8] == 0x0
  OPC_Decode, 149, 17, 61,                   //  7196:     decode to S2_clbnorm using decoder 61
                                             //  7196:    }
  2, 8,                                      //  7200:    case 0x2: {
  OPC_CheckField, 8, 6, 0,                   //  7202:     check Inst[13:8] == 0x0
  OPC_Decode, 221, 18, 61,                   //  7206:     decode to S2_vsplatrb using decoder 61
                                             //  7206:    }
  4, 8,                                      //  7210:    case 0x4: {
  OPC_CheckField, 8, 6, 0,                   //  7212:     check Inst[13:8] == 0x0
  OPC_Decode, 137, 8, 61,                    //  7216:     decode to A2_swiz using decoder 61
                                             //  7216:    }
  6, 0,                                      //  7220:    case 0x6: {
  OPC_CheckField, 8, 6, 0,                   //  7222:     check Inst[13:8] == 0x0
  OPC_Decode, 236, 7, 61,                    //  7226:     decode to A2_satb using decoder 61
                                             //  7226:    }
                                             //  7226:    } // switch Inst[23:21]
                                             //  7226:   }
                                             //  7226:   } // switch Inst[7:5]
                                             //  7226:  }
  13, 39,                                    //  7230:  case 0xd: {
  OPC_SwitchField, 13, 1,                    //  7232:   switch Inst[13] {
  0, 17,                                     //  7235:   case 0x0: {
  OPC_SwitchField, 23, 1,                    //  7237:    switch Inst[23] {
  0, 5,                                      //  7240:    case 0x0: {
  OPC_Decode, 158, 17, 130, 1,               //  7242:     decode to S2_extractu using decoder 130
                                             //  7242:    }
  1, 0,                                      //  7247:    case 0x1: {
  OPC_Decode, 241, 18, 130, 1,               //  7249:     decode to S4_extract using decoder 130
                                             //  7249:    }
                                             //  7249:    } // switch Inst[23]
                                             //  7249:   }
  1, 0,                                      //  7254:   case 0x1: {
  OPC_CheckPredicate, 4,                     //  7256:    check predicate 4
  OPC_CheckField, 23, 1, 0,                  //  7258:    check Inst[23] == 0x0
  OPC_CheckField, 16, 5, 0,                  //  7262:    check Inst[20:16] == 0x0
  OPC_Decode, 208, 17, 131, 1,               //  7266:    decode to S2_mask using decoder 131
                                             //  7266:   }
                                             //  7266:   } // switch Inst[13]
                                             //  7266:  }
  14, 134, 2,                                //  7271:  case 0xe: {
  OPC_SwitchField, 5, 3,                     //  7274:   switch Inst[7:5] {
  0, 25,                                     //  7277:   case 0x0: {
  OPC_SwitchField, 21, 3,                    //  7279:    switch Inst[23:21] {
  0, 9,                                      //  7282:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  7284:     check Inst[13] == 0x0
  OPC_Decode, 248, 16, 132, 1,               //  7288:     decode to S2_asr_i_r_nac using decoder 132
                                             //  7288:    }
  2, 0,                                      //  7293:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  7295:     check Inst[13] == 0x0
  OPC_Decode, 247, 16, 132, 1,               //  7299:     decode to S2_asr_i_r_and using decoder 132
                                             //  7299:    }
                                             //  7299:    } // switch Inst[23:21]
                                             //  7299:   }
  1, 36,                                     //  7304:   case 0x1: {
  OPC_SwitchField, 21, 3,                    //  7306:    switch Inst[23:21] {
  0, 9,                                      //  7309:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  7311:     check Inst[13] == 0x0
  OPC_Decode, 190, 17, 132, 1,               //  7315:     decode to S2_lsr_i_r_nac using decoder 132
                                             //  7315:    }
  2, 9,                                      //  7320:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  7322:     check Inst[13] == 0x0
  OPC_Decode, 189, 17, 132, 1,               //  7326:     decode to S2_lsr_i_r_and using decoder 132
                                             //  7326:    }
  4, 0,                                      //  7331:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  //  7333:     check Inst[13] == 0x0
  OPC_Decode, 192, 17, 132, 1,               //  7337:     decode to S2_lsr_i_r_xacc using decoder 132
                                             //  7337:    }
                                             //  7337:    } // switch Inst[23:21]
                                             //  7337:   }
  2, 36,                                     //  7342:   case 0x2: {
  OPC_SwitchField, 21, 3,                    //  7344:    switch Inst[23:21] {
  0, 9,                                      //  7347:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  7349:     check Inst[13] == 0x0
  OPC_Decode, 219, 16, 132, 1,               //  7353:     decode to S2_asl_i_r_nac using decoder 132
                                             //  7353:    }
  2, 9,                                      //  7358:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  7360:     check Inst[13] == 0x0
  OPC_Decode, 218, 16, 132, 1,               //  7364:     decode to S2_asl_i_r_and using decoder 132
                                             //  7364:    }
  4, 0,                                      //  7369:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  //  7371:     check Inst[13] == 0x0
  OPC_Decode, 222, 16, 132, 1,               //  7375:     decode to S2_asl_i_r_xacc using decoder 132
                                             //  7375:    }
                                             //  7375:    } // switch Inst[23:21]
                                             //  7375:   }
  3, 42,                                     //  7380:   case 0x3: {
  OPC_SwitchField, 21, 3,                    //  7382:    switch Inst[23:21] {
  0, 11,                                     //  7385:    case 0x0: {
  OPC_CheckPredicate, 1,                     //  7387:     check predicate 1
  OPC_CheckField, 13, 1, 0,                  //  7389:     check Inst[13] == 0x0
  OPC_Decode, 144, 20, 132, 1,               //  7393:     decode to S6_rol_i_r_nac using decoder 132
                                             //  7393:    }
  2, 11,                                     //  7398:    case 0x2: {
  OPC_CheckPredicate, 1,                     //  7400:     check predicate 1
  OPC_CheckField, 13, 1, 0,                  //  7402:     check Inst[13] == 0x0
  OPC_Decode, 143, 20, 132, 1,               //  7406:     decode to S6_rol_i_r_and using decoder 132
                                             //  7406:    }
  4, 0,                                      //  7411:    case 0x4: {
  OPC_CheckPredicate, 1,                     //  7413:     check predicate 1
  OPC_CheckField, 13, 1, 0,                  //  7415:     check Inst[13] == 0x0
  OPC_Decode, 146, 20, 132, 1,               //  7419:     decode to S6_rol_i_r_xacc using decoder 132
                                             //  7419:    }
                                             //  7419:    } // switch Inst[23:21]
                                             //  7419:   }
  4, 25,                                     //  7424:   case 0x4: {
  OPC_SwitchField, 21, 3,                    //  7426:    switch Inst[23:21] {
  0, 9,                                      //  7429:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  7431:     check Inst[13] == 0x0
  OPC_Decode, 246, 16, 132, 1,               //  7435:     decode to S2_asr_i_r_acc using decoder 132
                                             //  7435:    }
  2, 0,                                      //  7440:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  7442:     check Inst[13] == 0x0
  OPC_Decode, 249, 16, 132, 1,               //  7446:     decode to S2_asr_i_r_or using decoder 132
                                             //  7446:    }
                                             //  7446:    } // switch Inst[23:21]
                                             //  7446:   }
  5, 25,                                     //  7451:   case 0x5: {
  OPC_SwitchField, 21, 3,                    //  7453:    switch Inst[23:21] {
  0, 9,                                      //  7456:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  7458:     check Inst[13] == 0x0
  OPC_Decode, 188, 17, 132, 1,               //  7462:     decode to S2_lsr_i_r_acc using decoder 132
                                             //  7462:    }
  2, 0,                                      //  7467:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  7469:     check Inst[13] == 0x0
  OPC_Decode, 191, 17, 132, 1,               //  7473:     decode to S2_lsr_i_r_or using decoder 132
                                             //  7473:    }
                                             //  7473:    } // switch Inst[23:21]
                                             //  7473:   }
  6, 25,                                     //  7478:   case 0x6: {
  OPC_SwitchField, 21, 3,                    //  7480:    switch Inst[23:21] {
  0, 9,                                      //  7483:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  7485:     check Inst[13] == 0x0
  OPC_Decode, 217, 16, 132, 1,               //  7489:     decode to S2_asl_i_r_acc using decoder 132
                                             //  7489:    }
  2, 0,                                      //  7494:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  7496:     check Inst[13] == 0x0
  OPC_Decode, 220, 16, 132, 1,               //  7500:     decode to S2_asl_i_r_or using decoder 132
                                             //  7500:    }
                                             //  7500:    } // switch Inst[23:21]
                                             //  7500:   }
  7, 0,                                      //  7505:   case 0x7: {
  OPC_SwitchField, 21, 3,                    //  7507:    switch Inst[23:21] {
  0, 11,                                     //  7510:    case 0x0: {
  OPC_CheckPredicate, 1,                     //  7512:     check predicate 1
  OPC_CheckField, 13, 1, 0,                  //  7514:     check Inst[13] == 0x0
  OPC_Decode, 142, 20, 132, 1,               //  7518:     decode to S6_rol_i_r_acc using decoder 132
                                             //  7518:    }
  2, 0,                                      //  7523:    case 0x2: {
  OPC_CheckPredicate, 1,                     //  7525:     check predicate 1
  OPC_CheckField, 13, 1, 0,                  //  7527:     check Inst[13] == 0x0
  OPC_Decode, 145, 20, 132, 1,               //  7531:     decode to S6_rol_i_r_or using decoder 132
                                             //  7531:    }
                                             //  7531:    } // switch Inst[23:21]
                                             //  7531:   }
                                             //  7531:   } // switch Inst[7:5]
                                             //  7531:  }
  15, 0,                                     //  7536:  case 0xf: {
  OPC_CheckField, 23, 1, 0,                  //  7538:   check Inst[23] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  7542:   check Inst[13] == 0x0
  OPC_Decode, 162, 17, 133, 1,               //  7546:   decode to S2_insert using decoder 133
                                             //  7546:  }
                                             //  7546:  } // switch Inst[27:24]
                                             //  7546: }
  9, 147, 11,                                //  7551: case 0x9: {
  OPC_SwitchField, 21, 4,                    //  7554:  switch Inst[24:21] {
  0, 168, 1,                                 //  7557:  case 0x0: {
  OPC_SwitchField, 25, 3,                    //  7560:   switch Inst[27:25] {
  0, 8,                                      //  7563:   case 0x0: {
  OPC_CheckField, 5, 9, 0,                   //  7565:    check Inst[13:5] == 0x0
  OPC_Decode, 235, 11, 86,                   //  7569:    decode to L2_deallocframe using decoder 86
                                             //  7569:   }
  1, 61,                                     //  7573:   case 0x1: {
  OPC_SwitchField, 5, 3,                     //  7575:    switch Inst[7:5] {
  0, 43,                                     //  7578:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  7580:     switch Inst[13] {
  0, 31,                                     //  7583:     case 0x0: {
  OPC_SwitchField, 8, 5,                     //  7585:      switch Inst[12:8] {
  0, 4,                                      //  7588:      case 0x0: {
  OPC_Decode, 187, 12, 61,                   //  7590:       decode to L2_loadw_locked using decoder 61
                                             //  7590:      }
  8, 6,                                      //  7594:      case 0x8: {
  OPC_CheckPredicate, 9,                     //  7596:       check predicate 9
  OPC_Decode, 186, 12, 61,                   //  7598:       decode to L2_loadw_aq using decoder 61
                                             //  7598:      }
  16, 4,                                     //  7602:      case 0x10: {
  OPC_Decode, 139, 13, 86,                   //  7604:       decode to L4_loadd_locked using decoder 86
                                             //  7604:      }
  24, 0,                                     //  7608:      case 0x18: {
  OPC_CheckPredicate, 9,                     //  7610:       check predicate 9
  OPC_Decode, 138, 13, 86,                   //  7612:       decode to L4_loadd_aq using decoder 86
                                             //  7612:      }
                                             //  7612:      } // switch Inst[12:8]
                                             //  7612:     }
  1, 0,                                      //  7616:     case 0x1: {
  OPC_Decode, 158, 13, 134, 1,               //  7618:      decode to L4_loadw_phys using decoder 134
                                             //  7618:     }
                                             //  7618:     } // switch Inst[13]
                                             //  7618:    }
  2, 0,                                      //  7623:    case 0x2: {
  OPC_CheckPredicate, 4,                     //  7625:     check predicate 4
  OPC_CheckField, 0, 5, 0,                   //  7627:     check Inst[4:0] == 0x0
  OPC_Decode, 220, 13, 135, 1,               //  7631:     decode to L6_memcpy using decoder 135
                                             //  7631:    }
                                             //  7631:    } // switch Inst[7:5]
                                             //  7631:   }
  2, 9,                                      //  7636:   case 0x2: {
  OPC_CheckField, 11, 3, 0,                  //  7638:    check Inst[13:11] == 0x0
  OPC_Decode, 148, 26, 136, 1,               //  7642:    decode to Y2_dcfetchbo using decoder 136
                                             //  7642:   }
  3, 0,                                      //  7647:   case 0x3: {
  OPC_SwitchField, 10, 4,                    //  7649:    switch Inst[13:10] {
  0, 8,                                      //  7652:    case 0x0: {
  OPC_CheckField, 5, 5, 0,                   //  7654:     check Inst[9:5] == 0x0
  OPC_Decode, 210, 13, 86,                   //  7658:     decode to L4_return using decoder 86
                                             //  7658:    }
  2, 9,                                      //  7662:    case 0x2: {
  OPC_CheckField, 5, 3, 0,                   //  7664:     check Inst[7:5] == 0x0
  OPC_Decode, 215, 13, 137, 1,               //  7668:     decode to L4_return_tnew_pnt using decoder 137
                                             //  7668:    }
  4, 9,                                      //  7673:    case 0x4: {
  OPC_CheckField, 5, 3, 0,                   //  7675:     check Inst[7:5] == 0x0
  OPC_Decode, 214, 13, 137, 1,               //  7679:     decode to L4_return_t using decoder 137
                                             //  7679:    }
  6, 9,                                      //  7684:    case 0x6: {
  OPC_CheckField, 5, 3, 0,                   //  7686:     check Inst[7:5] == 0x0
  OPC_Decode, 216, 13, 137, 1,               //  7690:     decode to L4_return_tnew_pt using decoder 137
                                             //  7690:    }
  10, 9,                                     //  7695:    case 0xa: {
  OPC_CheckField, 5, 3, 0,                   //  7697:     check Inst[7:5] == 0x0
  OPC_Decode, 212, 13, 137, 1,               //  7701:     decode to L4_return_fnew_pnt using decoder 137
                                             //  7701:    }
  12, 9,                                     //  7706:    case 0xc: {
  OPC_CheckField, 5, 3, 0,                   //  7708:     check Inst[7:5] == 0x0
  OPC_Decode, 211, 13, 137, 1,               //  7712:     decode to L4_return_f using decoder 137
                                             //  7712:    }
  14, 0,                                     //  7717:    case 0xe: {
  OPC_CheckField, 5, 3, 0,                   //  7719:     check Inst[7:5] == 0x0
  OPC_Decode, 213, 13, 137, 1,               //  7723:     decode to L4_return_fnew_pt using decoder 137
                                             //  7723:    }
                                             //  7723:    } // switch Inst[13:10]
                                             //  7723:   }
                                             //  7723:   } // switch Inst[27:25]
                                             //  7723:  }
  1, 71,                                     //  7728:  case 0x1: {
  OPC_SwitchField, 27, 1,                    //  7730:   switch Inst[27] {
  0, 5,                                      //  7733:   case 0x0: {
  OPC_Decode, 248, 11, 138, 1,               //  7735:    decode to L2_loadbsw2_io using decoder 138
                                             //  7735:   }
  1, 0,                                      //  7740:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  7742:    switch Inst[26:25] {
  0, 21,                                     //  7745:    case 0x0: {
  OPC_SwitchField, 9, 4,                     //  7747:     switch Inst[12:9] {
  0, 5,                                      //  7750:     case 0x0: {
  OPC_Decode, 250, 11, 139, 1,               //  7752:      decode to L2_loadbsw2_pci using decoder 139
                                             //  7752:     }
  1, 0,                                      //  7757:     case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  7759:      check Inst[8:5] == 0x0
  OPC_Decode, 251, 11, 140, 1,               //  7763:      decode to L2_loadbsw2_pcr using decoder 140
                                             //  7763:     }
                                             //  7763:     } // switch Inst[12:9]
                                             //  7763:    }
  1, 9,                                      //  7768:    case 0x1: {
  OPC_CheckField, 9, 5, 0,                   //  7770:     check Inst[13:9] == 0x0
  OPC_Decode, 252, 11, 141, 1,               //  7774:     decode to L2_loadbsw2_pi using decoder 141
                                             //  7774:    }
  2, 9,                                      //  7779:    case 0x2: {
  OPC_CheckField, 5, 8, 0,                   //  7781:     check Inst[12:5] == 0x0
  OPC_Decode, 253, 11, 140, 1,               //  7785:     decode to L2_loadbsw2_pr using decoder 140
                                             //  7785:    }
  3, 0,                                      //  7790:    case 0x3: {
  OPC_CheckField, 5, 8, 0,                   //  7792:     check Inst[12:5] == 0x0
  OPC_Decode, 249, 11, 140, 1,               //  7796:     decode to L2_loadbsw2_pbr using decoder 140
                                             //  7796:    }
                                             //  7796:    } // switch Inst[26:25]
                                             //  7796:   }
                                             //  7796:   } // switch Inst[27]
                                             //  7796:  }
  2, 71,                                     //  7801:  case 0x2: {
  OPC_SwitchField, 27, 1,                    //  7803:   switch Inst[27] {
  0, 5,                                      //  7806:   case 0x0: {
  OPC_Decode, 242, 11, 142, 1,               //  7808:    decode to L2_loadalignh_io using decoder 142
                                             //  7808:   }
  1, 0,                                      //  7813:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  7815:    switch Inst[26:25] {
  0, 21,                                     //  7818:    case 0x0: {
  OPC_SwitchField, 9, 4,                     //  7820:     switch Inst[12:9] {
  0, 5,                                      //  7823:     case 0x0: {
  OPC_Decode, 244, 11, 143, 1,               //  7825:      decode to L2_loadalignh_pci using decoder 143
                                             //  7825:     }
  1, 0,                                      //  7830:     case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  7832:      check Inst[8:5] == 0x0
  OPC_Decode, 245, 11, 144, 1,               //  7836:      decode to L2_loadalignh_pcr using decoder 144
                                             //  7836:     }
                                             //  7836:     } // switch Inst[12:9]
                                             //  7836:    }
  1, 9,                                      //  7841:    case 0x1: {
  OPC_CheckField, 9, 5, 0,                   //  7843:     check Inst[13:9] == 0x0
  OPC_Decode, 246, 11, 145, 1,               //  7847:     decode to L2_loadalignh_pi using decoder 145
                                             //  7847:    }
  2, 9,                                      //  7852:    case 0x2: {
  OPC_CheckField, 5, 8, 0,                   //  7854:     check Inst[12:5] == 0x0
  OPC_Decode, 247, 11, 144, 1,               //  7858:     decode to L2_loadalignh_pr using decoder 144
                                             //  7858:    }
  3, 0,                                      //  7863:    case 0x3: {
  OPC_CheckField, 5, 8, 0,                   //  7865:     check Inst[12:5] == 0x0
  OPC_Decode, 243, 11, 144, 1,               //  7869:     decode to L2_loadalignh_pbr using decoder 144
                                             //  7869:    }
                                             //  7869:    } // switch Inst[26:25]
                                             //  7869:   }
                                             //  7869:   } // switch Inst[27]
                                             //  7869:  }
  3, 71,                                     //  7874:  case 0x3: {
  OPC_SwitchField, 27, 1,                    //  7876:   switch Inst[27] {
  0, 5,                                      //  7879:   case 0x0: {
  OPC_Decode, 132, 12, 138, 1,               //  7881:    decode to L2_loadbzw2_io using decoder 138
                                             //  7881:   }
  1, 0,                                      //  7886:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  7888:    switch Inst[26:25] {
  0, 21,                                     //  7891:    case 0x0: {
  OPC_SwitchField, 9, 4,                     //  7893:     switch Inst[12:9] {
  0, 5,                                      //  7896:     case 0x0: {
  OPC_Decode, 134, 12, 139, 1,               //  7898:      decode to L2_loadbzw2_pci using decoder 139
                                             //  7898:     }
  1, 0,                                      //  7903:     case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  7905:      check Inst[8:5] == 0x0
  OPC_Decode, 135, 12, 140, 1,               //  7909:      decode to L2_loadbzw2_pcr using decoder 140
                                             //  7909:     }
                                             //  7909:     } // switch Inst[12:9]
                                             //  7909:    }
  1, 9,                                      //  7914:    case 0x1: {
  OPC_CheckField, 9, 5, 0,                   //  7916:     check Inst[13:9] == 0x0
  OPC_Decode, 136, 12, 141, 1,               //  7920:     decode to L2_loadbzw2_pi using decoder 141
                                             //  7920:    }
  2, 9,                                      //  7925:    case 0x2: {
  OPC_CheckField, 5, 8, 0,                   //  7927:     check Inst[12:5] == 0x0
  OPC_Decode, 137, 12, 140, 1,               //  7931:     decode to L2_loadbzw2_pr using decoder 140
                                             //  7931:    }
  3, 0,                                      //  7936:    case 0x3: {
  OPC_CheckField, 5, 8, 0,                   //  7938:     check Inst[12:5] == 0x0
  OPC_Decode, 133, 12, 140, 1,               //  7942:     decode to L2_loadbzw2_pbr using decoder 140
                                             //  7942:    }
                                             //  7942:    } // switch Inst[26:25]
                                             //  7942:   }
                                             //  7942:   } // switch Inst[27]
                                             //  7942:  }
  4, 71,                                     //  7947:  case 0x4: {
  OPC_SwitchField, 27, 1,                    //  7949:   switch Inst[27] {
  0, 5,                                      //  7952:   case 0x0: {
  OPC_Decode, 236, 11, 146, 1,               //  7954:    decode to L2_loadalignb_io using decoder 146
                                             //  7954:   }
  1, 0,                                      //  7959:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  7961:    switch Inst[26:25] {
  0, 21,                                     //  7964:    case 0x0: {
  OPC_SwitchField, 9, 4,                     //  7966:     switch Inst[12:9] {
  0, 5,                                      //  7969:     case 0x0: {
  OPC_Decode, 238, 11, 147, 1,               //  7971:      decode to L2_loadalignb_pci using decoder 147
                                             //  7971:     }
  1, 0,                                      //  7976:     case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  7978:      check Inst[8:5] == 0x0
  OPC_Decode, 239, 11, 144, 1,               //  7982:      decode to L2_loadalignb_pcr using decoder 144
                                             //  7982:     }
                                             //  7982:     } // switch Inst[12:9]
                                             //  7982:    }
  1, 9,                                      //  7987:    case 0x1: {
  OPC_CheckField, 9, 5, 0,                   //  7989:     check Inst[13:9] == 0x0
  OPC_Decode, 240, 11, 148, 1,               //  7993:     decode to L2_loadalignb_pi using decoder 148
                                             //  7993:    }
  2, 9,                                      //  7998:    case 0x2: {
  OPC_CheckField, 5, 8, 0,                   //  8000:     check Inst[12:5] == 0x0
  OPC_Decode, 241, 11, 144, 1,               //  8004:     decode to L2_loadalignb_pr using decoder 144
                                             //  8004:    }
  3, 0,                                      //  8009:    case 0x3: {
  OPC_CheckField, 5, 8, 0,                   //  8011:     check Inst[12:5] == 0x0
  OPC_Decode, 237, 11, 144, 1,               //  8015:     decode to L2_loadalignb_pbr using decoder 144
                                             //  8015:    }
                                             //  8015:    } // switch Inst[26:25]
                                             //  8015:   }
                                             //  8015:   } // switch Inst[27]
                                             //  8015:  }
  5, 71,                                     //  8020:  case 0x5: {
  OPC_SwitchField, 27, 1,                    //  8022:   switch Inst[27] {
  0, 5,                                      //  8025:   case 0x0: {
  OPC_Decode, 138, 12, 149, 1,               //  8027:    decode to L2_loadbzw4_io using decoder 149
                                             //  8027:   }
  1, 0,                                      //  8032:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  8034:    switch Inst[26:25] {
  0, 21,                                     //  8037:    case 0x0: {
  OPC_SwitchField, 9, 4,                     //  8039:     switch Inst[12:9] {
  0, 5,                                      //  8042:     case 0x0: {
  OPC_Decode, 140, 12, 150, 1,               //  8044:      decode to L2_loadbzw4_pci using decoder 150
                                             //  8044:     }
  1, 0,                                      //  8049:     case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8051:      check Inst[8:5] == 0x0
  OPC_Decode, 141, 12, 151, 1,               //  8055:      decode to L2_loadbzw4_pcr using decoder 151
                                             //  8055:     }
                                             //  8055:     } // switch Inst[12:9]
                                             //  8055:    }
  1, 9,                                      //  8060:    case 0x1: {
  OPC_CheckField, 9, 5, 0,                   //  8062:     check Inst[13:9] == 0x0
  OPC_Decode, 142, 12, 152, 1,               //  8066:     decode to L2_loadbzw4_pi using decoder 152
                                             //  8066:    }
  2, 9,                                      //  8071:    case 0x2: {
  OPC_CheckField, 5, 8, 0,                   //  8073:     check Inst[12:5] == 0x0
  OPC_Decode, 143, 12, 151, 1,               //  8077:     decode to L2_loadbzw4_pr using decoder 151
                                             //  8077:    }
  3, 0,                                      //  8082:    case 0x3: {
  OPC_CheckField, 5, 8, 0,                   //  8084:     check Inst[12:5] == 0x0
  OPC_Decode, 139, 12, 151, 1,               //  8088:     decode to L2_loadbzw4_pbr using decoder 151
                                             //  8088:    }
                                             //  8088:    } // switch Inst[26:25]
                                             //  8088:   }
                                             //  8088:   } // switch Inst[27]
                                             //  8088:  }
  7, 71,                                     //  8093:  case 0x7: {
  OPC_SwitchField, 27, 1,                    //  8095:   switch Inst[27] {
  0, 5,                                      //  8098:   case 0x0: {
  OPC_Decode, 254, 11, 149, 1,               //  8100:    decode to L2_loadbsw4_io using decoder 149
                                             //  8100:   }
  1, 0,                                      //  8105:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  8107:    switch Inst[26:25] {
  0, 21,                                     //  8110:    case 0x0: {
  OPC_SwitchField, 9, 4,                     //  8112:     switch Inst[12:9] {
  0, 5,                                      //  8115:     case 0x0: {
  OPC_Decode, 128, 12, 150, 1,               //  8117:      decode to L2_loadbsw4_pci using decoder 150
                                             //  8117:     }
  1, 0,                                      //  8122:     case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8124:      check Inst[8:5] == 0x0
  OPC_Decode, 129, 12, 151, 1,               //  8128:      decode to L2_loadbsw4_pcr using decoder 151
                                             //  8128:     }
                                             //  8128:     } // switch Inst[12:9]
                                             //  8128:    }
  1, 9,                                      //  8133:    case 0x1: {
  OPC_CheckField, 9, 5, 0,                   //  8135:     check Inst[13:9] == 0x0
  OPC_Decode, 130, 12, 152, 1,               //  8139:     decode to L2_loadbsw4_pi using decoder 152
                                             //  8139:    }
  2, 9,                                      //  8144:    case 0x2: {
  OPC_CheckField, 5, 8, 0,                   //  8146:     check Inst[12:5] == 0x0
  OPC_Decode, 131, 12, 151, 1,               //  8150:     decode to L2_loadbsw4_pr using decoder 151
                                             //  8150:    }
  3, 0,                                      //  8155:    case 0x3: {
  OPC_CheckField, 5, 8, 0,                   //  8157:     check Inst[12:5] == 0x0
  OPC_Decode, 255, 11, 151, 1,               //  8161:     decode to L2_loadbsw4_pbr using decoder 151
                                             //  8161:    }
                                             //  8161:    } // switch Inst[26:25]
                                             //  8161:   }
                                             //  8161:   } // switch Inst[27]
                                             //  8161:  }
  8, 133, 1,                                 //  8166:  case 0x8: {
  OPC_SwitchField, 27, 1,                    //  8169:   switch Inst[27] {
  0, 5,                                      //  8172:   case 0x0: {
  OPC_Decode, 144, 12, 153, 1,               //  8174:    decode to L2_loadrb_io using decoder 153
                                             //  8174:   }
  1, 0,                                      //  8179:   case 0x1: {
  OPC_SwitchField, 11, 2,                    //  8181:    switch Inst[12:11] {
  0, 71,                                     //  8184:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  8186:     switch Inst[26:25] {
  0, 21,                                     //  8189:     case 0x0: {
  OPC_SwitchField, 9, 2,                     //  8191:      switch Inst[10:9] {
  0, 5,                                      //  8194:      case 0x0: {
  OPC_Decode, 146, 12, 154, 1,               //  8196:       decode to L2_loadrb_pci using decoder 154
                                             //  8196:      }
  1, 0,                                      //  8201:      case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8203:       check Inst[8:5] == 0x0
  OPC_Decode, 147, 12, 140, 1,               //  8207:       decode to L2_loadrb_pcr using decoder 140
                                             //  8207:      }
                                             //  8207:      } // switch Inst[10:9]
                                             //  8207:     }
  1, 21,                                     //  8212:     case 0x1: {
  OPC_SwitchField, 13, 1,                    //  8214:      switch Inst[13] {
  0, 9,                                      //  8217:      case 0x0: {
  OPC_CheckField, 9, 2, 0,                   //  8219:       check Inst[10:9] == 0x0
  OPC_Decode, 148, 12, 155, 1,               //  8223:       decode to L2_loadrb_pi using decoder 155
                                             //  8223:      }
  1, 0,                                      //  8228:      case 0x1: {
  OPC_Decode, 193, 12, 156, 1,               //  8230:       decode to L2_ploadrbt_pi using decoder 156
                                             //  8230:      }
                                             //  8230:      } // switch Inst[13]
                                             //  8230:     }
  2, 9,                                      //  8235:     case 0x2: {
  OPC_CheckField, 5, 6, 0,                   //  8237:      check Inst[10:5] == 0x0
  OPC_Decode, 149, 12, 140, 1,               //  8241:      decode to L2_loadrb_pr using decoder 140
                                             //  8241:     }
  3, 0,                                      //  8246:     case 0x3: {
  OPC_CheckField, 5, 6, 0,                   //  8248:      check Inst[10:5] == 0x0
  OPC_Decode, 145, 12, 140, 1,               //  8252:      decode to L2_loadrb_pbr using decoder 140
                                             //  8252:     }
                                             //  8252:     } // switch Inst[26:25]
                                             //  8252:    }
  1, 13,                                     //  8257:    case 0x1: {
  OPC_CheckField, 25, 2, 1,                  //  8259:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8263:     check Inst[13] == 0x1
  OPC_Decode, 189, 12, 156, 1,               //  8267:     decode to L2_ploadrbf_pi using decoder 156
                                             //  8267:    }
  2, 13,                                     //  8272:    case 0x2: {
  OPC_CheckField, 25, 2, 1,                  //  8274:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8278:     check Inst[13] == 0x1
  OPC_Decode, 195, 12, 156, 1,               //  8282:     decode to L2_ploadrbtnew_pi using decoder 156
                                             //  8282:    }
  3, 0,                                      //  8287:    case 0x3: {
  OPC_CheckField, 25, 2, 1,                  //  8289:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8293:     check Inst[13] == 0x1
  OPC_Decode, 191, 12, 156, 1,               //  8297:     decode to L2_ploadrbfnew_pi using decoder 156
                                             //  8297:    }
                                             //  8297:    } // switch Inst[12:11]
                                             //  8297:   }
                                             //  8297:   } // switch Inst[27]
                                             //  8297:  }
  9, 133, 1,                                 //  8302:  case 0x9: {
  OPC_SwitchField, 27, 1,                    //  8305:   switch Inst[27] {
  0, 5,                                      //  8308:   case 0x0: {
  OPC_Decode, 172, 12, 153, 1,               //  8310:    decode to L2_loadrub_io using decoder 153
                                             //  8310:   }
  1, 0,                                      //  8315:   case 0x1: {
  OPC_SwitchField, 11, 2,                    //  8317:    switch Inst[12:11] {
  0, 71,                                     //  8320:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  8322:     switch Inst[26:25] {
  0, 21,                                     //  8325:     case 0x0: {
  OPC_SwitchField, 9, 2,                     //  8327:      switch Inst[10:9] {
  0, 5,                                      //  8330:      case 0x0: {
  OPC_Decode, 174, 12, 154, 1,               //  8332:       decode to L2_loadrub_pci using decoder 154
                                             //  8332:      }
  1, 0,                                      //  8337:      case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8339:       check Inst[8:5] == 0x0
  OPC_Decode, 175, 12, 140, 1,               //  8343:       decode to L2_loadrub_pcr using decoder 140
                                             //  8343:      }
                                             //  8343:      } // switch Inst[10:9]
                                             //  8343:     }
  1, 21,                                     //  8348:     case 0x1: {
  OPC_SwitchField, 13, 1,                    //  8350:      switch Inst[13] {
  0, 9,                                      //  8353:      case 0x0: {
  OPC_CheckField, 9, 2, 0,                   //  8355:       check Inst[10:9] == 0x0
  OPC_Decode, 176, 12, 155, 1,               //  8359:       decode to L2_loadrub_pi using decoder 155
                                             //  8359:      }
  1, 0,                                      //  8364:      case 0x1: {
  OPC_Decode, 225, 12, 156, 1,               //  8366:       decode to L2_ploadrubt_pi using decoder 156
                                             //  8366:      }
                                             //  8366:      } // switch Inst[13]
                                             //  8366:     }
  2, 9,                                      //  8371:     case 0x2: {
  OPC_CheckField, 5, 6, 0,                   //  8373:      check Inst[10:5] == 0x0
  OPC_Decode, 177, 12, 140, 1,               //  8377:      decode to L2_loadrub_pr using decoder 140
                                             //  8377:     }
  3, 0,                                      //  8382:     case 0x3: {
  OPC_CheckField, 5, 6, 0,                   //  8384:      check Inst[10:5] == 0x0
  OPC_Decode, 173, 12, 140, 1,               //  8388:      decode to L2_loadrub_pbr using decoder 140
                                             //  8388:     }
                                             //  8388:     } // switch Inst[26:25]
                                             //  8388:    }
  1, 13,                                     //  8393:    case 0x1: {
  OPC_CheckField, 25, 2, 1,                  //  8395:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8399:     check Inst[13] == 0x1
  OPC_Decode, 221, 12, 156, 1,               //  8403:     decode to L2_ploadrubf_pi using decoder 156
                                             //  8403:    }
  2, 13,                                     //  8408:    case 0x2: {
  OPC_CheckField, 25, 2, 1,                  //  8410:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8414:     check Inst[13] == 0x1
  OPC_Decode, 227, 12, 156, 1,               //  8418:     decode to L2_ploadrubtnew_pi using decoder 156
                                             //  8418:    }
  3, 0,                                      //  8423:    case 0x3: {
  OPC_CheckField, 25, 2, 1,                  //  8425:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8429:     check Inst[13] == 0x1
  OPC_Decode, 223, 12, 156, 1,               //  8433:     decode to L2_ploadrubfnew_pi using decoder 156
                                             //  8433:    }
                                             //  8433:    } // switch Inst[12:11]
                                             //  8433:   }
                                             //  8433:   } // switch Inst[27]
                                             //  8433:  }
  10, 133, 1,                                //  8438:  case 0xa: {
  OPC_SwitchField, 27, 1,                    //  8441:   switch Inst[27] {
  0, 5,                                      //  8444:   case 0x0: {
  OPC_Decode, 158, 12, 138, 1,               //  8446:    decode to L2_loadrh_io using decoder 138
                                             //  8446:   }
  1, 0,                                      //  8451:   case 0x1: {
  OPC_SwitchField, 11, 2,                    //  8453:    switch Inst[12:11] {
  0, 71,                                     //  8456:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  8458:     switch Inst[26:25] {
  0, 21,                                     //  8461:     case 0x0: {
  OPC_SwitchField, 9, 2,                     //  8463:      switch Inst[10:9] {
  0, 5,                                      //  8466:      case 0x0: {
  OPC_Decode, 160, 12, 139, 1,               //  8468:       decode to L2_loadrh_pci using decoder 139
                                             //  8468:      }
  1, 0,                                      //  8473:      case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8475:       check Inst[8:5] == 0x0
  OPC_Decode, 161, 12, 140, 1,               //  8479:       decode to L2_loadrh_pcr using decoder 140
                                             //  8479:      }
                                             //  8479:      } // switch Inst[10:9]
                                             //  8479:     }
  1, 21,                                     //  8484:     case 0x1: {
  OPC_SwitchField, 13, 1,                    //  8486:      switch Inst[13] {
  0, 9,                                      //  8489:      case 0x0: {
  OPC_CheckField, 9, 2, 0,                   //  8491:       check Inst[10:9] == 0x0
  OPC_Decode, 162, 12, 141, 1,               //  8495:       decode to L2_loadrh_pi using decoder 141
                                             //  8495:      }
  1, 0,                                      //  8500:      case 0x1: {
  OPC_Decode, 209, 12, 157, 1,               //  8502:       decode to L2_ploadrht_pi using decoder 157
                                             //  8502:      }
                                             //  8502:      } // switch Inst[13]
                                             //  8502:     }
  2, 9,                                      //  8507:     case 0x2: {
  OPC_CheckField, 5, 6, 0,                   //  8509:      check Inst[10:5] == 0x0
  OPC_Decode, 163, 12, 140, 1,               //  8513:      decode to L2_loadrh_pr using decoder 140
                                             //  8513:     }
  3, 0,                                      //  8518:     case 0x3: {
  OPC_CheckField, 5, 6, 0,                   //  8520:      check Inst[10:5] == 0x0
  OPC_Decode, 159, 12, 140, 1,               //  8524:      decode to L2_loadrh_pbr using decoder 140
                                             //  8524:     }
                                             //  8524:     } // switch Inst[26:25]
                                             //  8524:    }
  1, 13,                                     //  8529:    case 0x1: {
  OPC_CheckField, 25, 2, 1,                  //  8531:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8535:     check Inst[13] == 0x1
  OPC_Decode, 205, 12, 157, 1,               //  8539:     decode to L2_ploadrhf_pi using decoder 157
                                             //  8539:    }
  2, 13,                                     //  8544:    case 0x2: {
  OPC_CheckField, 25, 2, 1,                  //  8546:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8550:     check Inst[13] == 0x1
  OPC_Decode, 211, 12, 157, 1,               //  8554:     decode to L2_ploadrhtnew_pi using decoder 157
                                             //  8554:    }
  3, 0,                                      //  8559:    case 0x3: {
  OPC_CheckField, 25, 2, 1,                  //  8561:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8565:     check Inst[13] == 0x1
  OPC_Decode, 207, 12, 157, 1,               //  8569:     decode to L2_ploadrhfnew_pi using decoder 157
                                             //  8569:    }
                                             //  8569:    } // switch Inst[12:11]
                                             //  8569:   }
                                             //  8569:   } // switch Inst[27]
                                             //  8569:  }
  11, 133, 1,                                //  8574:  case 0xb: {
  OPC_SwitchField, 27, 1,                    //  8577:   switch Inst[27] {
  0, 5,                                      //  8580:   case 0x0: {
  OPC_Decode, 179, 12, 138, 1,               //  8582:    decode to L2_loadruh_io using decoder 138
                                             //  8582:   }
  1, 0,                                      //  8587:   case 0x1: {
  OPC_SwitchField, 11, 2,                    //  8589:    switch Inst[12:11] {
  0, 71,                                     //  8592:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  8594:     switch Inst[26:25] {
  0, 21,                                     //  8597:     case 0x0: {
  OPC_SwitchField, 9, 2,                     //  8599:      switch Inst[10:9] {
  0, 5,                                      //  8602:      case 0x0: {
  OPC_Decode, 181, 12, 139, 1,               //  8604:       decode to L2_loadruh_pci using decoder 139
                                             //  8604:      }
  1, 0,                                      //  8609:      case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8611:       check Inst[8:5] == 0x0
  OPC_Decode, 182, 12, 140, 1,               //  8615:       decode to L2_loadruh_pcr using decoder 140
                                             //  8615:      }
                                             //  8615:      } // switch Inst[10:9]
                                             //  8615:     }
  1, 21,                                     //  8620:     case 0x1: {
  OPC_SwitchField, 13, 1,                    //  8622:      switch Inst[13] {
  0, 9,                                      //  8625:      case 0x0: {
  OPC_CheckField, 9, 2, 0,                   //  8627:       check Inst[10:9] == 0x0
  OPC_Decode, 183, 12, 141, 1,               //  8631:       decode to L2_loadruh_pi using decoder 141
                                             //  8631:      }
  1, 0,                                      //  8636:      case 0x1: {
  OPC_Decode, 233, 12, 157, 1,               //  8638:       decode to L2_ploadruht_pi using decoder 157
                                             //  8638:      }
                                             //  8638:      } // switch Inst[13]
                                             //  8638:     }
  2, 9,                                      //  8643:     case 0x2: {
  OPC_CheckField, 5, 6, 0,                   //  8645:      check Inst[10:5] == 0x0
  OPC_Decode, 184, 12, 140, 1,               //  8649:      decode to L2_loadruh_pr using decoder 140
                                             //  8649:     }
  3, 0,                                      //  8654:     case 0x3: {
  OPC_CheckField, 5, 6, 0,                   //  8656:      check Inst[10:5] == 0x0
  OPC_Decode, 180, 12, 140, 1,               //  8660:      decode to L2_loadruh_pbr using decoder 140
                                             //  8660:     }
                                             //  8660:     } // switch Inst[26:25]
                                             //  8660:    }
  1, 13,                                     //  8665:    case 0x1: {
  OPC_CheckField, 25, 2, 1,                  //  8667:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8671:     check Inst[13] == 0x1
  OPC_Decode, 229, 12, 157, 1,               //  8675:     decode to L2_ploadruhf_pi using decoder 157
                                             //  8675:    }
  2, 13,                                     //  8680:    case 0x2: {
  OPC_CheckField, 25, 2, 1,                  //  8682:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8686:     check Inst[13] == 0x1
  OPC_Decode, 235, 12, 157, 1,               //  8690:     decode to L2_ploadruhtnew_pi using decoder 157
                                             //  8690:    }
  3, 0,                                      //  8695:    case 0x3: {
  OPC_CheckField, 25, 2, 1,                  //  8697:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8701:     check Inst[13] == 0x1
  OPC_Decode, 231, 12, 157, 1,               //  8705:     decode to L2_ploadruhfnew_pi using decoder 157
                                             //  8705:    }
                                             //  8705:    } // switch Inst[12:11]
                                             //  8705:   }
                                             //  8705:   } // switch Inst[27]
                                             //  8705:  }
  12, 133, 1,                                //  8710:  case 0xc: {
  OPC_SwitchField, 27, 1,                    //  8713:   switch Inst[27] {
  0, 5,                                      //  8716:   case 0x0: {
  OPC_Decode, 165, 12, 158, 1,               //  8718:    decode to L2_loadri_io using decoder 158
                                             //  8718:   }
  1, 0,                                      //  8723:   case 0x1: {
  OPC_SwitchField, 11, 2,                    //  8725:    switch Inst[12:11] {
  0, 71,                                     //  8728:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  8730:     switch Inst[26:25] {
  0, 21,                                     //  8733:     case 0x0: {
  OPC_SwitchField, 9, 2,                     //  8735:      switch Inst[10:9] {
  0, 5,                                      //  8738:      case 0x0: {
  OPC_Decode, 167, 12, 159, 1,               //  8740:       decode to L2_loadri_pci using decoder 159
                                             //  8740:      }
  1, 0,                                      //  8745:      case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8747:       check Inst[8:5] == 0x0
  OPC_Decode, 168, 12, 140, 1,               //  8751:       decode to L2_loadri_pcr using decoder 140
                                             //  8751:      }
                                             //  8751:      } // switch Inst[10:9]
                                             //  8751:     }
  1, 21,                                     //  8756:     case 0x1: {
  OPC_SwitchField, 13, 1,                    //  8758:      switch Inst[13] {
  0, 9,                                      //  8761:      case 0x0: {
  OPC_CheckField, 9, 2, 0,                   //  8763:       check Inst[10:9] == 0x0
  OPC_Decode, 169, 12, 160, 1,               //  8767:       decode to L2_loadri_pi using decoder 160
                                             //  8767:      }
  1, 0,                                      //  8772:      case 0x1: {
  OPC_Decode, 217, 12, 161, 1,               //  8774:       decode to L2_ploadrit_pi using decoder 161
                                             //  8774:      }
                                             //  8774:      } // switch Inst[13]
                                             //  8774:     }
  2, 9,                                      //  8779:     case 0x2: {
  OPC_CheckField, 5, 6, 0,                   //  8781:      check Inst[10:5] == 0x0
  OPC_Decode, 170, 12, 140, 1,               //  8785:      decode to L2_loadri_pr using decoder 140
                                             //  8785:     }
  3, 0,                                      //  8790:     case 0x3: {
  OPC_CheckField, 5, 6, 0,                   //  8792:      check Inst[10:5] == 0x0
  OPC_Decode, 166, 12, 140, 1,               //  8796:      decode to L2_loadri_pbr using decoder 140
                                             //  8796:     }
                                             //  8796:     } // switch Inst[26:25]
                                             //  8796:    }
  1, 13,                                     //  8801:    case 0x1: {
  OPC_CheckField, 25, 2, 1,                  //  8803:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8807:     check Inst[13] == 0x1
  OPC_Decode, 213, 12, 161, 1,               //  8811:     decode to L2_ploadrif_pi using decoder 161
                                             //  8811:    }
  2, 13,                                     //  8816:    case 0x2: {
  OPC_CheckField, 25, 2, 1,                  //  8818:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8822:     check Inst[13] == 0x1
  OPC_Decode, 219, 12, 161, 1,               //  8826:     decode to L2_ploadritnew_pi using decoder 161
                                             //  8826:    }
  3, 0,                                      //  8831:    case 0x3: {
  OPC_CheckField, 25, 2, 1,                  //  8833:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8837:     check Inst[13] == 0x1
  OPC_Decode, 215, 12, 161, 1,               //  8841:     decode to L2_ploadrifnew_pi using decoder 161
                                             //  8841:    }
                                             //  8841:    } // switch Inst[12:11]
                                             //  8841:   }
                                             //  8841:   } // switch Inst[27]
                                             //  8841:  }
  14, 0,                                     //  8846:  case 0xe: {
  OPC_SwitchField, 27, 1,                    //  8848:   switch Inst[27] {
  0, 5,                                      //  8851:   case 0x0: {
  OPC_Decode, 151, 12, 162, 1,               //  8853:    decode to L2_loadrd_io using decoder 162
                                             //  8853:   }
  1, 0,                                      //  8858:   case 0x1: {
  OPC_SwitchField, 11, 2,                    //  8860:    switch Inst[12:11] {
  0, 71,                                     //  8863:    case 0x0: {
  OPC_SwitchField, 25, 2,                    //  8865:     switch Inst[26:25] {
  0, 21,                                     //  8868:     case 0x0: {
  OPC_SwitchField, 9, 2,                     //  8870:      switch Inst[10:9] {
  0, 5,                                      //  8873:      case 0x0: {
  OPC_Decode, 153, 12, 163, 1,               //  8875:       decode to L2_loadrd_pci using decoder 163
                                             //  8875:      }
  1, 0,                                      //  8880:      case 0x1: {
  OPC_CheckField, 5, 4, 0,                   //  8882:       check Inst[8:5] == 0x0
  OPC_Decode, 154, 12, 151, 1,               //  8886:       decode to L2_loadrd_pcr using decoder 151
                                             //  8886:      }
                                             //  8886:      } // switch Inst[10:9]
                                             //  8886:     }
  1, 21,                                     //  8891:     case 0x1: {
  OPC_SwitchField, 13, 1,                    //  8893:      switch Inst[13] {
  0, 9,                                      //  8896:      case 0x0: {
  OPC_CheckField, 9, 2, 0,                   //  8898:       check Inst[10:9] == 0x0
  OPC_Decode, 155, 12, 164, 1,               //  8902:       decode to L2_loadrd_pi using decoder 164
                                             //  8902:      }
  1, 0,                                      //  8907:      case 0x1: {
  OPC_Decode, 201, 12, 165, 1,               //  8909:       decode to L2_ploadrdt_pi using decoder 165
                                             //  8909:      }
                                             //  8909:      } // switch Inst[13]
                                             //  8909:     }
  2, 9,                                      //  8914:     case 0x2: {
  OPC_CheckField, 5, 6, 0,                   //  8916:      check Inst[10:5] == 0x0
  OPC_Decode, 156, 12, 151, 1,               //  8920:      decode to L2_loadrd_pr using decoder 151
                                             //  8920:     }
  3, 0,                                      //  8925:     case 0x3: {
  OPC_CheckField, 5, 6, 0,                   //  8927:      check Inst[10:5] == 0x0
  OPC_Decode, 152, 12, 151, 1,               //  8931:      decode to L2_loadrd_pbr using decoder 151
                                             //  8931:     }
                                             //  8931:     } // switch Inst[26:25]
                                             //  8931:    }
  1, 13,                                     //  8936:    case 0x1: {
  OPC_CheckField, 25, 2, 1,                  //  8938:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8942:     check Inst[13] == 0x1
  OPC_Decode, 197, 12, 165, 1,               //  8946:     decode to L2_ploadrdf_pi using decoder 165
                                             //  8946:    }
  2, 13,                                     //  8951:    case 0x2: {
  OPC_CheckField, 25, 2, 1,                  //  8953:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8957:     check Inst[13] == 0x1
  OPC_Decode, 203, 12, 165, 1,               //  8961:     decode to L2_ploadrdtnew_pi using decoder 165
                                             //  8961:    }
  3, 0,                                      //  8966:    case 0x3: {
  OPC_CheckField, 25, 2, 1,                  //  8968:     check Inst[26:25] == 0x1
  OPC_CheckField, 13, 1, 1,                  //  8972:     check Inst[13] == 0x1
  OPC_Decode, 199, 12, 165, 1,               //  8976:     decode to L2_ploadrdfnew_pi using decoder 165
                                             //  8976:    }
                                             //  8976:    } // switch Inst[12:11]
                                             //  8976:   }
                                             //  8976:   } // switch Inst[27]
                                             //  8976:  }
                                             //  8976:  } // switch Inst[24:21]
                                             //  8976: }
  10, 220, 13,                               //  8981: case 0xa: {
  OPC_SwitchField, 21, 4,                    //  8984:  switch Inst[24:21] {
  0, 203, 1,                                 //  8987:  case 0x0: {
  OPC_SwitchField, 5, 3,                     //  8990:   switch Inst[7:5] {
  0, 81,                                     //  8993:   case 0x0: {
  OPC_SwitchField, 25, 3,                    //  8995:    switch Inst[27:25] {
  0, 12,                                     //  8998:    case 0x0: {
  OPC_CheckField, 8, 6, 0,                   //  9000:     check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9004:     check Inst[4:0] == 0x0
  OPC_Decode, 144, 26, 56,                   //  9008:     decode to Y2_dccleana using decoder 56
                                             //  9008:    }
  1, 16,                                     //  9012:    case 0x1: {
  OPC_CheckField, 16, 5, 0,                  //  9014:     check Inst[20:16] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  9018:     check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9022:     check Inst[4:0] == 0x0
  OPC_Decode, 151, 26, 63,                   //  9026:     decode to Y2_dckill using decoder 63
                                             //  9026:    }
  2, 12,                                     //  9030:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  9032:     check Inst[13] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9036:     check Inst[4:0] == 0x0
  OPC_Decode, 153, 26, 62,                   //  9040:     decode to Y2_dctagw using decoder 62
                                             //  9040:    }
  3, 12,                                     //  9044:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  9046:     check Inst[13] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9050:     check Inst[4:0] == 0x0
  OPC_Decode, 188, 26, 62,                   //  9054:     decode to Y4_l2fetch using decoder 62
                                             //  9054:    }
  4, 0,                                      //  9058:    case 0x4: {
  OPC_CheckField, 16, 5, 0,                  //  9060:     check Inst[20:16] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  9064:     check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9068:     check Inst[4:0] == 0x0
  OPC_Decode, 139, 26, 63,                   //  9072:     decode to Y2_barrier using decoder 63
                                             //  9072:    }
                                             //  9072:    } // switch Inst[27:25]
                                             //  9072:   }
  1, 36,                                     //  9076:   case 0x1: {
  OPC_SwitchField, 25, 3,                    //  9078:    switch Inst[27:25] {
  3, 14,                                     //  9081:    case 0x3: {
  OPC_CheckPredicate, 9,                     //  9083:     check predicate 9
  OPC_CheckField, 8, 6, 0,                   //  9085:     check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9089:     check Inst[4:0] == 0x0
  OPC_Decode, 214, 26, 56,                   //  9093:     decode to Y6_dmstart using decoder 56
                                             //  9093:    }
  4, 0,                                      //  9097:    case 0x4: {
  OPC_CheckPredicate, 9,                     //  9099:     check predicate 9
  OPC_CheckField, 16, 5, 0,                  //  9101:     check Inst[20:16] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  9105:     check Inst[13:8] == 0x0
  OPC_Decode, 215, 26, 166, 1,               //  9109:     decode to Y6_dmwait using decoder 166
                                             //  9109:    }
                                             //  9109:    } // switch Inst[27:25]
                                             //  9109:   }
  2, 36,                                     //  9114:   case 0x2: {
  OPC_SwitchField, 25, 3,                    //  9116:    switch Inst[27:25] {
  3, 14,                                     //  9119:    case 0x3: {
  OPC_CheckPredicate, 9,                     //  9121:     check predicate 9
  OPC_CheckField, 13, 1, 0,                  //  9123:     check Inst[13] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9127:     check Inst[4:0] == 0x0
  OPC_Decode, 210, 26, 62,                   //  9131:     decode to Y6_dmlink using decoder 62
                                             //  9131:    }
  4, 0,                                      //  9135:    case 0x4: {
  OPC_CheckPredicate, 9,                     //  9137:     check predicate 9
  OPC_CheckField, 16, 5, 0,                  //  9139:     check Inst[20:16] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  9143:     check Inst[13:8] == 0x0
  OPC_Decode, 212, 26, 166, 1,               //  9147:     decode to Y6_dmpoll using decoder 166
                                             //  9147:    }
                                             //  9147:    } // switch Inst[27:25]
                                             //  9147:   }
  3, 19,                                     //  9152:   case 0x3: {
  OPC_CheckPredicate, 9,                     //  9154:    check predicate 9
  OPC_CheckField, 25, 3, 4,                  //  9156:    check Inst[27:25] == 0x4
  OPC_CheckField, 16, 5, 0,                  //  9160:    check Inst[20:16] == 0x0
  OPC_CheckField, 8, 6, 0,                   //  9164:    check Inst[13:8] == 0x0
  OPC_Decode, 211, 26, 166, 1,               //  9168:    decode to Y6_dmpause using decoder 166
                                             //  9168:   }
  4, 0,                                      //  9173:   case 0x4: {
  OPC_CheckPredicate, 9,                     //  9175:    check predicate 9
  OPC_CheckField, 25, 3, 3,                  //  9177:    check Inst[27:25] == 0x3
  OPC_CheckField, 8, 6, 0,                   //  9181:    check Inst[13:8] == 0x0
  OPC_CheckField, 0, 5, 0,                   //  9185:    check Inst[4:0] == 0x0
  OPC_Decode, 213, 26, 56,                   //  9189:    decode to Y6_dmresume using decoder 56
                                             //  9189:   }
                                             //  9189:   } // switch Inst[7:5]
                                             //  9189:  }
  1, 91,                                     //  9193:  case 0x1: {
  OPC_SwitchField, 25, 3,                    //  9195:   switch Inst[27:25] {
  0, 8,                                      //  9198:   case 0x0: {
  OPC_CheckField, 0, 14, 0,                  //  9200:    check Inst[13:0] == 0x0
  OPC_Decode, 149, 26, 56,                   //  9204:    decode to Y2_dcinva using decoder 56
                                             //  9204:   }
  1, 8,                                      //  9208:   case 0x1: {
  OPC_CheckField, 0, 14, 0,                  //  9210:    check Inst[13:0] == 0x0
  OPC_Decode, 145, 26, 56,                   //  9214:    decode to Y2_dccleanidx using decoder 56
                                             //  9214:   }
  2, 8,                                      //  9218:   case 0x2: {
  OPC_CheckField, 5, 9, 0,                   //  9220:    check Inst[13:5] == 0x0
  OPC_Decode, 152, 26, 61,                   //  9224:    decode to Y2_dctagr using decoder 61
                                             //  9224:   }
  3, 8,                                      //  9228:   case 0x3: {
  OPC_CheckField, 0, 14, 0,                  //  9230:    check Inst[13:0] == 0x0
  OPC_Decode, 197, 26, 56,                   //  9234:    decode to Y5_l2cleanidx using decoder 56
                                             //  9234:   }
  4, 0,                                      //  9238:   case 0x4: {
  OPC_SwitchField, 0, 14,                    //  9240:    switch Inst[13:0] {
  0, 8,                                      //  9243:    case 0x0: {
  OPC_CheckField, 16, 5, 0,                  //  9245:     check Inst[20:16] == 0x0
  OPC_Decode, 169, 26, 63,                   //  9249:     decode to Y2_l2kill using decoder 63
                                             //  9249:    }
  128, 16, 8,                                //  9253:    case 0x800: {
  OPC_CheckField, 16, 5, 0,                  //  9256:     check Inst[20:16] == 0x0
  OPC_Decode, 201, 26, 63,                   //  9260:     decode to Y5_l2gunlock using decoder 63
                                             //  9260:    }
  128, 32, 8,                                //  9264:    case 0x1000: {
  OPC_CheckField, 16, 5, 0,                  //  9267:     check Inst[20:16] == 0x0
  OPC_Decode, 199, 26, 63,                   //  9271:     decode to Y5_l2gclean using decoder 63
                                             //  9271:    }
  128, 48, 0,                                //  9275:    case 0x1800: {
  OPC_CheckField, 16, 5, 0,                  //  9278:     check Inst[20:16] == 0x0
  OPC_Decode, 200, 26, 63,                   //  9282:     decode to Y5_l2gcleaninv using decoder 63
                                             //  9282:    }
                                             //  9282:    } // switch Inst[13:0]
                                             //  9282:   }
                                             //  9282:   } // switch Inst[27:25]
                                             //  9282:  }
  2, 61,                                     //  9286:  case 0x2: {
  OPC_SwitchField, 25, 3,                    //  9288:   switch Inst[27:25] {
  0, 8,                                      //  9291:   case 0x0: {
  OPC_CheckField, 0, 14, 0,                  //  9293:    check Inst[13:0] == 0x0
  OPC_Decode, 146, 26, 56,                   //  9297:    decode to Y2_dccleaninva using decoder 56
                                             //  9297:   }
  1, 8,                                      //  9301:   case 0x1: {
  OPC_CheckField, 0, 14, 0,                  //  9303:    check Inst[13:0] == 0x0
  OPC_Decode, 150, 26, 56,                   //  9307:    decode to Y2_dcinvidx using decoder 56
                                             //  9307:   }
  2, 12,                                     //  9311:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  //  9313:    check Inst[13] == 0x0
  OPC_CheckField, 0, 8, 0,                   //  9317:    check Inst[7:0] == 0x0
  OPC_Decode, 190, 26, 62,                   //  9321:    decode to Y4_l2tagw using decoder 62
                                             //  9321:   }
  3, 8,                                      //  9325:   case 0x3: {
  OPC_CheckField, 0, 14, 0,                  //  9327:    check Inst[13:0] == 0x0
  OPC_Decode, 202, 26, 56,                   //  9331:    decode to Y5_l2invidx using decoder 56
                                             //  9331:   }
  4, 0,                                      //  9335:   case 0x4: {
  OPC_CheckField, 16, 5, 0,                  //  9337:    check Inst[20:16] == 0x0
  OPC_CheckField, 0, 14, 0,                  //  9341:    check Inst[13:0] == 0x0
  OPC_Decode, 176, 26, 63,                   //  9345:    decode to Y2_syncht using decoder 63
                                             //  9345:   }
                                             //  9345:   } // switch Inst[27:25]
                                             //  9345:  }
  3, 43,                                     //  9349:  case 0x3: {
  OPC_SwitchField, 25, 3,                    //  9351:   switch Inst[27:25] {
  1, 8,                                      //  9354:   case 0x1: {
  OPC_CheckField, 0, 14, 0,                  //  9356:    check Inst[13:0] == 0x0
  OPC_Decode, 147, 26, 56,                   //  9360:    decode to Y2_dccleaninvidx using decoder 56
                                             //  9360:   }
  2, 8,                                      //  9364:   case 0x2: {
  OPC_CheckField, 5, 9, 0,                   //  9366:    check Inst[13:5] == 0x0
  OPC_Decode, 189, 26, 61,                   //  9370:    decode to Y4_l2tagr using decoder 61
                                             //  9370:   }
  3, 8,                                      //  9374:   case 0x3: {
  OPC_CheckField, 0, 14, 0,                  //  9376:    check Inst[13:0] == 0x0
  OPC_Decode, 204, 26, 56,                   //  9380:    decode to Y5_l2unlocka using decoder 56
                                             //  9380:   }
  4, 0,                                      //  9384:   case 0x4: {
  OPC_CheckField, 0, 14, 0,                  //  9386:    check Inst[13:0] == 0x0
  OPC_Decode, 168, 26, 56,                   //  9390:    decode to Y2_l2cleaninvidx using decoder 56
                                             //  9390:   }
                                             //  9390:   } // switch Inst[27:25]
                                             //  9390:  }
  4, 29,                                     //  9394:  case 0x4: {
  OPC_SwitchField, 25, 3,                    //  9396:   switch Inst[27:25] {
  0, 9,                                      //  9399:   case 0x0: {
  OPC_CheckField, 11, 3, 0,                  //  9401:    check Inst[13:11] == 0x0
  OPC_Decode, 209, 16, 167, 1,               //  9405:    decode to S2_allocframe using decoder 167
                                             //  9405:   }
  3, 0,                                      //  9410:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  //  9412:    check Inst[13] == 0x0
  OPC_CheckField, 0, 8, 0,                   //  9416:    check Inst[7:0] == 0x0
  OPC_Decode, 198, 26, 168, 1,               //  9420:    decode to Y5_l2fetch using decoder 168
                                             //  9420:   }
                                             //  9420:   } // switch Inst[27:25]
                                             //  9420:  }
  5, 70,                                     //  9425:  case 0x5: {
  OPC_SwitchField, 2, 6,                     //  9427:   switch Inst[7:2] {
  0, 33,                                     //  9430:   case 0x0: {
  OPC_SwitchField, 25, 3,                    //  9432:    switch Inst[27:25] {
  0, 9,                                      //  9435:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  //  9437:     check Inst[13] == 0x0
  OPC_Decode, 193, 18, 169, 1,               //  9441:     decode to S2_storew_locked using decoder 169
                                             //  9441:    }
  3, 0,                                      //  9446:    case 0x3: {
  OPC_CheckField, 16, 5, 0,                  //  9448:     check Inst[20:16] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9452:     check Inst[13] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9456:     check Inst[1:0] == 0x0
  OPC_Decode, 217, 26, 170, 1,               //  9460:     decode to Y6_l2gcleanpa using decoder 170
                                             //  9460:    }
                                             //  9460:    } // switch Inst[27:25]
                                             //  9460:   }
  2, 14,                                     //  9465:   case 0x2: {
  OPC_CheckPredicate, 9,                     //  9467:    check predicate 9
  OPC_CheckField, 25, 3, 0,                  //  9469:    check Inst[27:25] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9473:    check Inst[13] == 0x0
  OPC_Decode, 194, 18, 62,                   //  9477:    decode to S2_storew_rl_at_vi using decoder 62
                                             //  9477:   }
  10, 0,                                     //  9481:   case 0xa: {
  OPC_CheckPredicate, 9,                     //  9483:    check predicate 9
  OPC_CheckField, 25, 3, 0,                  //  9485:    check Inst[27:25] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9489:    check Inst[13] == 0x0
  OPC_Decode, 195, 18, 62,                   //  9493:    decode to S2_storew_rl_st_vi using decoder 62
                                             //  9493:   }
                                             //  9493:   } // switch Inst[7:2]
                                             //  9493:  }
  6, 32,                                     //  9497:  case 0x6: {
  OPC_SwitchField, 25, 3,                    //  9499:   switch Inst[27:25] {
  0, 8,                                      //  9502:   case 0x0: {
  OPC_CheckField, 0, 14, 0,                  //  9504:    check Inst[13:0] == 0x0
  OPC_Decode, 154, 26, 56,                   //  9508:    decode to Y2_dczeroa using decoder 56
                                             //  9508:   }
  3, 0,                                      //  9512:   case 0x3: {
  OPC_CheckField, 16, 5, 0,                  //  9514:    check Inst[20:16] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9518:    check Inst[13] == 0x0
  OPC_CheckField, 0, 8, 0,                   //  9522:    check Inst[7:0] == 0x0
  OPC_Decode, 216, 26, 170, 1,               //  9526:    decode to Y6_l2gcleaninvpa using decoder 170
                                             //  9526:   }
                                             //  9526:   } // switch Inst[27:25]
                                             //  9526:  }
  7, 99,                                     //  9531:  case 0x7: {
  OPC_SwitchField, 2, 6,                     //  9533:   switch Inst[7:2] {
  0, 28,                                     //  9536:   case 0x0: {
  OPC_SwitchField, 13, 1,                    //  9538:    switch Inst[13] {
  0, 9,                                      //  9541:    case 0x0: {
  OPC_CheckField, 25, 3, 0,                  //  9543:     check Inst[27:25] == 0x0
  OPC_Decode, 206, 19, 171, 1,               //  9547:     decode to S4_stored_locked using decoder 171
                                             //  9547:    }
  1, 0,                                      //  9552:    case 0x1: {
  OPC_CheckField, 25, 3, 0,                  //  9554:     check Inst[27:25] == 0x0
  OPC_CheckField, 8, 5, 0,                   //  9558:     check Inst[12:8] == 0x0
  OPC_Decode, 203, 26, 117,                  //  9562:     decode to Y5_l2locka using decoder 117
                                             //  9562:    }
                                             //  9562:    } // switch Inst[13]
                                             //  9562:   }
  2, 15,                                     //  9566:   case 0x2: {
  OPC_CheckPredicate, 9,                     //  9568:    check predicate 9
  OPC_CheckField, 25, 3, 0,                  //  9570:    check Inst[27:25] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9574:    check Inst[13] == 0x0
  OPC_Decode, 207, 19, 168, 1,               //  9578:    decode to S4_stored_rl_at_vi using decoder 168
                                             //  9578:   }
  3, 14,                                     //  9583:   case 0x3: {
  OPC_CheckPredicate, 9,                     //  9585:    check predicate 9
  OPC_CheckField, 25, 3, 0,                  //  9587:    check Inst[27:25] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9591:    check Inst[13] == 0x0
  OPC_Decode, 198, 16, 56,                   //  9595:    decode to R6_release_at_vi using decoder 56
                                             //  9595:   }
  10, 15,                                    //  9599:   case 0xa: {
  OPC_CheckPredicate, 9,                     //  9601:    check predicate 9
  OPC_CheckField, 25, 3, 0,                  //  9603:    check Inst[27:25] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9607:    check Inst[13] == 0x0
  OPC_Decode, 208, 19, 168, 1,               //  9611:    decode to S4_stored_rl_st_vi using decoder 168
                                             //  9611:   }
  11, 0,                                     //  9616:   case 0xb: {
  OPC_CheckPredicate, 9,                     //  9618:    check predicate 9
  OPC_CheckField, 25, 3, 0,                  //  9620:    check Inst[27:25] == 0x0
  OPC_CheckField, 13, 1, 0,                  //  9624:    check Inst[13] == 0x0
  OPC_Decode, 199, 16, 56,                   //  9628:    decode to R6_release_st_vi using decoder 56
                                             //  9628:   }
                                             //  9628:   } // switch Inst[7:2]
                                             //  9628:  }
  8, 135, 1,                                 //  9632:  case 0x8: {
  OPC_SwitchField, 27, 1,                    //  9635:   switch Inst[27] {
  0, 5,                                      //  9638:   case 0x0: {
  OPC_Decode, 137, 18, 172, 1,               //  9640:    decode to S2_storerb_io using decoder 172
                                             //  9640:   }
  1, 0,                                      //  9645:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  9647:    switch Inst[26:25] {
  0, 25,                                     //  9650:    case 0x0: {
  OPC_SwitchField, 0, 3,                     //  9652:     switch Inst[2:0] {
  0, 9,                                      //  9655:     case 0x0: {
  OPC_CheckField, 7, 1, 0,                   //  9657:      check Inst[7] == 0x0
  OPC_Decode, 139, 18, 173, 1,               //  9661:      decode to S2_storerb_pci using decoder 173
                                             //  9661:     }
  2, 0,                                      //  9666:     case 0x2: {
  OPC_CheckField, 3, 5, 0,                   //  9668:      check Inst[7:3] == 0x0
  OPC_Decode, 140, 18, 174, 1,               //  9672:      decode to S2_storerb_pcr using decoder 174
                                             //  9672:     }
                                             //  9672:     } // switch Inst[2:0]
                                             //  9672:    }
  1, 69,                                     //  9677:    case 0x1: {
  OPC_SwitchField, 2, 1,                     //  9679:     switch Inst[2] {
  0, 37,                                     //  9682:     case 0x0: {
  OPC_SwitchField, 7, 1,                     //  9684:      switch Inst[7] {
  0, 21,                                     //  9687:      case 0x0: {
  OPC_SwitchField, 13, 1,                    //  9689:       switch Inst[13] {
  0, 9,                                      //  9692:       case 0x0: {
  OPC_CheckField, 0, 2, 0,                   //  9694:        check Inst[1:0] == 0x0
  OPC_Decode, 141, 18, 175, 1,               //  9698:        decode to S2_storerb_pi using decoder 175
                                             //  9698:       }
  1, 0,                                      //  9703:       case 0x1: {
  OPC_Decode, 221, 17, 176, 1,               //  9705:        decode to S2_pstorerbt_pi using decoder 176
                                             //  9705:       }
                                             //  9705:       } // switch Inst[13]
                                             //  9705:      }
  1, 0,                                      //  9710:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  //  9712:       check Inst[13] == 0x1
  OPC_Decode, 222, 17, 176, 1,               //  9716:       decode to S2_pstorerbtnew_pi using decoder 176
                                             //  9716:      }
                                             //  9716:      } // switch Inst[7]
                                             //  9716:     }
  1, 0,                                      //  9721:     case 0x1: {
  OPC_SwitchField, 7, 1,                     //  9723:      switch Inst[7] {
  0, 9,                                      //  9726:      case 0x0: {
  OPC_CheckField, 13, 1, 1,                  //  9728:       check Inst[13] == 0x1
  OPC_Decode, 212, 17, 176, 1,               //  9732:       decode to S2_pstorerbf_pi using decoder 176
                                             //  9732:      }
  1, 0,                                      //  9737:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  //  9739:       check Inst[13] == 0x1
  OPC_Decode, 213, 17, 176, 1,               //  9743:       decode to S2_pstorerbfnew_pi using decoder 176
                                             //  9743:      }
                                             //  9743:      } // switch Inst[7]
                                             //  9743:     }
                                             //  9743:     } // switch Inst[2]
                                             //  9743:    }
  2, 9,                                      //  9748:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   //  9750:     check Inst[7:0] == 0x0
  OPC_Decode, 142, 18, 174, 1,               //  9754:     decode to S2_storerb_pr using decoder 174
                                             //  9754:    }
  3, 0,                                      //  9759:    case 0x3: {
  OPC_CheckField, 0, 8, 0,                   //  9761:     check Inst[7:0] == 0x0
  OPC_Decode, 138, 18, 174, 1,               //  9765:     decode to S2_storerb_pbr using decoder 174
                                             //  9765:    }
                                             //  9765:    } // switch Inst[26:25]
                                             //  9765:   }
                                             //  9765:   } // switch Inst[27]
                                             //  9765:  }
  10, 135, 1,                                //  9770:  case 0xa: {
  OPC_SwitchField, 27, 1,                    //  9773:   switch Inst[27] {
  0, 5,                                      //  9776:   case 0x0: {
  OPC_Decode, 165, 18, 177, 1,               //  9778:    decode to S2_storerh_io using decoder 177
                                             //  9778:   }
  1, 0,                                      //  9783:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  9785:    switch Inst[26:25] {
  0, 25,                                     //  9788:    case 0x0: {
  OPC_SwitchField, 0, 3,                     //  9790:     switch Inst[2:0] {
  0, 9,                                      //  9793:     case 0x0: {
  OPC_CheckField, 7, 1, 0,                   //  9795:      check Inst[7] == 0x0
  OPC_Decode, 167, 18, 178, 1,               //  9799:      decode to S2_storerh_pci using decoder 178
                                             //  9799:     }
  2, 0,                                      //  9804:     case 0x2: {
  OPC_CheckField, 3, 5, 0,                   //  9806:      check Inst[7:3] == 0x0
  OPC_Decode, 168, 18, 174, 1,               //  9810:      decode to S2_storerh_pcr using decoder 174
                                             //  9810:     }
                                             //  9810:     } // switch Inst[2:0]
                                             //  9810:    }
  1, 69,                                     //  9815:    case 0x1: {
  OPC_SwitchField, 2, 1,                     //  9817:     switch Inst[2] {
  0, 37,                                     //  9820:     case 0x0: {
  OPC_SwitchField, 7, 1,                     //  9822:      switch Inst[7] {
  0, 21,                                     //  9825:      case 0x0: {
  OPC_SwitchField, 13, 1,                    //  9827:       switch Inst[13] {
  0, 9,                                      //  9830:       case 0x0: {
  OPC_CheckField, 0, 2, 0,                   //  9832:        check Inst[1:0] == 0x0
  OPC_Decode, 169, 18, 179, 1,               //  9836:        decode to S2_storerh_pi using decoder 179
                                             //  9836:       }
  1, 0,                                      //  9841:       case 0x1: {
  OPC_Decode, 245, 17, 180, 1,               //  9843:        decode to S2_pstorerht_pi using decoder 180
                                             //  9843:       }
                                             //  9843:       } // switch Inst[13]
                                             //  9843:      }
  1, 0,                                      //  9848:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  //  9850:       check Inst[13] == 0x1
  OPC_Decode, 246, 17, 180, 1,               //  9854:       decode to S2_pstorerhtnew_pi using decoder 180
                                             //  9854:      }
                                             //  9854:      } // switch Inst[7]
                                             //  9854:     }
  1, 0,                                      //  9859:     case 0x1: {
  OPC_SwitchField, 7, 1,                     //  9861:      switch Inst[7] {
  0, 9,                                      //  9864:      case 0x0: {
  OPC_CheckField, 13, 1, 1,                  //  9866:       check Inst[13] == 0x1
  OPC_Decode, 236, 17, 180, 1,               //  9870:       decode to S2_pstorerhf_pi using decoder 180
                                             //  9870:      }
  1, 0,                                      //  9875:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  //  9877:       check Inst[13] == 0x1
  OPC_Decode, 237, 17, 180, 1,               //  9881:       decode to S2_pstorerhfnew_pi using decoder 180
                                             //  9881:      }
                                             //  9881:      } // switch Inst[7]
                                             //  9881:     }
                                             //  9881:     } // switch Inst[2]
                                             //  9881:    }
  2, 9,                                      //  9886:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   //  9888:     check Inst[7:0] == 0x0
  OPC_Decode, 170, 18, 174, 1,               //  9892:     decode to S2_storerh_pr using decoder 174
                                             //  9892:    }
  3, 0,                                      //  9897:    case 0x3: {
  OPC_CheckField, 0, 8, 0,                   //  9899:     check Inst[7:0] == 0x0
  OPC_Decode, 166, 18, 174, 1,               //  9903:     decode to S2_storerh_pbr using decoder 174
                                             //  9903:    }
                                             //  9903:    } // switch Inst[26:25]
                                             //  9903:   }
                                             //  9903:   } // switch Inst[27]
                                             //  9903:  }
  11, 135, 1,                                //  9908:  case 0xb: {
  OPC_SwitchField, 27, 1,                    //  9911:   switch Inst[27] {
  0, 5,                                      //  9914:   case 0x0: {
  OPC_Decode, 158, 18, 177, 1,               //  9916:    decode to S2_storerf_io using decoder 177
                                             //  9916:   }
  1, 0,                                      //  9921:   case 0x1: {
  OPC_SwitchField, 25, 2,                    //  9923:    switch Inst[26:25] {
  0, 25,                                     //  9926:    case 0x0: {
  OPC_SwitchField, 0, 3,                     //  9928:     switch Inst[2:0] {
  0, 9,                                      //  9931:     case 0x0: {
  OPC_CheckField, 7, 1, 0,                   //  9933:      check Inst[7] == 0x0
  OPC_Decode, 160, 18, 178, 1,               //  9937:      decode to S2_storerf_pci using decoder 178
                                             //  9937:     }
  2, 0,                                      //  9942:     case 0x2: {
  OPC_CheckField, 3, 5, 0,                   //  9944:      check Inst[7:3] == 0x0
  OPC_Decode, 161, 18, 174, 1,               //  9948:      decode to S2_storerf_pcr using decoder 174
                                             //  9948:     }
                                             //  9948:     } // switch Inst[2:0]
                                             //  9948:    }
  1, 69,                                     //  9953:    case 0x1: {
  OPC_SwitchField, 2, 1,                     //  9955:     switch Inst[2] {
  0, 37,                                     //  9958:     case 0x0: {
  OPC_SwitchField, 7, 1,                     //  9960:      switch Inst[7] {
  0, 21,                                     //  9963:      case 0x0: {
  OPC_SwitchField, 13, 1,                    //  9965:       switch Inst[13] {
  0, 9,                                      //  9968:       case 0x0: {
  OPC_CheckField, 0, 2, 0,                   //  9970:        check Inst[1:0] == 0x0
  OPC_Decode, 162, 18, 179, 1,               //  9974:        decode to S2_storerf_pi using decoder 179
                                             //  9974:       }
  1, 0,                                      //  9979:       case 0x1: {
  OPC_Decode, 233, 17, 180, 1,               //  9981:        decode to S2_pstorerft_pi using decoder 180
                                             //  9981:       }
                                             //  9981:       } // switch Inst[13]
                                             //  9981:      }
  1, 0,                                      //  9986:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  //  9988:       check Inst[13] == 0x1
  OPC_Decode, 234, 17, 180, 1,               //  9992:       decode to S2_pstorerftnew_pi using decoder 180
                                             //  9992:      }
                                             //  9992:      } // switch Inst[7]
                                             //  9992:     }
  1, 0,                                      //  9997:     case 0x1: {
  OPC_SwitchField, 7, 1,                     //  9999:      switch Inst[7] {
  0, 9,                                      // 10002:      case 0x0: {
  OPC_CheckField, 13, 1, 1,                  // 10004:       check Inst[13] == 0x1
  OPC_Decode, 230, 17, 180, 1,               // 10008:       decode to S2_pstorerff_pi using decoder 180
                                             // 10008:      }
  1, 0,                                      // 10013:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10015:       check Inst[13] == 0x1
  OPC_Decode, 231, 17, 180, 1,               // 10019:       decode to S2_pstorerffnew_pi using decoder 180
                                             // 10019:      }
                                             // 10019:      } // switch Inst[7]
                                             // 10019:     }
                                             // 10019:     } // switch Inst[2]
                                             // 10019:    }
  2, 9,                                      // 10024:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   // 10026:     check Inst[7:0] == 0x0
  OPC_Decode, 163, 18, 174, 1,               // 10030:     decode to S2_storerf_pr using decoder 174
                                             // 10030:    }
  3, 0,                                      // 10035:    case 0x3: {
  OPC_CheckField, 0, 8, 0,                   // 10037:     check Inst[7:0] == 0x0
  OPC_Decode, 159, 18, 174, 1,               // 10041:     decode to S2_storerf_pbr using decoder 174
                                             // 10041:    }
                                             // 10041:    } // switch Inst[26:25]
                                             // 10041:   }
                                             // 10041:   } // switch Inst[27]
                                             // 10041:  }
  12, 135, 1,                                // 10046:  case 0xc: {
  OPC_SwitchField, 27, 1,                    // 10049:   switch Inst[27] {
  0, 5,                                      // 10052:   case 0x0: {
  OPC_Decode, 179, 18, 181, 1,               // 10054:    decode to S2_storeri_io using decoder 181
                                             // 10054:   }
  1, 0,                                      // 10059:   case 0x1: {
  OPC_SwitchField, 25, 2,                    // 10061:    switch Inst[26:25] {
  0, 25,                                     // 10064:    case 0x0: {
  OPC_SwitchField, 0, 3,                     // 10066:     switch Inst[2:0] {
  0, 9,                                      // 10069:     case 0x0: {
  OPC_CheckField, 7, 1, 0,                   // 10071:      check Inst[7] == 0x0
  OPC_Decode, 181, 18, 182, 1,               // 10075:      decode to S2_storeri_pci using decoder 182
                                             // 10075:     }
  2, 0,                                      // 10080:     case 0x2: {
  OPC_CheckField, 3, 5, 0,                   // 10082:      check Inst[7:3] == 0x0
  OPC_Decode, 182, 18, 174, 1,               // 10086:      decode to S2_storeri_pcr using decoder 174
                                             // 10086:     }
                                             // 10086:     } // switch Inst[2:0]
                                             // 10086:    }
  1, 69,                                     // 10091:    case 0x1: {
  OPC_SwitchField, 2, 1,                     // 10093:     switch Inst[2] {
  0, 37,                                     // 10096:     case 0x0: {
  OPC_SwitchField, 7, 1,                     // 10098:      switch Inst[7] {
  0, 21,                                     // 10101:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 10103:       switch Inst[13] {
  0, 9,                                      // 10106:       case 0x0: {
  OPC_CheckField, 0, 2, 0,                   // 10108:        check Inst[1:0] == 0x0
  OPC_Decode, 183, 18, 183, 1,               // 10112:        decode to S2_storeri_pi using decoder 183
                                             // 10112:       }
  1, 0,                                      // 10117:       case 0x1: {
  OPC_Decode, 129, 18, 184, 1,               // 10119:        decode to S2_pstorerit_pi using decoder 184
                                             // 10119:       }
                                             // 10119:       } // switch Inst[13]
                                             // 10119:      }
  1, 0,                                      // 10124:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10126:       check Inst[13] == 0x1
  OPC_Decode, 130, 18, 184, 1,               // 10130:       decode to S2_pstoreritnew_pi using decoder 184
                                             // 10130:      }
                                             // 10130:      } // switch Inst[7]
                                             // 10130:     }
  1, 0,                                      // 10135:     case 0x1: {
  OPC_SwitchField, 7, 1,                     // 10137:      switch Inst[7] {
  0, 9,                                      // 10140:      case 0x0: {
  OPC_CheckField, 13, 1, 1,                  // 10142:       check Inst[13] == 0x1
  OPC_Decode, 248, 17, 184, 1,               // 10146:       decode to S2_pstorerif_pi using decoder 184
                                             // 10146:      }
  1, 0,                                      // 10151:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10153:       check Inst[13] == 0x1
  OPC_Decode, 249, 17, 184, 1,               // 10157:       decode to S2_pstorerifnew_pi using decoder 184
                                             // 10157:      }
                                             // 10157:      } // switch Inst[7]
                                             // 10157:     }
                                             // 10157:     } // switch Inst[2]
                                             // 10157:    }
  2, 9,                                      // 10162:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   // 10164:     check Inst[7:0] == 0x0
  OPC_Decode, 184, 18, 174, 1,               // 10168:     decode to S2_storeri_pr using decoder 174
                                             // 10168:    }
  3, 0,                                      // 10173:    case 0x3: {
  OPC_CheckField, 0, 8, 0,                   // 10175:     check Inst[7:0] == 0x0
  OPC_Decode, 180, 18, 174, 1,               // 10179:     decode to S2_storeri_pbr using decoder 174
                                             // 10179:    }
                                             // 10179:    } // switch Inst[26:25]
                                             // 10179:   }
                                             // 10179:   } // switch Inst[27]
                                             // 10179:  }
  13, 160, 3,                                // 10184:  case 0xd: {
  OPC_SwitchField, 11, 2,                    // 10187:   switch Inst[12:11] {
  0, 135, 1,                                 // 10190:   case 0x0: {
  OPC_SwitchField, 27, 1,                    // 10193:    switch Inst[27] {
  0, 5,                                      // 10196:    case 0x0: {
  OPC_Decode, 144, 18, 185, 1,               // 10198:     decode to S2_storerbnew_io using decoder 185
                                             // 10198:    }
  1, 0,                                      // 10203:    case 0x1: {
  OPC_SwitchField, 25, 2,                    // 10205:     switch Inst[26:25] {
  0, 25,                                     // 10208:     case 0x0: {
  OPC_SwitchField, 0, 3,                     // 10210:      switch Inst[2:0] {
  0, 9,                                      // 10213:      case 0x0: {
  OPC_CheckField, 7, 1, 0,                   // 10215:       check Inst[7] == 0x0
  OPC_Decode, 146, 18, 186, 1,               // 10219:       decode to S2_storerbnew_pci using decoder 186
                                             // 10219:      }
  2, 0,                                      // 10224:      case 0x2: {
  OPC_CheckField, 3, 5, 0,                   // 10226:       check Inst[7:3] == 0x0
  OPC_Decode, 147, 18, 187, 1,               // 10230:       decode to S2_storerbnew_pcr using decoder 187
                                             // 10230:      }
                                             // 10230:      } // switch Inst[2:0]
                                             // 10230:     }
  1, 69,                                     // 10235:     case 0x1: {
  OPC_SwitchField, 2, 1,                     // 10237:      switch Inst[2] {
  0, 37,                                     // 10240:      case 0x0: {
  OPC_SwitchField, 7, 1,                     // 10242:       switch Inst[7] {
  0, 21,                                     // 10245:       case 0x0: {
  OPC_SwitchField, 13, 1,                    // 10247:        switch Inst[13] {
  0, 9,                                      // 10250:        case 0x0: {
  OPC_CheckField, 0, 2, 0,                   // 10252:         check Inst[1:0] == 0x0
  OPC_Decode, 148, 18, 188, 1,               // 10256:         decode to S2_storerbnew_pi using decoder 188
                                             // 10256:        }
  1, 0,                                      // 10261:        case 0x1: {
  OPC_Decode, 218, 17, 189, 1,               // 10263:         decode to S2_pstorerbnewt_pi using decoder 189
                                             // 10263:        }
                                             // 10263:        } // switch Inst[13]
                                             // 10263:       }
  1, 0,                                      // 10268:       case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10270:        check Inst[13] == 0x1
  OPC_Decode, 219, 17, 189, 1,               // 10274:        decode to S2_pstorerbnewtnew_pi using decoder 189
                                             // 10274:       }
                                             // 10274:       } // switch Inst[7]
                                             // 10274:      }
  1, 0,                                      // 10279:      case 0x1: {
  OPC_SwitchField, 7, 1,                     // 10281:       switch Inst[7] {
  0, 9,                                      // 10284:       case 0x0: {
  OPC_CheckField, 13, 1, 1,                  // 10286:        check Inst[13] == 0x1
  OPC_Decode, 215, 17, 189, 1,               // 10290:        decode to S2_pstorerbnewf_pi using decoder 189
                                             // 10290:       }
  1, 0,                                      // 10295:       case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10297:        check Inst[13] == 0x1
  OPC_Decode, 216, 17, 189, 1,               // 10301:        decode to S2_pstorerbnewfnew_pi using decoder 189
                                             // 10301:       }
                                             // 10301:       } // switch Inst[7]
                                             // 10301:      }
                                             // 10301:      } // switch Inst[2]
                                             // 10301:     }
  2, 9,                                      // 10306:     case 0x2: {
  OPC_CheckField, 0, 8, 0,                   // 10308:      check Inst[7:0] == 0x0
  OPC_Decode, 149, 18, 187, 1,               // 10312:      decode to S2_storerbnew_pr using decoder 187
                                             // 10312:     }
  3, 0,                                      // 10317:     case 0x3: {
  OPC_CheckField, 0, 8, 0,                   // 10319:      check Inst[7:0] == 0x0
  OPC_Decode, 145, 18, 187, 1,               // 10323:      decode to S2_storerbnew_pbr using decoder 187
                                             // 10323:     }
                                             // 10323:     } // switch Inst[26:25]
                                             // 10323:    }
                                             // 10323:    } // switch Inst[27]
                                             // 10323:   }
  1, 135, 1,                                 // 10328:   case 0x1: {
  OPC_SwitchField, 27, 1,                    // 10331:    switch Inst[27] {
  0, 5,                                      // 10334:    case 0x0: {
  OPC_Decode, 172, 18, 190, 1,               // 10336:     decode to S2_storerhnew_io using decoder 190
                                             // 10336:    }
  1, 0,                                      // 10341:    case 0x1: {
  OPC_SwitchField, 25, 2,                    // 10343:     switch Inst[26:25] {
  0, 25,                                     // 10346:     case 0x0: {
  OPC_SwitchField, 0, 3,                     // 10348:      switch Inst[2:0] {
  0, 9,                                      // 10351:      case 0x0: {
  OPC_CheckField, 7, 1, 0,                   // 10353:       check Inst[7] == 0x0
  OPC_Decode, 174, 18, 191, 1,               // 10357:       decode to S2_storerhnew_pci using decoder 191
                                             // 10357:      }
  2, 0,                                      // 10362:      case 0x2: {
  OPC_CheckField, 3, 5, 0,                   // 10364:       check Inst[7:3] == 0x0
  OPC_Decode, 175, 18, 187, 1,               // 10368:       decode to S2_storerhnew_pcr using decoder 187
                                             // 10368:      }
                                             // 10368:      } // switch Inst[2:0]
                                             // 10368:     }
  1, 69,                                     // 10373:     case 0x1: {
  OPC_SwitchField, 2, 1,                     // 10375:      switch Inst[2] {
  0, 37,                                     // 10378:      case 0x0: {
  OPC_SwitchField, 7, 1,                     // 10380:       switch Inst[7] {
  0, 21,                                     // 10383:       case 0x0: {
  OPC_SwitchField, 13, 1,                    // 10385:        switch Inst[13] {
  0, 9,                                      // 10388:        case 0x0: {
  OPC_CheckField, 0, 2, 0,                   // 10390:         check Inst[1:0] == 0x0
  OPC_Decode, 176, 18, 192, 1,               // 10394:         decode to S2_storerhnew_pi using decoder 192
                                             // 10394:        }
  1, 0,                                      // 10399:        case 0x1: {
  OPC_Decode, 242, 17, 193, 1,               // 10401:         decode to S2_pstorerhnewt_pi using decoder 193
                                             // 10401:        }
                                             // 10401:        } // switch Inst[13]
                                             // 10401:       }
  1, 0,                                      // 10406:       case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10408:        check Inst[13] == 0x1
  OPC_Decode, 243, 17, 193, 1,               // 10412:        decode to S2_pstorerhnewtnew_pi using decoder 193
                                             // 10412:       }
                                             // 10412:       } // switch Inst[7]
                                             // 10412:      }
  1, 0,                                      // 10417:      case 0x1: {
  OPC_SwitchField, 7, 1,                     // 10419:       switch Inst[7] {
  0, 9,                                      // 10422:       case 0x0: {
  OPC_CheckField, 13, 1, 1,                  // 10424:        check Inst[13] == 0x1
  OPC_Decode, 239, 17, 193, 1,               // 10428:        decode to S2_pstorerhnewf_pi using decoder 193
                                             // 10428:       }
  1, 0,                                      // 10433:       case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10435:        check Inst[13] == 0x1
  OPC_Decode, 240, 17, 193, 1,               // 10439:        decode to S2_pstorerhnewfnew_pi using decoder 193
                                             // 10439:       }
                                             // 10439:       } // switch Inst[7]
                                             // 10439:      }
                                             // 10439:      } // switch Inst[2]
                                             // 10439:     }
  2, 9,                                      // 10444:     case 0x2: {
  OPC_CheckField, 0, 8, 0,                   // 10446:      check Inst[7:0] == 0x0
  OPC_Decode, 177, 18, 187, 1,               // 10450:      decode to S2_storerhnew_pr using decoder 187
                                             // 10450:     }
  3, 0,                                      // 10455:     case 0x3: {
  OPC_CheckField, 0, 8, 0,                   // 10457:      check Inst[7:0] == 0x0
  OPC_Decode, 173, 18, 187, 1,               // 10461:      decode to S2_storerhnew_pbr using decoder 187
                                             // 10461:     }
                                             // 10461:     } // switch Inst[26:25]
                                             // 10461:    }
                                             // 10461:    } // switch Inst[27]
                                             // 10461:   }
  2, 0,                                      // 10466:   case 0x2: {
  OPC_SwitchField, 27, 1,                    // 10468:    switch Inst[27] {
  0, 5,                                      // 10471:    case 0x0: {
  OPC_Decode, 186, 18, 194, 1,               // 10473:     decode to S2_storerinew_io using decoder 194
                                             // 10473:    }
  1, 0,                                      // 10478:    case 0x1: {
  OPC_SwitchField, 25, 2,                    // 10480:     switch Inst[26:25] {
  0, 25,                                     // 10483:     case 0x0: {
  OPC_SwitchField, 0, 3,                     // 10485:      switch Inst[2:0] {
  0, 9,                                      // 10488:      case 0x0: {
  OPC_CheckField, 7, 1, 0,                   // 10490:       check Inst[7] == 0x0
  OPC_Decode, 188, 18, 195, 1,               // 10494:       decode to S2_storerinew_pci using decoder 195
                                             // 10494:      }
  2, 0,                                      // 10499:      case 0x2: {
  OPC_CheckField, 3, 5, 0,                   // 10501:       check Inst[7:3] == 0x0
  OPC_Decode, 189, 18, 187, 1,               // 10505:       decode to S2_storerinew_pcr using decoder 187
                                             // 10505:      }
                                             // 10505:      } // switch Inst[2:0]
                                             // 10505:     }
  1, 69,                                     // 10510:     case 0x1: {
  OPC_SwitchField, 2, 1,                     // 10512:      switch Inst[2] {
  0, 37,                                     // 10515:      case 0x0: {
  OPC_SwitchField, 7, 1,                     // 10517:       switch Inst[7] {
  0, 21,                                     // 10520:       case 0x0: {
  OPC_SwitchField, 13, 1,                    // 10522:        switch Inst[13] {
  0, 9,                                      // 10525:        case 0x0: {
  OPC_CheckField, 0, 2, 0,                   // 10527:         check Inst[1:0] == 0x0
  OPC_Decode, 190, 18, 196, 1,               // 10531:         decode to S2_storerinew_pi using decoder 196
                                             // 10531:        }
  1, 0,                                      // 10536:        case 0x1: {
  OPC_Decode, 254, 17, 197, 1,               // 10538:         decode to S2_pstorerinewt_pi using decoder 197
                                             // 10538:        }
                                             // 10538:        } // switch Inst[13]
                                             // 10538:       }
  1, 0,                                      // 10543:       case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10545:        check Inst[13] == 0x1
  OPC_Decode, 255, 17, 197, 1,               // 10549:        decode to S2_pstorerinewtnew_pi using decoder 197
                                             // 10549:       }
                                             // 10549:       } // switch Inst[7]
                                             // 10549:      }
  1, 0,                                      // 10554:      case 0x1: {
  OPC_SwitchField, 7, 1,                     // 10556:       switch Inst[7] {
  0, 9,                                      // 10559:       case 0x0: {
  OPC_CheckField, 13, 1, 1,                  // 10561:        check Inst[13] == 0x1
  OPC_Decode, 251, 17, 197, 1,               // 10565:        decode to S2_pstorerinewf_pi using decoder 197
                                             // 10565:       }
  1, 0,                                      // 10570:       case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10572:        check Inst[13] == 0x1
  OPC_Decode, 252, 17, 197, 1,               // 10576:        decode to S2_pstorerinewfnew_pi using decoder 197
                                             // 10576:       }
                                             // 10576:       } // switch Inst[7]
                                             // 10576:      }
                                             // 10576:      } // switch Inst[2]
                                             // 10576:     }
  2, 9,                                      // 10581:     case 0x2: {
  OPC_CheckField, 0, 8, 0,                   // 10583:      check Inst[7:0] == 0x0
  OPC_Decode, 191, 18, 187, 1,               // 10587:      decode to S2_storerinew_pr using decoder 187
                                             // 10587:     }
  3, 0,                                      // 10592:     case 0x3: {
  OPC_CheckField, 0, 8, 0,                   // 10594:      check Inst[7:0] == 0x0
  OPC_Decode, 187, 18, 187, 1,               // 10598:      decode to S2_storerinew_pbr using decoder 187
                                             // 10598:     }
                                             // 10598:     } // switch Inst[26:25]
                                             // 10598:    }
                                             // 10598:    } // switch Inst[27]
                                             // 10598:   }
                                             // 10598:   } // switch Inst[12:11]
                                             // 10598:  }
  14, 0,                                     // 10603:  case 0xe: {
  OPC_SwitchField, 27, 1,                    // 10605:   switch Inst[27] {
  0, 5,                                      // 10608:   case 0x0: {
  OPC_Decode, 151, 18, 198, 1,               // 10610:    decode to S2_storerd_io using decoder 198
                                             // 10610:   }
  1, 0,                                      // 10615:   case 0x1: {
  OPC_SwitchField, 25, 2,                    // 10617:    switch Inst[26:25] {
  0, 25,                                     // 10620:    case 0x0: {
  OPC_SwitchField, 0, 3,                     // 10622:     switch Inst[2:0] {
  0, 9,                                      // 10625:     case 0x0: {
  OPC_CheckField, 7, 1, 0,                   // 10627:      check Inst[7] == 0x0
  OPC_Decode, 153, 18, 199, 1,               // 10631:      decode to S2_storerd_pci using decoder 199
                                             // 10631:     }
  2, 0,                                      // 10636:     case 0x2: {
  OPC_CheckField, 3, 5, 0,                   // 10638:      check Inst[7:3] == 0x0
  OPC_Decode, 154, 18, 200, 1,               // 10642:      decode to S2_storerd_pcr using decoder 200
                                             // 10642:     }
                                             // 10642:     } // switch Inst[2:0]
                                             // 10642:    }
  1, 69,                                     // 10647:    case 0x1: {
  OPC_SwitchField, 2, 1,                     // 10649:     switch Inst[2] {
  0, 37,                                     // 10652:     case 0x0: {
  OPC_SwitchField, 7, 1,                     // 10654:      switch Inst[7] {
  0, 21,                                     // 10657:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 10659:       switch Inst[13] {
  0, 9,                                      // 10662:       case 0x0: {
  OPC_CheckField, 0, 2, 0,                   // 10664:        check Inst[1:0] == 0x0
  OPC_Decode, 155, 18, 201, 1,               // 10668:        decode to S2_storerd_pi using decoder 201
                                             // 10668:       }
  1, 0,                                      // 10673:       case 0x1: {
  OPC_Decode, 227, 17, 202, 1,               // 10675:        decode to S2_pstorerdt_pi using decoder 202
                                             // 10675:       }
                                             // 10675:       } // switch Inst[13]
                                             // 10675:      }
  1, 0,                                      // 10680:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10682:       check Inst[13] == 0x1
  OPC_Decode, 228, 17, 202, 1,               // 10686:       decode to S2_pstorerdtnew_pi using decoder 202
                                             // 10686:      }
                                             // 10686:      } // switch Inst[7]
                                             // 10686:     }
  1, 0,                                      // 10691:     case 0x1: {
  OPC_SwitchField, 7, 1,                     // 10693:      switch Inst[7] {
  0, 9,                                      // 10696:      case 0x0: {
  OPC_CheckField, 13, 1, 1,                  // 10698:       check Inst[13] == 0x1
  OPC_Decode, 224, 17, 202, 1,               // 10702:       decode to S2_pstorerdf_pi using decoder 202
                                             // 10702:      }
  1, 0,                                      // 10707:      case 0x1: {
  OPC_CheckField, 13, 1, 1,                  // 10709:       check Inst[13] == 0x1
  OPC_Decode, 225, 17, 202, 1,               // 10713:       decode to S2_pstorerdfnew_pi using decoder 202
                                             // 10713:      }
                                             // 10713:      } // switch Inst[7]
                                             // 10713:     }
                                             // 10713:     } // switch Inst[2]
                                             // 10713:    }
  2, 9,                                      // 10718:    case 0x2: {
  OPC_CheckField, 0, 8, 0,                   // 10720:     check Inst[7:0] == 0x0
  OPC_Decode, 156, 18, 200, 1,               // 10724:     decode to S2_storerd_pr using decoder 200
                                             // 10724:    }
  3, 0,                                      // 10729:    case 0x3: {
  OPC_CheckField, 0, 8, 0,                   // 10731:     check Inst[7:0] == 0x0
  OPC_Decode, 152, 18, 200, 1,               // 10735:     decode to S2_storerd_pbr using decoder 200
                                             // 10735:    }
                                             // 10735:    } // switch Inst[26:25]
                                             // 10735:   }
                                             // 10735:   } // switch Inst[27]
                                             // 10735:  }
                                             // 10735:  } // switch Inst[24:21]
                                             // 10735: }
  11, 5,                                     // 10740: case 0xb: {
  OPC_Decode, 178, 7, 203, 1,                // 10742:  decode to A2_addi using decoder 203
                                             // 10742: }
  12, 181, 12,                               // 10747: case 0xc: {
  OPC_SwitchField, 21, 7,                    // 10750:  switch Inst[27:21] {
  0, 9,                                      // 10753:  case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 10755:   check Inst[13] == 0x0
  OPC_Decode, 206, 18, 204, 1,               // 10759:   decode to S2_valignib using decoder 204
                                             // 10759:  }
  4, 9,                                      // 10764:  case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 10766:   check Inst[13] == 0x0
  OPC_Decode, 223, 18, 205, 1,               // 10770:   decode to S2_vspliceib using decoder 205
                                             // 10770:  }
  8, 47,                                     // 10775:  case 0x8: {
  OPC_SwitchField, 5, 3,                     // 10777:   switch Inst[7:5] {
  0, 9,                                      // 10780:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 10782:    check Inst[13] == 0x0
  OPC_Decode, 161, 17, 206, 1,               // 10786:    decode to S2_extractup_rp using decoder 206
                                             // 10786:   }
  2, 9,                                      // 10791:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 10793:    check Inst[13] == 0x0
  OPC_Decode, 133, 18, 206, 1,               // 10797:    decode to S2_shuffeb using decoder 206
                                             // 10797:   }
  4, 9,                                      // 10802:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 10804:    check Inst[13] == 0x0
  OPC_Decode, 135, 18, 207, 1,               // 10808:    decode to S2_shuffob using decoder 207
                                             // 10808:   }
  6, 0,                                      // 10813:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 10815:    check Inst[13] == 0x0
  OPC_Decode, 134, 18, 206, 1,               // 10819:    decode to S2_shuffeh using decoder 206
                                             // 10819:   }
                                             // 10819:   } // switch Inst[7:5]
                                             // 10819:  }
  10, 58,                                    // 10824:  case 0xa: {
  OPC_SwitchField, 5, 3,                     // 10826:   switch Inst[7:5] {
  0, 9,                                      // 10829:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 10831:    check Inst[13] == 0x0
  OPC_Decode, 255, 19, 206, 1,               // 10835:    decode to S4_vxaddsubw using decoder 206
                                             // 10835:   }
  1, 9,                                      // 10840:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 10842:    check Inst[13] == 0x0
  OPC_Decode, 167, 9, 208, 1,                // 10846:    decode to A5_vaddhubs using decoder 208
                                             // 10846:   }
  2, 9,                                      // 10851:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 10853:    check Inst[13] == 0x0
  OPC_Decode, 130, 20, 206, 1,               // 10857:    decode to S4_vxsubaddw using decoder 206
                                             // 10857:   }
  4, 9,                                      // 10862:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 10864:    check Inst[13] == 0x0
  OPC_Decode, 253, 19, 206, 1,               // 10868:    decode to S4_vxaddsubh using decoder 206
                                             // 10868:   }
  6, 0,                                      // 10873:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 10875:    check Inst[13] == 0x0
  OPC_Decode, 128, 20, 206, 1,               // 10879:    decode to S4_vxsubaddh using decoder 206
                                             // 10879:   }
                                             // 10879:   } // switch Inst[7:5]
                                             // 10879:  }
  12, 73,                                    // 10884:  case 0xc: {
  OPC_SwitchField, 5, 3,                     // 10886:   switch Inst[7:5] {
  0, 9,                                      // 10889:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 10891:    check Inst[13] == 0x0
  OPC_Decode, 136, 18, 207, 1,               // 10895:    decode to S2_shuffoh using decoder 207
                                             // 10895:   }
  2, 9,                                      // 10900:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 10902:    check Inst[13] == 0x0
  OPC_Decode, 228, 18, 206, 1,               // 10906:    decode to S2_vtrunewh using decoder 206
                                             // 10906:   }
  3, 11,                                     // 10911:   case 0x3: {
  OPC_CheckPredicate, 7,                     // 10913:    check predicate 7
  OPC_CheckField, 13, 1, 0,                  // 10915:    check Inst[13] == 0x0
  OPC_Decode, 148, 20, 206, 1,               // 10919:    decode to S6_vtrunehb_ppp using decoder 206
                                             // 10919:   }
  4, 9,                                      // 10924:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 10926:    check Inst[13] == 0x0
  OPC_Decode, 230, 18, 206, 1,               // 10930:    decode to S2_vtrunowh using decoder 206
                                             // 10930:   }
  5, 11,                                     // 10935:   case 0x5: {
  OPC_CheckPredicate, 7,                     // 10937:    check predicate 7
  OPC_CheckField, 13, 1, 0,                  // 10939:    check Inst[13] == 0x0
  OPC_Decode, 149, 20, 206, 1,               // 10943:    decode to S6_vtrunohb_ppp using decoder 206
                                             // 10943:   }
  6, 0,                                      // 10948:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 10950:    check Inst[13] == 0x0
  OPC_Decode, 167, 17, 206, 1,               // 10954:    decode to S2_lfsp using decoder 206
                                             // 10954:   }
                                             // 10954:   } // switch Inst[7:5]
                                             // 10954:  }
  14, 49,                                    // 10959:  case 0xe: {
  OPC_SwitchField, 5, 3,                     // 10961:   switch Inst[7:5] {
  0, 9,                                      // 10964:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 10966:    check Inst[13] == 0x0
  OPC_Decode, 254, 19, 206, 1,               // 10970:    decode to S4_vxaddsubhr using decoder 206
                                             // 10970:   }
  2, 9,                                      // 10975:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 10977:    check Inst[13] == 0x0
  OPC_Decode, 129, 20, 206, 1,               // 10981:    decode to S4_vxsubaddhr using decoder 206
                                             // 10981:   }
  4, 9,                                      // 10986:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 10988:    check Inst[13] == 0x0
  OPC_Decode, 244, 18, 206, 1,               // 10992:    decode to S4_extractp_rp using decoder 206
                                             // 10992:   }
  6, 0,                                      // 10997:   case 0x6: {
  OPC_CheckPredicate, 10,                    // 10999:    check predicate 10
  OPC_CheckField, 13, 1, 0,                  // 11001:    check Inst[13] == 0x0
  OPC_Decode, 143, 17, 206, 1,               // 11005:    decode to S2_cabacdecbin using decoder 206
                                             // 11005:   }
                                             // 11005:   } // switch Inst[7:5]
                                             // 11005:  }
  16, 13,                                    // 11010:  case 0x10: {
  OPC_CheckField, 13, 1, 0,                  // 11012:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 11016:   check Inst[7] == 0x0
  OPC_Decode, 207, 18, 209, 1,               // 11020:   decode to S2_valignrb using decoder 209
                                             // 11020:  }
  20, 13,                                    // 11025:  case 0x14: {
  OPC_CheckField, 13, 1, 0,                  // 11027:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 11031:   check Inst[7] == 0x0
  OPC_Decode, 224, 18, 210, 1,               // 11035:   decode to S2_vsplicerb using decoder 210
                                             // 11035:  }
  22, 13,                                    // 11040:  case 0x16: {
  OPC_CheckField, 13, 1, 0,                  // 11042:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 11046:   check Inst[7] == 0x0
  OPC_Decode, 211, 8, 211, 1,                // 11050:   decode to A4_addp_c using decoder 211
                                             // 11050:  }
  23, 13,                                    // 11055:  case 0x17: {
  OPC_CheckField, 13, 1, 0,                  // 11057:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 11061:   check Inst[7] == 0x0
  OPC_Decode, 143, 9, 211, 1,                // 11065:   decode to A4_subp_c using decoder 211
                                             // 11065:  }
  24, 47,                                    // 11070:  case 0x18: {
  OPC_SwitchField, 5, 3,                     // 11072:   switch Inst[7:5] {
  0, 9,                                      // 11075:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11077:    check Inst[13] == 0x0
  OPC_Decode, 140, 17, 212, 1,               // 11081:    decode to S2_asr_r_vw using decoder 212
                                             // 11081:   }
  2, 9,                                      // 11086:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11088:    check Inst[13] == 0x0
  OPC_Decode, 207, 17, 212, 1,               // 11092:    decode to S2_lsr_r_vw using decoder 212
                                             // 11092:   }
  4, 9,                                      // 11097:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11099:    check Inst[13] == 0x0
  OPC_Decode, 238, 16, 212, 1,               // 11103:    decode to S2_asl_r_vw using decoder 212
                                             // 11103:   }
  6, 0,                                      // 11108:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11110:    check Inst[13] == 0x0
  OPC_Decode, 180, 17, 212, 1,               // 11114:    decode to S2_lsl_r_vw using decoder 212
                                             // 11114:   }
                                             // 11114:   } // switch Inst[7:5]
                                             // 11114:  }
  26, 47,                                    // 11119:  case 0x1a: {
  OPC_SwitchField, 5, 3,                     // 11121:   switch Inst[7:5] {
  0, 9,                                      // 11124:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11126:    check Inst[13] == 0x0
  OPC_Decode, 139, 17, 212, 1,               // 11130:    decode to S2_asr_r_vh using decoder 212
                                             // 11130:   }
  2, 9,                                      // 11135:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11137:    check Inst[13] == 0x0
  OPC_Decode, 206, 17, 212, 1,               // 11141:    decode to S2_lsr_r_vh using decoder 212
                                             // 11141:   }
  4, 9,                                      // 11146:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11148:    check Inst[13] == 0x0
  OPC_Decode, 237, 16, 212, 1,               // 11152:    decode to S2_asl_r_vh using decoder 212
                                             // 11152:   }
  6, 0,                                      // 11157:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11159:    check Inst[13] == 0x0
  OPC_Decode, 179, 17, 212, 1,               // 11163:    decode to S2_lsl_r_vh using decoder 212
                                             // 11163:   }
                                             // 11163:   } // switch Inst[7:5]
                                             // 11163:  }
  28, 47,                                    // 11168:  case 0x1c: {
  OPC_SwitchField, 5, 3,                     // 11170:   switch Inst[7:5] {
  0, 9,                                      // 11173:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11175:    check Inst[13] == 0x0
  OPC_Decode, 254, 16, 212, 1,               // 11179:    decode to S2_asr_r_p using decoder 212
                                             // 11179:   }
  2, 9,                                      // 11184:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11186:    check Inst[13] == 0x0
  OPC_Decode, 195, 17, 212, 1,               // 11190:    decode to S2_lsr_r_p using decoder 212
                                             // 11190:   }
  4, 9,                                      // 11195:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11197:    check Inst[13] == 0x0
  OPC_Decode, 225, 16, 212, 1,               // 11201:    decode to S2_asl_r_p using decoder 212
                                             // 11201:   }
  6, 0,                                      // 11206:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11208:    check Inst[13] == 0x0
  OPC_Decode, 168, 17, 212, 1,               // 11212:    decode to S2_lsl_r_p using decoder 212
                                             // 11212:   }
                                             // 11212:   } // switch Inst[7:5]
                                             // 11212:  }
  30, 40,                                    // 11217:  case 0x1e: {
  OPC_SwitchField, 6, 2,                     // 11219:   switch Inst[7:6] {
  0, 13,                                     // 11222:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11224:    check Inst[13] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 11228:    check Inst[5] == 0x0
  OPC_Decode, 209, 18, 212, 1,               // 11232:    decode to S2_vcrotate using decoder 212
                                             // 11232:   }
  1, 13,                                     // 11237:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 11239:    check Inst[13] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 11243:    check Inst[5] == 0x0
  OPC_Decode, 208, 18, 212, 1,               // 11247:    decode to S2_vcnegh using decoder 212
                                             // 11247:   }
  3, 0,                                      // 11252:   case 0x3: {
  OPC_Decode, 251, 19, 213, 1,               // 11254:    decode to S4_vrcrotate using decoder 213
                                             // 11254:   }
                                             // 11254:   } // switch Inst[7:6]
                                             // 11254:  }
  32, 9,                                     // 11259:  case 0x20: {
  OPC_CheckField, 13, 1, 0,                  // 11261:   check Inst[13] == 0x0
  OPC_Decode, 208, 16, 214, 1,               // 11265:   decode to S2_addasl_rrri using decoder 214
                                             // 11265:  }
  40, 53,                                    // 11270:  case 0x28: {
  OPC_SwitchField, 5, 3,                     // 11272:   switch Inst[7:5] {
  2, 8,                                      // 11275:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11277:    check Inst[13] == 0x0
  OPC_Decode, 138, 17, 88,                   // 11281:    decode to S2_asr_r_svw_trun using decoder 88
                                             // 11281:   }
  4, 8,                                      // 11285:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11287:    check Inst[13] == 0x0
  OPC_Decode, 243, 15, 88,                   // 11291:    decode to M4_cmpyi_wh using decoder 88
                                             // 11291:   }
  5, 8,                                      // 11295:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 11297:    check Inst[13] == 0x0
  OPC_Decode, 244, 15, 88,                   // 11301:    decode to M4_cmpyi_whc using decoder 88
                                             // 11301:   }
  6, 8,                                      // 11305:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11307:    check Inst[13] == 0x0
  OPC_Decode, 245, 15, 88,                   // 11311:    decode to M4_cmpyr_wh using decoder 88
                                             // 11311:   }
  7, 0,                                      // 11315:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 11317:    check Inst[13] == 0x0
  OPC_Decode, 246, 15, 88,                   // 11321:    decode to M4_cmpyr_whc using decoder 88
                                             // 11321:   }
                                             // 11321:   } // switch Inst[7:5]
                                             // 11321:  }
  48, 25,                                    // 11325:  case 0x30: {
  OPC_SwitchField, 5, 3,                     // 11327:   switch Inst[7:5] {
  0, 9,                                      // 11330:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11332:    check Inst[13] == 0x0
  OPC_Decode, 137, 17, 134, 1,               // 11336:    decode to S2_asr_r_r_sat using decoder 134
                                             // 11336:   }
  4, 0,                                      // 11341:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11343:    check Inst[13] == 0x0
  OPC_Decode, 236, 16, 134, 1,               // 11347:    decode to S2_asl_r_r_sat using decoder 134
                                             // 11347:   }
                                             // 11347:   } // switch Inst[7:5]
                                             // 11347:  }
  50, 47,                                    // 11352:  case 0x32: {
  OPC_SwitchField, 5, 3,                     // 11354:   switch Inst[7:5] {
  0, 9,                                      // 11357:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11359:    check Inst[13] == 0x0
  OPC_Decode, 132, 17, 134, 1,               // 11363:    decode to S2_asr_r_r using decoder 134
                                             // 11363:   }
  2, 9,                                      // 11368:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11370:    check Inst[13] == 0x0
  OPC_Decode, 201, 17, 134, 1,               // 11374:    decode to S2_lsr_r_r using decoder 134
                                             // 11374:   }
  4, 9,                                      // 11379:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11381:    check Inst[13] == 0x0
  OPC_Decode, 231, 16, 134, 1,               // 11385:    decode to S2_asl_r_r using decoder 134
                                             // 11385:   }
  6, 0,                                      // 11390:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11392:    check Inst[13] == 0x0
  OPC_Decode, 174, 17, 134, 1,               // 11396:    decode to S2_lsl_r_r using decoder 134
                                             // 11396:   }
                                             // 11396:   } // switch Inst[7:5]
                                             // 11396:  }
  52, 59,                                    // 11401:  case 0x34: {
  OPC_SwitchField, 6, 2,                     // 11403:   switch Inst[7:6] {
  0, 13,                                     // 11406:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11408:    check Inst[13] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 11412:    check Inst[5] == 0x0
  OPC_Decode, 132, 18, 134, 1,               // 11416:    decode to S2_setbit_r using decoder 134
                                             // 11416:   }
  1, 13,                                     // 11421:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 11423:    check Inst[13] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 11427:    check Inst[5] == 0x0
  OPC_Decode, 152, 17, 134, 1,               // 11431:    decode to S2_clrbit_r using decoder 134
                                             // 11431:   }
  2, 13,                                     // 11436:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11438:    check Inst[13] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 11442:    check Inst[5] == 0x0
  OPC_Decode, 203, 18, 134, 1,               // 11446:    decode to S2_togglebit_r using decoder 134
                                             // 11446:   }
  3, 0,                                      // 11451:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 11453:    check Inst[13] == 0x0
  OPC_Decode, 245, 18, 215, 1,               // 11457:    decode to S4_lsli using decoder 215
                                             // 11457:   }
                                             // 11457:   } // switch Inst[7:6]
                                             // 11457:  }
  54, 49,                                    // 11462:  case 0x36: {
  OPC_SwitchField, 5, 3,                     // 11464:   switch Inst[7:5] {
  0, 9,                                      // 11467:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11469:    check Inst[13] == 0x0
  OPC_Decode, 234, 8, 134, 1,                // 11473:    decode to A4_cround_rr using decoder 134
                                             // 11473:   }
  2, 11,                                     // 11478:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 11480:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 11482:    check Inst[13] == 0x0
  OPC_Decode, 172, 9, 212, 1,                // 11486:    decode to A7_croundd_rr using decoder 212
                                             // 11486:   }
  4, 9,                                      // 11491:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11493:    check Inst[13] == 0x0
  OPC_Decode, 141, 9, 134, 1,                // 11497:    decode to A4_round_rr using decoder 134
                                             // 11497:   }
  6, 0,                                      // 11502:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11504:    check Inst[13] == 0x0
  OPC_Decode, 142, 9, 134, 1,                // 11508:    decode to A4_round_rr_sat using decoder 134
                                             // 11508:   }
                                             // 11508:   } // switch Inst[7:5]
                                             // 11508:  }
  56, 13,                                    // 11513:  case 0x38: {
  OPC_CheckField, 13, 1, 0,                  // 11515:   check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   // 11519:   check Inst[7:2] == 0x0
  OPC_Decode, 205, 18, 169, 1,               // 11523:   decode to S2_tstbit_r using decoder 169
                                             // 11523:  }
  57, 13,                                    // 11528:  case 0x39: {
  OPC_CheckField, 13, 1, 0,                  // 11530:   check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   // 11534:   check Inst[7:2] == 0x0
  OPC_Decode, 247, 18, 169, 1,               // 11538:   decode to S4_ntstbit_r using decoder 169
                                             // 11538:  }
  58, 13,                                    // 11543:  case 0x3a: {
  OPC_CheckField, 13, 1, 0,                  // 11545:   check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   // 11549:   check Inst[7:2] == 0x0
  OPC_Decode, 180, 9, 169, 1,                // 11553:   decode to C2_bitsset using decoder 169
                                             // 11553:  }
  59, 13,                                    // 11558:  case 0x3b: {
  OPC_CheckField, 13, 1, 0,                  // 11560:   check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   // 11564:   check Inst[7:2] == 0x0
  OPC_Decode, 226, 9, 169, 1,                // 11568:   decode to C4_nbitsset using decoder 169
                                             // 11568:  }
  60, 13,                                    // 11573:  case 0x3c: {
  OPC_CheckField, 13, 1, 0,                  // 11575:   check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   // 11579:   check Inst[7:2] == 0x0
  OPC_Decode, 178, 9, 169, 1,                // 11583:   decode to C2_bitsclr using decoder 169
                                             // 11583:  }
  61, 13,                                    // 11588:  case 0x3d: {
  OPC_CheckField, 13, 1, 0,                  // 11590:   check Inst[13] == 0x0
  OPC_CheckField, 2, 6, 0,                   // 11594:   check Inst[7:2] == 0x0
  OPC_Decode, 224, 9, 169, 1,                // 11598:   decode to C4_nbitsclr using decoder 169
                                             // 11598:  }
  62, 69,                                    // 11603:  case 0x3e: {
  OPC_SwitchField, 2, 6,                     // 11605:   switch Inst[7:2] {
  16, 9,                                     // 11608:   case 0x10: {
  OPC_CheckField, 13, 1, 0,                  // 11610:    check Inst[13] == 0x0
  OPC_Decode, 220, 8, 169, 1,                // 11614:    decode to A4_cmpbgt using decoder 169
                                             // 11614:   }
  24, 9,                                     // 11619:   case 0x18: {
  OPC_CheckField, 13, 1, 0,                  // 11621:    check Inst[13] == 0x0
  OPC_Decode, 224, 8, 169, 1,                // 11625:    decode to A4_cmpheq using decoder 169
                                             // 11625:   }
  32, 9,                                     // 11630:   case 0x20: {
  OPC_CheckField, 13, 1, 0,                  // 11632:    check Inst[13] == 0x0
  OPC_Decode, 226, 8, 169, 1,                // 11636:    decode to A4_cmphgt using decoder 169
                                             // 11636:   }
  40, 9,                                     // 11641:   case 0x28: {
  OPC_CheckField, 13, 1, 0,                  // 11643:    check Inst[13] == 0x0
  OPC_Decode, 228, 8, 169, 1,                // 11647:    decode to A4_cmphgtu using decoder 169
                                             // 11647:   }
  48, 9,                                     // 11652:   case 0x30: {
  OPC_CheckField, 13, 1, 0,                  // 11654:    check Inst[13] == 0x0
  OPC_Decode, 218, 8, 169, 1,                // 11658:    decode to A4_cmpbeq using decoder 169
                                             // 11658:   }
  56, 0,                                     // 11663:   case 0x38: {
  OPC_CheckField, 13, 1, 0,                  // 11665:    check Inst[13] == 0x0
  OPC_Decode, 222, 8, 169, 1,                // 11669:    decode to A4_cmpbgtu using decoder 169
                                             // 11669:   }
                                             // 11669:   } // switch Inst[7:2]
                                             // 11669:  }
  63, 47,                                    // 11674:  case 0x3f: {
  OPC_SwitchField, 2, 6,                     // 11676:   switch Inst[7:2] {
  0, 9,                                      // 11679:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11681:    check Inst[13] == 0x0
  OPC_Decode, 167, 10, 169, 1,               // 11685:    decode to F2_sfcmpge using decoder 169
                                             // 11685:   }
  8, 9,                                      // 11690:   case 0x8: {
  OPC_CheckField, 13, 1, 0,                  // 11692:    check Inst[13] == 0x0
  OPC_Decode, 169, 10, 169, 1,               // 11696:    decode to F2_sfcmpuo using decoder 169
                                             // 11696:   }
  24, 9,                                     // 11701:   case 0x18: {
  OPC_CheckField, 13, 1, 0,                  // 11703:    check Inst[13] == 0x0
  OPC_Decode, 166, 10, 169, 1,               // 11707:    decode to F2_sfcmpeq using decoder 169
                                             // 11707:   }
  32, 0,                                     // 11712:   case 0x20: {
  OPC_CheckField, 13, 1, 0,                  // 11714:    check Inst[13] == 0x0
  OPC_Decode, 168, 10, 169, 1,               // 11718:    decode to F2_sfcmpgt using decoder 169
                                             // 11718:   }
                                             // 11718:   } // switch Inst[7:2]
                                             // 11718:  }
  64, 13,                                    // 11723:  case 0x40: {
  OPC_CheckField, 13, 1, 0,                  // 11725:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 11729:   check Inst[7:5] == 0x0
  OPC_Decode, 163, 17, 216, 1,               // 11733:   decode to S2_insert_rp using decoder 216
                                             // 11733:  }
  72, 25,                                    // 11738:  case 0x48: {
  OPC_SwitchField, 5, 3,                     // 11740:   switch Inst[7:5] {
  0, 9,                                      // 11743:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11745:    check Inst[13] == 0x0
  OPC_Decode, 159, 17, 217, 1,               // 11749:    decode to S2_extractu_rp using decoder 217
                                             // 11749:   }
  2, 0,                                      // 11754:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11756:    check Inst[13] == 0x0
  OPC_Decode, 242, 18, 217, 1,               // 11760:    decode to S4_extract_rp using decoder 217
                                             // 11760:   }
                                             // 11760:   } // switch Inst[7:5]
                                             // 11760:  }
  80, 13,                                    // 11765:  case 0x50: {
  OPC_CheckField, 13, 1, 0,                  // 11767:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 11771:   check Inst[7:5] == 0x0
  OPC_Decode, 165, 17, 218, 1,               // 11775:   decode to S2_insertp_rp using decoder 218
                                             // 11775:  }
  84, 13,                                    // 11780:  case 0x54: {
  OPC_CheckField, 13, 1, 0,                  // 11782:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 11786:   check Inst[7:5] == 0x0
  OPC_Decode, 145, 16, 218, 1,               // 11790:   decode to M4_xor_xacc using decoder 218
                                             // 11790:  }
  88, 47,                                    // 11795:  case 0x58: {
  OPC_SwitchField, 5, 3,                     // 11797:   switch Inst[7:5] {
  0, 9,                                      // 11800:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11802:    check Inst[13] == 0x0
  OPC_Decode, 130, 17, 219, 1,               // 11806:    decode to S2_asr_r_p_or using decoder 219
                                             // 11806:   }
  2, 9,                                      // 11811:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11813:    check Inst[13] == 0x0
  OPC_Decode, 199, 17, 219, 1,               // 11817:    decode to S2_lsr_r_p_or using decoder 219
                                             // 11817:   }
  4, 9,                                      // 11822:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11824:    check Inst[13] == 0x0
  OPC_Decode, 229, 16, 219, 1,               // 11828:    decode to S2_asl_r_p_or using decoder 219
                                             // 11828:   }
  6, 0,                                      // 11833:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11835:    check Inst[13] == 0x0
  OPC_Decode, 172, 17, 219, 1,               // 11839:    decode to S2_lsl_r_p_or using decoder 219
                                             // 11839:   }
                                             // 11839:   } // switch Inst[7:5]
                                             // 11839:  }
  89, 90,                                    // 11844:  case 0x59: {
  OPC_SwitchField, 5, 3,                     // 11846:   switch Inst[7:5] {
  1, 17,                                     // 11849:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 11851:    switch Inst[13] {
  0, 5,                                      // 11854:    case 0x0: {
  OPC_Decode, 158, 9, 220, 1,                // 11856:     decode to A4_vrmaxh using decoder 220
                                             // 11856:    }
  1, 0,                                      // 11861:    case 0x1: {
  OPC_Decode, 159, 9, 220, 1,                // 11863:     decode to A4_vrmaxuh using decoder 220
                                             // 11863:    }
                                             // 11863:    } // switch Inst[13]
                                             // 11863:   }
  2, 17,                                     // 11868:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 11870:    switch Inst[13] {
  0, 5,                                      // 11873:    case 0x0: {
  OPC_Decode, 161, 9, 220, 1,                // 11875:     decode to A4_vrmaxw using decoder 220
                                             // 11875:    }
  1, 0,                                      // 11880:    case 0x1: {
  OPC_Decode, 160, 9, 220, 1,                // 11882:     decode to A4_vrmaxuw using decoder 220
                                             // 11882:    }
                                             // 11882:    } // switch Inst[13]
                                             // 11882:   }
  5, 17,                                     // 11887:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 11889:    switch Inst[13] {
  0, 5,                                      // 11892:    case 0x0: {
  OPC_Decode, 162, 9, 220, 1,                // 11894:     decode to A4_vrminh using decoder 220
                                             // 11894:    }
  1, 0,                                      // 11899:    case 0x1: {
  OPC_Decode, 163, 9, 220, 1,                // 11901:     decode to A4_vrminuh using decoder 220
                                             // 11901:    }
                                             // 11901:    } // switch Inst[13]
                                             // 11901:   }
  6, 17,                                     // 11906:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 11908:    switch Inst[13] {
  0, 5,                                      // 11911:    case 0x0: {
  OPC_Decode, 165, 9, 220, 1,                // 11913:     decode to A4_vrminw using decoder 220
                                             // 11913:    }
  1, 0,                                      // 11918:    case 0x1: {
  OPC_Decode, 164, 9, 220, 1,                // 11920:     decode to A4_vrminuw using decoder 220
                                             // 11920:    }
                                             // 11920:    } // switch Inst[13]
                                             // 11920:   }
  7, 0,                                      // 11925:   case 0x7: {
  OPC_CheckField, 13, 1, 1,                  // 11927:    check Inst[13] == 0x1
  OPC_Decode, 210, 18, 219, 1,               // 11931:    decode to S2_vrcnegh using decoder 219
                                             // 11931:   }
                                             // 11931:   } // switch Inst[7:5]
                                             // 11931:  }
  90, 47,                                    // 11936:  case 0x5a: {
  OPC_SwitchField, 5, 3,                     // 11938:   switch Inst[7:5] {
  0, 9,                                      // 11941:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11943:    check Inst[13] == 0x0
  OPC_Decode, 128, 17, 219, 1,               // 11947:    decode to S2_asr_r_p_and using decoder 219
                                             // 11947:   }
  2, 9,                                      // 11952:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 11954:    check Inst[13] == 0x0
  OPC_Decode, 197, 17, 219, 1,               // 11958:    decode to S2_lsr_r_p_and using decoder 219
                                             // 11958:   }
  4, 9,                                      // 11963:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 11965:    check Inst[13] == 0x0
  OPC_Decode, 227, 16, 219, 1,               // 11969:    decode to S2_asl_r_p_and using decoder 219
                                             // 11969:   }
  6, 0,                                      // 11974:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 11976:    check Inst[13] == 0x0
  OPC_Decode, 170, 17, 219, 1,               // 11980:    decode to S2_lsl_r_p_and using decoder 219
                                             // 11980:   }
                                             // 11980:   } // switch Inst[7:5]
                                             // 11980:  }
  91, 47,                                    // 11985:  case 0x5b: {
  OPC_SwitchField, 5, 3,                     // 11987:   switch Inst[7:5] {
  0, 9,                                      // 11990:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 11992:    check Inst[13] == 0x0
  OPC_Decode, 131, 17, 219, 1,               // 11996:    decode to S2_asr_r_p_xor using decoder 219
                                             // 11996:   }
  2, 9,                                      // 12001:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12003:    check Inst[13] == 0x0
  OPC_Decode, 200, 17, 219, 1,               // 12007:    decode to S2_lsr_r_p_xor using decoder 219
                                             // 12007:   }
  4, 9,                                      // 12012:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12014:    check Inst[13] == 0x0
  OPC_Decode, 230, 16, 219, 1,               // 12018:    decode to S2_asl_r_p_xor using decoder 219
                                             // 12018:   }
  6, 0,                                      // 12023:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12025:    check Inst[13] == 0x0
  OPC_Decode, 173, 17, 219, 1,               // 12029:    decode to S2_lsl_r_p_xor using decoder 219
                                             // 12029:   }
                                             // 12029:   } // switch Inst[7:5]
                                             // 12029:  }
  92, 47,                                    // 12034:  case 0x5c: {
  OPC_SwitchField, 5, 3,                     // 12036:   switch Inst[7:5] {
  0, 9,                                      // 12039:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12041:    check Inst[13] == 0x0
  OPC_Decode, 129, 17, 219, 1,               // 12045:    decode to S2_asr_r_p_nac using decoder 219
                                             // 12045:   }
  2, 9,                                      // 12050:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12052:    check Inst[13] == 0x0
  OPC_Decode, 198, 17, 219, 1,               // 12056:    decode to S2_lsr_r_p_nac using decoder 219
                                             // 12056:   }
  4, 9,                                      // 12061:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12063:    check Inst[13] == 0x0
  OPC_Decode, 228, 16, 219, 1,               // 12067:    decode to S2_asl_r_p_nac using decoder 219
                                             // 12067:   }
  6, 0,                                      // 12072:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12074:    check Inst[13] == 0x0
  OPC_Decode, 171, 17, 219, 1,               // 12078:    decode to S2_lsl_r_p_nac using decoder 219
                                             // 12078:   }
                                             // 12078:   } // switch Inst[7:5]
                                             // 12078:  }
  93, 9,                                     // 12083:  case 0x5d: {
  OPC_CheckField, 6, 2, 0,                   // 12085:   check Inst[7:6] == 0x0
  OPC_Decode, 252, 19, 221, 1,               // 12089:   decode to S4_vrcrotate_acc using decoder 221
                                             // 12089:  }
  94, 47,                                    // 12094:  case 0x5e: {
  OPC_SwitchField, 5, 3,                     // 12096:   switch Inst[7:5] {
  0, 9,                                      // 12099:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12101:    check Inst[13] == 0x0
  OPC_Decode, 255, 16, 219, 1,               // 12105:    decode to S2_asr_r_p_acc using decoder 219
                                             // 12105:   }
  2, 9,                                      // 12110:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12112:    check Inst[13] == 0x0
  OPC_Decode, 196, 17, 219, 1,               // 12116:    decode to S2_lsr_r_p_acc using decoder 219
                                             // 12116:   }
  4, 9,                                      // 12121:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12123:    check Inst[13] == 0x0
  OPC_Decode, 226, 16, 219, 1,               // 12127:    decode to S2_asl_r_p_acc using decoder 219
                                             // 12127:   }
  6, 0,                                      // 12132:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12134:    check Inst[13] == 0x0
  OPC_Decode, 169, 17, 219, 1,               // 12138:    decode to S2_lsl_r_p_acc using decoder 219
                                             // 12138:   }
                                             // 12138:   } // switch Inst[7:5]
                                             // 12138:  }
  96, 47,                                    // 12143:  case 0x60: {
  OPC_SwitchField, 5, 3,                     // 12145:   switch Inst[7:5] {
  0, 9,                                      // 12148:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12150:    check Inst[13] == 0x0
  OPC_Decode, 136, 17, 222, 1,               // 12154:    decode to S2_asr_r_r_or using decoder 222
                                             // 12154:   }
  2, 9,                                      // 12159:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12161:    check Inst[13] == 0x0
  OPC_Decode, 205, 17, 222, 1,               // 12165:    decode to S2_lsr_r_r_or using decoder 222
                                             // 12165:   }
  4, 9,                                      // 12170:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12172:    check Inst[13] == 0x0
  OPC_Decode, 235, 16, 222, 1,               // 12176:    decode to S2_asl_r_r_or using decoder 222
                                             // 12176:   }
  6, 0,                                      // 12181:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12183:    check Inst[13] == 0x0
  OPC_Decode, 178, 17, 222, 1,               // 12187:    decode to S2_lsl_r_r_or using decoder 222
                                             // 12187:   }
                                             // 12187:   } // switch Inst[7:5]
                                             // 12187:  }
  98, 47,                                    // 12192:  case 0x62: {
  OPC_SwitchField, 5, 3,                     // 12194:   switch Inst[7:5] {
  0, 9,                                      // 12197:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12199:    check Inst[13] == 0x0
  OPC_Decode, 134, 17, 222, 1,               // 12203:    decode to S2_asr_r_r_and using decoder 222
                                             // 12203:   }
  2, 9,                                      // 12208:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12210:    check Inst[13] == 0x0
  OPC_Decode, 203, 17, 222, 1,               // 12214:    decode to S2_lsr_r_r_and using decoder 222
                                             // 12214:   }
  4, 9,                                      // 12219:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12221:    check Inst[13] == 0x0
  OPC_Decode, 233, 16, 222, 1,               // 12225:    decode to S2_asl_r_r_and using decoder 222
                                             // 12225:   }
  6, 0,                                      // 12230:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12232:    check Inst[13] == 0x0
  OPC_Decode, 176, 17, 222, 1,               // 12236:    decode to S2_lsl_r_r_and using decoder 222
                                             // 12236:   }
                                             // 12236:   } // switch Inst[7:5]
                                             // 12236:  }
  100, 47,                                   // 12241:  case 0x64: {
  OPC_SwitchField, 5, 3,                     // 12243:   switch Inst[7:5] {
  0, 9,                                      // 12246:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12248:    check Inst[13] == 0x0
  OPC_Decode, 135, 17, 222, 1,               // 12252:    decode to S2_asr_r_r_nac using decoder 222
                                             // 12252:   }
  2, 9,                                      // 12257:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12259:    check Inst[13] == 0x0
  OPC_Decode, 204, 17, 222, 1,               // 12263:    decode to S2_lsr_r_r_nac using decoder 222
                                             // 12263:   }
  4, 9,                                      // 12268:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12270:    check Inst[13] == 0x0
  OPC_Decode, 234, 16, 222, 1,               // 12274:    decode to S2_asl_r_r_nac using decoder 222
                                             // 12274:   }
  6, 0,                                      // 12279:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12281:    check Inst[13] == 0x0
  OPC_Decode, 177, 17, 222, 1,               // 12285:    decode to S2_lsl_r_r_nac using decoder 222
                                             // 12285:   }
                                             // 12285:   } // switch Inst[7:5]
                                             // 12285:  }
  102, 0,                                    // 12290:  case 0x66: {
  OPC_SwitchField, 5, 3,                     // 12292:   switch Inst[7:5] {
  0, 9,                                      // 12295:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12297:    check Inst[13] == 0x0
  OPC_Decode, 133, 17, 222, 1,               // 12301:    decode to S2_asr_r_r_acc using decoder 222
                                             // 12301:   }
  2, 9,                                      // 12306:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12308:    check Inst[13] == 0x0
  OPC_Decode, 202, 17, 222, 1,               // 12312:    decode to S2_lsr_r_r_acc using decoder 222
                                             // 12312:   }
  4, 9,                                      // 12317:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12319:    check Inst[13] == 0x0
  OPC_Decode, 232, 16, 222, 1,               // 12323:    decode to S2_asl_r_r_acc using decoder 222
                                             // 12323:   }
  6, 0,                                      // 12328:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12330:    check Inst[13] == 0x0
  OPC_Decode, 175, 17, 222, 1,               // 12334:    decode to S2_lsl_r_r_acc using decoder 222
                                             // 12334:   }
                                             // 12334:   } // switch Inst[7:5]
                                             // 12334:  }
                                             // 12334:  } // switch Inst[27:21]
                                             // 12334: }
  13, 194, 14,                               // 12339: case 0xd: {
  OPC_SwitchField, 24, 4,                    // 12342:  switch Inst[27:24] {
  0, 17,                                     // 12345:  case 0x0: {
  OPC_CheckField, 21, 3, 0,                  // 12347:   check Inst[23:21] == 0x0
  OPC_CheckField, 13, 1, 0,                  // 12351:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 12355:   check Inst[7:5] == 0x0
  OPC_Decode, 210, 17, 208, 1,               // 12359:   decode to S2_parityp using decoder 208
                                             // 12359:  }
  1, 17,                                     // 12364:  case 0x1: {
  OPC_CheckField, 21, 3, 0,                  // 12366:   check Inst[23:21] == 0x0
  OPC_CheckField, 13, 1, 0,                  // 12370:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 12374:   check Inst[7] == 0x0
  OPC_Decode, 209, 9, 223, 1,                // 12378:   decode to C2_vmux using decoder 223
                                             // 12378:  }
  2, 131, 2,                                 // 12383:  case 0x2: {
  OPC_SwitchField, 2, 6,                     // 12386:   switch Inst[7:2] {
  0, 44,                                     // 12389:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 12391:    switch Inst[23:21] {
  0, 17,                                     // 12394:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 12396:     switch Inst[13] {
  0, 5,                                      // 12399:     case 0x0: {
  OPC_Decode, 175, 8, 224, 1,                // 12401:      decode to A2_vcmpweq using decoder 224
                                             // 12401:     }
  1, 0,                                      // 12406:     case 0x1: {
  OPC_Decode, 147, 9, 224, 1,                // 12408:      decode to A4_vcmpbeq_any using decoder 224
                                             // 12408:     }
                                             // 12408:     } // switch Inst[13]
                                             // 12408:    }
  4, 9,                                      // 12413:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12415:     check Inst[13] == 0x0
  OPC_Decode, 191, 9, 224, 1,                // 12419:     decode to C2_cmpeqp using decoder 224
                                             // 12419:    }
  7, 0,                                      // 12424:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 12426:     check Inst[13] == 0x0
  OPC_Decode, 151, 10, 224, 1,               // 12430:     decode to F2_dfcmpeq using decoder 224
                                             // 12430:    }
                                             // 12430:    } // switch Inst[23:21]
                                             // 12430:   }
  8, 35,                                     // 12435:   case 0x8: {
  OPC_SwitchField, 13, 1,                    // 12437:    switch Inst[13] {
  0, 17,                                     // 12440:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 12442:     switch Inst[23:21] {
  0, 5,                                      // 12445:     case 0x0: {
  OPC_Decode, 176, 8, 224, 1,                // 12447:      decode to A2_vcmpwgt using decoder 224
                                             // 12447:     }
  7, 0,                                      // 12452:     case 0x7: {
  OPC_Decode, 153, 10, 224, 1,               // 12454:      decode to F2_dfcmpgt using decoder 224
                                             // 12454:     }
                                             // 12454:     } // switch Inst[23:21]
                                             // 12454:    }
  1, 0,                                      // 12459:    case 0x1: {
  OPC_CheckPredicate, 3,                     // 12461:     check predicate 3
  OPC_CheckField, 21, 3, 0,                  // 12463:     check Inst[23:21] == 0x0
  OPC_Decode, 168, 9, 224, 1,                // 12467:     decode to A6_vcmpbeq_notany using decoder 224
                                             // 12467:    }
                                             // 12467:    } // switch Inst[13]
                                             // 12467:   }
  16, 44,                                    // 12472:   case 0x10: {
  OPC_SwitchField, 21, 3,                    // 12474:    switch Inst[23:21] {
  0, 17,                                     // 12477:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 12479:     switch Inst[13] {
  0, 5,                                      // 12482:     case 0x0: {
  OPC_Decode, 177, 8, 224, 1,                // 12484:      decode to A2_vcmpwgtu using decoder 224
                                             // 12484:     }
  1, 0,                                      // 12489:     case 0x1: {
  OPC_Decode, 149, 9, 224, 1,                // 12491:      decode to A4_vcmpbgt using decoder 224
                                             // 12491:     }
                                             // 12491:     } // switch Inst[13]
                                             // 12491:    }
  4, 9,                                      // 12496:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12498:     check Inst[13] == 0x0
  OPC_Decode, 194, 9, 224, 1,                // 12502:     decode to C2_cmpgtp using decoder 224
                                             // 12502:    }
  7, 0,                                      // 12507:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 12509:     check Inst[13] == 0x0
  OPC_Decode, 152, 10, 224, 1,               // 12513:     decode to F2_dfcmpge using decoder 224
                                             // 12513:    }
                                             // 12513:    } // switch Inst[23:21]
                                             // 12513:   }
  24, 33,                                    // 12518:   case 0x18: {
  OPC_SwitchField, 13, 1,                    // 12520:    switch Inst[13] {
  0, 17,                                     // 12523:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 12525:     switch Inst[23:21] {
  0, 5,                                      // 12528:     case 0x0: {
  OPC_Decode, 172, 8, 224, 1,                // 12530:      decode to A2_vcmpheq using decoder 224
                                             // 12530:     }
  7, 0,                                      // 12535:     case 0x7: {
  OPC_Decode, 154, 10, 224, 1,               // 12537:      decode to F2_dfcmpuo using decoder 224
                                             // 12537:     }
                                             // 12537:     } // switch Inst[23:21]
                                             // 12537:    }
  1, 0,                                      // 12542:    case 0x1: {
  OPC_CheckField, 21, 3, 0,                  // 12544:     check Inst[23:21] == 0x0
  OPC_Decode, 146, 9, 225, 1,                // 12548:     decode to A4_tlbmatch using decoder 225
                                             // 12548:    }
                                             // 12548:    } // switch Inst[13]
                                             // 12548:   }
  32, 33,                                    // 12553:   case 0x20: {
  OPC_SwitchField, 13, 1,                    // 12555:    switch Inst[13] {
  0, 17,                                     // 12558:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 12560:     switch Inst[23:21] {
  0, 5,                                      // 12563:     case 0x0: {
  OPC_Decode, 173, 8, 224, 1,                // 12565:      decode to A2_vcmphgt using decoder 224
                                             // 12565:     }
  4, 0,                                      // 12570:     case 0x4: {
  OPC_Decode, 197, 9, 224, 1,                // 12572:      decode to C2_cmpgtup using decoder 224
                                             // 12572:     }
                                             // 12572:     } // switch Inst[23:21]
                                             // 12572:    }
  1, 0,                                      // 12577:    case 0x1: {
  OPC_CheckField, 21, 3, 0,                  // 12579:     check Inst[23:21] == 0x0
  OPC_Decode, 217, 8, 224, 1,                // 12583:     decode to A4_boundscheck_lo using decoder 224
                                             // 12583:    }
                                             // 12583:    } // switch Inst[13]
                                             // 12583:   }
  40, 25,                                    // 12588:   case 0x28: {
  OPC_SwitchField, 13, 1,                    // 12590:    switch Inst[13] {
  0, 9,                                      // 12593:    case 0x0: {
  OPC_CheckField, 21, 3, 0,                  // 12595:     check Inst[23:21] == 0x0
  OPC_Decode, 174, 8, 224, 1,                // 12599:     decode to A2_vcmphgtu using decoder 224
                                             // 12599:    }
  1, 0,                                      // 12604:    case 0x1: {
  OPC_CheckField, 21, 3, 0,                  // 12606:     check Inst[23:21] == 0x0
  OPC_Decode, 216, 8, 224, 1,                // 12610:     decode to A4_boundscheck_hi using decoder 224
                                             // 12610:    }
                                             // 12610:    } // switch Inst[13]
                                             // 12610:   }
  48, 13,                                    // 12615:   case 0x30: {
  OPC_CheckField, 21, 3, 0,                  // 12617:    check Inst[23:21] == 0x0
  OPC_CheckField, 13, 1, 0,                  // 12621:    check Inst[13] == 0x0
  OPC_Decode, 170, 8, 224, 1,                // 12625:    decode to A2_vcmpbeq using decoder 224
                                             // 12625:   }
  56, 0,                                     // 12630:   case 0x38: {
  OPC_CheckField, 21, 3, 0,                  // 12632:    check Inst[23:21] == 0x0
  OPC_CheckField, 13, 1, 0,                  // 12636:    check Inst[13] == 0x0
  OPC_Decode, 171, 8, 224, 1,                // 12640:    decode to A2_vcmpbgtu using decoder 224
                                             // 12640:   }
                                             // 12640:   } // switch Inst[7:2]
                                             // 12640:  }
  3, 180, 5,                                 // 12645:  case 0x3: {
  OPC_SwitchField, 5, 3,                     // 12648:   switch Inst[7:5] {
  0, 91,                                     // 12651:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 12653:    switch Inst[23:21] {
  0, 9,                                      // 12656:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12658:     check Inst[13] == 0x0
  OPC_Decode, 153, 8, 206, 1,                // 12662:     decode to A2_vaddub using decoder 206
                                             // 12662:    }
  1, 9,                                      // 12667:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 12669:     check Inst[13] == 0x0
  OPC_Decode, 203, 8, 207, 1,                // 12673:     decode to A2_vsubub using decoder 207
                                             // 12673:    }
  2, 9,                                      // 12678:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12680:     check Inst[13] == 0x0
  OPC_Decode, 161, 8, 206, 1,                // 12684:     decode to A2_vavgub using decoder 206
                                             // 12684:    }
  3, 9,                                      // 12689:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 12691:     check Inst[13] == 0x0
  OPC_Decode, 167, 8, 206, 1,                // 12695:     decode to A2_vavgw using decoder 206
                                             // 12695:    }
  4, 9,                                      // 12700:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12702:     check Inst[13] == 0x0
  OPC_Decode, 191, 8, 207, 1,                // 12706:     decode to A2_vnavgh using decoder 207
                                             // 12706:    }
  5, 9,                                      // 12711:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 12713:     check Inst[13] == 0x0
  OPC_Decode, 187, 8, 207, 1,                // 12717:     decode to A2_vminub using decoder 207
                                             // 12717:    }
  6, 9,                                      // 12722:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12724:     check Inst[13] == 0x0
  OPC_Decode, 181, 8, 207, 1,                // 12728:     decode to A2_vmaxub using decoder 207
                                             // 12728:    }
  7, 0,                                      // 12733:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 12735:     check Inst[13] == 0x0
  OPC_Decode, 186, 7, 206, 1,                // 12739:     decode to A2_andp using decoder 206
                                             // 12739:    }
                                             // 12739:    } // switch Inst[23:21]
                                             // 12739:   }
  1, 91,                                     // 12744:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 12746:    switch Inst[23:21] {
  0, 9,                                      // 12749:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12751:     check Inst[13] == 0x0
  OPC_Decode, 154, 8, 206, 1,                // 12755:     decode to A2_vaddubs using decoder 206
                                             // 12755:    }
  1, 9,                                      // 12760:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 12762:     check Inst[13] == 0x0
  OPC_Decode, 204, 8, 207, 1,                // 12766:     decode to A2_vsububs using decoder 207
                                             // 12766:    }
  2, 9,                                      // 12771:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12773:     check Inst[13] == 0x0
  OPC_Decode, 162, 8, 206, 1,                // 12777:     decode to A2_vavgubr using decoder 206
                                             // 12777:    }
  3, 9,                                      // 12782:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 12784:     check Inst[13] == 0x0
  OPC_Decode, 169, 8, 206, 1,                // 12788:     decode to A2_vavgwr using decoder 206
                                             // 12788:    }
  4, 9,                                      // 12793:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12795:     check Inst[13] == 0x0
  OPC_Decode, 193, 8, 207, 1,                // 12799:     decode to A2_vnavghr using decoder 207
                                             // 12799:    }
  5, 9,                                      // 12804:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 12806:     check Inst[13] == 0x0
  OPC_Decode, 186, 8, 207, 1,                // 12810:     decode to A2_vminh using decoder 207
                                             // 12810:    }
  6, 9,                                      // 12815:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12817:     check Inst[13] == 0x0
  OPC_Decode, 180, 8, 207, 1,                // 12821:     decode to A2_vmaxh using decoder 207
                                             // 12821:    }
  7, 0,                                      // 12826:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 12828:     check Inst[13] == 0x0
  OPC_Decode, 213, 8, 207, 1,                // 12832:     decode to A4_andnp using decoder 207
                                             // 12832:    }
                                             // 12832:    } // switch Inst[23:21]
                                             // 12832:   }
  2, 91,                                     // 12837:   case 0x2: {
  OPC_SwitchField, 21, 3,                    // 12839:    switch Inst[23:21] {
  0, 9,                                      // 12842:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12844:     check Inst[13] == 0x0
  OPC_Decode, 151, 8, 206, 1,                // 12848:     decode to A2_vaddh using decoder 206
                                             // 12848:    }
  1, 9,                                      // 12853:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 12855:     check Inst[13] == 0x0
  OPC_Decode, 201, 8, 207, 1,                // 12859:     decode to A2_vsubh using decoder 207
                                             // 12859:    }
  2, 9,                                      // 12864:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12866:     check Inst[13] == 0x0
  OPC_Decode, 158, 8, 206, 1,                // 12870:     decode to A2_vavgh using decoder 206
                                             // 12870:    }
  3, 9,                                      // 12875:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 12877:     check Inst[13] == 0x0
  OPC_Decode, 168, 8, 206, 1,                // 12881:     decode to A2_vavgwcr using decoder 206
                                             // 12881:    }
  4, 9,                                      // 12886:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12888:     check Inst[13] == 0x0
  OPC_Decode, 192, 8, 207, 1,                // 12892:     decode to A2_vnavghcr using decoder 207
                                             // 12892:    }
  5, 9,                                      // 12897:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 12899:     check Inst[13] == 0x0
  OPC_Decode, 188, 8, 207, 1,                // 12903:     decode to A2_vminuh using decoder 207
                                             // 12903:    }
  6, 9,                                      // 12908:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 12910:     check Inst[13] == 0x0
  OPC_Decode, 182, 8, 207, 1,                // 12914:     decode to A2_vmaxuh using decoder 207
                                             // 12914:    }
  7, 0,                                      // 12919:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 12921:     check Inst[13] == 0x0
  OPC_Decode, 209, 7, 206, 1,                // 12925:     decode to A2_orp using decoder 206
                                             // 12925:    }
                                             // 12925:    } // switch Inst[23:21]
                                             // 12925:   }
  3, 91,                                     // 12930:   case 0x3: {
  OPC_SwitchField, 21, 3,                    // 12932:    switch Inst[23:21] {
  0, 9,                                      // 12935:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 12937:     check Inst[13] == 0x0
  OPC_Decode, 152, 8, 206, 1,                // 12941:     decode to A2_vaddhs using decoder 206
                                             // 12941:    }
  1, 9,                                      // 12946:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 12948:     check Inst[13] == 0x0
  OPC_Decode, 202, 8, 207, 1,                // 12952:     decode to A2_vsubhs using decoder 207
                                             // 12952:    }
  2, 9,                                      // 12957:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 12959:     check Inst[13] == 0x0
  OPC_Decode, 160, 8, 206, 1,                // 12963:     decode to A2_vavghr using decoder 206
                                             // 12963:    }
  3, 9,                                      // 12968:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 12970:     check Inst[13] == 0x0
  OPC_Decode, 165, 8, 206, 1,                // 12974:     decode to A2_vavguw using decoder 206
                                             // 12974:    }
  4, 9,                                      // 12979:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 12981:     check Inst[13] == 0x0
  OPC_Decode, 194, 8, 207, 1,                // 12985:     decode to A2_vnavgw using decoder 207
                                             // 12985:    }
  5, 9,                                      // 12990:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 12992:     check Inst[13] == 0x0
  OPC_Decode, 190, 8, 207, 1,                // 12996:     decode to A2_vminw using decoder 207
                                             // 12996:    }
  6, 9,                                      // 13001:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13003:     check Inst[13] == 0x0
  OPC_Decode, 184, 8, 207, 1,                // 13007:     decode to A2_vmaxw using decoder 207
                                             // 13007:    }
  7, 0,                                      // 13012:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 13014:     check Inst[13] == 0x0
  OPC_Decode, 238, 8, 207, 1,                // 13018:     decode to A4_ornp using decoder 207
                                             // 13018:    }
                                             // 13018:    } // switch Inst[23:21]
                                             // 13018:   }
  4, 91,                                     // 13023:   case 0x4: {
  OPC_SwitchField, 21, 3,                    // 13025:    switch Inst[23:21] {
  0, 9,                                      // 13028:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13030:     check Inst[13] == 0x0
  OPC_Decode, 155, 8, 206, 1,                // 13034:     decode to A2_vadduhs using decoder 206
                                             // 13034:    }
  1, 9,                                      // 13039:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13041:     check Inst[13] == 0x0
  OPC_Decode, 205, 8, 207, 1,                // 13045:     decode to A2_vsubuhs using decoder 207
                                             // 13045:    }
  2, 9,                                      // 13050:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13052:     check Inst[13] == 0x0
  OPC_Decode, 159, 8, 206, 1,                // 13056:     decode to A2_vavghcr using decoder 206
                                             // 13056:    }
  3, 9,                                      // 13061:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13063:     check Inst[13] == 0x0
  OPC_Decode, 166, 8, 206, 1,                // 13067:     decode to A2_vavguwr using decoder 206
                                             // 13067:    }
  4, 9,                                      // 13072:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 13074:     check Inst[13] == 0x0
  OPC_Decode, 196, 8, 207, 1,                // 13078:     decode to A2_vnavgwr using decoder 207
                                             // 13078:    }
  5, 9,                                      // 13083:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 13085:     check Inst[13] == 0x0
  OPC_Decode, 189, 8, 207, 1,                // 13089:     decode to A2_vminuw using decoder 207
                                             // 13089:    }
  6, 9,                                      // 13094:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13096:     check Inst[13] == 0x0
  OPC_Decode, 196, 7, 206, 1,                // 13100:     decode to A2_maxp using decoder 206
                                             // 13100:    }
  7, 0,                                      // 13105:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 13107:     check Inst[13] == 0x0
  OPC_Decode, 209, 8, 206, 1,                // 13111:     decode to A2_xorp using decoder 206
                                             // 13111:    }
                                             // 13111:    } // switch Inst[23:21]
                                             // 13111:   }
  5, 69,                                     // 13116:   case 0x5: {
  OPC_SwitchField, 21, 3,                    // 13118:    switch Inst[23:21] {
  0, 9,                                      // 13121:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13123:     check Inst[13] == 0x0
  OPC_Decode, 156, 8, 206, 1,                // 13127:     decode to A2_vaddw using decoder 206
                                             // 13127:    }
  1, 9,                                      // 13132:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13134:     check Inst[13] == 0x0
  OPC_Decode, 206, 8, 207, 1,                // 13138:     decode to A2_vsubw using decoder 207
                                             // 13138:    }
  2, 9,                                      // 13143:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13145:     check Inst[13] == 0x0
  OPC_Decode, 163, 8, 206, 1,                // 13149:     decode to A2_vavguh using decoder 206
                                             // 13149:    }
  3, 9,                                      // 13154:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13156:     check Inst[13] == 0x0
  OPC_Decode, 180, 7, 206, 1,                // 13160:     decode to A2_addpsat using decoder 206
                                             // 13160:    }
  5, 9,                                      // 13165:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 13167:     check Inst[13] == 0x0
  OPC_Decode, 183, 8, 207, 1,                // 13171:     decode to A2_vmaxuw using decoder 207
                                             // 13171:    }
  6, 0,                                      // 13176:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13178:     check Inst[13] == 0x0
  OPC_Decode, 198, 7, 206, 1,                // 13182:     decode to A2_maxup using decoder 206
                                             // 13182:    }
                                             // 13182:    } // switch Inst[23:21]
                                             // 13182:   }
  6, 80,                                     // 13187:   case 0x6: {
  OPC_SwitchField, 21, 3,                    // 13189:    switch Inst[23:21] {
  0, 9,                                      // 13192:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13194:     check Inst[13] == 0x0
  OPC_Decode, 157, 8, 206, 1,                // 13198:     decode to A2_vaddws using decoder 206
                                             // 13198:    }
  1, 9,                                      // 13203:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13205:     check Inst[13] == 0x0
  OPC_Decode, 207, 8, 207, 1,                // 13209:     decode to A2_vsubws using decoder 207
                                             // 13209:    }
  2, 9,                                      // 13214:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13216:     check Inst[13] == 0x0
  OPC_Decode, 164, 8, 206, 1,                // 13220:     decode to A2_vavguhr using decoder 206
                                             // 13220:    }
  3, 9,                                      // 13225:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13227:     check Inst[13] == 0x0
  OPC_Decode, 183, 7, 206, 1,                // 13231:     decode to A2_addspl using decoder 206
                                             // 13231:    }
  4, 9,                                      // 13236:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 13238:     check Inst[13] == 0x0
  OPC_Decode, 195, 8, 207, 1,                // 13242:     decode to A2_vnavgwcr using decoder 207
                                             // 13242:    }
  5, 9,                                      // 13247:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 13249:     check Inst[13] == 0x0
  OPC_Decode, 200, 7, 207, 1,                // 13253:     decode to A2_minp using decoder 207
                                             // 13253:    }
  6, 0,                                      // 13258:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13260:     check Inst[13] == 0x0
  OPC_Decode, 179, 8, 207, 1,                // 13264:     decode to A2_vmaxb using decoder 207
                                             // 13264:    }
                                             // 13264:    } // switch Inst[23:21]
                                             // 13264:   }
  7, 0,                                      // 13269:   case 0x7: {
  OPC_SwitchField, 21, 3,                    // 13271:    switch Inst[23:21] {
  0, 9,                                      // 13274:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13276:     check Inst[13] == 0x0
  OPC_Decode, 179, 7, 206, 1,                // 13280:     decode to A2_addp using decoder 206
                                             // 13280:    }
  1, 9,                                      // 13285:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13287:     check Inst[13] == 0x0
  OPC_Decode, 253, 7, 207, 1,                // 13291:     decode to A2_subp using decoder 207
                                             // 13291:    }
  3, 9,                                      // 13296:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13298:     check Inst[13] == 0x0
  OPC_Decode, 182, 7, 206, 1,                // 13302:     decode to A2_addsph using decoder 206
                                             // 13302:    }
  5, 9,                                      // 13307:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 13309:     check Inst[13] == 0x0
  OPC_Decode, 202, 7, 207, 1,                // 13313:     decode to A2_minup using decoder 207
                                             // 13313:    }
  6, 9,                                      // 13318:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13320:     check Inst[13] == 0x0
  OPC_Decode, 185, 8, 207, 1,                // 13324:     decode to A2_vminb using decoder 207
                                             // 13324:    }
  7, 0,                                      // 13329:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 13331:     check Inst[13] == 0x0
  OPC_Decode, 236, 8, 134, 1,                // 13335:     decode to A4_modwrapu using decoder 134
                                             // 13335:    }
                                             // 13335:    } // switch Inst[23:21]
                                             // 13335:   }
                                             // 13335:   } // switch Inst[7:5]
                                             // 13335:  }
  4, 33,                                     // 13340:  case 0x4: {
  OPC_SwitchField, 21, 3,                    // 13342:   switch Inst[23:21] {
  0, 13,                                     // 13345:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13347:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 13351:    check Inst[7:5] == 0x0
  OPC_Decode, 220, 26, 226, 1,               // 13355:    decode to dep_S2_packhl using decoder 226
                                             // 13355:   }
  1, 0,                                      // 13360:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13362:    check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 13366:    check Inst[7:5] == 0x0
  OPC_Decode, 214, 8, 226, 1,                // 13370:    decode to A4_bitsplit using decoder 226
                                             // 13370:   }
                                             // 13370:   } // switch Inst[23:21]
                                             // 13370:  }
  5, 128, 3,                                 // 13375:  case 0x5: {
  OPC_SwitchField, 5, 3,                     // 13378:   switch Inst[7:5] {
  0, 91,                                     // 13381:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 13383:    switch Inst[23:21] {
  0, 9,                                      // 13386:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13388:     check Inst[13] == 0x0
  OPC_Decode, 175, 7, 227, 1,                // 13392:     decode to A2_addh_l16_ll using decoder 227
                                             // 13392:    }
  1, 9,                                      // 13397:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13399:     check Inst[13] == 0x0
  OPC_Decode, 250, 7, 227, 1,                // 13403:     decode to A2_subh_l16_ll using decoder 227
                                             // 13403:    }
  2, 9,                                      // 13408:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13410:     check Inst[13] == 0x0
  OPC_Decode, 169, 7, 227, 1,                // 13414:     decode to A2_addh_h16_ll using decoder 227
                                             // 13414:    }
  3, 9,                                      // 13419:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13421:     check Inst[13] == 0x0
  OPC_Decode, 244, 7, 227, 1,                // 13425:     decode to A2_subh_h16_ll using decoder 227
                                             // 13425:    }
  4, 9,                                      // 13430:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 13432:     check Inst[13] == 0x0
  OPC_Decode, 218, 26, 134, 1,               // 13436:     decode to dep_A2_addsat using decoder 134
                                             // 13436:    }
  5, 9,                                      // 13441:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 13443:     check Inst[13] == 0x0
  OPC_Decode, 199, 7, 227, 1,                // 13447:     decode to A2_min using decoder 227
                                             // 13447:    }
  6, 9,                                      // 13452:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13454:     check Inst[13] == 0x0
  OPC_Decode, 195, 7, 134, 1,                // 13458:     decode to A2_max using decoder 134
                                             // 13458:    }
  7, 0,                                      // 13463:    case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 13465:     check Inst[13] == 0x0
  OPC_Decode, 253, 18, 134, 1,               // 13469:     decode to S4_parity using decoder 134
                                             // 13469:    }
                                             // 13469:    } // switch Inst[23:21]
                                             // 13469:   }
  1, 25,                                     // 13474:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 13476:    switch Inst[23:21] {
  2, 9,                                      // 13479:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13481:     check Inst[13] == 0x0
  OPC_Decode, 168, 7, 227, 1,                // 13485:     decode to A2_addh_h16_lh using decoder 227
                                             // 13485:    }
  3, 0,                                      // 13490:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13492:     check Inst[13] == 0x0
  OPC_Decode, 243, 7, 227, 1,                // 13496:     decode to A2_subh_h16_lh using decoder 227
                                             // 13496:    }
                                             // 13496:    } // switch Inst[23:21]
                                             // 13496:   }
  2, 47,                                     // 13501:   case 0x2: {
  OPC_SwitchField, 21, 3,                    // 13503:    switch Inst[23:21] {
  0, 9,                                      // 13506:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13508:     check Inst[13] == 0x0
  OPC_Decode, 174, 7, 227, 1,                // 13512:     decode to A2_addh_l16_hl using decoder 227
                                             // 13512:    }
  1, 9,                                      // 13517:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13519:     check Inst[13] == 0x0
  OPC_Decode, 249, 7, 227, 1,                // 13523:     decode to A2_subh_l16_hl using decoder 227
                                             // 13523:    }
  2, 9,                                      // 13528:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13530:     check Inst[13] == 0x0
  OPC_Decode, 167, 7, 227, 1,                // 13534:     decode to A2_addh_h16_hl using decoder 227
                                             // 13534:    }
  3, 0,                                      // 13539:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13541:     check Inst[13] == 0x0
  OPC_Decode, 242, 7, 227, 1,                // 13545:     decode to A2_subh_h16_hl using decoder 227
                                             // 13545:    }
                                             // 13545:    } // switch Inst[23:21]
                                             // 13545:   }
  3, 25,                                     // 13550:   case 0x3: {
  OPC_SwitchField, 21, 3,                    // 13552:    switch Inst[23:21] {
  2, 9,                                      // 13555:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13557:     check Inst[13] == 0x0
  OPC_Decode, 166, 7, 227, 1,                // 13561:     decode to A2_addh_h16_hh using decoder 227
                                             // 13561:    }
  3, 0,                                      // 13566:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13568:     check Inst[13] == 0x0
  OPC_Decode, 241, 7, 227, 1,                // 13572:     decode to A2_subh_h16_hh using decoder 227
                                             // 13572:    }
                                             // 13572:    } // switch Inst[23:21]
                                             // 13572:   }
  4, 80,                                     // 13577:   case 0x4: {
  OPC_SwitchField, 21, 3,                    // 13579:    switch Inst[23:21] {
  0, 9,                                      // 13582:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13584:     check Inst[13] == 0x0
  OPC_Decode, 177, 7, 227, 1,                // 13588:     decode to A2_addh_l16_sat_ll using decoder 227
                                             // 13588:    }
  1, 9,                                      // 13593:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13595:     check Inst[13] == 0x0
  OPC_Decode, 252, 7, 227, 1,                // 13599:     decode to A2_subh_l16_sat_ll using decoder 227
                                             // 13599:    }
  2, 9,                                      // 13604:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13606:     check Inst[13] == 0x0
  OPC_Decode, 173, 7, 227, 1,                // 13610:     decode to A2_addh_h16_sat_ll using decoder 227
                                             // 13610:    }
  3, 9,                                      // 13615:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13617:     check Inst[13] == 0x0
  OPC_Decode, 248, 7, 227, 1,                // 13621:     decode to A2_subh_h16_sat_ll using decoder 227
                                             // 13621:    }
  4, 9,                                      // 13626:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 13628:     check Inst[13] == 0x0
  OPC_Decode, 219, 26, 227, 1,               // 13632:     decode to dep_A2_subsat using decoder 227
                                             // 13632:    }
  5, 9,                                      // 13637:    case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 13639:     check Inst[13] == 0x0
  OPC_Decode, 201, 7, 227, 1,                // 13643:     decode to A2_minu using decoder 227
                                             // 13643:    }
  6, 0,                                      // 13648:    case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 13650:     check Inst[13] == 0x0
  OPC_Decode, 197, 7, 134, 1,                // 13654:     decode to A2_maxu using decoder 134
                                             // 13654:    }
                                             // 13654:    } // switch Inst[23:21]
                                             // 13654:   }
  5, 25,                                     // 13659:   case 0x5: {
  OPC_SwitchField, 21, 3,                    // 13661:    switch Inst[23:21] {
  2, 9,                                      // 13664:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13666:     check Inst[13] == 0x0
  OPC_Decode, 172, 7, 227, 1,                // 13670:     decode to A2_addh_h16_sat_lh using decoder 227
                                             // 13670:    }
  3, 0,                                      // 13675:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13677:     check Inst[13] == 0x0
  OPC_Decode, 247, 7, 227, 1,                // 13681:     decode to A2_subh_h16_sat_lh using decoder 227
                                             // 13681:    }
                                             // 13681:    } // switch Inst[23:21]
                                             // 13681:   }
  6, 47,                                     // 13686:   case 0x6: {
  OPC_SwitchField, 21, 3,                    // 13688:    switch Inst[23:21] {
  0, 9,                                      // 13691:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13693:     check Inst[13] == 0x0
  OPC_Decode, 176, 7, 227, 1,                // 13697:     decode to A2_addh_l16_sat_hl using decoder 227
                                             // 13697:    }
  1, 9,                                      // 13702:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 13704:     check Inst[13] == 0x0
  OPC_Decode, 251, 7, 227, 1,                // 13708:     decode to A2_subh_l16_sat_hl using decoder 227
                                             // 13708:    }
  2, 9,                                      // 13713:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13715:     check Inst[13] == 0x0
  OPC_Decode, 171, 7, 227, 1,                // 13719:     decode to A2_addh_h16_sat_hl using decoder 227
                                             // 13719:    }
  3, 0,                                      // 13724:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13726:     check Inst[13] == 0x0
  OPC_Decode, 246, 7, 227, 1,                // 13730:     decode to A2_subh_h16_sat_hl using decoder 227
                                             // 13730:    }
                                             // 13730:    } // switch Inst[23:21]
                                             // 13730:   }
  7, 0,                                      // 13735:   case 0x7: {
  OPC_SwitchField, 21, 3,                    // 13737:    switch Inst[23:21] {
  2, 9,                                      // 13740:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13742:     check Inst[13] == 0x0
  OPC_Decode, 170, 7, 227, 1,                // 13746:     decode to A2_addh_h16_sat_hh using decoder 227
                                             // 13746:    }
  3, 0,                                      // 13751:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 13753:     check Inst[13] == 0x0
  OPC_Decode, 245, 7, 227, 1,                // 13757:     decode to A2_subh_h16_sat_hh using decoder 227
                                             // 13757:    }
                                             // 13757:    } // switch Inst[23:21]
                                             // 13757:   }
                                             // 13757:   } // switch Inst[7:5]
                                             // 13757:  }
  6, 25,                                     // 13762:  case 0x6: {
  OPC_SwitchField, 22, 2,                    // 13764:   switch Inst[23:22] {
  0, 9,                                      // 13767:   case 0x0: {
  OPC_CheckField, 16, 5, 0,                  // 13769:    check Inst[20:16] == 0x0
  OPC_Decode, 179, 10, 228, 1,               // 13773:    decode to F2_sfimm_p using decoder 228
                                             // 13773:   }
  1, 0,                                      // 13778:   case 0x1: {
  OPC_CheckField, 16, 5, 0,                  // 13780:    check Inst[20:16] == 0x0
  OPC_Decode, 178, 10, 228, 1,               // 13784:    decode to F2_sfimm_n using decoder 228
                                             // 13784:   }
                                             // 13784:   } // switch Inst[23:22]
                                             // 13784:  }
  7, 9,                                      // 13789:  case 0x7: {
  OPC_CheckField, 23, 1, 0,                  // 13791:   check Inst[23] == 0x0
  OPC_Decode, 251, 15, 229, 1,               // 13795:   decode to M4_mpyrr_addi using decoder 229
                                             // 13795:  }
  8, 5,                                      // 13800:  case 0x8: {
  OPC_Decode, 248, 15, 230, 1,               // 13802:   decode to M4_mpyri_addi using decoder 230
                                             // 13802:  }
  9, 25,                                     // 13807:  case 0x9: {
  OPC_SwitchField, 22, 2,                    // 13809:   switch Inst[23:22] {
  0, 9,                                      // 13812:   case 0x0: {
  OPC_CheckField, 16, 5, 0,                  // 13814:    check Inst[20:16] == 0x0
  OPC_Decode, 156, 10, 231, 1,               // 13818:    decode to F2_dfimm_p using decoder 231
                                             // 13818:   }
  1, 0,                                      // 13823:   case 0x1: {
  OPC_CheckField, 16, 5, 0,                  // 13825:    check Inst[20:16] == 0x0
  OPC_Decode, 155, 10, 231, 1,               // 13829:    decode to F2_dfimm_n using decoder 231
                                             // 13829:   }
                                             // 13829:   } // switch Inst[23:22]
                                             // 13829:  }
  10, 24,                                    // 13834:  case 0xa: {
  OPC_SwitchField, 22, 2,                    // 13836:   switch Inst[23:22] {
  0, 5,                                      // 13839:   case 0x0: {
  OPC_Decode, 248, 18, 232, 1,               // 13841:    decode to S4_or_andi using decoder 232
                                             // 13841:   }
  1, 5,                                      // 13846:   case 0x1: {
  OPC_Decode, 249, 18, 233, 1,               // 13848:    decode to S4_or_andix using decoder 233
                                             // 13848:   }
  2, 0,                                      // 13853:   case 0x2: {
  OPC_Decode, 250, 18, 232, 1,               // 13855:    decode to S4_or_ori using decoder 232
                                             // 13855:   }
                                             // 13855:   } // switch Inst[23:22]
                                             // 13855:  }
  11, 17,                                    // 13860:  case 0xb: {
  OPC_SwitchField, 23, 1,                    // 13862:   switch Inst[23] {
  0, 5,                                      // 13865:   case 0x0: {
  OPC_Decode, 233, 18, 234, 1,               // 13867:    decode to S4_addaddi using decoder 234
                                             // 13867:   }
  1, 0,                                      // 13872:   case 0x1: {
  OPC_Decode, 248, 19, 235, 1,               // 13874:    decode to S4_subaddi using decoder 235
                                             // 13874:   }
                                             // 13874:   } // switch Inst[23]
                                             // 13874:  }
  12, 132, 1,                                // 13879:  case 0xc: {
  OPC_SwitchField, 21, 3,                    // 13882:   switch Inst[23:21] {
  0, 36,                                     // 13885:   case 0x0: {
  OPC_SwitchField, 2, 3,                     // 13887:    switch Inst[4:2] {
  0, 9,                                      // 13890:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13892:     check Inst[13] == 0x0
  OPC_Decode, 148, 9, 236, 1,                // 13896:     decode to A4_vcmpbeqi using decoder 236
                                             // 13896:    }
  2, 9,                                      // 13901:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13903:     check Inst[13] == 0x0
  OPC_Decode, 152, 9, 237, 1,                // 13907:     decode to A4_vcmpheqi using decoder 237
                                             // 13907:    }
  4, 0,                                      // 13912:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 13914:     check Inst[13] == 0x0
  OPC_Decode, 155, 9, 237, 1,                // 13918:     decode to A4_vcmpweqi using decoder 237
                                             // 13918:    }
                                             // 13918:    } // switch Inst[4:2]
                                             // 13918:   }
  1, 36,                                     // 13923:   case 0x1: {
  OPC_SwitchField, 2, 3,                     // 13925:    switch Inst[4:2] {
  0, 9,                                      // 13928:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 13930:     check Inst[13] == 0x0
  OPC_Decode, 150, 9, 237, 1,                // 13934:     decode to A4_vcmpbgti using decoder 237
                                             // 13934:    }
  2, 9,                                      // 13939:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 13941:     check Inst[13] == 0x0
  OPC_Decode, 153, 9, 237, 1,                // 13945:     decode to A4_vcmphgti using decoder 237
                                             // 13945:    }
  4, 0,                                      // 13950:    case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 13952:     check Inst[13] == 0x0
  OPC_Decode, 156, 9, 237, 1,                // 13956:     decode to A4_vcmpwgti using decoder 237
                                             // 13956:    }
                                             // 13956:    } // switch Inst[4:2]
                                             // 13956:   }
  2, 36,                                     // 13961:   case 0x2: {
  OPC_SwitchField, 2, 3,                     // 13963:    switch Inst[4:2] {
  0, 9,                                      // 13966:    case 0x0: {
  OPC_CheckField, 12, 2, 0,                  // 13968:     check Inst[13:12] == 0x0
  OPC_Decode, 151, 9, 238, 1,                // 13972:     decode to A4_vcmpbgtui using decoder 238
                                             // 13972:    }
  2, 9,                                      // 13977:    case 0x2: {
  OPC_CheckField, 12, 2, 0,                  // 13979:     check Inst[13:12] == 0x0
  OPC_Decode, 154, 9, 238, 1,                // 13983:     decode to A4_vcmphgtui using decoder 238
                                             // 13983:    }
  4, 0,                                      // 13988:    case 0x4: {
  OPC_CheckField, 12, 2, 0,                  // 13990:     check Inst[13:12] == 0x0
  OPC_Decode, 157, 9, 238, 1,                // 13994:     decode to A4_vcmpwgtui using decoder 238
                                             // 13994:    }
                                             // 13994:    } // switch Inst[4:2]
                                             // 13994:   }
  4, 0,                                      // 13999:   case 0x4: {
  OPC_CheckField, 10, 4, 0,                  // 14001:    check Inst[13:10] == 0x0
  OPC_CheckField, 2, 3, 4,                   // 14005:    check Inst[4:2] == 0x4
  OPC_Decode, 150, 10, 239, 1,               // 14009:    decode to F2_dfclass using decoder 239
                                             // 14009:   }
                                             // 14009:   } // switch Inst[23:21]
                                             // 14009:  }
  13, 84,                                    // 14014:  case 0xd: {
  OPC_SwitchField, 21, 3,                    // 14016:   switch Inst[23:21] {
  0, 25,                                     // 14019:   case 0x0: {
  OPC_SwitchField, 2, 3,                     // 14021:    switch Inst[4:2] {
  0, 9,                                      // 14024:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14026:     check Inst[13] == 0x0
  OPC_Decode, 219, 8, 240, 1,                // 14030:     decode to A4_cmpbeqi using decoder 240
                                             // 14030:    }
  2, 0,                                      // 14035:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14037:     check Inst[13] == 0x0
  OPC_Decode, 225, 8, 241, 1,                // 14041:     decode to A4_cmpheqi using decoder 241
                                             // 14041:    }
                                             // 14041:    } // switch Inst[4:2]
                                             // 14041:   }
  1, 25,                                     // 14046:   case 0x1: {
  OPC_SwitchField, 2, 3,                     // 14048:    switch Inst[4:2] {
  0, 9,                                      // 14051:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14053:     check Inst[13] == 0x0
  OPC_Decode, 221, 8, 242, 1,                // 14057:     decode to A4_cmpbgti using decoder 242
                                             // 14057:    }
  2, 0,                                      // 14062:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14064:     check Inst[13] == 0x0
  OPC_Decode, 227, 8, 241, 1,                // 14068:     decode to A4_cmphgti using decoder 241
                                             // 14068:    }
                                             // 14068:    } // switch Inst[4:2]
                                             // 14068:   }
  2, 0,                                      // 14073:   case 0x2: {
  OPC_SwitchField, 2, 3,                     // 14075:    switch Inst[4:2] {
  0, 9,                                      // 14078:    case 0x0: {
  OPC_CheckField, 12, 2, 0,                  // 14080:     check Inst[13:12] == 0x0
  OPC_Decode, 223, 8, 243, 1,                // 14084:     decode to A4_cmpbgtui using decoder 243
                                             // 14084:    }
  2, 0,                                      // 14089:    case 0x2: {
  OPC_CheckField, 12, 2, 0,                  // 14091:     check Inst[13:12] == 0x0
  OPC_Decode, 229, 8, 243, 1,                // 14095:     decode to A4_cmphgtui using decoder 243
                                             // 14095:    }
                                             // 14095:    } // switch Inst[4:2]
                                             // 14095:   }
                                             // 14095:   } // switch Inst[23:21]
                                             // 14095:  }
  14, 79,                                    // 14100:  case 0xe: {
  OPC_SwitchField, 0, 3,                     // 14102:   switch Inst[2:0] {
  0, 17,                                     // 14105:   case 0x0: {
  OPC_SwitchField, 4, 1,                     // 14107:    switch Inst[4] {
  0, 5,                                      // 14110:    case 0x0: {
  OPC_Decode, 236, 18, 244, 1,               // 14112:     decode to S4_andi_asl_ri using decoder 244
                                             // 14112:    }
  1, 0,                                      // 14117:    case 0x1: {
  OPC_Decode, 237, 18, 244, 1,               // 14119:     decode to S4_andi_lsr_ri using decoder 244
                                             // 14119:    }
                                             // 14119:    } // switch Inst[4]
                                             // 14119:   }
  2, 17,                                     // 14124:   case 0x2: {
  OPC_SwitchField, 4, 1,                     // 14126:    switch Inst[4] {
  0, 5,                                      // 14129:    case 0x0: {
  OPC_Decode, 251, 18, 244, 1,               // 14131:     decode to S4_ori_asl_ri using decoder 244
                                             // 14131:    }
  1, 0,                                      // 14136:    case 0x1: {
  OPC_Decode, 252, 18, 244, 1,               // 14138:     decode to S4_ori_lsr_ri using decoder 244
                                             // 14138:    }
                                             // 14138:    } // switch Inst[4]
                                             // 14138:   }
  4, 17,                                     // 14143:   case 0x4: {
  OPC_SwitchField, 4, 1,                     // 14145:    switch Inst[4] {
  0, 5,                                      // 14148:    case 0x0: {
  OPC_Decode, 234, 18, 244, 1,               // 14150:     decode to S4_addi_asl_ri using decoder 244
                                             // 14150:    }
  1, 0,                                      // 14155:    case 0x1: {
  OPC_Decode, 235, 18, 244, 1,               // 14157:     decode to S4_addi_lsr_ri using decoder 244
                                             // 14157:    }
                                             // 14157:    } // switch Inst[4]
                                             // 14157:   }
  6, 0,                                      // 14162:   case 0x6: {
  OPC_SwitchField, 4, 1,                     // 14164:    switch Inst[4] {
  0, 5,                                      // 14167:    case 0x0: {
  OPC_Decode, 249, 19, 244, 1,               // 14169:     decode to S4_subi_asl_ri using decoder 244
                                             // 14169:    }
  1, 0,                                      // 14174:    case 0x1: {
  OPC_Decode, 250, 19, 244, 1,               // 14176:     decode to S4_subi_lsr_ri using decoder 244
                                             // 14176:    }
                                             // 14176:    } // switch Inst[4]
                                             // 14176:   }
                                             // 14176:   } // switch Inst[2:0]
                                             // 14176:  }
  15, 0,                                     // 14181:  case 0xf: {
  OPC_SwitchField, 23, 1,                    // 14183:   switch Inst[23] {
  0, 5,                                      // 14186:   case 0x0: {
  OPC_Decode, 250, 15, 245, 1,               // 14188:    decode to M4_mpyri_addr_u2 using decoder 245
                                             // 14188:   }
  1, 0,                                      // 14193:   case 0x1: {
  OPC_Decode, 249, 15, 246, 1,               // 14195:    decode to M4_mpyri_addr using decoder 246
                                             // 14195:   }
                                             // 14195:   } // switch Inst[23]
                                             // 14195:  }
                                             // 14195:  } // switch Inst[27:24]
                                             // 14195: }
  14, 146, 34,                               // 14200: case 0xe: {
  OPC_SwitchField, 21, 7,                    // 14203:  switch Inst[27:21] {
  0, 9,                                      // 14206:  case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14208:   check Inst[13] == 0x0
  OPC_Decode, 136, 15, 247, 1,               // 14212:   decode to M2_mpysip using decoder 247
                                             // 14212:  }
  4, 9,                                      // 14217:  case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 14219:   check Inst[13] == 0x0
  OPC_Decode, 135, 15, 247, 1,               // 14223:   decode to M2_mpysin using decoder 247
                                             // 14223:  }
  8, 9,                                      // 14228:  case 0x8: {
  OPC_CheckField, 13, 1, 0,                  // 14230:   check Inst[13] == 0x0
  OPC_Decode, 129, 14, 248, 1,               // 14234:   decode to M2_macsip using decoder 248
                                             // 14234:  }
  12, 9,                                     // 14239:  case 0xc: {
  OPC_CheckField, 13, 1, 0,                  // 14241:   check Inst[13] == 0x0
  OPC_Decode, 128, 14, 248, 1,               // 14245:   decode to M2_macsin using decoder 248
                                             // 14245:  }
  16, 9,                                     // 14250:  case 0x10: {
  OPC_CheckField, 13, 1, 0,                  // 14252:   check Inst[13] == 0x0
  OPC_Decode, 223, 13, 249, 1,               // 14256:   decode to M2_accii using decoder 249
                                             // 14256:  }
  20, 9,                                     // 14261:  case 0x14: {
  OPC_CheckField, 13, 1, 0,                  // 14263:   check Inst[13] == 0x0
  OPC_Decode, 188, 15, 249, 1,               // 14267:   decode to M2_naccii using decoder 249
                                             // 14267:  }
  24, 13,                                    // 14272:  case 0x18: {
  OPC_CheckField, 13, 1, 0,                  // 14274:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 14278:   check Inst[7:5] == 0x0
  OPC_Decode, 252, 15, 250, 1,               // 14282:   decode to M4_mpyrr_addr using decoder 250
                                             // 14282:  }
  32, 47,                                    // 14287:  case 0x20: {
  OPC_SwitchField, 5, 3,                     // 14289:   switch Inst[7:5] {
  0, 9,                                      // 14292:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14294:    check Inst[13] == 0x0
  OPC_Decode, 244, 14, 226, 1,               // 14298:    decode to M2_mpyd_ll_s0 using decoder 226
                                             // 14298:   }
  1, 9,                                      // 14303:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14305:    check Inst[13] == 0x0
  OPC_Decode, 242, 14, 226, 1,               // 14309:    decode to M2_mpyd_lh_s0 using decoder 226
                                             // 14309:   }
  2, 9,                                      // 14314:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14316:    check Inst[13] == 0x0
  OPC_Decode, 240, 14, 226, 1,               // 14320:    decode to M2_mpyd_hl_s0 using decoder 226
                                             // 14320:   }
  3, 0,                                      // 14325:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14327:    check Inst[13] == 0x0
  OPC_Decode, 238, 14, 226, 1,               // 14331:    decode to M2_mpyd_hh_s0 using decoder 226
                                             // 14331:   }
                                             // 14331:   } // switch Inst[7:5]
                                             // 14331:  }
  33, 47,                                    // 14336:  case 0x21: {
  OPC_SwitchField, 5, 3,                     // 14338:   switch Inst[7:5] {
  0, 9,                                      // 14341:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14343:    check Inst[13] == 0x0
  OPC_Decode, 132, 15, 226, 1,               // 14347:    decode to M2_mpyd_rnd_ll_s0 using decoder 226
                                             // 14347:   }
  1, 9,                                      // 14352:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14354:    check Inst[13] == 0x0
  OPC_Decode, 130, 15, 226, 1,               // 14358:    decode to M2_mpyd_rnd_lh_s0 using decoder 226
                                             // 14358:   }
  2, 9,                                      // 14363:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14365:    check Inst[13] == 0x0
  OPC_Decode, 128, 15, 226, 1,               // 14369:    decode to M2_mpyd_rnd_hl_s0 using decoder 226
                                             // 14369:   }
  3, 0,                                      // 14374:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14376:    check Inst[13] == 0x0
  OPC_Decode, 254, 14, 226, 1,               // 14380:    decode to M2_mpyd_rnd_hh_s0 using decoder 226
                                             // 14380:   }
                                             // 14380:   } // switch Inst[7:5]
                                             // 14380:  }
  34, 47,                                    // 14385:  case 0x22: {
  OPC_SwitchField, 5, 3,                     // 14387:   switch Inst[7:5] {
  0, 9,                                      // 14390:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14392:    check Inst[13] == 0x0
  OPC_Decode, 177, 15, 226, 1,               // 14396:    decode to M2_mpyud_ll_s0 using decoder 226
                                             // 14396:   }
  1, 9,                                      // 14401:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14403:    check Inst[13] == 0x0
  OPC_Decode, 175, 15, 226, 1,               // 14407:    decode to M2_mpyud_lh_s0 using decoder 226
                                             // 14407:   }
  2, 9,                                      // 14412:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14414:    check Inst[13] == 0x0
  OPC_Decode, 173, 15, 226, 1,               // 14418:    decode to M2_mpyud_hl_s0 using decoder 226
                                             // 14418:   }
  3, 0,                                      // 14423:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14425:    check Inst[13] == 0x0
  OPC_Decode, 171, 15, 226, 1,               // 14429:    decode to M2_mpyud_hh_s0 using decoder 226
                                             // 14429:   }
                                             // 14429:   } // switch Inst[7:5]
                                             // 14429:  }
  36, 47,                                    // 14434:  case 0x24: {
  OPC_SwitchField, 5, 3,                     // 14436:   switch Inst[7:5] {
  0, 9,                                      // 14439:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14441:    check Inst[13] == 0x0
  OPC_Decode, 245, 14, 226, 1,               // 14445:    decode to M2_mpyd_ll_s1 using decoder 226
                                             // 14445:   }
  1, 9,                                      // 14450:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14452:    check Inst[13] == 0x0
  OPC_Decode, 243, 14, 226, 1,               // 14456:    decode to M2_mpyd_lh_s1 using decoder 226
                                             // 14456:   }
  2, 9,                                      // 14461:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14463:    check Inst[13] == 0x0
  OPC_Decode, 241, 14, 226, 1,               // 14467:    decode to M2_mpyd_hl_s1 using decoder 226
                                             // 14467:   }
  3, 0,                                      // 14472:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14474:    check Inst[13] == 0x0
  OPC_Decode, 239, 14, 226, 1,               // 14478:    decode to M2_mpyd_hh_s1 using decoder 226
                                             // 14478:   }
                                             // 14478:   } // switch Inst[7:5]
                                             // 14478:  }
  37, 47,                                    // 14483:  case 0x25: {
  OPC_SwitchField, 5, 3,                     // 14485:   switch Inst[7:5] {
  0, 9,                                      // 14488:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14490:    check Inst[13] == 0x0
  OPC_Decode, 133, 15, 226, 1,               // 14494:    decode to M2_mpyd_rnd_ll_s1 using decoder 226
                                             // 14494:   }
  1, 9,                                      // 14499:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14501:    check Inst[13] == 0x0
  OPC_Decode, 131, 15, 226, 1,               // 14505:    decode to M2_mpyd_rnd_lh_s1 using decoder 226
                                             // 14505:   }
  2, 9,                                      // 14510:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14512:    check Inst[13] == 0x0
  OPC_Decode, 129, 15, 226, 1,               // 14516:    decode to M2_mpyd_rnd_hl_s1 using decoder 226
                                             // 14516:   }
  3, 0,                                      // 14521:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14523:    check Inst[13] == 0x0
  OPC_Decode, 255, 14, 226, 1,               // 14527:    decode to M2_mpyd_rnd_hh_s1 using decoder 226
                                             // 14527:   }
                                             // 14527:   } // switch Inst[7:5]
                                             // 14527:  }
  38, 47,                                    // 14532:  case 0x26: {
  OPC_SwitchField, 5, 3,                     // 14534:   switch Inst[7:5] {
  0, 9,                                      // 14537:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14539:    check Inst[13] == 0x0
  OPC_Decode, 178, 15, 226, 1,               // 14543:    decode to M2_mpyud_ll_s1 using decoder 226
                                             // 14543:   }
  1, 9,                                      // 14548:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14550:    check Inst[13] == 0x0
  OPC_Decode, 176, 15, 226, 1,               // 14554:    decode to M2_mpyud_lh_s1 using decoder 226
                                             // 14554:   }
  2, 9,                                      // 14559:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14561:    check Inst[13] == 0x0
  OPC_Decode, 174, 15, 226, 1,               // 14565:    decode to M2_mpyud_hl_s1 using decoder 226
                                             // 14565:   }
  3, 0,                                      // 14570:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14572:    check Inst[13] == 0x0
  OPC_Decode, 172, 15, 226, 1,               // 14576:    decode to M2_mpyud_hh_s1 using decoder 226
                                             // 14576:   }
                                             // 14576:   } // switch Inst[7:5]
                                             // 14576:  }
  40, 69,                                    // 14581:  case 0x28: {
  OPC_SwitchField, 5, 3,                     // 14583:   switch Inst[7:5] {
  0, 9,                                      // 14586:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14588:    check Inst[13] == 0x0
  OPC_Decode, 247, 13, 226, 1,               // 14592:    decode to M2_dpmpyss_s0 using decoder 226
                                             // 14592:   }
  1, 9,                                      // 14597:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14599:    check Inst[13] == 0x0
  OPC_Decode, 230, 13, 226, 1,               // 14603:    decode to M2_cmpyi_s0 using decoder 226
                                             // 14603:   }
  2, 9,                                      // 14608:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14610:    check Inst[13] == 0x0
  OPC_Decode, 231, 13, 226, 1,               // 14614:    decode to M2_cmpyr_s0 using decoder 226
                                             // 14614:   }
  5, 9,                                      // 14619:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 14621:    check Inst[13] == 0x0
  OPC_Decode, 214, 15, 226, 1,               // 14625:    decode to M2_vmpy2s_s0 using decoder 226
                                             // 14625:   }
  6, 9,                                      // 14630:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 14632:    check Inst[13] == 0x0
  OPC_Decode, 236, 13, 226, 1,               // 14636:    decode to M2_cmpys_s0 using decoder 226
                                             // 14636:   }
  7, 0,                                      // 14641:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 14643:    check Inst[13] == 0x0
  OPC_Decode, 218, 15, 226, 1,               // 14647:    decode to M2_vmpy2su_s0 using decoder 226
                                             // 14647:   }
                                             // 14647:   } // switch Inst[7:5]
                                             // 14647:  }
  42, 47,                                    // 14652:  case 0x2a: {
  OPC_SwitchField, 5, 3,                     // 14654:   switch Inst[7:5] {
  0, 9,                                      // 14657:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14659:    check Inst[13] == 0x0
  OPC_Decode, 250, 13, 226, 1,               // 14663:    decode to M2_dpmpyuu_s0 using decoder 226
                                             // 14663:   }
  1, 9,                                      // 14668:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14670:    check Inst[13] == 0x0
  OPC_Decode, 150, 16, 226, 1,               // 14674:    decode to M5_vmpybsu using decoder 226
                                             // 14674:   }
  6, 9,                                      // 14679:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 14681:    check Inst[13] == 0x0
  OPC_Decode, 238, 13, 226, 1,               // 14685:    decode to M2_cmpysc_s0 using decoder 226
                                             // 14685:   }
  7, 0,                                      // 14690:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 14692:    check Inst[13] == 0x0
  OPC_Decode, 130, 16, 226, 1,               // 14696:    decode to M4_pmpyw using decoder 226
                                             // 14696:   }
                                             // 14696:   } // switch Inst[7:5]
                                             // 14696:  }
  44, 47,                                    // 14701:  case 0x2c: {
  OPC_SwitchField, 5, 3,                     // 14703:   switch Inst[7:5] {
  1, 9,                                      // 14706:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14708:    check Inst[13] == 0x0
  OPC_Decode, 151, 16, 226, 1,               // 14712:    decode to M5_vmpybuu using decoder 226
                                             // 14712:   }
  5, 9,                                      // 14717:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 14719:    check Inst[13] == 0x0
  OPC_Decode, 216, 15, 226, 1,               // 14723:    decode to M2_vmpy2s_s1 using decoder 226
                                             // 14723:   }
  6, 9,                                      // 14728:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 14730:    check Inst[13] == 0x0
  OPC_Decode, 237, 13, 226, 1,               // 14734:    decode to M2_cmpys_s1 using decoder 226
                                             // 14734:   }
  7, 0,                                      // 14739:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 14741:    check Inst[13] == 0x0
  OPC_Decode, 219, 15, 226, 1,               // 14745:    decode to M2_vmpy2su_s1 using decoder 226
                                             // 14745:   }
                                             // 14745:   } // switch Inst[7:5]
                                             // 14745:  }
  46, 25,                                    // 14750:  case 0x2e: {
  OPC_SwitchField, 5, 3,                     // 14752:   switch Inst[7:5] {
  6, 9,                                      // 14755:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 14757:    check Inst[13] == 0x0
  OPC_Decode, 239, 13, 226, 1,               // 14761:    decode to M2_cmpysc_s1 using decoder 226
                                             // 14761:   }
  7, 0,                                      // 14766:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 14768:    check Inst[13] == 0x0
  OPC_Decode, 132, 16, 226, 1,               // 14772:    decode to M4_vpmpyh using decoder 226
                                             // 14772:   }
                                             // 14772:   } // switch Inst[7:5]
                                             // 14772:  }
  48, 47,                                    // 14777:  case 0x30: {
  OPC_SwitchField, 5, 3,                     // 14779:   switch Inst[7:5] {
  0, 9,                                      // 14782:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14784:    check Inst[13] == 0x0
  OPC_Decode, 236, 14, 251, 1,               // 14788:    decode to M2_mpyd_acc_ll_s0 using decoder 251
                                             // 14788:   }
  1, 9,                                      // 14793:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14795:    check Inst[13] == 0x0
  OPC_Decode, 234, 14, 251, 1,               // 14799:    decode to M2_mpyd_acc_lh_s0 using decoder 251
                                             // 14799:   }
  2, 9,                                      // 14804:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14806:    check Inst[13] == 0x0
  OPC_Decode, 232, 14, 251, 1,               // 14810:    decode to M2_mpyd_acc_hl_s0 using decoder 251
                                             // 14810:   }
  3, 0,                                      // 14815:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14817:    check Inst[13] == 0x0
  OPC_Decode, 230, 14, 251, 1,               // 14821:    decode to M2_mpyd_acc_hh_s0 using decoder 251
                                             // 14821:   }
                                             // 14821:   } // switch Inst[7:5]
                                             // 14821:  }
  49, 47,                                    // 14826:  case 0x31: {
  OPC_SwitchField, 5, 3,                     // 14828:   switch Inst[7:5] {
  0, 9,                                      // 14831:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14833:    check Inst[13] == 0x0
  OPC_Decode, 252, 14, 251, 1,               // 14837:    decode to M2_mpyd_nac_ll_s0 using decoder 251
                                             // 14837:   }
  1, 9,                                      // 14842:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14844:    check Inst[13] == 0x0
  OPC_Decode, 250, 14, 251, 1,               // 14848:    decode to M2_mpyd_nac_lh_s0 using decoder 251
                                             // 14848:   }
  2, 9,                                      // 14853:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14855:    check Inst[13] == 0x0
  OPC_Decode, 248, 14, 251, 1,               // 14859:    decode to M2_mpyd_nac_hl_s0 using decoder 251
                                             // 14859:   }
  3, 0,                                      // 14864:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14866:    check Inst[13] == 0x0
  OPC_Decode, 246, 14, 251, 1,               // 14870:    decode to M2_mpyd_nac_hh_s0 using decoder 251
                                             // 14870:   }
                                             // 14870:   } // switch Inst[7:5]
                                             // 14870:  }
  50, 47,                                    // 14875:  case 0x32: {
  OPC_SwitchField, 5, 3,                     // 14877:   switch Inst[7:5] {
  0, 9,                                      // 14880:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14882:    check Inst[13] == 0x0
  OPC_Decode, 169, 15, 251, 1,               // 14886:    decode to M2_mpyud_acc_ll_s0 using decoder 251
                                             // 14886:   }
  1, 9,                                      // 14891:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14893:    check Inst[13] == 0x0
  OPC_Decode, 167, 15, 251, 1,               // 14897:    decode to M2_mpyud_acc_lh_s0 using decoder 251
                                             // 14897:   }
  2, 9,                                      // 14902:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14904:    check Inst[13] == 0x0
  OPC_Decode, 165, 15, 251, 1,               // 14908:    decode to M2_mpyud_acc_hl_s0 using decoder 251
                                             // 14908:   }
  3, 0,                                      // 14913:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14915:    check Inst[13] == 0x0
  OPC_Decode, 163, 15, 251, 1,               // 14919:    decode to M2_mpyud_acc_hh_s0 using decoder 251
                                             // 14919:   }
                                             // 14919:   } // switch Inst[7:5]
                                             // 14919:  }
  51, 47,                                    // 14924:  case 0x33: {
  OPC_SwitchField, 5, 3,                     // 14926:   switch Inst[7:5] {
  0, 9,                                      // 14929:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14931:    check Inst[13] == 0x0
  OPC_Decode, 185, 15, 251, 1,               // 14935:    decode to M2_mpyud_nac_ll_s0 using decoder 251
                                             // 14935:   }
  1, 9,                                      // 14940:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14942:    check Inst[13] == 0x0
  OPC_Decode, 183, 15, 251, 1,               // 14946:    decode to M2_mpyud_nac_lh_s0 using decoder 251
                                             // 14946:   }
  2, 9,                                      // 14951:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 14953:    check Inst[13] == 0x0
  OPC_Decode, 181, 15, 251, 1,               // 14957:    decode to M2_mpyud_nac_hl_s0 using decoder 251
                                             // 14957:   }
  3, 0,                                      // 14962:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 14964:    check Inst[13] == 0x0
  OPC_Decode, 179, 15, 251, 1,               // 14968:    decode to M2_mpyud_nac_hh_s0 using decoder 251
                                             // 14968:   }
                                             // 14968:   } // switch Inst[7:5]
                                             // 14968:  }
  52, 47,                                    // 14973:  case 0x34: {
  OPC_SwitchField, 5, 3,                     // 14975:   switch Inst[7:5] {
  0, 9,                                      // 14978:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 14980:    check Inst[13] == 0x0
  OPC_Decode, 237, 14, 251, 1,               // 14984:    decode to M2_mpyd_acc_ll_s1 using decoder 251
                                             // 14984:   }
  1, 9,                                      // 14989:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 14991:    check Inst[13] == 0x0
  OPC_Decode, 235, 14, 251, 1,               // 14995:    decode to M2_mpyd_acc_lh_s1 using decoder 251
                                             // 14995:   }
  2, 9,                                      // 15000:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15002:    check Inst[13] == 0x0
  OPC_Decode, 233, 14, 251, 1,               // 15006:    decode to M2_mpyd_acc_hl_s1 using decoder 251
                                             // 15006:   }
  3, 0,                                      // 15011:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 15013:    check Inst[13] == 0x0
  OPC_Decode, 231, 14, 251, 1,               // 15017:    decode to M2_mpyd_acc_hh_s1 using decoder 251
                                             // 15017:   }
                                             // 15017:   } // switch Inst[7:5]
                                             // 15017:  }
  53, 47,                                    // 15022:  case 0x35: {
  OPC_SwitchField, 5, 3,                     // 15024:   switch Inst[7:5] {
  0, 9,                                      // 15027:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15029:    check Inst[13] == 0x0
  OPC_Decode, 253, 14, 251, 1,               // 15033:    decode to M2_mpyd_nac_ll_s1 using decoder 251
                                             // 15033:   }
  1, 9,                                      // 15038:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15040:    check Inst[13] == 0x0
  OPC_Decode, 251, 14, 251, 1,               // 15044:    decode to M2_mpyd_nac_lh_s1 using decoder 251
                                             // 15044:   }
  2, 9,                                      // 15049:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15051:    check Inst[13] == 0x0
  OPC_Decode, 249, 14, 251, 1,               // 15055:    decode to M2_mpyd_nac_hl_s1 using decoder 251
                                             // 15055:   }
  3, 0,                                      // 15060:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 15062:    check Inst[13] == 0x0
  OPC_Decode, 247, 14, 251, 1,               // 15066:    decode to M2_mpyd_nac_hh_s1 using decoder 251
                                             // 15066:   }
                                             // 15066:   } // switch Inst[7:5]
                                             // 15066:  }
  54, 47,                                    // 15071:  case 0x36: {
  OPC_SwitchField, 5, 3,                     // 15073:   switch Inst[7:5] {
  0, 9,                                      // 15076:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15078:    check Inst[13] == 0x0
  OPC_Decode, 170, 15, 251, 1,               // 15082:    decode to M2_mpyud_acc_ll_s1 using decoder 251
                                             // 15082:   }
  1, 9,                                      // 15087:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15089:    check Inst[13] == 0x0
  OPC_Decode, 168, 15, 251, 1,               // 15093:    decode to M2_mpyud_acc_lh_s1 using decoder 251
                                             // 15093:   }
  2, 9,                                      // 15098:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15100:    check Inst[13] == 0x0
  OPC_Decode, 166, 15, 251, 1,               // 15104:    decode to M2_mpyud_acc_hl_s1 using decoder 251
                                             // 15104:   }
  3, 0,                                      // 15109:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 15111:    check Inst[13] == 0x0
  OPC_Decode, 164, 15, 251, 1,               // 15115:    decode to M2_mpyud_acc_hh_s1 using decoder 251
                                             // 15115:   }
                                             // 15115:   } // switch Inst[7:5]
                                             // 15115:  }
  55, 47,                                    // 15120:  case 0x37: {
  OPC_SwitchField, 5, 3,                     // 15122:   switch Inst[7:5] {
  0, 9,                                      // 15125:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15127:    check Inst[13] == 0x0
  OPC_Decode, 186, 15, 251, 1,               // 15131:    decode to M2_mpyud_nac_ll_s1 using decoder 251
                                             // 15131:   }
  1, 9,                                      // 15136:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15138:    check Inst[13] == 0x0
  OPC_Decode, 184, 15, 251, 1,               // 15142:    decode to M2_mpyud_nac_lh_s1 using decoder 251
                                             // 15142:   }
  2, 9,                                      // 15147:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15149:    check Inst[13] == 0x0
  OPC_Decode, 182, 15, 251, 1,               // 15153:    decode to M2_mpyud_nac_hl_s1 using decoder 251
                                             // 15153:   }
  3, 0,                                      // 15158:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 15160:    check Inst[13] == 0x0
  OPC_Decode, 180, 15, 251, 1,               // 15164:    decode to M2_mpyud_nac_hh_s1 using decoder 251
                                             // 15164:   }
                                             // 15164:   } // switch Inst[7:5]
                                             // 15164:  }
  56, 69,                                    // 15169:  case 0x38: {
  OPC_SwitchField, 5, 3,                     // 15171:   switch Inst[7:5] {
  0, 9,                                      // 15174:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15176:    check Inst[13] == 0x0
  OPC_Decode, 244, 13, 251, 1,               // 15180:    decode to M2_dpmpyss_acc_s0 using decoder 251
                                             // 15180:   }
  1, 9,                                      // 15185:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15187:    check Inst[13] == 0x0
  OPC_Decode, 224, 13, 251, 1,               // 15191:    decode to M2_cmaci_s0 using decoder 251
                                             // 15191:   }
  2, 9,                                      // 15196:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15198:    check Inst[13] == 0x0
  OPC_Decode, 225, 13, 251, 1,               // 15202:    decode to M2_cmacr_s0 using decoder 251
                                             // 15202:   }
  5, 9,                                      // 15207:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15209:    check Inst[13] == 0x0
  OPC_Decode, 208, 15, 251, 1,               // 15213:    decode to M2_vmac2s_s0 using decoder 251
                                             // 15213:   }
  6, 9,                                      // 15218:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15220:    check Inst[13] == 0x0
  OPC_Decode, 226, 13, 251, 1,               // 15224:    decode to M2_cmacs_s0 using decoder 251
                                             // 15224:   }
  7, 0,                                      // 15229:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15231:    check Inst[13] == 0x0
  OPC_Decode, 240, 13, 251, 1,               // 15235:    decode to M2_cnacs_s0 using decoder 251
                                             // 15235:   }
                                             // 15235:   } // switch Inst[7:5]
                                             // 15235:  }
  57, 36,                                    // 15240:  case 0x39: {
  OPC_SwitchField, 5, 3,                     // 15242:   switch Inst[7:5] {
  0, 9,                                      // 15245:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15247:    check Inst[13] == 0x0
  OPC_Decode, 245, 13, 251, 1,               // 15251:    decode to M2_dpmpyss_nac_s0 using decoder 251
                                             // 15251:   }
  1, 9,                                      // 15256:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15258:    check Inst[13] == 0x0
  OPC_Decode, 204, 15, 251, 1,               // 15262:    decode to M2_vmac2 using decoder 251
                                             // 15262:   }
  7, 0,                                      // 15267:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15269:    check Inst[13] == 0x0
  OPC_Decode, 131, 16, 251, 1,               // 15273:    decode to M4_pmpyw_acc using decoder 251
                                             // 15273:   }
                                             // 15273:   } // switch Inst[7:5]
                                             // 15273:  }
  58, 36,                                    // 15278:  case 0x3a: {
  OPC_SwitchField, 5, 3,                     // 15280:   switch Inst[7:5] {
  0, 9,                                      // 15283:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15285:    check Inst[13] == 0x0
  OPC_Decode, 248, 13, 251, 1,               // 15289:    decode to M2_dpmpyuu_acc_s0 using decoder 251
                                             // 15289:   }
  6, 9,                                      // 15294:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15296:    check Inst[13] == 0x0
  OPC_Decode, 228, 13, 251, 1,               // 15300:    decode to M2_cmacsc_s0 using decoder 251
                                             // 15300:   }
  7, 0,                                      // 15305:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15307:    check Inst[13] == 0x0
  OPC_Decode, 242, 13, 251, 1,               // 15311:    decode to M2_cnacsc_s0 using decoder 251
                                             // 15311:   }
                                             // 15311:   } // switch Inst[7:5]
                                             // 15311:  }
  59, 25,                                    // 15316:  case 0x3b: {
  OPC_SwitchField, 5, 3,                     // 15318:   switch Inst[7:5] {
  0, 9,                                      // 15321:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15323:    check Inst[13] == 0x0
  OPC_Decode, 249, 13, 251, 1,               // 15327:    decode to M2_dpmpyuu_nac_s0 using decoder 251
                                             // 15327:   }
  5, 0,                                      // 15332:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15334:    check Inst[13] == 0x0
  OPC_Decode, 210, 15, 251, 1,               // 15338:    decode to M2_vmac2su_s0 using decoder 251
                                             // 15338:   }
                                             // 15338:   } // switch Inst[7:5]
                                             // 15338:  }
  60, 47,                                    // 15343:  case 0x3c: {
  OPC_SwitchField, 5, 3,                     // 15345:   switch Inst[7:5] {
  1, 9,                                      // 15348:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15350:    check Inst[13] == 0x0
  OPC_Decode, 149, 16, 251, 1,               // 15354:    decode to M5_vmacbuu using decoder 251
                                             // 15354:   }
  5, 9,                                      // 15359:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15361:    check Inst[13] == 0x0
  OPC_Decode, 209, 15, 251, 1,               // 15365:    decode to M2_vmac2s_s1 using decoder 251
                                             // 15365:   }
  6, 9,                                      // 15370:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15372:    check Inst[13] == 0x0
  OPC_Decode, 227, 13, 251, 1,               // 15376:    decode to M2_cmacs_s1 using decoder 251
                                             // 15376:   }
  7, 0,                                      // 15381:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15383:    check Inst[13] == 0x0
  OPC_Decode, 241, 13, 251, 1,               // 15387:    decode to M2_cnacs_s1 using decoder 251
                                             // 15387:   }
                                             // 15387:   } // switch Inst[7:5]
                                             // 15387:  }
  61, 13,                                    // 15392:  case 0x3d: {
  OPC_CheckField, 13, 1, 0,                  // 15394:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 7,                   // 15398:   check Inst[7:5] == 0x7
  OPC_Decode, 133, 16, 251, 1,               // 15402:   decode to M4_vpmpyh_acc using decoder 251
                                             // 15402:  }
  62, 36,                                    // 15407:  case 0x3e: {
  OPC_SwitchField, 5, 3,                     // 15409:   switch Inst[7:5] {
  1, 9,                                      // 15412:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15414:    check Inst[13] == 0x0
  OPC_Decode, 148, 16, 251, 1,               // 15418:    decode to M5_vmacbsu using decoder 251
                                             // 15418:   }
  6, 9,                                      // 15423:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15425:    check Inst[13] == 0x0
  OPC_Decode, 229, 13, 251, 1,               // 15429:    decode to M2_cmacsc_s1 using decoder 251
                                             // 15429:   }
  7, 0,                                      // 15434:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15436:    check Inst[13] == 0x0
  OPC_Decode, 243, 13, 251, 1,               // 15440:    decode to M2_cnacsc_s1 using decoder 251
                                             // 15440:   }
                                             // 15440:   } // switch Inst[7:5]
                                             // 15440:  }
  63, 13,                                    // 15445:  case 0x3f: {
  OPC_CheckField, 13, 1, 0,                  // 15447:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 5,                   // 15451:   check Inst[7:5] == 0x5
  OPC_Decode, 211, 15, 251, 1,               // 15455:   decode to M2_vmac2su_s1 using decoder 251
                                             // 15455:  }
  64, 93,                                    // 15460:  case 0x40: {
  OPC_SwitchField, 5, 3,                     // 15462:   switch Inst[7:5] {
  0, 9,                                      // 15465:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15467:    check Inst[13] == 0x0
  OPC_Decode, 226, 15, 206, 1,               // 15471:    decode to M2_vrcmpyi_s0 using decoder 206
                                             // 15471:   }
  1, 9,                                      // 15476:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15478:    check Inst[13] == 0x0
  OPC_Decode, 228, 15, 206, 1,               // 15482:    decode to M2_vrcmpyr_s0 using decoder 206
                                             // 15482:   }
  2, 9,                                      // 15487:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15489:    check Inst[13] == 0x0
  OPC_Decode, 237, 15, 206, 1,               // 15493:    decode to M2_vrmpy_s0 using decoder 206
                                             // 15493:   }
  3, 11,                                     // 15498:   case 0x3: {
  OPC_CheckPredicate, 4,                     // 15500:    check predicate 4
  OPC_CheckField, 13, 1, 0,                  // 15502:    check Inst[13] == 0x0
  OPC_Decode, 149, 10, 206, 1,               // 15506:    decode to F2_dfadd using decoder 206
                                             // 15506:   }
  4, 9,                                      // 15511:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 15513:    check Inst[13] == 0x0
  OPC_Decode, 202, 15, 206, 1,               // 15517:    decode to M2_vdmpys_s0 using decoder 206
                                             // 15517:   }
  5, 9,                                      // 15522:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15524:    check Inst[13] == 0x0
  OPC_Decode, 152, 14, 206, 1,               // 15528:    decode to M2_mmpyl_s0 using decoder 206
                                             // 15528:   }
  6, 9,                                      // 15533:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15535:    check Inst[13] == 0x0
  OPC_Decode, 212, 15, 206, 1,               // 15539:    decode to M2_vmpy2es_s0 using decoder 206
                                             // 15539:   }
  7, 0,                                      // 15544:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15546:    check Inst[13] == 0x0
  OPC_Decode, 148, 14, 206, 1,               // 15550:    decode to M2_mmpyh_s0 using decoder 206
                                             // 15550:   }
                                             // 15550:   } // switch Inst[7:5]
                                             // 15550:  }
  65, 71,                                    // 15555:  case 0x41: {
  OPC_SwitchField, 5, 3,                     // 15557:   switch Inst[7:5] {
  0, 9,                                      // 15560:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15562:    check Inst[13] == 0x0
  OPC_Decode, 191, 15, 207, 1,               // 15566:    decode to M2_vabsdiffw using decoder 207
                                             // 15566:   }
  2, 9,                                      // 15571:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15573:    check Inst[13] == 0x0
  OPC_Decode, 140, 16, 206, 1,               // 15577:    decode to M4_vrmpyoh_s0 using decoder 206
                                             // 15577:   }
  3, 11,                                     // 15582:   case 0x3: {
  OPC_CheckPredicate, 5,                     // 15584:    check predicate 5
  OPC_CheckField, 13, 1, 0,                  // 15586:    check Inst[13] == 0x0
  OPC_Decode, 157, 10, 206, 1,               // 15590:    decode to F2_dfmax using decoder 206
                                             // 15590:   }
  5, 9,                                      // 15595:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15597:    check Inst[13] == 0x0
  OPC_Decode, 150, 14, 206, 1,               // 15601:    decode to M2_mmpyl_rs0 using decoder 206
                                             // 15601:   }
  6, 9,                                      // 15606:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15608:    check Inst[13] == 0x0
  OPC_Decode, 195, 15, 206, 1,               // 15612:    decode to M2_vcmpy_s0_sat_r using decoder 206
                                             // 15612:   }
  7, 0,                                      // 15617:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15619:    check Inst[13] == 0x0
  OPC_Decode, 146, 14, 206, 1,               // 15623:    decode to M2_mmpyh_rs0 using decoder 206
                                             // 15623:   }
                                             // 15623:   } // switch Inst[7:5]
                                             // 15623:  }
  66, 93,                                    // 15628:  case 0x42: {
  OPC_SwitchField, 5, 3,                     // 15630:   switch Inst[7:5] {
  0, 9,                                      // 15633:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15635:    check Inst[13] == 0x0
  OPC_Decode, 227, 15, 206, 1,               // 15639:    decode to M2_vrcmpyi_s0c using decoder 206
                                             // 15639:   }
  1, 9,                                      // 15644:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15646:    check Inst[13] == 0x0
  OPC_Decode, 197, 8, 206, 1,                // 15650:    decode to A2_vraddub using decoder 206
                                             // 15650:   }
  2, 9,                                      // 15655:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15657:    check Inst[13] == 0x0
  OPC_Decode, 199, 8, 206, 1,                // 15661:    decode to A2_vrsadub using decoder 206
                                             // 15661:   }
  3, 11,                                     // 15666:   case 0x3: {
  OPC_CheckPredicate, 5,                     // 15668:    check predicate 5
  OPC_CheckField, 13, 1, 0,                  // 15670:    check Inst[13] == 0x0
  OPC_Decode, 159, 10, 206, 1,               // 15674:    decode to F2_dfmpyfix using decoder 206
                                             // 15674:   }
  4, 9,                                      // 15679:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 15681:    check Inst[13] == 0x0
  OPC_Decode, 136, 16, 206, 1,               // 15685:    decode to M4_vrmpyeh_s0 using decoder 206
                                             // 15685:   }
  5, 9,                                      // 15690:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15692:    check Inst[13] == 0x0
  OPC_Decode, 160, 14, 206, 1,               // 15696:    decode to M2_mmpyul_s0 using decoder 206
                                             // 15696:   }
  6, 9,                                      // 15701:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15703:    check Inst[13] == 0x0
  OPC_Decode, 194, 15, 206, 1,               // 15707:    decode to M2_vcmpy_s0_sat_i using decoder 206
                                             // 15707:   }
  7, 0,                                      // 15712:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15714:    check Inst[13] == 0x0
  OPC_Decode, 156, 14, 206, 1,               // 15718:    decode to M2_mmpyuh_s0 using decoder 206
                                             // 15718:   }
                                             // 15718:   } // switch Inst[7:5]
                                             // 15718:  }
  67, 60,                                    // 15723:  case 0x43: {
  OPC_SwitchField, 5, 3,                     // 15725:   switch Inst[7:5] {
  0, 9,                                      // 15728:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 15730:    check Inst[13] == 0x0
  OPC_Decode, 190, 15, 207, 1,               // 15734:    decode to M2_vabsdiffh using decoder 207
                                             // 15734:   }
  1, 9,                                      // 15739:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15741:    check Inst[13] == 0x0
  OPC_Decode, 229, 15, 206, 1,               // 15745:    decode to M2_vrcmpyr_s0c using decoder 206
                                             // 15745:   }
  2, 11,                                     // 15750:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 15752:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 15754:    check Inst[13] == 0x0
  OPC_Decode, 158, 16, 206, 1,               // 15758:    decode to M7_dcmpyiw using decoder 206
                                             // 15758:   }
  5, 9,                                      // 15763:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15765:    check Inst[13] == 0x0
  OPC_Decode, 158, 14, 206, 1,               // 15769:    decode to M2_mmpyul_rs0 using decoder 206
                                             // 15769:   }
  7, 0,                                      // 15774:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15776:    check Inst[13] == 0x0
  OPC_Decode, 154, 14, 206, 1,               // 15780:    decode to M2_mmpyuh_rs0 using decoder 206
                                             // 15780:   }
                                             // 15780:   } // switch Inst[7:5]
                                             // 15780:  }
  68, 84,                                    // 15785:  case 0x44: {
  OPC_SwitchField, 5, 3,                     // 15787:   switch Inst[7:5] {
  1, 9,                                      // 15790:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15792:    check Inst[13] == 0x0
  OPC_Decode, 155, 16, 206, 1,               // 15796:    decode to M5_vrmpybuu using decoder 206
                                             // 15796:   }
  2, 11,                                     // 15801:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 15803:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 15805:    check Inst[13] == 0x0
  OPC_Decode, 162, 16, 206, 1,               // 15809:    decode to M7_dcmpyrw using decoder 206
                                             // 15809:   }
  3, 11,                                     // 15814:   case 0x3: {
  OPC_CheckPredicate, 4,                     // 15816:    check predicate 4
  OPC_CheckField, 13, 1, 0,                  // 15818:    check Inst[13] == 0x0
  OPC_Decode, 163, 10, 206, 1,               // 15822:    decode to F2_dfsub using decoder 206
                                             // 15822:   }
  4, 9,                                      // 15827:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 15829:    check Inst[13] == 0x0
  OPC_Decode, 203, 15, 206, 1,               // 15833:    decode to M2_vdmpys_s1 using decoder 206
                                             // 15833:   }
  5, 9,                                      // 15838:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15840:    check Inst[13] == 0x0
  OPC_Decode, 153, 14, 206, 1,               // 15844:    decode to M2_mmpyl_s1 using decoder 206
                                             // 15844:   }
  6, 9,                                      // 15849:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15851:    check Inst[13] == 0x0
  OPC_Decode, 213, 15, 206, 1,               // 15855:    decode to M2_vmpy2es_s1 using decoder 206
                                             // 15855:   }
  7, 0,                                      // 15860:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15862:    check Inst[13] == 0x0
  OPC_Decode, 149, 14, 206, 1,               // 15866:    decode to M2_mmpyh_s1 using decoder 206
                                             // 15866:   }
                                             // 15866:   } // switch Inst[7:5]
                                             // 15866:  }
  69, 95,                                    // 15871:  case 0x45: {
  OPC_SwitchField, 5, 3,                     // 15873:   switch Inst[7:5] {
  0, 11,                                     // 15876:   case 0x0: {
  OPC_CheckPredicate, 7,                     // 15878:    check predicate 7
  OPC_CheckField, 13, 1, 0,                  // 15880:    check Inst[13] == 0x0
  OPC_Decode, 157, 16, 207, 1,               // 15884:    decode to M6_vabsdiffub using decoder 207
                                             // 15884:   }
  1, 9,                                      // 15889:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15891:    check Inst[13] == 0x0
  OPC_Decode, 147, 16, 206, 1,               // 15895:    decode to M5_vdmpybsu using decoder 206
                                             // 15895:   }
  2, 9,                                      // 15900:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 15902:    check Inst[13] == 0x0
  OPC_Decode, 141, 16, 206, 1,               // 15906:    decode to M4_vrmpyoh_s1 using decoder 206
                                             // 15906:   }
  3, 11,                                     // 15911:   case 0x3: {
  OPC_CheckPredicate, 5,                     // 15913:    check predicate 5
  OPC_CheckField, 13, 1, 0,                  // 15915:    check Inst[13] == 0x0
  OPC_Decode, 162, 10, 206, 1,               // 15919:    decode to F2_dfmpyll using decoder 206
                                             // 15919:   }
  4, 9,                                      // 15924:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 15926:    check Inst[13] == 0x0
  OPC_Decode, 232, 15, 206, 1,               // 15930:    decode to M2_vrcmpys_s1_h using decoder 206
                                             // 15930:   }
  5, 9,                                      // 15935:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 15937:    check Inst[13] == 0x0
  OPC_Decode, 151, 14, 206, 1,               // 15941:    decode to M2_mmpyl_rs1 using decoder 206
                                             // 15941:   }
  6, 9,                                      // 15946:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 15948:    check Inst[13] == 0x0
  OPC_Decode, 197, 15, 206, 1,               // 15952:    decode to M2_vcmpy_s1_sat_r using decoder 206
                                             // 15952:   }
  7, 0,                                      // 15957:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 15959:    check Inst[13] == 0x0
  OPC_Decode, 147, 14, 206, 1,               // 15963:    decode to M2_mmpyh_rs1 using decoder 206
                                             // 15963:   }
                                             // 15963:   } // switch Inst[7:5]
                                             // 15963:  }
  70, 84,                                    // 15968:  case 0x46: {
  OPC_SwitchField, 5, 3,                     // 15970:   switch Inst[7:5] {
  1, 9,                                      // 15973:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 15975:    check Inst[13] == 0x0
  OPC_Decode, 154, 16, 206, 1,               // 15979:    decode to M5_vrmpybsu using decoder 206
                                             // 15979:   }
  2, 11,                                     // 15984:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 15986:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 15988:    check Inst[13] == 0x0
  OPC_Decode, 164, 16, 206, 1,               // 15992:    decode to M7_dcmpyrwc using decoder 206
                                             // 15992:   }
  3, 11,                                     // 15997:   case 0x3: {
  OPC_CheckPredicate, 5,                     // 15999:    check predicate 5
  OPC_CheckField, 13, 1, 0,                  // 16001:    check Inst[13] == 0x0
  OPC_Decode, 158, 10, 206, 1,               // 16005:    decode to F2_dfmin using decoder 206
                                             // 16005:   }
  4, 9,                                      // 16010:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 16012:    check Inst[13] == 0x0
  OPC_Decode, 137, 16, 206, 1,               // 16016:    decode to M4_vrmpyeh_s1 using decoder 206
                                             // 16016:   }
  5, 9,                                      // 16021:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16023:    check Inst[13] == 0x0
  OPC_Decode, 161, 14, 206, 1,               // 16027:    decode to M2_mmpyul_s1 using decoder 206
                                             // 16027:   }
  6, 9,                                      // 16032:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 16034:    check Inst[13] == 0x0
  OPC_Decode, 196, 15, 206, 1,               // 16038:    decode to M2_vcmpy_s1_sat_i using decoder 206
                                             // 16038:   }
  7, 0,                                      // 16043:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16045:    check Inst[13] == 0x0
  OPC_Decode, 157, 14, 206, 1,               // 16049:    decode to M2_mmpyuh_s1 using decoder 206
                                             // 16049:   }
                                             // 16049:   } // switch Inst[7:5]
                                             // 16049:  }
  71, 62,                                    // 16054:  case 0x47: {
  OPC_SwitchField, 5, 3,                     // 16056:   switch Inst[7:5] {
  0, 11,                                     // 16059:   case 0x0: {
  OPC_CheckPredicate, 7,                     // 16061:    check predicate 7
  OPC_CheckField, 13, 1, 0,                  // 16063:    check Inst[13] == 0x0
  OPC_Decode, 156, 16, 207, 1,               // 16067:    decode to M6_vabsdiffb using decoder 207
                                             // 16067:   }
  2, 11,                                     // 16072:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 16074:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16076:    check Inst[13] == 0x0
  OPC_Decode, 160, 16, 206, 1,               // 16080:    decode to M7_dcmpyiwc using decoder 206
                                             // 16080:   }
  4, 9,                                      // 16085:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 16087:    check Inst[13] == 0x0
  OPC_Decode, 233, 15, 206, 1,               // 16091:    decode to M2_vrcmpys_s1_l using decoder 206
                                             // 16091:   }
  5, 9,                                      // 16096:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16098:    check Inst[13] == 0x0
  OPC_Decode, 159, 14, 206, 1,               // 16102:    decode to M2_mmpyul_rs1 using decoder 206
                                             // 16102:   }
  7, 0,                                      // 16107:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16109:    check Inst[13] == 0x0
  OPC_Decode, 155, 14, 206, 1,               // 16113:    decode to M2_mmpyuh_rs1 using decoder 206
                                             // 16113:   }
                                             // 16113:   } // switch Inst[7:5]
                                             // 16113:  }
  72, 38,                                    // 16118:  case 0x48: {
  OPC_SwitchField, 5, 3,                     // 16120:   switch Inst[7:5] {
  0, 9,                                      // 16123:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16125:    check Inst[13] == 0x0
  OPC_Decode, 200, 15, 208, 1,               // 16129:    decode to M2_vdmpyrs_s0 using decoder 208
                                             // 16129:   }
  1, 9,                                      // 16134:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16136:    check Inst[13] == 0x0
  OPC_Decode, 221, 15, 208, 1,               // 16140:    decode to M2_vradduh using decoder 208
                                             // 16140:   }
  4, 0,                                      // 16145:   case 0x4: {
  OPC_CheckPredicate, 8,                     // 16147:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16149:    check Inst[13] == 0x0
  OPC_Decode, 168, 16, 208, 1,               // 16153:    decode to M7_wcmpyiwc using decoder 208
                                             // 16153:   }
                                             // 16153:   } // switch Inst[7:5]
                                             // 16153:  }
  73, 27,                                    // 16158:  case 0x49: {
  OPC_SwitchField, 5, 3,                     // 16160:   switch Inst[7:5] {
  0, 11,                                     // 16163:   case 0x0: {
  OPC_CheckPredicate, 8,                     // 16165:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16167:    check Inst[13] == 0x0
  OPC_Decode, 166, 16, 208, 1,               // 16171:    decode to M7_wcmpyiw using decoder 208
                                             // 16171:   }
  7, 0,                                      // 16176:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16178:    check Inst[13] == 0x0
  OPC_Decode, 220, 15, 208, 1,               // 16182:    decode to M2_vraddh using decoder 208
                                             // 16182:   }
                                             // 16182:   } // switch Inst[7:5]
                                             // 16182:  }
  74, 15,                                    // 16187:  case 0x4a: {
  OPC_CheckPredicate, 8,                     // 16189:   check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16191:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 16195:   check Inst[7:5] == 0x0
  OPC_Decode, 170, 16, 208, 1,               // 16199:   decode to M7_wcmpyrw using decoder 208
                                             // 16199:  }
  75, 15,                                    // 16204:  case 0x4b: {
  OPC_CheckPredicate, 8,                     // 16206:   check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16208:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 16212:   check Inst[7:5] == 0x0
  OPC_Decode, 172, 16, 208, 1,               // 16216:   decode to M7_wcmpyrwc using decoder 208
                                             // 16216:  }
  76, 27,                                    // 16221:  case 0x4c: {
  OPC_SwitchField, 5, 3,                     // 16223:   switch Inst[7:5] {
  0, 9,                                      // 16226:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16228:    check Inst[13] == 0x0
  OPC_Decode, 201, 15, 208, 1,               // 16232:    decode to M2_vdmpyrs_s1 using decoder 208
                                             // 16232:   }
  4, 0,                                      // 16237:   case 0x4: {
  OPC_CheckPredicate, 8,                     // 16239:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16241:    check Inst[13] == 0x0
  OPC_Decode, 169, 16, 208, 1,               // 16245:    decode to M7_wcmpyiwc_rnd using decoder 208
                                             // 16245:   }
                                             // 16245:   } // switch Inst[7:5]
                                             // 16245:  }
  77, 38,                                    // 16250:  case 0x4d: {
  OPC_SwitchField, 5, 3,                     // 16252:   switch Inst[7:5] {
  0, 11,                                     // 16255:   case 0x0: {
  OPC_CheckPredicate, 8,                     // 16257:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16259:    check Inst[13] == 0x0
  OPC_Decode, 167, 16, 208, 1,               // 16263:    decode to M7_wcmpyiw_rnd using decoder 208
                                             // 16263:   }
  6, 9,                                      // 16268:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 16270:    check Inst[13] == 0x0
  OPC_Decode, 234, 15, 208, 1,               // 16274:    decode to M2_vrcmpys_s1rp_h using decoder 208
                                             // 16274:   }
  7, 0,                                      // 16279:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16281:    check Inst[13] == 0x0
  OPC_Decode, 235, 15, 208, 1,               // 16285:    decode to M2_vrcmpys_s1rp_l using decoder 208
                                             // 16285:   }
                                             // 16285:   } // switch Inst[7:5]
                                             // 16285:  }
  78, 15,                                    // 16290:  case 0x4e: {
  OPC_CheckPredicate, 8,                     // 16292:   check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16294:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 16298:   check Inst[7:5] == 0x0
  OPC_Decode, 171, 16, 208, 1,               // 16302:   decode to M7_wcmpyrw_rnd using decoder 208
                                             // 16302:  }
  79, 15,                                    // 16307:  case 0x4f: {
  OPC_CheckPredicate, 8,                     // 16309:   check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16311:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 16315:   check Inst[7:5] == 0x0
  OPC_Decode, 173, 16, 208, 1,               // 16319:   decode to M7_wcmpyrwc_rnd using decoder 208
                                             // 16319:  }
  80, 93,                                    // 16324:  case 0x50: {
  OPC_SwitchField, 5, 3,                     // 16326:   switch Inst[7:5] {
  0, 9,                                      // 16329:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16331:    check Inst[13] == 0x0
  OPC_Decode, 222, 15, 218, 1,               // 16335:    decode to M2_vrcmaci_s0 using decoder 218
                                             // 16335:   }
  1, 9,                                      // 16340:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16342:    check Inst[13] == 0x0
  OPC_Decode, 224, 15, 218, 1,               // 16346:    decode to M2_vrcmacr_s0 using decoder 218
                                             // 16346:   }
  2, 9,                                      // 16351:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 16353:    check Inst[13] == 0x0
  OPC_Decode, 236, 15, 218, 1,               // 16357:    decode to M2_vrmac_s0 using decoder 218
                                             // 16357:   }
  3, 11,                                     // 16362:   case 0x3: {
  OPC_CheckPredicate, 5,                     // 16364:    check predicate 5
  OPC_CheckField, 13, 1, 0,                  // 16366:    check Inst[13] == 0x0
  OPC_Decode, 161, 10, 218, 1,               // 16370:    decode to F2_dfmpylh using decoder 218
                                             // 16370:   }
  4, 9,                                      // 16375:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 16377:    check Inst[13] == 0x0
  OPC_Decode, 198, 15, 218, 1,               // 16381:    decode to M2_vdmacs_s0 using decoder 218
                                             // 16381:   }
  5, 9,                                      // 16386:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16388:    check Inst[13] == 0x0
  OPC_Decode, 136, 14, 218, 1,               // 16392:    decode to M2_mmacls_s0 using decoder 218
                                             // 16392:   }
  6, 9,                                      // 16397:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 16399:    check Inst[13] == 0x0
  OPC_Decode, 206, 15, 218, 1,               // 16403:    decode to M2_vmac2es_s0 using decoder 218
                                             // 16403:   }
  7, 0,                                      // 16408:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16410:    check Inst[13] == 0x0
  OPC_Decode, 132, 14, 218, 1,               // 16414:    decode to M2_mmachs_s0 using decoder 218
                                             // 16414:   }
                                             // 16414:   } // switch Inst[7:5]
                                             // 16414:  }
  81, 69,                                    // 16419:  case 0x51: {
  OPC_SwitchField, 5, 3,                     // 16421:   switch Inst[7:5] {
  1, 9,                                      // 16424:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16426:    check Inst[13] == 0x0
  OPC_Decode, 146, 16, 218, 1,               // 16430:    decode to M5_vdmacbsu using decoder 218
                                             // 16430:   }
  2, 9,                                      // 16435:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 16437:    check Inst[13] == 0x0
  OPC_Decode, 205, 15, 218, 1,               // 16441:    decode to M2_vmac2es using decoder 218
                                             // 16441:   }
  4, 9,                                      // 16446:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 16448:    check Inst[13] == 0x0
  OPC_Decode, 193, 15, 218, 1,               // 16452:    decode to M2_vcmac_s0_sat_r using decoder 218
                                             // 16452:   }
  5, 9,                                      // 16457:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16459:    check Inst[13] == 0x0
  OPC_Decode, 134, 14, 218, 1,               // 16463:    decode to M2_mmacls_rs0 using decoder 218
                                             // 16463:   }
  6, 9,                                      // 16468:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 16470:    check Inst[13] == 0x0
  OPC_Decode, 134, 16, 218, 1,               // 16474:    decode to M4_vrmpyeh_acc_s0 using decoder 218
                                             // 16474:   }
  7, 0,                                      // 16479:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16481:    check Inst[13] == 0x0
  OPC_Decode, 130, 14, 218, 1,               // 16485:    decode to M2_mmachs_rs0 using decoder 218
                                             // 16485:   }
                                             // 16485:   } // switch Inst[7:5]
                                             // 16485:  }
  82, 82,                                    // 16490:  case 0x52: {
  OPC_SwitchField, 5, 3,                     // 16492:   switch Inst[7:5] {
  0, 9,                                      // 16495:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16497:    check Inst[13] == 0x0
  OPC_Decode, 223, 15, 218, 1,               // 16501:    decode to M2_vrcmaci_s0c using decoder 218
                                             // 16501:   }
  1, 9,                                      // 16506:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16508:    check Inst[13] == 0x0
  OPC_Decode, 198, 8, 218, 1,                // 16512:    decode to A2_vraddub_acc using decoder 218
                                             // 16512:   }
  2, 9,                                      // 16517:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 16519:    check Inst[13] == 0x0
  OPC_Decode, 200, 8, 218, 1,                // 16523:    decode to A2_vrsadub_acc using decoder 218
                                             // 16523:   }
  4, 9,                                      // 16528:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 16530:    check Inst[13] == 0x0
  OPC_Decode, 192, 15, 218, 1,               // 16534:    decode to M2_vcmac_s0_sat_i using decoder 218
                                             // 16534:   }
  5, 9,                                      // 16539:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16541:    check Inst[13] == 0x0
  OPC_Decode, 144, 14, 218, 1,               // 16545:    decode to M2_mmaculs_s0 using decoder 218
                                             // 16545:   }
  6, 11,                                     // 16550:   case 0x6: {
  OPC_CheckPredicate, 8,                     // 16552:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16554:    check Inst[13] == 0x0
  OPC_Decode, 161, 16, 218, 1,               // 16558:    decode to M7_dcmpyiwc_acc using decoder 218
                                             // 16558:   }
  7, 0,                                      // 16563:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16565:    check Inst[13] == 0x0
  OPC_Decode, 140, 14, 218, 1,               // 16569:    decode to M2_mmacuhs_s0 using decoder 218
                                             // 16569:   }
                                             // 16569:   } // switch Inst[7:5]
                                             // 16569:  }
  83, 60,                                    // 16574:  case 0x53: {
  OPC_SwitchField, 5, 3,                     // 16576:   switch Inst[7:5] {
  1, 9,                                      // 16579:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16581:    check Inst[13] == 0x0
  OPC_Decode, 225, 15, 218, 1,               // 16585:    decode to M2_vrcmacr_s0c using decoder 218
                                             // 16585:   }
  2, 11,                                     // 16590:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 16592:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16594:    check Inst[13] == 0x0
  OPC_Decode, 159, 16, 218, 1,               // 16598:    decode to M7_dcmpyiw_acc using decoder 218
                                             // 16598:   }
  5, 9,                                      // 16603:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16605:    check Inst[13] == 0x0
  OPC_Decode, 142, 14, 218, 1,               // 16609:    decode to M2_mmaculs_rs0 using decoder 218
                                             // 16609:   }
  6, 9,                                      // 16614:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 16616:    check Inst[13] == 0x0
  OPC_Decode, 138, 16, 218, 1,               // 16620:    decode to M4_vrmpyoh_acc_s0 using decoder 218
                                             // 16620:   }
  7, 0,                                      // 16625:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16627:    check Inst[13] == 0x0
  OPC_Decode, 138, 14, 218, 1,               // 16631:    decode to M2_mmacuhs_rs0 using decoder 218
                                             // 16631:   }
                                             // 16631:   } // switch Inst[7:5]
                                             // 16631:  }
  84, 84,                                    // 16636:  case 0x54: {
  OPC_SwitchField, 5, 3,                     // 16638:   switch Inst[7:5] {
  1, 9,                                      // 16641:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16643:    check Inst[13] == 0x0
  OPC_Decode, 153, 16, 218, 1,               // 16647:    decode to M5_vrmacbuu using decoder 218
                                             // 16647:   }
  2, 11,                                     // 16652:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 16654:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16656:    check Inst[13] == 0x0
  OPC_Decode, 163, 16, 218, 1,               // 16660:    decode to M7_dcmpyrw_acc using decoder 218
                                             // 16660:   }
  3, 11,                                     // 16665:   case 0x3: {
  OPC_CheckPredicate, 5,                     // 16667:    check predicate 5
  OPC_CheckField, 13, 1, 0,                  // 16669:    check Inst[13] == 0x0
  OPC_Decode, 160, 10, 218, 1,               // 16673:    decode to F2_dfmpyhh using decoder 218
                                             // 16673:   }
  4, 9,                                      // 16678:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 16680:    check Inst[13] == 0x0
  OPC_Decode, 199, 15, 218, 1,               // 16684:    decode to M2_vdmacs_s1 using decoder 218
                                             // 16684:   }
  5, 9,                                      // 16689:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16691:    check Inst[13] == 0x0
  OPC_Decode, 137, 14, 218, 1,               // 16695:    decode to M2_mmacls_s1 using decoder 218
                                             // 16695:   }
  6, 9,                                      // 16700:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 16702:    check Inst[13] == 0x0
  OPC_Decode, 207, 15, 218, 1,               // 16706:    decode to M2_vmac2es_s1 using decoder 218
                                             // 16706:   }
  7, 0,                                      // 16711:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16713:    check Inst[13] == 0x0
  OPC_Decode, 133, 14, 218, 1,               // 16717:    decode to M2_mmachs_s1 using decoder 218
                                             // 16717:   }
                                             // 16717:   } // switch Inst[7:5]
                                             // 16717:  }
  85, 65,                                    // 16722:  case 0x55: {
  OPC_SwitchField, 7, 1,                     // 16724:   switch Inst[7] {
  0, 11,                                     // 16727:   case 0x0: {
  OPC_CheckPredicate, 11,                    // 16729:    check predicate 11
  OPC_CheckField, 13, 1, 0,                  // 16731:    check Inst[13] == 0x0
  OPC_Decode, 166, 9, 252, 1,                // 16735:    decode to A5_ACS using decoder 252
                                             // 16735:   }
  1, 0,                                      // 16740:   case 0x1: {
  OPC_SwitchField, 5, 2,                     // 16742:    switch Inst[6:5] {
  0, 9,                                      // 16745:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16747:     check Inst[13] == 0x0
  OPC_Decode, 230, 15, 218, 1,               // 16751:     decode to M2_vrcmpys_acc_s1_h using decoder 218
                                             // 16751:    }
  1, 9,                                      // 16756:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16758:     check Inst[13] == 0x0
  OPC_Decode, 135, 14, 218, 1,               // 16762:     decode to M2_mmacls_rs1 using decoder 218
                                             // 16762:    }
  2, 9,                                      // 16767:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 16769:     check Inst[13] == 0x0
  OPC_Decode, 135, 16, 218, 1,               // 16773:     decode to M4_vrmpyeh_acc_s1 using decoder 218
                                             // 16773:    }
  3, 0,                                      // 16778:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 16780:     check Inst[13] == 0x0
  OPC_Decode, 131, 14, 218, 1,               // 16784:     decode to M2_mmachs_rs1 using decoder 218
                                             // 16784:    }
                                             // 16784:    } // switch Inst[6:5]
                                             // 16784:   }
                                             // 16784:   } // switch Inst[7]
                                             // 16784:  }
  86, 49,                                    // 16789:  case 0x56: {
  OPC_SwitchField, 5, 3,                     // 16791:   switch Inst[7:5] {
  1, 9,                                      // 16794:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16796:    check Inst[13] == 0x0
  OPC_Decode, 152, 16, 218, 1,               // 16800:    decode to M5_vrmacbsu using decoder 218
                                             // 16800:   }
  2, 11,                                     // 16805:   case 0x2: {
  OPC_CheckPredicate, 8,                     // 16807:    check predicate 8
  OPC_CheckField, 13, 1, 0,                  // 16809:    check Inst[13] == 0x0
  OPC_Decode, 165, 16, 218, 1,               // 16813:    decode to M7_dcmpyrwc_acc using decoder 218
                                             // 16813:   }
  5, 9,                                      // 16818:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 16820:    check Inst[13] == 0x0
  OPC_Decode, 145, 14, 218, 1,               // 16824:    decode to M2_mmaculs_s1 using decoder 218
                                             // 16824:   }
  7, 0,                                      // 16829:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 16831:    check Inst[13] == 0x0
  OPC_Decode, 141, 14, 218, 1,               // 16835:    decode to M2_mmacuhs_s1 using decoder 218
                                             // 16835:   }
                                             // 16835:   } // switch Inst[7:5]
                                             // 16835:  }
  87, 65,                                    // 16840:  case 0x57: {
  OPC_SwitchField, 7, 1,                     // 16842:   switch Inst[7] {
  0, 11,                                     // 16845:   case 0x0: {
  OPC_CheckPredicate, 7,                     // 16847:    check predicate 7
  OPC_CheckField, 13, 1, 0,                  // 16849:    check Inst[13] == 0x0
  OPC_Decode, 169, 9, 253, 1,                // 16853:    decode to A6_vminub_RdP using decoder 253
                                             // 16853:   }
  1, 0,                                      // 16858:   case 0x1: {
  OPC_SwitchField, 5, 2,                     // 16860:    switch Inst[6:5] {
  0, 9,                                      // 16863:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16865:     check Inst[13] == 0x0
  OPC_Decode, 231, 15, 218, 1,               // 16869:     decode to M2_vrcmpys_acc_s1_l using decoder 218
                                             // 16869:    }
  1, 9,                                      // 16874:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16876:     check Inst[13] == 0x0
  OPC_Decode, 143, 14, 218, 1,               // 16880:     decode to M2_mmaculs_rs1 using decoder 218
                                             // 16880:    }
  2, 9,                                      // 16885:    case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 16887:     check Inst[13] == 0x0
  OPC_Decode, 139, 16, 218, 1,               // 16891:     decode to M4_vrmpyoh_acc_s1 using decoder 218
                                             // 16891:    }
  3, 0,                                      // 16896:    case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 16898:     check Inst[13] == 0x0
  OPC_Decode, 139, 14, 218, 1,               // 16902:     decode to M2_mmacuhs_rs1 using decoder 218
                                             // 16902:    }
                                             // 16902:    } // switch Inst[6:5]
                                             // 16902:   }
                                             // 16902:   } // switch Inst[7]
                                             // 16902:  }
  88, 25,                                    // 16907:  case 0x58: {
  OPC_SwitchField, 5, 3,                     // 16909:   switch Inst[7:5] {
  0, 9,                                      // 16912:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16914:    check Inst[13] == 0x0
  OPC_Decode, 164, 10, 134, 1,               // 16918:    decode to F2_sfadd using decoder 134
                                             // 16918:   }
  1, 0,                                      // 16923:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16925:    check Inst[13] == 0x0
  OPC_Decode, 185, 10, 134, 1,               // 16929:    decode to F2_sfsub using decoder 134
                                             // 16929:   }
                                             // 16929:   } // switch Inst[7:5]
                                             // 16929:  }
  90, 13,                                    // 16934:  case 0x5a: {
  OPC_CheckField, 13, 1, 0,                  // 16936:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 16940:   check Inst[7:5] == 0x0
  OPC_Decode, 183, 10, 134, 1,               // 16944:   decode to F2_sfmpy using decoder 134
                                             // 16944:  }
  92, 25,                                    // 16949:  case 0x5c: {
  OPC_SwitchField, 5, 3,                     // 16951:   switch Inst[7:5] {
  0, 9,                                      // 16954:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16956:    check Inst[13] == 0x0
  OPC_Decode, 181, 10, 134, 1,               // 16960:    decode to F2_sfmax using decoder 134
                                             // 16960:   }
  1, 0,                                      // 16965:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16967:    check Inst[13] == 0x0
  OPC_Decode, 182, 10, 134, 1,               // 16971:    decode to F2_sfmin using decoder 134
                                             // 16971:   }
                                             // 16971:   } // switch Inst[7:5]
                                             // 16971:  }
  94, 25,                                    // 16976:  case 0x5e: {
  OPC_SwitchField, 5, 3,                     // 16978:   switch Inst[7:5] {
  0, 9,                                      // 16981:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 16983:    check Inst[13] == 0x0
  OPC_Decode, 171, 10, 134, 1,               // 16987:    decode to F2_sffixupn using decoder 134
                                             // 16987:   }
  1, 0,                                      // 16992:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 16994:    check Inst[13] == 0x0
  OPC_Decode, 170, 10, 134, 1,               // 16998:    decode to F2_sffixupd using decoder 134
                                             // 16998:   }
                                             // 16998:   } // switch Inst[7:5]
                                             // 16998:  }
  95, 13,                                    // 17003:  case 0x5f: {
  OPC_CheckField, 13, 1, 0,                  // 17005:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 17009:   check Inst[7] == 0x1
  OPC_Decode, 184, 10, 254, 1,               // 17013:   decode to F2_sfrecipa using decoder 254
                                             // 17013:  }
  96, 91,                                    // 17018:  case 0x60: {
  OPC_SwitchField, 5, 3,                     // 17020:   switch Inst[7:5] {
  0, 9,                                      // 17023:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17025:    check Inst[13] == 0x0
  OPC_Decode, 185, 14, 134, 1,               // 17029:    decode to M2_mpy_ll_s0 using decoder 134
                                             // 17029:   }
  1, 9,                                      // 17034:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17036:    check Inst[13] == 0x0
  OPC_Decode, 183, 14, 134, 1,               // 17040:    decode to M2_mpy_lh_s0 using decoder 134
                                             // 17040:   }
  2, 9,                                      // 17045:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17047:    check Inst[13] == 0x0
  OPC_Decode, 181, 14, 134, 1,               // 17051:    decode to M2_mpy_hl_s0 using decoder 134
                                             // 17051:   }
  3, 9,                                      // 17056:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17058:    check Inst[13] == 0x0
  OPC_Decode, 179, 14, 134, 1,               // 17062:    decode to M2_mpy_hh_s0 using decoder 134
                                             // 17062:   }
  4, 9,                                      // 17067:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17069:    check Inst[13] == 0x0
  OPC_Decode, 217, 14, 134, 1,               // 17073:    decode to M2_mpy_sat_ll_s0 using decoder 134
                                             // 17073:   }
  5, 9,                                      // 17078:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 17080:    check Inst[13] == 0x0
  OPC_Decode, 215, 14, 134, 1,               // 17084:    decode to M2_mpy_sat_lh_s0 using decoder 134
                                             // 17084:   }
  6, 9,                                      // 17089:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17091:    check Inst[13] == 0x0
  OPC_Decode, 213, 14, 134, 1,               // 17095:    decode to M2_mpy_sat_hl_s0 using decoder 134
                                             // 17095:   }
  7, 0,                                      // 17100:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17102:    check Inst[13] == 0x0
  OPC_Decode, 211, 14, 134, 1,               // 17106:    decode to M2_mpy_sat_hh_s0 using decoder 134
                                             // 17106:   }
                                             // 17106:   } // switch Inst[7:5]
                                             // 17106:  }
  97, 91,                                    // 17111:  case 0x61: {
  OPC_SwitchField, 5, 3,                     // 17113:   switch Inst[7:5] {
  0, 9,                                      // 17116:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17118:    check Inst[13] == 0x0
  OPC_Decode, 209, 14, 134, 1,               // 17122:    decode to M2_mpy_rnd_ll_s0 using decoder 134
                                             // 17122:   }
  1, 9,                                      // 17127:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17129:    check Inst[13] == 0x0
  OPC_Decode, 207, 14, 134, 1,               // 17133:    decode to M2_mpy_rnd_lh_s0 using decoder 134
                                             // 17133:   }
  2, 9,                                      // 17138:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17140:    check Inst[13] == 0x0
  OPC_Decode, 205, 14, 134, 1,               // 17144:    decode to M2_mpy_rnd_hl_s0 using decoder 134
                                             // 17144:   }
  3, 9,                                      // 17149:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17151:    check Inst[13] == 0x0
  OPC_Decode, 203, 14, 134, 1,               // 17155:    decode to M2_mpy_rnd_hh_s0 using decoder 134
                                             // 17155:   }
  4, 9,                                      // 17160:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17162:    check Inst[13] == 0x0
  OPC_Decode, 225, 14, 134, 1,               // 17166:    decode to M2_mpy_sat_rnd_ll_s0 using decoder 134
                                             // 17166:   }
  5, 9,                                      // 17171:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 17173:    check Inst[13] == 0x0
  OPC_Decode, 223, 14, 134, 1,               // 17177:    decode to M2_mpy_sat_rnd_lh_s0 using decoder 134
                                             // 17177:   }
  6, 9,                                      // 17182:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17184:    check Inst[13] == 0x0
  OPC_Decode, 221, 14, 134, 1,               // 17188:    decode to M2_mpy_sat_rnd_hl_s0 using decoder 134
                                             // 17188:   }
  7, 0,                                      // 17193:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17195:    check Inst[13] == 0x0
  OPC_Decode, 219, 14, 134, 1,               // 17199:    decode to M2_mpy_sat_rnd_hh_s0 using decoder 134
                                             // 17199:   }
                                             // 17199:   } // switch Inst[7:5]
                                             // 17199:  }
  98, 47,                                    // 17204:  case 0x62: {
  OPC_SwitchField, 5, 3,                     // 17206:   switch Inst[7:5] {
  0, 9,                                      // 17209:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17211:    check Inst[13] == 0x0
  OPC_Decode, 152, 15, 134, 1,               // 17215:    decode to M2_mpyu_ll_s0 using decoder 134
                                             // 17215:   }
  1, 9,                                      // 17220:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17222:    check Inst[13] == 0x0
  OPC_Decode, 150, 15, 134, 1,               // 17226:    decode to M2_mpyu_lh_s0 using decoder 134
                                             // 17226:   }
  2, 9,                                      // 17231:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17233:    check Inst[13] == 0x0
  OPC_Decode, 148, 15, 134, 1,               // 17237:    decode to M2_mpyu_hl_s0 using decoder 134
                                             // 17237:   }
  3, 0,                                      // 17242:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17244:    check Inst[13] == 0x0
  OPC_Decode, 146, 15, 134, 1,               // 17248:    decode to M2_mpyu_hh_s0 using decoder 134
                                             // 17248:   }
                                             // 17248:   } // switch Inst[7:5]
                                             // 17248:  }
  100, 91,                                   // 17253:  case 0x64: {
  OPC_SwitchField, 5, 3,                     // 17255:   switch Inst[7:5] {
  0, 9,                                      // 17258:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17260:    check Inst[13] == 0x0
  OPC_Decode, 186, 14, 134, 1,               // 17264:    decode to M2_mpy_ll_s1 using decoder 134
                                             // 17264:   }
  1, 9,                                      // 17269:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17271:    check Inst[13] == 0x0
  OPC_Decode, 184, 14, 134, 1,               // 17275:    decode to M2_mpy_lh_s1 using decoder 134
                                             // 17275:   }
  2, 9,                                      // 17280:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17282:    check Inst[13] == 0x0
  OPC_Decode, 182, 14, 134, 1,               // 17286:    decode to M2_mpy_hl_s1 using decoder 134
                                             // 17286:   }
  3, 9,                                      // 17291:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17293:    check Inst[13] == 0x0
  OPC_Decode, 180, 14, 134, 1,               // 17297:    decode to M2_mpy_hh_s1 using decoder 134
                                             // 17297:   }
  4, 9,                                      // 17302:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17304:    check Inst[13] == 0x0
  OPC_Decode, 218, 14, 134, 1,               // 17308:    decode to M2_mpy_sat_ll_s1 using decoder 134
                                             // 17308:   }
  5, 9,                                      // 17313:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 17315:    check Inst[13] == 0x0
  OPC_Decode, 216, 14, 134, 1,               // 17319:    decode to M2_mpy_sat_lh_s1 using decoder 134
                                             // 17319:   }
  6, 9,                                      // 17324:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17326:    check Inst[13] == 0x0
  OPC_Decode, 214, 14, 134, 1,               // 17330:    decode to M2_mpy_sat_hl_s1 using decoder 134
                                             // 17330:   }
  7, 0,                                      // 17335:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17337:    check Inst[13] == 0x0
  OPC_Decode, 212, 14, 134, 1,               // 17341:    decode to M2_mpy_sat_hh_s1 using decoder 134
                                             // 17341:   }
                                             // 17341:   } // switch Inst[7:5]
                                             // 17341:  }
  101, 91,                                   // 17346:  case 0x65: {
  OPC_SwitchField, 5, 3,                     // 17348:   switch Inst[7:5] {
  0, 9,                                      // 17351:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17353:    check Inst[13] == 0x0
  OPC_Decode, 210, 14, 134, 1,               // 17357:    decode to M2_mpy_rnd_ll_s1 using decoder 134
                                             // 17357:   }
  1, 9,                                      // 17362:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17364:    check Inst[13] == 0x0
  OPC_Decode, 208, 14, 134, 1,               // 17368:    decode to M2_mpy_rnd_lh_s1 using decoder 134
                                             // 17368:   }
  2, 9,                                      // 17373:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17375:    check Inst[13] == 0x0
  OPC_Decode, 206, 14, 134, 1,               // 17379:    decode to M2_mpy_rnd_hl_s1 using decoder 134
                                             // 17379:   }
  3, 9,                                      // 17384:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17386:    check Inst[13] == 0x0
  OPC_Decode, 204, 14, 134, 1,               // 17390:    decode to M2_mpy_rnd_hh_s1 using decoder 134
                                             // 17390:   }
  4, 9,                                      // 17395:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17397:    check Inst[13] == 0x0
  OPC_Decode, 226, 14, 134, 1,               // 17401:    decode to M2_mpy_sat_rnd_ll_s1 using decoder 134
                                             // 17401:   }
  5, 9,                                      // 17406:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 17408:    check Inst[13] == 0x0
  OPC_Decode, 224, 14, 134, 1,               // 17412:    decode to M2_mpy_sat_rnd_lh_s1 using decoder 134
                                             // 17412:   }
  6, 9,                                      // 17417:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17419:    check Inst[13] == 0x0
  OPC_Decode, 222, 14, 134, 1,               // 17423:    decode to M2_mpy_sat_rnd_hl_s1 using decoder 134
                                             // 17423:   }
  7, 0,                                      // 17428:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17430:    check Inst[13] == 0x0
  OPC_Decode, 220, 14, 134, 1,               // 17434:    decode to M2_mpy_sat_rnd_hh_s1 using decoder 134
                                             // 17434:   }
                                             // 17434:   } // switch Inst[7:5]
                                             // 17434:  }
  102, 47,                                   // 17439:  case 0x66: {
  OPC_SwitchField, 5, 3,                     // 17441:   switch Inst[7:5] {
  0, 9,                                      // 17444:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17446:    check Inst[13] == 0x0
  OPC_Decode, 153, 15, 134, 1,               // 17450:    decode to M2_mpyu_ll_s1 using decoder 134
                                             // 17450:   }
  1, 9,                                      // 17455:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17457:    check Inst[13] == 0x0
  OPC_Decode, 151, 15, 134, 1,               // 17461:    decode to M2_mpyu_lh_s1 using decoder 134
                                             // 17461:   }
  2, 9,                                      // 17466:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17468:    check Inst[13] == 0x0
  OPC_Decode, 149, 15, 134, 1,               // 17472:    decode to M2_mpyu_hl_s1 using decoder 134
                                             // 17472:   }
  3, 0,                                      // 17477:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17479:    check Inst[13] == 0x0
  OPC_Decode, 147, 15, 134, 1,               // 17483:    decode to M2_mpyu_hh_s1 using decoder 134
                                             // 17483:   }
                                             // 17483:   } // switch Inst[7:5]
                                             // 17483:  }
  104, 25,                                   // 17488:  case 0x68: {
  OPC_SwitchField, 5, 3,                     // 17490:   switch Inst[7:5] {
  0, 9,                                      // 17493:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17495:    check Inst[13] == 0x0
  OPC_Decode, 134, 15, 134, 1,               // 17499:    decode to M2_mpyi using decoder 134
                                             // 17499:   }
  1, 0,                                      // 17504:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17506:    check Inst[13] == 0x0
  OPC_Decode, 227, 14, 134, 1,               // 17510:    decode to M2_mpy_up using decoder 134
                                             // 17510:   }
                                             // 17510:   } // switch Inst[7:5]
                                             // 17510:  }
  105, 36,                                   // 17515:  case 0x69: {
  OPC_SwitchField, 5, 3,                     // 17517:   switch Inst[7:5] {
  1, 9,                                      // 17520:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17522:    check Inst[13] == 0x0
  OPC_Decode, 246, 13, 134, 1,               // 17526:    decode to M2_dpmpyss_rnd_s0 using decoder 134
                                             // 17526:   }
  6, 9,                                      // 17531:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17533:    check Inst[13] == 0x0
  OPC_Decode, 232, 13, 134, 1,               // 17537:    decode to M2_cmpyrs_s0 using decoder 134
                                             // 17537:   }
  7, 0,                                      // 17542:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17544:    check Inst[13] == 0x0
  OPC_Decode, 215, 15, 134, 1,               // 17548:    decode to M2_vmpy2s_s0pack using decoder 134
                                             // 17548:   }
                                             // 17548:   } // switch Inst[7:5]
                                             // 17548:  }
  106, 13,                                   // 17553:  case 0x6a: {
  OPC_CheckField, 13, 1, 0,                  // 17555:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 1,                   // 17559:   check Inst[7:5] == 0x1
  OPC_Decode, 162, 15, 134, 1,               // 17563:   decode to M2_mpyu_up using decoder 134
                                             // 17563:  }
  107, 25,                                   // 17568:  case 0x6b: {
  OPC_SwitchField, 5, 3,                     // 17570:   switch Inst[7:5] {
  1, 9,                                      // 17573:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17575:    check Inst[13] == 0x0
  OPC_Decode, 137, 15, 134, 1,               // 17579:    decode to M2_mpysu_up using decoder 134
                                             // 17579:   }
  6, 0,                                      // 17584:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17586:    check Inst[13] == 0x0
  OPC_Decode, 234, 13, 134, 1,               // 17590:    decode to M2_cmpyrsc_s0 using decoder 134
                                             // 17590:   }
                                             // 17590:   } // switch Inst[7:5]
                                             // 17590:  }
  109, 69,                                   // 17595:  case 0x6d: {
  OPC_SwitchField, 5, 3,                     // 17597:   switch Inst[7:5] {
  0, 9,                                      // 17600:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17602:    check Inst[13] == 0x0
  OPC_Decode, 252, 13, 134, 1,               // 17606:    decode to M2_hmmpyh_s1 using decoder 134
                                             // 17606:   }
  1, 9,                                      // 17611:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17613:    check Inst[13] == 0x0
  OPC_Decode, 254, 13, 134, 1,               // 17617:    decode to M2_hmmpyl_s1 using decoder 134
                                             // 17617:   }
  2, 9,                                      // 17622:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17624:    check Inst[13] == 0x0
  OPC_Decode, 228, 14, 134, 1,               // 17628:    decode to M2_mpy_up_s1 using decoder 134
                                             // 17628:   }
  4, 9,                                      // 17633:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17635:    check Inst[13] == 0x0
  OPC_Decode, 251, 13, 134, 1,               // 17639:    decode to M2_hmmpyh_rs1 using decoder 134
                                             // 17639:   }
  6, 9,                                      // 17644:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17646:    check Inst[13] == 0x0
  OPC_Decode, 233, 13, 134, 1,               // 17650:    decode to M2_cmpyrs_s1 using decoder 134
                                             // 17650:   }
  7, 0,                                      // 17655:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17657:    check Inst[13] == 0x0
  OPC_Decode, 217, 15, 134, 1,               // 17661:    decode to M2_vmpy2s_s1pack using decoder 134
                                             // 17661:   }
                                             // 17661:   } // switch Inst[7:5]
                                             // 17661:  }
  111, 36,                                   // 17666:  case 0x6f: {
  OPC_SwitchField, 5, 3,                     // 17668:   switch Inst[7:5] {
  0, 9,                                      // 17671:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17673:    check Inst[13] == 0x0
  OPC_Decode, 229, 14, 134, 1,               // 17677:    decode to M2_mpy_up_s1_sat using decoder 134
                                             // 17677:   }
  4, 9,                                      // 17682:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17684:    check Inst[13] == 0x0
  OPC_Decode, 253, 13, 134, 1,               // 17688:    decode to M2_hmmpyl_rs1 using decoder 134
                                             // 17688:   }
  6, 0,                                      // 17693:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17695:    check Inst[13] == 0x0
  OPC_Decode, 235, 13, 134, 1,               // 17699:    decode to M2_cmpyrsc_s1 using decoder 134
                                             // 17699:   }
                                             // 17699:   } // switch Inst[7:5]
                                             // 17699:  }
  112, 91,                                   // 17704:  case 0x70: {
  OPC_SwitchField, 5, 3,                     // 17706:   switch Inst[7:5] {
  0, 9,                                      // 17709:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17711:    check Inst[13] == 0x0
  OPC_Decode, 169, 14, 222, 1,               // 17715:    decode to M2_mpy_acc_ll_s0 using decoder 222
                                             // 17715:   }
  1, 9,                                      // 17720:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17722:    check Inst[13] == 0x0
  OPC_Decode, 167, 14, 222, 1,               // 17726:    decode to M2_mpy_acc_lh_s0 using decoder 222
                                             // 17726:   }
  2, 9,                                      // 17731:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17733:    check Inst[13] == 0x0
  OPC_Decode, 165, 14, 222, 1,               // 17737:    decode to M2_mpy_acc_hl_s0 using decoder 222
                                             // 17737:   }
  3, 9,                                      // 17742:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17744:    check Inst[13] == 0x0
  OPC_Decode, 163, 14, 222, 1,               // 17748:    decode to M2_mpy_acc_hh_s0 using decoder 222
                                             // 17748:   }
  4, 9,                                      // 17753:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17755:    check Inst[13] == 0x0
  OPC_Decode, 177, 14, 222, 1,               // 17759:    decode to M2_mpy_acc_sat_ll_s0 using decoder 222
                                             // 17759:   }
  5, 9,                                      // 17764:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 17766:    check Inst[13] == 0x0
  OPC_Decode, 175, 14, 222, 1,               // 17770:    decode to M2_mpy_acc_sat_lh_s0 using decoder 222
                                             // 17770:   }
  6, 9,                                      // 17775:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17777:    check Inst[13] == 0x0
  OPC_Decode, 173, 14, 222, 1,               // 17781:    decode to M2_mpy_acc_sat_hl_s0 using decoder 222
                                             // 17781:   }
  7, 0,                                      // 17786:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17788:    check Inst[13] == 0x0
  OPC_Decode, 171, 14, 222, 1,               // 17792:    decode to M2_mpy_acc_sat_hh_s0 using decoder 222
                                             // 17792:   }
                                             // 17792:   } // switch Inst[7:5]
                                             // 17792:  }
  113, 91,                                   // 17797:  case 0x71: {
  OPC_SwitchField, 5, 3,                     // 17799:   switch Inst[7:5] {
  0, 9,                                      // 17802:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17804:    check Inst[13] == 0x0
  OPC_Decode, 193, 14, 222, 1,               // 17808:    decode to M2_mpy_nac_ll_s0 using decoder 222
                                             // 17808:   }
  1, 9,                                      // 17813:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17815:    check Inst[13] == 0x0
  OPC_Decode, 191, 14, 222, 1,               // 17819:    decode to M2_mpy_nac_lh_s0 using decoder 222
                                             // 17819:   }
  2, 9,                                      // 17824:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17826:    check Inst[13] == 0x0
  OPC_Decode, 189, 14, 222, 1,               // 17830:    decode to M2_mpy_nac_hl_s0 using decoder 222
                                             // 17830:   }
  3, 9,                                      // 17835:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17837:    check Inst[13] == 0x0
  OPC_Decode, 187, 14, 222, 1,               // 17841:    decode to M2_mpy_nac_hh_s0 using decoder 222
                                             // 17841:   }
  4, 9,                                      // 17846:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 17848:    check Inst[13] == 0x0
  OPC_Decode, 201, 14, 222, 1,               // 17852:    decode to M2_mpy_nac_sat_ll_s0 using decoder 222
                                             // 17852:   }
  5, 9,                                      // 17857:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 17859:    check Inst[13] == 0x0
  OPC_Decode, 199, 14, 222, 1,               // 17863:    decode to M2_mpy_nac_sat_lh_s0 using decoder 222
                                             // 17863:   }
  6, 9,                                      // 17868:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 17870:    check Inst[13] == 0x0
  OPC_Decode, 197, 14, 222, 1,               // 17874:    decode to M2_mpy_nac_sat_hl_s0 using decoder 222
                                             // 17874:   }
  7, 0,                                      // 17879:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 17881:    check Inst[13] == 0x0
  OPC_Decode, 195, 14, 222, 1,               // 17885:    decode to M2_mpy_nac_sat_hh_s0 using decoder 222
                                             // 17885:   }
                                             // 17885:   } // switch Inst[7:5]
                                             // 17885:  }
  114, 47,                                   // 17890:  case 0x72: {
  OPC_SwitchField, 5, 3,                     // 17892:   switch Inst[7:5] {
  0, 9,                                      // 17895:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17897:    check Inst[13] == 0x0
  OPC_Decode, 144, 15, 222, 1,               // 17901:    decode to M2_mpyu_acc_ll_s0 using decoder 222
                                             // 17901:   }
  1, 9,                                      // 17906:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17908:    check Inst[13] == 0x0
  OPC_Decode, 142, 15, 222, 1,               // 17912:    decode to M2_mpyu_acc_lh_s0 using decoder 222
                                             // 17912:   }
  2, 9,                                      // 17917:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17919:    check Inst[13] == 0x0
  OPC_Decode, 140, 15, 222, 1,               // 17923:    decode to M2_mpyu_acc_hl_s0 using decoder 222
                                             // 17923:   }
  3, 0,                                      // 17928:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17930:    check Inst[13] == 0x0
  OPC_Decode, 138, 15, 222, 1,               // 17934:    decode to M2_mpyu_acc_hh_s0 using decoder 222
                                             // 17934:   }
                                             // 17934:   } // switch Inst[7:5]
                                             // 17934:  }
  115, 47,                                   // 17939:  case 0x73: {
  OPC_SwitchField, 5, 3,                     // 17941:   switch Inst[7:5] {
  0, 9,                                      // 17944:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17946:    check Inst[13] == 0x0
  OPC_Decode, 160, 15, 222, 1,               // 17950:    decode to M2_mpyu_nac_ll_s0 using decoder 222
                                             // 17950:   }
  1, 9,                                      // 17955:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 17957:    check Inst[13] == 0x0
  OPC_Decode, 158, 15, 222, 1,               // 17961:    decode to M2_mpyu_nac_lh_s0 using decoder 222
                                             // 17961:   }
  2, 9,                                      // 17966:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 17968:    check Inst[13] == 0x0
  OPC_Decode, 156, 15, 222, 1,               // 17972:    decode to M2_mpyu_nac_hl_s0 using decoder 222
                                             // 17972:   }
  3, 0,                                      // 17977:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 17979:    check Inst[13] == 0x0
  OPC_Decode, 154, 15, 222, 1,               // 17983:    decode to M2_mpyu_nac_hh_s0 using decoder 222
                                             // 17983:   }
                                             // 17983:   } // switch Inst[7:5]
                                             // 17983:  }
  116, 91,                                   // 17988:  case 0x74: {
  OPC_SwitchField, 5, 3,                     // 17990:   switch Inst[7:5] {
  0, 9,                                      // 17993:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 17995:    check Inst[13] == 0x0
  OPC_Decode, 170, 14, 222, 1,               // 17999:    decode to M2_mpy_acc_ll_s1 using decoder 222
                                             // 17999:   }
  1, 9,                                      // 18004:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18006:    check Inst[13] == 0x0
  OPC_Decode, 168, 14, 222, 1,               // 18010:    decode to M2_mpy_acc_lh_s1 using decoder 222
                                             // 18010:   }
  2, 9,                                      // 18015:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18017:    check Inst[13] == 0x0
  OPC_Decode, 166, 14, 222, 1,               // 18021:    decode to M2_mpy_acc_hl_s1 using decoder 222
                                             // 18021:   }
  3, 9,                                      // 18026:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18028:    check Inst[13] == 0x0
  OPC_Decode, 164, 14, 222, 1,               // 18032:    decode to M2_mpy_acc_hh_s1 using decoder 222
                                             // 18032:   }
  4, 9,                                      // 18037:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 18039:    check Inst[13] == 0x0
  OPC_Decode, 178, 14, 222, 1,               // 18043:    decode to M2_mpy_acc_sat_ll_s1 using decoder 222
                                             // 18043:   }
  5, 9,                                      // 18048:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 18050:    check Inst[13] == 0x0
  OPC_Decode, 176, 14, 222, 1,               // 18054:    decode to M2_mpy_acc_sat_lh_s1 using decoder 222
                                             // 18054:   }
  6, 9,                                      // 18059:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 18061:    check Inst[13] == 0x0
  OPC_Decode, 174, 14, 222, 1,               // 18065:    decode to M2_mpy_acc_sat_hl_s1 using decoder 222
                                             // 18065:   }
  7, 0,                                      // 18070:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 18072:    check Inst[13] == 0x0
  OPC_Decode, 172, 14, 222, 1,               // 18076:    decode to M2_mpy_acc_sat_hh_s1 using decoder 222
                                             // 18076:   }
                                             // 18076:   } // switch Inst[7:5]
                                             // 18076:  }
  117, 91,                                   // 18081:  case 0x75: {
  OPC_SwitchField, 5, 3,                     // 18083:   switch Inst[7:5] {
  0, 9,                                      // 18086:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18088:    check Inst[13] == 0x0
  OPC_Decode, 194, 14, 222, 1,               // 18092:    decode to M2_mpy_nac_ll_s1 using decoder 222
                                             // 18092:   }
  1, 9,                                      // 18097:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18099:    check Inst[13] == 0x0
  OPC_Decode, 192, 14, 222, 1,               // 18103:    decode to M2_mpy_nac_lh_s1 using decoder 222
                                             // 18103:   }
  2, 9,                                      // 18108:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18110:    check Inst[13] == 0x0
  OPC_Decode, 190, 14, 222, 1,               // 18114:    decode to M2_mpy_nac_hl_s1 using decoder 222
                                             // 18114:   }
  3, 9,                                      // 18119:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18121:    check Inst[13] == 0x0
  OPC_Decode, 188, 14, 222, 1,               // 18125:    decode to M2_mpy_nac_hh_s1 using decoder 222
                                             // 18125:   }
  4, 9,                                      // 18130:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 18132:    check Inst[13] == 0x0
  OPC_Decode, 202, 14, 222, 1,               // 18136:    decode to M2_mpy_nac_sat_ll_s1 using decoder 222
                                             // 18136:   }
  5, 9,                                      // 18141:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 18143:    check Inst[13] == 0x0
  OPC_Decode, 200, 14, 222, 1,               // 18147:    decode to M2_mpy_nac_sat_lh_s1 using decoder 222
                                             // 18147:   }
  6, 9,                                      // 18152:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 18154:    check Inst[13] == 0x0
  OPC_Decode, 198, 14, 222, 1,               // 18158:    decode to M2_mpy_nac_sat_hl_s1 using decoder 222
                                             // 18158:   }
  7, 0,                                      // 18163:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 18165:    check Inst[13] == 0x0
  OPC_Decode, 196, 14, 222, 1,               // 18169:    decode to M2_mpy_nac_sat_hh_s1 using decoder 222
                                             // 18169:   }
                                             // 18169:   } // switch Inst[7:5]
                                             // 18169:  }
  118, 47,                                   // 18174:  case 0x76: {
  OPC_SwitchField, 5, 3,                     // 18176:   switch Inst[7:5] {
  0, 9,                                      // 18179:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18181:    check Inst[13] == 0x0
  OPC_Decode, 145, 15, 222, 1,               // 18185:    decode to M2_mpyu_acc_ll_s1 using decoder 222
                                             // 18185:   }
  1, 9,                                      // 18190:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18192:    check Inst[13] == 0x0
  OPC_Decode, 143, 15, 222, 1,               // 18196:    decode to M2_mpyu_acc_lh_s1 using decoder 222
                                             // 18196:   }
  2, 9,                                      // 18201:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18203:    check Inst[13] == 0x0
  OPC_Decode, 141, 15, 222, 1,               // 18207:    decode to M2_mpyu_acc_hl_s1 using decoder 222
                                             // 18207:   }
  3, 0,                                      // 18212:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18214:    check Inst[13] == 0x0
  OPC_Decode, 139, 15, 222, 1,               // 18218:    decode to M2_mpyu_acc_hh_s1 using decoder 222
                                             // 18218:   }
                                             // 18218:   } // switch Inst[7:5]
                                             // 18218:  }
  119, 47,                                   // 18223:  case 0x77: {
  OPC_SwitchField, 5, 3,                     // 18225:   switch Inst[7:5] {
  0, 9,                                      // 18228:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18230:    check Inst[13] == 0x0
  OPC_Decode, 161, 15, 222, 1,               // 18234:    decode to M2_mpyu_nac_ll_s1 using decoder 222
                                             // 18234:   }
  1, 9,                                      // 18239:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18241:    check Inst[13] == 0x0
  OPC_Decode, 159, 15, 222, 1,               // 18245:    decode to M2_mpyu_nac_lh_s1 using decoder 222
                                             // 18245:   }
  2, 9,                                      // 18250:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18252:    check Inst[13] == 0x0
  OPC_Decode, 157, 15, 222, 1,               // 18256:    decode to M2_mpyu_nac_hl_s1 using decoder 222
                                             // 18256:   }
  3, 0,                                      // 18261:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18263:    check Inst[13] == 0x0
  OPC_Decode, 155, 15, 222, 1,               // 18267:    decode to M2_mpyu_nac_hh_s1 using decoder 222
                                             // 18267:   }
                                             // 18267:   } // switch Inst[7:5]
                                             // 18267:  }
  120, 80,                                   // 18272:  case 0x78: {
  OPC_SwitchField, 5, 3,                     // 18274:   switch Inst[7:5] {
  0, 9,                                      // 18277:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18279:    check Inst[13] == 0x0
  OPC_Decode, 255, 13, 222, 1,               // 18283:    decode to M2_maci using decoder 222
                                             // 18283:   }
  1, 9,                                      // 18288:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18290:    check Inst[13] == 0x0
  OPC_Decode, 222, 13, 222, 1,               // 18294:    decode to M2_acci using decoder 222
                                             // 18294:   }
  3, 9,                                      // 18299:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18301:    check Inst[13] == 0x0
  OPC_Decode, 189, 15, 255, 1,               // 18305:    decode to M2_subacc using decoder 255
                                             // 18305:   }
  4, 9,                                      // 18310:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 18312:    check Inst[13] == 0x0
  OPC_Decode, 173, 10, 222, 1,               // 18316:    decode to F2_sffma using decoder 222
                                             // 18316:   }
  5, 9,                                      // 18321:   case 0x5: {
  OPC_CheckField, 13, 1, 0,                  // 18323:    check Inst[13] == 0x0
  OPC_Decode, 176, 10, 222, 1,               // 18327:    decode to F2_sffms using decoder 222
                                             // 18327:   }
  6, 9,                                      // 18332:   case 0x6: {
  OPC_CheckField, 13, 1, 0,                  // 18334:    check Inst[13] == 0x0
  OPC_Decode, 174, 10, 222, 1,               // 18338:    decode to F2_sffma_lib using decoder 222
                                             // 18338:   }
  7, 0,                                      // 18343:   case 0x7: {
  OPC_CheckField, 13, 1, 0,                  // 18345:    check Inst[13] == 0x0
  OPC_Decode, 177, 10, 222, 1,               // 18349:    decode to F2_sffms_lib using decoder 222
                                             // 18349:   }
                                             // 18349:   } // switch Inst[7:5]
                                             // 18349:  }
  121, 36,                                   // 18354:  case 0x79: {
  OPC_SwitchField, 5, 3,                     // 18356:   switch Inst[7:5] {
  0, 9,                                      // 18359:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18361:    check Inst[13] == 0x0
  OPC_Decode, 255, 15, 222, 1,               // 18365:    decode to M4_or_andn using decoder 222
                                             // 18365:   }
  1, 9,                                      // 18370:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18372:    check Inst[13] == 0x0
  OPC_Decode, 240, 15, 222, 1,               // 18376:    decode to M4_and_andn using decoder 222
                                             // 18376:   }
  2, 0,                                      // 18381:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18383:    check Inst[13] == 0x0
  OPC_Decode, 143, 16, 222, 1,               // 18387:    decode to M4_xor_andn using decoder 222
                                             // 18387:   }
                                             // 18387:   } // switch Inst[7:5]
                                             // 18387:  }
  122, 47,                                   // 18392:  case 0x7a: {
  OPC_SwitchField, 5, 3,                     // 18394:   switch Inst[7:5] {
  0, 9,                                      // 18397:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18399:    check Inst[13] == 0x0
  OPC_Decode, 239, 15, 222, 1,               // 18403:    decode to M4_and_and using decoder 222
                                             // 18403:   }
  1, 9,                                      // 18408:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18410:    check Inst[13] == 0x0
  OPC_Decode, 241, 15, 222, 1,               // 18414:    decode to M4_and_or using decoder 222
                                             // 18414:   }
  2, 9,                                      // 18419:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18421:    check Inst[13] == 0x0
  OPC_Decode, 242, 15, 222, 1,               // 18425:    decode to M4_and_xor using decoder 222
                                             // 18425:   }
  3, 0,                                      // 18430:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18432:    check Inst[13] == 0x0
  OPC_Decode, 254, 15, 222, 1,               // 18436:    decode to M4_or_and using decoder 222
                                             // 18436:   }
                                             // 18436:   } // switch Inst[7:5]
                                             // 18436:  }
  123, 41,                                   // 18441:  case 0x7b: {
  OPC_SwitchField, 7, 1,                     // 18443:   switch Inst[7] {
  0, 25,                                     // 18446:   case 0x0: {
  OPC_SwitchField, 5, 2,                     // 18448:    switch Inst[6:5] {
  0, 9,                                      // 18451:    case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18453:     check Inst[13] == 0x0
  OPC_Decode, 247, 15, 222, 1,               // 18457:     decode to M4_mac_up_s1_sat using decoder 222
                                             // 18457:    }
  1, 0,                                      // 18462:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18464:     check Inst[13] == 0x0
  OPC_Decode, 253, 15, 222, 1,               // 18468:     decode to M4_nac_up_s1_sat using decoder 222
                                             // 18468:    }
                                             // 18468:    } // switch Inst[6:5]
                                             // 18468:   }
  1, 0,                                      // 18473:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18475:    check Inst[13] == 0x0
  OPC_Decode, 175, 10, 128, 2,               // 18479:    decode to F2_sffma_sc using decoder 256
                                             // 18479:   }
                                             // 18479:   } // switch Inst[7]
                                             // 18479:  }
  124, 38,                                   // 18484:  case 0x7c: {
  OPC_SwitchField, 5, 3,                     // 18486:   switch Inst[7:5] {
  0, 11,                                     // 18489:   case 0x0: {
  OPC_CheckPredicate, 4,                     // 18491:    check predicate 4
  OPC_CheckField, 13, 1, 0,                  // 18493:    check Inst[13] == 0x0
  OPC_Decode, 162, 14, 222, 1,               // 18497:    decode to M2_mnaci using decoder 222
                                             // 18497:   }
  1, 9,                                      // 18502:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18504:    check Inst[13] == 0x0
  OPC_Decode, 187, 15, 222, 1,               // 18508:    decode to M2_nacci using decoder 222
                                             // 18508:   }
  3, 0,                                      // 18513:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18515:    check Inst[13] == 0x0
  OPC_Decode, 238, 15, 222, 1,               // 18519:    decode to M2_xor_xacc using decoder 222
                                             // 18519:   }
                                             // 18519:   } // switch Inst[7:5]
                                             // 18519:  }
  126, 0,                                    // 18524:  case 0x7e: {
  OPC_SwitchField, 5, 3,                     // 18526:   switch Inst[7:5] {
  0, 9,                                      // 18529:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18531:    check Inst[13] == 0x0
  OPC_Decode, 128, 16, 222, 1,               // 18535:    decode to M4_or_or using decoder 222
                                             // 18535:   }
  1, 9,                                      // 18540:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 18542:    check Inst[13] == 0x0
  OPC_Decode, 129, 16, 222, 1,               // 18546:    decode to M4_or_xor using decoder 222
                                             // 18546:   }
  2, 9,                                      // 18551:   case 0x2: {
  OPC_CheckField, 13, 1, 0,                  // 18553:    check Inst[13] == 0x0
  OPC_Decode, 142, 16, 222, 1,               // 18557:    decode to M4_xor_and using decoder 222
                                             // 18557:   }
  3, 0,                                      // 18562:   case 0x3: {
  OPC_CheckField, 13, 1, 0,                  // 18564:    check Inst[13] == 0x0
  OPC_Decode, 144, 16, 222, 1,               // 18568:    decode to M4_xor_or using decoder 222
                                             // 18568:   }
                                             // 18568:   } // switch Inst[7:5]
                                             // 18568:  }
                                             // 18568:  } // switch Inst[27:21]
                                             // 18568: }
  15, 0,                                     // 18573: case 0xf: {
  OPC_SwitchField, 21, 7,                    // 18575:  switch Inst[27:21] {
  8, 13,                                     // 18578:  case 0x8: {
  OPC_CheckField, 13, 1, 0,                  // 18580:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18584:   check Inst[7:5] == 0x0
  OPC_Decode, 184, 7, 134, 1,                // 18588:   decode to A2_and using decoder 134
                                             // 18588:  }
  9, 13,                                     // 18593:  case 0x9: {
  OPC_CheckField, 13, 1, 0,                  // 18595:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18599:   check Inst[7:5] == 0x0
  OPC_Decode, 207, 7, 134, 1,                // 18603:   decode to A2_or using decoder 134
                                             // 18603:  }
  11, 13,                                    // 18608:  case 0xb: {
  OPC_CheckField, 13, 1, 0,                  // 18610:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18614:   check Inst[7:5] == 0x0
  OPC_Decode, 208, 8, 134, 1,                // 18618:   decode to A2_xor using decoder 134
                                             // 18618:  }
  12, 13,                                    // 18623:  case 0xc: {
  OPC_CheckField, 13, 1, 0,                  // 18625:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18629:   check Inst[7:5] == 0x0
  OPC_Decode, 212, 8, 227, 1,                // 18633:   decode to A4_andn using decoder 227
                                             // 18633:  }
  13, 13,                                    // 18638:  case 0xd: {
  OPC_CheckField, 13, 1, 0,                  // 18640:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18644:   check Inst[7:5] == 0x0
  OPC_Decode, 237, 8, 227, 1,                // 18648:   decode to A4_orn using decoder 227
                                             // 18648:  }
  16, 25,                                    // 18653:  case 0x10: {
  OPC_SwitchField, 2, 6,                     // 18655:   switch Inst[7:2] {
  0, 9,                                      // 18658:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18660:    check Inst[13] == 0x0
  OPC_Decode, 189, 9, 169, 1,                // 18664:    decode to C2_cmpeq using decoder 169
                                             // 18664:   }
  4, 0,                                      // 18669:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 18671:    check Inst[13] == 0x0
  OPC_Decode, 220, 9, 169, 1,                // 18675:    decode to C4_cmpneq using decoder 169
                                             // 18675:   }
                                             // 18675:   } // switch Inst[7:2]
                                             // 18675:  }
  18, 25,                                    // 18680:  case 0x12: {
  OPC_SwitchField, 2, 6,                     // 18682:   switch Inst[7:2] {
  0, 9,                                      // 18685:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18687:    check Inst[13] == 0x0
  OPC_Decode, 192, 9, 169, 1,                // 18691:    decode to C2_cmpgt using decoder 169
                                             // 18691:   }
  4, 0,                                      // 18696:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 18698:    check Inst[13] == 0x0
  OPC_Decode, 216, 9, 169, 1,                // 18702:    decode to C4_cmplte using decoder 169
                                             // 18702:   }
                                             // 18702:   } // switch Inst[7:2]
                                             // 18702:  }
  19, 25,                                    // 18707:  case 0x13: {
  OPC_SwitchField, 2, 6,                     // 18709:   switch Inst[7:2] {
  0, 9,                                      // 18712:   case 0x0: {
  OPC_CheckField, 13, 1, 0,                  // 18714:    check Inst[13] == 0x0
  OPC_Decode, 195, 9, 169, 1,                // 18718:    decode to C2_cmpgtu using decoder 169
                                             // 18718:   }
  4, 0,                                      // 18723:   case 0x4: {
  OPC_CheckField, 13, 1, 0,                  // 18725:    check Inst[13] == 0x0
  OPC_Decode, 218, 9, 169, 1,                // 18729:    decode to C4_cmplteu using decoder 169
                                             // 18729:   }
                                             // 18729:   } // switch Inst[7:2]
                                             // 18729:  }
  24, 13,                                    // 18734:  case 0x18: {
  OPC_CheckField, 13, 1, 0,                  // 18736:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18740:   check Inst[7:5] == 0x0
  OPC_Decode, 165, 7, 134, 1,                // 18744:   decode to A2_add using decoder 134
                                             // 18744:  }
  25, 13,                                    // 18749:  case 0x19: {
  OPC_CheckField, 13, 1, 0,                  // 18751:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18755:   check Inst[7:5] == 0x0
  OPC_Decode, 240, 7, 227, 1,                // 18759:   decode to A2_sub using decoder 227
                                             // 18759:  }
  26, 13,                                    // 18764:  case 0x1a: {
  OPC_CheckField, 13, 1, 0,                  // 18766:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18770:   check Inst[7:5] == 0x0
  OPC_Decode, 135, 9, 134, 1,                // 18774:   decode to A4_rcmpeq using decoder 134
                                             // 18774:  }
  27, 13,                                    // 18779:  case 0x1b: {
  OPC_CheckField, 13, 1, 0,                  // 18781:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18785:   check Inst[7:5] == 0x0
  OPC_Decode, 137, 9, 134, 1,                // 18789:   decode to A4_rcmpneq using decoder 134
                                             // 18789:  }
  28, 13,                                    // 18794:  case 0x1c: {
  OPC_CheckField, 13, 1, 0,                  // 18796:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18800:   check Inst[7:5] == 0x0
  OPC_Decode, 189, 7, 227, 1,                // 18804:   decode to A2_combine_hh using decoder 227
                                             // 18804:  }
  29, 13,                                    // 18809:  case 0x1d: {
  OPC_CheckField, 13, 1, 0,                  // 18811:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18815:   check Inst[7:5] == 0x0
  OPC_Decode, 190, 7, 227, 1,                // 18819:   decode to A2_combine_hl using decoder 227
                                             // 18819:  }
  30, 13,                                    // 18824:  case 0x1e: {
  OPC_CheckField, 13, 1, 0,                  // 18826:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18830:   check Inst[7:5] == 0x0
  OPC_Decode, 191, 7, 227, 1,                // 18834:   decode to A2_combine_lh using decoder 227
                                             // 18834:  }
  31, 13,                                    // 18839:  case 0x1f: {
  OPC_CheckField, 13, 1, 0,                  // 18841:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18845:   check Inst[7:5] == 0x0
  OPC_Decode, 192, 7, 227, 1,                // 18849:   decode to A2_combine_ll using decoder 227
                                             // 18849:  }
  32, 13,                                    // 18854:  case 0x20: {
  OPC_CheckField, 13, 1, 0,                  // 18856:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 18860:   check Inst[7] == 0x0
  OPC_Decode, 199, 9, 254, 1,                // 18864:   decode to C2_mux using decoder 254
                                             // 18864:  }
  40, 13,                                    // 18869:  case 0x28: {
  OPC_CheckField, 13, 1, 0,                  // 18871:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18875:   check Inst[7:5] == 0x0
  OPC_Decode, 194, 7, 226, 1,                // 18879:   decode to A2_combinew using decoder 226
                                             // 18879:  }
  44, 13,                                    // 18884:  case 0x2c: {
  OPC_CheckField, 13, 1, 0,                  // 18886:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18890:   check Inst[7:5] == 0x0
  OPC_Decode, 209, 17, 226, 1,               // 18894:   decode to S2_packhl using decoder 226
                                             // 18894:  }
  48, 13,                                    // 18899:  case 0x30: {
  OPC_CheckField, 13, 1, 0,                  // 18901:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18905:   check Inst[7:5] == 0x0
  OPC_Decode, 128, 8, 134, 1,                // 18909:   decode to A2_svaddh using decoder 134
                                             // 18909:  }
  49, 13,                                    // 18914:  case 0x31: {
  OPC_CheckField, 13, 1, 0,                  // 18916:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18920:   check Inst[7:5] == 0x0
  OPC_Decode, 129, 8, 134, 1,                // 18924:   decode to A2_svaddhs using decoder 134
                                             // 18924:  }
  50, 13,                                    // 18929:  case 0x32: {
  OPC_CheckField, 13, 1, 0,                  // 18931:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18935:   check Inst[7:5] == 0x0
  OPC_Decode, 181, 7, 134, 1,                // 18939:   decode to A2_addsat using decoder 134
                                             // 18939:  }
  51, 13,                                    // 18944:  case 0x33: {
  OPC_CheckField, 13, 1, 0,                  // 18946:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18950:   check Inst[7:5] == 0x0
  OPC_Decode, 130, 8, 134, 1,                // 18954:   decode to A2_svadduhs using decoder 134
                                             // 18954:  }
  52, 13,                                    // 18959:  case 0x34: {
  OPC_CheckField, 13, 1, 0,                  // 18961:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18965:   check Inst[7:5] == 0x0
  OPC_Decode, 134, 8, 227, 1,                // 18969:   decode to A2_svsubh using decoder 227
                                             // 18969:  }
  53, 13,                                    // 18974:  case 0x35: {
  OPC_CheckField, 13, 1, 0,                  // 18976:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18980:   check Inst[7:5] == 0x0
  OPC_Decode, 135, 8, 227, 1,                // 18984:   decode to A2_svsubhs using decoder 227
                                             // 18984:  }
  54, 13,                                    // 18989:  case 0x36: {
  OPC_CheckField, 13, 1, 0,                  // 18991:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 18995:   check Inst[7:5] == 0x0
  OPC_Decode, 255, 7, 227, 1,                // 18999:   decode to A2_subsat using decoder 227
                                             // 18999:  }
  55, 13,                                    // 19004:  case 0x37: {
  OPC_CheckField, 13, 1, 0,                  // 19006:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 19010:   check Inst[7:5] == 0x0
  OPC_Decode, 136, 8, 227, 1,                // 19014:   decode to A2_svsubuhs using decoder 227
                                             // 19014:  }
  56, 13,                                    // 19019:  case 0x38: {
  OPC_CheckField, 13, 1, 0,                  // 19021:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 19025:   check Inst[7:5] == 0x0
  OPC_Decode, 131, 8, 134, 1,                // 19029:   decode to A2_svavgh using decoder 134
                                             // 19029:  }
  57, 13,                                    // 19034:  case 0x39: {
  OPC_CheckField, 13, 1, 0,                  // 19036:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 19040:   check Inst[7:5] == 0x0
  OPC_Decode, 132, 8, 134, 1,                // 19044:   decode to A2_svavghs using decoder 134
                                             // 19044:  }
  59, 13,                                    // 19049:  case 0x3b: {
  OPC_CheckField, 13, 1, 0,                  // 19051:   check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 0,                   // 19055:   check Inst[7:5] == 0x0
  OPC_Decode, 133, 8, 227, 1,                // 19059:   decode to A2_svnavgh using decoder 227
                                             // 19059:  }
  72, 41,                                    // 19064:  case 0x48: {
  OPC_SwitchField, 7, 1,                     // 19066:   switch Inst[7] {
  0, 17,                                     // 19069:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 19071:    switch Inst[13] {
  0, 5,                                      // 19074:    case 0x0: {
  OPC_Decode, 220, 7, 254, 1,                // 19076:     decode to A2_pandt using decoder 254
                                             // 19076:    }
  1, 0,                                      // 19081:    case 0x1: {
  OPC_Decode, 221, 7, 254, 1,                // 19083:     decode to A2_pandtnew using decoder 254
                                             // 19083:    }
                                             // 19083:    } // switch Inst[13]
                                             // 19083:   }
  1, 0,                                      // 19088:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 19090:    switch Inst[13] {
  0, 5,                                      // 19093:    case 0x0: {
  OPC_Decode, 218, 7, 254, 1,                // 19095:     decode to A2_pandf using decoder 254
                                             // 19095:    }
  1, 0,                                      // 19100:    case 0x1: {
  OPC_Decode, 219, 7, 254, 1,                // 19102:     decode to A2_pandfnew using decoder 254
                                             // 19102:    }
                                             // 19102:    } // switch Inst[13]
                                             // 19102:   }
                                             // 19102:   } // switch Inst[7]
                                             // 19102:  }
  73, 41,                                    // 19107:  case 0x49: {
  OPC_SwitchField, 7, 1,                     // 19109:   switch Inst[7] {
  0, 17,                                     // 19112:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 19114:    switch Inst[13] {
  0, 5,                                      // 19117:    case 0x0: {
  OPC_Decode, 224, 7, 254, 1,                // 19119:     decode to A2_port using decoder 254
                                             // 19119:    }
  1, 0,                                      // 19124:    case 0x1: {
  OPC_Decode, 225, 7, 254, 1,                // 19126:     decode to A2_portnew using decoder 254
                                             // 19126:    }
                                             // 19126:    } // switch Inst[13]
                                             // 19126:   }
  1, 0,                                      // 19131:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 19133:    switch Inst[13] {
  0, 5,                                      // 19136:    case 0x0: {
  OPC_Decode, 222, 7, 254, 1,                // 19138:     decode to A2_porf using decoder 254
                                             // 19138:    }
  1, 0,                                      // 19143:    case 0x1: {
  OPC_Decode, 223, 7, 254, 1,                // 19145:     decode to A2_porfnew using decoder 254
                                             // 19145:    }
                                             // 19145:    } // switch Inst[13]
                                             // 19145:   }
                                             // 19145:   } // switch Inst[7]
                                             // 19145:  }
  75, 41,                                    // 19150:  case 0x4b: {
  OPC_SwitchField, 7, 1,                     // 19152:   switch Inst[7] {
  0, 17,                                     // 19155:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 19157:    switch Inst[13] {
  0, 5,                                      // 19160:    case 0x0: {
  OPC_Decode, 232, 7, 254, 1,                // 19162:     decode to A2_pxort using decoder 254
                                             // 19162:    }
  1, 0,                                      // 19167:    case 0x1: {
  OPC_Decode, 233, 7, 254, 1,                // 19169:     decode to A2_pxortnew using decoder 254
                                             // 19169:    }
                                             // 19169:    } // switch Inst[13]
                                             // 19169:   }
  1, 0,                                      // 19174:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 19176:    switch Inst[13] {
  0, 5,                                      // 19179:    case 0x0: {
  OPC_Decode, 230, 7, 254, 1,                // 19181:     decode to A2_pxorf using decoder 254
                                             // 19181:    }
  1, 0,                                      // 19186:    case 0x1: {
  OPC_Decode, 231, 7, 254, 1,                // 19188:     decode to A2_pxorfnew using decoder 254
                                             // 19188:    }
                                             // 19188:    } // switch Inst[13]
                                             // 19188:   }
                                             // 19188:   } // switch Inst[7]
                                             // 19188:  }
  88, 41,                                    // 19193:  case 0x58: {
  OPC_SwitchField, 7, 1,                     // 19195:   switch Inst[7] {
  0, 17,                                     // 19198:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 19200:    switch Inst[13] {
  0, 5,                                      // 19203:    case 0x0: {
  OPC_Decode, 216, 7, 254, 1,                // 19205:     decode to A2_paddt using decoder 254
                                             // 19205:    }
  1, 0,                                      // 19210:    case 0x1: {
  OPC_Decode, 217, 7, 254, 1,                // 19212:     decode to A2_paddtnew using decoder 254
                                             // 19212:    }
                                             // 19212:    } // switch Inst[13]
                                             // 19212:   }
  1, 0,                                      // 19217:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 19219:    switch Inst[13] {
  0, 5,                                      // 19222:    case 0x0: {
  OPC_Decode, 210, 7, 254, 1,                // 19224:     decode to A2_paddf using decoder 254
                                             // 19224:    }
  1, 0,                                      // 19229:    case 0x1: {
  OPC_Decode, 211, 7, 254, 1,                // 19231:     decode to A2_paddfnew using decoder 254
                                             // 19231:    }
                                             // 19231:    } // switch Inst[13]
                                             // 19231:   }
                                             // 19231:   } // switch Inst[7]
                                             // 19231:  }
  89, 41,                                    // 19236:  case 0x59: {
  OPC_SwitchField, 7, 1,                     // 19238:   switch Inst[7] {
  0, 17,                                     // 19241:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 19243:    switch Inst[13] {
  0, 5,                                      // 19246:    case 0x0: {
  OPC_Decode, 228, 7, 129, 2,                // 19248:     decode to A2_psubt using decoder 257
                                             // 19248:    }
  1, 0,                                      // 19253:    case 0x1: {
  OPC_Decode, 229, 7, 129, 2,                // 19255:     decode to A2_psubtnew using decoder 257
                                             // 19255:    }
                                             // 19255:    } // switch Inst[13]
                                             // 19255:   }
  1, 0,                                      // 19260:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 19262:    switch Inst[13] {
  0, 5,                                      // 19265:    case 0x0: {
  OPC_Decode, 226, 7, 129, 2,                // 19267:     decode to A2_psubf using decoder 257
                                             // 19267:    }
  1, 0,                                      // 19272:    case 0x1: {
  OPC_Decode, 227, 7, 129, 2,                // 19274:     decode to A2_psubfnew using decoder 257
                                             // 19274:    }
                                             // 19274:    } // switch Inst[13]
                                             // 19274:   }
                                             // 19274:   } // switch Inst[7]
                                             // 19274:  }
  104, 0,                                    // 19279:  case 0x68: {
  OPC_SwitchField, 7, 1,                     // 19281:   switch Inst[7] {
  0, 17,                                     // 19284:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 19286:    switch Inst[13] {
  0, 5,                                      // 19289:    case 0x0: {
  OPC_Decode, 184, 9, 130, 2,                // 19291:     decode to C2_ccombinewt using decoder 258
                                             // 19291:    }
  1, 0,                                      // 19296:    case 0x1: {
  OPC_Decode, 183, 9, 130, 2,                // 19298:     decode to C2_ccombinewnewt using decoder 258
                                             // 19298:    }
                                             // 19298:    } // switch Inst[13]
                                             // 19298:   }
  1, 0,                                      // 19303:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 19305:    switch Inst[13] {
  0, 5,                                      // 19308:    case 0x0: {
  OPC_Decode, 181, 9, 130, 2,                // 19310:     decode to C2_ccombinewf using decoder 258
                                             // 19310:    }
  1, 0,                                      // 19315:    case 0x1: {
  OPC_Decode, 182, 9, 130, 2,                // 19317:     decode to C2_ccombinewnewf using decoder 258
                                             // 19317:    }
                                             // 19317:    } // switch Inst[13]
                                             // 19317:   }
                                             // 19317:   } // switch Inst[7]
                                             // 19317:  }
                                             // 19317:  } // switch Inst[27:21]
                                             // 19317: }
                                             // 19317: } // switch Inst[31:28]
};
static const uint8_t DecoderTableEXT_mmvec32[8767] = {
  OPC_SwitchField, 24, 8,                    //    0: switch Inst[31:24] {
  24, 114,                                   //    3: case 0x18: {
  OPC_SwitchField, 5, 3,                     //    5:  switch Inst[7:5] {
  0, 11,                                     //    8:  case 0x0: {
  OPC_CheckPredicate, 12,                    //   10:   check predicate 12
  OPC_CheckField, 13, 1, 0,                  //   12:   check Inst[13] == 0x0
  OPC_Decode, 180, 22, 131, 2,               //   16:   decode to V6_vasrhbsat using decoder 259
                                             //   16:  }
  1, 11,                                     //   21:  case 0x1: {
  OPC_CheckPredicate, 12,                    //   23:   check predicate 12
  OPC_CheckField, 13, 1, 0,                  //   25:   check Inst[13] == 0x0
  OPC_Decode, 186, 22, 131, 2,               //   29:   decode to V6_vasruwuhrndsat using decoder 259
                                             //   29:  }
  2, 11,                                     //   34:  case 0x2: {
  OPC_CheckPredicate, 12,                    //   36:   check predicate 12
  OPC_CheckField, 13, 1, 0,                  //   38:   check Inst[13] == 0x0
  OPC_Decode, 197, 22, 131, 2,               //   42:   decode to V6_vasrwuhrndsat using decoder 259
                                             //   42:  }
  3, 11,                                     //   47:  case 0x3: {
  OPC_CheckPredicate, 12,                    //   49:   check predicate 12
  OPC_CheckField, 13, 1, 0,                  //   51:   check Inst[13] == 0x0
  OPC_Decode, 240, 23, 131, 2,               //   55:   decode to V6_vlutvvb_nm using decoder 259
                                             //   55:  }
  4, 21,                                     //   60:  case 0x4: {
  OPC_SwitchField, 13, 1,                    //   62:   switch Inst[13] {
  0, 7,                                      //   65:   case 0x0: {
  OPC_CheckPredicate, 12,                    //   67:    check predicate 12
  OPC_Decode, 245, 23, 132, 2,               //   69:    decode to V6_vlutvwh_nm using decoder 260
                                             //   69:   }
  1, 0,                                      //   74:   case 0x1: {
  OPC_CheckPredicate, 13,                    //   76:    check predicate 13
  OPC_Decode, 187, 22, 131, 2,               //   78:    decode to V6_vasruwuhsat using decoder 259
                                             //   78:   }
                                             //   78:   } // switch Inst[13]
                                             //   78:  }
  5, 21,                                     //   83:  case 0x5: {
  OPC_SwitchField, 13, 1,                    //   85:   switch Inst[13] {
  0, 7,                                      //   88:   case 0x0: {
  OPC_CheckPredicate, 14,                    //   90:    check predicate 14
  OPC_Decode, 158, 22, 131, 2,               //   92:    decode to V6_valign4 using decoder 259
                                             //   92:   }
  1, 0,                                      //   97:   case 0x1: {
  OPC_CheckPredicate, 13,                    //   99:    check predicate 13
  OPC_Decode, 185, 22, 131, 2,               //  101:    decode to V6_vasruhubsat using decoder 259
                                             //  101:   }
                                             //  101:   } // switch Inst[13]
                                             //  101:  }
  7, 0,                                      //  106:  case 0x7: {
  OPC_CheckPredicate, 13,                    //  108:   check predicate 13
  OPC_CheckField, 13, 1, 0,                  //  110:   check Inst[13] == 0x0
  OPC_Decode, 184, 22, 131, 2,               //  114:   decode to V6_vasruhubrndsat using decoder 259
                                             //  114:  }
                                             //  114:  } // switch Inst[7:5]
                                             //  114: }
  25, 243, 11,                               //  119: case 0x19: {
  OPC_SwitchField, 21, 3,                    //  122:  switch Inst[23:21] {
  0, 187, 1,                                 //  125:  case 0x0: {
  OPC_SwitchField, 5, 3,                     //  128:   switch Inst[7:5] {
  0, 21,                                     //  131:   case 0x0: {
  OPC_SwitchField, 13, 1,                    //  133:    switch Inst[13] {
  0, 7,                                      //  136:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  138:     check predicate 15
  OPC_Decode, 237, 25, 133, 2,               //  140:     decode to V6_vtmpyb using decoder 261
                                             //  140:    }
  1, 0,                                      //  145:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  147:     check predicate 15
  OPC_Decode, 238, 25, 134, 2,               //  149:     decode to V6_vtmpyb_acc using decoder 262
                                             //  149:    }
                                             //  149:    } // switch Inst[13]
                                             //  149:   }
  1, 21,                                     //  154:   case 0x1: {
  OPC_SwitchField, 13, 1,                    //  156:    switch Inst[13] {
  0, 7,                                      //  159:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  161:     check predicate 15
  OPC_Decode, 239, 25, 133, 2,               //  163:     decode to V6_vtmpybus using decoder 261
                                             //  163:    }
  1, 0,                                      //  168:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  170:     check predicate 15
  OPC_Decode, 240, 25, 134, 2,               //  172:     decode to V6_vtmpybus_acc using decoder 262
                                             //  172:    }
                                             //  172:    } // switch Inst[13]
                                             //  172:   }
  2, 21,                                     //  177:   case 0x2: {
  OPC_SwitchField, 13, 1,                    //  179:    switch Inst[13] {
  0, 7,                                      //  182:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  184:     check predicate 15
  OPC_Decode, 137, 23, 135, 2,               //  186:     decode to V6_vdmpyhb using decoder 263
                                             //  186:    }
  1, 0,                                      //  191:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  193:     check predicate 15
  OPC_Decode, 242, 25, 134, 2,               //  195:     decode to V6_vtmpyhb_acc using decoder 262
                                             //  195:    }
                                             //  195:    } // switch Inst[13]
                                             //  195:   }
  3, 21,                                     //  200:   case 0x3: {
  OPC_SwitchField, 13, 1,                    //  202:    switch Inst[13] {
  0, 7,                                      //  205:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  207:     check predicate 15
  OPC_Decode, 130, 25, 135, 2,               //  209:     decode to V6_vrmpyub using decoder 263
                                             //  209:    }
  1, 0,                                      //  214:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  216:     check predicate 15
  OPC_Decode, 138, 23, 136, 2,               //  218:     decode to V6_vdmpyhb_acc using decoder 264
                                             //  218:    }
                                             //  218:    } // switch Inst[13]
                                             //  218:   }
  4, 21,                                     //  223:   case 0x4: {
  OPC_SwitchField, 13, 1,                    //  225:    switch Inst[13] {
  0, 7,                                      //  228:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  230:     check predicate 15
  OPC_Decode, 250, 24, 135, 2,               //  232:     decode to V6_vrmpybus using decoder 263
                                             //  232:    }
  1, 0,                                      //  237:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  239:     check predicate 15
  OPC_Decode, 131, 25, 136, 2,               //  241:     decode to V6_vrmpyub_acc using decoder 264
                                             //  241:    }
                                             //  241:    } // switch Inst[13]
                                             //  241:   }
  5, 21,                                     //  246:   case 0x5: {
  OPC_SwitchField, 13, 1,                    //  248:    switch Inst[13] {
  0, 7,                                      //  251:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  253:     check predicate 15
  OPC_Decode, 151, 23, 133, 2,               //  255:     decode to V6_vdsaduh using decoder 261
                                             //  255:    }
  1, 0,                                      //  260:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  262:     check predicate 15
  OPC_Decode, 251, 24, 136, 2,               //  264:     decode to V6_vrmpybus_acc using decoder 264
                                             //  264:    }
                                             //  264:    } // switch Inst[13]
                                             //  264:   }
  6, 21,                                     //  269:   case 0x6: {
  OPC_SwitchField, 13, 1,                    //  271:    switch Inst[13] {
  0, 7,                                      //  274:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  276:     check predicate 15
  OPC_Decode, 133, 23, 135, 2,               //  278:     decode to V6_vdmpybus using decoder 263
                                             //  278:    }
  1, 0,                                      //  283:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  285:     check predicate 15
  OPC_Decode, 134, 23, 136, 2,               //  287:     decode to V6_vdmpybus_acc using decoder 264
                                             //  287:    }
                                             //  287:    } // switch Inst[13]
                                             //  287:   }
  7, 0,                                      //  292:   case 0x7: {
  OPC_SwitchField, 13, 1,                    //  294:    switch Inst[13] {
  0, 7,                                      //  297:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  299:     check predicate 15
  OPC_Decode, 135, 23, 133, 2,               //  301:     decode to V6_vdmpybus_dv using decoder 261
                                             //  301:    }
  1, 0,                                      //  306:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  308:     check predicate 15
  OPC_Decode, 136, 23, 134, 2,               //  310:     decode to V6_vdmpybus_dv_acc using decoder 262
                                             //  310:    }
                                             //  310:    } // switch Inst[13]
                                             //  310:   }
                                             //  310:   } // switch Inst[7:5]
                                             //  310:  }
  1, 187, 1,                                 //  315:  case 0x1: {
  OPC_SwitchField, 5, 3,                     //  318:   switch Inst[7:5] {
  0, 21,                                     //  321:   case 0x0: {
  OPC_SwitchField, 13, 1,                    //  323:    switch Inst[13] {
  0, 7,                                      //  326:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  328:     check predicate 15
  OPC_Decode, 147, 23, 135, 2,               //  330:     decode to V6_vdmpyhsusat using decoder 263
                                             //  330:    }
  1, 0,                                      //  335:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  337:     check predicate 15
  OPC_Decode, 148, 23, 136, 2,               //  339:     decode to V6_vdmpyhsusat_acc using decoder 264
                                             //  339:    }
                                             //  339:    } // switch Inst[13]
                                             //  339:   }
  1, 21,                                     //  344:   case 0x1: {
  OPC_SwitchField, 13, 1,                    //  346:    switch Inst[13] {
  0, 7,                                      //  349:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  351:     check predicate 15
  OPC_Decode, 145, 23, 137, 2,               //  353:     decode to V6_vdmpyhsuisat using decoder 265
                                             //  353:    }
  1, 0,                                      //  358:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  360:     check predicate 15
  OPC_Decode, 146, 23, 138, 2,               //  362:     decode to V6_vdmpyhsuisat_acc using decoder 266
                                             //  362:    }
                                             //  362:    } // switch Inst[13]
                                             //  362:   }
  2, 21,                                     //  367:   case 0x2: {
  OPC_SwitchField, 13, 1,                    //  369:    switch Inst[13] {
  0, 7,                                      //  372:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  374:     check predicate 15
  OPC_Decode, 143, 23, 135, 2,               //  376:     decode to V6_vdmpyhsat using decoder 263
                                             //  376:    }
  1, 0,                                      //  381:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  383:     check predicate 15
  OPC_Decode, 142, 23, 138, 2,               //  385:     decode to V6_vdmpyhisat_acc using decoder 266
                                             //  385:    }
                                             //  385:    } // switch Inst[13]
                                             //  385:   }
  3, 21,                                     //  390:   case 0x3: {
  OPC_SwitchField, 13, 1,                    //  392:    switch Inst[13] {
  0, 7,                                      //  395:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  397:     check predicate 15
  OPC_Decode, 141, 23, 137, 2,               //  399:     decode to V6_vdmpyhisat using decoder 265
                                             //  399:    }
  1, 0,                                      //  404:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  406:     check predicate 15
  OPC_Decode, 144, 23, 136, 2,               //  408:     decode to V6_vdmpyhsat_acc using decoder 264
                                             //  408:    }
                                             //  408:    } // switch Inst[13]
                                             //  408:   }
  4, 21,                                     //  413:   case 0x4: {
  OPC_SwitchField, 13, 1,                    //  415:    switch Inst[13] {
  0, 7,                                      //  418:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  420:     check predicate 15
  OPC_Decode, 139, 23, 133, 2,               //  422:     decode to V6_vdmpyhb_dv using decoder 261
                                             //  422:    }
  1, 0,                                      //  427:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  429:     check predicate 15
  OPC_Decode, 140, 23, 134, 2,               //  431:     decode to V6_vdmpyhb_dv_acc using decoder 262
                                             //  431:    }
                                             //  431:    } // switch Inst[13]
                                             //  431:   }
  5, 21,                                     //  436:   case 0x5: {
  OPC_SwitchField, 13, 1,                    //  438:    switch Inst[13] {
  0, 7,                                      //  441:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  443:     check predicate 15
  OPC_Decode, 171, 24, 139, 2,               //  445:     decode to V6_vmpybus using decoder 267
                                             //  445:    }
  1, 0,                                      //  450:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  452:     check predicate 15
  OPC_Decode, 172, 24, 140, 2,               //  454:     decode to V6_vmpybus_acc using decoder 268
                                             //  454:    }
                                             //  454:    } // switch Inst[13]
                                             //  454:   }
  6, 21,                                     //  459:   case 0x6: {
  OPC_SwitchField, 13, 1,                    //  461:    switch Inst[13] {
  0, 7,                                      //  464:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  466:     check predicate 15
  OPC_Decode, 138, 24, 133, 2,               //  468:     decode to V6_vmpabus using decoder 261
                                             //  468:    }
  1, 0,                                      //  473:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  475:     check predicate 15
  OPC_Decode, 139, 24, 134, 2,               //  477:     decode to V6_vmpabus_acc using decoder 262
                                             //  477:    }
                                             //  477:    } // switch Inst[13]
                                             //  477:   }
  7, 0,                                      //  482:   case 0x7: {
  OPC_SwitchField, 13, 1,                    //  484:    switch Inst[13] {
  0, 7,                                      //  487:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  489:     check predicate 15
  OPC_Decode, 144, 24, 133, 2,               //  491:     decode to V6_vmpahb using decoder 261
                                             //  491:    }
  1, 0,                                      //  496:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  498:     check predicate 15
  OPC_Decode, 145, 24, 134, 2,               //  500:     decode to V6_vmpahb_acc using decoder 262
                                             //  500:    }
                                             //  500:    } // switch Inst[13]
                                             //  500:   }
                                             //  500:   } // switch Inst[7:5]
                                             //  500:  }
  2, 151, 1,                                 //  505:  case 0x2: {
  OPC_SwitchField, 6, 2,                     //  508:   switch Inst[7:6] {
  0, 49,                                     //  511:   case 0x0: {
  OPC_SwitchField, 5, 1,                     //  513:    switch Inst[5] {
  0, 21,                                     //  516:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  518:     switch Inst[13] {
  0, 7,                                      //  521:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  523:      check predicate 15
  OPC_Decode, 179, 24, 139, 2,               //  525:      decode to V6_vmpyh using decoder 267
                                             //  525:     }
  1, 0,                                      //  530:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  532:      check predicate 15
  OPC_Decode, 181, 24, 140, 2,               //  534:      decode to V6_vmpyhsat_acc using decoder 268
                                             //  534:     }
                                             //  534:     } // switch Inst[13]
                                             //  534:    }
  1, 0,                                      //  539:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  541:     switch Inst[13] {
  0, 7,                                      //  544:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  546:      check predicate 15
  OPC_Decode, 183, 24, 135, 2,               //  548:      decode to V6_vmpyhss using decoder 263
                                             //  548:     }
  1, 0,                                      //  553:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  555:      check predicate 15
  OPC_Decode, 214, 24, 140, 2,               //  557:      decode to V6_vmpyuh_acc using decoder 268
                                             //  557:     }
                                             //  557:     } // switch Inst[13]
                                             //  557:    }
                                             //  557:    } // switch Inst[5]
                                             //  557:   }
  1, 49,                                     //  562:   case 0x1: {
  OPC_SwitchField, 5, 1,                     //  564:    switch Inst[5] {
  0, 21,                                     //  567:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  569:     switch Inst[13] {
  0, 7,                                      //  572:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  574:      check predicate 15
  OPC_Decode, 182, 24, 135, 2,               //  576:      decode to V6_vmpyhsrs using decoder 263
                                             //  576:     }
  1, 0,                                      //  581:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  583:      check predicate 15
  OPC_Decode, 199, 24, 136, 2,               //  585:      decode to V6_vmpyiwb_acc using decoder 264
                                             //  585:     }
                                             //  585:     } // switch Inst[13]
                                             //  585:    }
  1, 0,                                      //  590:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  592:     switch Inst[13] {
  0, 7,                                      //  595:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  597:      check predicate 15
  OPC_Decode, 213, 24, 139, 2,               //  599:      decode to V6_vmpyuh using decoder 267
                                             //  599:     }
  1, 0,                                      //  604:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  606:      check predicate 15
  OPC_Decode, 201, 24, 136, 2,               //  608:      decode to V6_vmpyiwh_acc using decoder 264
                                             //  608:     }
                                             //  608:     } // switch Inst[13]
                                             //  608:    }
                                             //  608:    } // switch Inst[5]
                                             //  608:   }
  2, 21,                                     //  613:   case 0x2: {
  OPC_SwitchField, 13, 1,                    //  615:    switch Inst[13] {
  0, 7,                                      //  618:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  620:     check predicate 15
  OPC_Decode, 252, 24, 141, 2,               //  622:     decode to V6_vrmpybusi using decoder 269
                                             //  622:    }
  1, 0,                                      //  627:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  629:     check predicate 15
  OPC_Decode, 253, 24, 142, 2,               //  631:     decode to V6_vrmpybusi_acc using decoder 270
                                             //  631:    }
                                             //  631:    } // switch Inst[13]
                                             //  631:   }
  3, 0,                                      //  636:   case 0x3: {
  OPC_SwitchField, 13, 1,                    //  638:    switch Inst[13] {
  0, 7,                                      //  641:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  643:     check predicate 15
  OPC_Decode, 166, 25, 141, 2,               //  645:     decode to V6_vrsadubi using decoder 269
                                             //  645:    }
  1, 0,                                      //  650:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  652:     check predicate 15
  OPC_Decode, 167, 25, 142, 2,               //  654:     decode to V6_vrsadubi_acc using decoder 270
                                             //  654:    }
                                             //  654:    } // switch Inst[13]
                                             //  654:   }
                                             //  654:   } // switch Inst[7:6]
                                             //  654:  }
  3, 211, 1,                                 //  659:  case 0x3: {
  OPC_SwitchField, 6, 2,                     //  662:   switch Inst[7:6] {
  0, 49,                                     //  665:   case 0x0: {
  OPC_SwitchField, 5, 1,                     //  667:    switch Inst[5] {
  0, 21,                                     //  670:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  672:     switch Inst[13] {
  0, 7,                                      //  675:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  677:      check predicate 15
  OPC_Decode, 195, 24, 135, 2,               //  679:      decode to V6_vmpyihb using decoder 263
                                             //  679:     }
  1, 0,                                      //  684:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  686:      check predicate 15
  OPC_Decode, 152, 23, 134, 2,               //  688:      decode to V6_vdsaduh_acc using decoder 262
                                             //  688:     }
                                             //  688:     } // switch Inst[13]
                                             //  688:    }
  1, 0,                                      //  693:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  695:     switch Inst[13] {
  0, 7,                                      //  698:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  700:      check predicate 15
  OPC_Decode, 158, 25, 135, 2,               //  702:      decode to V6_vror using decoder 263
                                             //  702:     }
  1, 0,                                      //  707:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  709:      check predicate 15
  OPC_Decode, 196, 24, 136, 2,               //  711:      decode to V6_vmpyihb_acc using decoder 264
                                             //  711:     }
                                             //  711:     } // switch Inst[13]
                                             //  711:    }
                                             //  711:    } // switch Inst[5]
                                             //  711:   }
  1, 63,                                     //  716:   case 0x1: {
  OPC_SwitchField, 5, 1,                     //  718:    switch Inst[5] {
  0, 21,                                     //  721:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  723:     switch Inst[13] {
  0, 7,                                      //  726:     case 0x0: {
  OPC_CheckPredicate, 13,                    //  728:      check predicate 13
  OPC_Decode, 215, 24, 135, 2,               //  730:      decode to V6_vmpyuhe using decoder 263
                                             //  730:     }
  1, 0,                                      //  735:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  737:      check predicate 15
  OPC_Decode, 174, 22, 136, 2,               //  739:      decode to V6_vaslw_acc using decoder 264
                                             //  739:     }
                                             //  739:     } // switch Inst[13]
                                             //  739:    }
  1, 0,                                      //  744:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  746:     switch Inst[13] {
  0, 7,                                      //  749:     case 0x0: {
  OPC_CheckPredicate, 13,                    //  751:      check predicate 13
  OPC_Decode, 141, 24, 133, 2,               //  753:      decode to V6_vmpabuu using decoder 261
                                             //  753:     }
  1, 0,                                      //  758:     case 0x1: {
  OPC_SwitchField, 10, 3,                    //  760:      switch Inst[12:10] {
  0, 7,                                      //  763:      case 0x0: {
  OPC_CheckPredicate, 15,                    //  765:       check predicate 15
  OPC_Decode, 165, 22, 143, 2,               //  767:       decode to V6_vandqrt_acc using decoder 271
                                             //  767:      }
  1, 0,                                      //  772:      case 0x1: {
  OPC_CheckPredicate, 12,                    //  774:       check predicate 12
  OPC_Decode, 163, 22, 143, 2,               //  776:       decode to V6_vandnqrt_acc using decoder 271
                                             //  776:      }
                                             //  776:      } // switch Inst[12:10]
                                             //  776:     }
                                             //  776:     } // switch Inst[13]
                                             //  776:    }
                                             //  776:    } // switch Inst[5]
                                             //  776:   }
  2, 53,                                     //  781:   case 0x2: {
  OPC_SwitchField, 5, 1,                     //  783:    switch Inst[5] {
  0, 25,                                     //  786:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  788:     switch Inst[13] {
  0, 7,                                      //  791:     case 0x0: {
  OPC_CheckPredicate, 13,                    //  793:      check predicate 13
  OPC_Decode, 238, 23, 144, 2,               //  795:      decode to V6_vlut4 using decoder 272
                                             //  795:     }
  1, 0,                                      //  800:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  802:      check predicate 15
  OPC_CheckField, 2, 3, 0,                   //  804:      check Inst[4:2] == 0x0
  OPC_Decode, 169, 22, 145, 2,               //  808:      decode to V6_vandvrt_acc using decoder 273
                                             //  808:     }
                                             //  808:     } // switch Inst[13]
                                             //  808:    }
  1, 0,                                      //  813:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  815:     switch Inst[13] {
  0, 7,                                      //  818:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  820:      check predicate 15
  OPC_Decode, 192, 22, 135, 2,               //  822:      decode to V6_vasrw using decoder 263
                                             //  822:     }
  1, 0,                                      //  827:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  829:      check predicate 15
  OPC_Decode, 193, 22, 136, 2,               //  831:      decode to V6_vasrw_acc using decoder 264
                                             //  831:     }
                                             //  831:     } // switch Inst[13]
                                             //  831:    }
                                             //  831:    } // switch Inst[5]
                                             //  831:   }
  3, 0,                                      //  836:   case 0x3: {
  OPC_SwitchField, 13, 1,                    //  838:    switch Inst[13] {
  0, 21,                                     //  841:    case 0x0: {
  OPC_SwitchField, 5, 1,                     //  843:     switch Inst[5] {
  0, 7,                                      //  846:     case 0x0: {
  OPC_CheckPredicate, 15,                    //  848:      check predicate 15
  OPC_Decode, 177, 22, 135, 2,               //  850:      decode to V6_vasrh using decoder 263
                                             //  850:     }
  1, 0,                                      //  855:     case 0x1: {
  OPC_CheckPredicate, 15,                    //  857:      check predicate 15
  OPC_Decode, 173, 22, 135, 2,               //  859:      decode to V6_vaslw using decoder 263
                                             //  859:     }
                                             //  859:     } // switch Inst[5]
                                             //  859:    }
  1, 0,                                      //  864:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  866:     check predicate 15
  OPC_Decode, 135, 25, 142, 2,               //  868:     decode to V6_vrmpyubi_acc using decoder 270
                                             //  868:    }
                                             //  868:    } // switch Inst[13]
                                             //  868:   }
                                             //  868:   } // switch Inst[7:6]
                                             //  868:  }
  4, 177, 1,                                 //  873:  case 0x4: {
  OPC_SwitchField, 5, 3,                     //  876:   switch Inst[7:5] {
  0, 21,                                     //  879:   case 0x0: {
  OPC_SwitchField, 13, 1,                    //  881:    switch Inst[13] {
  0, 7,                                      //  884:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  886:     check predicate 15
  OPC_Decode, 170, 22, 135, 2,               //  888:     decode to V6_vaslh using decoder 263
                                             //  888:    }
  1, 0,                                      //  893:    case 0x1: {
  OPC_CheckPredicate, 15,                    //  895:     check predicate 15
  OPC_Decode, 210, 24, 140, 2,               //  897:     decode to V6_vmpyub_acc using decoder 268
                                             //  897:    }
                                             //  897:    } // switch Inst[13]
                                             //  897:   }
  1, 21,                                     //  902:   case 0x1: {
  OPC_SwitchField, 13, 1,                    //  904:    switch Inst[13] {
  0, 7,                                      //  907:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  909:     check predicate 15
  OPC_Decode, 236, 23, 135, 2,               //  911:     decode to V6_vlsrw using decoder 263
                                             //  911:    }
  1, 0,                                      //  916:    case 0x1: {
  OPC_CheckPredicate, 12,                    //  918:     check predicate 12
  OPC_Decode, 203, 24, 136, 2,               //  920:     decode to V6_vmpyiwub_acc using decoder 264
                                             //  920:    }
                                             //  920:    } // switch Inst[13]
                                             //  920:   }
  2, 21,                                     //  925:   case 0x2: {
  OPC_SwitchField, 13, 1,                    //  927:    switch Inst[13] {
  0, 7,                                      //  930:    case 0x0: {
  OPC_CheckPredicate, 15,                    //  932:     check predicate 15
  OPC_Decode, 234, 23, 135, 2,               //  934:     decode to V6_vlsrh using decoder 263
                                             //  934:    }
  1, 0,                                      //  939:    case 0x1: {
  OPC_CheckPredicate, 12,                    //  941:     check predicate 12
  OPC_Decode, 148, 24, 134, 2,               //  943:     decode to V6_vmpauhb_acc using decoder 262
                                             //  943:    }
                                             //  943:    } // switch Inst[13]
                                             //  943:   }
  3, 21,                                     //  948:   case 0x3: {
  OPC_SwitchField, 13, 1,                    //  950:    switch Inst[13] {
  0, 7,                                      //  953:    case 0x0: {
  OPC_CheckPredicate, 12,                    //  955:     check predicate 12
  OPC_Decode, 233, 23, 135, 2,               //  957:     decode to V6_vlsrb using decoder 263
                                             //  957:    }
  1, 0,                                      //  962:    case 0x1: {
  OPC_CheckPredicate, 13,                    //  964:     check predicate 13
  OPC_Decode, 216, 24, 136, 2,               //  966:     decode to V6_vmpyuhe_acc using decoder 264
                                             //  966:    }
                                             //  966:    } // switch Inst[13]
                                             //  966:   }
  4, 11,                                     //  971:   case 0x4: {
  OPC_CheckPredicate, 13,                    //  973:    check predicate 13
  OPC_CheckField, 13, 1, 1,                  //  975:    check Inst[13] == 0x1
  OPC_Decode, 146, 24, 146, 2,               //  979:    decode to V6_vmpahhsat using decoder 274
                                             //  979:   }
  5, 21,                                     //  984:   case 0x5: {
  OPC_SwitchField, 13, 1,                    //  986:    switch Inst[13] {
  0, 7,                                      //  989:    case 0x0: {
  OPC_CheckPredicate, 12,                    //  991:     check predicate 12
  OPC_Decode, 147, 24, 133, 2,               //  993:     decode to V6_vmpauhb using decoder 261
                                             //  993:    }
  1, 0,                                      //  998:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 1000:     check predicate 13
  OPC_Decode, 149, 24, 146, 2,               // 1002:     decode to V6_vmpauhuhsat using decoder 274
                                             // 1002:    }
                                             // 1002:    } // switch Inst[13]
                                             // 1002:   }
  6, 21,                                     // 1007:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 1009:    switch Inst[13] {
  0, 7,                                      // 1012:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 1014:     check predicate 12
  OPC_Decode, 202, 24, 135, 2,               // 1016:     decode to V6_vmpyiwub using decoder 263
                                             // 1016:    }
  1, 0,                                      // 1021:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 1023:     check predicate 13
  OPC_Decode, 150, 24, 146, 2,               // 1025:     decode to V6_vmpsuhuhsat using decoder 274
                                             // 1025:    }
                                             // 1025:    } // switch Inst[13]
                                             // 1025:   }
  7, 0,                                      // 1030:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 1032:    switch Inst[13] {
  0, 7,                                      // 1035:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 1037:     check predicate 15
  OPC_Decode, 200, 24, 135, 2,               // 1039:     decode to V6_vmpyiwh using decoder 263
                                             // 1039:    }
  1, 0,                                      // 1044:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 1046:     check predicate 13
  OPC_Decode, 178, 22, 136, 2,               // 1048:     decode to V6_vasrh_acc using decoder 264
                                             // 1048:    }
                                             // 1048:    } // switch Inst[13]
                                             // 1048:   }
                                             // 1048:   } // switch Inst[7:5]
                                             // 1048:  }
  5, 209, 1,                                 // 1053:  case 0x5: {
  OPC_SwitchField, 6, 2,                     // 1056:   switch Inst[7:6] {
  0, 58,                                     // 1059:   case 0x0: {
  OPC_SwitchField, 5, 1,                     // 1061:    switch Inst[5] {
  0, 21,                                     // 1064:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1066:     switch Inst[13] {
  0, 7,                                      // 1069:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 1071:      check predicate 15
  OPC_Decode, 198, 24, 135, 2,               // 1073:      decode to V6_vmpyiwb using decoder 263
                                             // 1073:     }
  1, 0,                                      // 1078:     case 0x1: {
  OPC_CheckPredicate, 13,                    // 1080:      check predicate 13
  OPC_Decode, 249, 24, 147, 2,               // 1082:      decode to V6_vrmpybub_rtt_acc using decoder 275
                                             // 1082:     }
                                             // 1082:     } // switch Inst[13]
                                             // 1082:    }
  1, 0,                                      // 1087:    case 0x1: {
  OPC_SwitchField, 8, 6,                     // 1089:     switch Inst[13:8] {
  0, 7,                                      // 1092:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 1094:      check predicate 15
  OPC_Decode, 217, 20, 148, 2,               // 1096:      decode to V6_lvsplatw using decoder 276
                                             // 1096:     }
  1, 7,                                      // 1101:     case 0x1: {
  OPC_CheckPredicate, 16,                    // 1103:      check predicate 16
  OPC_Decode, 138, 26, 148, 2,               // 1105:      decode to V6_zextract using decoder 276
                                             // 1105:     }
  32, 0,                                     // 1110:     case 0x20: {
  OPC_CheckPredicate, 15,                    // 1112:      check predicate 15
  OPC_Decode, 230, 23, 149, 2,               // 1114:      decode to V6_vinsertwr using decoder 277
                                             // 1114:     }
                                             // 1114:     } // switch Inst[13:8]
                                             // 1114:    }
                                             // 1114:    } // switch Inst[5]
                                             // 1114:   }
  1, 42,                                     // 1119:   case 0x1: {
  OPC_SwitchField, 2, 4,                     // 1121:    switch Inst[5:2] {
  1, 11,                                     // 1124:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 1126:     check predicate 15
  OPC_CheckField, 8, 6, 0,                   // 1128:     check Inst[13:8] == 0x0
  OPC_Decode, 223, 20, 150, 2,               // 1132:     decode to V6_pred_scalar2 using decoder 278
                                             // 1132:    }
  2, 11,                                     // 1137:    case 0x2: {
  OPC_CheckPredicate, 15,                    // 1139:     check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 1141:     check Inst[13] == 0x0
  OPC_Decode, 168, 22, 151, 2,               // 1145:     decode to V6_vandvrt using decoder 279
                                             // 1145:    }
  3, 0,                                      // 1150:    case 0x3: {
  OPC_CheckPredicate, 12,                    // 1152:     check predicate 12
  OPC_CheckField, 8, 6, 0,                   // 1154:     check Inst[13:8] == 0x0
  OPC_Decode, 224, 20, 150, 2,               // 1158:     decode to V6_pred_scalar2v2 using decoder 278
                                             // 1158:    }
                                             // 1158:    } // switch Inst[5:2]
                                             // 1158:   }
  2, 63,                                     // 1163:   case 0x2: {
  OPC_SwitchField, 5, 1,                     // 1165:    switch Inst[5] {
  0, 21,                                     // 1168:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1170:     switch Inst[13] {
  0, 7,                                      // 1173:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 1175:      check predicate 15
  OPC_Decode, 241, 25, 133, 2,               // 1177:      decode to V6_vtmpyhb using decoder 261
                                             // 1177:     }
  1, 0,                                      // 1182:     case 0x1: {
  OPC_CheckPredicate, 13,                    // 1184:      check predicate 13
  OPC_Decode, 142, 24, 134, 2,               // 1186:      decode to V6_vmpabuu_acc using decoder 262
                                             // 1186:     }
                                             // 1186:     } // switch Inst[13]
                                             // 1186:    }
  1, 0,                                      // 1191:    case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1193:     switch Inst[13] {
  0, 21,                                     // 1196:     case 0x0: {
  OPC_SwitchField, 10, 3,                    // 1198:      switch Inst[12:10] {
  0, 7,                                      // 1201:      case 0x0: {
  OPC_CheckPredicate, 15,                    // 1203:       check predicate 15
  OPC_Decode, 164, 22, 152, 2,               // 1205:       decode to V6_vandqrt using decoder 280
                                             // 1205:      }
  1, 0,                                      // 1210:      case 0x1: {
  OPC_CheckPredicate, 12,                    // 1212:       check predicate 12
  OPC_Decode, 162, 22, 152, 2,               // 1214:       decode to V6_vandnqrt using decoder 280
                                             // 1214:      }
                                             // 1214:      } // switch Inst[12:10]
                                             // 1214:     }
  1, 0,                                      // 1219:     case 0x1: {
  OPC_CheckPredicate, 13,                    // 1221:      check predicate 13
  OPC_Decode, 171, 22, 136, 2,               // 1223:      decode to V6_vaslh_acc using decoder 264
                                             // 1223:     }
                                             // 1223:     } // switch Inst[13]
                                             // 1223:    }
                                             // 1223:    } // switch Inst[5]
                                             // 1223:   }
  3, 0,                                      // 1228:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 1230:    switch Inst[13] {
  0, 7,                                      // 1233:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 1235:     check predicate 15
  OPC_Decode, 134, 25, 141, 2,               // 1237:     decode to V6_vrmpyubi using decoder 269
                                             // 1237:    }
  1, 0,                                      // 1242:    case 0x1: {
  OPC_SwitchField, 5, 1,                     // 1244:     switch Inst[5] {
  0, 7,                                      // 1247:     case 0x0: {
  OPC_CheckPredicate, 13,                    // 1249:      check predicate 13
  OPC_Decode, 180, 24, 140, 2,               // 1251:      decode to V6_vmpyh_acc using decoder 268
                                             // 1251:     }
  1, 0,                                      // 1256:     case 0x1: {
  OPC_CheckPredicate, 13,                    // 1258:      check predicate 13
  OPC_Decode, 133, 25, 147, 2,               // 1260:      decode to V6_vrmpyub_rtt_acc using decoder 275
                                             // 1260:     }
                                             // 1260:     } // switch Inst[5]
                                             // 1260:    }
                                             // 1260:    } // switch Inst[13]
                                             // 1260:   }
                                             // 1260:   } // switch Inst[7:6]
                                             // 1260:  }
  6, 223, 1,                                 // 1265:  case 0x6: {
  OPC_SwitchField, 5, 3,                     // 1268:   switch Inst[7:5] {
  0, 11,                                     // 1271:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1273:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 1275:    check Inst[13] == 0x0
  OPC_Decode, 209, 24, 139, 2,               // 1279:    decode to V6_vmpyub using decoder 267
                                             // 1279:   }
  1, 57,                                     // 1284:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1286:    switch Inst[13] {
  0, 11,                                     // 1289:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 1291:     check predicate 12
  OPC_CheckField, 8, 5, 0,                   // 1293:     check Inst[12:8] == 0x0
  OPC_Decode, 216, 20, 148, 2,               // 1297:     decode to V6_lvsplath using decoder 276
                                             // 1297:    }
  1, 0,                                      // 1302:    case 0x1: {
  OPC_SwitchField, 19, 2,                    // 1304:     switch Inst[20:19] {
  0, 7,                                      // 1307:     case 0x0: {
  OPC_CheckPredicate, 16,                    // 1309:      check predicate 16
  OPC_Decode, 151, 25, 153, 2,               // 1311:      decode to V6_vrmpyzcbs_rt_acc using decoder 281
                                             // 1311:     }
  1, 7,                                      // 1316:     case 0x1: {
  OPC_CheckPredicate, 16,                    // 1318:      check predicate 16
  OPC_Decode, 153, 25, 154, 2,               // 1320:      decode to V6_vrmpyzcbs_rx_acc using decoder 282
                                             // 1320:     }
  2, 7,                                      // 1325:     case 0x2: {
  OPC_CheckPredicate, 16,                    // 1327:      check predicate 16
  OPC_Decode, 143, 25, 153, 2,               // 1329:      decode to V6_vrmpyzbub_rt_acc using decoder 281
                                             // 1329:     }
  3, 0,                                      // 1334:     case 0x3: {
  OPC_CheckPredicate, 16,                    // 1336:      check predicate 16
  OPC_Decode, 145, 25, 154, 2,               // 1338:      decode to V6_vrmpyzbub_rx_acc using decoder 282
                                             // 1338:     }
                                             // 1338:     } // switch Inst[20:19]
                                             // 1338:    }
                                             // 1338:    } // switch Inst[13]
                                             // 1338:   }
  2, 57,                                     // 1343:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 1345:    switch Inst[13] {
  0, 11,                                     // 1348:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 1350:     check predicate 12
  OPC_CheckField, 8, 5, 0,                   // 1352:     check Inst[12:8] == 0x0
  OPC_Decode, 215, 20, 148, 2,               // 1356:     decode to V6_lvsplatb using decoder 276
                                             // 1356:    }
  1, 0,                                      // 1361:    case 0x1: {
  OPC_SwitchField, 19, 2,                    // 1363:     switch Inst[20:19] {
  0, 7,                                      // 1366:     case 0x0: {
  OPC_CheckPredicate, 16,                    // 1368:      check predicate 16
  OPC_Decode, 139, 25, 153, 2,               // 1370:      decode to V6_vrmpyzbb_rt_acc using decoder 281
                                             // 1370:     }
  1, 7,                                      // 1375:     case 0x1: {
  OPC_CheckPredicate, 16,                    // 1377:      check predicate 16
  OPC_Decode, 141, 25, 154, 2,               // 1379:      decode to V6_vrmpyzbb_rx_acc using decoder 282
                                             // 1379:     }
  2, 7,                                      // 1384:     case 0x2: {
  OPC_CheckPredicate, 16,                    // 1386:      check predicate 16
  OPC_Decode, 155, 25, 153, 2,               // 1388:      decode to V6_vrmpyznb_rt_acc using decoder 281
                                             // 1388:     }
  3, 0,                                      // 1393:     case 0x3: {
  OPC_CheckPredicate, 16,                    // 1395:      check predicate 16
  OPC_Decode, 157, 25, 154, 2,               // 1397:      decode to V6_vrmpyznb_rx_acc using decoder 282
                                             // 1397:     }
                                             // 1397:     } // switch Inst[20:19]
                                             // 1397:    }
                                             // 1397:    } // switch Inst[13]
                                             // 1397:   }
  3, 35,                                     // 1402:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 1404:    switch Inst[13] {
  0, 7,                                      // 1407:    case 0x0: {
  OPC_CheckPredicate, 17,                    // 1409:     check predicate 17
  OPC_Decode, 226, 20, 135, 2,               // 1411:     decode to V6_set_qfext using decoder 263
                                             // 1411:    }
  1, 0,                                      // 1416:    case 0x1: {
  OPC_SwitchField, 19, 2,                    // 1418:     switch Inst[20:19] {
  0, 7,                                      // 1421:     case 0x0: {
  OPC_CheckPredicate, 16,                    // 1423:      check predicate 16
  OPC_Decode, 147, 25, 153, 2,               // 1425:      decode to V6_vrmpyzcb_rt_acc using decoder 281
                                             // 1425:     }
  1, 0,                                      // 1430:     case 0x1: {
  OPC_CheckPredicate, 16,                    // 1432:      check predicate 16
  OPC_Decode, 149, 25, 154, 2,               // 1434:      decode to V6_vrmpyzcb_rx_acc using decoder 282
                                             // 1434:     }
                                             // 1434:     } // switch Inst[20:19]
                                             // 1434:    }
                                             // 1434:    } // switch Inst[13]
                                             // 1434:   }
  4, 11,                                     // 1439:   case 0x4: {
  OPC_CheckPredicate, 13,                    // 1441:    check predicate 13
  OPC_CheckField, 13, 1, 0,                  // 1443:    check Inst[13] == 0x0
  OPC_Decode, 132, 25, 155, 2,               // 1447:    decode to V6_vrmpyub_rtt using decoder 283
                                             // 1447:   }
  5, 11,                                     // 1452:   case 0x5: {
  OPC_CheckPredicate, 13,                    // 1454:    check predicate 13
  OPC_CheckField, 13, 1, 0,                  // 1456:    check Inst[13] == 0x0
  OPC_Decode, 248, 24, 155, 2,               // 1460:    decode to V6_vrmpybub_rtt using decoder 283
                                             // 1460:   }
  6, 11,                                     // 1465:   case 0x6: {
  OPC_CheckPredicate, 17,                    // 1467:    check predicate 17
  OPC_CheckField, 13, 1, 0,                  // 1469:    check Inst[13] == 0x0
  OPC_Decode, 214, 20, 136, 2,               // 1473:    decode to V6_get_qfext_oracc using decoder 264
                                             // 1473:   }
  7, 0,                                      // 1478:   case 0x7: {
  OPC_CheckPredicate, 17,                    // 1480:    check predicate 17
  OPC_CheckField, 13, 1, 0,                  // 1482:    check Inst[13] == 0x0
  OPC_Decode, 213, 20, 135, 2,               // 1486:    decode to V6_get_qfext using decoder 263
                                             // 1486:   }
                                             // 1486:   } // switch Inst[7:5]
                                             // 1486:  }
  7, 0,                                      // 1491:  case 0x7: {
  OPC_SwitchField, 5, 3,                     // 1493:   switch Inst[7:5] {
  0, 55,                                     // 1496:   case 0x0: {
  OPC_SwitchField, 19, 2,                    // 1498:    switch Inst[20:19] {
  0, 11,                                     // 1501:    case 0x0: {
  OPC_CheckPredicate, 16,                    // 1503:     check predicate 16
  OPC_CheckField, 13, 1, 0,                  // 1505:     check Inst[13] == 0x0
  OPC_Decode, 140, 25, 156, 2,               // 1509:     decode to V6_vrmpyzbb_rx using decoder 284
                                             // 1509:    }
  1, 11,                                     // 1514:    case 0x1: {
  OPC_CheckPredicate, 16,                    // 1516:     check predicate 16
  OPC_CheckField, 13, 1, 0,                  // 1518:     check Inst[13] == 0x0
  OPC_Decode, 138, 25, 157, 2,               // 1522:     decode to V6_vrmpyzbb_rt using decoder 285
                                             // 1522:    }
  2, 11,                                     // 1527:    case 0x2: {
  OPC_CheckPredicate, 16,                    // 1529:     check predicate 16
  OPC_CheckField, 13, 1, 0,                  // 1531:     check Inst[13] == 0x0
  OPC_Decode, 156, 25, 156, 2,               // 1535:     decode to V6_vrmpyznb_rx using decoder 284
                                             // 1535:    }
  3, 0,                                      // 1540:    case 0x3: {
  OPC_CheckPredicate, 16,                    // 1542:     check predicate 16
  OPC_CheckField, 13, 1, 0,                  // 1544:     check Inst[13] == 0x0
  OPC_Decode, 154, 25, 157, 2,               // 1548:     decode to V6_vrmpyznb_rt using decoder 285
                                             // 1548:    }
                                             // 1548:    } // switch Inst[20:19]
                                             // 1548:   }
  1, 35,                                     // 1553:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1555:    switch Inst[13] {
  0, 21,                                     // 1558:    case 0x0: {
  OPC_SwitchField, 19, 2,                    // 1560:     switch Inst[20:19] {
  0, 7,                                      // 1563:     case 0x0: {
  OPC_CheckPredicate, 16,                    // 1565:      check predicate 16
  OPC_Decode, 148, 25, 156, 2,               // 1567:      decode to V6_vrmpyzcb_rx using decoder 284
                                             // 1567:     }
  1, 0,                                      // 1572:     case 0x1: {
  OPC_CheckPredicate, 16,                    // 1574:      check predicate 16
  OPC_Decode, 146, 25, 157, 2,               // 1576:      decode to V6_vrmpyzcb_rt using decoder 285
                                             // 1576:     }
                                             // 1576:     } // switch Inst[20:19]
                                             // 1576:    }
  1, 0,                                      // 1581:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 1583:     check predicate 15
  OPC_Decode, 184, 25, 158, 2,               // 1585:     decode to V6_vshuff using decoder 286
                                             // 1585:    }
                                             // 1585:    } // switch Inst[13]
                                             // 1585:   }
  2, 0,                                      // 1590:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 1592:    switch Inst[13] {
  0, 39,                                     // 1595:    case 0x0: {
  OPC_SwitchField, 19, 2,                    // 1597:     switch Inst[20:19] {
  0, 7,                                      // 1600:     case 0x0: {
  OPC_CheckPredicate, 16,                    // 1602:      check predicate 16
  OPC_Decode, 152, 25, 156, 2,               // 1604:      decode to V6_vrmpyzcbs_rx using decoder 284
                                             // 1604:     }
  1, 7,                                      // 1609:     case 0x1: {
  OPC_CheckPredicate, 16,                    // 1611:      check predicate 16
  OPC_Decode, 150, 25, 157, 2,               // 1613:      decode to V6_vrmpyzcbs_rt using decoder 285
                                             // 1613:     }
  2, 7,                                      // 1618:     case 0x2: {
  OPC_CheckPredicate, 16,                    // 1620:      check predicate 16
  OPC_Decode, 144, 25, 156, 2,               // 1622:      decode to V6_vrmpyzbub_rx using decoder 284
                                             // 1622:     }
  3, 0,                                      // 1627:     case 0x3: {
  OPC_CheckPredicate, 16,                    // 1629:      check predicate 16
  OPC_Decode, 142, 25, 157, 2,               // 1631:      decode to V6_vrmpyzbub_rt using decoder 285
                                             // 1631:     }
                                             // 1631:     } // switch Inst[20:19]
                                             // 1631:    }
  1, 0,                                      // 1636:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 1638:     check predicate 15
  OPC_Decode, 253, 22, 158, 2,               // 1640:     decode to V6_vdeal using decoder 286
                                             // 1640:    }
                                             // 1640:    } // switch Inst[13]
                                             // 1640:   }
                                             // 1640:   } // switch Inst[7:5]
                                             // 1640:  }
                                             // 1640:  } // switch Inst[23:21]
                                             // 1640: }
  26, 204, 1,                                // 1645: case 0x1a: {
  OPC_SwitchField, 21, 3,                    // 1648:  switch Inst[23:21] {
  0, 79,                                     // 1651:  case 0x0: {
  OPC_SwitchField, 7, 1,                     // 1653:   switch Inst[7] {
  0, 57,                                     // 1656:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1658:    switch Inst[13] {
  0, 11,                                     // 1661:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 1663:     check predicate 15
  OPC_CheckField, 16, 5, 0,                  // 1665:     check Inst[20:16] == 0x0
  OPC_Decode, 218, 22, 159, 2,               // 1669:     decode to V6_vcmov using decoder 287
                                             // 1669:    }
  1, 0,                                      // 1674:    case 0x1: {
  OPC_SwitchField, 5, 2,                     // 1676:     switch Inst[6:5] {
  0, 7,                                      // 1679:     case 0x0: {
  OPC_CheckPredicate, 18,                    // 1681:      check predicate 18
  OPC_Decode, 204, 25, 160, 2,               // 1683:      decode to V6_vsub_sf_mix using decoder 288
                                             // 1683:     }
  1, 7,                                      // 1688:     case 0x1: {
  OPC_CheckPredicate, 17,                    // 1690:      check predicate 17
  OPC_Decode, 165, 24, 135, 2,               // 1692:      decode to V6_vmpy_rt_sf using decoder 263
                                             // 1692:     }
  2, 7,                                      // 1697:     case 0x2: {
  OPC_CheckPredicate, 17,                    // 1699:      check predicate 17
  OPC_Decode, 164, 24, 135, 2,               // 1701:      decode to V6_vmpy_rt_qf16 using decoder 263
                                             // 1701:     }
  3, 0,                                      // 1706:     case 0x3: {
  OPC_CheckPredicate, 17,                    // 1708:      check predicate 17
  OPC_Decode, 163, 24, 135, 2,               // 1710:      decode to V6_vmpy_rt_hf using decoder 263
                                             // 1710:     }
                                             // 1710:     } // switch Inst[6:5]
                                             // 1710:    }
                                             // 1710:    } // switch Inst[13]
                                             // 1710:   }
  1, 0,                                      // 1715:   case 0x1: {
  OPC_CheckPredicate, 18,                    // 1717:    check predicate 18
  OPC_CheckField, 13, 1, 1,                  // 1719:    check Inst[13] == 0x1
  OPC_CheckField, 5, 2, 0,                   // 1723:    check Inst[6:5] == 0x0
  OPC_Decode, 196, 25, 160, 2,               // 1727:    decode to V6_vsub_hf_mix using decoder 288
                                             // 1727:   }
                                             // 1727:   } // switch Inst[7]
                                             // 1727:  }
  1, 19,                                     // 1732:  case 0x1: {
  OPC_CheckPredicate, 15,                    // 1734:   check predicate 15
  OPC_CheckField, 16, 5, 0,                  // 1736:   check Inst[20:16] == 0x0
  OPC_CheckField, 13, 1, 0,                  // 1740:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 1744:   check Inst[7] == 0x0
  OPC_Decode, 226, 24, 159, 2,               // 1748:   decode to V6_vncmov using decoder 287
                                             // 1748:  }
  2, 15,                                     // 1753:  case 0x2: {
  OPC_CheckPredicate, 15,                    // 1755:   check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 1757:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 1761:   check Inst[7] == 0x0
  OPC_Decode, 225, 24, 161, 2,               // 1765:   decode to V6_vnccombine using decoder 289
                                             // 1765:  }
  3, 15,                                     // 1770:  case 0x3: {
  OPC_CheckPredicate, 15,                    // 1772:   check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 1774:   check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 1778:   check Inst[7] == 0x0
  OPC_Decode, 215, 22, 161, 2,               // 1782:   decode to V6_vccombine using decoder 289
                                             // 1782:  }
  4, 15,                                     // 1787:  case 0x4: {
  OPC_CheckPredicate, 19,                    // 1789:   check predicate 19
  OPC_CheckField, 13, 1, 1,                  // 1791:   check Inst[13] == 0x1
  OPC_CheckField, 5, 3, 7,                   // 1795:   check Inst[7:5] == 0x7
  OPC_Decode, 159, 25, 160, 2,               // 1799:   decode to V6_vrotr using decoder 288
                                             // 1799:  }
  5, 15,                                     // 1804:  case 0x5: {
  OPC_CheckPredicate, 19,                    // 1806:   check predicate 19
  OPC_CheckField, 13, 1, 1,                  // 1808:   check Inst[13] == 0x1
  OPC_CheckField, 5, 3, 7,                   // 1812:   check Inst[7:5] == 0x7
  OPC_Decode, 176, 22, 162, 2,               // 1816:   decode to V6_vasr_into using decoder 290
                                             // 1816:  }
  6, 0,                                      // 1821:  case 0x6: {
  OPC_SwitchField, 5, 3,                     // 1823:   switch Inst[7:5] {
  6, 11,                                     // 1826:   case 0x6: {
  OPC_CheckPredicate, 20,                    // 1828:    check predicate 20
  OPC_CheckField, 13, 1, 1,                  // 1830:    check Inst[13] == 0x1
  OPC_Decode, 236, 22, 160, 2,               // 1834:    decode to V6_vcvt2_b_hf using decoder 288
                                             // 1834:   }
  7, 0,                                      // 1839:   case 0x7: {
  OPC_CheckPredicate, 20,                    // 1841:    check predicate 20
  OPC_CheckField, 13, 1, 1,                  // 1843:    check Inst[13] == 0x1
  OPC_Decode, 239, 22, 160, 2,               // 1847:    decode to V6_vcvt2_ub_hf using decoder 288
                                             // 1847:   }
                                             // 1847:   } // switch Inst[7:5]
                                             // 1847:  }
                                             // 1847:  } // switch Inst[23:21]
                                             // 1847: }
  27, 177, 1,                                // 1852: case 0x1b: {
  OPC_SwitchField, 5, 3,                     // 1855:  switch Inst[7:5] {
  0, 21,                                     // 1858:  case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1860:   switch Inst[13] {
  0, 7,                                      // 1863:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1865:    check predicate 15
  OPC_Decode, 159, 22, 131, 2,               // 1867:    decode to V6_valignb using decoder 259
                                             // 1867:   }
  1, 0,                                      // 1872:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 1874:    check predicate 15
  OPC_Decode, 179, 22, 131, 2,               // 1876:    decode to V6_vasrhbrndsat using decoder 259
                                             // 1876:   }
                                             // 1876:   } // switch Inst[13]
                                             // 1876:  }
  1, 21,                                     // 1881:  case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1883:   switch Inst[13] {
  0, 7,                                      // 1886:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1888:    check predicate 15
  OPC_Decode, 231, 23, 131, 2,               // 1890:    decode to V6_vlalignb using decoder 259
                                             // 1890:   }
  1, 0,                                      // 1895:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 1897:    check predicate 15
  OPC_Decode, 239, 23, 131, 2,               // 1899:    decode to V6_vlutvvb using decoder 259
                                             // 1899:   }
                                             // 1899:   } // switch Inst[13]
                                             // 1899:  }
  2, 11,                                     // 1904:  case 0x2: {
  OPC_CheckPredicate, 15,                    // 1906:   check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 1908:   check Inst[13] == 0x0
  OPC_Decode, 194, 22, 131, 2,               // 1912:   decode to V6_vasrwh using decoder 259
                                             // 1912:  }
  3, 21,                                     // 1917:  case 0x3: {
  OPC_SwitchField, 13, 1,                    // 1919:   switch Inst[13] {
  0, 7,                                      // 1922:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1924:    check predicate 15
  OPC_Decode, 196, 22, 131, 2,               // 1926:    decode to V6_vasrwhsat using decoder 259
                                             // 1926:   }
  1, 0,                                      // 1931:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 1933:    check predicate 15
  OPC_Decode, 189, 25, 132, 2,               // 1935:    decode to V6_vshuffvdd using decoder 260
                                             // 1935:   }
                                             // 1935:   } // switch Inst[13]
                                             // 1935:  }
  4, 21,                                     // 1940:  case 0x4: {
  OPC_SwitchField, 13, 1,                    // 1942:   switch Inst[13] {
  0, 7,                                      // 1945:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1947:    check predicate 15
  OPC_Decode, 195, 22, 131, 2,               // 1949:    decode to V6_vasrwhrndsat using decoder 259
                                             // 1949:   }
  1, 0,                                      // 1954:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 1956:    check predicate 15
  OPC_Decode, 129, 23, 132, 2,               // 1958:    decode to V6_vdealvdd using decoder 260
                                             // 1958:   }
                                             // 1958:   } // switch Inst[13]
                                             // 1958:  }
  5, 21,                                     // 1963:  case 0x5: {
  OPC_SwitchField, 13, 1,                    // 1965:   switch Inst[13] {
  0, 7,                                      // 1968:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1970:    check predicate 15
  OPC_Decode, 198, 22, 131, 2,               // 1972:    decode to V6_vasrwuhsat using decoder 259
                                             // 1972:   }
  1, 0,                                      // 1977:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 1979:    check predicate 15
  OPC_Decode, 241, 23, 163, 2,               // 1981:    decode to V6_vlutvvb_oracc using decoder 291
                                             // 1981:   }
                                             // 1981:   } // switch Inst[13]
                                             // 1981:  }
  6, 21,                                     // 1986:  case 0x6: {
  OPC_SwitchField, 13, 1,                    // 1988:   switch Inst[13] {
  0, 7,                                      // 1991:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 1993:    check predicate 15
  OPC_Decode, 182, 22, 131, 2,               // 1995:    decode to V6_vasrhubsat using decoder 259
                                             // 1995:   }
  1, 0,                                      // 2000:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 2002:    check predicate 15
  OPC_Decode, 244, 23, 132, 2,               // 2004:    decode to V6_vlutvwh using decoder 260
                                             // 2004:   }
                                             // 2004:   } // switch Inst[13]
                                             // 2004:  }
  7, 0,                                      // 2009:  case 0x7: {
  OPC_SwitchField, 13, 1,                    // 2011:   switch Inst[13] {
  0, 7,                                      // 2014:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 2016:    check predicate 15
  OPC_Decode, 181, 22, 131, 2,               // 2018:    decode to V6_vasrhubrndsat using decoder 259
                                             // 2018:   }
  1, 0,                                      // 2023:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 2025:    check predicate 15
  OPC_Decode, 246, 23, 164, 2,               // 2027:    decode to V6_vlutvwh_oracc using decoder 292
                                             // 2027:   }
                                             // 2027:   } // switch Inst[13]
                                             // 2027:  }
                                             // 2027:  } // switch Inst[7:5]
                                             // 2027: }
  28, 203, 12,                               // 2032: case 0x1c: {
  OPC_SwitchField, 21, 3,                    // 2035:  switch Inst[23:21] {
  0, 187, 1,                                 // 2038:  case 0x0: {
  OPC_SwitchField, 5, 3,                     // 2041:   switch Inst[7:5] {
  0, 21,                                     // 2044:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 2046:    switch Inst[13] {
  0, 7,                                      // 2049:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2051:     check predicate 15
  OPC_Decode, 136, 25, 160, 2,               // 2053:     decode to V6_vrmpyubv using decoder 288
                                             // 2053:    }
  1, 0,                                      // 2058:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2060:     check predicate 15
  OPC_Decode, 137, 25, 165, 2,               // 2062:     decode to V6_vrmpyubv_acc using decoder 293
                                             // 2062:    }
                                             // 2062:    } // switch Inst[13]
                                             // 2062:   }
  1, 21,                                     // 2067:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 2069:    switch Inst[13] {
  0, 7,                                      // 2072:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2074:     check predicate 15
  OPC_Decode, 128, 25, 160, 2,               // 2076:     decode to V6_vrmpybv using decoder 288
                                             // 2076:    }
  1, 0,                                      // 2081:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2083:     check predicate 15
  OPC_Decode, 129, 25, 165, 2,               // 2085:     decode to V6_vrmpybv_acc using decoder 293
                                             // 2085:    }
                                             // 2085:    } // switch Inst[13]
                                             // 2085:   }
  2, 21,                                     // 2090:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 2092:    switch Inst[13] {
  0, 7,                                      // 2095:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2097:     check predicate 15
  OPC_Decode, 254, 24, 160, 2,               // 2099:     decode to V6_vrmpybusv using decoder 288
                                             // 2099:    }
  1, 0,                                      // 2104:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2106:     check predicate 15
  OPC_Decode, 255, 24, 165, 2,               // 2108:     decode to V6_vrmpybusv_acc using decoder 293
                                             // 2108:    }
                                             // 2108:    } // switch Inst[13]
                                             // 2108:   }
  3, 21,                                     // 2113:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 2115:    switch Inst[13] {
  0, 7,                                      // 2118:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2120:     check predicate 15
  OPC_Decode, 149, 23, 160, 2,               // 2122:     decode to V6_vdmpyhvsat using decoder 288
                                             // 2122:    }
  1, 0,                                      // 2127:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2129:     check predicate 15
  OPC_Decode, 150, 23, 165, 2,               // 2131:     decode to V6_vdmpyhvsat_acc using decoder 293
                                             // 2131:    }
                                             // 2131:    } // switch Inst[13]
                                             // 2131:   }
  4, 21,                                     // 2136:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 2138:    switch Inst[13] {
  0, 7,                                      // 2141:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2143:     check predicate 15
  OPC_Decode, 175, 24, 166, 2,               // 2145:     decode to V6_vmpybv using decoder 294
                                             // 2145:    }
  1, 0,                                      // 2150:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2152:     check predicate 15
  OPC_Decode, 176, 24, 162, 2,               // 2154:     decode to V6_vmpybv_acc using decoder 290
                                             // 2154:    }
                                             // 2154:    } // switch Inst[13]
                                             // 2154:   }
  5, 21,                                     // 2159:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 2161:    switch Inst[13] {
  0, 7,                                      // 2164:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2166:     check predicate 15
  OPC_Decode, 211, 24, 166, 2,               // 2168:     decode to V6_vmpyubv using decoder 294
                                             // 2168:    }
  1, 0,                                      // 2173:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2175:     check predicate 15
  OPC_Decode, 212, 24, 162, 2,               // 2177:     decode to V6_vmpyubv_acc using decoder 290
                                             // 2177:    }
                                             // 2177:    } // switch Inst[13]
                                             // 2177:   }
  6, 21,                                     // 2182:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 2184:    switch Inst[13] {
  0, 7,                                      // 2187:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2189:     check predicate 15
  OPC_Decode, 173, 24, 166, 2,               // 2191:     decode to V6_vmpybusv using decoder 294
                                             // 2191:    }
  1, 0,                                      // 2196:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2198:     check predicate 15
  OPC_Decode, 174, 24, 162, 2,               // 2200:     decode to V6_vmpybusv_acc using decoder 290
                                             // 2200:    }
                                             // 2200:    } // switch Inst[13]
                                             // 2200:   }
  7, 0,                                      // 2205:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 2207:    switch Inst[13] {
  0, 7,                                      // 2210:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2212:     check predicate 15
  OPC_Decode, 186, 24, 166, 2,               // 2214:     decode to V6_vmpyhv using decoder 294
                                             // 2214:    }
  1, 0,                                      // 2219:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2221:     check predicate 15
  OPC_Decode, 187, 24, 162, 2,               // 2223:     decode to V6_vmpyhv_acc using decoder 290
                                             // 2223:    }
                                             // 2223:    } // switch Inst[13]
                                             // 2223:   }
                                             // 2223:   } // switch Inst[7:5]
                                             // 2223:  }
  1, 187, 1,                                 // 2228:  case 0x1: {
  OPC_SwitchField, 5, 3,                     // 2231:   switch Inst[7:5] {
  0, 21,                                     // 2234:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 2236:    switch Inst[13] {
  0, 7,                                      // 2239:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2241:     check predicate 15
  OPC_Decode, 217, 24, 166, 2,               // 2243:     decode to V6_vmpyuhv using decoder 294
                                             // 2243:    }
  1, 0,                                      // 2248:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2250:     check predicate 15
  OPC_Decode, 218, 24, 162, 2,               // 2252:     decode to V6_vmpyuhv_acc using decoder 290
                                             // 2252:    }
                                             // 2252:    } // switch Inst[13]
                                             // 2252:   }
  1, 21,                                     // 2257:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 2259:    switch Inst[13] {
  0, 7,                                      // 2262:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2264:     check predicate 15
  OPC_Decode, 188, 24, 160, 2,               // 2266:     decode to V6_vmpyhvsrs using decoder 288
                                             // 2266:    }
  1, 0,                                      // 2271:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2273:     check predicate 15
  OPC_Decode, 185, 24, 162, 2,               // 2275:     decode to V6_vmpyhus_acc using decoder 290
                                             // 2275:    }
                                             // 2275:    } // switch Inst[13]
                                             // 2275:   }
  2, 21,                                     // 2280:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 2282:    switch Inst[13] {
  0, 7,                                      // 2285:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2287:     check predicate 15
  OPC_Decode, 184, 24, 166, 2,               // 2289:     decode to V6_vmpyhus using decoder 294
                                             // 2289:    }
  1, 0,                                      // 2294:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 2296:     check predicate 12
  OPC_Decode, 140, 22, 162, 2,               // 2298:     decode to V6_vaddhw_acc using decoder 290
                                             // 2298:    }
                                             // 2298:    } // switch Inst[13]
                                             // 2298:   }
  3, 21,                                     // 2303:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 2305:    switch Inst[13] {
  0, 7,                                      // 2308:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2310:     check predicate 15
  OPC_Decode, 140, 24, 167, 2,               // 2312:     decode to V6_vmpabusv using decoder 295
                                             // 2312:    }
  1, 0,                                      // 2317:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 2319:     check predicate 12
  OPC_Decode, 205, 24, 162, 2,               // 2321:     decode to V6_vmpyowh_64_acc using decoder 290
                                             // 2321:    }
                                             // 2321:    } // switch Inst[13]
                                             // 2321:   }
  4, 21,                                     // 2326:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 2328:    switch Inst[13] {
  0, 7,                                      // 2331:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2333:     check predicate 15
  OPC_Decode, 193, 24, 160, 2,               // 2335:     decode to V6_vmpyih using decoder 288
                                             // 2335:    }
  1, 0,                                      // 2340:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2342:     check predicate 15
  OPC_Decode, 194, 24, 165, 2,               // 2344:     decode to V6_vmpyih_acc using decoder 293
                                             // 2344:    }
                                             // 2344:    } // switch Inst[13]
                                             // 2344:   }
  5, 21,                                     // 2349:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 2351:    switch Inst[13] {
  0, 7,                                      // 2354:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2356:     check predicate 15
  OPC_Decode, 161, 22, 160, 2,               // 2358:     decode to V6_vand using decoder 288
                                             // 2358:    }
  1, 0,                                      // 2363:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2365:     check predicate 15
  OPC_Decode, 192, 24, 165, 2,               // 2367:     decode to V6_vmpyiewuh_acc using decoder 293
                                             // 2367:    }
                                             // 2367:    } // switch Inst[13]
                                             // 2367:   }
  6, 21,                                     // 2372:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 2374:    switch Inst[13] {
  0, 7,                                      // 2377:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2379:     check predicate 15
  OPC_Decode, 234, 24, 160, 2,               // 2381:     decode to V6_vor using decoder 288
                                             // 2381:    }
  1, 0,                                      // 2386:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2388:     check predicate 15
  OPC_Decode, 208, 24, 165, 2,               // 2390:     decode to V6_vmpyowh_sacc using decoder 293
                                             // 2390:    }
                                             // 2390:    } // switch Inst[13]
                                             // 2390:   }
  7, 0,                                      // 2395:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 2397:    switch Inst[13] {
  0, 7,                                      // 2400:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2402:     check predicate 15
  OPC_Decode, 129, 26, 160, 2,               // 2404:     decode to V6_vxor using decoder 288
                                             // 2404:    }
  1, 0,                                      // 2409:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2411:     check predicate 15
  OPC_Decode, 207, 24, 165, 2,               // 2413:     decode to V6_vmpyowh_rnd_sacc using decoder 293
                                             // 2413:    }
                                             // 2413:    } // switch Inst[13]
                                             // 2413:   }
                                             // 2413:   } // switch Inst[7:5]
                                             // 2413:  }
  2, 167, 1,                                 // 2418:  case 0x2: {
  OPC_SwitchField, 5, 3,                     // 2421:   switch Inst[7:5] {
  0, 21,                                     // 2424:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 2426:    switch Inst[13] {
  0, 7,                                      // 2429:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2431:     check predicate 15
  OPC_Decode, 152, 22, 160, 2,               // 2433:     decode to V6_vaddw using decoder 288
                                             // 2433:    }
  1, 0,                                      // 2438:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 2440:     check predicate 15
  OPC_Decode, 190, 24, 165, 2,               // 2442:     decode to V6_vmpyiewh_acc using decoder 293
                                             // 2442:    }
                                             // 2442:    } // switch Inst[13]
                                             // 2442:   }
  1, 21,                                     // 2447:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 2449:    switch Inst[13] {
  0, 7,                                      // 2452:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2454:     check predicate 15
  OPC_Decode, 143, 22, 160, 2,               // 2456:     decode to V6_vaddubsat using decoder 288
                                             // 2456:    }
  1, 0,                                      // 2461:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2463:     check predicate 21
  OPC_Decode, 169, 24, 162, 2,               // 2465:     decode to V6_vmpy_sf_hf_acc using decoder 290
                                             // 2465:    }
                                             // 2465:    } // switch Inst[13]
                                             // 2465:   }
  2, 21,                                     // 2470:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 2472:    switch Inst[13] {
  0, 7,                                      // 2475:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2477:     check predicate 15
  OPC_Decode, 146, 22, 160, 2,               // 2479:     decode to V6_vadduhsat using decoder 288
                                             // 2479:    }
  1, 0,                                      // 2484:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2486:     check predicate 21
  OPC_Decode, 154, 24, 165, 2,               // 2488:     decode to V6_vmpy_hf_hf_acc using decoder 293
                                             // 2488:    }
                                             // 2488:    } // switch Inst[13]
                                             // 2488:   }
  3, 21,                                     // 2493:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 2495:    switch Inst[13] {
  0, 7,                                      // 2498:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2500:     check predicate 15
  OPC_Decode, 137, 22, 160, 2,               // 2502:     decode to V6_vaddhsat using decoder 288
                                             // 2502:    }
  1, 0,                                      // 2507:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2509:     check predicate 21
  OPC_Decode, 132, 23, 165, 2,               // 2511:     decode to V6_vdmpy_sf_hf_acc using decoder 293
                                             // 2511:    }
                                             // 2511:    } // switch Inst[13]
                                             // 2511:   }
  4, 21,                                     // 2516:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 2518:    switch Inst[13] {
  0, 7,                                      // 2521:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2523:     check predicate 15
  OPC_Decode, 156, 22, 160, 2,               // 2525:     decode to V6_vaddwsat using decoder 288
                                             // 2525:    }
  1, 0,                                      // 2530:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 2532:     check predicate 12
  OPC_Decode, 149, 22, 162, 2,               // 2534:     decode to V6_vadduhw_acc using decoder 290
                                             // 2534:    }
                                             // 2534:    } // switch Inst[13]
                                             // 2534:   }
  5, 21,                                     // 2539:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 2541:    switch Inst[13] {
  0, 7,                                      // 2544:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2546:     check predicate 15
  OPC_Decode, 206, 25, 160, 2,               // 2548:     decode to V6_vsubb using decoder 288
                                             // 2548:    }
  1, 0,                                      // 2553:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 2555:     check predicate 12
  OPC_Decode, 142, 22, 162, 2,               // 2557:     decode to V6_vaddubh_acc using decoder 290
                                             // 2557:    }
                                             // 2557:    } // switch Inst[13]
                                             // 2557:   }
  6, 11,                                     // 2562:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 2564:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 2566:    check Inst[13] == 0x0
  OPC_Decode, 214, 25, 160, 2,               // 2570:    decode to V6_vsubh using decoder 288
                                             // 2570:   }
  7, 0,                                      // 2575:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 2577:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 2579:    check Inst[13] == 0x0
  OPC_Decode, 230, 25, 160, 2,               // 2583:    decode to V6_vsubw using decoder 288
                                             // 2583:   }
                                             // 2583:   } // switch Inst[7:5]
                                             // 2583:  }
  3, 195, 1,                                 // 2588:  case 0x3: {
  OPC_SwitchField, 5, 3,                     // 2591:   switch Inst[7:5] {
  0, 21,                                     // 2594:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 2596:    switch Inst[13] {
  0, 7,                                      // 2599:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2601:     check predicate 15
  OPC_Decode, 222, 25, 160, 2,               // 2603:     decode to V6_vsububsat using decoder 288
                                             // 2603:    }
  1, 0,                                      // 2608:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2610:     check predicate 21
  OPC_Decode, 177, 23, 160, 2,               // 2612:     decode to V6_vfmin_hf using decoder 288
                                             // 2612:    }
                                             // 2612:    } // switch Inst[13]
                                             // 2612:   }
  1, 21,                                     // 2617:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 2619:    switch Inst[13] {
  0, 7,                                      // 2622:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2624:     check predicate 15
  OPC_Decode, 225, 25, 160, 2,               // 2626:     decode to V6_vsubuhsat using decoder 288
                                             // 2626:    }
  1, 0,                                      // 2631:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2633:     check predicate 21
  OPC_Decode, 178, 23, 160, 2,               // 2635:     decode to V6_vfmin_sf using decoder 288
                                             // 2635:    }
                                             // 2635:    } // switch Inst[13]
                                             // 2635:   }
  2, 21,                                     // 2640:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 2642:    switch Inst[13] {
  0, 7,                                      // 2645:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2647:     check predicate 15
  OPC_Decode, 218, 25, 160, 2,               // 2649:     decode to V6_vsubhsat using decoder 288
                                             // 2649:    }
  1, 0,                                      // 2654:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2656:     check predicate 21
  OPC_Decode, 174, 23, 160, 2,               // 2658:     decode to V6_vfmax_hf using decoder 288
                                             // 2658:    }
                                             // 2658:    } // switch Inst[13]
                                             // 2658:   }
  3, 21,                                     // 2663:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 2665:    switch Inst[13] {
  0, 7,                                      // 2668:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2670:     check predicate 15
  OPC_Decode, 234, 25, 160, 2,               // 2672:     decode to V6_vsubwsat using decoder 288
                                             // 2672:    }
  1, 0,                                      // 2677:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 2679:     check predicate 21
  OPC_Decode, 175, 23, 160, 2,               // 2681:     decode to V6_vfmax_sf using decoder 288
                                             // 2681:    }
                                             // 2681:    } // switch Inst[13]
                                             // 2681:   }
  4, 21,                                     // 2686:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 2688:    switch Inst[13] {
  0, 7,                                      // 2691:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2693:     check predicate 15
  OPC_Decode, 251, 21, 167, 2,               // 2695:     decode to V6_vaddb_dv using decoder 295
                                             // 2695:    }
  1, 0,                                      // 2700:    case 0x1: {
  OPC_CheckPredicate, 20,                    // 2702:     check predicate 20
  OPC_Decode, 176, 23, 160, 2,               // 2704:     decode to V6_vfmin_f8 using decoder 288
                                             // 2704:    }
                                             // 2704:    } // switch Inst[13]
                                             // 2704:   }
  5, 21,                                     // 2709:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 2711:    switch Inst[13] {
  0, 7,                                      // 2714:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2716:     check predicate 15
  OPC_Decode, 134, 22, 167, 2,               // 2718:     decode to V6_vaddh_dv using decoder 295
                                             // 2718:    }
  1, 0,                                      // 2723:    case 0x1: {
  OPC_CheckPredicate, 20,                    // 2725:     check predicate 20
  OPC_Decode, 173, 23, 160, 2,               // 2727:     decode to V6_vfmax_f8 using decoder 288
                                             // 2727:    }
                                             // 2727:    } // switch Inst[13]
                                             // 2727:   }
  6, 25,                                     // 2732:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 2734:    switch Inst[13] {
  0, 7,                                      // 2737:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2739:     check predicate 15
  OPC_Decode, 153, 22, 167, 2,               // 2741:     decode to V6_vaddw_dv using decoder 295
                                             // 2741:    }
  1, 0,                                      // 2746:    case 0x1: {
  OPC_CheckPredicate, 20,                    // 2748:     check predicate 20
  OPC_CheckField, 16, 5, 6,                  // 2750:     check Inst[20:16] == 0x6
  OPC_Decode, 222, 21, 168, 2,               // 2754:     decode to V6_vabs_f8 using decoder 296
                                             // 2754:    }
                                             // 2754:    } // switch Inst[13]
                                             // 2754:   }
  7, 0,                                      // 2759:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 2761:    switch Inst[13] {
  0, 7,                                      // 2764:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2766:     check predicate 15
  OPC_Decode, 144, 22, 167, 2,               // 2768:     decode to V6_vaddubsat_dv using decoder 295
                                             // 2768:    }
  1, 0,                                      // 2773:    case 0x1: {
  OPC_CheckPredicate, 20,                    // 2775:     check predicate 20
  OPC_CheckField, 16, 5, 6,                  // 2777:     check Inst[20:16] == 0x6
  OPC_Decode, 179, 23, 168, 2,               // 2781:     decode to V6_vfneg_f8 using decoder 296
                                             // 2781:    }
                                             // 2781:    } // switch Inst[13]
                                             // 2781:   }
                                             // 2781:   } // switch Inst[7:5]
                                             // 2781:  }
  4, 176, 4,                                 // 2786:  case 0x4: {
  OPC_SwitchField, 5, 3,                     // 2789:   switch Inst[7:5] {
  0, 89,                                     // 2792:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 2794:    switch Inst[13] {
  0, 7,                                      // 2797:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2799:     check predicate 15
  OPC_Decode, 147, 22, 167, 2,               // 2801:     decode to V6_vadduhsat_dv using decoder 295
                                             // 2801:    }
  1, 0,                                      // 2806:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 2808:     switch Inst[4:2] {
  0, 7,                                      // 2811:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 2813:      check predicate 15
  OPC_Decode, 154, 23, 169, 2,               // 2815:      decode to V6_veqb_and using decoder 297
                                             // 2815:     }
  1, 7,                                      // 2820:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 2822:      check predicate 15
  OPC_Decode, 158, 23, 169, 2,               // 2824:      decode to V6_veqh_and using decoder 297
                                             // 2824:     }
  2, 7,                                      // 2829:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 2831:      check predicate 15
  OPC_Decode, 170, 23, 169, 2,               // 2833:      decode to V6_veqw_and using decoder 297
                                             // 2833:     }
  3, 7,                                      // 2838:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 2840:      check predicate 18
  OPC_Decode, 166, 23, 169, 2,               // 2842:      decode to V6_veqsf_and using decoder 297
                                             // 2842:     }
  4, 7,                                      // 2847:     case 0x4: {
  OPC_CheckPredicate, 15,                    // 2849:      check predicate 15
  OPC_Decode, 189, 23, 169, 2,               // 2851:      decode to V6_vgtb_and using decoder 297
                                             // 2851:     }
  5, 7,                                      // 2856:     case 0x5: {
  OPC_CheckPredicate, 15,                    // 2858:      check predicate 15
  OPC_Decode, 197, 23, 169, 2,               // 2860:      decode to V6_vgth_and using decoder 297
                                             // 2860:     }
  6, 7,                                      // 2865:     case 0x6: {
  OPC_CheckPredicate, 15,                    // 2867:      check predicate 15
  OPC_Decode, 221, 23, 169, 2,               // 2869:      decode to V6_vgtw_and using decoder 297
                                             // 2869:     }
  7, 0,                                      // 2874:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 2876:      check predicate 18
  OPC_Decode, 162, 23, 169, 2,               // 2878:      decode to V6_veqhf_and using decoder 297
                                             // 2878:     }
                                             // 2878:     } // switch Inst[4:2]
                                             // 2878:    }
                                             // 2878:    } // switch Inst[13]
                                             // 2878:   }
  1, 71,                                     // 2883:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 2885:    switch Inst[13] {
  0, 7,                                      // 2888:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2890:     check predicate 15
  OPC_Decode, 138, 22, 167, 2,               // 2892:     decode to V6_vaddhsat_dv using decoder 295
                                             // 2892:    }
  1, 0,                                      // 2897:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 2899:     switch Inst[4:2] {
  0, 7,                                      // 2902:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 2904:      check predicate 15
  OPC_Decode, 209, 23, 169, 2,               // 2906:      decode to V6_vgtub_and using decoder 297
                                             // 2906:     }
  1, 7,                                      // 2911:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 2913:      check predicate 15
  OPC_Decode, 213, 23, 169, 2,               // 2915:      decode to V6_vgtuh_and using decoder 297
                                             // 2915:     }
  2, 7,                                      // 2920:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 2922:      check predicate 15
  OPC_Decode, 217, 23, 169, 2,               // 2924:      decode to V6_vgtuw_and using decoder 297
                                             // 2924:     }
  4, 7,                                      // 2929:     case 0x4: {
  OPC_CheckPredicate, 22,                    // 2931:      check predicate 22
  OPC_Decode, 206, 23, 169, 2,               // 2933:      decode to V6_vgtsf_or using decoder 297
                                             // 2933:     }
  5, 7,                                      // 2938:     case 0x5: {
  OPC_CheckPredicate, 22,                    // 2940:      check predicate 22
  OPC_Decode, 202, 23, 169, 2,               // 2942:      decode to V6_vgthf_or using decoder 297
                                             // 2942:     }
  6, 0,                                      // 2947:     case 0x6: {
  OPC_CheckPredicate, 23,                    // 2949:      check predicate 23
  OPC_Decode, 194, 23, 169, 2,               // 2951:      decode to V6_vgtbf_or using decoder 297
                                             // 2951:     }
                                             // 2951:     } // switch Inst[4:2]
                                             // 2951:    }
                                             // 2951:    } // switch Inst[13]
                                             // 2951:   }
  2, 89,                                     // 2956:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 2958:    switch Inst[13] {
  0, 7,                                      // 2961:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 2963:     check predicate 15
  OPC_Decode, 157, 22, 167, 2,               // 2965:     decode to V6_vaddwsat_dv using decoder 295
                                             // 2965:    }
  1, 0,                                      // 2970:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 2972:     switch Inst[4:2] {
  0, 7,                                      // 2975:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 2977:      check predicate 15
  OPC_Decode, 155, 23, 169, 2,               // 2979:      decode to V6_veqb_or using decoder 297
                                             // 2979:     }
  1, 7,                                      // 2984:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 2986:      check predicate 15
  OPC_Decode, 159, 23, 169, 2,               // 2988:      decode to V6_veqh_or using decoder 297
                                             // 2988:     }
  2, 7,                                      // 2993:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 2995:      check predicate 15
  OPC_Decode, 171, 23, 169, 2,               // 2997:      decode to V6_veqw_or using decoder 297
                                             // 2997:     }
  3, 7,                                      // 3002:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 3004:      check predicate 18
  OPC_Decode, 167, 23, 169, 2,               // 3006:      decode to V6_veqsf_or using decoder 297
                                             // 3006:     }
  4, 7,                                      // 3011:     case 0x4: {
  OPC_CheckPredicate, 15,                    // 3013:      check predicate 15
  OPC_Decode, 190, 23, 169, 2,               // 3015:      decode to V6_vgtb_or using decoder 297
                                             // 3015:     }
  5, 7,                                      // 3020:     case 0x5: {
  OPC_CheckPredicate, 15,                    // 3022:      check predicate 15
  OPC_Decode, 198, 23, 169, 2,               // 3024:      decode to V6_vgth_or using decoder 297
                                             // 3024:     }
  6, 7,                                      // 3029:     case 0x6: {
  OPC_CheckPredicate, 15,                    // 3031:      check predicate 15
  OPC_Decode, 222, 23, 169, 2,               // 3033:      decode to V6_vgtw_or using decoder 297
                                             // 3033:     }
  7, 0,                                      // 3038:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 3040:      check predicate 18
  OPC_Decode, 163, 23, 169, 2,               // 3042:      decode to V6_veqhf_or using decoder 297
                                             // 3042:     }
                                             // 3042:     } // switch Inst[4:2]
                                             // 3042:    }
                                             // 3042:    } // switch Inst[13]
                                             // 3042:   }
  3, 71,                                     // 3047:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 3049:    switch Inst[13] {
  0, 7,                                      // 3052:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3054:     check predicate 15
  OPC_Decode, 207, 25, 167, 2,               // 3056:     decode to V6_vsubb_dv using decoder 295
                                             // 3056:    }
  1, 0,                                      // 3061:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 3063:     switch Inst[4:2] {
  0, 7,                                      // 3066:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 3068:      check predicate 15
  OPC_Decode, 210, 23, 169, 2,               // 3070:      decode to V6_vgtub_or using decoder 297
                                             // 3070:     }
  1, 7,                                      // 3075:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 3077:      check predicate 15
  OPC_Decode, 214, 23, 169, 2,               // 3079:      decode to V6_vgtuh_or using decoder 297
                                             // 3079:     }
  2, 7,                                      // 3084:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 3086:      check predicate 15
  OPC_Decode, 218, 23, 169, 2,               // 3088:      decode to V6_vgtuw_or using decoder 297
                                             // 3088:     }
  4, 7,                                      // 3093:     case 0x4: {
  OPC_CheckPredicate, 22,                    // 3095:      check predicate 22
  OPC_Decode, 204, 23, 170, 2,               // 3097:      decode to V6_vgtsf using decoder 298
                                             // 3097:     }
  5, 7,                                      // 3102:     case 0x5: {
  OPC_CheckPredicate, 22,                    // 3104:      check predicate 22
  OPC_Decode, 200, 23, 170, 2,               // 3106:      decode to V6_vgthf using decoder 298
                                             // 3106:     }
  6, 0,                                      // 3111:     case 0x6: {
  OPC_CheckPredicate, 23,                    // 3113:      check predicate 23
  OPC_Decode, 192, 23, 170, 2,               // 3115:      decode to V6_vgtbf using decoder 298
                                             // 3115:     }
                                             // 3115:     } // switch Inst[4:2]
                                             // 3115:    }
                                             // 3115:    } // switch Inst[13]
                                             // 3115:   }
  4, 89,                                     // 3120:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 3122:    switch Inst[13] {
  0, 7,                                      // 3125:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3127:     check predicate 15
  OPC_Decode, 215, 25, 167, 2,               // 3129:     decode to V6_vsubh_dv using decoder 295
                                             // 3129:    }
  1, 0,                                      // 3134:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 3136:     switch Inst[4:2] {
  0, 7,                                      // 3139:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 3141:      check predicate 15
  OPC_Decode, 156, 23, 169, 2,               // 3143:      decode to V6_veqb_xor using decoder 297
                                             // 3143:     }
  1, 7,                                      // 3148:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 3150:      check predicate 15
  OPC_Decode, 160, 23, 169, 2,               // 3152:      decode to V6_veqh_xor using decoder 297
                                             // 3152:     }
  2, 7,                                      // 3157:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 3159:      check predicate 15
  OPC_Decode, 172, 23, 169, 2,               // 3161:      decode to V6_veqw_xor using decoder 297
                                             // 3161:     }
  3, 7,                                      // 3166:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 3168:      check predicate 18
  OPC_Decode, 168, 23, 169, 2,               // 3170:      decode to V6_veqsf_xor using decoder 297
                                             // 3170:     }
  4, 7,                                      // 3175:     case 0x4: {
  OPC_CheckPredicate, 15,                    // 3177:      check predicate 15
  OPC_Decode, 191, 23, 169, 2,               // 3179:      decode to V6_vgtb_xor using decoder 297
                                             // 3179:     }
  5, 7,                                      // 3184:     case 0x5: {
  OPC_CheckPredicate, 15,                    // 3186:      check predicate 15
  OPC_Decode, 199, 23, 169, 2,               // 3188:      decode to V6_vgth_xor using decoder 297
                                             // 3188:     }
  6, 7,                                      // 3193:     case 0x6: {
  OPC_CheckPredicate, 15,                    // 3195:      check predicate 15
  OPC_Decode, 223, 23, 169, 2,               // 3197:      decode to V6_vgtw_xor using decoder 297
                                             // 3197:     }
  7, 0,                                      // 3202:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 3204:      check predicate 18
  OPC_Decode, 164, 23, 169, 2,               // 3206:      decode to V6_veqhf_xor using decoder 297
                                             // 3206:     }
                                             // 3206:     } // switch Inst[4:2]
                                             // 3206:    }
                                             // 3206:    } // switch Inst[13]
                                             // 3206:   }
  5, 44,                                     // 3211:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 3213:    switch Inst[13] {
  0, 7,                                      // 3216:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3218:     check predicate 15
  OPC_Decode, 231, 25, 167, 2,               // 3220:     decode to V6_vsubw_dv using decoder 295
                                             // 3220:    }
  1, 0,                                      // 3225:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 3227:     switch Inst[4:2] {
  0, 7,                                      // 3230:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 3232:      check predicate 15
  OPC_Decode, 211, 23, 169, 2,               // 3234:      decode to V6_vgtub_xor using decoder 297
                                             // 3234:     }
  1, 7,                                      // 3239:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 3241:      check predicate 15
  OPC_Decode, 215, 23, 169, 2,               // 3243:      decode to V6_vgtuh_xor using decoder 297
                                             // 3243:     }
  2, 0,                                      // 3248:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 3250:      check predicate 15
  OPC_Decode, 219, 23, 169, 2,               // 3252:      decode to V6_vgtuw_xor using decoder 297
                                             // 3252:     }
                                             // 3252:     } // switch Inst[4:2]
                                             // 3252:    }
                                             // 3252:    } // switch Inst[13]
                                             // 3252:   }
  6, 44,                                     // 3257:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 3259:    switch Inst[13] {
  0, 7,                                      // 3262:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3264:     check predicate 15
  OPC_Decode, 223, 25, 167, 2,               // 3266:     decode to V6_vsububsat_dv using decoder 295
                                             // 3266:    }
  1, 0,                                      // 3271:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 3273:     switch Inst[4:2] {
  2, 7,                                      // 3276:     case 0x2: {
  OPC_CheckPredicate, 22,                    // 3278:      check predicate 22
  OPC_Decode, 205, 23, 169, 2,               // 3280:      decode to V6_vgtsf_and using decoder 297
                                             // 3280:     }
  3, 7,                                      // 3285:     case 0x3: {
  OPC_CheckPredicate, 22,                    // 3287:      check predicate 22
  OPC_Decode, 201, 23, 169, 2,               // 3289:      decode to V6_vgthf_and using decoder 297
                                             // 3289:     }
  4, 0,                                      // 3294:     case 0x4: {
  OPC_CheckPredicate, 23,                    // 3296:      check predicate 23
  OPC_Decode, 193, 23, 169, 2,               // 3298:      decode to V6_vgtbf_and using decoder 297
                                             // 3298:     }
                                             // 3298:     } // switch Inst[4:2]
                                             // 3298:    }
                                             // 3298:    } // switch Inst[13]
                                             // 3298:   }
  7, 0,                                      // 3303:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 3305:    switch Inst[13] {
  0, 7,                                      // 3308:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3310:     check predicate 15
  OPC_Decode, 226, 25, 167, 2,               // 3312:     decode to V6_vsubuhsat_dv using decoder 295
                                             // 3312:    }
  1, 0,                                      // 3317:    case 0x1: {
  OPC_SwitchField, 2, 3,                     // 3319:     switch Inst[4:2] {
  2, 7,                                      // 3322:     case 0x2: {
  OPC_CheckPredicate, 22,                    // 3324:      check predicate 22
  OPC_Decode, 207, 23, 169, 2,               // 3326:      decode to V6_vgtsf_xor using decoder 297
                                             // 3326:     }
  3, 7,                                      // 3331:     case 0x3: {
  OPC_CheckPredicate, 22,                    // 3333:      check predicate 22
  OPC_Decode, 203, 23, 169, 2,               // 3335:      decode to V6_vgthf_xor using decoder 297
                                             // 3335:     }
  4, 0,                                      // 3340:     case 0x4: {
  OPC_CheckPredicate, 23,                    // 3342:      check predicate 23
  OPC_Decode, 195, 23, 169, 2,               // 3344:      decode to V6_vgtbf_xor using decoder 297
                                             // 3344:     }
                                             // 3344:     } // switch Inst[4:2]
                                             // 3344:    }
                                             // 3344:    } // switch Inst[13]
                                             // 3344:   }
                                             // 3344:   } // switch Inst[7:5]
                                             // 3344:  }
  5, 113,                                    // 3349:  case 0x5: {
  OPC_SwitchField, 7, 1,                     // 3351:   switch Inst[7] {
  0, 53,                                     // 3354:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 3356:    switch Inst[13] {
  0, 39,                                     // 3359:    case 0x0: {
  OPC_SwitchField, 5, 2,                     // 3361:     switch Inst[6:5] {
  0, 7,                                      // 3364:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 3366:      check predicate 15
  OPC_Decode, 219, 25, 167, 2,               // 3368:      decode to V6_vsubhsat_dv using decoder 295
                                             // 3368:     }
  1, 7,                                      // 3373:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 3375:      check predicate 15
  OPC_Decode, 235, 25, 167, 2,               // 3377:      decode to V6_vsubwsat_dv using decoder 295
                                             // 3377:     }
  2, 7,                                      // 3382:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 3384:      check predicate 15
  OPC_Decode, 141, 22, 166, 2,               // 3386:      decode to V6_vaddubh using decoder 294
                                             // 3386:     }
  3, 0,                                      // 3391:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 3393:      check predicate 15
  OPC_Decode, 148, 22, 166, 2,               // 3395:      decode to V6_vadduhw using decoder 294
                                             // 3395:     }
                                             // 3395:     } // switch Inst[6:5]
                                             // 3395:    }
  1, 0,                                      // 3400:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 3402:     check predicate 12
  OPC_Decode, 128, 22, 171, 2,               // 3404:     decode to V6_vaddcarry using decoder 299
                                             // 3404:    }
                                             // 3404:    } // switch Inst[13]
                                             // 3404:   }
  1, 0,                                      // 3409:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 3411:    switch Inst[13] {
  0, 39,                                     // 3414:    case 0x0: {
  OPC_SwitchField, 5, 2,                     // 3416:     switch Inst[6:5] {
  0, 7,                                      // 3419:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 3421:      check predicate 15
  OPC_Decode, 139, 22, 166, 2,               // 3423:      decode to V6_vaddhw using decoder 294
                                             // 3423:     }
  1, 7,                                      // 3428:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 3430:      check predicate 15
  OPC_Decode, 221, 25, 166, 2,               // 3432:      decode to V6_vsububh using decoder 294
                                             // 3432:     }
  2, 7,                                      // 3437:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 3439:      check predicate 15
  OPC_Decode, 227, 25, 166, 2,               // 3441:      decode to V6_vsubuhw using decoder 294
                                             // 3441:     }
  3, 0,                                      // 3446:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 3448:      check predicate 15
  OPC_Decode, 220, 25, 166, 2,               // 3450:      decode to V6_vsubhw using decoder 294
                                             // 3450:     }
                                             // 3450:     } // switch Inst[6:5]
                                             // 3450:    }
  1, 0,                                      // 3455:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 3457:     check predicate 12
  OPC_Decode, 212, 25, 171, 2,               // 3459:     decode to V6_vsubcarry using decoder 299
                                             // 3459:    }
                                             // 3459:    } // switch Inst[13]
                                             // 3459:   }
                                             // 3459:   } // switch Inst[7]
                                             // 3459:  }
  6, 89,                                     // 3464:  case 0x6: {
  OPC_SwitchField, 13, 1,                    // 3466:   switch Inst[13] {
  0, 75,                                     // 3469:   case 0x0: {
  OPC_SwitchField, 5, 3,                     // 3471:    switch Inst[7:5] {
  0, 7,                                      // 3474:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3476:     check predicate 15
  OPC_Decode, 232, 21, 160, 2,               // 3478:     decode to V6_vabsdiffub using decoder 288
                                             // 3478:    }
  1, 7,                                      // 3483:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 3485:     check predicate 15
  OPC_Decode, 231, 21, 160, 2,               // 3487:     decode to V6_vabsdiffh using decoder 288
                                             // 3487:    }
  2, 7,                                      // 3492:    case 0x2: {
  OPC_CheckPredicate, 15,                    // 3494:     check predicate 15
  OPC_Decode, 233, 21, 160, 2,               // 3496:     decode to V6_vabsdiffuh using decoder 288
                                             // 3496:    }
  3, 7,                                      // 3501:    case 0x3: {
  OPC_CheckPredicate, 15,                    // 3503:     check predicate 15
  OPC_Decode, 234, 21, 160, 2,               // 3505:     decode to V6_vabsdiffw using decoder 288
                                             // 3505:    }
  4, 7,                                      // 3510:    case 0x4: {
  OPC_CheckPredicate, 15,                    // 3512:     check predicate 15
  OPC_Decode, 207, 22, 160, 2,               // 3514:     decode to V6_vavgub using decoder 288
                                             // 3514:    }
  5, 7,                                      // 3519:    case 0x5: {
  OPC_CheckPredicate, 15,                    // 3521:     check predicate 15
  OPC_Decode, 209, 22, 160, 2,               // 3523:     decode to V6_vavguh using decoder 288
                                             // 3523:    }
  6, 7,                                      // 3528:    case 0x6: {
  OPC_CheckPredicate, 15,                    // 3530:     check predicate 15
  OPC_Decode, 205, 22, 160, 2,               // 3532:     decode to V6_vavgh using decoder 288
                                             // 3532:    }
  7, 0,                                      // 3537:    case 0x7: {
  OPC_CheckPredicate, 15,                    // 3539:     check predicate 15
  OPC_Decode, 213, 22, 160, 2,               // 3541:     decode to V6_vavgw using decoder 288
                                             // 3541:    }
                                             // 3541:    } // switch Inst[7:5]
                                             // 3541:   }
  1, 0,                                      // 3546:   case 0x1: {
  OPC_CheckPredicate, 12,                    // 3548:    check predicate 12
  OPC_Decode, 242, 23, 172, 2,               // 3550:    decode to V6_vlutvvb_oracci using decoder 300
                                             // 3550:   }
                                             // 3550:   } // switch Inst[13]
                                             // 3550:  }
  7, 0,                                      // 3555:  case 0x7: {
  OPC_SwitchField, 13, 1,                    // 3557:   switch Inst[13] {
  0, 75,                                     // 3560:   case 0x0: {
  OPC_SwitchField, 5, 3,                     // 3562:    switch Inst[7:5] {
  0, 7,                                      // 3565:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 3567:     check predicate 15
  OPC_Decode, 223, 24, 160, 2,               // 3569:     decode to V6_vnavgub using decoder 288
                                             // 3569:    }
  1, 7,                                      // 3574:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 3576:     check predicate 15
  OPC_Decode, 222, 24, 160, 2,               // 3578:     decode to V6_vnavgh using decoder 288
                                             // 3578:    }
  2, 7,                                      // 3583:    case 0x2: {
  OPC_CheckPredicate, 15,                    // 3585:     check predicate 15
  OPC_Decode, 224, 24, 160, 2,               // 3587:     decode to V6_vnavgw using decoder 288
                                             // 3587:    }
  3, 7,                                      // 3592:    case 0x3: {
  OPC_CheckPredicate, 15,                    // 3594:     check predicate 15
  OPC_Decode, 208, 22, 160, 2,               // 3596:     decode to V6_vavgubrnd using decoder 288
                                             // 3596:    }
  4, 7,                                      // 3601:    case 0x4: {
  OPC_CheckPredicate, 15,                    // 3603:     check predicate 15
  OPC_Decode, 210, 22, 160, 2,               // 3605:     decode to V6_vavguhrnd using decoder 288
                                             // 3605:    }
  5, 7,                                      // 3610:    case 0x5: {
  OPC_CheckPredicate, 15,                    // 3612:     check predicate 15
  OPC_Decode, 206, 22, 160, 2,               // 3614:     decode to V6_vavghrnd using decoder 288
                                             // 3614:    }
  6, 7,                                      // 3619:    case 0x6: {
  OPC_CheckPredicate, 15,                    // 3621:     check predicate 15
  OPC_Decode, 214, 22, 160, 2,               // 3623:     decode to V6_vavgwrnd using decoder 288
                                             // 3623:    }
  7, 0,                                      // 3628:    case 0x7: {
  OPC_CheckPredicate, 15,                    // 3630:     check predicate 15
  OPC_Decode, 143, 24, 167, 2,               // 3632:     decode to V6_vmpabuuv using decoder 295
                                             // 3632:    }
                                             // 3632:    } // switch Inst[7:5]
                                             // 3632:   }
  1, 0,                                      // 3637:   case 0x1: {
  OPC_CheckPredicate, 12,                    // 3639:    check predicate 12
  OPC_Decode, 247, 23, 173, 2,               // 3641:    decode to V6_vlutvwh_oracci using decoder 301
                                             // 3641:   }
                                             // 3641:   } // switch Inst[13]
                                             // 3641:  }
                                             // 3641:  } // switch Inst[23:21]
                                             // 3641: }
  29, 219, 1,                                // 3646: case 0x1d: {
  OPC_SwitchField, 21, 3,                    // 3649:  switch Inst[23:21] {
  0, 65,                                     // 3652:  case 0x0: {
  OPC_SwitchField, 5, 3,                     // 3654:   switch Inst[7:5] {
  0, 21,                                     // 3657:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 3659:    switch Inst[13] {
  0, 7,                                      // 3662:    case 0x0: {
  OPC_CheckPredicate, 24,                    // 3664:     check predicate 24
  OPC_Decode, 191, 22, 174, 2,               // 3666:     decode to V6_vasrvwuhsat using decoder 302
                                             // 3666:    }
  1, 0,                                      // 3671:    case 0x1: {
  OPC_CheckPredicate, 25,                    // 3673:     check predicate 25
  OPC_Decode, 167, 24, 162, 2,               // 3675:     decode to V6_vmpy_sf_bf_acc using decoder 290
                                             // 3675:    }
                                             // 3675:    } // switch Inst[13]
                                             // 3675:   }
  1, 11,                                     // 3680:   case 0x1: {
  OPC_CheckPredicate, 24,                    // 3682:    check predicate 24
  OPC_CheckField, 13, 1, 0,                  // 3684:    check Inst[13] == 0x0
  OPC_Decode, 190, 22, 174, 2,               // 3688:    decode to V6_vasrvwuhrndsat using decoder 302
                                             // 3688:   }
  2, 11,                                     // 3693:   case 0x2: {
  OPC_CheckPredicate, 24,                    // 3695:    check predicate 24
  OPC_CheckField, 13, 1, 0,                  // 3697:    check Inst[13] == 0x0
  OPC_Decode, 189, 22, 174, 2,               // 3701:    decode to V6_vasrvuhubsat using decoder 302
                                             // 3701:   }
  3, 0,                                      // 3706:   case 0x3: {
  OPC_CheckPredicate, 24,                    // 3708:    check predicate 24
  OPC_CheckField, 13, 1, 0,                  // 3710:    check Inst[13] == 0x0
  OPC_Decode, 188, 22, 174, 2,               // 3714:    decode to V6_vasrvuhubrndsat using decoder 302
                                             // 3714:   }
                                             // 3714:   } // switch Inst[7:5]
                                             // 3714:  }
  2, 81,                                     // 3719:  case 0x2: {
  OPC_SwitchField, 5, 3,                     // 3721:   switch Inst[7:5] {
  0, 11,                                     // 3724:   case 0x0: {
  OPC_CheckPredicate, 25,                    // 3726:    check predicate 25
  OPC_CheckField, 13, 1, 1,                  // 3728:    check Inst[13] == 0x1
  OPC_Decode, 130, 24, 160, 2,               // 3732:    decode to V6_vmin_bf using decoder 288
                                             // 3732:   }
  3, 11,                                     // 3737:   case 0x3: {
  OPC_CheckPredicate, 25,                    // 3739:    check predicate 25
  OPC_CheckField, 13, 1, 1,                  // 3741:    check Inst[13] == 0x1
  OPC_Decode, 241, 22, 160, 2,               // 3745:    decode to V6_vcvt_bf_sf using decoder 288
                                             // 3745:   }
  4, 11,                                     // 3750:   case 0x4: {
  OPC_CheckPredicate, 25,                    // 3752:    check predicate 25
  OPC_CheckField, 13, 1, 1,                  // 3754:    check Inst[13] == 0x1
  OPC_Decode, 166, 24, 166, 2,               // 3758:    decode to V6_vmpy_sf_bf using decoder 294
                                             // 3758:   }
  5, 11,                                     // 3763:   case 0x5: {
  OPC_CheckPredicate, 25,                    // 3765:    check predicate 25
  OPC_CheckField, 13, 1, 1,                  // 3767:    check Inst[13] == 0x1
  OPC_Decode, 202, 25, 166, 2,               // 3771:    decode to V6_vsub_sf_bf using decoder 294
                                             // 3771:   }
  6, 11,                                     // 3776:   case 0x6: {
  OPC_CheckPredicate, 25,                    // 3778:    check predicate 25
  OPC_CheckField, 13, 1, 1,                  // 3780:    check Inst[13] == 0x1
  OPC_Decode, 247, 21, 166, 2,               // 3784:    decode to V6_vadd_sf_bf using decoder 294
                                             // 3784:   }
  7, 0,                                      // 3789:   case 0x7: {
  OPC_CheckPredicate, 25,                    // 3791:    check predicate 25
  OPC_CheckField, 13, 1, 1,                  // 3793:    check Inst[13] == 0x1
  OPC_Decode, 249, 23, 160, 2,               // 3797:    decode to V6_vmax_bf using decoder 288
                                             // 3797:   }
                                             // 3797:   } // switch Inst[7:5]
                                             // 3797:  }
  4, 33,                                     // 3802:  case 0x4: {
  OPC_SwitchField, 7, 1,                     // 3804:   switch Inst[7] {
  0, 11,                                     // 3807:   case 0x0: {
  OPC_CheckPredicate, 19,                    // 3809:    check predicate 19
  OPC_CheckField, 13, 1, 1,                  // 3811:    check Inst[13] == 0x1
  OPC_Decode, 130, 22, 175, 2,               // 3815:    decode to V6_vaddcarrysat using decoder 303
                                             // 3815:   }
  1, 0,                                      // 3820:   case 0x1: {
  OPC_CheckPredicate, 19,                    // 3822:    check predicate 19
  OPC_CheckField, 13, 1, 1,                  // 3824:    check Inst[13] == 0x1
  OPC_CheckField, 5, 2, 3,                   // 3828:    check Inst[6:5] == 0x3
  OPC_Decode, 168, 25, 160, 2,               // 3832:    decode to V6_vsatdw using decoder 288
                                             // 3832:   }
                                             // 3832:   } // switch Inst[7]
                                             // 3832:  }
  5, 0,                                      // 3837:  case 0x5: {
  OPC_SwitchField, 7, 1,                     // 3839:   switch Inst[7] {
  0, 11,                                     // 3842:   case 0x0: {
  OPC_CheckPredicate, 19,                    // 3844:    check predicate 19
  OPC_CheckField, 13, 1, 1,                  // 3846:    check Inst[13] == 0x1
  OPC_Decode, 129, 22, 176, 2,               // 3850:    decode to V6_vaddcarryo using decoder 304
                                             // 3850:   }
  1, 0,                                      // 3855:   case 0x1: {
  OPC_CheckPredicate, 19,                    // 3857:    check predicate 19
  OPC_CheckField, 13, 1, 1,                  // 3859:    check Inst[13] == 0x1
  OPC_Decode, 213, 25, 176, 2,               // 3863:    decode to V6_vsubcarryo using decoder 304
                                             // 3863:   }
                                             // 3863:   } // switch Inst[7]
                                             // 3863:  }
                                             // 3863:  } // switch Inst[23:21]
                                             // 3863: }
  30, 227, 12,                               // 3868: case 0x1e: {
  OPC_SwitchField, 13, 1,                    // 3871:  switch Inst[13] {
  0, 204, 4,                                 // 3874:  case 0x0: {
  OPC_SwitchField, 21, 1,                    // 3877:   switch Inst[21] {
  0, 226, 3,                                 // 3880:   case 0x0: {
  OPC_SwitchField, 5, 3,                     // 3883:    switch Inst[7:5] {
  0, 155, 1,                                 // 3886:    case 0x0: {
  OPC_SwitchField, 16, 5,                    // 3889:     switch Inst[20:16] {
  0, 11,                                     // 3892:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 3894:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 3896:      check Inst[23:22] == 0x0
  OPC_Decode, 235, 21, 168, 2,               // 3900:      decode to V6_vabsh using decoder 296
                                             // 3900:     }
  1, 11,                                     // 3905:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 3907:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 3909:      check Inst[23:22] == 0x0
  OPC_Decode, 247, 25, 177, 2,               // 3913:      decode to V6_vunpackub using decoder 305
                                             // 3913:     }
  2, 11,                                     // 3918:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 3920:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 3922:      check Inst[23:22] == 0x0
  OPC_Decode, 185, 25, 168, 2,               // 3926:      decode to V6_vshuffb using decoder 296
                                             // 3926:     }
  3, 0,                                      // 3931:     case 0x3: {
  OPC_SwitchField, 2, 3,                     // 3933:      switch Inst[4:2] {
  0, 11,                                     // 3936:      case 0x0: {
  OPC_CheckPredicate, 15,                    // 3938:       check predicate 15
  OPC_CheckField, 10, 3, 0,                  // 3940:       check Inst[12:10] == 0x0
  OPC_Decode, 218, 20, 178, 2,               // 3944:       decode to V6_pred_and using decoder 306
                                             // 3944:      }
  1, 11,                                     // 3949:      case 0x1: {
  OPC_CheckPredicate, 15,                    // 3951:       check predicate 15
  OPC_CheckField, 10, 3, 0,                  // 3953:       check Inst[12:10] == 0x0
  OPC_Decode, 221, 20, 178, 2,               // 3957:       decode to V6_pred_or using decoder 306
                                             // 3957:      }
  2, 15,                                     // 3962:      case 0x2: {
  OPC_CheckPredicate, 15,                    // 3964:       check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 3966:       check Inst[23:22] == 0x0
  OPC_CheckField, 10, 3, 0,                  // 3970:       check Inst[12:10] == 0x0
  OPC_Decode, 220, 20, 179, 2,               // 3974:       decode to V6_pred_not using decoder 307
                                             // 3974:      }
  3, 11,                                     // 3979:      case 0x3: {
  OPC_CheckPredicate, 15,                    // 3981:       check predicate 15
  OPC_CheckField, 10, 3, 0,                  // 3983:       check Inst[12:10] == 0x0
  OPC_Decode, 225, 20, 178, 2,               // 3987:       decode to V6_pred_xor using decoder 306
                                             // 3987:      }
  4, 11,                                     // 3992:      case 0x4: {
  OPC_CheckPredicate, 15,                    // 3994:       check predicate 15
  OPC_CheckField, 10, 3, 0,                  // 3996:       check Inst[12:10] == 0x0
  OPC_Decode, 222, 20, 178, 2,               // 4000:       decode to V6_pred_or_n using decoder 306
                                             // 4000:      }
  5, 11,                                     // 4005:      case 0x5: {
  OPC_CheckPredicate, 15,                    // 4007:       check predicate 15
  OPC_CheckField, 10, 3, 0,                  // 4009:       check Inst[12:10] == 0x0
  OPC_Decode, 219, 20, 178, 2,               // 4013:       decode to V6_pred_and_n using decoder 306
                                             // 4013:      }
  6, 11,                                     // 4018:      case 0x6: {
  OPC_CheckPredicate, 12,                    // 4020:       check predicate 12
  OPC_CheckField, 10, 3, 0,                  // 4022:       check Inst[12:10] == 0x0
  OPC_Decode, 227, 20, 178, 2,               // 4026:       decode to V6_shuffeqh using decoder 306
                                             // 4026:      }
  7, 0,                                      // 4031:      case 0x7: {
  OPC_CheckPredicate, 12,                    // 4033:       check predicate 12
  OPC_CheckField, 10, 3, 0,                  // 4035:       check Inst[12:10] == 0x0
  OPC_Decode, 228, 20, 178, 2,               // 4039:       decode to V6_shuffeqw using decoder 306
                                             // 4039:      }
                                             // 4039:      } // switch Inst[4:2]
                                             // 4039:     }
                                             // 4039:     } // switch Inst[20:16]
                                             // 4039:    }
  1, 42,                                     // 4044:    case 0x1: {
  OPC_SwitchField, 16, 5,                    // 4046:     switch Inst[20:16] {
  0, 11,                                     // 4049:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4051:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4053:      check Inst[23:22] == 0x0
  OPC_Decode, 236, 21, 168, 2,               // 4057:      decode to V6_vabsh_sat using decoder 296
                                             // 4057:     }
  1, 11,                                     // 4062:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4064:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4066:      check Inst[23:22] == 0x0
  OPC_Decode, 248, 25, 177, 2,               // 4070:      decode to V6_vunpackuh using decoder 305
                                             // 4070:     }
  2, 0,                                      // 4075:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4077:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4079:      check Inst[23:22] == 0x0
  OPC_Decode, 130, 26, 177, 2,               // 4083:      decode to V6_vzb using decoder 305
                                             // 4083:     }
                                             // 4083:     } // switch Inst[20:16]
                                             // 4083:    }
  2, 42,                                     // 4088:    case 0x2: {
  OPC_SwitchField, 16, 5,                    // 4090:     switch Inst[20:16] {
  0, 11,                                     // 4093:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4095:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4097:      check Inst[23:22] == 0x0
  OPC_Decode, 237, 21, 168, 2,               // 4101:      decode to V6_vabsw using decoder 296
                                             // 4101:     }
  1, 11,                                     // 4106:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4108:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4110:      check Inst[23:22] == 0x0
  OPC_Decode, 243, 25, 177, 2,               // 4114:      decode to V6_vunpackb using decoder 305
                                             // 4114:     }
  2, 0,                                      // 4119:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4121:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4123:      check Inst[23:22] == 0x0
  OPC_Decode, 131, 26, 177, 2,               // 4127:      decode to V6_vzh using decoder 305
                                             // 4127:     }
                                             // 4127:     } // switch Inst[20:16]
                                             // 4127:    }
  3, 42,                                     // 4132:    case 0x3: {
  OPC_SwitchField, 16, 5,                    // 4134:     switch Inst[20:16] {
  0, 11,                                     // 4137:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4139:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4141:      check Inst[23:22] == 0x0
  OPC_Decode, 238, 21, 168, 2,               // 4145:      decode to V6_vabsw_sat using decoder 296
                                             // 4145:     }
  1, 11,                                     // 4150:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4152:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4154:      check Inst[23:22] == 0x0
  OPC_Decode, 244, 25, 177, 2,               // 4158:      decode to V6_vunpackh using decoder 305
                                             // 4158:     }
  2, 0,                                      // 4163:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4165:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4167:      check Inst[23:22] == 0x0
  OPC_Decode, 172, 25, 177, 2,               // 4171:      decode to V6_vsb using decoder 305
                                             // 4171:     }
                                             // 4171:     } // switch Inst[20:16]
                                             // 4171:    }
  4, 55,                                     // 4176:    case 0x4: {
  OPC_SwitchField, 16, 5,                    // 4178:     switch Inst[20:16] {
  0, 11,                                     // 4181:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4183:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4185:      check Inst[23:22] == 0x0
  OPC_Decode, 233, 24, 168, 2,               // 4189:      decode to V6_vnot using decoder 296
                                             // 4189:     }
  1, 11,                                     // 4194:     case 0x1: {
  OPC_CheckPredicate, 13,                    // 4196:      check predicate 13
  OPC_CheckField, 22, 2, 0,                  // 4198:      check Inst[23:22] == 0x0
  OPC_Decode, 229, 21, 168, 2,               // 4202:      decode to V6_vabsb using decoder 296
                                             // 4202:     }
  2, 11,                                     // 4207:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4209:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4211:      check Inst[23:22] == 0x0
  OPC_Decode, 182, 25, 177, 2,               // 4215:      decode to V6_vsh using decoder 305
                                             // 4215:     }
  3, 0,                                      // 4220:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 4222:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4224:      check Inst[23:22] == 0x0
  OPC_Decode, 232, 24, 168, 2,               // 4228:      decode to V6_vnormamtw using decoder 296
                                             // 4228:     }
                                             // 4228:     } // switch Inst[20:16]
                                             // 4228:    }
  5, 42,                                     // 4233:    case 0x5: {
  OPC_SwitchField, 16, 5,                    // 4235:     switch Inst[20:16] {
  1, 11,                                     // 4238:     case 0x1: {
  OPC_CheckPredicate, 13,                    // 4240:      check predicate 13
  OPC_CheckField, 22, 2, 0,                  // 4242:      check Inst[23:22] == 0x0
  OPC_Decode, 230, 21, 168, 2,               // 4246:      decode to V6_vabsb_sat using decoder 296
                                             // 4246:     }
  2, 11,                                     // 4251:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4253:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4255:      check Inst[23:22] == 0x0
  OPC_Decode, 217, 22, 168, 2,               // 4259:      decode to V6_vcl0w using decoder 296
                                             // 4259:     }
  3, 0,                                      // 4264:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 4266:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4268:      check Inst[23:22] == 0x0
  OPC_Decode, 231, 24, 168, 2,               // 4272:      decode to V6_vnormamth using decoder 296
                                             // 4272:     }
                                             // 4272:     } // switch Inst[20:16]
                                             // 4272:    }
  6, 42,                                     // 4277:    case 0x6: {
  OPC_SwitchField, 16, 5,                    // 4279:     switch Inst[20:16] {
  0, 11,                                     // 4282:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4284:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4286:      check Inst[23:22] == 0x0
  OPC_Decode, 128, 23, 168, 2,               // 4290:      decode to V6_vdealh using decoder 296
                                             // 4290:     }
  1, 11,                                     // 4295:     case 0x1: {
  OPC_CheckPredicate, 24,                    // 4297:      check predicate 24
  OPC_CheckField, 22, 2, 0,                  // 4299:      check Inst[23:22] == 0x0
  OPC_Decode, 202, 22, 168, 2,               // 4303:      decode to V6_vassign_tmp using decoder 296
                                             // 4303:     }
  2, 0,                                      // 4308:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4310:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4312:      check Inst[23:22] == 0x0
  OPC_Decode, 243, 24, 168, 2,               // 4316:      decode to V6_vpopcounth using decoder 296
                                             // 4316:     }
                                             // 4316:     } // switch Inst[20:16]
                                             // 4316:    }
  7, 0,                                      // 4321:    case 0x7: {
  OPC_SwitchField, 16, 5,                    // 4323:     switch Inst[20:16] {
  0, 11,                                     // 4326:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4328:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4330:      check Inst[23:22] == 0x0
  OPC_Decode, 254, 22, 168, 2,               // 4334:      decode to V6_vdealb using decoder 296
                                             // 4334:     }
  1, 11,                                     // 4339:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4341:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4343:      check Inst[23:22] == 0x0
  OPC_Decode, 187, 25, 168, 2,               // 4347:      decode to V6_vshuffh using decoder 296
                                             // 4347:     }
  2, 0,                                      // 4352:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4354:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4356:      check Inst[23:22] == 0x0
  OPC_Decode, 216, 22, 168, 2,               // 4360:      decode to V6_vcl0h using decoder 296
                                             // 4360:     }
                                             // 4360:     } // switch Inst[20:16]
                                             // 4360:    }
                                             // 4360:    } // switch Inst[7:5]
                                             // 4360:   }
  1, 0,                                      // 4365:   case 0x1: {
  OPC_SwitchField, 22, 2,                    // 4367:    switch Inst[23:22] {
  0, 7,                                      // 4370:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 4372:     check predicate 12
  OPC_Decode, 243, 23, 180, 2,               // 4374:     decode to V6_vlutvvbi using decoder 308
                                             // 4374:    }
  1, 7,                                      // 4379:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 4381:     check predicate 12
  OPC_Decode, 248, 23, 181, 2,               // 4383:     decode to V6_vlutvwhi using decoder 309
                                             // 4383:    }
  2, 0,                                      // 4388:    case 0x2: {
  OPC_SwitchField, 5, 3,                     // 4390:     switch Inst[7:5] {
  0, 7,                                      // 4393:     case 0x0: {
  OPC_CheckPredicate, 12,                    // 4395:      check predicate 12
  OPC_Decode, 255, 21, 167, 2,               // 4397:      decode to V6_vaddbsat_dv using decoder 295
                                             // 4397:     }
  1, 7,                                      // 4402:     case 0x1: {
  OPC_CheckPredicate, 12,                    // 4404:      check predicate 12
  OPC_Decode, 211, 25, 167, 2,               // 4406:      decode to V6_vsubbsat_dv using decoder 295
                                             // 4406:     }
  2, 7,                                      // 4411:     case 0x2: {
  OPC_CheckPredicate, 12,                    // 4413:      check predicate 12
  OPC_Decode, 151, 22, 167, 2,               // 4415:      decode to V6_vadduwsat_dv using decoder 295
                                             // 4415:     }
  3, 7,                                      // 4420:     case 0x3: {
  OPC_CheckPredicate, 12,                    // 4422:      check predicate 12
  OPC_Decode, 229, 25, 167, 2,               // 4424:      decode to V6_vsubuwsat_dv using decoder 295
                                             // 4424:     }
  4, 7,                                      // 4429:     case 0x4: {
  OPC_CheckPredicate, 12,                    // 4431:      check predicate 12
  OPC_Decode, 145, 22, 160, 2,               // 4433:      decode to V6_vaddububb_sat using decoder 288
                                             // 4433:     }
  5, 7,                                      // 4438:     case 0x5: {
  OPC_CheckPredicate, 12,                    // 4440:      check predicate 12
  OPC_Decode, 224, 25, 160, 2,               // 4442:      decode to V6_vsubububb_sat using decoder 288
                                             // 4442:     }
  6, 7,                                      // 4447:     case 0x6: {
  OPC_CheckPredicate, 12,                    // 4449:      check predicate 12
  OPC_Decode, 178, 24, 166, 2,               // 4451:      decode to V6_vmpyewuh_64 using decoder 294
                                             // 4451:     }
  7, 0,                                      // 4456:     case 0x7: {
  OPC_CheckPredicate, 24,                    // 4458:      check predicate 24
  OPC_Decode, 220, 22, 166, 2,               // 4460:      decode to V6_vcombine_tmp using decoder 294
                                             // 4460:     }
                                             // 4460:     } // switch Inst[7:5]
                                             // 4460:    }
                                             // 4460:    } // switch Inst[23:22]
                                             // 4460:   }
                                             // 4460:   } // switch Inst[21]
                                             // 4460:  }
  1, 0,                                      // 4465:  case 0x1: {
  OPC_SwitchField, 21, 1,                    // 4467:   switch Inst[21] {
  0, 216, 7,                                 // 4470:   case 0x0: {
  OPC_SwitchField, 16, 5,                    // 4473:    switch Inst[20:16] {
  0, 128, 1,                                 // 4476:    case 0x0: {
  OPC_SwitchField, 5, 3,                     // 4479:     switch Inst[7:5] {
  0, 11,                                     // 4482:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4484:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4486:      check Inst[23:22] == 0x0
  OPC_Decode, 245, 25, 182, 2,               // 4490:      decode to V6_vunpackob using decoder 310
                                             // 4490:     }
  1, 11,                                     // 4495:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4497:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4499:      check Inst[23:22] == 0x0
  OPC_Decode, 246, 25, 182, 2,               // 4503:      decode to V6_vunpackoh using decoder 310
                                             // 4503:     }
  4, 0,                                      // 4508:     case 0x4: {
  OPC_SwitchField, 9, 4,                     // 4510:      switch Inst[12:9] {
  0, 18,                                     // 4513:      case 0x0: {
  OPC_CheckPredicate, 15,                    // 4515:       check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4517:       check Inst[23:22] == 0x0
  OPC_CheckField, 8, 1, 0,                   // 4521:       check Inst[8] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4525:       check Inst[4:0] == 0x0
  OPC_Decode, 224, 23, 63,                   // 4529:       decode to V6_vhist using decoder 63
                                             // 4529:      }
  1, 35,                                     // 4533:      case 0x1: {
  OPC_SwitchField, 8, 1,                     // 4535:       switch Inst[8] {
  0, 14,                                     // 4538:       case 0x0: {
  OPC_CheckPredicate, 12,                    // 4540:        check predicate 12
  OPC_CheckField, 22, 2, 0,                  // 4542:        check Inst[23:22] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4546:        check Inst[4:0] == 0x0
  OPC_Decode, 253, 25, 63,                   // 4550:        decode to V6_vwhist256 using decoder 63
                                             // 4550:       }
  1, 0,                                      // 4554:       case 0x1: {
  OPC_CheckPredicate, 12,                    // 4556:        check predicate 12
  OPC_CheckField, 22, 2, 0,                  // 4558:        check Inst[23:22] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4562:        check Inst[4:0] == 0x0
  OPC_Decode, 254, 25, 63,                   // 4566:        decode to V6_vwhist256_sat using decoder 63
                                             // 4566:       }
                                             // 4566:       } // switch Inst[8]
                                             // 4566:      }
  2, 18,                                     // 4570:      case 0x2: {
  OPC_CheckPredicate, 12,                    // 4572:       check predicate 12
  OPC_CheckField, 22, 2, 0,                  // 4574:       check Inst[23:22] == 0x0
  OPC_CheckField, 8, 1, 0,                   // 4578:       check Inst[8] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4582:       check Inst[4:0] == 0x0
  OPC_Decode, 249, 25, 63,                   // 4586:       decode to V6_vwhist128 using decoder 63
                                             // 4586:      }
  3, 0,                                      // 4590:      case 0x3: {
  OPC_CheckPredicate, 12,                    // 4592:       check predicate 12
  OPC_CheckField, 22, 2, 0,                  // 4594:       check Inst[23:22] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4598:       check Inst[4:0] == 0x0
  OPC_Decode, 250, 25, 183, 2,               // 4602:       decode to V6_vwhist128m using decoder 311
                                             // 4602:      }
                                             // 4602:      } // switch Inst[12:9]
                                             // 4602:     }
                                             // 4602:     } // switch Inst[7:5]
                                             // 4602:    }
  1, 75,                                     // 4607:    case 0x1: {
  OPC_SwitchField, 5, 3,                     // 4609:     switch Inst[7:5] {
  0, 7,                                      // 4612:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4614:      check predicate 15
  OPC_Decode, 253, 21, 184, 2,               // 4616:      decode to V6_vaddbq using decoder 312
                                             // 4616:     }
  1, 7,                                      // 4621:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4623:      check predicate 15
  OPC_Decode, 136, 22, 184, 2,               // 4625:      decode to V6_vaddhq using decoder 312
                                             // 4625:     }
  2, 7,                                      // 4630:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4632:      check predicate 15
  OPC_Decode, 155, 22, 184, 2,               // 4634:      decode to V6_vaddwq using decoder 312
                                             // 4634:     }
  3, 7,                                      // 4639:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 4641:      check predicate 15
  OPC_Decode, 252, 21, 184, 2,               // 4643:      decode to V6_vaddbnq using decoder 312
                                             // 4643:     }
  4, 7,                                      // 4648:     case 0x4: {
  OPC_CheckPredicate, 15,                    // 4650:      check predicate 15
  OPC_Decode, 135, 22, 184, 2,               // 4652:      decode to V6_vaddhnq using decoder 312
                                             // 4652:     }
  5, 7,                                      // 4657:     case 0x5: {
  OPC_CheckPredicate, 15,                    // 4659:      check predicate 15
  OPC_Decode, 154, 22, 184, 2,               // 4661:      decode to V6_vaddwnq using decoder 312
                                             // 4661:     }
  6, 7,                                      // 4666:     case 0x6: {
  OPC_CheckPredicate, 15,                    // 4668:      check predicate 15
  OPC_Decode, 209, 25, 184, 2,               // 4670:      decode to V6_vsubbq using decoder 312
                                             // 4670:     }
  7, 0,                                      // 4675:     case 0x7: {
  OPC_CheckPredicate, 15,                    // 4677:      check predicate 15
  OPC_Decode, 217, 25, 184, 2,               // 4679:      decode to V6_vsubhq using decoder 312
                                             // 4679:     }
                                             // 4679:     } // switch Inst[7:5]
                                             // 4679:    }
  2, 122,                                    // 4684:    case 0x2: {
  OPC_SwitchField, 5, 3,                     // 4686:     switch Inst[7:5] {
  0, 7,                                      // 4689:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 4691:      check predicate 15
  OPC_Decode, 233, 25, 184, 2,               // 4693:      decode to V6_vsubwq using decoder 312
                                             // 4693:     }
  1, 7,                                      // 4698:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 4700:      check predicate 15
  OPC_Decode, 208, 25, 184, 2,               // 4702:      decode to V6_vsubbnq using decoder 312
                                             // 4702:     }
  2, 7,                                      // 4707:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 4709:      check predicate 15
  OPC_Decode, 216, 25, 184, 2,               // 4711:      decode to V6_vsubhnq using decoder 312
                                             // 4711:     }
  3, 7,                                      // 4716:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 4718:      check predicate 15
  OPC_Decode, 232, 25, 184, 2,               // 4720:      decode to V6_vsubwnq using decoder 312
                                             // 4720:     }
  4, 0,                                      // 4725:     case 0x4: {
  OPC_SwitchField, 9, 4,                     // 4727:      switch Inst[12:9] {
  0, 15,                                     // 4730:      case 0x0: {
  OPC_CheckPredicate, 15,                    // 4732:       check predicate 15
  OPC_CheckField, 8, 1, 0,                   // 4734:       check Inst[8] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4738:       check Inst[4:0] == 0x0
  OPC_Decode, 225, 23, 185, 2,               // 4742:       decode to V6_vhistq using decoder 313
                                             // 4742:      }
  1, 29,                                     // 4747:      case 0x1: {
  OPC_SwitchField, 8, 1,                     // 4749:       switch Inst[8] {
  0, 11,                                     // 4752:       case 0x0: {
  OPC_CheckPredicate, 12,                    // 4754:        check predicate 12
  OPC_CheckField, 0, 5, 0,                   // 4756:        check Inst[4:0] == 0x0
  OPC_Decode, 255, 25, 185, 2,               // 4760:        decode to V6_vwhist256q using decoder 313
                                             // 4760:       }
  1, 0,                                      // 4765:       case 0x1: {
  OPC_CheckPredicate, 12,                    // 4767:        check predicate 12
  OPC_CheckField, 0, 5, 0,                   // 4769:        check Inst[4:0] == 0x0
  OPC_Decode, 128, 26, 185, 2,               // 4773:        decode to V6_vwhist256q_sat using decoder 313
                                             // 4773:       }
                                             // 4773:       } // switch Inst[8]
                                             // 4773:      }
  2, 15,                                     // 4778:      case 0x2: {
  OPC_CheckPredicate, 12,                    // 4780:       check predicate 12
  OPC_CheckField, 8, 1, 0,                   // 4782:       check Inst[8] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 4786:       check Inst[4:0] == 0x0
  OPC_Decode, 251, 25, 185, 2,               // 4790:       decode to V6_vwhist128q using decoder 313
                                             // 4790:      }
  3, 0,                                      // 4795:      case 0x3: {
  OPC_CheckPredicate, 12,                    // 4797:       check predicate 12
  OPC_CheckField, 0, 5, 0,                   // 4799:       check Inst[4:0] == 0x0
  OPC_Decode, 252, 25, 186, 2,               // 4803:       decode to V6_vwhist128qm using decoder 314
                                             // 4803:      }
                                             // 4803:      } // switch Inst[12:9]
                                             // 4803:     }
                                             // 4803:     } // switch Inst[7:5]
                                             // 4803:    }
  3, 66,                                     // 4808:    case 0x3: {
  OPC_SwitchField, 5, 3,                     // 4810:     switch Inst[7:5] {
  0, 7,                                      // 4813:     case 0x0: {
  OPC_CheckPredicate, 12,                    // 4815:      check predicate 12
  OPC_Decode, 167, 22, 187, 2,               // 4817:      decode to V6_vandvqv using decoder 315
                                             // 4817:     }
  1, 7,                                      // 4822:     case 0x1: {
  OPC_CheckPredicate, 12,                    // 4824:      check predicate 12
  OPC_Decode, 166, 22, 187, 2,               // 4826:      decode to V6_vandvnqv using decoder 315
                                             // 4826:     }
  2, 30,                                     // 4831:     case 0x2: {
  OPC_SwitchField, 8, 5,                     // 4833:      switch Inst[12:8] {
  0, 7,                                      // 4836:      case 0x0: {
  OPC_CheckPredicate, 13,                    // 4838:       check predicate 13
  OPC_Decode, 244, 24, 188, 2,               // 4840:       decode to V6_vprefixqb using decoder 316
                                             // 4840:      }
  1, 7,                                      // 4845:      case 0x1: {
  OPC_CheckPredicate, 13,                    // 4847:       check predicate 13
  OPC_Decode, 245, 24, 188, 2,               // 4849:       decode to V6_vprefixqh using decoder 316
                                             // 4849:      }
  2, 0,                                      // 4854:      case 0x2: {
  OPC_CheckPredicate, 13,                    // 4856:       check predicate 13
  OPC_Decode, 246, 24, 188, 2,               // 4858:       decode to V6_vprefixqw using decoder 316
                                             // 4858:      }
                                             // 4858:      } // switch Inst[12:8]
                                             // 4858:     }
  7, 0,                                      // 4863:     case 0x7: {
  OPC_CheckPredicate, 15,                    // 4865:      check predicate 15
  OPC_CheckField, 22, 2, 0,                  // 4867:      check Inst[23:22] == 0x0
  OPC_Decode, 200, 22, 168, 2,               // 4871:      decode to V6_vassign using decoder 296
                                             // 4871:     }
                                             // 4871:     } // switch Inst[7:5]
                                             // 4871:    }
  4, 107,                                    // 4876:    case 0x4: {
  OPC_SwitchField, 5, 3,                     // 4878:     switch Inst[7:5] {
  0, 11,                                     // 4881:     case 0x0: {
  OPC_CheckPredicate, 26,                    // 4883:      check predicate 26
  OPC_CheckField, 22, 2, 0,                  // 4885:      check Inst[23:22] == 0x0
  OPC_Decode, 233, 22, 168, 2,               // 4889:      decode to V6_vconv_sf_qf32 using decoder 296
                                             // 4889:     }
  1, 11,                                     // 4894:     case 0x1: {
  OPC_CheckPredicate, 21,                    // 4896:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 4898:      check Inst[23:22] == 0x0
  OPC_Decode, 248, 22, 177, 2,               // 4902:      decode to V6_vcvt_hf_ub using decoder 305
                                             // 4902:     }
  2, 11,                                     // 4907:     case 0x2: {
  OPC_CheckPredicate, 21,                    // 4909:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 4911:      check Inst[23:22] == 0x0
  OPC_Decode, 244, 22, 177, 2,               // 4915:      decode to V6_vcvt_hf_b using decoder 305
                                             // 4915:     }
  3, 11,                                     // 4920:     case 0x3: {
  OPC_CheckPredicate, 26,                    // 4922:      check predicate 26
  OPC_CheckField, 22, 2, 0,                  // 4924:      check Inst[23:22] == 0x0
  OPC_Decode, 226, 22, 168, 2,               // 4928:      decode to V6_vconv_hf_qf16 using decoder 296
                                             // 4928:     }
  4, 11,                                     // 4933:     case 0x4: {
  OPC_CheckPredicate, 21,                    // 4935:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 4937:      check Inst[23:22] == 0x0
  OPC_Decode, 250, 22, 177, 2,               // 4941:      decode to V6_vcvt_sf_hf using decoder 305
                                             // 4941:     }
  5, 11,                                     // 4946:     case 0x5: {
  OPC_CheckPredicate, 21,                    // 4948:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 4950:      check Inst[23:22] == 0x0
  OPC_Decode, 249, 22, 168, 2,               // 4954:      decode to V6_vcvt_hf_uh using decoder 296
                                             // 4954:     }
  6, 11,                                     // 4959:     case 0x6: {
  OPC_CheckPredicate, 26,                    // 4961:      check predicate 26
  OPC_CheckField, 22, 2, 0,                  // 4963:      check Inst[23:22] == 0x0
  OPC_Decode, 227, 22, 189, 2,               // 4967:      decode to V6_vconv_hf_qf32 using decoder 317
                                             // 4967:     }
  7, 0,                                      // 4972:     case 0x7: {
  OPC_CheckPredicate, 21,                    // 4974:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 4976:      check Inst[23:22] == 0x0
  OPC_Decode, 246, 22, 168, 2,               // 4980:      decode to V6_vcvt_hf_h using decoder 296
                                             // 4980:     }
                                             // 4980:     } // switch Inst[7:5]
                                             // 4980:    }
  5, 81,                                     // 4985:    case 0x5: {
  OPC_SwitchField, 5, 3,                     // 4987:     switch Inst[7:5] {
  0, 11,                                     // 4990:     case 0x0: {
  OPC_CheckPredicate, 21,                    // 4992:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 4994:      check Inst[23:22] == 0x0
  OPC_Decode, 252, 22, 168, 2,               // 4998:      decode to V6_vcvt_uh_hf using decoder 296
                                             // 4998:     }
  1, 11,                                     // 5003:     case 0x1: {
  OPC_CheckPredicate, 27,                    // 5005:      check predicate 27
  OPC_CheckField, 22, 2, 0,                  // 5007:      check Inst[23:22] == 0x0
  OPC_Decode, 235, 22, 168, 2,               // 5011:      decode to V6_vconv_w_sf using decoder 296
                                             // 5011:     }
  2, 11,                                     // 5016:     case 0x2: {
  OPC_CheckPredicate, 27,                    // 5018:      check predicate 27
  OPC_CheckField, 22, 2, 0,                  // 5020:      check Inst[23:22] == 0x0
  OPC_Decode, 223, 22, 168, 2,               // 5024:      decode to V6_vconv_h_hf using decoder 296
                                             // 5024:     }
  3, 11,                                     // 5029:     case 0x3: {
  OPC_CheckPredicate, 27,                    // 5031:      check predicate 27
  OPC_CheckField, 22, 2, 0,                  // 5033:      check Inst[23:22] == 0x0
  OPC_Decode, 234, 22, 168, 2,               // 5037:      decode to V6_vconv_sf_w using decoder 296
                                             // 5037:     }
  4, 11,                                     // 5042:     case 0x4: {
  OPC_CheckPredicate, 27,                    // 5044:      check predicate 27
  OPC_CheckField, 22, 2, 0,                  // 5046:      check Inst[23:22] == 0x0
  OPC_Decode, 225, 22, 168, 2,               // 5050:      decode to V6_vconv_hf_h using decoder 296
                                             // 5050:     }
  5, 0,                                      // 5055:     case 0x5: {
  OPC_CheckPredicate, 20,                    // 5057:      check predicate 20
  OPC_CheckField, 22, 2, 0,                  // 5059:      check Inst[23:22] == 0x0
  OPC_Decode, 245, 22, 177, 2,               // 5063:      decode to V6_vcvt_hf_f8 using decoder 305
                                             // 5063:     }
                                             // 5063:     } // switch Inst[7:5]
                                             // 5063:    }
  6, 107,                                    // 5068:    case 0x6: {
  OPC_SwitchField, 5, 3,                     // 5070:     switch Inst[7:5] {
  0, 11,                                     // 5073:     case 0x0: {
  OPC_CheckPredicate, 21,                    // 5075:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 5077:      check Inst[23:22] == 0x0
  OPC_Decode, 243, 22, 168, 2,               // 5081:      decode to V6_vcvt_h_hf using decoder 296
                                             // 5081:     }
  1, 11,                                     // 5086:     case 0x1: {
  OPC_CheckPredicate, 21,                    // 5088:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 5090:      check Inst[23:22] == 0x0
  OPC_Decode, 201, 22, 168, 2,               // 5094:      decode to V6_vassign_fp using decoder 296
                                             // 5094:     }
  2, 11,                                     // 5099:     case 0x2: {
  OPC_CheckPredicate, 21,                    // 5101:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 5103:      check Inst[23:22] == 0x0
  OPC_Decode, 180, 23, 168, 2,               // 5107:      decode to V6_vfneg_hf using decoder 296
                                             // 5107:     }
  3, 11,                                     // 5112:     case 0x3: {
  OPC_CheckPredicate, 21,                    // 5114:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 5116:      check Inst[23:22] == 0x0
  OPC_Decode, 181, 23, 168, 2,               // 5120:      decode to V6_vfneg_sf using decoder 296
                                             // 5120:     }
  4, 11,                                     // 5125:     case 0x4: {
  OPC_CheckPredicate, 21,                    // 5127:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 5129:      check Inst[23:22] == 0x0
  OPC_Decode, 223, 21, 168, 2,               // 5133:      decode to V6_vabs_hf using decoder 296
                                             // 5133:     }
  5, 11,                                     // 5138:     case 0x5: {
  OPC_CheckPredicate, 21,                    // 5140:      check predicate 21
  OPC_CheckField, 22, 2, 0,                  // 5142:      check Inst[23:22] == 0x0
  OPC_Decode, 228, 21, 168, 2,               // 5146:      decode to V6_vabs_sf using decoder 296
                                             // 5146:     }
  6, 11,                                     // 5151:     case 0x6: {
  OPC_CheckPredicate, 14,                    // 5153:      check predicate 14
  OPC_CheckField, 22, 2, 0,                  // 5155:      check Inst[23:22] == 0x0
  OPC_Decode, 224, 22, 168, 2,               // 5159:      decode to V6_vconv_h_hf_rnd using decoder 296
                                             // 5159:     }
  7, 0,                                      // 5164:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 5166:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5168:      check Inst[23:22] == 0x0
  OPC_Decode, 221, 22, 189, 2,               // 5172:      decode to V6_vconv_bf_qf32 using decoder 317
                                             // 5172:     }
                                             // 5172:     } // switch Inst[7:5]
                                             // 5172:    }
  12, 107,                                   // 5177:    case 0xc: {
  OPC_SwitchField, 5, 3,                     // 5179:     switch Inst[7:5] {
  0, 11,                                     // 5182:     case 0x0: {
  OPC_CheckPredicate, 18,                    // 5184:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5186:      check Inst[23:22] == 0x0
  OPC_Decode, 228, 23, 168, 2,               // 5190:      decode to V6_vilog2_qf32 using decoder 296
                                             // 5190:     }
  1, 11,                                     // 5195:     case 0x1: {
  OPC_CheckPredicate, 18,                    // 5197:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5199:      check Inst[23:22] == 0x0
  OPC_Decode, 227, 23, 168, 2,               // 5203:      decode to V6_vilog2_qf16 using decoder 296
                                             // 5203:     }
  2, 11,                                     // 5208:     case 0x2: {
  OPC_CheckPredicate, 18,                    // 5210:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5212:      check Inst[23:22] == 0x0
  OPC_Decode, 229, 23, 168, 2,               // 5216:      decode to V6_vilog2_sf using decoder 296
                                             // 5216:     }
  3, 11,                                     // 5221:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 5223:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5225:      check Inst[23:22] == 0x0
  OPC_Decode, 226, 23, 168, 2,               // 5229:      decode to V6_vilog2_hf using decoder 296
                                             // 5229:     }
  4, 11,                                     // 5234:     case 0x4: {
  OPC_CheckPredicate, 18,                    // 5236:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5238:      check Inst[23:22] == 0x0
  OPC_Decode, 229, 22, 168, 2,               // 5242:      decode to V6_vconv_qf16_hf using decoder 296
                                             // 5242:     }
  5, 11,                                     // 5247:     case 0x5: {
  OPC_CheckPredicate, 18,                    // 5249:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5251:      check Inst[23:22] == 0x0
  OPC_Decode, 228, 22, 177, 2,               // 5255:      decode to V6_vconv_qf16_f8 using decoder 305
                                             // 5255:     }
  6, 11,                                     // 5260:     case 0x6: {
  OPC_CheckPredicate, 18,                    // 5262:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5264:      check Inst[23:22] == 0x0
  OPC_Decode, 230, 22, 168, 2,               // 5268:      decode to V6_vconv_qf16_qf16 using decoder 296
                                             // 5268:     }
  7, 0,                                      // 5273:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 5275:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5277:      check Inst[23:22] == 0x0
  OPC_Decode, 222, 22, 168, 2,               // 5281:      decode to V6_vconv_f8_qf16 using decoder 296
                                             // 5281:     }
                                             // 5281:     } // switch Inst[7:5]
                                             // 5281:    }
  13, 29,                                    // 5286:    case 0xd: {
  OPC_SwitchField, 5, 3,                     // 5288:     switch Inst[7:5] {
  6, 11,                                     // 5291:     case 0x6: {
  OPC_CheckPredicate, 18,                    // 5293:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5295:      check Inst[23:22] == 0x0
  OPC_Decode, 232, 22, 168, 2,               // 5299:      decode to V6_vconv_qf32_sf using decoder 296
                                             // 5299:     }
  7, 0,                                      // 5304:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 5306:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5308:      check Inst[23:22] == 0x0
  OPC_Decode, 231, 22, 168, 2,               // 5312:      decode to V6_vconv_qf32_qf32 using decoder 296
                                             // 5312:     }
                                             // 5312:     } // switch Inst[7:5]
                                             // 5312:    }
  14, 107,                                   // 5317:    case 0xe: {
  OPC_SwitchField, 5, 3,                     // 5319:     switch Inst[7:5] {
  0, 11,                                     // 5322:     case 0x0: {
  OPC_CheckPredicate, 18,                    // 5324:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5326:      check Inst[23:22] == 0x0
  OPC_Decode, 230, 24, 168, 2,               // 5330:      decode to V6_vneg_qf32_sf using decoder 296
                                             // 5330:     }
  1, 11,                                     // 5335:     case 0x1: {
  OPC_CheckPredicate, 18,                    // 5337:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5339:      check Inst[23:22] == 0x0
  OPC_Decode, 229, 24, 168, 2,               // 5343:      decode to V6_vneg_qf32_qf32 using decoder 296
                                             // 5343:     }
  2, 11,                                     // 5348:     case 0x2: {
  OPC_CheckPredicate, 18,                    // 5350:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5352:      check Inst[23:22] == 0x0
  OPC_Decode, 227, 24, 168, 2,               // 5356:      decode to V6_vneg_qf16_hf using decoder 296
                                             // 5356:     }
  3, 11,                                     // 5361:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 5363:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5365:      check Inst[23:22] == 0x0
  OPC_Decode, 228, 24, 168, 2,               // 5369:      decode to V6_vneg_qf16_qf16 using decoder 296
                                             // 5369:     }
  4, 11,                                     // 5374:     case 0x4: {
  OPC_CheckPredicate, 18,                    // 5376:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5378:      check Inst[23:22] == 0x0
  OPC_Decode, 227, 21, 168, 2,               // 5382:      decode to V6_vabs_qf32_sf using decoder 296
                                             // 5382:     }
  5, 11,                                     // 5387:     case 0x5: {
  OPC_CheckPredicate, 18,                    // 5389:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5391:      check Inst[23:22] == 0x0
  OPC_Decode, 226, 21, 168, 2,               // 5395:      decode to V6_vabs_qf32_qf32 using decoder 296
                                             // 5395:     }
  6, 11,                                     // 5400:     case 0x6: {
  OPC_CheckPredicate, 18,                    // 5402:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5404:      check Inst[23:22] == 0x0
  OPC_Decode, 224, 21, 168, 2,               // 5408:      decode to V6_vabs_qf16_hf using decoder 296
                                             // 5408:     }
  7, 0,                                      // 5413:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 5415:      check predicate 18
  OPC_CheckField, 22, 2, 0,                  // 5417:      check Inst[23:22] == 0x0
  OPC_Decode, 225, 21, 168, 2,               // 5421:      decode to V6_vabs_qf16_qf16 using decoder 296
                                             // 5421:     }
                                             // 5421:     } // switch Inst[7:5]
                                             // 5421:    }
  21, 0,                                     // 5426:    case 0x15: {
  OPC_SwitchField, 5, 3,                     // 5428:     switch Inst[7:5] {
  6, 11,                                     // 5431:     case 0x6: {
  OPC_CheckPredicate, 20,                    // 5433:      check predicate 20
  OPC_CheckField, 22, 2, 3,                  // 5435:      check Inst[23:22] == 0x3
  OPC_Decode, 237, 22, 177, 2,               // 5439:      decode to V6_vcvt2_hf_b using decoder 305
                                             // 5439:     }
  7, 0,                                      // 5444:     case 0x7: {
  OPC_CheckPredicate, 20,                    // 5446:      check predicate 20
  OPC_CheckField, 22, 2, 3,                  // 5448:      check Inst[23:22] == 0x3
  OPC_Decode, 238, 22, 177, 2,               // 5452:      decode to V6_vcvt2_hf_ub using decoder 305
                                             // 5452:     }
                                             // 5452:     } // switch Inst[7:5]
                                             // 5452:    }
                                             // 5452:    } // switch Inst[20:16]
                                             // 5452:   }
  1, 0,                                      // 5457:   case 0x1: {
  OPC_SwitchField, 22, 2,                    // 5459:    switch Inst[23:22] {
  0, 7,                                      // 5462:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5464:     check predicate 15
  OPC_Decode, 160, 22, 180, 2,               // 5466:     decode to V6_valignbi using decoder 308
                                             // 5466:    }
  1, 7,                                      // 5471:    case 0x1: {
  OPC_CheckPredicate, 15,                    // 5473:     check predicate 15
  OPC_Decode, 232, 23, 180, 2,               // 5475:     decode to V6_vlalignbi using decoder 308
                                             // 5475:    }
  2, 11,                                     // 5480:    case 0x2: {
  OPC_CheckPredicate, 15,                    // 5482:     check predicate 15
  OPC_CheckField, 7, 1, 0,                   // 5484:     check Inst[7] == 0x0
  OPC_Decode, 236, 25, 190, 2,               // 5488:     decode to V6_vswap using decoder 318
                                             // 5488:    }
  3, 0,                                      // 5493:    case 0x3: {
  OPC_CheckPredicate, 15,                    // 5495:     check predicate 15
  OPC_CheckField, 7, 1, 0,                   // 5497:     check Inst[7] == 0x0
  OPC_Decode, 220, 24, 176, 2,               // 5501:     decode to V6_vmux using decoder 304
                                             // 5501:    }
                                             // 5501:    } // switch Inst[23:22]
                                             // 5501:   }
                                             // 5501:   } // switch Inst[21]
                                             // 5501:  }
                                             // 5501:  } // switch Inst[13]
                                             // 5501: }
  31, 143, 11,                               // 5506: case 0x1f: {
  OPC_SwitchField, 21, 3,                    // 5509:  switch Inst[23:21] {
  0, 187, 1,                                 // 5512:  case 0x0: {
  OPC_SwitchField, 5, 3,                     // 5515:   switch Inst[7:5] {
  0, 21,                                     // 5518:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 5520:    switch Inst[13] {
  0, 7,                                      // 5523:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 5525:     check predicate 12
  OPC_Decode, 254, 21, 160, 2,               // 5527:     decode to V6_vaddbsat using decoder 288
                                             // 5527:    }
  1, 0,                                      // 5532:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 5534:     check predicate 12
  OPC_Decode, 131, 22, 160, 2,               // 5536:     decode to V6_vaddclbh using decoder 288
                                             // 5536:    }
                                             // 5536:    } // switch Inst[13]
                                             // 5536:   }
  1, 21,                                     // 5541:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 5543:    switch Inst[13] {
  0, 7,                                      // 5546:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5548:     check predicate 15
  OPC_Decode, 135, 24, 160, 2,               // 5550:     decode to V6_vminub using decoder 288
                                             // 5550:    }
  1, 0,                                      // 5555:    case 0x1: {
  OPC_CheckPredicate, 12,                    // 5557:     check predicate 12
  OPC_Decode, 132, 22, 160, 2,               // 5559:     decode to V6_vaddclbw using decoder 288
                                             // 5559:    }
                                             // 5559:    } // switch Inst[13]
                                             // 5559:   }
  2, 21,                                     // 5564:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 5566:    switch Inst[13] {
  0, 7,                                      // 5569:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5571:     check predicate 15
  OPC_Decode, 136, 24, 160, 2,               // 5573:     decode to V6_vminuh using decoder 288
                                             // 5573:    }
  1, 0,                                      // 5578:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 5580:     check predicate 13
  OPC_Decode, 211, 22, 160, 2,               // 5582:     decode to V6_vavguw using decoder 288
                                             // 5582:    }
                                             // 5582:    } // switch Inst[13]
                                             // 5582:   }
  3, 21,                                     // 5587:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 5589:    switch Inst[13] {
  0, 7,                                      // 5592:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5594:     check predicate 15
  OPC_Decode, 134, 24, 160, 2,               // 5596:     decode to V6_vminh using decoder 288
                                             // 5596:    }
  1, 0,                                      // 5601:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 5603:     check predicate 13
  OPC_Decode, 212, 22, 160, 2,               // 5605:     decode to V6_vavguwrnd using decoder 288
                                             // 5605:    }
                                             // 5605:    } // switch Inst[13]
                                             // 5605:   }
  4, 21,                                     // 5610:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 5612:    switch Inst[13] {
  0, 7,                                      // 5615:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5617:     check predicate 15
  OPC_Decode, 137, 24, 160, 2,               // 5619:     decode to V6_vminw using decoder 288
                                             // 5619:    }
  1, 0,                                      // 5624:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 5626:     check predicate 13
  OPC_Decode, 203, 22, 160, 2,               // 5628:     decode to V6_vavgb using decoder 288
                                             // 5628:    }
                                             // 5628:    } // switch Inst[13]
                                             // 5628:   }
  5, 21,                                     // 5633:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 5635:    switch Inst[13] {
  0, 7,                                      // 5638:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5640:     check predicate 15
  OPC_Decode, 254, 23, 160, 2,               // 5642:     decode to V6_vmaxub using decoder 288
                                             // 5642:    }
  1, 0,                                      // 5647:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 5649:     check predicate 13
  OPC_Decode, 204, 22, 160, 2,               // 5651:     decode to V6_vavgbrnd using decoder 288
                                             // 5651:    }
                                             // 5651:    } // switch Inst[13]
                                             // 5651:   }
  6, 21,                                     // 5656:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 5658:    switch Inst[13] {
  0, 7,                                      // 5661:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5663:     check predicate 15
  OPC_Decode, 255, 23, 160, 2,               // 5665:     decode to V6_vmaxuh using decoder 288
                                             // 5665:    }
  1, 0,                                      // 5670:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 5672:     check predicate 13
  OPC_Decode, 221, 24, 160, 2,               // 5674:     decode to V6_vnavgb using decoder 288
                                             // 5674:    }
                                             // 5674:    } // switch Inst[13]
                                             // 5674:   }
  7, 0,                                      // 5679:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 5681:    switch Inst[13] {
  0, 7,                                      // 5684:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5686:     check predicate 15
  OPC_Decode, 253, 23, 160, 2,               // 5688:     decode to V6_vmaxh using decoder 288
                                             // 5688:    }
  1, 0,                                      // 5693:    case 0x1: {
  OPC_CheckPredicate, 17,                    // 5695:     check predicate 17
  OPC_Decode, 129, 24, 160, 2,               // 5697:     decode to V6_vmerge_qf using decoder 288
                                             // 5697:    }
                                             // 5697:    } // switch Inst[13]
                                             // 5697:   }
                                             // 5697:   } // switch Inst[7:5]
                                             // 5697:  }
  1, 113,                                    // 5702:  case 0x1: {
  OPC_SwitchField, 7, 1,                     // 5704:   switch Inst[7] {
  0, 53,                                     // 5707:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 5709:    switch Inst[13] {
  0, 39,                                     // 5712:    case 0x0: {
  OPC_SwitchField, 5, 2,                     // 5714:     switch Inst[6:5] {
  0, 7,                                      // 5717:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 5719:      check predicate 15
  OPC_Decode, 128, 24, 160, 2,               // 5721:      decode to V6_vmaxw using decoder 288
                                             // 5721:     }
  1, 7,                                      // 5726:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 5728:      check predicate 15
  OPC_Decode, 130, 23, 160, 2,               // 5730:      decode to V6_vdelta using decoder 288
                                             // 5730:     }
  2, 7,                                      // 5735:     case 0x2: {
  OPC_CheckPredicate, 12,                    // 5737:      check predicate 12
  OPC_Decode, 210, 25, 160, 2,               // 5739:      decode to V6_vsubbsat using decoder 288
                                             // 5739:     }
  3, 0,                                      // 5744:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 5746:      check predicate 15
  OPC_Decode, 247, 24, 160, 2,               // 5748:      decode to V6_vrdelta using decoder 288
                                             // 5748:     }
                                             // 5748:     } // switch Inst[6:5]
                                             // 5748:    }
  1, 0,                                      // 5753:    case 0x1: {
  OPC_CheckPredicate, 22,                    // 5755:     check predicate 22
  OPC_Decode, 232, 20, 191, 2,               // 5757:     decode to V6_v6mpyvubs10_vxx using decoder 319
                                             // 5757:    }
                                             // 5757:    } // switch Inst[13]
                                             // 5757:   }
  1, 0,                                      // 5762:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 5764:    switch Inst[13] {
  0, 39,                                     // 5767:    case 0x0: {
  OPC_SwitchField, 5, 2,                     // 5769:     switch Inst[6:5] {
  0, 7,                                      // 5772:     case 0x0: {
  OPC_CheckPredicate, 12,                    // 5774:      check predicate 12
  OPC_Decode, 133, 24, 160, 2,               // 5776:      decode to V6_vminb using decoder 288
                                             // 5776:     }
  1, 7,                                      // 5781:     case 0x1: {
  OPC_CheckPredicate, 12,                    // 5783:      check predicate 12
  OPC_Decode, 252, 23, 160, 2,               // 5785:      decode to V6_vmaxb using decoder 288
                                             // 5785:     }
  2, 7,                                      // 5790:     case 0x2: {
  OPC_CheckPredicate, 12,                    // 5792:      check predicate 12
  OPC_Decode, 170, 25, 160, 2,               // 5794:      decode to V6_vsatuwuh using decoder 288
                                             // 5794:     }
  3, 0,                                      // 5799:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 5801:      check predicate 15
  OPC_Decode, 255, 22, 160, 2,               // 5803:      decode to V6_vdealb4w using decoder 288
                                             // 5803:     }
                                             // 5803:     } // switch Inst[6:5]
                                             // 5803:    }
  1, 0,                                      // 5808:    case 0x1: {
  OPC_CheckPredicate, 22,                    // 5810:     check predicate 22
  OPC_Decode, 230, 20, 191, 2,               // 5812:     decode to V6_v6mpyhubs10_vxx using decoder 319
                                             // 5812:    }
                                             // 5812:    } // switch Inst[13]
                                             // 5812:   }
                                             // 5812:   } // switch Inst[7]
                                             // 5812:  }
  2, 113,                                    // 5817:  case 0x2: {
  OPC_SwitchField, 7, 1,                     // 5819:   switch Inst[7] {
  0, 53,                                     // 5822:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 5824:    switch Inst[13] {
  0, 39,                                     // 5827:    case 0x0: {
  OPC_SwitchField, 5, 2,                     // 5829:     switch Inst[6:5] {
  0, 7,                                      // 5832:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 5834:      check predicate 15
  OPC_Decode, 206, 24, 160, 2,               // 5836:      decode to V6_vmpyowh_rnd using decoder 288
                                             // 5836:     }
  1, 7,                                      // 5841:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 5843:      check predicate 15
  OPC_Decode, 186, 25, 160, 2,               // 5845:      decode to V6_vshuffeb using decoder 288
                                             // 5845:     }
  2, 7,                                      // 5850:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 5852:      check predicate 15
  OPC_Decode, 188, 25, 160, 2,               // 5854:      decode to V6_vshuffob using decoder 288
                                             // 5854:     }
  3, 0,                                      // 5859:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 5861:      check predicate 15
  OPC_Decode, 183, 25, 160, 2,               // 5863:      decode to V6_vshufeh using decoder 288
                                             // 5863:     }
                                             // 5863:     } // switch Inst[6:5]
                                             // 5863:    }
  1, 0,                                      // 5868:    case 0x1: {
  OPC_CheckPredicate, 22,                    // 5870:     check predicate 22
  OPC_Decode, 231, 20, 192, 2,               // 5872:     decode to V6_v6mpyvubs10 using decoder 320
                                             // 5872:    }
                                             // 5872:    } // switch Inst[13]
                                             // 5872:   }
  1, 0,                                      // 5877:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 5879:    switch Inst[13] {
  0, 39,                                     // 5882:    case 0x0: {
  OPC_SwitchField, 5, 2,                     // 5884:     switch Inst[6:5] {
  0, 7,                                      // 5887:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 5889:      check predicate 15
  OPC_Decode, 192, 25, 160, 2,               // 5891:      decode to V6_vshufoh using decoder 288
                                             // 5891:     }
  1, 7,                                      // 5896:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 5898:      check predicate 15
  OPC_Decode, 191, 25, 166, 2,               // 5900:      decode to V6_vshufoeh using decoder 294
                                             // 5900:     }
  2, 7,                                      // 5905:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 5907:      check predicate 15
  OPC_Decode, 190, 25, 166, 2,               // 5909:      decode to V6_vshufoeb using decoder 294
                                             // 5909:     }
  3, 0,                                      // 5914:     case 0x3: {
  OPC_CheckPredicate, 15,                    // 5916:      check predicate 15
  OPC_Decode, 219, 22, 166, 2,               // 5918:      decode to V6_vcombine using decoder 294
                                             // 5918:     }
                                             // 5918:     } // switch Inst[6:5]
                                             // 5918:    }
  1, 0,                                      // 5923:    case 0x1: {
  OPC_CheckPredicate, 22,                    // 5925:     check predicate 22
  OPC_Decode, 229, 20, 192, 2,               // 5927:     decode to V6_v6mpyhubs10 using decoder 320
                                             // 5927:    }
                                             // 5927:    } // switch Inst[13]
                                             // 5927:   }
                                             // 5927:   } // switch Inst[7]
                                             // 5927:  }
  3, 187, 1,                                 // 5932:  case 0x3: {
  OPC_SwitchField, 5, 3,                     // 5935:   switch Inst[7:5] {
  0, 21,                                     // 5938:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 5940:    switch Inst[13] {
  0, 7,                                      // 5943:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5945:     check predicate 15
  OPC_Decode, 189, 24, 160, 2,               // 5947:     decode to V6_vmpyieoh using decoder 288
                                             // 5947:    }
  1, 0,                                      // 5952:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 5954:     check predicate 21
  OPC_Decode, 195, 25, 160, 2,               // 5956:     decode to V6_vsub_hf_hf using decoder 288
                                             // 5956:    }
                                             // 5956:    } // switch Inst[13]
                                             // 5956:   }
  1, 21,                                     // 5961:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 5963:    switch Inst[13] {
  0, 7,                                      // 5966:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 5968:     check predicate 12
  OPC_Decode, 150, 22, 160, 2,               // 5970:     decode to V6_vadduwsat using decoder 288
                                             // 5970:    }
  1, 0,                                      // 5975:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 5977:     check predicate 21
  OPC_Decode, 247, 22, 160, 2,               // 5979:     decode to V6_vcvt_hf_sf using decoder 288
                                             // 5979:    }
                                             // 5979:    } // switch Inst[13]
                                             // 5979:   }
  2, 21,                                     // 5984:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 5986:    switch Inst[13] {
  0, 7,                                      // 5989:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 5991:     check predicate 15
  OPC_Decode, 169, 25, 160, 2,               // 5993:     decode to V6_vsathub using decoder 288
                                             // 5993:    }
  1, 0,                                      // 5998:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6000:     check predicate 26
  OPC_Decode, 242, 21, 160, 2,               // 6002:     decode to V6_vadd_qf16 using decoder 288
                                             // 6002:    }
                                             // 6002:    } // switch Inst[13]
                                             // 6002:   }
  3, 21,                                     // 6007:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 6009:    switch Inst[13] {
  0, 7,                                      // 6012:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6014:     check predicate 15
  OPC_Decode, 171, 25, 160, 2,               // 6016:     decode to V6_vsatwh using decoder 288
                                             // 6016:    }
  1, 0,                                      // 6021:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6023:     check predicate 26
  OPC_Decode, 239, 21, 160, 2,               // 6025:     decode to V6_vadd_hf using decoder 288
                                             // 6025:    }
                                             // 6025:    } // switch Inst[13]
                                             // 6025:   }
  4, 21,                                     // 6030:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 6032:    switch Inst[13] {
  0, 7,                                      // 6035:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6037:     check predicate 15
  OPC_Decode, 164, 25, 160, 2,               // 6039:     decode to V6_vroundwh using decoder 288
                                             // 6039:    }
  1, 0,                                      // 6044:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6046:     check predicate 26
  OPC_Decode, 243, 21, 160, 2,               // 6048:     decode to V6_vadd_qf16_mix using decoder 288
                                             // 6048:    }
                                             // 6048:    } // switch Inst[13]
                                             // 6048:   }
  5, 21,                                     // 6053:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 6055:    switch Inst[13] {
  0, 7,                                      // 6058:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6060:     check predicate 15
  OPC_Decode, 165, 25, 160, 2,               // 6062:     decode to V6_vroundwuh using decoder 288
                                             // 6062:    }
  1, 0,                                      // 6067:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6069:     check predicate 26
  OPC_Decode, 197, 25, 160, 2,               // 6071:     decode to V6_vsub_qf16 using decoder 288
                                             // 6071:    }
                                             // 6071:    } // switch Inst[13]
                                             // 6071:   }
  6, 21,                                     // 6076:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 6078:    switch Inst[13] {
  0, 7,                                      // 6081:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6083:     check predicate 15
  OPC_Decode, 160, 25, 160, 2,               // 6085:     decode to V6_vroundhb using decoder 288
                                             // 6085:    }
  1, 0,                                      // 6090:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6092:     check predicate 26
  OPC_Decode, 193, 25, 160, 2,               // 6094:     decode to V6_vsub_hf using decoder 288
                                             // 6094:    }
                                             // 6094:    } // switch Inst[13]
                                             // 6094:   }
  7, 0,                                      // 6099:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 6101:    switch Inst[13] {
  0, 7,                                      // 6104:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6106:     check predicate 15
  OPC_Decode, 161, 25, 160, 2,               // 6108:     decode to V6_vroundhub using decoder 288
                                             // 6108:    }
  1, 0,                                      // 6113:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6115:     check predicate 26
  OPC_Decode, 198, 25, 160, 2,               // 6117:     decode to V6_vsub_qf16_mix using decoder 288
                                             // 6117:    }
                                             // 6117:    } // switch Inst[13]
                                             // 6117:   }
                                             // 6117:   } // switch Inst[7:5]
                                             // 6117:  }
  4, 130, 2,                                 // 6122:  case 0x4: {
  OPC_SwitchField, 5, 3,                     // 6125:   switch Inst[7:5] {
  0, 89,                                     // 6128:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 6130:    switch Inst[13] {
  0, 75,                                     // 6133:    case 0x0: {
  OPC_SwitchField, 2, 3,                     // 6135:     switch Inst[4:2] {
  0, 7,                                      // 6138:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 6140:      check predicate 15
  OPC_Decode, 153, 23, 170, 2,               // 6142:      decode to V6_veqb using decoder 298
                                             // 6142:     }
  1, 7,                                      // 6147:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 6149:      check predicate 15
  OPC_Decode, 157, 23, 170, 2,               // 6151:      decode to V6_veqh using decoder 298
                                             // 6151:     }
  2, 7,                                      // 6156:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 6158:      check predicate 15
  OPC_Decode, 169, 23, 170, 2,               // 6160:      decode to V6_veqw using decoder 298
                                             // 6160:     }
  3, 7,                                      // 6165:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 6167:      check predicate 18
  OPC_Decode, 165, 23, 170, 2,               // 6169:      decode to V6_veqsf using decoder 298
                                             // 6169:     }
  4, 7,                                      // 6174:     case 0x4: {
  OPC_CheckPredicate, 15,                    // 6176:      check predicate 15
  OPC_Decode, 188, 23, 170, 2,               // 6178:      decode to V6_vgtb using decoder 298
                                             // 6178:     }
  5, 7,                                      // 6183:     case 0x5: {
  OPC_CheckPredicate, 15,                    // 6185:      check predicate 15
  OPC_Decode, 196, 23, 170, 2,               // 6187:      decode to V6_vgth using decoder 298
                                             // 6187:     }
  6, 7,                                      // 6192:     case 0x6: {
  OPC_CheckPredicate, 15,                    // 6194:      check predicate 15
  OPC_Decode, 220, 23, 170, 2,               // 6196:      decode to V6_vgtw using decoder 298
                                             // 6196:     }
  7, 0,                                      // 6201:     case 0x7: {
  OPC_CheckPredicate, 18,                    // 6203:      check predicate 18
  OPC_Decode, 161, 23, 170, 2,               // 6205:      decode to V6_veqhf using decoder 298
                                             // 6205:     }
                                             // 6205:     } // switch Inst[4:2]
                                             // 6205:    }
  1, 0,                                      // 6210:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6212:     check predicate 26
  OPC_Decode, 160, 24, 166, 2,               // 6214:     decode to V6_vmpy_qf32_mix_hf using decoder 294
                                             // 6214:    }
                                             // 6214:    } // switch Inst[13]
                                             // 6214:   }
  1, 44,                                     // 6219:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 6221:    switch Inst[13] {
  0, 30,                                     // 6224:    case 0x0: {
  OPC_SwitchField, 2, 3,                     // 6226:     switch Inst[4:2] {
  0, 7,                                      // 6229:     case 0x0: {
  OPC_CheckPredicate, 15,                    // 6231:      check predicate 15
  OPC_Decode, 208, 23, 170, 2,               // 6233:      decode to V6_vgtub using decoder 298
                                             // 6233:     }
  1, 7,                                      // 6238:     case 0x1: {
  OPC_CheckPredicate, 15,                    // 6240:      check predicate 15
  OPC_Decode, 212, 23, 170, 2,               // 6242:      decode to V6_vgtuh using decoder 298
                                             // 6242:     }
  2, 0,                                      // 6247:     case 0x2: {
  OPC_CheckPredicate, 15,                    // 6249:      check predicate 15
  OPC_Decode, 216, 23, 170, 2,               // 6251:      decode to V6_vgtuw using decoder 298
                                             // 6251:     }
                                             // 6251:     } // switch Inst[4:2]
                                             // 6251:    }
  1, 0,                                      // 6256:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6258:     check predicate 21
  OPC_Decode, 170, 24, 160, 2,               // 6260:     decode to V6_vmpy_sf_sf using decoder 288
                                             // 6260:    }
                                             // 6260:    } // switch Inst[13]
                                             // 6260:   }
  2, 11,                                     // 6265:   case 0x2: {
  OPC_CheckPredicate, 21,                    // 6267:    check predicate 21
  OPC_CheckField, 13, 1, 1,                  // 6269:    check Inst[13] == 0x1
  OPC_Decode, 168, 24, 166, 2,               // 6273:    decode to V6_vmpy_sf_hf using decoder 294
                                             // 6273:   }
  3, 11,                                     // 6278:   case 0x3: {
  OPC_CheckPredicate, 21,                    // 6280:    check predicate 21
  OPC_CheckField, 13, 1, 1,                  // 6282:    check Inst[13] == 0x1
  OPC_Decode, 153, 24, 160, 2,               // 6286:    decode to V6_vmpy_hf_hf using decoder 288
                                             // 6286:   }
  4, 21,                                     // 6291:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 6293:    switch Inst[13] {
  0, 7,                                      // 6296:    case 0x0: {
  OPC_CheckPredicate, 20,                    // 6298:     check predicate 20
  OPC_Decode, 240, 21, 166, 2,               // 6300:     decode to V6_vadd_hf_f8 using decoder 294
                                             // 6300:    }
  1, 0,                                      // 6305:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6307:     check predicate 21
  OPC_Decode, 248, 21, 166, 2,               // 6309:     decode to V6_vadd_sf_hf using decoder 294
                                             // 6309:    }
                                             // 6309:    } // switch Inst[13]
                                             // 6309:   }
  5, 21,                                     // 6314:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 6316:    switch Inst[13] {
  0, 7,                                      // 6319:    case 0x0: {
  OPC_CheckPredicate, 20,                    // 6321:     check predicate 20
  OPC_Decode, 194, 25, 166, 2,               // 6323:     decode to V6_vsub_hf_f8 using decoder 294
                                             // 6323:    }
  1, 0,                                      // 6328:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6330:     check predicate 21
  OPC_Decode, 203, 25, 166, 2,               // 6332:     decode to V6_vsub_sf_hf using decoder 294
                                             // 6332:    }
                                             // 6332:    } // switch Inst[13]
                                             // 6332:   }
  6, 21,                                     // 6337:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 6339:    switch Inst[13] {
  0, 7,                                      // 6342:    case 0x0: {
  OPC_CheckPredicate, 20,                    // 6344:     check predicate 20
  OPC_Decode, 151, 24, 166, 2,               // 6346:     decode to V6_vmpy_hf_f8 using decoder 294
                                             // 6346:    }
  1, 0,                                      // 6351:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6353:     check predicate 21
  OPC_Decode, 249, 21, 160, 2,               // 6355:     decode to V6_vadd_sf_sf using decoder 288
                                             // 6355:    }
                                             // 6355:    } // switch Inst[13]
                                             // 6355:   }
  7, 0,                                      // 6360:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 6362:    switch Inst[13] {
  0, 7,                                      // 6365:    case 0x0: {
  OPC_CheckPredicate, 20,                    // 6367:     check predicate 20
  OPC_Decode, 152, 24, 162, 2,               // 6369:     decode to V6_vmpy_hf_f8_acc using decoder 290
                                             // 6369:    }
  1, 0,                                      // 6374:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6376:     check predicate 21
  OPC_Decode, 205, 25, 160, 2,               // 6378:     decode to V6_vsub_sf_sf using decoder 288
                                             // 6378:    }
                                             // 6378:    } // switch Inst[13]
                                             // 6378:   }
                                             // 6378:   } // switch Inst[7:5]
                                             // 6378:  }
  5, 187, 1,                                 // 6383:  case 0x5: {
  OPC_SwitchField, 5, 3,                     // 6386:   switch Inst[7:5] {
  0, 21,                                     // 6389:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 6391:    switch Inst[13] {
  0, 7,                                      // 6394:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6396:     check predicate 15
  OPC_Decode, 199, 22, 160, 2,               // 6398:     decode to V6_vasrwv using decoder 288
                                             // 6398:    }
  1, 0,                                      // 6403:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6405:     check predicate 26
  OPC_Decode, 244, 21, 160, 2,               // 6407:     decode to V6_vadd_qf32 using decoder 288
                                             // 6407:    }
                                             // 6407:    } // switch Inst[13]
                                             // 6407:   }
  1, 21,                                     // 6412:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 6414:    switch Inst[13] {
  0, 7,                                      // 6417:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6419:     check predicate 15
  OPC_Decode, 237, 23, 160, 2,               // 6421:     decode to V6_vlsrwv using decoder 288
                                             // 6421:    }
  1, 0,                                      // 6426:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6428:     check predicate 26
  OPC_Decode, 246, 21, 160, 2,               // 6430:     decode to V6_vadd_sf using decoder 288
                                             // 6430:    }
                                             // 6430:    } // switch Inst[13]
                                             // 6430:   }
  2, 21,                                     // 6435:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 6437:    switch Inst[13] {
  0, 7,                                      // 6440:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6442:     check predicate 15
  OPC_Decode, 235, 23, 160, 2,               // 6444:     decode to V6_vlsrhv using decoder 288
                                             // 6444:    }
  1, 0,                                      // 6449:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6451:     check predicate 26
  OPC_Decode, 245, 21, 160, 2,               // 6453:     decode to V6_vadd_qf32_mix using decoder 288
                                             // 6453:    }
                                             // 6453:    } // switch Inst[13]
                                             // 6453:   }
  3, 21,                                     // 6458:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 6460:    switch Inst[13] {
  0, 7,                                      // 6463:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6465:     check predicate 15
  OPC_Decode, 183, 22, 160, 2,               // 6467:     decode to V6_vasrhv using decoder 288
                                             // 6467:    }
  1, 0,                                      // 6472:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6474:     check predicate 26
  OPC_Decode, 199, 25, 160, 2,               // 6476:     decode to V6_vsub_qf32 using decoder 288
                                             // 6476:    }
                                             // 6476:    } // switch Inst[13]
                                             // 6476:   }
  4, 21,                                     // 6481:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 6483:    switch Inst[13] {
  0, 7,                                      // 6486:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6488:     check predicate 15
  OPC_Decode, 175, 22, 160, 2,               // 6490:     decode to V6_vaslwv using decoder 288
                                             // 6490:    }
  1, 0,                                      // 6495:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6497:     check predicate 26
  OPC_Decode, 201, 25, 160, 2,               // 6499:     decode to V6_vsub_sf using decoder 288
                                             // 6499:    }
                                             // 6499:    } // switch Inst[13]
                                             // 6499:   }
  5, 21,                                     // 6504:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 6506:    switch Inst[13] {
  0, 7,                                      // 6509:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6511:     check predicate 15
  OPC_Decode, 172, 22, 160, 2,               // 6513:     decode to V6_vaslhv using decoder 288
                                             // 6513:    }
  1, 0,                                      // 6518:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6520:     check predicate 26
  OPC_Decode, 200, 25, 160, 2,               // 6522:     decode to V6_vsub_qf32_mix using decoder 288
                                             // 6522:    }
                                             // 6522:    } // switch Inst[13]
                                             // 6522:   }
  6, 21,                                     // 6527:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 6529:    switch Inst[13] {
  0, 7,                                      // 6532:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6534:     check predicate 15
  OPC_Decode, 250, 21, 160, 2,               // 6536:     decode to V6_vaddb using decoder 288
                                             // 6536:    }
  1, 0,                                      // 6541:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6543:     check predicate 21
  OPC_Decode, 131, 23, 160, 2,               // 6545:     decode to V6_vdmpy_sf_hf using decoder 288
                                             // 6545:    }
                                             // 6545:    } // switch Inst[13]
                                             // 6545:   }
  7, 0,                                      // 6550:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 6552:    switch Inst[13] {
  0, 7,                                      // 6555:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6557:     check predicate 15
  OPC_Decode, 133, 22, 160, 2,               // 6559:     decode to V6_vaddh using decoder 288
                                             // 6559:    }
  1, 0,                                      // 6564:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6566:     check predicate 21
  OPC_Decode, 241, 21, 160, 2,               // 6568:     decode to V6_vadd_hf_hf using decoder 288
                                             // 6568:    }
                                             // 6568:    } // switch Inst[13]
                                             // 6568:   }
                                             // 6568:   } // switch Inst[7:5]
                                             // 6568:  }
  6, 177, 1,                                 // 6573:  case 0x6: {
  OPC_SwitchField, 5, 3,                     // 6576:   switch Inst[7:5] {
  0, 11,                                     // 6579:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 6581:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 6583:    check Inst[13] == 0x0
  OPC_Decode, 191, 24, 160, 2,               // 6587:    decode to V6_vmpyiewuh using decoder 288
                                             // 6587:   }
  1, 21,                                     // 6592:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 6594:    switch Inst[13] {
  0, 7,                                      // 6597:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6599:     check predicate 15
  OPC_Decode, 197, 24, 160, 2,               // 6601:     decode to V6_vmpyiowh using decoder 288
                                             // 6601:    }
  1, 0,                                      // 6606:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6608:     check predicate 26
  OPC_Decode, 251, 23, 160, 2,               // 6610:     decode to V6_vmax_sf using decoder 288
                                             // 6610:    }
                                             // 6610:    } // switch Inst[13]
                                             // 6610:   }
  2, 21,                                     // 6615:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 6617:    switch Inst[13] {
  0, 7,                                      // 6620:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6622:     check predicate 15
  OPC_Decode, 235, 24, 160, 2,               // 6624:     decode to V6_vpackeb using decoder 288
                                             // 6624:    }
  1, 0,                                      // 6629:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6631:     check predicate 26
  OPC_Decode, 132, 24, 160, 2,               // 6633:     decode to V6_vmin_sf using decoder 288
                                             // 6633:    }
                                             // 6633:    } // switch Inst[13]
                                             // 6633:   }
  3, 21,                                     // 6638:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 6640:    switch Inst[13] {
  0, 7,                                      // 6643:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6645:     check predicate 15
  OPC_Decode, 236, 24, 160, 2,               // 6647:     decode to V6_vpackeh using decoder 288
                                             // 6647:    }
  1, 0,                                      // 6652:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6654:     check predicate 26
  OPC_Decode, 250, 23, 160, 2,               // 6656:     decode to V6_vmax_hf using decoder 288
                                             // 6656:    }
                                             // 6656:    } // switch Inst[13]
                                             // 6656:   }
  4, 21,                                     // 6661:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 6663:    switch Inst[13] {
  0, 7,                                      // 6666:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 6668:     check predicate 12
  OPC_Decode, 228, 25, 160, 2,               // 6670:     decode to V6_vsubuwsat using decoder 288
                                             // 6670:    }
  1, 0,                                      // 6675:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6677:     check predicate 26
  OPC_Decode, 131, 24, 160, 2,               // 6679:     decode to V6_vmin_hf using decoder 288
                                             // 6679:    }
                                             // 6679:    } // switch Inst[13]
                                             // 6679:   }
  5, 21,                                     // 6684:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 6686:    switch Inst[13] {
  0, 7,                                      // 6689:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6691:     check predicate 15
  OPC_Decode, 238, 24, 160, 2,               // 6693:     decode to V6_vpackhub_sat using decoder 288
                                             // 6693:    }
  1, 0,                                      // 6698:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6700:     check predicate 21
  OPC_Decode, 251, 22, 160, 2,               // 6702:     decode to V6_vcvt_ub_hf using decoder 288
                                             // 6702:    }
                                             // 6702:    } // switch Inst[13]
                                             // 6702:   }
  6, 21,                                     // 6707:   case 0x6: {
  OPC_SwitchField, 13, 1,                    // 6709:    switch Inst[13] {
  0, 7,                                      // 6712:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6714:     check predicate 15
  OPC_Decode, 237, 24, 160, 2,               // 6716:     decode to V6_vpackhb_sat using decoder 288
                                             // 6716:    }
  1, 0,                                      // 6721:    case 0x1: {
  OPC_CheckPredicate, 21,                    // 6723:     check predicate 21
  OPC_Decode, 240, 22, 160, 2,               // 6725:     decode to V6_vcvt_b_hf using decoder 288
                                             // 6725:    }
                                             // 6725:    } // switch Inst[13]
                                             // 6725:   }
  7, 0,                                      // 6730:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 6732:    switch Inst[13] {
  0, 7,                                      // 6735:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6737:     check predicate 15
  OPC_Decode, 242, 24, 160, 2,               // 6739:     decode to V6_vpackwuh_sat using decoder 288
                                             // 6739:    }
  1, 0,                                      // 6744:    case 0x1: {
  OPC_CheckPredicate, 24,                    // 6746:     check predicate 24
  OPC_Decode, 219, 24, 160, 2,               // 6748:     decode to V6_vmpyuhvs using decoder 288
                                             // 6748:    }
                                             // 6748:    } // switch Inst[13]
                                             // 6748:   }
                                             // 6748:   } // switch Inst[7:5]
                                             // 6748:  }
  7, 0,                                      // 6753:  case 0x7: {
  OPC_SwitchField, 5, 3,                     // 6755:   switch Inst[7:5] {
  0, 21,                                     // 6758:   case 0x0: {
  OPC_SwitchField, 13, 1,                    // 6760:    switch Inst[13] {
  0, 7,                                      // 6763:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6765:     check predicate 15
  OPC_Decode, 241, 24, 160, 2,               // 6767:     decode to V6_vpackwh_sat using decoder 288
                                             // 6767:    }
  1, 0,                                      // 6772:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6774:     check predicate 26
  OPC_Decode, 158, 24, 160, 2,               // 6776:     decode to V6_vmpy_qf32 using decoder 288
                                             // 6776:    }
                                             // 6776:    } // switch Inst[13]
                                             // 6776:   }
  1, 21,                                     // 6781:   case 0x1: {
  OPC_SwitchField, 13, 1,                    // 6783:    switch Inst[13] {
  0, 7,                                      // 6786:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6788:     check predicate 15
  OPC_Decode, 239, 24, 160, 2,               // 6790:     decode to V6_vpackob using decoder 288
                                             // 6790:    }
  1, 0,                                      // 6795:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6797:     check predicate 26
  OPC_Decode, 162, 24, 160, 2,               // 6799:     decode to V6_vmpy_qf32_sf using decoder 288
                                             // 6799:    }
                                             // 6799:    } // switch Inst[13]
                                             // 6799:   }
  2, 21,                                     // 6804:   case 0x2: {
  OPC_SwitchField, 13, 1,                    // 6806:    switch Inst[13] {
  0, 7,                                      // 6809:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6811:     check predicate 15
  OPC_Decode, 240, 24, 160, 2,               // 6813:     decode to V6_vpackoh using decoder 288
                                             // 6813:    }
  1, 0,                                      // 6818:    case 0x1: {
  OPC_CheckPredicate, 20,                    // 6820:     check predicate 20
  OPC_Decode, 242, 22, 160, 2,               // 6822:     decode to V6_vcvt_f8_hf using decoder 288
                                             // 6822:    }
                                             // 6822:    } // switch Inst[13]
                                             // 6822:   }
  3, 21,                                     // 6827:   case 0x3: {
  OPC_SwitchField, 13, 1,                    // 6829:    switch Inst[13] {
  0, 7,                                      // 6832:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 6834:     check predicate 12
  OPC_Decode, 162, 25, 160, 2,               // 6836:     decode to V6_vrounduhub using decoder 288
                                             // 6836:    }
  1, 0,                                      // 6841:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6843:     check predicate 26
  OPC_Decode, 155, 24, 160, 2,               // 6845:     decode to V6_vmpy_qf16 using decoder 288
                                             // 6845:    }
                                             // 6845:    } // switch Inst[13]
                                             // 6845:   }
  4, 21,                                     // 6850:   case 0x4: {
  OPC_SwitchField, 13, 1,                    // 6852:    switch Inst[13] {
  0, 7,                                      // 6855:    case 0x0: {
  OPC_CheckPredicate, 12,                    // 6857:     check predicate 12
  OPC_Decode, 163, 25, 160, 2,               // 6859:     decode to V6_vrounduwuh using decoder 288
                                             // 6859:    }
  1, 0,                                      // 6864:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6866:     check predicate 26
  OPC_Decode, 156, 24, 160, 2,               // 6868:     decode to V6_vmpy_qf16_hf using decoder 288
                                             // 6868:    }
                                             // 6868:    } // switch Inst[13]
                                             // 6868:   }
  5, 21,                                     // 6873:   case 0x5: {
  OPC_SwitchField, 13, 1,                    // 6875:    switch Inst[13] {
  0, 7,                                      // 6878:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6880:     check predicate 15
  OPC_Decode, 177, 24, 160, 2,               // 6882:     decode to V6_vmpyewuh using decoder 288
                                             // 6882:    }
  1, 0,                                      // 6887:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6889:     check predicate 26
  OPC_Decode, 157, 24, 160, 2,               // 6891:     decode to V6_vmpy_qf16_mix_hf using decoder 288
                                             // 6891:    }
                                             // 6891:    } // switch Inst[13]
                                             // 6891:   }
  6, 11,                                     // 6896:   case 0x6: {
  OPC_CheckPredicate, 26,                    // 6898:    check predicate 26
  OPC_CheckField, 13, 1, 1,                  // 6900:    check Inst[13] == 0x1
  OPC_Decode, 161, 24, 166, 2,               // 6904:    decode to V6_vmpy_qf32_qf16 using decoder 294
                                             // 6904:   }
  7, 0,                                      // 6909:   case 0x7: {
  OPC_SwitchField, 13, 1,                    // 6911:    switch Inst[13] {
  0, 7,                                      // 6914:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 6916:     check predicate 15
  OPC_Decode, 204, 24, 160, 2,               // 6918:     decode to V6_vmpyowh using decoder 288
                                             // 6918:    }
  1, 0,                                      // 6923:    case 0x1: {
  OPC_CheckPredicate, 26,                    // 6925:     check predicate 26
  OPC_Decode, 159, 24, 166, 2,               // 6927:     decode to V6_vmpy_qf32_hf using decoder 294
                                             // 6927:    }
                                             // 6927:    } // switch Inst[13]
                                             // 6927:   }
                                             // 6927:   } // switch Inst[7:5]
                                             // 6927:  }
                                             // 6927:  } // switch Inst[23:21]
                                             // 6927: }
  40, 219, 3,                                // 6932: case 0x28: {
  OPC_SwitchField, 5, 3,                     // 6935:  switch Inst[7:5] {
  0, 91,                                     // 6938:  case 0x0: {
  OPC_SwitchField, 21, 3,                    // 6940:   switch Inst[23:21] {
  0, 11,                                     // 6943:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 6945:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 6947:    check Inst[12:11] == 0x0
  OPC_Decode, 236, 20, 193, 2,               // 6951:    decode to V6_vL32b_ai using decoder 321
                                             // 6951:   }
  1, 11,                                     // 6956:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 6958:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 6960:    check Inst[12:11] == 0x0
  OPC_Decode, 171, 21, 194, 2,               // 6964:    decode to V6_vS32b_ai using decoder 322
                                             // 6964:   }
  2, 11,                                     // 6969:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 6971:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 6973:    check Inst[12:11] == 0x0
  OPC_Decode, 249, 20, 193, 2,               // 6977:    decode to V6_vL32b_nt_ai using decoder 321
                                             // 6977:   }
  3, 11,                                     // 6982:   case 0x3: {
  OPC_CheckPredicate, 15,                    // 6984:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 6986:    check Inst[12:11] == 0x0
  OPC_Decode, 187, 21, 194, 2,               // 6990:    decode to V6_vS32b_nt_ai using decoder 322
                                             // 6990:   }
  4, 7,                                      // 6995:   case 0x4: {
  OPC_CheckPredicate, 15,                    // 6997:    check predicate 15
  OPC_Decode, 216, 21, 195, 2,               // 6999:    decode to V6_vS32b_qpred_ai using decoder 323
                                             // 6999:   }
  5, 7,                                      // 7004:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7006:    check predicate 15
  OPC_Decode, 213, 21, 196, 2,               // 7008:    decode to V6_vS32b_pred_ai using decoder 324
                                             // 7008:   }
  6, 7,                                      // 7013:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 7015:    check predicate 15
  OPC_Decode, 208, 21, 195, 2,               // 7017:    decode to V6_vS32b_nt_qpred_ai using decoder 323
                                             // 7017:   }
  7, 0,                                      // 7022:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7024:    check predicate 15
  OPC_Decode, 205, 21, 196, 2,               // 7026:    decode to V6_vS32b_nt_pred_ai using decoder 324
                                             // 7026:   }
                                             // 7026:   } // switch Inst[23:21]
                                             // 7026:  }
  1, 117,                                    // 7031:  case 0x1: {
  OPC_SwitchField, 21, 3,                    // 7033:   switch Inst[23:21] {
  0, 11,                                     // 7036:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7038:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7040:    check Inst[12:11] == 0x0
  OPC_Decode, 237, 20, 193, 2,               // 7044:    decode to V6_vL32b_cur_ai using decoder 321
                                             // 7044:   }
  1, 33,                                     // 7049:   case 0x1: {
  OPC_SwitchField, 3, 2,                     // 7051:    switch Inst[4:3] {
  0, 11,                                     // 7054:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 7056:     check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7058:     check Inst[12:11] == 0x0
  OPC_Decode, 172, 21, 197, 2,               // 7062:     decode to V6_vS32b_new_ai using decoder 325
                                             // 7062:    }
  1, 0,                                      // 7067:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 7069:     check predicate 13
  OPC_CheckField, 11, 2, 0,                  // 7071:     check Inst[12:11] == 0x0
  OPC_CheckField, 0, 3, 0,                   // 7075:     check Inst[2:0] == 0x0
  OPC_Decode, 219, 21, 198, 2,               // 7079:     decode to V6_vS32b_srls_ai using decoder 326
                                             // 7079:    }
                                             // 7079:    } // switch Inst[4:3]
                                             // 7079:   }
  2, 11,                                     // 7084:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 7086:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7088:    check Inst[12:11] == 0x0
  OPC_Decode, 250, 20, 193, 2,               // 7092:    decode to V6_vL32b_nt_cur_ai using decoder 321
                                             // 7092:   }
  3, 15,                                     // 7097:   case 0x3: {
  OPC_CheckPredicate, 15,                    // 7099:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7101:    check Inst[12:11] == 0x0
  OPC_CheckField, 3, 2, 0,                   // 7105:    check Inst[4:3] == 0x0
  OPC_Decode, 188, 21, 197, 2,               // 7109:    decode to V6_vS32b_nt_new_ai using decoder 325
                                             // 7109:   }
  4, 7,                                      // 7114:   case 0x4: {
  OPC_CheckPredicate, 15,                    // 7116:    check predicate 15
  OPC_Decode, 184, 21, 195, 2,               // 7118:    decode to V6_vS32b_nqpred_ai using decoder 323
                                             // 7118:   }
  5, 7,                                      // 7123:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7125:    check predicate 15
  OPC_Decode, 181, 21, 196, 2,               // 7127:    decode to V6_vS32b_npred_ai using decoder 324
                                             // 7127:   }
  6, 7,                                      // 7132:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 7134:    check predicate 15
  OPC_Decode, 200, 21, 195, 2,               // 7136:    decode to V6_vS32b_nt_nqpred_ai using decoder 323
                                             // 7136:   }
  7, 0,                                      // 7141:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7143:    check predicate 15
  OPC_Decode, 197, 21, 196, 2,               // 7145:    decode to V6_vS32b_nt_npred_ai using decoder 324
                                             // 7145:   }
                                             // 7145:   } // switch Inst[23:21]
                                             // 7145:  }
  2, 73,                                     // 7150:  case 0x2: {
  OPC_SwitchField, 21, 3,                    // 7152:   switch Inst[23:21] {
  0, 11,                                     // 7155:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7157:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7159:    check Inst[12:11] == 0x0
  OPC_Decode, 153, 21, 193, 2,               // 7163:    decode to V6_vL32b_tmp_ai using decoder 321
                                             // 7163:   }
  2, 11,                                     // 7168:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 7170:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7172:    check Inst[12:11] == 0x0
  OPC_Decode, 139, 21, 193, 2,               // 7176:    decode to V6_vL32b_nt_tmp_ai using decoder 321
                                             // 7176:   }
  4, 7,                                      // 7181:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7183:    check predicate 12
  OPC_Decode, 150, 21, 199, 2,               // 7185:    decode to V6_vL32b_pred_ai using decoder 327
                                             // 7185:   }
  5, 11,                                     // 7190:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7192:    check predicate 15
  OPC_CheckField, 3, 2, 0,                   // 7194:    check Inst[4:3] == 0x0
  OPC_Decode, 178, 21, 200, 2,               // 7198:    decode to V6_vS32b_new_pred_ai using decoder 328
                                             // 7198:   }
  6, 7,                                      // 7203:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7205:    check predicate 12
  OPC_Decode, 136, 21, 199, 2,               // 7207:    decode to V6_vL32b_nt_pred_ai using decoder 327
                                             // 7207:   }
  7, 0,                                      // 7212:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7214:    check predicate 15
  OPC_CheckField, 3, 2, 2,                   // 7216:    check Inst[4:3] == 0x2
  OPC_Decode, 194, 21, 200, 2,               // 7220:    decode to V6_vS32b_nt_new_pred_ai using decoder 328
                                             // 7220:   }
                                             // 7220:   } // switch Inst[23:21]
                                             // 7220:  }
  3, 47,                                     // 7225:  case 0x3: {
  OPC_SwitchField, 21, 3,                    // 7227:   switch Inst[23:21] {
  4, 7,                                      // 7230:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7232:    check predicate 12
  OPC_Decode, 246, 20, 199, 2,               // 7234:    decode to V6_vL32b_npred_ai using decoder 327
                                             // 7234:   }
  5, 11,                                     // 7239:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7241:    check predicate 15
  OPC_CheckField, 3, 2, 1,                   // 7243:    check Inst[4:3] == 0x1
  OPC_Decode, 173, 21, 200, 2,               // 7247:    decode to V6_vS32b_new_npred_ai using decoder 328
                                             // 7247:   }
  6, 7,                                      // 7252:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7254:    check predicate 12
  OPC_Decode, 131, 21, 199, 2,               // 7256:    decode to V6_vL32b_nt_npred_ai using decoder 327
                                             // 7256:   }
  7, 0,                                      // 7261:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7263:    check predicate 15
  OPC_CheckField, 3, 2, 3,                   // 7265:    check Inst[4:3] == 0x3
  OPC_Decode, 189, 21, 200, 2,               // 7269:    decode to V6_vS32b_nt_new_npred_ai using decoder 328
                                             // 7269:   }
                                             // 7269:   } // switch Inst[23:21]
                                             // 7269:  }
  4, 21,                                     // 7274:  case 0x4: {
  OPC_SwitchField, 21, 3,                    // 7276:   switch Inst[23:21] {
  4, 7,                                      // 7279:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7281:    check predicate 12
  OPC_Decode, 243, 20, 199, 2,               // 7283:    decode to V6_vL32b_cur_pred_ai using decoder 327
                                             // 7283:   }
  6, 0,                                      // 7288:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7290:    check predicate 12
  OPC_Decode, 128, 21, 199, 2,               // 7292:    decode to V6_vL32b_nt_cur_pred_ai using decoder 327
                                             // 7292:   }
                                             // 7292:   } // switch Inst[23:21]
                                             // 7292:  }
  5, 21,                                     // 7297:  case 0x5: {
  OPC_SwitchField, 21, 3,                    // 7299:   switch Inst[23:21] {
  4, 7,                                      // 7302:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7304:    check predicate 12
  OPC_Decode, 238, 20, 199, 2,               // 7306:    decode to V6_vL32b_cur_npred_ai using decoder 327
                                             // 7306:   }
  6, 0,                                      // 7311:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7313:    check predicate 12
  OPC_Decode, 251, 20, 199, 2,               // 7315:    decode to V6_vL32b_nt_cur_npred_ai using decoder 327
                                             // 7315:   }
                                             // 7315:   } // switch Inst[23:21]
                                             // 7315:  }
  6, 30,                                     // 7320:  case 0x6: {
  OPC_SwitchField, 21, 3,                    // 7322:   switch Inst[23:21] {
  4, 7,                                      // 7325:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7327:    check predicate 12
  OPC_Decode, 159, 21, 199, 2,               // 7329:    decode to V6_vL32b_tmp_pred_ai using decoder 327
                                             // 7329:   }
  5, 7,                                      // 7334:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7336:    check predicate 15
  OPC_Decode, 168, 21, 196, 2,               // 7338:    decode to V6_vS32Ub_pred_ai using decoder 324
                                             // 7338:   }
  6, 0,                                      // 7343:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7345:    check predicate 12
  OPC_Decode, 145, 21, 199, 2,               // 7347:    decode to V6_vL32b_nt_tmp_pred_ai using decoder 327
                                             // 7347:   }
                                             // 7347:   } // switch Inst[23:21]
                                             // 7347:  }
  7, 0,                                      // 7352:  case 0x7: {
  OPC_SwitchField, 21, 3,                    // 7354:   switch Inst[23:21] {
  0, 11,                                     // 7357:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7359:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7361:    check Inst[12:11] == 0x0
  OPC_Decode, 233, 20, 193, 2,               // 7365:    decode to V6_vL32Ub_ai using decoder 321
                                             // 7365:   }
  1, 11,                                     // 7370:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 7372:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 7374:    check Inst[12:11] == 0x0
  OPC_Decode, 162, 21, 194, 2,               // 7378:    decode to V6_vS32Ub_ai using decoder 322
                                             // 7378:   }
  4, 7,                                      // 7383:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7385:    check predicate 12
  OPC_Decode, 154, 21, 199, 2,               // 7387:    decode to V6_vL32b_tmp_npred_ai using decoder 327
                                             // 7387:   }
  5, 7,                                      // 7392:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7394:    check predicate 15
  OPC_Decode, 163, 21, 196, 2,               // 7396:    decode to V6_vS32Ub_npred_ai using decoder 324
                                             // 7396:   }
  6, 0,                                      // 7401:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7403:    check predicate 12
  OPC_Decode, 140, 21, 199, 2,               // 7405:    decode to V6_vL32b_nt_tmp_npred_ai using decoder 327
                                             // 7405:   }
                                             // 7405:   } // switch Inst[23:21]
                                             // 7405:  }
                                             // 7405:  } // switch Inst[7:5]
                                             // 7405: }
  41, 196, 4,                                // 7410: case 0x29: {
  OPC_SwitchField, 5, 3,                     // 7413:  switch Inst[7:5] {
  0, 107,                                    // 7416:  case 0x0: {
  OPC_SwitchField, 21, 3,                    // 7418:   switch Inst[23:21] {
  0, 11,                                     // 7421:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7423:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7425:    check Inst[13:11] == 0x0
  OPC_Decode, 148, 21, 201, 2,               // 7429:    decode to V6_vL32b_pi using decoder 329
                                             // 7429:   }
  1, 11,                                     // 7434:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 7436:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7438:    check Inst[13:11] == 0x0
  OPC_Decode, 211, 21, 202, 2,               // 7442:    decode to V6_vS32b_pi using decoder 330
                                             // 7442:   }
  2, 11,                                     // 7447:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 7449:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7451:    check Inst[13:11] == 0x0
  OPC_Decode, 134, 21, 201, 2,               // 7455:    decode to V6_vL32b_nt_pi using decoder 329
                                             // 7455:   }
  3, 11,                                     // 7460:   case 0x3: {
  OPC_CheckPredicate, 15,                    // 7462:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7464:    check Inst[13:11] == 0x0
  OPC_Decode, 203, 21, 202, 2,               // 7468:    decode to V6_vS32b_nt_pi using decoder 330
                                             // 7468:   }
  4, 11,                                     // 7473:   case 0x4: {
  OPC_CheckPredicate, 15,                    // 7475:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7477:    check Inst[13] == 0x0
  OPC_Decode, 217, 21, 203, 2,               // 7481:    decode to V6_vS32b_qpred_pi using decoder 331
                                             // 7481:   }
  5, 11,                                     // 7486:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7488:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7490:    check Inst[13] == 0x0
  OPC_Decode, 214, 21, 204, 2,               // 7494:    decode to V6_vS32b_pred_pi using decoder 332
                                             // 7494:   }
  6, 11,                                     // 7499:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 7501:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7503:    check Inst[13] == 0x0
  OPC_Decode, 209, 21, 203, 2,               // 7507:    decode to V6_vS32b_nt_qpred_pi using decoder 331
                                             // 7507:   }
  7, 0,                                      // 7512:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7514:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7516:    check Inst[13] == 0x0
  OPC_Decode, 206, 21, 204, 2,               // 7520:    decode to V6_vS32b_nt_pred_pi using decoder 332
                                             // 7520:   }
                                             // 7520:   } // switch Inst[23:21]
                                             // 7520:  }
  1, 133, 1,                                 // 7525:  case 0x1: {
  OPC_SwitchField, 21, 3,                    // 7528:   switch Inst[23:21] {
  0, 11,                                     // 7531:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7533:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7535:    check Inst[13:11] == 0x0
  OPC_Decode, 241, 20, 201, 2,               // 7539:    decode to V6_vL32b_cur_pi using decoder 329
                                             // 7539:   }
  1, 33,                                     // 7544:   case 0x1: {
  OPC_SwitchField, 3, 2,                     // 7546:    switch Inst[4:3] {
  0, 11,                                     // 7549:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 7551:     check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7553:     check Inst[13:11] == 0x0
  OPC_Decode, 176, 21, 205, 2,               // 7557:     decode to V6_vS32b_new_pi using decoder 333
                                             // 7557:    }
  1, 0,                                      // 7562:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 7564:     check predicate 13
  OPC_CheckField, 11, 3, 0,                  // 7566:     check Inst[13:11] == 0x0
  OPC_CheckField, 0, 3, 0,                   // 7570:     check Inst[2:0] == 0x0
  OPC_Decode, 220, 21, 206, 2,               // 7574:     decode to V6_vS32b_srls_pi using decoder 334
                                             // 7574:    }
                                             // 7574:    } // switch Inst[4:3]
                                             // 7574:   }
  2, 11,                                     // 7579:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 7581:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7583:    check Inst[13:11] == 0x0
  OPC_Decode, 254, 20, 201, 2,               // 7587:    decode to V6_vL32b_nt_cur_pi using decoder 329
                                             // 7587:   }
  3, 15,                                     // 7592:   case 0x3: {
  OPC_CheckPredicate, 15,                    // 7594:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7596:    check Inst[13:11] == 0x0
  OPC_CheckField, 3, 2, 0,                   // 7600:    check Inst[4:3] == 0x0
  OPC_Decode, 192, 21, 205, 2,               // 7604:    decode to V6_vS32b_nt_new_pi using decoder 333
                                             // 7604:   }
  4, 11,                                     // 7609:   case 0x4: {
  OPC_CheckPredicate, 15,                    // 7611:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7613:    check Inst[13] == 0x0
  OPC_Decode, 185, 21, 203, 2,               // 7617:    decode to V6_vS32b_nqpred_pi using decoder 331
                                             // 7617:   }
  5, 11,                                     // 7622:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7624:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7626:    check Inst[13] == 0x0
  OPC_Decode, 182, 21, 204, 2,               // 7630:    decode to V6_vS32b_npred_pi using decoder 332
                                             // 7630:   }
  6, 11,                                     // 7635:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 7637:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7639:    check Inst[13] == 0x0
  OPC_Decode, 201, 21, 203, 2,               // 7643:    decode to V6_vS32b_nt_nqpred_pi using decoder 331
                                             // 7643:   }
  7, 0,                                      // 7648:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7650:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7652:    check Inst[13] == 0x0
  OPC_Decode, 198, 21, 204, 2,               // 7656:    decode to V6_vS32b_nt_npred_pi using decoder 332
                                             // 7656:   }
                                             // 7656:   } // switch Inst[23:21]
                                             // 7656:  }
  2, 89,                                     // 7661:  case 0x2: {
  OPC_SwitchField, 21, 3,                    // 7663:   switch Inst[23:21] {
  0, 11,                                     // 7666:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7668:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7670:    check Inst[13:11] == 0x0
  OPC_Decode, 157, 21, 201, 2,               // 7674:    decode to V6_vL32b_tmp_pi using decoder 329
                                             // 7674:   }
  2, 11,                                     // 7679:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 7681:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7683:    check Inst[13:11] == 0x0
  OPC_Decode, 143, 21, 201, 2,               // 7687:    decode to V6_vL32b_nt_tmp_pi using decoder 329
                                             // 7687:   }
  4, 11,                                     // 7692:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7694:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7696:    check Inst[13] == 0x0
  OPC_Decode, 151, 21, 207, 2,               // 7700:    decode to V6_vL32b_pred_pi using decoder 335
                                             // 7700:   }
  5, 15,                                     // 7705:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7707:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7709:    check Inst[13] == 0x0
  OPC_CheckField, 3, 2, 0,                   // 7713:    check Inst[4:3] == 0x0
  OPC_Decode, 179, 21, 208, 2,               // 7717:    decode to V6_vS32b_new_pred_pi using decoder 336
                                             // 7717:   }
  6, 11,                                     // 7722:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7724:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7726:    check Inst[13] == 0x0
  OPC_Decode, 137, 21, 207, 2,               // 7730:    decode to V6_vL32b_nt_pred_pi using decoder 335
                                             // 7730:   }
  7, 0,                                      // 7735:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7737:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7739:    check Inst[13] == 0x0
  OPC_CheckField, 3, 2, 2,                   // 7743:    check Inst[4:3] == 0x2
  OPC_Decode, 195, 21, 208, 2,               // 7747:    decode to V6_vS32b_nt_new_pred_pi using decoder 336
                                             // 7747:   }
                                             // 7747:   } // switch Inst[23:21]
                                             // 7747:  }
  3, 63,                                     // 7752:  case 0x3: {
  OPC_SwitchField, 21, 3,                    // 7754:   switch Inst[23:21] {
  4, 11,                                     // 7757:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7759:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7761:    check Inst[13] == 0x0
  OPC_Decode, 247, 20, 207, 2,               // 7765:    decode to V6_vL32b_npred_pi using decoder 335
                                             // 7765:   }
  5, 15,                                     // 7770:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7772:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7774:    check Inst[13] == 0x0
  OPC_CheckField, 3, 2, 1,                   // 7778:    check Inst[4:3] == 0x1
  OPC_Decode, 174, 21, 208, 2,               // 7782:    decode to V6_vS32b_new_npred_pi using decoder 336
                                             // 7782:   }
  6, 11,                                     // 7787:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7789:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7791:    check Inst[13] == 0x0
  OPC_Decode, 132, 21, 207, 2,               // 7795:    decode to V6_vL32b_nt_npred_pi using decoder 335
                                             // 7795:   }
  7, 0,                                      // 7800:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 7802:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7804:    check Inst[13] == 0x0
  OPC_CheckField, 3, 2, 3,                   // 7808:    check Inst[4:3] == 0x3
  OPC_Decode, 190, 21, 208, 2,               // 7812:    decode to V6_vS32b_nt_new_npred_pi using decoder 336
                                             // 7812:   }
                                             // 7812:   } // switch Inst[23:21]
                                             // 7812:  }
  4, 29,                                     // 7817:  case 0x4: {
  OPC_SwitchField, 21, 3,                    // 7819:   switch Inst[23:21] {
  4, 11,                                     // 7822:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7824:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7826:    check Inst[13] == 0x0
  OPC_Decode, 244, 20, 207, 2,               // 7830:    decode to V6_vL32b_cur_pred_pi using decoder 335
                                             // 7830:   }
  6, 0,                                      // 7835:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7837:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7839:    check Inst[13] == 0x0
  OPC_Decode, 129, 21, 207, 2,               // 7843:    decode to V6_vL32b_nt_cur_pred_pi using decoder 335
                                             // 7843:   }
                                             // 7843:   } // switch Inst[23:21]
                                             // 7843:  }
  5, 29,                                     // 7848:  case 0x5: {
  OPC_SwitchField, 21, 3,                    // 7850:   switch Inst[23:21] {
  4, 11,                                     // 7853:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7855:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7857:    check Inst[13] == 0x0
  OPC_Decode, 239, 20, 207, 2,               // 7861:    decode to V6_vL32b_cur_npred_pi using decoder 335
                                             // 7861:   }
  6, 0,                                      // 7866:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7868:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7870:    check Inst[13] == 0x0
  OPC_Decode, 252, 20, 207, 2,               // 7874:    decode to V6_vL32b_nt_cur_npred_pi using decoder 335
                                             // 7874:   }
                                             // 7874:   } // switch Inst[23:21]
                                             // 7874:  }
  6, 42,                                     // 7879:  case 0x6: {
  OPC_SwitchField, 21, 3,                    // 7881:   switch Inst[23:21] {
  4, 11,                                     // 7884:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7886:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7888:    check Inst[13] == 0x0
  OPC_Decode, 160, 21, 207, 2,               // 7892:    decode to V6_vL32b_tmp_pred_pi using decoder 335
                                             // 7892:   }
  5, 11,                                     // 7897:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7899:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7901:    check Inst[13] == 0x0
  OPC_Decode, 169, 21, 204, 2,               // 7905:    decode to V6_vS32Ub_pred_pi using decoder 332
                                             // 7905:   }
  6, 0,                                      // 7910:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7912:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7914:    check Inst[13] == 0x0
  OPC_Decode, 146, 21, 207, 2,               // 7918:    decode to V6_vL32b_nt_tmp_pred_pi using decoder 335
                                             // 7918:   }
                                             // 7918:   } // switch Inst[23:21]
                                             // 7918:  }
  7, 0,                                      // 7923:  case 0x7: {
  OPC_SwitchField, 21, 3,                    // 7925:   switch Inst[23:21] {
  0, 11,                                     // 7928:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 7930:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7932:    check Inst[13:11] == 0x0
  OPC_Decode, 234, 20, 201, 2,               // 7936:    decode to V6_vL32Ub_pi using decoder 329
                                             // 7936:   }
  1, 11,                                     // 7941:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 7943:    check predicate 15
  OPC_CheckField, 11, 3, 0,                  // 7945:    check Inst[13:11] == 0x0
  OPC_Decode, 166, 21, 202, 2,               // 7949:    decode to V6_vS32Ub_pi using decoder 330
                                             // 7949:   }
  4, 11,                                     // 7954:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 7956:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7958:    check Inst[13] == 0x0
  OPC_Decode, 155, 21, 207, 2,               // 7962:    decode to V6_vL32b_tmp_npred_pi using decoder 335
                                             // 7962:   }
  5, 11,                                     // 7967:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 7969:    check predicate 15
  OPC_CheckField, 13, 1, 0,                  // 7971:    check Inst[13] == 0x0
  OPC_Decode, 164, 21, 204, 2,               // 7975:    decode to V6_vS32Ub_npred_pi using decoder 332
                                             // 7975:   }
  6, 0,                                      // 7980:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 7982:    check predicate 12
  OPC_CheckField, 13, 1, 0,                  // 7984:    check Inst[13] == 0x0
  OPC_Decode, 141, 21, 207, 2,               // 7988:    decode to V6_vL32b_nt_tmp_npred_pi using decoder 335
                                             // 7988:   }
                                             // 7988:   } // switch Inst[23:21]
                                             // 7988:  }
                                             // 7988:  } // switch Inst[7:5]
                                             // 7988: }
  43, 219, 3,                                // 7993: case 0x2b: {
  OPC_SwitchField, 5, 6,                     // 7996:  switch Inst[10:5] {
  0, 91,                                     // 7999:  case 0x0: {
  OPC_SwitchField, 21, 3,                    // 8001:   switch Inst[23:21] {
  0, 11,                                     // 8004:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 8006:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8008:    check Inst[12:11] == 0x0
  OPC_Decode, 149, 21, 209, 2,               // 8012:    decode to V6_vL32b_ppu using decoder 337
                                             // 8012:   }
  1, 11,                                     // 8017:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 8019:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8021:    check Inst[12:11] == 0x0
  OPC_Decode, 212, 21, 210, 2,               // 8025:    decode to V6_vS32b_ppu using decoder 338
                                             // 8025:   }
  2, 11,                                     // 8030:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 8032:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8034:    check Inst[12:11] == 0x0
  OPC_Decode, 135, 21, 209, 2,               // 8038:    decode to V6_vL32b_nt_ppu using decoder 337
                                             // 8038:   }
  3, 11,                                     // 8043:   case 0x3: {
  OPC_CheckPredicate, 15,                    // 8045:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8047:    check Inst[12:11] == 0x0
  OPC_Decode, 204, 21, 210, 2,               // 8051:    decode to V6_vS32b_nt_ppu using decoder 338
                                             // 8051:   }
  4, 7,                                      // 8056:   case 0x4: {
  OPC_CheckPredicate, 15,                    // 8058:    check predicate 15
  OPC_Decode, 218, 21, 211, 2,               // 8060:    decode to V6_vS32b_qpred_ppu using decoder 339
                                             // 8060:   }
  5, 7,                                      // 8065:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 8067:    check predicate 15
  OPC_Decode, 215, 21, 212, 2,               // 8069:    decode to V6_vS32b_pred_ppu using decoder 340
                                             // 8069:   }
  6, 7,                                      // 8074:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 8076:    check predicate 15
  OPC_Decode, 210, 21, 211, 2,               // 8078:    decode to V6_vS32b_nt_qpred_ppu using decoder 339
                                             // 8078:   }
  7, 0,                                      // 8083:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 8085:    check predicate 15
  OPC_Decode, 207, 21, 212, 2,               // 8087:    decode to V6_vS32b_nt_pred_ppu using decoder 340
                                             // 8087:   }
                                             // 8087:   } // switch Inst[23:21]
                                             // 8087:  }
  1, 117,                                    // 8092:  case 0x1: {
  OPC_SwitchField, 21, 3,                    // 8094:   switch Inst[23:21] {
  0, 11,                                     // 8097:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 8099:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8101:    check Inst[12:11] == 0x0
  OPC_Decode, 242, 20, 209, 2,               // 8105:    decode to V6_vL32b_cur_ppu using decoder 337
                                             // 8105:   }
  1, 33,                                     // 8110:   case 0x1: {
  OPC_SwitchField, 3, 2,                     // 8112:    switch Inst[4:3] {
  0, 11,                                     // 8115:    case 0x0: {
  OPC_CheckPredicate, 15,                    // 8117:     check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8119:     check Inst[12:11] == 0x0
  OPC_Decode, 177, 21, 213, 2,               // 8123:     decode to V6_vS32b_new_ppu using decoder 341
                                             // 8123:    }
  1, 0,                                      // 8128:    case 0x1: {
  OPC_CheckPredicate, 13,                    // 8130:     check predicate 13
  OPC_CheckField, 11, 2, 0,                  // 8132:     check Inst[12:11] == 0x0
  OPC_CheckField, 0, 3, 0,                   // 8136:     check Inst[2:0] == 0x0
  OPC_Decode, 221, 21, 214, 2,               // 8140:     decode to V6_vS32b_srls_ppu using decoder 342
                                             // 8140:    }
                                             // 8140:    } // switch Inst[4:3]
                                             // 8140:   }
  2, 11,                                     // 8145:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 8147:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8149:    check Inst[12:11] == 0x0
  OPC_Decode, 255, 20, 209, 2,               // 8153:    decode to V6_vL32b_nt_cur_ppu using decoder 337
                                             // 8153:   }
  3, 15,                                     // 8158:   case 0x3: {
  OPC_CheckPredicate, 15,                    // 8160:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8162:    check Inst[12:11] == 0x0
  OPC_CheckField, 3, 2, 0,                   // 8166:    check Inst[4:3] == 0x0
  OPC_Decode, 193, 21, 213, 2,               // 8170:    decode to V6_vS32b_nt_new_ppu using decoder 341
                                             // 8170:   }
  4, 7,                                      // 8175:   case 0x4: {
  OPC_CheckPredicate, 15,                    // 8177:    check predicate 15
  OPC_Decode, 186, 21, 211, 2,               // 8179:    decode to V6_vS32b_nqpred_ppu using decoder 339
                                             // 8179:   }
  5, 7,                                      // 8184:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 8186:    check predicate 15
  OPC_Decode, 183, 21, 212, 2,               // 8188:    decode to V6_vS32b_npred_ppu using decoder 340
                                             // 8188:   }
  6, 7,                                      // 8193:   case 0x6: {
  OPC_CheckPredicate, 15,                    // 8195:    check predicate 15
  OPC_Decode, 202, 21, 211, 2,               // 8197:    decode to V6_vS32b_nt_nqpred_ppu using decoder 339
                                             // 8197:   }
  7, 0,                                      // 8202:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 8204:    check predicate 15
  OPC_Decode, 199, 21, 212, 2,               // 8206:    decode to V6_vS32b_nt_npred_ppu using decoder 340
                                             // 8206:   }
                                             // 8206:   } // switch Inst[23:21]
                                             // 8206:  }
  2, 73,                                     // 8211:  case 0x2: {
  OPC_SwitchField, 21, 3,                    // 8213:   switch Inst[23:21] {
  0, 11,                                     // 8216:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 8218:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8220:    check Inst[12:11] == 0x0
  OPC_Decode, 158, 21, 209, 2,               // 8224:    decode to V6_vL32b_tmp_ppu using decoder 337
                                             // 8224:   }
  2, 11,                                     // 8229:   case 0x2: {
  OPC_CheckPredicate, 15,                    // 8231:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8233:    check Inst[12:11] == 0x0
  OPC_Decode, 144, 21, 209, 2,               // 8237:    decode to V6_vL32b_nt_tmp_ppu using decoder 337
                                             // 8237:   }
  4, 7,                                      // 8242:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 8244:    check predicate 12
  OPC_Decode, 152, 21, 215, 2,               // 8246:    decode to V6_vL32b_pred_ppu using decoder 343
                                             // 8246:   }
  5, 11,                                     // 8251:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 8253:    check predicate 15
  OPC_CheckField, 3, 2, 0,                   // 8255:    check Inst[4:3] == 0x0
  OPC_Decode, 180, 21, 216, 2,               // 8259:    decode to V6_vS32b_new_pred_ppu using decoder 344
                                             // 8259:   }
  6, 7,                                      // 8264:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 8266:    check predicate 12
  OPC_Decode, 138, 21, 215, 2,               // 8268:    decode to V6_vL32b_nt_pred_ppu using decoder 343
                                             // 8268:   }
  7, 0,                                      // 8273:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 8275:    check predicate 15
  OPC_CheckField, 3, 2, 2,                   // 8277:    check Inst[4:3] == 0x2
  OPC_Decode, 196, 21, 216, 2,               // 8281:    decode to V6_vS32b_nt_new_pred_ppu using decoder 344
                                             // 8281:   }
                                             // 8281:   } // switch Inst[23:21]
                                             // 8281:  }
  3, 47,                                     // 8286:  case 0x3: {
  OPC_SwitchField, 21, 3,                    // 8288:   switch Inst[23:21] {
  4, 7,                                      // 8291:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 8293:    check predicate 12
  OPC_Decode, 248, 20, 215, 2,               // 8295:    decode to V6_vL32b_npred_ppu using decoder 343
                                             // 8295:   }
  5, 11,                                     // 8300:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 8302:    check predicate 15
  OPC_CheckField, 3, 2, 1,                   // 8304:    check Inst[4:3] == 0x1
  OPC_Decode, 175, 21, 216, 2,               // 8308:    decode to V6_vS32b_new_npred_ppu using decoder 344
                                             // 8308:   }
  6, 7,                                      // 8313:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 8315:    check predicate 12
  OPC_Decode, 133, 21, 215, 2,               // 8317:    decode to V6_vL32b_nt_npred_ppu using decoder 343
                                             // 8317:   }
  7, 0,                                      // 8322:   case 0x7: {
  OPC_CheckPredicate, 15,                    // 8324:    check predicate 15
  OPC_CheckField, 3, 2, 3,                   // 8326:    check Inst[4:3] == 0x3
  OPC_Decode, 191, 21, 216, 2,               // 8330:    decode to V6_vS32b_nt_new_npred_ppu using decoder 344
                                             // 8330:   }
                                             // 8330:   } // switch Inst[23:21]
                                             // 8330:  }
  4, 21,                                     // 8335:  case 0x4: {
  OPC_SwitchField, 21, 3,                    // 8337:   switch Inst[23:21] {
  4, 7,                                      // 8340:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 8342:    check predicate 12
  OPC_Decode, 245, 20, 215, 2,               // 8344:    decode to V6_vL32b_cur_pred_ppu using decoder 343
                                             // 8344:   }
  6, 0,                                      // 8349:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 8351:    check predicate 12
  OPC_Decode, 130, 21, 215, 2,               // 8353:    decode to V6_vL32b_nt_cur_pred_ppu using decoder 343
                                             // 8353:   }
                                             // 8353:   } // switch Inst[23:21]
                                             // 8353:  }
  5, 21,                                     // 8358:  case 0x5: {
  OPC_SwitchField, 21, 3,                    // 8360:   switch Inst[23:21] {
  4, 7,                                      // 8363:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 8365:    check predicate 12
  OPC_Decode, 240, 20, 215, 2,               // 8367:    decode to V6_vL32b_cur_npred_ppu using decoder 343
                                             // 8367:   }
  6, 0,                                      // 8372:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 8374:    check predicate 12
  OPC_Decode, 253, 20, 215, 2,               // 8376:    decode to V6_vL32b_nt_cur_npred_ppu using decoder 343
                                             // 8376:   }
                                             // 8376:   } // switch Inst[23:21]
                                             // 8376:  }
  6, 30,                                     // 8381:  case 0x6: {
  OPC_SwitchField, 21, 3,                    // 8383:   switch Inst[23:21] {
  4, 7,                                      // 8386:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 8388:    check predicate 12
  OPC_Decode, 161, 21, 215, 2,               // 8390:    decode to V6_vL32b_tmp_pred_ppu using decoder 343
                                             // 8390:   }
  5, 7,                                      // 8395:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 8397:    check predicate 15
  OPC_Decode, 170, 21, 212, 2,               // 8399:    decode to V6_vS32Ub_pred_ppu using decoder 340
                                             // 8399:   }
  6, 0,                                      // 8404:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 8406:    check predicate 12
  OPC_Decode, 147, 21, 215, 2,               // 8408:    decode to V6_vL32b_nt_tmp_pred_ppu using decoder 343
                                             // 8408:   }
                                             // 8408:   } // switch Inst[23:21]
                                             // 8408:  }
  7, 0,                                      // 8413:  case 0x7: {
  OPC_SwitchField, 21, 3,                    // 8415:   switch Inst[23:21] {
  0, 11,                                     // 8418:   case 0x0: {
  OPC_CheckPredicate, 15,                    // 8420:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8422:    check Inst[12:11] == 0x0
  OPC_Decode, 235, 20, 209, 2,               // 8426:    decode to V6_vL32Ub_ppu using decoder 337
                                             // 8426:   }
  1, 11,                                     // 8431:   case 0x1: {
  OPC_CheckPredicate, 15,                    // 8433:    check predicate 15
  OPC_CheckField, 11, 2, 0,                  // 8435:    check Inst[12:11] == 0x0
  OPC_Decode, 167, 21, 210, 2,               // 8439:    decode to V6_vS32Ub_ppu using decoder 338
                                             // 8439:   }
  4, 7,                                      // 8444:   case 0x4: {
  OPC_CheckPredicate, 12,                    // 8446:    check predicate 12
  OPC_Decode, 156, 21, 215, 2,               // 8448:    decode to V6_vL32b_tmp_npred_ppu using decoder 343
                                             // 8448:   }
  5, 7,                                      // 8453:   case 0x5: {
  OPC_CheckPredicate, 15,                    // 8455:    check predicate 15
  OPC_Decode, 165, 21, 212, 2,               // 8457:    decode to V6_vS32Ub_npred_ppu using decoder 340
                                             // 8457:   }
  6, 0,                                      // 8462:   case 0x6: {
  OPC_CheckPredicate, 12,                    // 8464:    check predicate 12
  OPC_Decode, 142, 21, 215, 2,               // 8466:    decode to V6_vL32b_nt_tmp_npred_ppu using decoder 343
                                             // 8466:   }
                                             // 8466:   } // switch Inst[23:21]
                                             // 8466:  }
                                             // 8466:  } // switch Inst[10:5]
                                             // 8466: }
  44, 33,                                    // 8471: case 0x2c: {
  OPC_SwitchField, 21, 3,                    // 8473:  switch Inst[23:21] {
  0, 15,                                     // 8476:  case 0x0: {
  OPC_CheckPredicate, 16,                    // 8478:   check predicate 16
  OPC_CheckField, 11, 2, 0,                  // 8480:   check Inst[12:11] == 0x0
  OPC_CheckField, 0, 8, 0,                   // 8484:   check Inst[7:0] == 0x0
  OPC_Decode, 132, 26, 198, 2,               // 8488:   decode to V6_zLd_ai using decoder 326
                                             // 8488:  }
  4, 0,                                      // 8493:  case 0x4: {
  OPC_CheckPredicate, 16,                    // 8495:   check predicate 16
  OPC_CheckField, 0, 8, 0,                   // 8497:   check Inst[7:0] == 0x0
  OPC_Decode, 135, 26, 217, 2,               // 8501:   decode to V6_zLd_pred_ai using decoder 345
                                             // 8501:  }
                                             // 8501:  } // switch Inst[23:21]
                                             // 8501: }
  45, 65,                                    // 8506: case 0x2d: {
  OPC_SwitchField, 0, 8,                     // 8508:  switch Inst[7:0] {
  0, 29,                                     // 8511:  case 0x0: {
  OPC_SwitchField, 21, 3,                    // 8513:   switch Inst[23:21] {
  0, 11,                                     // 8516:   case 0x0: {
  OPC_CheckPredicate, 16,                    // 8518:    check predicate 16
  OPC_CheckField, 11, 3, 0,                  // 8520:    check Inst[13:11] == 0x0
  OPC_Decode, 133, 26, 206, 2,               // 8524:    decode to V6_zLd_pi using decoder 334
                                             // 8524:   }
  4, 0,                                      // 8529:   case 0x4: {
  OPC_CheckPredicate, 16,                    // 8531:    check predicate 16
  OPC_CheckField, 13, 1, 0,                  // 8533:    check Inst[13] == 0x0
  OPC_Decode, 136, 26, 218, 2,               // 8537:    decode to V6_zLd_pred_pi using decoder 346
                                             // 8537:   }
                                             // 8537:   } // switch Inst[23:21]
                                             // 8537:  }
  1, 0,                                      // 8542:  case 0x1: {
  OPC_SwitchField, 21, 3,                    // 8544:   switch Inst[23:21] {
  0, 11,                                     // 8547:   case 0x0: {
  OPC_CheckPredicate, 16,                    // 8549:    check predicate 16
  OPC_CheckField, 8, 5, 0,                   // 8551:    check Inst[12:8] == 0x0
  OPC_Decode, 134, 26, 214, 2,               // 8555:    decode to V6_zLd_ppu using decoder 342
                                             // 8555:   }
  4, 0,                                      // 8560:   case 0x4: {
  OPC_CheckPredicate, 16,                    // 8562:    check predicate 16
  OPC_CheckField, 8, 3, 0,                   // 8564:    check Inst[10:8] == 0x0
  OPC_Decode, 137, 26, 219, 2,               // 8568:    decode to V6_zLd_pred_ppu using decoder 347
                                             // 8568:   }
                                             // 8568:   } // switch Inst[23:21]
                                             // 8568:  }
                                             // 8568:  } // switch Inst[7:0]
                                             // 8568: }
  47, 169, 1,                                // 8573: case 0x2f: {
  OPC_SwitchField, 21, 3,                    // 8576:  switch Inst[23:21] {
  0, 69,                                     // 8579:  case 0x0: {
  OPC_SwitchField, 7, 6,                     // 8581:   switch Inst[12:7] {
  0, 11,                                     // 8584:   case 0x0: {
  OPC_CheckPredicate, 13,                    // 8586:    check predicate 13
  OPC_CheckField, 5, 2, 0,                   // 8588:    check Inst[6:5] == 0x0
  OPC_Decode, 186, 23, 220, 2,               // 8592:    decode to V6_vgathermw using decoder 348
                                             // 8592:   }
  2, 11,                                     // 8597:   case 0x2: {
  OPC_CheckPredicate, 13,                    // 8599:    check predicate 13
  OPC_CheckField, 5, 2, 0,                   // 8601:    check Inst[6:5] == 0x0
  OPC_Decode, 182, 23, 220, 2,               // 8605:    decode to V6_vgathermh using decoder 348
                                             // 8605:   }
  4, 11,                                     // 8610:   case 0x4: {
  OPC_CheckPredicate, 13,                    // 8612:    check predicate 13
  OPC_CheckField, 5, 2, 0,                   // 8614:    check Inst[6:5] == 0x0
  OPC_Decode, 184, 23, 221, 2,               // 8618:    decode to V6_vgathermhw using decoder 349
                                             // 8618:   }
  8, 7,                                      // 8623:   case 0x8: {
  OPC_CheckPredicate, 13,                    // 8625:    check predicate 13
  OPC_Decode, 187, 23, 222, 2,               // 8627:    decode to V6_vgathermwq using decoder 350
                                             // 8627:   }
  10, 7,                                     // 8632:   case 0xa: {
  OPC_CheckPredicate, 13,                    // 8634:    check predicate 13
  OPC_Decode, 183, 23, 222, 2,               // 8636:    decode to V6_vgathermhq using decoder 350
                                             // 8636:   }
  12, 0,                                     // 8641:   case 0xc: {
  OPC_CheckPredicate, 13,                    // 8643:    check predicate 13
  OPC_Decode, 185, 23, 223, 2,               // 8645:    decode to V6_vgathermhwq using decoder 351
                                             // 8645:   }
                                             // 8645:   } // switch Inst[12:7]
                                             // 8645:  }
  1, 57,                                     // 8650:  case 0x1: {
  OPC_SwitchField, 5, 3,                     // 8652:   switch Inst[7:5] {
  0, 7,                                      // 8655:   case 0x0: {
  OPC_CheckPredicate, 13,                    // 8657:    check predicate 13
  OPC_Decode, 179, 25, 224, 2,               // 8659:    decode to V6_vscattermw using decoder 352
                                             // 8659:   }
  1, 7,                                      // 8664:   case 0x1: {
  OPC_CheckPredicate, 13,                    // 8666:    check predicate 13
  OPC_Decode, 173, 25, 224, 2,               // 8668:    decode to V6_vscattermh using decoder 352
                                             // 8668:   }
  2, 7,                                      // 8673:   case 0x2: {
  OPC_CheckPredicate, 13,                    // 8675:    check predicate 13
  OPC_Decode, 176, 25, 225, 2,               // 8677:    decode to V6_vscattermhw using decoder 353
                                             // 8677:   }
  4, 7,                                      // 8682:   case 0x4: {
  OPC_CheckPredicate, 13,                    // 8684:    check predicate 13
  OPC_Decode, 180, 25, 224, 2,               // 8686:    decode to V6_vscattermw_add using decoder 352
                                             // 8686:   }
  5, 7,                                      // 8691:   case 0x5: {
  OPC_CheckPredicate, 13,                    // 8693:    check predicate 13
  OPC_Decode, 174, 25, 224, 2,               // 8695:    decode to V6_vscattermh_add using decoder 352
                                             // 8695:   }
  6, 0,                                      // 8700:   case 0x6: {
  OPC_CheckPredicate, 13,                    // 8702:    check predicate 13
  OPC_Decode, 177, 25, 225, 2,               // 8704:    decode to V6_vscattermhw_add using decoder 353
                                             // 8704:   }
                                             // 8704:   } // switch Inst[7:5]
                                             // 8704:  }
  4, 21,                                     // 8709:  case 0x4: {
  OPC_SwitchField, 7, 1,                     // 8711:   switch Inst[7] {
  0, 7,                                      // 8714:   case 0x0: {
  OPC_CheckPredicate, 13,                    // 8716:    check predicate 13
  OPC_Decode, 181, 25, 226, 2,               // 8718:    decode to V6_vscattermwq using decoder 354
                                             // 8718:   }
  1, 0,                                      // 8723:   case 0x1: {
  OPC_CheckPredicate, 13,                    // 8725:    check predicate 13
  OPC_Decode, 175, 25, 226, 2,               // 8727:    decode to V6_vscattermhq using decoder 354
                                             // 8727:   }
                                             // 8727:   } // switch Inst[7]
                                             // 8727:  }
  5, 0,                                      // 8732:  case 0x5: {
  OPC_CheckPredicate, 13,                    // 8734:   check predicate 13
  OPC_CheckField, 7, 1, 0,                   // 8736:   check Inst[7] == 0x0
  OPC_Decode, 178, 25, 227, 2,               // 8740:   decode to V6_vscattermhwq using decoder 355
                                             // 8740:  }
                                             // 8740:  } // switch Inst[23:21]
                                             // 8740: }
  146, 1, 0,                                 // 8745: case 0x92: {
  OPC_CheckPredicate, 15,                    // 8748:  check predicate 15
  OPC_CheckField, 21, 3, 0,                  // 8750:  check Inst[23:21] == 0x0
  OPC_CheckField, 13, 1, 0,                  // 8754:  check Inst[13] == 0x0
  OPC_CheckField, 5, 3, 1,                   // 8758:  check Inst[7:5] == 0x1
  OPC_Decode, 212, 20, 228, 2,               // 8762:  decode to V6_extractw using decoder 356
                                             // 8762: }
                                             // 8762: } // switch Inst[31:24]
};
static const uint8_t DecoderTableMustExtend32[1687] = {
  OPC_SwitchField, 21, 4,                    //    0: switch Inst[24:21] {
  0, 8,                                      //    3: case 0x0: {
  OPC_CheckField, 27, 5, 9,                  //    5:  check Inst[31:27] == 0x9
  OPC_Decode, 189, 16, 35,                   //    9:  decode to PS_storerbabs using decoder 35
                                             //    9: }
  1, 29,                                     //   13: case 0x1: {
  OPC_SwitchField, 25, 7,                    //   15:  switch Inst[31:25] {
  77, 13,                                    //   18:  case 0x4d: {
  OPC_CheckField, 12, 2, 1,                  //   20:   check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //   24:   check Inst[7] == 0x0
  OPC_Decode, 130, 13, 229, 2,               //   28:   decode to L4_loadbsw2_ap using decoder 357
                                             //   28:  }
  78, 0,                                     //   33:  case 0x4e: {
  OPC_CheckField, 12, 1, 1,                  //   35:   check Inst[12] == 0x1
  OPC_Decode, 131, 13, 230, 2,               //   39:   decode to L4_loadbsw2_ur using decoder 358
                                             //   39:  }
                                             //   39:  } // switch Inst[31:25]
                                             //   39: }
  2, 40,                                     //   44: case 0x2: {
  OPC_SwitchField, 27, 5,                    //   46:  switch Inst[31:27] {
  9, 4,                                      //   49:  case 0x9: {
  OPC_Decode, 193, 16, 37,                   //   51:   decode to PS_storerhabs using decoder 37
                                             //   51:  }
  19, 0,                                     //   55:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //   57:   switch Inst[26:25] {
  1, 13,                                     //   60:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //   62:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //   66:    check Inst[7] == 0x0
  OPC_Decode, 128, 13, 231, 2,               //   70:    decode to L4_loadalignh_ap using decoder 359
                                             //   70:   }
  2, 0,                                      //   75:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //   77:    check Inst[12] == 0x1
  OPC_Decode, 129, 13, 232, 2,               //   81:    decode to L4_loadalignh_ur using decoder 360
                                             //   81:   }
                                             //   81:   } // switch Inst[26:25]
                                             //   81:  }
                                             //   81:  } // switch Inst[31:27]
                                             //   81: }
  3, 40,                                     //   86: case 0x3: {
  OPC_SwitchField, 27, 5,                    //   88:  switch Inst[31:27] {
  9, 4,                                      //   91:  case 0x9: {
  OPC_Decode, 192, 16, 37,                   //   93:   decode to PS_storerfabs using decoder 37
                                             //   93:  }
  19, 0,                                     //   97:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //   99:   switch Inst[26:25] {
  1, 13,                                     //  102:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  104:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  108:    check Inst[7] == 0x0
  OPC_Decode, 134, 13, 229, 2,               //  112:    decode to L4_loadbzw2_ap using decoder 357
                                             //  112:   }
  2, 0,                                      //  117:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //  119:    check Inst[12] == 0x1
  OPC_Decode, 135, 13, 230, 2,               //  123:    decode to L4_loadbzw2_ur using decoder 358
                                             //  123:   }
                                             //  123:   } // switch Inst[26:25]
                                             //  123:  }
                                             //  123:  } // switch Inst[31:27]
                                             //  123: }
  4, 40,                                     //  128: case 0x4: {
  OPC_SwitchField, 27, 5,                    //  130:  switch Inst[31:27] {
  9, 4,                                      //  133:  case 0x9: {
  OPC_Decode, 195, 16, 39,                   //  135:   decode to PS_storeriabs using decoder 39
                                             //  135:  }
  19, 0,                                     //  139:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //  141:   switch Inst[26:25] {
  1, 13,                                     //  144:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  146:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  150:    check Inst[7] == 0x0
  OPC_Decode, 254, 12, 231, 2,               //  154:    decode to L4_loadalignb_ap using decoder 359
                                             //  154:   }
  2, 0,                                      //  159:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //  161:    check Inst[12] == 0x1
  OPC_Decode, 255, 12, 232, 2,               //  165:    decode to L4_loadalignb_ur using decoder 360
                                             //  165:   }
                                             //  165:   } // switch Inst[26:25]
                                             //  165:  }
                                             //  165:  } // switch Inst[31:27]
                                             //  165: }
  5, 70,                                     //  170: case 0x5: {
  OPC_SwitchField, 12, 1,                    //  172:  switch Inst[12] {
  0, 23,                                     //  175:  case 0x0: {
  OPC_SwitchField, 11, 1,                    //  177:   switch Inst[11] {
  0, 8,                                      //  180:   case 0x0: {
  OPC_CheckField, 27, 5, 9,                  //  182:    check Inst[31:27] == 0x9
  OPC_Decode, 190, 16, 41,                   //  186:    decode to PS_storerbnewabs using decoder 41
                                             //  186:   }
  1, 0,                                      //  190:   case 0x1: {
  OPC_CheckField, 27, 5, 9,                  //  192:    check Inst[31:27] == 0x9
  OPC_Decode, 194, 16, 43,                   //  196:    decode to PS_storerhnewabs using decoder 43
                                             //  196:   }
                                             //  196:   } // switch Inst[11]
                                             //  196:  }
  1, 0,                                      //  200:  case 0x1: {
  OPC_SwitchField, 27, 5,                    //  202:   switch Inst[31:27] {
  9, 8,                                      //  205:   case 0x9: {
  OPC_CheckField, 11, 1, 0,                  //  207:    check Inst[11] == 0x0
  OPC_Decode, 196, 16, 45,                   //  211:    decode to PS_storerinewabs using decoder 45
                                             //  211:   }
  19, 0,                                     //  215:   case 0x13: {
  OPC_SwitchField, 25, 2,                    //  217:    switch Inst[26:25] {
  1, 13,                                     //  220:    case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  222:     check Inst[13] == 0x0
  OPC_CheckField, 7, 1, 0,                   //  226:     check Inst[7] == 0x0
  OPC_Decode, 136, 13, 233, 2,               //  230:     decode to L4_loadbzw4_ap using decoder 361
                                             //  230:    }
  2, 0,                                      //  235:    case 0x2: {
  OPC_Decode, 137, 13, 234, 2,               //  237:     decode to L4_loadbzw4_ur using decoder 362
                                             //  237:    }
                                             //  237:    } // switch Inst[26:25]
                                             //  237:   }
                                             //  237:   } // switch Inst[31:27]
                                             //  237:  }
                                             //  237:  } // switch Inst[12]
                                             //  237: }
  6, 8,                                      //  242: case 0x6: {
  OPC_CheckField, 27, 5, 9,                  //  244:  check Inst[31:27] == 0x9
  OPC_Decode, 191, 16, 47,                   //  248:  decode to PS_storerdabs using decoder 47
                                             //  248: }
  7, 29,                                     //  252: case 0x7: {
  OPC_SwitchField, 25, 7,                    //  254:  switch Inst[31:25] {
  77, 13,                                    //  257:  case 0x4d: {
  OPC_CheckField, 12, 2, 1,                  //  259:   check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  263:   check Inst[7] == 0x0
  OPC_Decode, 132, 13, 233, 2,               //  267:   decode to L4_loadbsw4_ap using decoder 361
                                             //  267:  }
  78, 0,                                     //  272:  case 0x4e: {
  OPC_CheckField, 12, 1, 1,                  //  274:   check Inst[12] == 0x1
  OPC_Decode, 133, 13, 234, 2,               //  278:   decode to L4_loadbsw4_ur using decoder 362
                                             //  278:  }
                                             //  278:  } // switch Inst[31:25]
                                             //  278: }
  8, 195, 1,                                 //  283: case 0x8: {
  OPC_SwitchField, 27, 5,                    //  286:  switch Inst[31:27] {
  9, 4,                                      //  289:  case 0x9: {
  OPC_Decode, 183, 16, 49,                   //  291:   decode to PS_loadrbabs using decoder 49
                                             //  291:  }
  19, 78,                                    //  295:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //  297:   switch Inst[26:25] {
  1, 13,                                     //  300:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  302:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  306:    check Inst[7] == 0x0
  OPC_Decode, 140, 13, 229, 2,               //  310:    decode to L4_loadrb_ap using decoder 357
                                             //  310:   }
  2, 9,                                      //  315:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //  317:    check Inst[12] == 0x1
  OPC_Decode, 142, 13, 230, 2,               //  321:    decode to L4_loadrb_ur using decoder 358
                                             //  321:   }
  3, 0,                                      //  326:   case 0x3: {
  OPC_SwitchField, 11, 3,                    //  328:    switch Inst[13:11] {
  4, 9,                                      //  331:    case 0x4: {
  OPC_CheckField, 5, 3, 4,                   //  333:     check Inst[7:5] == 0x4
  OPC_Decode, 166, 13, 235, 2,               //  337:     decode to L4_ploadrbt_abs using decoder 363
                                             //  337:    }
  5, 9,                                      //  342:    case 0x5: {
  OPC_CheckField, 5, 3, 4,                   //  344:     check Inst[7:5] == 0x4
  OPC_Decode, 162, 13, 235, 2,               //  348:     decode to L4_ploadrbf_abs using decoder 363
                                             //  348:    }
  6, 9,                                      //  353:    case 0x6: {
  OPC_CheckField, 5, 3, 4,                   //  355:     check Inst[7:5] == 0x4
  OPC_Decode, 168, 13, 235, 2,               //  359:     decode to L4_ploadrbtnew_abs using decoder 363
                                             //  359:    }
  7, 0,                                      //  364:    case 0x7: {
  OPC_CheckField, 5, 3, 4,                   //  366:     check Inst[7:5] == 0x4
  OPC_Decode, 164, 13, 235, 2,               //  370:     decode to L4_ploadrbfnew_abs using decoder 363
                                             //  370:    }
                                             //  370:    } // switch Inst[13:11]
                                             //  370:   }
                                             //  370:   } // switch Inst[26:25]
                                             //  370:  }
  21, 0,                                     //  375:  case 0x15: {
  OPC_SwitchField, 25, 2,                    //  377:   switch Inst[26:25] {
  1, 13,                                     //  380:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  382:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   //  386:    check Inst[7:6] == 0x2
  OPC_Decode, 224, 19, 236, 2,               //  390:    decode to S4_storerb_ap using decoder 364
                                             //  390:   }
  2, 9,                                      //  395:   case 0x2: {
  OPC_CheckField, 7, 1, 1,                   //  397:    check Inst[7] == 0x1
  OPC_Decode, 226, 19, 237, 2,               //  401:    decode to S4_storerb_ur using decoder 365
                                             //  401:   }
  3, 0,                                      //  406:   case 0x3: {
  OPC_SwitchField, 2, 1,                     //  408:    switch Inst[2] {
  0, 33,                                     //  411:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  413:     switch Inst[13] {
  0, 13,                                     //  416:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  //  418:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  422:      check Inst[7] == 0x1
  OPC_Decode, 141, 19, 238, 2,               //  426:      decode to S4_pstorerbt_abs using decoder 366
                                             //  426:     }
  1, 0,                                      //  431:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  //  433:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  437:      check Inst[7] == 0x1
  OPC_Decode, 143, 19, 238, 2,               //  441:      decode to S4_pstorerbtnew_abs using decoder 366
                                             //  441:     }
                                             //  441:     } // switch Inst[13]
                                             //  441:    }
  1, 0,                                      //  446:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  448:     switch Inst[13] {
  0, 13,                                     //  451:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  //  453:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  457:      check Inst[7] == 0x1
  OPC_Decode, 254, 18, 238, 2,               //  461:      decode to S4_pstorerbf_abs using decoder 366
                                             //  461:     }
  1, 0,                                      //  466:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  //  468:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  472:      check Inst[7] == 0x1
  OPC_Decode, 128, 19, 238, 2,               //  476:      decode to S4_pstorerbfnew_abs using decoder 366
                                             //  476:     }
                                             //  476:     } // switch Inst[13]
                                             //  476:    }
                                             //  476:    } // switch Inst[2]
                                             //  476:   }
                                             //  476:   } // switch Inst[26:25]
                                             //  476:  }
                                             //  476:  } // switch Inst[31:27]
                                             //  476: }
  9, 89,                                     //  481: case 0x9: {
  OPC_SwitchField, 27, 5,                    //  483:  switch Inst[31:27] {
  9, 4,                                      //  486:  case 0x9: {
  OPC_Decode, 187, 16, 49,                   //  488:   decode to PS_loadrubabs using decoder 49
                                             //  488:  }
  19, 0,                                     //  492:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //  494:   switch Inst[26:25] {
  1, 13,                                     //  497:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  499:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  503:    check Inst[7] == 0x0
  OPC_Decode, 152, 13, 229, 2,               //  507:    decode to L4_loadrub_ap using decoder 357
                                             //  507:   }
  2, 9,                                      //  512:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //  514:    check Inst[12] == 0x1
  OPC_Decode, 154, 13, 230, 2,               //  518:    decode to L4_loadrub_ur using decoder 358
                                             //  518:   }
  3, 0,                                      //  523:   case 0x3: {
  OPC_SwitchField, 11, 3,                    //  525:    switch Inst[13:11] {
  4, 9,                                      //  528:    case 0x4: {
  OPC_CheckField, 5, 3, 4,                   //  530:     check Inst[7:5] == 0x4
  OPC_Decode, 198, 13, 235, 2,               //  534:     decode to L4_ploadrubt_abs using decoder 363
                                             //  534:    }
  5, 9,                                      //  539:    case 0x5: {
  OPC_CheckField, 5, 3, 4,                   //  541:     check Inst[7:5] == 0x4
  OPC_Decode, 194, 13, 235, 2,               //  545:     decode to L4_ploadrubf_abs using decoder 363
                                             //  545:    }
  6, 9,                                      //  550:    case 0x6: {
  OPC_CheckField, 5, 3, 4,                   //  552:     check Inst[7:5] == 0x4
  OPC_Decode, 200, 13, 235, 2,               //  556:     decode to L4_ploadrubtnew_abs using decoder 363
                                             //  556:    }
  7, 0,                                      //  561:    case 0x7: {
  OPC_CheckField, 5, 3, 4,                   //  563:     check Inst[7:5] == 0x4
  OPC_Decode, 196, 13, 235, 2,               //  567:     decode to L4_ploadrubfnew_abs using decoder 363
                                             //  567:    }
                                             //  567:    } // switch Inst[13:11]
                                             //  567:   }
                                             //  567:   } // switch Inst[26:25]
                                             //  567:  }
                                             //  567:  } // switch Inst[31:27]
                                             //  567: }
  10, 195, 1,                                //  572: case 0xa: {
  OPC_SwitchField, 27, 5,                    //  575:  switch Inst[31:27] {
  9, 4,                                      //  578:  case 0x9: {
  OPC_Decode, 185, 16, 51,                   //  580:   decode to PS_loadrhabs using decoder 51
                                             //  580:  }
  19, 78,                                    //  584:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //  586:   switch Inst[26:25] {
  1, 13,                                     //  589:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  591:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  595:    check Inst[7] == 0x0
  OPC_Decode, 146, 13, 229, 2,               //  599:    decode to L4_loadrh_ap using decoder 357
                                             //  599:   }
  2, 9,                                      //  604:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //  606:    check Inst[12] == 0x1
  OPC_Decode, 148, 13, 230, 2,               //  610:    decode to L4_loadrh_ur using decoder 358
                                             //  610:   }
  3, 0,                                      //  615:   case 0x3: {
  OPC_SwitchField, 11, 3,                    //  617:    switch Inst[13:11] {
  4, 9,                                      //  620:    case 0x4: {
  OPC_CheckField, 5, 3, 4,                   //  622:     check Inst[7:5] == 0x4
  OPC_Decode, 182, 13, 235, 2,               //  626:     decode to L4_ploadrht_abs using decoder 363
                                             //  626:    }
  5, 9,                                      //  631:    case 0x5: {
  OPC_CheckField, 5, 3, 4,                   //  633:     check Inst[7:5] == 0x4
  OPC_Decode, 178, 13, 235, 2,               //  637:     decode to L4_ploadrhf_abs using decoder 363
                                             //  637:    }
  6, 9,                                      //  642:    case 0x6: {
  OPC_CheckField, 5, 3, 4,                   //  644:     check Inst[7:5] == 0x4
  OPC_Decode, 184, 13, 235, 2,               //  648:     decode to L4_ploadrhtnew_abs using decoder 363
                                             //  648:    }
  7, 0,                                      //  653:    case 0x7: {
  OPC_CheckField, 5, 3, 4,                   //  655:     check Inst[7:5] == 0x4
  OPC_Decode, 180, 13, 235, 2,               //  659:     decode to L4_ploadrhfnew_abs using decoder 363
                                             //  659:    }
                                             //  659:    } // switch Inst[13:11]
                                             //  659:   }
                                             //  659:   } // switch Inst[26:25]
                                             //  659:  }
  21, 0,                                     //  664:  case 0x15: {
  OPC_SwitchField, 25, 2,                    //  666:   switch Inst[26:25] {
  1, 13,                                     //  669:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  671:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   //  675:    check Inst[7:6] == 0x2
  OPC_Decode, 236, 19, 236, 2,               //  679:    decode to S4_storerh_ap using decoder 364
                                             //  679:   }
  2, 9,                                      //  684:   case 0x2: {
  OPC_CheckField, 7, 1, 1,                   //  686:    check Inst[7] == 0x1
  OPC_Decode, 238, 19, 237, 2,               //  690:    decode to S4_storerh_ur using decoder 365
                                             //  690:   }
  3, 0,                                      //  695:   case 0x3: {
  OPC_SwitchField, 2, 1,                     //  697:    switch Inst[2] {
  0, 33,                                     //  700:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  702:     switch Inst[13] {
  0, 13,                                     //  705:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  //  707:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  711:      check Inst[7] == 0x1
  OPC_Decode, 181, 19, 238, 2,               //  715:      decode to S4_pstorerht_abs using decoder 366
                                             //  715:     }
  1, 0,                                      //  720:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  //  722:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  726:      check Inst[7] == 0x1
  OPC_Decode, 183, 19, 238, 2,               //  730:      decode to S4_pstorerhtnew_abs using decoder 366
                                             //  730:     }
                                             //  730:     } // switch Inst[13]
                                             //  730:    }
  1, 0,                                      //  735:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  737:     switch Inst[13] {
  0, 13,                                     //  740:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  //  742:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  746:      check Inst[7] == 0x1
  OPC_Decode, 166, 19, 238, 2,               //  750:      decode to S4_pstorerhf_abs using decoder 366
                                             //  750:     }
  1, 0,                                      //  755:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  //  757:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  761:      check Inst[7] == 0x1
  OPC_Decode, 168, 19, 238, 2,               //  765:      decode to S4_pstorerhfnew_abs using decoder 366
                                             //  765:     }
                                             //  765:     } // switch Inst[13]
                                             //  765:    }
                                             //  765:    } // switch Inst[2]
                                             //  765:   }
                                             //  765:   } // switch Inst[26:25]
                                             //  765:  }
                                             //  765:  } // switch Inst[31:27]
                                             //  765: }
  11, 195, 1,                                //  770: case 0xb: {
  OPC_SwitchField, 27, 5,                    //  773:  switch Inst[31:27] {
  9, 4,                                      //  776:  case 0x9: {
  OPC_Decode, 188, 16, 51,                   //  778:   decode to PS_loadruhabs using decoder 51
                                             //  778:  }
  19, 78,                                    //  782:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //  784:   switch Inst[26:25] {
  1, 13,                                     //  787:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  789:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  793:    check Inst[7] == 0x0
  OPC_Decode, 155, 13, 229, 2,               //  797:    decode to L4_loadruh_ap using decoder 357
                                             //  797:   }
  2, 9,                                      //  802:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  //  804:    check Inst[12] == 0x1
  OPC_Decode, 157, 13, 230, 2,               //  808:    decode to L4_loadruh_ur using decoder 358
                                             //  808:   }
  3, 0,                                      //  813:   case 0x3: {
  OPC_SwitchField, 11, 3,                    //  815:    switch Inst[13:11] {
  4, 9,                                      //  818:    case 0x4: {
  OPC_CheckField, 5, 3, 4,                   //  820:     check Inst[7:5] == 0x4
  OPC_Decode, 206, 13, 235, 2,               //  824:     decode to L4_ploadruht_abs using decoder 363
                                             //  824:    }
  5, 9,                                      //  829:    case 0x5: {
  OPC_CheckField, 5, 3, 4,                   //  831:     check Inst[7:5] == 0x4
  OPC_Decode, 202, 13, 235, 2,               //  835:     decode to L4_ploadruhf_abs using decoder 363
                                             //  835:    }
  6, 9,                                      //  840:    case 0x6: {
  OPC_CheckField, 5, 3, 4,                   //  842:     check Inst[7:5] == 0x4
  OPC_Decode, 208, 13, 235, 2,               //  846:     decode to L4_ploadruhtnew_abs using decoder 363
                                             //  846:    }
  7, 0,                                      //  851:    case 0x7: {
  OPC_CheckField, 5, 3, 4,                   //  853:     check Inst[7:5] == 0x4
  OPC_Decode, 204, 13, 235, 2,               //  857:     decode to L4_ploadruhfnew_abs using decoder 363
                                             //  857:    }
                                             //  857:    } // switch Inst[13:11]
                                             //  857:   }
                                             //  857:   } // switch Inst[26:25]
                                             //  857:  }
  21, 0,                                     //  862:  case 0x15: {
  OPC_SwitchField, 25, 2,                    //  864:   switch Inst[26:25] {
  1, 13,                                     //  867:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  //  869:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   //  873:    check Inst[7:6] == 0x2
  OPC_Decode, 233, 19, 236, 2,               //  877:    decode to S4_storerf_ap using decoder 364
                                             //  877:   }
  2, 9,                                      //  882:   case 0x2: {
  OPC_CheckField, 7, 1, 1,                   //  884:    check Inst[7] == 0x1
  OPC_Decode, 235, 19, 237, 2,               //  888:    decode to S4_storerf_ur using decoder 365
                                             //  888:   }
  3, 0,                                      //  893:   case 0x3: {
  OPC_SwitchField, 2, 1,                     //  895:    switch Inst[2] {
  0, 33,                                     //  898:    case 0x0: {
  OPC_SwitchField, 13, 1,                    //  900:     switch Inst[13] {
  0, 13,                                     //  903:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  //  905:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  909:      check Inst[7] == 0x1
  OPC_Decode, 161, 19, 238, 2,               //  913:      decode to S4_pstorerft_abs using decoder 366
                                             //  913:     }
  1, 0,                                      //  918:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  //  920:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  924:      check Inst[7] == 0x1
  OPC_Decode, 163, 19, 238, 2,               //  928:      decode to S4_pstorerftnew_abs using decoder 366
                                             //  928:     }
                                             //  928:     } // switch Inst[13]
                                             //  928:    }
  1, 0,                                      //  933:    case 0x1: {
  OPC_SwitchField, 13, 1,                    //  935:     switch Inst[13] {
  0, 13,                                     //  938:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  //  940:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  944:      check Inst[7] == 0x1
  OPC_Decode, 156, 19, 238, 2,               //  948:      decode to S4_pstorerff_abs using decoder 366
                                             //  948:     }
  1, 0,                                      //  953:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  //  955:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   //  959:      check Inst[7] == 0x1
  OPC_Decode, 158, 19, 238, 2,               //  963:      decode to S4_pstorerffnew_abs using decoder 366
                                             //  963:     }
                                             //  963:     } // switch Inst[13]
                                             //  963:    }
                                             //  963:    } // switch Inst[2]
                                             //  963:   }
                                             //  963:   } // switch Inst[26:25]
                                             //  963:  }
                                             //  963:  } // switch Inst[31:27]
                                             //  963: }
  12, 195, 1,                                //  968: case 0xc: {
  OPC_SwitchField, 27, 5,                    //  971:  switch Inst[31:27] {
  9, 4,                                      //  974:  case 0x9: {
  OPC_Decode, 186, 16, 53,                   //  976:   decode to PS_loadriabs using decoder 53
                                             //  976:  }
  19, 78,                                    //  980:  case 0x13: {
  OPC_SwitchField, 25, 2,                    //  982:   switch Inst[26:25] {
  1, 13,                                     //  985:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  //  987:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   //  991:    check Inst[7] == 0x0
  OPC_Decode, 149, 13, 229, 2,               //  995:    decode to L4_loadri_ap using decoder 357
                                             //  995:   }
  2, 9,                                      // 1000:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  // 1002:    check Inst[12] == 0x1
  OPC_Decode, 151, 13, 230, 2,               // 1006:    decode to L4_loadri_ur using decoder 358
                                             // 1006:   }
  3, 0,                                      // 1011:   case 0x3: {
  OPC_SwitchField, 11, 3,                    // 1013:    switch Inst[13:11] {
  4, 9,                                      // 1016:    case 0x4: {
  OPC_CheckField, 5, 3, 4,                   // 1018:     check Inst[7:5] == 0x4
  OPC_Decode, 190, 13, 235, 2,               // 1022:     decode to L4_ploadrit_abs using decoder 363
                                             // 1022:    }
  5, 9,                                      // 1027:    case 0x5: {
  OPC_CheckField, 5, 3, 4,                   // 1029:     check Inst[7:5] == 0x4
  OPC_Decode, 186, 13, 235, 2,               // 1033:     decode to L4_ploadrif_abs using decoder 363
                                             // 1033:    }
  6, 9,                                      // 1038:    case 0x6: {
  OPC_CheckField, 5, 3, 4,                   // 1040:     check Inst[7:5] == 0x4
  OPC_Decode, 192, 13, 235, 2,               // 1044:     decode to L4_ploadritnew_abs using decoder 363
                                             // 1044:    }
  7, 0,                                      // 1049:    case 0x7: {
  OPC_CheckField, 5, 3, 4,                   // 1051:     check Inst[7:5] == 0x4
  OPC_Decode, 188, 13, 235, 2,               // 1055:     decode to L4_ploadrifnew_abs using decoder 363
                                             // 1055:    }
                                             // 1055:    } // switch Inst[13:11]
                                             // 1055:   }
                                             // 1055:   } // switch Inst[26:25]
                                             // 1055:  }
  21, 0,                                     // 1060:  case 0x15: {
  OPC_SwitchField, 25, 2,                    // 1062:   switch Inst[26:25] {
  1, 13,                                     // 1065:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 1067:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   // 1071:    check Inst[7:6] == 0x2
  OPC_Decode, 242, 19, 236, 2,               // 1075:    decode to S4_storeri_ap using decoder 364
                                             // 1075:   }
  2, 9,                                      // 1080:   case 0x2: {
  OPC_CheckField, 7, 1, 1,                   // 1082:    check Inst[7] == 0x1
  OPC_Decode, 244, 19, 237, 2,               // 1086:    decode to S4_storeri_ur using decoder 365
                                             // 1086:   }
  3, 0,                                      // 1091:   case 0x3: {
  OPC_SwitchField, 2, 1,                     // 1093:    switch Inst[2] {
  0, 33,                                     // 1096:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1098:     switch Inst[13] {
  0, 13,                                     // 1101:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1103:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1107:      check Inst[7] == 0x1
  OPC_Decode, 201, 19, 238, 2,               // 1111:      decode to S4_pstorerit_abs using decoder 366
                                             // 1111:     }
  1, 0,                                      // 1116:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1118:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1122:      check Inst[7] == 0x1
  OPC_Decode, 203, 19, 238, 2,               // 1126:      decode to S4_pstoreritnew_abs using decoder 366
                                             // 1126:     }
                                             // 1126:     } // switch Inst[13]
                                             // 1126:    }
  1, 0,                                      // 1131:    case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1133:     switch Inst[13] {
  0, 13,                                     // 1136:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1138:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1142:      check Inst[7] == 0x1
  OPC_Decode, 186, 19, 238, 2,               // 1146:      decode to S4_pstorerif_abs using decoder 366
                                             // 1146:     }
  1, 0,                                      // 1151:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1153:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1157:      check Inst[7] == 0x1
  OPC_Decode, 188, 19, 238, 2,               // 1161:      decode to S4_pstorerifnew_abs using decoder 366
                                             // 1161:     }
                                             // 1161:     } // switch Inst[13]
                                             // 1161:    }
                                             // 1161:    } // switch Inst[2]
                                             // 1161:   }
                                             // 1161:   } // switch Inst[26:25]
                                             // 1161:  }
                                             // 1161:  } // switch Inst[31:27]
                                             // 1161: }
  13, 193, 2,                                // 1166: case 0xd: {
  OPC_SwitchField, 11, 2,                    // 1169:  switch Inst[12:11] {
  0, 104,                                    // 1172:  case 0x0: {
  OPC_SwitchField, 25, 7,                    // 1174:   switch Inst[31:25] {
  85, 13,                                    // 1177:   case 0x55: {
  OPC_CheckField, 13, 1, 0,                  // 1179:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   // 1183:    check Inst[7:6] == 0x2
  OPC_Decode, 227, 19, 239, 2,               // 1187:    decode to S4_storerbnew_ap using decoder 367
                                             // 1187:   }
  86, 9,                                     // 1192:   case 0x56: {
  OPC_CheckField, 7, 1, 1,                   // 1194:    check Inst[7] == 0x1
  OPC_Decode, 229, 19, 240, 2,               // 1198:    decode to S4_storerbnew_ur using decoder 368
                                             // 1198:   }
  87, 0,                                     // 1203:   case 0x57: {
  OPC_SwitchField, 2, 1,                     // 1205:    switch Inst[2] {
  0, 33,                                     // 1208:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1210:     switch Inst[13] {
  0, 13,                                     // 1213:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1215:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1219:      check Inst[7] == 0x1
  OPC_Decode, 136, 19, 241, 2,               // 1223:      decode to S4_pstorerbnewt_abs using decoder 369
                                             // 1223:     }
  1, 0,                                      // 1228:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1230:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1234:      check Inst[7] == 0x1
  OPC_Decode, 138, 19, 241, 2,               // 1238:      decode to S4_pstorerbnewtnew_abs using decoder 369
                                             // 1238:     }
                                             // 1238:     } // switch Inst[13]
                                             // 1238:    }
  1, 0,                                      // 1243:    case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1245:     switch Inst[13] {
  0, 13,                                     // 1248:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1250:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1254:      check Inst[7] == 0x1
  OPC_Decode, 131, 19, 241, 2,               // 1258:      decode to S4_pstorerbnewf_abs using decoder 369
                                             // 1258:     }
  1, 0,                                      // 1263:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1265:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1269:      check Inst[7] == 0x1
  OPC_Decode, 133, 19, 241, 2,               // 1273:      decode to S4_pstorerbnewfnew_abs using decoder 369
                                             // 1273:     }
                                             // 1273:     } // switch Inst[13]
                                             // 1273:    }
                                             // 1273:    } // switch Inst[2]
                                             // 1273:   }
                                             // 1273:   } // switch Inst[31:25]
                                             // 1273:  }
  1, 104,                                    // 1278:  case 0x1: {
  OPC_SwitchField, 25, 7,                    // 1280:   switch Inst[31:25] {
  85, 13,                                    // 1283:   case 0x55: {
  OPC_CheckField, 13, 1, 0,                  // 1285:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   // 1289:    check Inst[7:6] == 0x2
  OPC_Decode, 239, 19, 239, 2,               // 1293:    decode to S4_storerhnew_ap using decoder 367
                                             // 1293:   }
  86, 9,                                     // 1298:   case 0x56: {
  OPC_CheckField, 7, 1, 1,                   // 1300:    check Inst[7] == 0x1
  OPC_Decode, 241, 19, 240, 2,               // 1304:    decode to S4_storerhnew_ur using decoder 368
                                             // 1304:   }
  87, 0,                                     // 1309:   case 0x57: {
  OPC_SwitchField, 2, 1,                     // 1311:    switch Inst[2] {
  0, 33,                                     // 1314:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1316:     switch Inst[13] {
  0, 13,                                     // 1319:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1321:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1325:      check Inst[7] == 0x1
  OPC_Decode, 176, 19, 241, 2,               // 1329:      decode to S4_pstorerhnewt_abs using decoder 369
                                             // 1329:     }
  1, 0,                                      // 1334:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1336:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1340:      check Inst[7] == 0x1
  OPC_Decode, 178, 19, 241, 2,               // 1344:      decode to S4_pstorerhnewtnew_abs using decoder 369
                                             // 1344:     }
                                             // 1344:     } // switch Inst[13]
                                             // 1344:    }
  1, 0,                                      // 1349:    case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1351:     switch Inst[13] {
  0, 13,                                     // 1354:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1356:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1360:      check Inst[7] == 0x1
  OPC_Decode, 171, 19, 241, 2,               // 1364:      decode to S4_pstorerhnewf_abs using decoder 369
                                             // 1364:     }
  1, 0,                                      // 1369:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1371:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1375:      check Inst[7] == 0x1
  OPC_Decode, 173, 19, 241, 2,               // 1379:      decode to S4_pstorerhnewfnew_abs using decoder 369
                                             // 1379:     }
                                             // 1379:     } // switch Inst[13]
                                             // 1379:    }
                                             // 1379:    } // switch Inst[2]
                                             // 1379:   }
                                             // 1379:   } // switch Inst[31:25]
                                             // 1379:  }
  2, 0,                                      // 1384:  case 0x2: {
  OPC_SwitchField, 25, 7,                    // 1386:   switch Inst[31:25] {
  85, 13,                                    // 1389:   case 0x55: {
  OPC_CheckField, 13, 1, 0,                  // 1391:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   // 1395:    check Inst[7:6] == 0x2
  OPC_Decode, 245, 19, 239, 2,               // 1399:    decode to S4_storerinew_ap using decoder 367
                                             // 1399:   }
  86, 9,                                     // 1404:   case 0x56: {
  OPC_CheckField, 7, 1, 1,                   // 1406:    check Inst[7] == 0x1
  OPC_Decode, 247, 19, 240, 2,               // 1410:    decode to S4_storerinew_ur using decoder 368
                                             // 1410:   }
  87, 0,                                     // 1415:   case 0x57: {
  OPC_SwitchField, 2, 1,                     // 1417:    switch Inst[2] {
  0, 33,                                     // 1420:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1422:     switch Inst[13] {
  0, 13,                                     // 1425:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1427:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1431:      check Inst[7] == 0x1
  OPC_Decode, 196, 19, 241, 2,               // 1435:      decode to S4_pstorerinewt_abs using decoder 369
                                             // 1435:     }
  1, 0,                                      // 1440:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1442:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1446:      check Inst[7] == 0x1
  OPC_Decode, 198, 19, 241, 2,               // 1450:      decode to S4_pstorerinewtnew_abs using decoder 369
                                             // 1450:     }
                                             // 1450:     } // switch Inst[13]
                                             // 1450:    }
  1, 0,                                      // 1455:    case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1457:     switch Inst[13] {
  0, 13,                                     // 1460:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1462:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1466:      check Inst[7] == 0x1
  OPC_Decode, 191, 19, 241, 2,               // 1470:      decode to S4_pstorerinewf_abs using decoder 369
                                             // 1470:     }
  1, 0,                                      // 1475:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1477:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1481:      check Inst[7] == 0x1
  OPC_Decode, 193, 19, 241, 2,               // 1485:      decode to S4_pstorerinewfnew_abs using decoder 369
                                             // 1485:     }
                                             // 1485:     } // switch Inst[13]
                                             // 1485:    }
                                             // 1485:    } // switch Inst[2]
                                             // 1485:   }
                                             // 1485:   } // switch Inst[31:25]
                                             // 1485:  }
                                             // 1485:  } // switch Inst[12:11]
                                             // 1485: }
  14, 0,                                     // 1490: case 0xe: {
  OPC_SwitchField, 27, 5,                    // 1492:  switch Inst[31:27] {
  9, 4,                                      // 1495:  case 0x9: {
  OPC_Decode, 184, 16, 55,                   // 1497:   decode to PS_loadrdabs using decoder 55
                                             // 1497:  }
  19, 78,                                    // 1501:  case 0x13: {
  OPC_SwitchField, 25, 2,                    // 1503:   switch Inst[26:25] {
  1, 13,                                     // 1506:   case 0x1: {
  OPC_CheckField, 12, 2, 1,                  // 1508:    check Inst[13:12] == 0x1
  OPC_CheckField, 7, 1, 0,                   // 1512:    check Inst[7] == 0x0
  OPC_Decode, 143, 13, 233, 2,               // 1516:    decode to L4_loadrd_ap using decoder 361
                                             // 1516:   }
  2, 9,                                      // 1521:   case 0x2: {
  OPC_CheckField, 12, 1, 1,                  // 1523:    check Inst[12] == 0x1
  OPC_Decode, 145, 13, 234, 2,               // 1527:    decode to L4_loadrd_ur using decoder 362
                                             // 1527:   }
  3, 0,                                      // 1532:   case 0x3: {
  OPC_SwitchField, 11, 3,                    // 1534:    switch Inst[13:11] {
  4, 9,                                      // 1537:    case 0x4: {
  OPC_CheckField, 5, 3, 4,                   // 1539:     check Inst[7:5] == 0x4
  OPC_Decode, 174, 13, 242, 2,               // 1543:     decode to L4_ploadrdt_abs using decoder 370
                                             // 1543:    }
  5, 9,                                      // 1548:    case 0x5: {
  OPC_CheckField, 5, 3, 4,                   // 1550:     check Inst[7:5] == 0x4
  OPC_Decode, 170, 13, 242, 2,               // 1554:     decode to L4_ploadrdf_abs using decoder 370
                                             // 1554:    }
  6, 9,                                      // 1559:    case 0x6: {
  OPC_CheckField, 5, 3, 4,                   // 1561:     check Inst[7:5] == 0x4
  OPC_Decode, 176, 13, 242, 2,               // 1565:     decode to L4_ploadrdtnew_abs using decoder 370
                                             // 1565:    }
  7, 0,                                      // 1570:    case 0x7: {
  OPC_CheckField, 5, 3, 4,                   // 1572:     check Inst[7:5] == 0x4
  OPC_Decode, 172, 13, 242, 2,               // 1576:     decode to L4_ploadrdfnew_abs using decoder 370
                                             // 1576:    }
                                             // 1576:    } // switch Inst[13:11]
                                             // 1576:   }
                                             // 1576:   } // switch Inst[26:25]
                                             // 1576:  }
  21, 0,                                     // 1581:  case 0x15: {
  OPC_SwitchField, 25, 2,                    // 1583:   switch Inst[26:25] {
  1, 13,                                     // 1586:   case 0x1: {
  OPC_CheckField, 13, 1, 0,                  // 1588:    check Inst[13] == 0x0
  OPC_CheckField, 6, 2, 2,                   // 1592:    check Inst[7:6] == 0x2
  OPC_Decode, 230, 19, 243, 2,               // 1596:    decode to S4_storerd_ap using decoder 371
                                             // 1596:   }
  2, 9,                                      // 1601:   case 0x2: {
  OPC_CheckField, 7, 1, 1,                   // 1603:    check Inst[7] == 0x1
  OPC_Decode, 232, 19, 244, 2,               // 1607:    decode to S4_storerd_ur using decoder 372
                                             // 1607:   }
  3, 0,                                      // 1612:   case 0x3: {
  OPC_SwitchField, 2, 1,                     // 1614:    switch Inst[2] {
  0, 33,                                     // 1617:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 1619:     switch Inst[13] {
  0, 13,                                     // 1622:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1624:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1628:      check Inst[7] == 0x1
  OPC_Decode, 151, 19, 245, 2,               // 1632:      decode to S4_pstorerdt_abs using decoder 373
                                             // 1632:     }
  1, 0,                                      // 1637:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1639:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1643:      check Inst[7] == 0x1
  OPC_Decode, 153, 19, 245, 2,               // 1647:      decode to S4_pstorerdtnew_abs using decoder 373
                                             // 1647:     }
                                             // 1647:     } // switch Inst[13]
                                             // 1647:    }
  1, 0,                                      // 1652:    case 0x1: {
  OPC_SwitchField, 13, 1,                    // 1654:     switch Inst[13] {
  0, 13,                                     // 1657:     case 0x0: {
  OPC_CheckField, 18, 3, 0,                  // 1659:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1663:      check Inst[7] == 0x1
  OPC_Decode, 146, 19, 245, 2,               // 1667:      decode to S4_pstorerdf_abs using decoder 373
                                             // 1667:     }
  1, 0,                                      // 1672:     case 0x1: {
  OPC_CheckField, 18, 3, 0,                  // 1674:      check Inst[20:18] == 0x0
  OPC_CheckField, 7, 1, 1,                   // 1678:      check Inst[7] == 0x1
  OPC_Decode, 148, 19, 245, 2,               // 1682:      decode to S4_pstorerdfnew_abs using decoder 373
                                             // 1682:     }
                                             // 1682:     } // switch Inst[13]
                                             // 1682:    }
                                             // 1682:    } // switch Inst[2]
                                             // 1682:   }
                                             // 1682:   } // switch Inst[26:25]
                                             // 1682:  }
                                             // 1682:  } // switch Inst[31:27]
                                             // 1682: }
                                             // 1682: } // switch Inst[24:21]
};
static const uint8_t DecoderTableSUBINSN_A32[317] = {
  OPC_SwitchField, 11, 2,                    //   0: switch Inst[12:11] {
  0, 9,                                      //   3: case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //   5:  check Inst[31:28] == 0x0
  OPC_Decode, 150, 20, 246, 2,               //   9:  decode to SA1_addi using decoder 374
                                             //   9: }
  1, 25,                                     //  14: case 0x1: {
  OPC_SwitchField, 10, 1,                    //  16:  switch Inst[10] {
  0, 9,                                      //  19:  case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  21:   check Inst[31:28] == 0x0
  OPC_Decode, 167, 20, 247, 2,               //  25:   decode to SA1_seti using decoder 375
                                             //  25:  }
  1, 0,                                      //  30:  case 0x1: {
  OPC_CheckField, 28, 4, 0,                  //  32:   check Inst[31:28] == 0x0
  OPC_Decode, 152, 20, 248, 2,               //  36:   decode to SA1_addsp using decoder 376
                                             //  36:  }
                                             //  36:  } // switch Inst[10]
                                             //  36: }
  2, 91,                                     //  41: case 0x2: {
  OPC_SwitchField, 8, 3,                     //  43:  switch Inst[10:8] {
  0, 9,                                      //  46:  case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  48:   check Inst[31:28] == 0x0
  OPC_Decode, 171, 20, 249, 2,               //  52:   decode to SA1_tfr using decoder 377
                                             //  52:  }
  1, 9,                                      //  57:  case 0x1: {
  OPC_CheckField, 28, 4, 0,                  //  59:   check Inst[31:28] == 0x0
  OPC_Decode, 166, 20, 249, 2,               //  63:   decode to SA1_inc using decoder 377
                                             //  63:  }
  2, 9,                                      //  68:  case 0x2: {
  OPC_CheckField, 28, 4, 0,                  //  70:   check Inst[31:28] == 0x0
  OPC_Decode, 153, 20, 249, 2,               //  74:   decode to SA1_and1 using decoder 377
                                             //  74:  }
  3, 9,                                      //  79:  case 0x3: {
  OPC_CheckField, 28, 4, 0,                  //  81:   check Inst[31:28] == 0x0
  OPC_Decode, 165, 20, 250, 2,               //  85:   decode to SA1_dec using decoder 378
                                             //  85:  }
  4, 9,                                      //  90:  case 0x4: {
  OPC_CheckField, 28, 4, 0,                  //  92:   check Inst[31:28] == 0x0
  OPC_Decode, 170, 20, 249, 2,               //  96:   decode to SA1_sxth using decoder 377
                                             //  96:  }
  5, 9,                                      // 101:  case 0x5: {
  OPC_CheckField, 28, 4, 0,                  // 103:   check Inst[31:28] == 0x0
  OPC_Decode, 169, 20, 249, 2,               // 107:   decode to SA1_sxtb using decoder 377
                                             // 107:  }
  6, 9,                                      // 112:  case 0x6: {
  OPC_CheckField, 28, 4, 0,                  // 114:   check Inst[31:28] == 0x0
  OPC_Decode, 173, 20, 249, 2,               // 118:   decode to SA1_zxth using decoder 377
                                             // 118:  }
  7, 0,                                      // 123:  case 0x7: {
  OPC_CheckField, 28, 4, 0,                  // 125:   check Inst[31:28] == 0x0
  OPC_Decode, 172, 20, 249, 2,               // 129:   decode to SA1_zxtb using decoder 377
                                             // 129:  }
                                             // 129:  } // switch Inst[10:8]
                                             // 129: }
  3, 0,                                      // 134: case 0x3: {
  OPC_SwitchField, 8, 3,                     // 136:  switch Inst[10:8] {
  0, 9,                                      // 139:  case 0x0: {
  OPC_CheckField, 28, 4, 0,                  // 141:   check Inst[31:28] == 0x0
  OPC_Decode, 151, 20, 251, 2,               // 145:   decode to SA1_addrx using decoder 379
                                             // 145:  }
  1, 13,                                     // 150:  case 0x1: {
  OPC_CheckField, 28, 4, 0,                  // 152:   check Inst[31:28] == 0x0
  OPC_CheckField, 2, 2, 0,                   // 156:   check Inst[3:2] == 0x0
  OPC_Decode, 158, 20, 252, 2,               // 160:   decode to SA1_cmpeqi using decoder 380
                                             // 160:  }
  2, 58,                                     // 165:  case 0x2: {
  OPC_SwitchField, 4, 4,                     // 167:   switch Inst[7:4] {
  0, 9,                                      // 170:   case 0x0: {
  OPC_CheckField, 28, 4, 0,                  // 172:    check Inst[31:28] == 0x0
  OPC_Decode, 168, 20, 253, 2,               // 176:    decode to SA1_setin1 using decoder 381
                                             // 176:   }
  4, 9,                                      // 181:   case 0x4: {
  OPC_CheckField, 28, 4, 0,                  // 183:    check Inst[31:28] == 0x0
  OPC_Decode, 157, 20, 254, 2,               // 187:    decode to SA1_clrtnew using decoder 382
                                             // 187:   }
  5, 9,                                      // 192:   case 0x5: {
  OPC_CheckField, 28, 4, 0,                  // 194:    check Inst[31:28] == 0x0
  OPC_Decode, 155, 20, 254, 2,               // 198:    decode to SA1_clrfnew using decoder 382
                                             // 198:   }
  6, 9,                                      // 203:   case 0x6: {
  OPC_CheckField, 28, 4, 0,                  // 205:    check Inst[31:28] == 0x0
  OPC_Decode, 156, 20, 254, 2,               // 209:    decode to SA1_clrt using decoder 382
                                             // 209:   }
  7, 0,                                      // 214:   case 0x7: {
  OPC_CheckField, 28, 4, 0,                  // 216:    check Inst[31:28] == 0x0
  OPC_Decode, 154, 20, 254, 2,               // 220:    decode to SA1_clrf using decoder 382
                                             // 220:   }
                                             // 220:   } // switch Inst[7:4]
                                             // 220:  }
  4, 63,                                     // 225:  case 0x4: {
  OPC_SwitchField, 3, 2,                     // 227:   switch Inst[4:3] {
  0, 13,                                     // 230:   case 0x0: {
  OPC_CheckField, 28, 4, 0,                  // 232:    check Inst[31:28] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 236:    check Inst[7] == 0x0
  OPC_Decode, 159, 20, 255, 2,               // 240:    decode to SA1_combine0i using decoder 383
                                             // 240:   }
  1, 13,                                     // 245:   case 0x1: {
  OPC_CheckField, 28, 4, 0,                  // 247:    check Inst[31:28] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 251:    check Inst[7] == 0x0
  OPC_Decode, 160, 20, 255, 2,               // 255:    decode to SA1_combine1i using decoder 383
                                             // 255:   }
  2, 13,                                     // 260:   case 0x2: {
  OPC_CheckField, 28, 4, 0,                  // 262:    check Inst[31:28] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 266:    check Inst[7] == 0x0
  OPC_Decode, 161, 20, 255, 2,               // 270:    decode to SA1_combine2i using decoder 383
                                             // 270:   }
  3, 0,                                      // 275:   case 0x3: {
  OPC_CheckField, 28, 4, 0,                  // 277:    check Inst[31:28] == 0x0
  OPC_CheckField, 7, 1, 0,                   // 281:    check Inst[7] == 0x0
  OPC_Decode, 162, 20, 255, 2,               // 285:    decode to SA1_combine3i using decoder 383
                                             // 285:   }
                                             // 285:   } // switch Inst[4:3]
                                             // 285:  }
  5, 0,                                      // 290:  case 0x5: {
  OPC_SwitchField, 3, 1,                     // 292:   switch Inst[3] {
  0, 9,                                      // 295:   case 0x0: {
  OPC_CheckField, 28, 4, 0,                  // 297:    check Inst[31:28] == 0x0
  OPC_Decode, 164, 20, 128, 3,               // 301:    decode to SA1_combinezr using decoder 384
                                             // 301:   }
  1, 0,                                      // 306:   case 0x1: {
  OPC_CheckField, 28, 4, 0,                  // 308:    check Inst[31:28] == 0x0
  OPC_Decode, 163, 20, 128, 3,               // 312:    decode to SA1_combinerz using decoder 384
                                             // 312:   }
                                             // 312:   } // switch Inst[3]
                                             // 312:  }
                                             // 312:  } // switch Inst[10:8]
                                             // 312: }
                                             // 312: } // switch Inst[12:11]
};
static const uint8_t DecoderTableSUBINSN_L132[25] = {
  OPC_SwitchField, 12, 1,                    //  0: switch Inst[12] {
  0, 9,                                      //  3: case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  5:  check Inst[31:28] == 0x0
  OPC_Decode, 182, 20, 129, 3,               //  9:  decode to SL1_loadri_io using decoder 385
                                             //  9: }
  1, 0,                                      // 14: case 0x1: {
  OPC_CheckField, 28, 4, 0,                  // 16:  check Inst[31:28] == 0x0
  OPC_Decode, 183, 20, 130, 3,               // 20:  decode to SL1_loadrub_io using decoder 386
                                             // 20: }
                                             // 20: } // switch Inst[12]
};
static const uint8_t DecoderTableSUBINSN_L232[188] = {
  OPC_SwitchField, 11, 2,                    //   0: switch Inst[12:11] {
  0, 9,                                      //   3: case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //   5:  check Inst[31:28] == 0x0
  OPC_Decode, 192, 20, 131, 3,               //   9:  decode to SL2_loadrh_io using decoder 387
                                             //   9: }
  1, 9,                                      //  14: case 0x1: {
  OPC_CheckField, 28, 4, 0,                  //  16:  check Inst[31:28] == 0x0
  OPC_Decode, 194, 20, 131, 3,               //  20:  decode to SL2_loadruh_io using decoder 387
                                             //  20: }
  2, 9,                                      //  25: case 0x2: {
  OPC_CheckField, 28, 4, 0,                  //  27:  check Inst[31:28] == 0x0
  OPC_Decode, 190, 20, 132, 3,               //  31:  decode to SL2_loadrb_io using decoder 388
                                             //  31: }
  3, 0,                                      //  36: case 0x3: {
  OPC_SwitchField, 9, 2,                     //  38:  switch Inst[10:9] {
  2, 9,                                      //  41:  case 0x2: {
  OPC_CheckField, 28, 4, 0,                  //  43:   check Inst[31:28] == 0x0
  OPC_Decode, 193, 20, 133, 3,               //  47:   decode to SL2_loadri_sp using decoder 389
                                             //  47:  }
  3, 0,                                      //  52:  case 0x3: {
  OPC_SwitchField, 8, 1,                     //  54:   switch Inst[8] {
  0, 9,                                      //  57:   case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  59:    check Inst[31:28] == 0x0
  OPC_Decode, 191, 20, 134, 3,               //  63:    decode to SL2_loadrd_sp using decoder 390
                                             //  63:   }
  1, 0,                                      //  68:   case 0x1: {
  OPC_SwitchField, 0, 8,                     //  70:    switch Inst[7:0] {
  0, 8,                                      //  73:    case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  75:     check Inst[31:28] == 0x0
  OPC_Decode, 184, 20, 63,                   //  79:     decode to SL2_deallocframe using decoder 63
                                             //  79:    }
  64, 8,                                     //  83:    case 0x40: {
  OPC_CheckField, 28, 4, 0,                  //  85:     check Inst[31:28] == 0x0
  OPC_Decode, 195, 20, 63,                   //  89:     decode to SL2_return using decoder 63
                                             //  89:    }
  68, 8,                                     //  93:    case 0x44: {
  OPC_CheckField, 28, 4, 0,                  //  95:     check Inst[31:28] == 0x0
  OPC_Decode, 198, 20, 63,                   //  99:     decode to SL2_return_t using decoder 63
                                             //  99:    }
  69, 8,                                     // 103:    case 0x45: {
  OPC_CheckField, 28, 4, 0,                  // 105:     check Inst[31:28] == 0x0
  OPC_Decode, 196, 20, 63,                   // 109:     decode to SL2_return_f using decoder 63
                                             // 109:    }
  70, 8,                                     // 113:    case 0x46: {
  OPC_CheckField, 28, 4, 0,                  // 115:     check Inst[31:28] == 0x0
  OPC_Decode, 199, 20, 63,                   // 119:     decode to SL2_return_tnew using decoder 63
                                             // 119:    }
  71, 8,                                     // 123:    case 0x47: {
  OPC_CheckField, 28, 4, 0,                  // 125:     check Inst[31:28] == 0x0
  OPC_Decode, 197, 20, 63,                   // 129:     decode to SL2_return_fnew using decoder 63
                                             // 129:    }
  192, 1, 8,                                 // 133:    case 0xc0: {
  OPC_CheckField, 28, 4, 0,                  // 136:     check Inst[31:28] == 0x0
  OPC_Decode, 185, 20, 63,                   // 140:     decode to SL2_jumpr31 using decoder 63
                                             // 140:    }
  196, 1, 8,                                 // 144:    case 0xc4: {
  OPC_CheckField, 28, 4, 0,                  // 147:     check Inst[31:28] == 0x0
  OPC_Decode, 188, 20, 63,                   // 151:     decode to SL2_jumpr31_t using decoder 63
                                             // 151:    }
  197, 1, 8,                                 // 155:    case 0xc5: {
  OPC_CheckField, 28, 4, 0,                  // 158:     check Inst[31:28] == 0x0
  OPC_Decode, 186, 20, 63,                   // 162:     decode to SL2_jumpr31_f using decoder 63
                                             // 162:    }
  198, 1, 8,                                 // 166:    case 0xc6: {
  OPC_CheckField, 28, 4, 0,                  // 169:     check Inst[31:28] == 0x0
  OPC_Decode, 189, 20, 63,                   // 173:     decode to SL2_jumpr31_tnew using decoder 63
                                             // 173:    }
  199, 1, 0,                                 // 177:    case 0xc7: {
  OPC_CheckField, 28, 4, 0,                  // 180:     check Inst[31:28] == 0x0
  OPC_Decode, 187, 20, 63,                   // 184:     decode to SL2_jumpr31_fnew using decoder 63
                                             // 184:    }
                                             // 184:    } // switch Inst[7:0]
                                             // 184:   }
                                             // 184:   } // switch Inst[8]
                                             // 184:  }
                                             // 184:  } // switch Inst[10:9]
                                             // 184: }
                                             // 184: } // switch Inst[12:11]
};
static const uint8_t DecoderTableSUBINSN_S132[25] = {
  OPC_SwitchField, 12, 1,                    //  0: switch Inst[12] {
  0, 9,                                      //  3: case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  5:  check Inst[31:28] == 0x0
  OPC_Decode, 201, 20, 135, 3,               //  9:  decode to SS1_storew_io using decoder 391
                                             //  9: }
  1, 0,                                      // 14: case 0x1: {
  OPC_CheckField, 28, 4, 0,                  // 16:  check Inst[31:28] == 0x0
  OPC_Decode, 200, 20, 136, 3,               // 20:  decode to SS1_storeb_io using decoder 392
                                             // 20: }
                                             // 20: } // switch Inst[12]
};
static const uint8_t DecoderTableSUBINSN_S232[109] = {
  OPC_SwitchField, 11, 2,                    //   0: switch Inst[12:11] {
  0, 9,                                      //   3: case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //   5:  check Inst[31:28] == 0x0
  OPC_Decode, 206, 20, 137, 3,               //   9:  decode to SS2_storeh_io using decoder 393
                                             //   9: }
  1, 25,                                     //  14: case 0x1: {
  OPC_SwitchField, 9, 2,                     //  16:  switch Inst[10:9] {
  0, 9,                                      //  19:  case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  21:   check Inst[31:28] == 0x0
  OPC_Decode, 207, 20, 138, 3,               //  25:   decode to SS2_storew_sp using decoder 394
                                             //  25:  }
  1, 0,                                      //  30:  case 0x1: {
  OPC_CheckField, 28, 4, 0,                  //  32:   check Inst[31:28] == 0x0
  OPC_Decode, 205, 20, 139, 3,               //  36:   decode to SS2_stored_sp using decoder 395
                                             //  36:  }
                                             //  36:  } // switch Inst[10:9]
                                             //  36: }
  2, 47,                                     //  41: case 0x2: {
  OPC_SwitchField, 8, 3,                     //  43:  switch Inst[10:8] {
  0, 9,                                      //  46:  case 0x0: {
  OPC_CheckField, 28, 4, 0,                  //  48:   check Inst[31:28] == 0x0
  OPC_Decode, 208, 20, 140, 3,               //  52:   decode to SS2_storewi0 using decoder 396
                                             //  52:  }
  1, 9,                                      //  57:  case 0x1: {
  OPC_CheckField, 28, 4, 0,                  //  59:   check Inst[31:28] == 0x0
  OPC_Decode, 209, 20, 140, 3,               //  63:   decode to SS2_storewi1 using decoder 396
                                             //  63:  }
  2, 9,                                      //  68:  case 0x2: {
  OPC_CheckField, 28, 4, 0,                  //  70:   check Inst[31:28] == 0x0
  OPC_Decode, 203, 20, 141, 3,               //  74:   decode to SS2_storebi0 using decoder 397
                                             //  74:  }
  3, 0,                                      //  79:  case 0x3: {
  OPC_CheckField, 28, 4, 0,                  //  81:   check Inst[31:28] == 0x0
  OPC_Decode, 204, 20, 141, 3,               //  85:   decode to SS2_storebi1 using decoder 397
                                             //  85:  }
                                             //  85:  } // switch Inst[10:8]
                                             //  85: }
  3, 0,                                      //  90: case 0x3: {
  OPC_CheckField, 28, 4, 0,                  //  92:  check Inst[31:28] == 0x0
  OPC_CheckField, 9, 2, 2,                   //  96:  check Inst[10:9] == 0x2
  OPC_CheckField, 0, 4, 0,                   // 100:  check Inst[3:0] == 0x0
  OPC_Decode, 202, 20, 142, 3,               // 104:  decode to SS2_allocframe using decoder 398
                                             // 104: }
                                             // 104: } // switch Inst[12:11]
};
// Handling 399 cases.
template <typename InsnType>
static DecodeStatus decodeToMCInst(unsigned Idx, DecodeStatus S, InsnType insn, MCInst &MI, uint64_t Address, const MCDisassembler *Decoder, bool &DecodeComplete) {
  DecodeComplete = true;
  using TmpType = std::conditional_t<std::is_integral<InsnType>::value, InsnType, uint64_t>;
  TmpType tmp;
  switch (Idx) {
  default: llvm_unreachable("Invalid decoder index!");
  case 0:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 14) << 6;
    tmp |= fieldFromInstruction(insn, 16, 12) << 20;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 1:
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 2:
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, n1ConstDecoder(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 3:
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 4:
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 5:
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 6:
    tmp = fieldFromInstruction(insn, 8, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 7:
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 8:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 9:
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 10:
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 11:
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, n1ConstDecoder(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2) << 9;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 12:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 13:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 14:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 15:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 16:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 17:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 5);
    tmp |= fieldFromInstruction(insn, 13, 1) << 5;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 18:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 5);
    tmp |= fieldFromInstruction(insn, 13, 1) << 5;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 19:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 5);
    tmp |= fieldFromInstruction(insn, 13, 1) << 5;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 20:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 21:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 22:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 23:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 24:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 25:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 7);
    tmp |= fieldFromInstruction(insn, 13, 1) << 7;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 26:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 7);
    tmp |= fieldFromInstruction(insn, 13, 1) << 7;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 27:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 7);
    tmp |= fieldFromInstruction(insn, 13, 1) << 7;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 28:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 29:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 30:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 31:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 32:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 33:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 34:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5);
    tmp |= fieldFromInstruction(insn, 13, 1) << 5;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 35:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8);
    tmp |= fieldFromInstruction(insn, 13, 1) << 8;
    tmp |= fieldFromInstruction(insn, 16, 5) << 9;
    tmp |= fieldFromInstruction(insn, 25, 2) << 14;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 36:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 6;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 37:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 9;
    tmp |= fieldFromInstruction(insn, 16, 5) << 10;
    tmp |= fieldFromInstruction(insn, 25, 2) << 15;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 38:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 7;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 39:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 10;
    tmp |= fieldFromInstruction(insn, 16, 5) << 11;
    tmp |= fieldFromInstruction(insn, 25, 2) << 16;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 40:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5);
    tmp |= fieldFromInstruction(insn, 13, 1) << 5;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 41:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8);
    tmp |= fieldFromInstruction(insn, 13, 1) << 8;
    tmp |= fieldFromInstruction(insn, 16, 5) << 9;
    tmp |= fieldFromInstruction(insn, 25, 2) << 14;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 42:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 6;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 43:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 9;
    tmp |= fieldFromInstruction(insn, 16, 5) << 10;
    tmp |= fieldFromInstruction(insn, 25, 2) << 15;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 44:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 7;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 45:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 10;
    tmp |= fieldFromInstruction(insn, 16, 5) << 11;
    tmp |= fieldFromInstruction(insn, 25, 2) << 16;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 46:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 5) << 3;
    tmp |= fieldFromInstruction(insn, 13, 1) << 8;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 47:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 3;
    tmp |= fieldFromInstruction(insn, 13, 1) << 11;
    tmp |= fieldFromInstruction(insn, 16, 5) << 12;
    tmp |= fieldFromInstruction(insn, 25, 2) << 17;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 48:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 49:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 16, 5) << 9;
    tmp |= fieldFromInstruction(insn, 25, 2) << 14;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 50:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 6) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 51:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 1;
    tmp |= fieldFromInstruction(insn, 16, 5) << 10;
    tmp |= fieldFromInstruction(insn, 25, 2) << 15;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 52:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 6) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 53:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 2;
    tmp |= fieldFromInstruction(insn, 16, 5) << 11;
    tmp |= fieldFromInstruction(insn, 25, 2) << 16;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 54:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 6) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 55:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 3;
    tmp |= fieldFromInstruction(insn, 16, 5) << 12;
    tmp |= fieldFromInstruction(insn, 25, 2) << 17;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 56:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 57:
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 58:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 3);
    tmp |= fieldFromInstruction(insn, 8, 5) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 59:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 3);
    tmp |= fieldFromInstruction(insn, 8, 5) << 3;
    tmp |= fieldFromInstruction(insn, 16, 2) << 8;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 60:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 3);
    tmp |= fieldFromInstruction(insn, 8, 5) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 61:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 62:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 63:
    return S;
  case 64:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 13) << 2;
    tmp |= fieldFromInstruction(insn, 16, 9) << 15;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 65:
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 7) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 9;
    tmp |= fieldFromInstruction(insn, 16, 5) << 10;
    tmp |= fieldFromInstruction(insn, 22, 2) << 15;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 66:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 2) << 2;
    tmp |= fieldFromInstruction(insn, 8, 5) << 4;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 67:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 11) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 13;
    tmp |= fieldFromInstruction(insn, 21, 1) << 14;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 68:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeGuestRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 69:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeCtrRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 70:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 71:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeGuestRegs64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 72:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeCtrRegs64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 73:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 74:
    tmp = fieldFromInstruction(insn, 0, 7);
    if (!Check(S, DecodeSysRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 75:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeCtrRegs64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 76:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeGuestRegs64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 77:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 2) << 2;
    tmp |= fieldFromInstruction(insn, 8, 5) << 4;
    if (!Check(S, brtargetDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 5, 3) << 2;
    tmp |= fieldFromInstruction(insn, 16, 5) << 5;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 78:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeCtrRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 79:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeGuestRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 80:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 81:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 82:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 83:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 84:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 85:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 86:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 87:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 88:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 89:
    tmp = fieldFromInstruction(insn, 0, 7);
    if (!Check(S, DecodeSysRegs64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 90:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, sgp10ConstDecoder(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 91:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 7);
    if (!Check(S, DecodeSysRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 92:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 7);
    if (!Check(S, DecodeSysRegs64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 93:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 94:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 14);
    tmp |= fieldFromInstruction(insn, 22, 2) << 14;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 95:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 21, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 96:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 21, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 97:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 98:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 99:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 100:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 101:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 102:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 103:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 104:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 16, 5) << 9;
    tmp |= fieldFromInstruction(insn, 22, 2) << 14;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 105:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 23, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 13, 1);
    tmp |= fieldFromInstruction(insn, 16, 7) << 1;
    if (!Check(S, s8_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 106:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 13, 1);
    tmp |= fieldFromInstruction(insn, 16, 7) << 1;
    if (!Check(S, s8_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 107:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s8_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 13, 1);
    tmp |= fieldFromInstruction(insn, 16, 5) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 108:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 21, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 8);
    tmp |= fieldFromInstruction(insn, 16, 4) << 8;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 109:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 110:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 111:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 112:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 113:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 21, 3) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 114:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 115:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 21, 3) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 116:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 117:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 118:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 119:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 120:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 21, 1) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, s6_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 121:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, s6_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 122:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 123:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 124:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 125:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 126:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 127:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 128:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 129:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 6);
    if (!Check(S, s6_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 130:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 21, 2) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 131:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 21, 2) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 132:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 133:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 21, 2) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 134:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 135:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 136:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 11) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 137:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 138:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 1;
    tmp |= fieldFromInstruction(insn, 25, 2) << 10;
    if (!Check(S, s31_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 139:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 140:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 141:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 142:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 1;
    tmp |= fieldFromInstruction(insn, 25, 2) << 10;
    if (!Check(S, s31_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 143:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 144:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 145:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 146:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 25, 2) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 147:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 148:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 149:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 2;
    tmp |= fieldFromInstruction(insn, 25, 2) << 11;
    if (!Check(S, s30_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 150:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 151:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 152:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 153:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 25, 2) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 154:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 155:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 156:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 9, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 157:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 9, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 158:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 2;
    tmp |= fieldFromInstruction(insn, 25, 2) << 11;
    if (!Check(S, s30_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 159:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 160:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 161:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 9, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 162:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9) << 3;
    tmp |= fieldFromInstruction(insn, 25, 2) << 12;
    if (!Check(S, s29_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 163:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 3;
    if (!Check(S, s4_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 164:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 3;
    if (!Check(S, s4_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 165:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 9, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4) << 3;
    if (!Check(S, s4_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 166:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 167:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 11) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 168:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 169:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 170:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 171:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 172:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8);
    tmp |= fieldFromInstruction(insn, 13, 1) << 8;
    tmp |= fieldFromInstruction(insn, 25, 2) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 173:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 174:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 175:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 176:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 177:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 9;
    tmp |= fieldFromInstruction(insn, 25, 2) << 10;
    if (!Check(S, s31_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 178:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 179:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 180:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 181:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 10;
    tmp |= fieldFromInstruction(insn, 25, 2) << 11;
    if (!Check(S, s30_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 182:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 183:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 184:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 185:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8);
    tmp |= fieldFromInstruction(insn, 13, 1) << 8;
    tmp |= fieldFromInstruction(insn, 25, 2) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 186:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 187:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 188:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 189:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4);
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 190:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 9;
    tmp |= fieldFromInstruction(insn, 25, 2) << 10;
    if (!Check(S, s31_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 191:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 192:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 193:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 1;
    if (!Check(S, s4_1ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 194:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 10;
    tmp |= fieldFromInstruction(insn, 25, 2) << 11;
    if (!Check(S, s30_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 195:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 196:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 197:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 2;
    if (!Check(S, s4_2ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 198:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 8) << 3;
    tmp |= fieldFromInstruction(insn, 13, 1) << 11;
    tmp |= fieldFromInstruction(insn, 25, 2) << 12;
    if (!Check(S, s29_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 199:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 3;
    if (!Check(S, s4_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 200:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 201:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 3;
    if (!Check(S, s4_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 202:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 4) << 3;
    if (!Check(S, s4_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 203:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 7) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 204:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 205:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 206:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 207:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 208:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 209:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 210:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 211:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 212:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 213:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 214:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 215:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 1);
    tmp |= fieldFromInstruction(insn, 16, 5) << 1;
    if (!Check(S, s6_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 216:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 217:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 218:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 219:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 220:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 221:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 222:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 223:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 224:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 225:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 226:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 227:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 228:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 229:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    tmp |= fieldFromInstruction(insn, 21, 2) << 4;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 230:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    tmp |= fieldFromInstruction(insn, 21, 2) << 4;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 5);
    tmp |= fieldFromInstruction(insn, 23, 1) << 5;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 231:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 232:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 233:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 9);
    tmp |= fieldFromInstruction(insn, 21, 1) << 9;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 234:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    tmp |= fieldFromInstruction(insn, 21, 2) << 4;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 235:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    tmp |= fieldFromInstruction(insn, 21, 2) << 4;
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 236:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 237:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s8_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 238:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 7);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 239:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 240:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 241:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 242:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s8_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 243:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 7);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 244:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 1);
    tmp |= fieldFromInstruction(insn, 5, 3) << 1;
    tmp |= fieldFromInstruction(insn, 13, 1) << 4;
    tmp |= fieldFromInstruction(insn, 21, 3) << 5;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 245:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3) << 2;
    tmp |= fieldFromInstruction(insn, 13, 1) << 5;
    tmp |= fieldFromInstruction(insn, 21, 2) << 6;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 246:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    tmp |= fieldFromInstruction(insn, 21, 2) << 4;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 247:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 248:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 249:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 250:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 251:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 252:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 253:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 254:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 255:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 256:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 257:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 258:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 259:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 260:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 261:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 262:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 263:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 264:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 265:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 266:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 267:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 268:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 269:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 270:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 271:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 272:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 273:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 274:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 275:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 276:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 277:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 278:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 279:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 280:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 281:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 282:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 283:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 284:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 285:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 286:
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 287:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 288:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 289:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 290:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 291:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 292:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeIntRegsLow8RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 293:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 294:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 295:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 296:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 297:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 298:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 299:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 300:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 301:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 302:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 303:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 304:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 305:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 306:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 307:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 308:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 309:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 310:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 311:
    tmp = fieldFromInstruction(insn, 8, 1);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 312:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 313:
    tmp = fieldFromInstruction(insn, 22, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 314:
    tmp = fieldFromInstruction(insn, 22, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 1);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 315:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 316:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 317:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 318:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 319:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 320:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 321:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 322:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 323:
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 324:
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 325:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 326:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 327:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 328:
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 329:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 330:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 331:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 332:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 333:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 334:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 335:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 336:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 337:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 338:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 339:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 340:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 341:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 342:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 343:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 344:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 345:
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 3);
    tmp |= fieldFromInstruction(insn, 13, 1) << 3;
    if (!Check(S, s4_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 346:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, s3_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 347:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 348:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 349:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 350:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 351:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 352:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 353:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 354:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 355:
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeHvxQRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    if (!Check(S, DecodeModRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxWRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 356:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeHvxVRRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 357:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 2);
    tmp |= fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 358:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 2);
    tmp |= fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 359:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 2);
    tmp |= fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 360:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 2);
    tmp |= fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 361:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 2);
    tmp |= fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 362:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 7, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 2);
    tmp |= fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 363:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 9, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 1);
    tmp |= fieldFromInstruction(insn, 16, 5) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 364:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 365:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 6, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 366:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 4);
    tmp |= fieldFromInstruction(insn, 16, 2) << 4;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 367:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 368:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 6, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 369:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 4);
    tmp |= fieldFromInstruction(insn, 16, 2) << 4;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 370:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 9, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 8, 1);
    tmp |= fieldFromInstruction(insn, 16, 5) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 371:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 372:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeIntRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 6, 1);
    tmp |= fieldFromInstruction(insn, 13, 1) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 373:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodePredRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 4);
    tmp |= fieldFromInstruction(insn, 16, 2) << 4;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 5);
    if (!Check(S, DecodeDoubleRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 374:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 7);
    if (!Check(S, s32_0ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 375:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 6);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 376:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 6) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 377:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 378:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, n1ConstDecoder(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 379:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 380:
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 381:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, n1ConstDecoder(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 382:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 383:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeGeneralDoubleLow8RegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 384:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeGeneralDoubleLow8RegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 385:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 386:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 387:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 388:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 389:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 5) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 390:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeGeneralDoubleLow8RegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 5) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 391:
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 392:
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 4);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 393:
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 3) << 1;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 394:
    tmp = fieldFromInstruction(insn, 4, 5) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 395:
    tmp = fieldFromInstruction(insn, 3, 6) << 3;
    if (!Check(S, s6_3ImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeGeneralDoubleLow8RegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 396:
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4) << 2;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 397:
    tmp = fieldFromInstruction(insn, 4, 4);
    if (!Check(S, DecodeGeneralSubRegsRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 398:
    tmp = fieldFromInstruction(insn, 4, 5) << 3;
    if (!Check(S, unsignedImmDecoder(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  }
}
static bool checkDecoderPredicate(unsigned Idx, const FeatureBitset &FB) {
  switch (Idx) {
  default: llvm_unreachable("Invalid index!");
  case 0:
    return FB[Hexagon::ArchV73];
  case 1:
    return FB[Hexagon::ArchV60];
  case 2:
    return FB[Hexagon::FeaturePreV65];
  case 3:
    return FB[Hexagon::ArchV65];
  case 4:
    return FB[Hexagon::ArchV66];
  case 5:
    return FB[Hexagon::ArchV67];
  case 6:
    return FB[Hexagon::ArchV81];
  case 7:
    return FB[Hexagon::ArchV62];
  case 8:
    return FB[Hexagon::ArchV67] && FB[Hexagon::ExtensionAudio];
  case 9:
    return FB[Hexagon::ArchV68];
  case 10:
    return FB[Hexagon::FeatureCabac];
  case 11:
    return FB[Hexagon::ArchV55];
  case 12:
    return FB[Hexagon::ExtensionHVXV62];
  case 13:
    return FB[Hexagon::ExtensionHVXV65];
  case 14:
    return FB[Hexagon::ExtensionHVXV81];
  case 15:
    return FB[Hexagon::ExtensionHVXV60];
  case 16:
    return FB[Hexagon::ExtensionHVXV66] && FB[Hexagon::ExtensionZReg];
  case 17:
    return FB[Hexagon::ExtensionHVXV79] && FB[Hexagon::ExtensionHVXQFloat];
  case 18:
    return FB[Hexagon::ExtensionHVXV81] && FB[Hexagon::ExtensionHVXQFloat];
  case 19:
    return FB[Hexagon::ExtensionHVXV66];
  case 20:
    return FB[Hexagon::ExtensionHVXV79] && FB[Hexagon::ExtensionHVXIEEEFP];
  case 21:
    return FB[Hexagon::ExtensionHVXV68] && FB[Hexagon::ExtensionHVXIEEEFP];
  case 22:
    return FB[Hexagon::ExtensionHVXV68];
  case 23:
    return FB[Hexagon::ExtensionHVXV73] && FB[Hexagon::ExtensionHVXQFloat];
  case 24:
    return FB[Hexagon::ExtensionHVXV69];
  case 25:
    return FB[Hexagon::ExtensionHVXV73] && FB[Hexagon::ExtensionHVXIEEEFP];
  case 26:
    return FB[Hexagon::ExtensionHVXV68] && FB[Hexagon::ExtensionHVXQFloat];
  case 27:
    return FB[Hexagon::ExtensionHVXV73];
  }
}


template <typename InsnType>
static DecodeStatus decodeInstruction(const uint8_t DecodeTable[], MCInst &MI,
                                      InsnType insn, uint64_t Address,
                                      const MCDisassembler *DisAsm,
                                      const MCSubtargetInfo &STI) {
  const FeatureBitset &Bits = STI.getFeatureBits();
  const uint8_t *Ptr = DecodeTable;

  SmallVector<const uint8_t *, 8> ScopeStack;
  DecodeStatus S = MCDisassembler::Success;
  while (true) {
    ptrdiff_t Loc = Ptr - DecodeTable;
    const uint8_t DecoderOp = *Ptr++;
    switch (DecoderOp) {
    default:
      errs() << Loc << ": Unexpected decode table opcode: "
             << (int)DecoderOp << '\n';
      return MCDisassembler::Fail;
    case OPC_Scope: {
      unsigned NumToSkip = decodeULEB128AndIncUnsafe(Ptr);
      const uint8_t *SkipTo = Ptr + NumToSkip;
      ScopeStack.push_back(SkipTo);
      LLVM_DEBUG(dbgs() << Loc << ": OPC_Scope(" << SkipTo - DecodeTable
                        << ")\n");
      continue;
    }
    case OPC_SwitchField: {
      // Decode the start value.
      unsigned Start = decodeULEB128AndIncUnsafe(Ptr);
      unsigned Len = *Ptr++;
      uint64_t FieldValue = fieldFromInstruction(insn, Start, Len);
      uint64_t CaseValue;
      unsigned CaseSize;
      while (true) {
        CaseValue = decodeULEB128AndIncUnsafe(Ptr);
        CaseSize = decodeULEB128AndIncUnsafe(Ptr);
        if (FieldValue == CaseValue || !CaseSize)
          break;
        Ptr += CaseSize;
      }
      if (FieldValue == CaseValue) {
        LLVM_DEBUG(dbgs() << Loc << ": OPC_SwitchField(" << Start << ", " << Len
                          << "): " << FieldValue << '\n');
        continue;
      }
      break;
    }
    case OPC_CheckField: {
      // Decode the start value.
      unsigned Start = decodeULEB128AndIncUnsafe(Ptr);
      unsigned Len = *Ptr;
      uint64_t FieldValue = fieldFromInstruction(insn, Start, Len);
      // Decode the field value.
      unsigned PtrLen = 0;
      uint64_t ExpectedValue = decodeULEB128(++Ptr, &PtrLen);
      Ptr += PtrLen;
      bool Failed = ExpectedValue != FieldValue;

      LLVM_DEBUG(dbgs() << Loc << ": OPC_CheckField(" << Start << ", " << Len
                        << ", " << ExpectedValue << "): FieldValue = "
                        << FieldValue << ", ExpectedValue = " << ExpectedValue
                        << ": " << (Failed ? "FAIL, " : "PASS\n"););
      if (!Failed)
        continue;
      break;
    }
    case OPC_CheckPredicate: {
      // Decode the Predicate Index value.
      unsigned PIdx = decodeULEB128AndIncUnsafe(Ptr);
      // Check the predicate.
      bool Failed = !checkDecoderPredicate(PIdx, Bits);

      LLVM_DEBUG(dbgs() << Loc << ": OPC_CheckPredicate(" << PIdx << "): "
                        << (Failed ? "FAIL, " : "PASS\n"););
      if (!Failed)
        continue;
      break;
    }
    case OPC_Decode: {
      // Decode the Opcode value.
      unsigned Opc = decodeULEB128AndIncUnsafe(Ptr);
      unsigned DecodeIdx = decodeULEB128AndIncUnsafe(Ptr);

      MI.clear();
      MI.setOpcode(Opc);
      bool DecodeComplete;
      S = decodeToMCInst(DecodeIdx, S, insn, MI, Address, DisAsm,
                         DecodeComplete);
      LLVM_DEBUG(dbgs() << Loc << ": OPC_Decode: opcode " << Opc
                        << ", using decoder " << DecodeIdx << ": "
                        << (S ? "PASS, " : "FAIL, "));

      if (DecodeComplete) {
        LLVM_DEBUG(dbgs() << "decoding complete\n");
        return S;
      }
      assert(S == MCDisassembler::Fail);
      // Reset decode status. This also drops a SoftFail status that could be
      // set before the decode attempt.
      S = MCDisassembler::Success;
      break;
    }
    }
    if (ScopeStack.empty()) {
      LLVM_DEBUG(dbgs() << "returning Fail\n");
      return MCDisassembler::Fail;
    }
    Ptr = ScopeStack.pop_back_val();
    LLVM_DEBUG(dbgs() << "continuing at " << Ptr - DecodeTable << '\n');
  }
  llvm_unreachable("bogosity detected in disassembler state machine!");
}


} // namespace
